/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#include <any.h>

Any::Any()
{
}

Any::Any(const Any & a)
{
  data = a.rawData().copy();
}

Any::~Any()
{
  data.truncate(0);
}

Any::Any(const QByteArray &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)ByteArray<<value;
}

Any::Any(const QDateTime &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)DateTime<<value;
}

Any::Any(const QColor &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)Color<<value;
}
Any::Any(const DateClass &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)ExtDateTime<<value;
}

Any::Any(const QFont &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)Font<<value;
}

Any::Any(const QPoint &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)Point<<value;
}

Any::Any(const QSize &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)Size<<value;
}
		
Any::Any(const QStringList &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)StringList<<value;
}

Any::Any(const QString &value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)String<<value;
}

Any::Any(int value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)Int<<value;
}

Any::Any(double value)
{
	QDataStream d(data, IO_WriteOnly);
	d<<(unsigned int)Double<<value;
}

Any::operator QByteArray()
{
	unsigned int t;
	QByteArray res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==ByteArray) d>>res;
	}
	return res;
}

QByteArray Any::toByteArray()
{
	unsigned int t;
	QByteArray res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==ByteArray) d>>res;
	}
	return res;
}

QDateTime Any::toDateTime()
{
	unsigned int t;
	QDateTime res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==DateTime) d>>res;
	}
	return res;
}

Any::operator QDateTime()
{
  return toDateTime();
}

QColor Any::toColor()
{
	unsigned int t;
	QColor res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Color) d>>res;
	}
	return res;
}

Any::operator QColor()
{
  return toColor();
}

DateClass Any::toDateClass()
{
	unsigned int t;
	DateClass res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==ExtDateTime) d>>res;
	}
	return res;
}

Any::operator DateClass()
{
	return toDateClass();
}
		
QFont Any::toFont()
{
	unsigned int t;
	QFont res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Font) d>>res;
	}
	return res;
}

Any::operator QFont()
{
  return toFont();
}
		
QPoint Any::toPoint()
{
	unsigned int t;
	QPoint res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Point) d>>res;
	}
	return res;
}

Any::operator QPoint()
{
  return toPoint();
}

QSize Any::toSize()
{
	unsigned int t;
	QSize res(0, 0);
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Size) d>>res;
	}
	return res;
}
		
Any::operator QSize()
{
  return toSize();
}

QString Any::toString()
{
	unsigned int t;
	QString res=QString::null;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==String) d>>res;
	}
	return res;
}

Any::operator QString()
{
	unsigned int t;
	QString res=QString::null;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==String) d>>res;
	}
	return res;
}

QStringList Any::toStringList()
{
	unsigned int t;
	QStringList res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==StringList) d>>res;
	}
	return res;
}

Any::operator QStringList()
{
	unsigned int t;
	QStringList res;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==StringList) d>>res;
	}
	return res;
}

int Any::toInt()
{
	unsigned int t;
	int res=0;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Int) d>>res;
	}
	return res;
}

Any::operator int()
{
	unsigned int t;
	int res=0;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Int) d>>res;
	}
	return res;
}

double Any::toDouble()
{
	unsigned int t;
	double res=0;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Double) d>>res;
	}
	return res;
}

Any::operator double()
{
	unsigned int t;
	double res=0;
	if(data.size()>3)
	{
		QDataStream d(data, IO_ReadOnly);
		d>>t;
		if(t==Double) d>>res;
	}
	return res;
}

const Any& Any::operator=(const Any &a)
{
	data = a.rawData().copy();
	return *this;
}

bool Any::operator==(const Any &a) const
{
	return data == a.rawData();
}

bool Any::operator!=(const Any &a) const
{
	return data != a.rawData();
}

Any::Type Any::type()
{
	if(data.size()<4) return Invalid;
	QDataStream d(data, IO_ReadOnly);
	unsigned int t;
	d>>t;
	return (Type)t;
}

QString Any::typeName()
{
	Type t=type();
	switch(t)
	{
		case ByteArray:
			return "ByteArray";
		case StringList:
			return "StringList";
		case String:
			return "String";
		case Int:
			return "Int";
		case Font:
			return "Font";
		case Point:
			return "Point";
		case Size:
			return "Size";
		case DateTime:
			return "DateTime";
		case ExtDateTime:
			return "ExtDateTime";
		case Color:
			return "Color";
		case Double:
			return "Double";
		default:
			return "Invalid";
	}
}

bool Any::canCast(Type t)
{
	return t==type();
}

bool Any::canCast(const QString &t)
{
	return t==typeName();
}

bool Any::isValid()
{
	return type()!=Invalid;
}

bool Any::load(const QByteArray &val)
{
	if(val.size()<4) return false;
	QDataStream d(val, IO_ReadOnly);
	unsigned int t;
	d>>t;
	if(isValidType((Type)t))
	{
		data=val.copy();
		return true;
	}
	return false;
}

bool Any::isValidType(Type t)
{
	if(t==ByteArray||t==StringList||t==String||t==Int||t==Font||t==Point||t==Size||t==DateTime||t==ExtDateTime||t==Double||t==Color) return true;
	return false;
}

bool Any::isValidTypeName(const QString &t)
{
	if(t=="ByteArray"||t=="StringList"||t=="String"||t=="Int"||t=="Font"||t=="Point"||t=="Size"||t=="DateTime"||t=="ExtDateTime"||t=="Double"||t=="Color") return true;
	return false;
}

const QByteArray& Any::rawData() const
{
  return (QByteArray &)data;
}

void Any::clear()
{
  data.truncate(0);
}

QDataStream& operator>>(QDataStream &s, Any &t)
{
  QByteArray d;
  s>>d;
  t.load(d);
  d.truncate(0);
  return s;
}

QDataStream& operator<<(QDataStream &s, const Any &t)
{
  s<<t.data;
  return s;
}
