/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include "appstatus.h"

#include <qlabel.h>
#include <qprogressbar.h>
#include <qlayout.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qhbox.h>

#include <qiconloader.h>
#include <komcontainerwidget.h>

AppStatus * AppStatus::instance_ = 0L;

class AppStatus::Private
{
  public:
  
    Private(AppStatus*);
    virtual ~Private();
    
    AppStatus::Status serverStatus;
    AppStatus::Status clientStatus;
    
    QLabel * serverLabelStatus;
    QLabel * clientLabelStatus;
    
    KomContainerWidget * serverWidget;
    KomContainerWidget * clientWidget;
    
    QLabel * serverText;
    QLabel * clientText;
    
    QWidget * serverProgressWidget;
    QWidget * clientProgressWidget;
    
    QLabel * serverProgressText;
    QLabel * clientProgressText;
    
    QProgressBar * serverProgressBar;
    QProgressBar * clientProgressBar;
    
    void setStatus(AppStatus::Level level, AppStatus::Status status);
  
  private:
  
    AppStatus * parent_;
};

AppStatus::Private::Private(AppStatus * parent)
  : serverStatus( AppStatus::Good ),
    clientStatus( AppStatus::Good ),
    serverLabelStatus(0L),
    clientLabelStatus(0L),
    serverWidget(0L),
    clientWidget(0L),
    serverText(0L),
    clientText(0L),
    serverProgressWidget(0L),
    clientProgressWidget(0L),
    serverProgressText(0L),
    clientProgressText(0L),
    serverProgressBar(0L),
    clientProgressBar(0L),
    parent_(parent)
{
}

AppStatus::Private::~Private()
{
}
    
void AppStatus::Private::setStatus(AppStatus::Level level, AppStatus::Status status)
{
  QLabel * label;
  
  if( AppStatus::Server == level )
  {
    label = serverLabelStatus;
    
    switch(status)
    {
      case AppStatus::Warning:
        label->setPixmap( BarIcon("protocol_warning.png") );
        break;
      case AppStatus::Error:
        label->setPixmap( BarIcon("protocol-dis.png") );
        break;
      case AppStatus::Waiting:
        label->setPixmap( BarIcon("protocol_waiting.png") );
        break;
      case AppStatus::Good:
      default :
        label->setPixmap( BarIcon("protocol.png") );
        break;
    }
  }
  else
  {
    label = clientLabelStatus;
    
    switch(status)
    {
      case AppStatus::Warning:
        label->setPixmap( BarIcon("interface_warning.png") );
        break;
      case AppStatus::Error:
        label->setPixmap( BarIcon("interface-dis.png") );
        break;
      case AppStatus::Waiting:
        label->setPixmap( BarIcon("interface_waiting.png") );
        break;
      case AppStatus::Good:
      default :
        label->setPixmap( BarIcon("interface.png") );
        break;
    }
  }
}
  
AppStatus::AppStatus(QWidget * parent, const char * name)
  : QStatusBar(parent, name)
{
  d = new Private(this);
  
  instance_ = this;
  
  // main widget
  QHBox * widget = new QHBox(this, "AppStatusWidget");
  widget->setMargin(0);
  widget->setSpacing(0);
  
  // Server status
  d->serverLabelStatus = new QLabel(widget);
  d->serverLabelStatus->setFixedWidth(25);
  d->serverLabelStatus->setFrameShape( QFrame::NoFrame );
  d->serverWidget = new KomContainerWidget(widget);
  
  d->serverProgressWidget = new QWidget( d->serverWidget );
  QHBoxLayout * serverLayout = new QHBoxLayout( d->serverProgressWidget );
  serverLayout->setMargin(0);
  serverLayout->setSpacing(0);
  d->serverProgressText = new QLabel( d->serverProgressWidget );
  d->serverProgressText->setFrameShape( QFrame::NoFrame );
  d->serverProgressBar = new QProgressBar( d->serverProgressWidget );
  d->serverProgressBar->setFixedWidth(100);
  serverLayout->addWidget( d->serverProgressText, 1);
  serverLayout->addWidget( d->serverProgressBar );
  d->serverWidget->addChildWidget( d->serverProgressWidget );
  
  d->serverText = new QLabel( d->serverWidget );
  d->serverText->setFrameShape( QFrame::NoFrame );
  d->serverWidget->addChildWidget( d->serverText );
  d->serverWidget->setFrameShape( QFrame::NoFrame );

  // Client status
  d->clientLabelStatus = new QLabel(widget);
  d->clientLabelStatus->setFixedWidth(25);
  d->clientLabelStatus->setFrameShape( QFrame::NoFrame );
  d->clientWidget = new KomContainerWidget(widget);
  
  d->clientProgressWidget = new QWidget( d->clientWidget );
  QHBoxLayout * clientLayout = new QHBoxLayout( d->clientProgressWidget );
  clientLayout->setMargin(0);
  clientLayout->setSpacing(0);
  d->clientProgressText = new QLabel( d->clientProgressWidget );
  d->clientProgressText->setFrameShape( QFrame::NoFrame );
  d->clientProgressBar = new QProgressBar( d->clientProgressWidget );
  d->clientProgressBar->setFixedWidth(100);

  clientLayout->addWidget( d->clientProgressText, 1);
  clientLayout->addWidget( d->clientProgressBar );
  d->clientWidget->addChildWidget( d->clientProgressWidget );
  
  d->clientText = new QLabel( d->clientWidget );
  d->clientText->setFrameShape( QFrame::NoFrame );
  d->clientWidget->addChildWidget( d->clientText );
  d->clientWidget->setFrameShape( QFrame::NoFrame );
  
  // Status Bar
  addWidget(widget, 400, true);
  widget->setStretchFactor(d->serverLabelStatus, 0);
  widget->setStretchFactor(d->serverWidget, 200);
  widget->setStretchFactor(d->clientLabelStatus, 0);
  widget->setStretchFactor(d->clientWidget, 200);
  
  d->setStatus(Server, Good);
  d->setStatus(Client, Good);
}

AppStatus::~AppStatus()
{
  if(d)
  {
    delete d;
    d = 0L;
  }
}

AppStatus * AppStatus::instance()
{
  return instance_;
}

void AppStatus::message(const QString&)
{
}

void AppStatus::message(const QString&, int ms)
{
}

void AppStatus::clear()
{
}

void AppStatus::message(Level level, Type type, Status status, const QString & text, int iStep, int iTotal)
{
  QProgressBar * progressBar;
  QLabel       * progressText;
  QLabel       * textInfo;
  
  d->setStatus(level, status);
  
  if( Server == level )
  {
    progressBar = d->serverProgressBar;
    progressText = d->serverProgressText;
    textInfo = d->serverText;
    
    if( Text == type )
      d->serverWidget->showChild( d->serverText );
    else
      d->serverWidget->showChild( d->serverProgressWidget );
  }
  else
  {
    progressBar = d->clientProgressBar;
    progressText = d->clientProgressText;
    textInfo = d->clientText;
    
    if( Text == type )
      d->clientWidget->showChild( d->clientText );
    else
      d->clientWidget->showChild( d->clientProgressWidget );
  }
  
  if( Text == type )
  {
    textInfo->setText(text);
  }
  else
  {
    progressText->setText(text);
    if( iTotal )
      progressBar->setTotalSteps(iTotal);
    progressBar->setProgress(iStep);
  }
}

