//
// C++ Implementation: certificatesmanager
//
// Description: 
//
//
// Author: Eugen Constantinescu <eug@thekompany.com>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "certificatesmanager.h"

#include <settings.h>
#include <acceptcertificatebase.h>

#include <qdir.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qtextbrowser.h>

using namespace tkcSSL;

CertificatesManager * CertificatesManager::instance_ = 0;

class CertificatesManager::Private
{
  public:
  
    Private(CertificatesManager*);
    ~Private();
    
    QString certFolder;
    QString certSettings;
    
  private:
  
    CertificatesManager * parent_;
};

CertificatesManager::Private::Private(CertificatesManager * parent)
  : certSettings("/Aethera/CertificatesManager/"),
    parent_(parent)
{
  certFolder = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir") + 
               "/certificates";
  QDir certDir(certFolder);
  if( !certDir.exists() )
  {
    if( !certDir.mkdir( certDir.path() ) )
    {
      qDebug("Cannot create the certificates folder.");
    }
  }
}

CertificatesManager::Private::~Private()
{
}

CertificatesManager * CertificatesManager::instance()
{
  if( !instance_ )
  {
    instance_ = new CertificatesManager();
  }
  
  return instance_;
}

CertificatesManager::CertificatesManager(QObject *parent, const char *name)
  : QObject(parent, name)
{
  d = new Private(this);
}

CertificatesManager::~CertificatesManager()
{
  if(d)
  {
    delete d;
    d = 0;
  }
}

QString CertificatesManager::certificatesFolder() const
{
  return d->certFolder;
}

void CertificatesManager::setCertificatesFolder(const QString & path)
{
  QDir pathDir(path);
  
  if( pathDir.exists() )
  {
    d->certFolder = path;
  }
}

bool CertificatesManager::isCertificateFile
  ( const QString & accountName,
    const QString & protocol ) const
{
  QFile certFile( certificateFile(accountName, protocol) );
  if( certFile.exists() && certFile.isReadWrite() )
  {
    return true;
  }
  
  return false;
}

QString CertificatesManager::certificateFile
  ( const QString & accountName,
    const QString & protocol ) const
{
  return d->certFolder + "/" + accountName + "_" + protocol + ".pem";
}

QString CertificatesManager::setCertificate
  ( const QString & accountName,
    const QString & protocol,
    const QString & importPath )
{
  QString certFileName;
  QFile importCert(importPath);
  
  if( importCert.exists() &&
      importCert.open( IO_ReadOnly ) )
  {
    QFile certFile(certificateFile(accountName, protocol));
    if( certFile.open( IO_WriteOnly ) )
    {
      int iRead = 0;
      char buffer[4096];
      
      while( 0 < (iRead = importCert.readBlock(buffer, 4096)) )
      {
        certFile.writeBlock(buffer, iRead);
      }
      
      certFile.close();
      
      certFileName = certFile.name();
    }
    
    importCert.close();
  }
  
  return certFileName;
}

bool CertificatesManager::clearAll()
{
  // TODO
  return true;
}

bool CertificatesManager::clearCertificate
  ( const QString & accountName,
    const QString & protocol )
{
  if( isCertificateFile(accountName, protocol) )
  {
    return QFile::remove( certificateFile(accountName, protocol) );
  }
  
  return true;
}

void CertificatesManager::slotAcceptCertificateDialog
  ( SSLDevice * device,
    const QString & info )
{
  // TODO show the protocol and account info etc 
  
  AcceptCertificateBase acceptDialog(0, "Accept Certificate", true);
  acceptDialog.certificateInfo->setText(info);
  int certAccept = acceptDialog.exec();
  
  switch( certAccept )
  {
    case 1:
      // TODO save it TEMPORARY (current session)
      // keep a list with temporary certificates etc
      device->setVerifyType( SSLDevice::NO_CHECKING );
      break;
    
    case 2:
      device->setVerifyType( SSLDevice::SAVE );
      break;
    
    case 0:
    default:
      device->setVerifyType( SSLDevice::NOT_ACCEPTED );
      break;
  }
}

void CertificatesManager::slotCertificateFailed(const QString & info)
{
  QMessageBox::information( 0,
    tr("Aethera Certificates Manager"),
    tr("Certificate failed :") + "\n" + info );
}
