//
// C++ Interface: certificatesmanager
//
// Description: 
//
//
// Author: Eugen Constantinescu <eug@thekompany.com>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef CERTIFICATESMANAGER_H
#define CERTIFICATESMANAGER_H

#include <qobject.h>

#include <defines.h>
#include <ssldevice.h>

using namespace tkcSSL;

/**
Certificates manager gives you access to the Aethera SSL certificates.
Also it provides a certificate accept dialog.


@author Eugen Constantinescu
*/
class AETHERA_API CertificatesManager : public QObject
{
  Q_OBJECT
  
  public:
  
    static CertificatesManager * instance();
    
    virtual ~CertificatesManager();
    
    /** Returns the certificates store folder.*/
    virtual QString certificatesFolder() const;
    
    /** Set the certificates store folder.*/
    virtual void setCertificatesFolder(const QString&);
    
    /** Check if there is a certificate file for account and protocol.*/
    virtual bool isCertificateFile(const QString&, const QString&) const;
    
    /** Returns the path associated with the certificate (account and protocol).*/
    virtual QString certificateFile(const QString&, const QString&) const;
    
    /** Import an external certificate.*/
    virtual QString setCertificate(const QString&, const QString&, const QString&);
    
    /** Remove all certificates.*/
    virtual bool clearAll();
    
    /** It will remove the certificate associated with account and protocol.*/
    virtual bool clearCertificate(const QString&, const QString&);
    
  public slots:
  
    /** Open the accept certificate dialog with caption and information.*/
    virtual void slotAcceptCertificateDialog(SSLDevice*, const QString&);
    
    virtual void slotCertificateFailed(const QString&);
    
  private:
  
    CertificatesManager(QObject *parent = 0, const char *name = 0);
    
    static CertificatesManager * instance_;

    class Private;
    Private * d;
};

#endif
