/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "commframe.h"

#include <qdatastream.h>

bool CommFrame::check = false;

CommFrame::CommFrame()
  : frameLength(0),
    majorRevNo(0),
    minorRevNo(0),
    requestFrame(false),
    replyFrame(false),
    serverFrame(false),
    clientFrame(false),
    requestsReply(false),
    deniesReply(false),
    hasStatus(false),
    hasData(false),
    transactionTag(0),
    sequenceTag(0),
    commandCode(0),
    statusCode(0),
    validity(false)
{
}

CommFrame::CommFrame(const QByteArray & data)
  : frameLength(0),
    majorRevNo(0),
    minorRevNo(0),
    requestFrame(false),
    replyFrame(false),
    serverFrame(false),
    clientFrame(false),
    requestsReply(false),
    deniesReply(false),
    hasStatus(false),
    hasData(false),
    transactionTag(0),
    sequenceTag(0),
    commandCode(0),
    statusCode(0),
    validity(false)
{
  setData(data);
}

CommFrame::~CommFrame()
{
  extraData.truncate(0);
}

bool CommFrame::setData(const QByteArray & data)
{
  // reset frame
  if(validity)
    reset();

  // get a data stream
  QDataStream s(data, IO_ReadOnly);
  unsigned char t=0,u=0;
  
  if(check)
  {
    // check if we have at least the whole header
    if(data.size()<14)
    {
      qDebug("Invalid frame (length<14), bailing out");
      return false;
    }
    
    // check the Aethera signature byte
    s>>t;
    if(t!='A')
    {
      qDebug("Invalid frame (wrong signature byte), bailing out");
      return false;
    }
      
    // get the software version
    s>>t;
    u=t;
    t=t>>4;
    u=u<<4;
    u=u>>4;
    majorRevNo=t;
    minorRevNo=u;
    
    // check software version (only 1.1 is accepted right now)
    if(majorRevNo!=1||minorRevNo!=1)
    {
      qDebug("Invalid frame (wrong software revision), bailing out");
      return false;
    }
    
    // get the frame characteristics
    s>>t;
    s>>u; // reserved for future use
    if(t&0x80) requestFrame=true;
    else replyFrame=true;
    if(t&0x40) clientFrame=true;
    else serverFrame=true;
    
    // test the controlled reply behavior
    if( (~t) & 0x20 )
    {
      if(t&0x10) requestsReply=true;
      else deniesReply=true;
    }
    if(t&0x08) hasStatus=true;
    if(t&0x04) hasData=true;
    
    // test the frame characteristics
    if(requestFrame==replyFrame
        || requestsReply&&deniesReply
        || serverFrame==clientFrame)
    {
      qDebug("Invalid frame (wrong combination of flags), bailing out");
      return false;
    }
      
    // get the transaction number
    s>>transactionTag;
    
    // get the sequence number
    s>>sequenceTag;
  }
  else
  {
    // get the frame characteristics
    s >> t;
    if(t&0x80) requestFrame=true;
    else replyFrame=true;
    if(t&0x40) clientFrame=true;
    else serverFrame=true;
    
    // test the controlled reply behavior
    if( (~t) & 0x20 )
    {
      if(t&0x10) requestsReply=true;
      else deniesReply=true;
    }
    if(t&0x08) hasStatus=true;
    if(t&0x04) hasData=true;
  }
  
  // get the command code
  s>>commandCode;
  
  // get the status, if available
  if(hasStatus)
  {
    s>>statusCode;
    s>>statusString;
  }
  if(hasData)
    s>>extraData;
  
  // we should have reached the end of the stream
  if(!s.atEnd())
  {
    qDebug("Invalid frame (could not reach the end of data stream), bailing out");
    return false;
  }
  
  validity=true;
  
  return true;
}

bool CommFrame::isValid()
{
  return validity;
}

void CommFrame::reset()
{
  frameLength=0;
  majorRevNo=0;
  minorRevNo=0;
  requestFrame=false;
  replyFrame=false;
  serverFrame=false;
  clientFrame=false;
  requestsReply=false;
  deniesReply=false;
  hasStatus=false;
  hasData=false;
  transactionTag=0;
  sequenceTag=0;
  commandCode=0;
  statusCode=0;
  statusString=QString::null;
  extraData.truncate(0);
  validity=false;
}

CommFrame::operator QByteArray()
{
  QByteArray a;
  QDataStream s(a, IO_WriteOnly);
  
  if(check)
  {
    // write the Aethera signature byte
    s<<(unsigned char)'A';
      
    // write the software version number
    s<<(unsigned char)0x11;
    
    // compute the type byte
    unsigned char t=0;
    if(requestFrame) t=t|0x80;
    if(clientFrame) t=t|0x40;
    if(requestsReply||deniesReply)
    {
      t=t|0x20;
      if(requestsReply) t=t|0x10;
    }
    if(hasStatus) t=t|0x08;
    if(hasData) t=t|0x04;
    
    // write the frame type (2 bytes, 10 bits reserved for future use)
    s<<(unsigned char)t;
    s<<(unsigned char)0;
    
    // write the transaction number
    s<<(unsigned int)transactionTag;
    
    // write the sequence number
    s<<(unsigned int)sequenceTag;
  }
  else
  {
    // compute the type byte
    unsigned char t=0;
    if(requestFrame) t=t|0x80;
    if(clientFrame) t=t|0x40;
    if(requestsReply||deniesReply)
    {
      t=t|0x20;
      if(requestsReply) t=t|0x10;
    }
    if(hasStatus) t=t|0x08;
    if(hasData) t=t|0x04;
    
    // write the frame type
    s<<(unsigned char)t;
  }
  
  // write the command code
  s<<(unsigned int)commandCode;
  
  // write the frame data
  if(hasStatus)
  {
    s<<(unsigned int)statusCode;
    s<<statusString;
  }
  if(hasData)
    s<<extraData;
  
  return a;
}

void CommFrame::setCheck(bool enable)
{
  check = enable;
}
