/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include <datahandler.h>
#include <clipboard.h>
#include <qdragobject.h>
#include <vobject.h>

static QStrList qStringListToStrList(const QStringList & l)
{
  QStrList ret;

  for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
    ret.append((*it).utf8());

  return ret;
}

extern ClipboardWrapper *Clipboard;

DataHandler::DataHandler(VFSModule* vfsModule)
  : lastID(-1),
    vfs(vfsModule),
    _parentFolder(0L)
{
  ClipboardWrapper::thisInstance()->registerDataHandler(this);
}

DataHandler::~DataHandler()
{
  ClipboardWrapper::thisInstance()->cancelClipboardOperation(lastID);
  ClipboardWrapper::thisInstance()->unregisterDataHandler(this);
}

void DataHandler::clipboardOperationCompleted(int id)
{
  if(id!=lastID)
  {
    discardCut();
  }

  lastID=-1;
}

bool DataHandler::canPaste()
{
  int cid=ClipboardWrapper::thisInstance()->getLastID();

  if(lastID!=cid && cid!=-1 && ClipboardWrapper::thisInstance()->data())
    return true;
  return false;
}

void DataHandler::grayOutSelection()
{
  lastID=copySelection();
}

void DataHandler::discardCut()
{
}

int DataHandler::copySelection()
{
  QStringList uriList = getSelection();

  if (uriList.isEmpty())
    return -1;

  return ClipboardWrapper::thisInstance()->copyData
    (new QUriDrag(qStringListToStrList(uriList)));
}

int DataHandler::cutSelection()
{
  QStringList uriList = getSelection();

  if (uriList.isEmpty())
    return -1;

  DataHandler::grayOutSelection();

  return ClipboardWrapper::thisInstance()->cutData
    (new QUriDrag(qStringListToStrList(uriList)));
}

void DataHandler::deleteSelection()
{
  QStringList uriList = getSelection();

  if( uriList.count() )
    vfs->remove( uriList );
  
  uriList.clear();
}

void DataHandler::paste()
{
  if
    (
      canPaste()
      && QUriDrag::canDecode(ClipboardWrapper::thisInstance()->data())
    )
  {
    QUriDrag * uriList =
      static_cast<QUriDrag *>(ClipboardWrapper::thisInstance()->pasteData());

    QStrList pathList;

    QUriDrag::decode(uriList, pathList);

    bool shouldMove = ClipboardWrapper::thisInstance()->shouldCutSource();

    QStringList l(QStringList::fromStrList(pathList));

    VFolder * targetFolder = parentFolder();
    if( targetFolder )
    {
      if(shouldMove)
        // move source here
        vfs->move(l, targetFolder->path());
      else
        // copy source here
        vfs->copy(l, targetFolder->path());
    }
    
//    for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
//    {
//      QString sourcePath(*it);
//
//      VFolder * targetFolder = parentFolder();
//      if( targetFolder )
//      {
//        if(shouldMove)
//          // move source here
//          vfs->move(sourcePath, targetFolder->path());
//        else
//          // copy source here
//          vfs->copy(sourcePath, targetFolder->path());
//      }
//      else
//      {
//        qDebug("DataHandler::paste: folder null");
//      }
//    }

  }
}

void DataHandler::copySelectionTo()
{
#if 0
  if(!isSelectionEmpty())
  {
    DestinationDialog *d=new DestinationDialog("Copy", parentFolder()->mimeType(), 0);
    int res=d->exec();

    if(res==QDialog::Accepted)
    {
      FolderTreeItem *selectedItem=d->selectedItem();
      QString filePath=selectedItem->getFilePath();

      if(filePath)
      {
        // export current selection
      }
      else
      {
        // move current selection	
        QStrList uriList = getSelection();
        QString destFolder=selectedItem->path();

        for( QStrListIterator it(uriList); it.current(); ++it)
        {
          QString copiedURL = (const char *)it.current();
          vfs->copy(copiedURL, destFolder);
        }
      }
    }
    delete d;
  }
  else
  {
    qDebug("Warning: there is nothing to copy, bailing out");
  }
#endif
}

void DataHandler::moveSelectionTo()
{
#if 0
  if(!isSelectionEmpty())
  {
    DestinationDialog *d=new DestinationDialog("Copy", parentFolder()->mimeType(), 0);
    int res=d->exec();

    if(res==QDialog::Accepted)
    {
      FolderTreeItem *selectedItem=d->selectedItem();
      QString filePath=selectedItem->getFilePath();

      if(filePath)
      {
        // export current selection
      }
      else
      {
        // move current selection	
        QStrList uriList = getSelection();
        QString destFolder=selectedItem->path();

        for( QStrListIterator it(uriList); it.current(); ++it)
        {
          QString copiedURL = (const char *)it.current();
          vfs->move(copiedURL, destFolder);
        }
      }
    }
    delete d;
  }
  else
  {
    qDebug("Warning: there is nothing to copy, bailing out");
  }
#endif
}

void DataHandler::dragStart(QWidget *dragSource)
{
  QStringList uriList = getSelection();

  if (!uriList.isEmpty())
  {
    qDebug("Starting drag for %d items", uriList.count());
    DataHandler::grayOutSelection();
    QUriDrag * d = new QUriDrag(qStringListToStrList(uriList), dragSource);
    d->drag();
  }
}

void DataHandler::dragEnter(QDragEnterEvent* event)
{
  event->accept(QUriDrag::canDecode(event));
}

void DataHandler::dragDrop(QDropEvent * event)
{
  QStrList pathList;

  if (QUriDrag::decode(event, pathList))
  {
    QStringList l(QStringList::fromStrList(pathList));

    for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
    {
      QString sourcePath(*it);
      
      VFolder * targetFolder = parentFolder();
      if( targetFolder )
      {
        vfs->move(sourcePath, targetFolder->path());
      }
      else
      {
        qDebug("DataHandler::dragDrop: folder null");
      }
    }
  }
}

VFolder* DataHandler::parentFolder()
{
  return _parentFolder;
}

void DataHandler::setParentFolder(VFolder *folder)
{
  _parentFolder=folder;
}

VFSModule* DataHandler::vfsModule()
{
  return vfs;
}

