/*!
  ______________________________________________________________________________
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.

               COPYRIGHT   : (C) 2003 by theKompany.com
  ______________________________________________________________________________

  \date  2003
  \author Iulian Ghiciuc <ilie@eco-spaces.com.ro>
  \file Implementation of the EListViewItem class

*/

//own includes
#include "elistviewitem.h"
#include "elistview.h"

/*
----------------------------------------------------------------------------------
EListViewItem::Private class
----------------------------------------------------------------------------------
*/

class EListViewItem::Private
{
  public:
    Private( EListViewItem* _owner );
    ~Private();

    bool useAlternativeColor();

    EListViewItem* owner;

};

EListViewItem::Private::Private( EListViewItem* _owner )
  : owner( _owner )
{
}

EListViewItem::Private::~Private()
{
}

bool EListViewItem::Private::useAlternativeColor()
{
  EListView* eList = static_cast<EListView*>( owner->listView() );
  if( !eList || !eList->alternativeBackgroundColor().isValid() )
  {
    qDebug("EListViewItem::Private -- > Invalid list or color handed to the paint function");
    return false;
  }

  
  if( eList->alternativeBackground() )
  {
    //compute the index of the item
    EListViewItem* above = static_cast<EListViewItem*>( owner->itemAbove() );
    if( above )
      owner->m_known = above->m_known;
    else
      owner->m_known = true;

    if( owner->m_known )
    {
      if( above )
        owner->m_odd = !above->m_odd;
      else
        owner->m_odd = false;
    }
    else
    {
      EListViewItem* item;
      bool previous = true;

      if( owner->parent() )
      {
        item = static_cast<EListViewItem*>( owner->parent() );
        if( item )
          previous = item->m_odd;
        item = static_cast<EListViewItem*>( owner->parent()->firstChild() );
      }
      else
        item = static_cast<EListViewItem*>( eList->firstChild() );

      while( item )
      {
        previous = !previous;
        item->m_odd = previous;
        item->m_known = true;
        item = static_cast<EListViewItem*>( item->nextSibling() );
      }

    }
      return owner->m_odd;
  }
  else
    return false;
}

/*
----------------------------------------------------------------------------------
EListViewItem class
----------------------------------------------------------------------------------
*/

EListViewItem::EListViewItem( QListView* parent )
  : QListViewItem( parent ),
    m_known( false )
{
  d = new Private( this );
}


EListViewItem::EListViewItem( QListView* parent, QListViewItem* after )
  : QListViewItem( parent, after ),
    m_known( false )
{
  d = new Private( this );
}


EListViewItem::~EListViewItem()
{
  delete d;
  d = 0;
}


void EListViewItem::paintCell( QPainter* p, const QColorGroup& cg, int column, int width, int align )
{
  if( d->useAlternativeColor() )
  {
    QColorGroup _cg = cg;
    _cg.setColor( QColorGroup::Background,  static_cast< EListView* >(listView())->alternativeBackgroundColor() );
    _cg.setColor( QColorGroup::Base,  static_cast< EListView* >(listView())->alternativeBackgroundColor() );
    QListViewItem::paintCell( p, _cg, column, width, align );
  }
  else
    QListViewItem::paintCell( p, cg, column, width, align );
}

