/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
// vim:tabstop=2:shiftwidth=2:expandtab:cinoptions=(s,U1,m1
// Copyright (C) 2002 Rik Hemsley (rikkus) <rik@kde.org>

#include <qnetwork.h>
#include <qurloperator.h>
#include <qurl.h>
#include <qdir.h>
#include <qfile.h>

#include <filedownloader.h>

//#define FILEDOWNLOAD_DEBUG

class FileDownloader::Private
{
  public:

    QUrlOperator  urlOperator;
    QUrl          packageUrl;
    QUrl          savePath;
    QFile         outputFile;
};

FileDownloader::FileDownloader(QObject * parent)
  : QObject(parent, "file downloader")
{
  d = new Private;

  // init ftp and http protocols
  qInitNetworkProtocols();
    
  connect(&d->urlOperator, SIGNAL(data(const QByteArray&, QNetworkOperation*)),
          this, SLOT(slotData(const QByteArray&, QNetworkOperation*)));
  connect(&d->urlOperator, SIGNAL(finished(QNetworkOperation*)),
          this, SLOT(slotFinished(QNetworkOperation*)));
}

FileDownloader::~FileDownloader()
{
  delete d;
}

  void
FileDownloader::download
(
  const QUrl    & serverUrl,
  const QUrl    & savePath
)
{
  d->urlOperator.stop();
  d->outputFile.close();
  d->savePath = savePath;
  d->packageUrl = serverUrl;

  d->outputFile.setName( d->savePath.toString(false, false) );

  if (!d->outputFile.open(IO_WriteOnly))
  {
    Result result;

    result.type         = Result::TargetUnwritable;
    result.description  =
      QString("Can't save package to %1") .arg(d->savePath);

    emit(finished(result, d->packageUrl, d->savePath));
  }

//  d->urlOperator.copy(serverUrl.toString(), d->savePath.dirPath());
  const QNetworkOperation * op = d->urlOperator.get(serverUrl.toString());
  if(op && QNetworkProtocol::NoError != op->errorCode())
  {
    qDebug("Hmm, error : %s", op->protocolDetail().latin1());
  }
}

  void
FileDownloader::slotFinished(QNetworkOperation * op)
{
  Result result;

  d->outputFile.close();
  
  switch (op->errorCode())
  {
    case QNetworkProtocol::NoError:

      result.type         = Result::Success;
      result.description  = "Success";
      
      break;

    case QNetworkProtocol::ErrLoginIncorrect:

      result.type         = Result::ConnectionRefused;
      result.description  = "Can't log in to server";

      break;

    case QNetworkProtocol::ErrHostNotFound:

      result.type         = Result::HostNotFound;
      result.description  = "Can't find server";
      break;


    case QNetworkProtocol::ErrGet:

      result.type         = Result::SocketRead;
      result.description  = "Connection was broken during download";

      break;

    case QNetworkProtocol::ErrFileNotExisting:

      result.type         = Result::BrokenServer;
      result.description  = "Can't find package on server";

      break;

    default:

      result.type         = Result::Internal;
      result.description  = "Internal error";

      break;
  }

  emit(finished(result, d->packageUrl, d->savePath.toString(false, false)));
}

  void
FileDownloader::slotData(const QByteArray & data, QNetworkOperation *)
{
#ifdef FILEDOWNLOAD_DEBUG
  qDebug("FileDownloader::slotData");
#endif
  d->outputFile.writeBlock(data);
}

  void
FileDownloader::slotDataTransferProgress
(
  int done,
  int total,
  QNetworkOperation *
)
{
#ifdef FILEDOWNLOAD_DEBUG
  qDebug("FileDownloader::slotDataTransferProgress(%d / %d)", done, total);
#endif

  emit(progress(done, total));
}

