/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
// vim:tabstop=2:shiftwidth=2:expandtab:cinoptions=(s,U1,m1
// Copyright (C) 2002 Rik Hemsley (rikkus) <rik@kde.org>

#ifndef FILE_DOWNLOADER_H
#define FILE_DOWNLOADER_H

#include <qobject.h>
#include <qcstring.h>

#include <defines.h>

class QUrl;
class QNetworkOperation;

class AETHERA_API FileDownloader: public QObject
{
  Q_OBJECT

  public:

    struct Result
    {
      enum Type
      {
        Success,
        Internal,
        ConnectionRefused,
        HostNotFound,
        SocketRead,
        NoSuchPackage,
        BrokenServer,
        TargetUnwritable
      };

      Type    type;
      QString description;
    };

    FileDownloader(QObject * parent = 0);
    virtual ~FileDownloader();

    /**
     * Connect to server (referenced in packageUrl) and download package
     * (referenced in packageUrl), saving to savePath.
     * The packageRetrievalComplete signal will be sent on completion.
     *
     * Calling this method while a retrieval is in progress will cancel
     * the active retrieval.
     */
    void download(const QUrl & packageUrl, const QUrl & savePath);

  signals:

    void finished
      (
        const FileDownloader::Result  & result,
        const QString & packageUrl,
        const QString & savePath
      );

    void progress(int bytesDone, int bytesTotal);

  protected slots:

    void slotFinished(QNetworkOperation *);
    void slotData(const QByteArray &, QNetworkOperation *);
    void slotDataTransferProgress(int, int, QNetworkOperation *);

  private:

    class Private;
    Private * d;
};

#endif // FILE_DOWNLOADER_H
