/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  folderchooserdialog.cpp
  -  Allows the user to select a VFS folder.
                             -------------------
    begin                : Sat Dec  8 19:27:41 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <defines.h>
#include <qpushbutton.h>
#include <qlayout.h>

#include <folderlistwidget.h>
#include <folderchooserdialog.h>

class FolderChooserDialog::Private
{
  public:

    FolderListWidget * folderListWidget;
};

FolderChooserDialog::FolderChooserDialog
(
  const QStringList & selection,
  const QString   & root,
  const QString   & caption,
  QWidget         * parent,
  const char      * name
)
  : QDialog(parent, name, true)
{
  d = new Private;

  d->folderListWidget = new FolderListWidget(root, this, "FolderListWidget");

  init();

  d->folderListWidget->setSelectionMode( QListView::Extended );

  if( caption.isEmpty() )
    setCaption(tr("Choose folder") + QString(" - %1").arg(AppName));
  else
    setCaption(caption + QString(" - %1").arg(AppName));
  
  if( selection.count() )
  {
    d->folderListWidget->fill(selection);
  }
}

FolderChooserDialog::FolderChooserDialog
(
  const QString   & root,
  QWidget         * parent,
  const char      * name
)
  : QDialog(parent, name, true)
{
  d = new Private;

  d->folderListWidget = new FolderListWidget(root, this, "FolderListWidget");

  init();
  
  setCaption(tr("Choose folder") + QString(" - %1").arg(AppName));
  
  d->folderListWidget->fill();
}

FolderChooserDialog::~FolderChooserDialog()
{
  delete d;
}

  void
FolderChooserDialog::init()
{
  QPushButton * ok      = new QPushButton(tr("&Ok"),      this);
  QPushButton * cancel  = new QPushButton(tr("&Cancel"),  this);

  ok->setAutoDefault(true);

  QVBoxLayout * mainLayout = new QVBoxLayout(this, 6, 4);

  mainLayout->addWidget(d->folderListWidget);

  QHBoxLayout * buttonLayout = new QHBoxLayout(mainLayout);

  buttonLayout->addStretch(1);
  buttonLayout->addWidget(ok);
  buttonLayout->addWidget(cancel);

  connect(ok,     SIGNAL(clicked()), SLOT(accept()));
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));

  connect
    (
      d->folderListWidget,
      SIGNAL(doubleClicked(QListViewItem *)),
      SLOT(accept())
    );

  connect
    (
      d->folderListWidget,
      SIGNAL(returnPressed(QListViewItem *)),
      SLOT(accept())
    );

  connect
    (
      d->folderListWidget,
      SIGNAL(spacePressed(QListViewItem *)),
      SLOT(accept())
    );
}

  QString
FolderChooserDialog::getFolder(const QString & root, QWidget * parent)
{
  FolderChooserDialog d(root, parent, "FolderChooserDialog");
  d.exec();
  return d.selectedFolder();
}

  void
FolderChooserDialog::getFolders(
  QStringList & selection,
  const QString & caption,
  const QString & root,
  QWidget * parent )
{
  FolderChooserDialog d(selection, root, caption, parent, "FolderChooserDialog");
  int iRet = d.exec();
  selection.clear();
  if( iRet == QDialog::Accepted )
  {
    d.selectedFolders(selection);
  }
}

  QString
FolderChooserDialog::selectedFolder() const
{
  QListViewItemIterator it(d->folderListWidget);

  for (; it.current(); ++it)
  {
    if (it.current()->isSelected())
      return d->folderListWidget->vfsPath(it.current());
  }

  return QString::null;
}
  
  void
FolderChooserDialog::selectedFolders(QStringList & selection) const
{
  QListViewItemIterator it(d->folderListWidget);

  for (; it.current(); ++it)
  {
    if (it.current()->isSelected())
      selection += d->folderListWidget->vfsPath(it.current());
  }
}
