/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  folderlistwidget.cpp
  -  Shows a list of VFS folders.
                             -------------------
    begin                : Sat Dec  8 14:22:38 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
    modified             : Ilie Ghiciuc <ilie@eco-spaces.com.ro> 9 May 2003
 ***************************************************************************/


#include <vfolder.h>
#include <qheader.h>
#include <vfsmodule.h>
#include <folderlistwidget.h>

class FolderListWidget::Private
{
  public:

    QString root;
};

FolderListWidget::FolderListWidget
(
  const QString & root,
  QWidget       * parent,
  const char    * name
)
  : QListView(parent, name)
{
  d = new Private;
  d->root = root;

  if (d->root[0] == '/')
    d->root = d->root.mid(1);

  addColumn("");
  header()->hide();

  setRootIsDecorated(true);

  connect(this,
          SIGNAL(selectionChanged(QListViewItem*)),
          SLOT(slotFolderChanged(QListViewItem*))
         );
}

FolderListWidget::~FolderListWidget()
{
  delete d;
}

  void
FolderListWidget::fill(const QStringList & pathList)
{
  clear();

  for (
    QStringList::ConstIterator pathIt(pathList.begin());
    pathIt != pathList.end();
    ++pathIt
  )
  {
    findOrCreateFolderItem(*pathIt);
  }
}

  void
FolderListWidget::fill()
{
  clear();

  VFSModule *vfs=0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
    else
    {
      qDebug("FolderListWidget: Can't find VFS module");
      return;
    }
  }

  QString realRoot = "/Aethera/VFS/" + d->root;

  QStringList pathList;

  QDict<VFolder> folderDict = vfs->folderDict();

  QDictIterator<VFolder> it(folderDict);

  for (; it.current(); ++it)
  {
    QString path(it.current()->path());

    if (
          path.left(realRoot.length()) == realRoot
//      &&  path != realRoot
    )
    {
      pathList << it.current()->path();
    }
  }

  pathList.sort();

  for (
    QStringList::ConstIterator pathIt(pathList.begin());
    pathIt != pathList.end();
    ++pathIt
  )
  {
    findOrCreateFolderItem(*pathIt);
  }
}


  QListViewItem *
FolderListWidget::findOrCreateFolderItem(const QString & path)
{
  QStringList componentList = QStringList::split('/', path);

  if (componentList.size() < 2)
    return 0;

  componentList.remove(componentList.begin());    // "Aethera"
  componentList.remove(componentList.begin());    // "VFS"

  if (componentList.isEmpty())
    return 0;

  QListViewItem * root = findOrCreateChild(0, componentList.first());

  QStringList::ConstIterator it(componentList.begin());

  ++it;

  QListViewItem * i = root;

  for (; it != componentList.end(); ++it)
  {
    i = findOrCreateChild(i, *it);
  }

  return i;
}

  QListViewItem *
FolderListWidget::findOrCreateChild(QListViewItem * root, const QString & text)
{
  QListViewItem * i;

  if (0 == root)
    i = firstChild();
  else
    i = root->firstChild();

  while (0 != i)
  {
    if (i->text(0) == text)
      return i;

    i = i->nextSibling();
  }

  QListViewItem * ret;

  if (0 == root)
    ret = new QListViewItem(this, text);
  else
    ret = new QListViewItem(root, text);

  ret->setOpen(true);

  return ret;
}

  QString
FolderListWidget::vfsPath(QListViewItem * i) const
{
  if (0 == i)
    return QString::null;

  QString path(i->text(0));

  while (i->depth() > 0)
  {
    i = i->parent();
    path.prepend(i->text(0) + "/");
  }

  path.prepend("/Aethera/VFS/");

  return path;
}


void FolderListWidget::slotFolderChanged(QListViewItem *it)
{
    QString folderURL=vfsPath(it);

    emit fullPathText(folderURL);
}
