/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  qcolorbutton.cpp
  -  Color selection widget.
                             -------------------
    begin                : Friday 02 May 2003
    copyright            : (C) 2003 by theKompany.com
    author               : Eugen Constantinescu <eug@thekompany.com>
    author               : Ilie <ilie@eco-spaces.com.ro>
 ***************************************************************************/

#include "fontcolorbutton.h"

//tkcBase includes
#include<qiconloader.h>

//QT includes
#include <qpainter.h>
#include <qcolordialog.h>
#include <qfont.h>
#include <qpopupmenu.h>
#include <qrect.h>
#include <qcursor.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qptrlist.h>


/*
----------------------------------------------------------------------------
The color table
----------------------------------------------------------------------------
*/

struct ColorMatrixStruct {
  Q_UINT8 r;
  Q_UINT8 g;
  Q_UINT8 b;
  const char *name;
};

static ColorMatrixStruct colorTable[100] =
{
  {   0,   0,   0, "Black" },
  { 255, 255, 255, "White" },
  {   0,   0, 255, "Blue" },
  {   0, 255, 255, "Cyan" },
  {   0, 255,   0, "Green" },
  { 255, 255,   0, "Yellow" },
  { 255,   0,   0, "Red" },
  {  25,  25,  25, "90% Black" },
  {  51,  51,  51, "80% Black" },
  {  76,  76,  76, "70% Black" },
  { 102, 102, 102, "60% Black" },
  { 127, 127, 127, "50% Black" },
  { 153, 153, 153, "40% Black" },
  { 178, 178, 178, "30% Black" },
  { 204, 204, 204, "20% Black" },
  { 229, 229, 229, "10% Black" },
  { 255,   0, 255, "Magenta" },
  { 153,   0, 204, "Purple" },
  { 255, 102,   0, "Orange" },
  { 255, 153, 204, "Pink" },
  { 102,  51,  51, "Dark Brown" },
  { 204, 204, 255, "Powder Blue" },
  { 153, 153, 255, "Pastel Blue" },
  { 102, 153, 255, "Baby Blue" },
  { 102, 102, 255, "Electric Blue" },
  { 102, 102, 204, "Twilight Blue" },
  {   0,  51, 153, "Navy Blue" },
  {   0,   0, 102, "Deep Navy Blue" },
  {  51, 102, 153, "Desert Blue" },
  {   0, 204, 255, "Sky Blue" },
  { 153, 255, 255, "Ice Blue" },
  { 153, 204, 204, "Light BlueGreen" },
  { 102, 153, 153, "Ocean Green" },
  {  51, 102, 102, "Moss Green" },
  {   0,  51,  51, "Dark Green" },
  {   0, 102,  51, "Forest Green" },
  {   0, 153,  51, "Grass Green" },
  {  51, 153, 102, "Kentucky Green" },
  {  51, 204, 102, "Light Green" },
  {  51, 204,  51, "Spring Green" },
  { 102, 255, 204, "Turquoise" },
  {  51, 204, 153, "Sea Green" },
  { 153, 204, 153, "Faded Green" },
  { 204, 255, 204, "Ghost Green" },
  { 153, 255, 153, "Mint Green" },
  { 102, 153, 102, "Army Green" },
  { 102, 153,  51, "Avocado Green" },
  { 153, 204,  51, "Martian Green" },
  { 153, 204, 102, "Dull Green" },
  { 153, 255,   0, "Chartreuse" },
  { 204, 255, 102, "Moon Green" },
  {  51,  51,   0, "Murky Green" },
  { 102, 102,  51, "Olive Drab" },
  { 153, 153, 102, "Khaki" },
  { 153, 153,  51, "Olive" },
  { 204, 204,  51, "Banana Yellow" },
  { 255, 255, 102, "Light Yellow" },
  { 255, 255, 153, "Chalk" },
  { 255, 255, 204, "Pale Yellow" },
  { 153, 102,  51, "Brown" },
  { 204, 102,  51, "Red Brown" },
  { 204, 153,  51, "Gold" },
  { 255, 102,  51, "Autumn Orange" },
  { 255, 153,  51, "Light Orange" },
  { 255, 153, 102, "Peach" },
  { 255, 204,   0, "Deep Yellow" },
  { 255, 204, 153, "Sand" },
  { 102,  51,   0, "Walnut" },
  { 153,   0,   0, "Ruby Red" },
  { 204,  51,   0, "Brick Red" },
  { 255, 102, 102, "Tropical Pink" },
  { 255, 153, 153, "Soft Pink" },
  { 255, 204, 204, "Faded Pink" },
  { 153,  51, 102, "Crimson" },
  { 204,  51, 102, "Regal Red" },
  { 204,  51, 153, "Deep Rose" },
  { 255,   0, 102, "Neon Red" },
  { 255, 102, 153, "Deep Pink" },
  { 255,  51, 153, "Hot Pink" },
  { 204, 102, 153, "Dusty Rose" },
  { 102,   0, 102, "Plum" },
  { 153,   0, 153, "Deep Violet" },
  { 255, 153, 255, "Light Violet" },
  { 204, 102, 204, "Violet" },
  { 153, 102, 153, "Dusty Plum" },
  { 204, 153, 204, "Pale Purple" },
  { 153,  51, 204, "Majestic Purple" },
  { 204,  51, 255, "Neon Purple" },
  { 204, 102, 255, "Light Purple" },
  { 153, 102, 204, "Twilight Violet" },
  { 204, 153, 255, "Easter Purple" },
  {  51,   0, 102, "Deep Purple" },
  { 102,  51, 153, "Grape" },
  { 153, 102, 255, "Blue Violet" },
  { 153,   0, 255, "Blue Purple" },
  { 102,   0, 204, "Deep River" },
  { 102,  51, 255, "Deep Azure" },
  {  51,   0, 153, "Storm Blue" },
  {  51,   0, 204, "Deep Blue" },
  {   0,   0,   0, "" }
};

ColorMatrix::ColorMatrix()
{
}

ColorMatrix::~ColorMatrix()
{
}

QColor ColorMatrix::color( int index )
{
  return QColor( colorTable[index].r,
                 colorTable[index].g,
                 colorTable[index].b
               );
}

QString ColorMatrix::colorName( int index )
{
  return QString( colorTable[index].name );
}
/*
----------------------------------------------------------------------------
The color cell widget
----------------------------------------------------------------------------
*/

ColorCell* ColorCell::previousSelectedCell = 0L;

ColorCell::ColorCell( QColor color, const QString& colorName, int width, int height, QWidget* parent , const char* name)
  : QWidget( parent, name ),
    color_( color ),
    name_( colorName ),
    mouseOver( false ),
    isSelected( false )
{
  setFixedWidth( width );
  setFixedHeight( height );

  setMouseTracking( true );

  QToolTip::add( this, tr(name_) );
}


ColorCell::~ColorCell()
{
}

void ColorCell::paintEvent( QPaintEvent* )
{
  QPixmap buffer(size());
  buffer.fill(backgroundColor());
  QPainter p(&buffer);

  QRect rect( 0, 0, width(), height() );

  p.fillRect( rect, color_ );
  p.drawRect( rect );


  if( mouseOver )
  {
    p.setPen( QColor("black") );
    p.drawLine( 1, 1, 1, height()-1  );
    p.drawLine( 1, 1, width()-1, 1 );

    p.setPen( QColor("white") );
    p.drawLine( width()-1, 1, width()-1, height()-1 );
    p.drawLine( 1, height()-1, width()-1, height()-1 );
    
  }
  else if( isSelected )
  {
    p.setPen( QColor("black") );
    p.drawLine( 1, 1, 1, height()-1  );
    p.drawLine( 1, 1, width()-1, 1 );

    QRect rect( 8, 8, 12, 12 );
    p.fillRect( rect, QColor("black") );
    p.drawRect( rect );

    p.setPen( QColor("white") );
    p.drawLine( width()-1, 1, width()-1, height()-1 );
    p.drawLine( 1, height()-1, width()-1, height()-1 );
  }
  else
  {
    QRect rect( 0, 0, width(), height() );

    p.fillRect( rect, color_ );
    p.drawRect( rect );
  }

  bitBlt(this, 0, 0, &buffer);
}

void ColorCell::mouseMoveEvent( QMouseEvent* )
{
  mouseOver = true;
  repaint( false );
}

void ColorCell::leaveEvent( QEvent* )
{
  mouseOver = false;
  repaint( false );
}

void ColorCell::mousePressEvent(QMouseEvent *)
{
  isSelected = true;
  emit( pressed() );
  emit( colorSelected(color_) );
  repaint( false );

  if( previousSelectedCell )
    previousSelectedCell->slotSetNormalState();

  previousSelectedCell = this;
}

void ColorCell::slotSetNormalState()
{
  mouseOver = false;
  isSelected = false;
  repaint( false );
}

/*
----------------------------------------------------------------------------
The color bar
----------------------------------------------------------------------------
*/

ColorMatrixWidget::ColorMatrixWidget( QWidget* parent, const char* name)
  : QWidget( parent, name ),
    currentColor( "black" )
{

  setFixedWidth( 130 );
  setFixedHeight( 150 );

  setCursor( QCursor(Qt::ArrowCursor) );

  QVBoxLayout* widgetLayout = new QVBoxLayout( this );
  QGridLayout* colorLayout = new QGridLayout( this, 13, 13, 0, -1, "widgetLayout" );
  widgetLayout->addLayout( colorLayout );

  for( int i=0; i<10; i++ )
    for( int j=0; j<10; j++ )
    {
      ColorCell* cell = new ColorCell( ColorMatrix::color( i*10+j ),
                                       ColorMatrix::colorName( i*10+j ),
                                       13,
                                       13,
                                       this,
                                       "colorCell"
                                      );
      connect( cell,
               SIGNAL( colorSelected(const QColor&) ),
               this,
               SLOT( slotCellColorSelected(const QColor&) )
             );
      colorCellList.append( cell );
      colorLayout->addWidget( cell, i, j );
    }

  QPushButton* moreColorsButton = new QPushButton( this, "moreColorButton" );
  widgetLayout->addWidget( moreColorsButton );
  moreColorsButton->setText( "More..." );

  connect( moreColorsButton,
           SIGNAL( clicked() ),
           this,
           SLOT( slotOpenColorDialog() )
         );
}

ColorMatrixWidget::~ColorMatrixWidget()
{
}

void ColorMatrixWidget::slotCellColorSelected( const QColor& color )
{
  currentColor = color;
  emit( colorSelected(color) );
}

void ColorMatrixWidget::slotOpenColorDialog()
{
  QColor c( QColorDialog::getColor( currentColor, this, "FontColorButton's dialog") );

  if( c.isValid() && c!=currentColor )
    emit( colorSelected(c) );

  currentColor = c;

  QPtrListIterator<ColorCell> it( colorCellList );

  for( ; it.current(); ++it)
  {
    (*it)->slotSetNormalState();
  }
}


/*
----------------------------------------------------------------------------
The button
----------------------------------------------------------------------------
*/
class FontColorButton::Private
{
  public:
    QColor color_;
    QPopupMenu* popupMenu;
    ColorMatrixWidget* colorTableWidget;
};



FontColorButton::FontColorButton(QWidget * parent, const char * name)
  : QPushButton(parent, name)
{
  d = new Private();

  setFixedWidth( 40 );
  
  QFont widgetFont;
  widgetFont.setBold( true );
  widgetFont.setItalic( true );
  setFont( widgetFont );

  setText( "Aa" );
  
  d->popupMenu = new QPopupMenu( this, "d->popupMenu" );
  setPopup( d->popupMenu );
  d->colorTableWidget = new ColorMatrixWidget( d->popupMenu, "d->colorTableWidget" );
  d->popupMenu->insertItem( d->colorTableWidget );

  connect( d->colorTableWidget,
           SIGNAL( colorSelected(const QColor&) ),
           this,
           SLOT( setColor(const QColor&) )
         );

  connect( d->colorTableWidget,
           SIGNAL( colorSelected(const QColor&) ),
           d->popupMenu,
           SLOT( hide() )
         );
}

FontColorButton::~FontColorButton()
{
}

  QColor
FontColorButton::color() const
{
  return d->color_;
}

  void
FontColorButton::setColor(const QColor & c)
{
  d->color_ = c;
  setPaletteForegroundColor( d->color_ );

  emit(colorChanged( d->color_ ));
  update();
}





