/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    fontcolorbutton.h
    -  Color selection widget.
                             -------------------
    begin                : Friday 02 May 2003
    copyright            : (C) 2003 by theKompany.com
    author               : Eugen Constantinescu <eug@thekompany.com>
    author               : Iulian Ghiciuc <ilie@eco-spaces.com.ro>
 ***************************************************************************/

#ifndef FONT_COLOR_BUTTON_H
#define FONT_COLOR_BUTTON_H

#include <qpushbutton.h>
#include <qcolor.h>
#include <qptrlist.h>

#include <defines.h>

/**
 * FontColorButton provides a QPushButton which displays and may be used to select
 * a color. Pressing the button will create a modal QColorDialog from which
 * the user may choose a color. If a color is chosen, the front face of the
 * button will display that color.
 */

class AETHERA_API FontColorButton : public QPushButton
{
  Q_OBJECT
  Q_PROPERTY(QColor color READ color WRITE setColor);

  public:

    FontColorButton(QWidget * parent = 0, const char * name = 0);
    virtual ~FontColorButton();

    /**
     * @return the stored color.
     */
    QColor color() const;

  public slots:
  
    /**
     * Set the stored color and change the appearance of the front face.
     */
    virtual void setColor(const QColor &);
    
  signals:

    void colorChanged(const QColor &);

  private:
    class Private;
    Private* d;
};

class ColorCell : public QWidget
{
  Q_OBJECT

  public:
    ColorCell( QColor color,
               const QString& colorName,
               int width=10,
               int height=10,
               QWidget* parent=0,
               const char* name=0
              );
    ~ColorCell();

  public slots:
    void slotSetNormalState();

  protected:
    virtual void paintEvent( QPaintEvent* );
    virtual void mouseMoveEvent( QMouseEvent* );
    virtual void mousePressEvent(QMouseEvent *);
    virtual void leaveEvent( QEvent* );

  signals:
    void pressed();
    void colorSelected( const QColor& );

  private:
    QColor color_;
    QString name_;

    bool mouseOver;
    bool isSelected;

    static ColorCell* previousSelectedCell;
};
class ColorMatrixWidget : public QWidget
{
  Q_OBJECT
  
  public:
    ColorMatrixWidget( QWidget* parent=0, const char* name=0 );
    ~ColorMatrixWidget();

  public slots:
    void slotCellColorSelected( const QColor& );
    void slotOpenColorDialog();

  signals:
    void colorSelected( const QColor& );

  private:
    QPtrList<ColorCell> colorCellList;
    QColor currentColor;
};

class ColorMatrix
{
  public:
    ColorMatrix();
    ~ColorMatrix();

    static QColor color(int index);
    static QString colorName(int index);
};
#endif
