/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */


#include <groupobj.h>
#include <qvariant.h>
#include <queryprocessor.h>
#include <vfsmodule.h>
#include <any.h>

#define IDSTRING "GroupObject: "

// #define DEBUG_GO

GroupObject::GroupObject(const QString & path)
  : VObject(path)
{
#ifdef DEBUG_GO
	qDebug(IDSTRING"constructor: syncing object...");
#endif	
	// Restore the object properties from the server.
  sync();
	
#ifdef DEBUG_GO
	qDebug(IDSTRING"contructor: done");
#endif
}

GroupObject::GroupObject(const QString & path, const QByteArray & data)
  : VObject(path)
{
#ifdef DEBUG_GO
	qDebug(IDSTRING"constructor: syncing object...");
#endif	

	// Restore the object properties from the server.

	if (!data || data.isEmpty())
	  sync();
	else
	  _narrow(data);
	
#ifdef DEBUG_GO
	qDebug(IDSTRING"contructor: done");
#endif
}

GroupObject::~GroupObject()
{
#ifdef DEBUG_GO
	qDebug(IDSTRING"destructor: syncing object...");
#endif	
}

/**
 *	Methods for accessing the fields
 */
QString GroupObject::groupName() const
{
	return groupName_;
}

void GroupObject::setGroupName(const QString & newName)
{
	groupName_ = newName;
}

QStringList GroupObject::members() const
{
	return members_;
}

void GroupObject::setMembers(const QStringList & newMembers)
{
	members_ = newMembers;
}

QStringList GroupObject::classes() const
{
	return classes_;
}

void GroupObject::setClasses(const QStringList & newClasses)
{
	classes_ = newClasses;
}

QString GroupObject::icon() const
{
	return icon_;
}

void GroupObject::setIcon(const QString & newIcon)
{
	icon_ = newIcon;
}

QByteArray GroupObject::_stream()
{
	QByteArray tdata;
	QDataStream tstream(tdata, IO_WriteOnly);
	
	tstream << groupName_ << members_ << classes_ << category_ << icon_;
	
	return tdata;
}

void GroupObject::_narrow(const QByteArray & tdata)
{
	if (!tdata.isEmpty())
	{
		QDataStream tstream(tdata, IO_ReadOnly);

		// unstream the object properties
		tstream >> groupName_ >> members_ >> classes_ >> category_ >> icon_;

#ifdef DEBUG_GO
		qDebug(IDSTRING"Object read : %d", tdata.size());
#endif
	}
	else
	{
		groupName_ = category_ = icon_ = QString::null;

		members_ = QStringList();
		classes_ = QStringList();

		qDebug(IDSTRING"Empty object!");
	}
}


bool GroupObject::getProperty(const QString & query, Any & property) const
{
	if (query == "groupname")
	{
		property = Any(groupName_);
		return true;
	}
	if (query == "members")
	{
		property = Any(members_);
		return true;
	}
	if (query == "classes")
	{
		property = Any(classes_);
		return true;
	}
	if (query == "icon")
	{
		property = Any(icon_);
		return true;
	}
	if (query == "category")
	{
		property = Any(category_);
		return true;
	}
	return false;
}

bool GroupObject::setProperty(const QString & query, Any & property)
{
	if (query == "groupname")
	{
		groupName_ = property.toString();
		return true;
	}
	if (query == "members")
	{
		members_ = property.toStringList();
		return true;
	}
	if (query == "classes")
	{
		classes_ = property.toStringList();
		return true;
	}
	if (query == "icon")
	{
		icon_ = property.toString();
		return true;
	}
	if (query == "category")
	{
		category_ = property.toString();
		return true;
	}
	return false;
}

QueryProcessor::QueryType GroupObject::queryType(const QString & query)
{
	if (
        query == "name"
    ||  query == "classes"
    ||  query == "members"
    ||  query == "category"
  )
  {
		return QueryProcessor::ReadWriteQuery;
  }

	return QueryProcessor::InvalidQuery;
}


VObject *GroupObjectFactory::_new(const QString & _path)
{
  VFSModule *vfs=0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
    else
    {
      qDebug("Group Object: Can't find VFS module");
      return 0;
    }
  }

  // check if it is possible to create a new contact
  if(vfs->canDefine(_path) && !vfs->reference(_path))
    return new GroupObject(_path);
  else
    return 0;
}

VObject *GroupObjectFactory::_new(const QString & _path, const QByteArray &_data)
{
  return new GroupObject(_path, _data);
}



