/*
    Copyright (C) 2001-2004 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

/***************************************************************************
      kolabsharedfolder.h  -  utilities for kolab imap shared folders
                             -------------------
    begin                : jeu jun 3 2004
    
    copyright            : (C) 2004 by ATOL Conseils et dveloppements
                           <http://www.atolcd.com>
    copyright            : (C) 2004-2005 by theKompany.com
    
    author               : Fabien Armand <f.armand@atolcd.com>
    author               : Eugen Constantinescu <eug@thekompany.com>
 ***************************************************************************/

#include "kolabsharedfolder.h"

#include <vfsmodule.h>
#include <clientrmi.h>
#include <vfolder.h>
#include <folderchooserdialog.h>

#include "../plugins/kommailplugin/mailagentservice.h"
#include "../plugins/kommailplugin/mailobj.h"

#include <qmessagebox.h>

// #define DEBUG_Kolab

/** The singleton Kolab Shared Folder instance.*/

KolabSharedFolder *KolabSharedFolder::thisInstance = 0;

class KolabSharedFolderPrivate
{
  public:
  
    KolabSharedFolderPrivate();
    
    ~KolabSharedFolderPrivate();
    
    VFSModule * vfs;
    ClientRMIModule * clientRMI;
    MailAgentService * mailAgent;
    
    QString defaultType;
    
  private:
};

KolabSharedFolderPrivate::KolabSharedFolderPrivate()
  : vfs(0),
    clientRMI(0),
    mailAgent(0),
    defaultType("unknow")
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
      if( ref.isValid() )
        clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILAGENT_SERVICE );
    if( ref.isValid() )
    {
      mailAgent = static_cast<MailAgentService*>( ref.instance() );
    }
  }
}

KolabSharedFolderPrivate::~KolabSharedFolderPrivate()
{
}
  

KolabSharedFolder::KolabSharedFolder()
  : QObject()
{
  d = new KolabSharedFolderPrivate();
}

KolabSharedFolder::~KolabSharedFolder()
{
  delete d;
  d = 0;
}

KolabSharedFolder *KolabSharedFolder::ref()
{
  return thisInstance ? thisInstance : (thisInstance=new KolabSharedFolder());
}

QString KolabSharedFolder::type(const QString & account, const QString & mailbox) const
{
  return type(d->vfs->root() + "/Mail/" + account + "/" + mailbox);
}

QString KolabSharedFolder::type(const QString & folderPath) const
{
  #ifdef DEBUG_Kolab
  qDebug("KolabFolder::type() - Folder Path : " + folderPath);
  #endif

  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return QString("unknow");
  }
  
  QString accountName;

  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');

  int iFind = -1;
  QString accountPath;
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); ++itList)
  {
    accountPath = d->vfs->root() + "/Mail/" + *itList + "/";
    iFind = folderPath.find( accountPath );
    // iFind must be 0
    if( !iFind )
    {
      accountName = *itList;
      break;
    }
  }
  if( accountName.isEmpty() )
    return QString::null;
  
  // check the settings first
  QStringList foldersType;
  // Kolab main folders
  kolabFolders(foldersType, accountName, "calendar");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "calendar";
  }
  
  foldersType.clear();
  kolabFolders(foldersType, accountName, "todo");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "todo";
  }
  
  foldersType.clear();
  kolabFolders(foldersType, accountName, "contacts");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "contacts";
  }
  
  foldersType.clear();
  kolabFolders(foldersType, accountName, "notes");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "notes";
  }
  
/*  foldersType.clear();
  kolabFolders(foldersType, accountName, "jabber");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "jabber";
  }*/
  
  // Shared folders
  sharedFolders(foldersType, accountName, "calendar");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "calendar";
  }
  
  foldersType.clear();
  sharedFolders(foldersType, accountName, "todo");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "todo";
  }
  
  foldersType.clear();
  sharedFolders(foldersType, accountName, "contacts");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "contacts";
  }
  
  foldersType.clear();
  sharedFolders(foldersType, accountName, "notes");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "notes";
  }
  
  foldersType.clear();
  sharedFolders(foldersType, accountName, "jabber");
  if( -1 != foldersType.findIndex(folderPath) )
  {
    return "jabber";
  }
  
  VFolder * folder = d->vfs->folderRef(folderPath);
  if (folder)
  {
    QStringList urls = d->vfs->children(folderPath);

    if( urls.count() )
    {
      d->vfs->createVInstances(*urls.begin(), false);
  
      MailObject * object = static_cast<MailObject*> 
        (d->vfs->_fact_ref((*urls.begin()), VFS_MAILS_MIMETYPE));
      if(object)
      {
        if(object->contentType().contains("text/calendar"))
        {
          #ifdef DEBUG_Kolab
          qDebug(folderPath + " type calendar");
          #endif
          return QString("calendar");
        }
        else if(object->contentType().contains("text/x-vcard"))
        {
          #ifdef DEBUG_Kolab
          qDebug(folderPath + " type contact");
          #endif
          return QString("contacts");
        }
        else if( "Note-Id: " == object->subject().left(9) ||
                 object->contentType().contains("text/") )
        {
          #ifdef DEBUG_Kolab
          qDebug(folderPath + " type note");
          #endif
          if( d->defaultType == "jabber" )
          {
            QString body = object->text();
            if( body.startsWith("###tkJabberPro###") )
            {
              return QString("jabber");
            }
          }
          
          return QString("notes");
        }
        else
        {
          #ifdef DEBUG_Kolab
          qDebug(folderPath + " type unknow");
          #endif
          return QString("unknow");
        }
      }
      else
      {
        return QString("unknow");
      }
    }
    else
    {
      return d->defaultType;
    }
  }
  else
  {
    return QString("folder does not exist");
    #ifdef DEBUG_Kolab
    qDebug("folder does not exist");
    #endif
  }
}

void KolabSharedFolder::subscribeFolders(const QString & account)
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return;
  }
  
  QString mailbox="\"\"";
  d->clientRMI->subfolders(account, mailbox, "genmail");
}

void KolabSharedFolder::scannSharedFolders
  ( const QString & account,
    const QString & defaultType )
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return;
  }
  
  if( isKolabAccount(account) )
  {
    d->defaultType = defaultType;
    d->clientRMI->runKolabSharedJob(account, QString::null, "subscribe");
  }
  else
  {
    qDebug("scannSharedFolders : no %s Kolab account", account.latin1());
  }
}

void KolabSharedFolder::synchSharedFolders(const QString & account)
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return;
  }
  
  QStringList shared;
  VFolder * folderRef = d->vfs->folderRef( d->vfs->root() + "/Mail/" + account );

  bool bCitadel = isCitadelAccount(account);
  
  QStringList kfolders;
  kolabFolders(kfolders, account);
  sharedFolders(kfolders, account);
  
  if(folderRef)
  {
    VFolderList folderRefList;
    folderRef->tree(folderRefList, -1);
    
    for(VFolderListIterator it(folderRefList); it.current(); ++it)
    {
      QString folderPath=it.current()->path();
      // it has to not be allready Kolab shared or main folder
      // for Kolab1, it has to start with "user."
      if( -1 == kfolders.findIndex(folderPath) &&
          (bCitadel || !folderPath.find(d->vfs->root() + "/Mail/" + account + "/user.")) )
      {
        shared += folderPath;
      }
    }
    
    if( shared.count() )
    {
      FolderChooserDialog::getFolders(shared, folderRef->path(), tr("Choose Shared Folders"), 0);
    }
    else
    {
      QMessageBox::warning(0, tr("Shared Folders"),
        tr("There aren't shared folders available for account ") + account);
    }
    
    if( shared.count() )
    {
      for(QStringList::Iterator it=shared.begin(); it!=shared.end(); ++it)
      {
        // add them temporary
        addSharedFolder(account, *it, QString::null);
      }
      
      // synch all the shared folders
      shared.clear();
      sharedFolders(shared, account);
      
      for(QStringList::Iterator it=shared.begin(); it!=shared.end(); ++it)
      {
        // synchronize with the server
        d->clientRMI->mount(*it);
        d->clientRMI->unmount(*it);
      }
    }
  }
}

void KolabSharedFolder::createSharedFolders(const QString & account)
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return;
  }
  
  VFolder * folderRef = d->vfs->folderRef( d->vfs->root() + "/Mail/" + account );

  if(folderRef)
  {
    VFolderList folderRefList;
    folderRef->tree(folderRefList, -1);
    
    for(VFolderListIterator it(folderRefList); it.current(); ++it)
    {
      QString folderPath=it.current()->path();
      
      if(isKolabSharedFolder(folderPath))
      {
        // to know the type of the folder (calendar, notes, contact)
        QString typeRef = type(folderPath);
        if(typeRef=="calendar")
        {
          // call the method to create the folder in the datebook
          createSharedCalendarFolders(it.current()->path());
        }
        else if(typeRef=="contacts")
        {
          // call the method to create the folder in the addressbook
          createSharedContactFolders(it.current()->path());
        }
        else if(typeRef=="notes")
        {
          // call the method to create the folder in the notebook
          createSharedNoteFolders(it.current()->path());
        }
        else if(typeRef=="jabber")
        {
          // call the method to create the folder in the jabber client
          createSharedJabberFolders(it.current()->path());
        }
        else
        {
          typeRef = QString::null;
          // ask OR remove it from shared folders
          // set typeRef !
        }
        
        if( typeRef.isEmpty() )
          removeSharedFolder(account, it.current()->path());
        else
          addSharedFolder(account, it.current()->path(), typeRef);
      }
    }
  }
  else
  {
    #ifdef DEBUG_Kolab
    qDebug("KolabFolder::createSharedFolders Couldn't find main folder");
    #endif
  }
  
  Settings::forceWrite();
}

/** method createSharedContactFolders
    arguments: a folder path
    ( ex: /Aethera/VFS/Mail/fabien.armand/user.calendar )
    return: void
*/

void KolabSharedFolder::createSharedCalendarFolders(const QString & path)
{
  QString accountName;
  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  QString emailRoot = d->vfs->root() + "/Mail/";
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); ++itList)
  {
    // iFind must be 0
    if( !path.find( emailRoot + (*itList) + "/" ) )
    {
      accountName = *itList;
      break;
    }
  }

  if( !accountName.isEmpty() )
  {
    VFolder * folderRef = d->vfs->folderRef(d->vfs->root() + "/Datebook/Kolab");
    VFolder * folderRefNext = 0;
    // folderPath = account/mailbox
    QString folderPath = path.mid( emailRoot.length() );

    #ifdef DEBUG_Kolab
    qDebug("folderPath name: " + folderPath);
    #endif

    QString folderName;
    while(folderRef && folderPath.contains("/"))
    {
      // while a folder is not the last child, create the parent folder
      // for example : user.calendar/fabien.armand/myCalendar
      // first iteration
      // user.calendar is created ( if it doesn't exist )
      // second iteration
      // fabien.armand is created ( if it doesn't exist )

      folderName = folderPath.mid(0,folderPath.find("/"));

      folderRefNext = d->vfs->folderRef(folderRef->path() + "/" + folderName);
      if( !folderRefNext )
      {
        folderRefNext = d->vfs->createFolder(folderRef, folderName, "datebook", "datebook");
      }

      folderPath=folderPath.mid(folderPath.find("/") + 1);
      folderRef = folderRefNext;
    }

    if( folderRef && !d->vfs->folderRef(folderRef->path() + folderName) )
    {
      folderRef = d->vfs->createFolder(folderRef, folderPath, "datebook", "datebook");
    }
    
    if( !folderRef )
    {
      qDebug("Cannot create calendar shared folder.");
    }
  }
  else
  {
    qDebug("There isn't a Kolab account for this folder.");
  }
}

/** method createSharedContactFolders
    arguments: a folder path
    ( ex: /Aethera/VFS/Mail/fabien.armand/user.contacts )
    return: void
*/

void KolabSharedFolder::createSharedContactFolders(const QString & path)
{
  QString accountName;
  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  QString accountPath;
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); ++itList)
  {
    accountPath = d->vfs->root() + "/Mail/" + *itList + "/";
    // iFind must be 0
    if( !path.find( accountPath ) )
    {
      accountName = *itList;
      break;
    }
  }

  if( !accountName.isEmpty() )
  {
    VFolder * folderRef = d->vfs->folderRef(d->vfs->root() + "/Contacts/Kolab");
    VFolder * folderRefNext = 0;
    QString folderPath = path.mid( QString(d->vfs->root() + "/Mail/").length() );

    #ifdef DEBUG_Kolab
    qDebug("folderPath name: %s", folderPath.latin1());
    #endif

    QString folderName;
    while(folderRef && folderPath.contains("/"))
    {
      // while a folder is not the last child, create the parent folder
      // for example : user.contacts/fabien.armand/myContacts
      // first iteration
      // user.contacts is created ( if it doesn't exist )
      // second iteration
      // fabien.armand is created ( if it doesn't exist )

      folderName = folderPath.mid(0,folderPath.find("/"));

      folderRefNext = d->vfs->folderRef(folderRef->path() + "/" + folderName);
      if( !folderRefNext )
      {
        folderRefNext = d->vfs->createFolder(folderRef, folderName, "contacts", "contacts");
      }

      folderPath = folderPath.mid(folderPath.find("/") + 1);
      folderRef = folderRefNext;
    }

    if( folderRef && !d->vfs->folderRef(folderRef->path() + folderName) )
    {
      folderRef = d->vfs->createFolder(folderRef, folderPath, "contacts", "contacts");
    }
    
    if( !folderRef )
    {
      qDebug("Cannot create contacts shared folder.");
    }
  }
  else
  {
    qDebug("There isn't a Kolab account for this folder.");
  }
}

/** method createSharedContactFolders
    arguments: a folder path
    ( ex: /Aethera/VFS/Mail/fabien.armand/user.notes )
    return: void
*/

void KolabSharedFolder::createSharedNoteFolders(const QString & path)
{
  QString accountName;
  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  QString accountPath;
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); ++itList)
  {
    accountPath = d->vfs->root() + "/Mail/" + *itList + "/";
    // iFind must be 0
    if( !path.find(accountPath) )
    {
      accountName = *itList;
      break;
    }
  }

  if( !accountName.isEmpty() )
  {
    VFolder * folderRef = d->vfs->folderRef(d->vfs->root() + "/Notes/Kolab");
    VFolder * folderRefNext = 0;
    QString folderPath = path.mid( QString(d->vfs->root() + "/Mail/").length() );

    #ifdef DEBUG_Kolab
    qDebug("folderPath name: " + folderPath);
    #endif

    QString folderName;
    while(folderRef && folderPath.contains("/"))
    {
      // while a folder is not the last child, create the parent folder
      // for example : user.notes/fabien.armand/myNotes
      // first iteration
      // user.notes is created ( if it doesn't exist )
      // second iteration
      // fabien.armand is created ( if it doesn't exist )

      folderName = folderPath.mid(0,folderPath.find("/"));

      folderRefNext = d->vfs->folderRef(folderRef->path() + "/" + folderName);
      if( !folderRefNext )
      {
        folderRefNext = d->vfs->createFolder(folderRef, folderName, "notes", "notes");
      }

      folderPath=folderPath.mid(folderPath.find("/") + 1);
      folderRef = folderRefNext;
    }

    if( folderRef && !d->vfs->folderRef(folderRef->path()+folderName) )
    {
      folderRef = d->vfs->createFolder(folderRef, folderPath, "notes", "notes");
    }
    
    if( !folderRef )
    {
      qDebug("Cannot create notes shared folder.");
    }
  }
  else
  {
    qDebug("There isn't a Kolab account for this folder.");
  }
}

void KolabSharedFolder::createSharedJabberFolders(const QString & path)
{
  QString accountName;
  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  QString accountPath;
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); ++itList)
  {
    accountPath = d->vfs->root() + "/Mail/" + *itList + "/";
    // iFind must be 0
    if( !path.find(accountPath) )
    {
      accountName = *itList;
      break;
    }
  }

  if( !accountName.isEmpty() )
  {
    VFolder * folderRef = d->vfs->folderRef(d->vfs->root() + "/Jabber/Kolab");
    VFolder * folderRefNext = 0;
    QString folderPath = path.mid( QString(d->vfs->root() + "/Mail/").length() );

    #ifdef DEBUG_Kolab
    qDebug("folderPath name: " + folderPath);
    #endif

    QString folderName;
    while(folderRef && folderPath.contains("/"))
    {
      folderName = folderPath.mid(0,folderPath.find("/"));

      folderRefNext = d->vfs->folderRef(folderRef->path() + "/" + folderName);
      if( !folderRefNext )
      {
        folderRefNext = d->vfs->createFolder(folderRef, folderName, "jabber", "jabber");
      }

      folderPath=folderPath.mid(folderPath.find("/") + 1);
      folderRef = folderRefNext;
    }

    if( folderRef && !d->vfs->folderRef(folderRef->path()+folderName) )
    {
      folderRef = d->vfs->createFolder(folderRef, folderPath, "jabber", "jabber");
    }
    
    if( !folderRef )
    {
      qDebug("Cannot create jabber shared folder.");
    }
  }
  else
  {
    qDebug("There isn't a Kolab account for this folder.");
  }
}

// method for testing if an imap  folder is a Kolab Shared Folder
// (ex: user.calendar)

bool KolabSharedFolder::isKolabSharedFolder
  (const QString & account, const QString & mailbox) const
{
  QStringList sharedList;
  sharedFolders(sharedList, account);
  
  if( -1 == sharedList.findIndex(d->vfs->root() + "/Mail/" + account + "/" + mailbox) )
    return false;
  else
    return true;
}

bool KolabSharedFolder::isKolabSharedFolder(const QString & path) const
{
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end();++itList)
  {
    if( !path.find(d->vfs->root() + "/Mail/" + *itList + "/") )
    {
      QStringList sharedList;
      sharedFolders(sharedList, *itList);
      
      if( -1 == sharedList.findIndex(path) )
        return false;
      else
        return true;
    }
  }

  return false;
}

bool KolabSharedFolder::isKolabFolder(const QString & account, const QString & mailbox) const
{
  return isKolabFolder( d->vfs->root() + "/Mail/" + account + "/" + mailbox );
}

bool KolabSharedFolder::isKolabFolder(const QString & path) const
{
  QStringList kolabAccounts =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabAccounts.begin(); itList!=kolabAccounts.end(); ++itList)
  {
    if( !path.find(d->vfs->root() + "/Mail/" + *itList + "/") )
    {
      QStringList kolabList;
      kolabFolders(kolabList, *itList);
      
      if( -1 == kolabList.findIndex(path) )
        return false;
      else
        return true;
    }
  }

  return false;
}

bool KolabSharedFolder::isKolabParentFolder(const QString & path) const
{
  QStringList kolabAccounts =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabAccounts.begin(); itList!=kolabAccounts.end(); ++itList)
  {
    if( !path.find(d->vfs->root() + "/Mail/" + *itList + "/") )
    {
      QStringList kolabList;
      kolabFolders(kolabList, *itList);
      sharedFolders(kolabList, *itList);
      
      for(QStringList::Iterator it=kolabList.begin(); it!=kolabList.end(); ++it)
      {
        if( !(*it).find(path) )
        {
          return true;
        }
      }
    }
  }

  return false;
}

bool KolabSharedFolder::isKolabAccount(const QString & account) const
{
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  return ( -1 != kolabList.findIndex(account) );
}

bool KolabSharedFolder::isCitadelAccount(const QString & account) const
{
  QStringList citadelList =
    Settings::instance()->readListEntry("/Aethera/Citadel/Accounts", ',');
  
  return ( -1 != citadelList.findIndex(account) );
}

void KolabSharedFolder::kolabFolders(QStringList & folders, const QString & account) const
{
  QString accountFolder = d->vfs->root() + "/Mail/" + account + "/";
  QString group = QString("/Aethera/Accounts/") + account + "/Kolab/";
  
  folders.append( accountFolder + Settings::instance()->readEntry(group + "Calendar") );
  folders.append( accountFolder + Settings::instance()->readEntry(group + "Contacts") );
  folders.append( accountFolder + Settings::instance()->readEntry(group + "Notes") );
  folders.append( accountFolder + Settings::instance()->readEntry(group + "ToDo") );
}

void KolabSharedFolder::kolabFolders
  ( QStringList & folders,
    const QString & account,
    const QString & type ) const
{
  QString accountFolder = d->vfs->root() + "/Mail/" + account + "/";
  QString group = QString("/Aethera/Accounts/") + account + "/Kolab/";
  QString settingType = settingsType(type);
  if( !settingType.isEmpty() )
  {
    folders.append( accountFolder + Settings::instance()->readEntry(group + settingType) );
  }
}

bool KolabSharedFolder::isTempSharedFolder
  (const QString & account, const QString & mailbox) const
{
  QString group = QString("/Aethera/Accounts/") + account + "/Kolab/";
  QStringList sharedList =
    Settings::instance()->readListEntry(group + "SharedFolders/Temporary", '^');
  
  return( -1 != sharedList.findIndex(mailbox) );
}

void KolabSharedFolder::sharedFolders(QStringList & folders, const QString & account) const
{
  sharedFolders(folders, account, "calendar");
  sharedFolders(folders, account, "todo");
  sharedFolders(folders, account, "contacts");
  sharedFolders(folders, account, "notes");
  sharedFolders(folders, account, "jabber");
  sharedFolders(folders, account, QString::null);
}

void KolabSharedFolder::sharedFolders
  ( QStringList & folders,
    const QString & account,
    const QString & type ) const
{
  QString accountFolder = d->vfs->root() + "/Mail/" + account + "/";
  QString group = QString("/Aethera/Accounts/") + account + "/Kolab/";
  QString settingType = settingsType(type);
  if( settingType.isEmpty() )
    settingType = "/Temporary";
  else
    settingType.prepend("/");
  
  QStringList sharedList =
    Settings::instance()->readListEntry(group + "SharedFolders" + settingType, '^');
  
  for(QStringList::Iterator it=sharedList.begin(); it!= sharedList.end(); ++it)
  {
    folders.append( accountFolder + (*it) );
  }
}

bool KolabSharedFolder::checkKolabFolders()
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return false;
  }
  
  QStringList kolabList = 
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  QStringList accList = 
    Settings::instance()->readListEntry("/Aethera/Receive Options/Accounts", ',');
  
  QString currentAccount;
  
  // for all Kolab accounts
  for(QStringList::Iterator it=kolabList.begin(); it!=kolabList.end(); ++it)
  {
    currentAccount = (*it);
    // check if there is an account with this name
    if( -1 != accList.findIndex(currentAccount) )
    {
      d->clientRMI->runKolabJob(currentAccount, QString::null, "subscribe");
    }
    else
    {
      qDebug("wrong kolab account: %s", currentAccount.latin1());
    }
  }

  return true;
}

bool KolabSharedFolder::checkIMAPFolders(const QString & accName)
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return false;
  }
  
  QString group = QString("/Aethera/Accounts/") + accName + "/Kolab/";
  QString kolabFolder = d->vfs->root() + "/Mail/" + accName;
  
  // build the new IMAP folders from VFS
  VFolder * folderRef = d->vfs->folderRef(kolabFolder + "/Inbox");
  if( folderRef )
  {
    QString subfolder = Settings::instance()->readEntry(group + "Calendar");
    folderRef = d->vfs->folderRef(kolabFolder + "/" + subfolder);
    if( !folderRef )
    {
      createIMAPFolder(accName, kolabFolder + "/" + subfolder);
    }
    
    subfolder = Settings::instance()->readEntry(group + "Contacts");
    folderRef = d->vfs->folderRef(kolabFolder + "/" + subfolder);
    if( !folderRef )
    {
      createIMAPFolder(accName, kolabFolder + "/" + subfolder);
    }
    
    subfolder = Settings::instance()->readEntry(group + "Notes");
    folderRef = d->vfs->folderRef(kolabFolder + "/" + subfolder);
    if( !folderRef )
    {
      createIMAPFolder(accName, kolabFolder + "/" + subfolder);
    }
    
    subfolder = Settings::instance()->readEntry(group + "ToDo");
    folderRef = d->vfs->folderRef(kolabFolder + "/" + subfolder);
    if( !folderRef )
    {
      createIMAPFolder(accName, kolabFolder + "/" + subfolder);
    }
  }
  else
  {
    qDebug("No IMAP folder for Kolab account: %s", accName.latin1());
    return false;
  }
  
  return true;
}

bool KolabSharedFolder::checkVFSFolders(const QString & accName)
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return false;
  }
  
  // build Kolab folder for DateBook
  VFolder * folderRef = d->vfs->folderRef( d->vfs->root() + "/Datebook/Kolab");
  if( !folderRef )
  {
    folderRef = d->vfs->folderRef( d->vfs->root() + "/Datebook");
    if(folderRef)
    {
      folderRef = d->vfs->createFolder(folderRef, "Kolab", "datebook", "datebook");
      if( !folderRef )
      {
        qDebug("Kolab folder for Datebook cannot be created.");
      }
      else
      {
        // build the new calendar object for Kolab
      }
    }
  }
  
  // build Kolab folder for Notes
  folderRef = d->vfs->folderRef( d->vfs->root() + "/Notes/Kolab");
  if( !folderRef )
  {
    folderRef = d->vfs->folderRef( d->vfs->root() + "/Notes");
    if(folderRef)
    {
      folderRef = d->vfs->createFolder(folderRef, "Kolab", "notes", "notes");
      if( !folderRef )
      {
        qDebug("Kolab folder for Notes cannot be created.");
      }
    }
  }
  folderRef = d->vfs->folderRef( d->vfs->root() + "/Notes/Kolab/" + accName);
  if( !folderRef )
  {
    folderRef = d->vfs->folderRef( d->vfs->root() + "/Notes/Kolab");
    if(folderRef)
    {
      folderRef = d->vfs->createFolder(folderRef, accName, "notes", "notes");
      if( !folderRef )
      {
        qDebug("Kolab folder for Notes cannot be created.");
      }
    }
  }
  
  // build Kolab folder for Jabber
  folderRef = d->vfs->folderRef( d->vfs->root() + "/Jabber/Kolab");
  if( !folderRef )
  {
    folderRef = d->vfs->folderRef( d->vfs->root() + "/Jabber");
    if(folderRef)
    {
      folderRef = d->vfs->createFolder(folderRef, "Kolab", "jabber", "jabber");
      if( !folderRef )
      {
        qDebug("Kolab folder for Jabber cannot be created.");
      }
    }
  }
/*  folderRef = d->vfs->folderRef( d->vfs->root() + "/Jabber/Kolab/" + accName);
  if( !folderRef )
  {
    folderRef = d->vfs->folderRef( d->vfs->root() + "/Jabber/Kolab");
    if(folderRef)
    {
      folderRef = d->vfs->createFolder(folderRef, accName, "jabber", "jabber");
      if( !folderRef )
      {
        qDebug("Kolab folder for Jabber cannot be created.");
      }
    }
  }*/
  
  // build Kolab folder for Contacts
  folderRef = d->vfs->folderRef( d->vfs->root() + "/Contacts/Kolab");
  if( !folderRef )
  {
    folderRef = d->vfs->folderRef( d->vfs->root() + "/Contacts");
    if(folderRef)
    {
      folderRef = d->vfs->createFolder(folderRef, "Kolab", "contacts", "contacts");
      if( !folderRef )
      {
        qDebug("Kolab folder for Contacts cannot be created.");
      }
    }
  }
  folderRef = d->vfs->folderRef( d->vfs->root() + "/Contacts/Kolab/" + accName);
  if( !folderRef )
  {
    folderRef = d->vfs->folderRef( d->vfs->root() + "/Contacts/Kolab");
    if(folderRef)
    {
      folderRef = d->vfs->createFolder(folderRef, accName, "contacts", "contacts");
      if( !folderRef )
      {
        qDebug("Kolab folder for Contacts cannot be created.");
      }
    }
  }
  
  return true;
}

bool KolabSharedFolder::createIMAPFolder(const QString & accName, const QString & folderPath)
{
  if( !d->mailAgent )
  {
    qDebug("Email plugin isn't loaded.");
    return false;
  }
  
  QString parentFolder = d->vfs->parentFolder(folderPath);
  QString folderName = folderPath.mid( folderPath.findRev('/') + 1 );
  
  VFolder * folderRef = d->vfs->folderRef(parentFolder);
  if(folderRef)
  {
    folderRef = d->vfs->createFolder(folderRef, folderName, folderRef->type(), "mail");
  }
  else
  {
    qDebug("Kolab parent folder is missing for account %s.", accName.latin1());
  }
  
  return true;
}

bool KolabSharedFolder::addSharedFolder
  ( const QString & account,
    const QString & path,
    const QString & type )
{
  QString accountFolder = d->vfs->root() + "/Mail/" + account + "/";
  QString settingType = settingsType(type);
  if( settingType.isEmpty() )
    settingType = "/Temporary";
  else
    settingType.prepend("/");
  
  if( !path.find(accountFolder) )
  {
    QString group = QString("/Aethera/Accounts/") + account + "/Kolab/";
    QStringList folders = Settings::instance()->readListEntry
      (group + "SharedFolders" + settingType, '^');
      
    if( -1 == folders.findIndex( path.mid(accountFolder.length()) ) )
    {
      removeSharedFolder(account, path);
      
      folders = Settings::instance()->readListEntry
        (group + "SharedFolders" + settingType, '^');
      folders += path.mid( accountFolder.length() );
      Settings::instance()->writeEntry
        (group + "SharedFolders" + settingType, folders, '^');
    }
    
    return true;
  }
  
  return false;
}

bool KolabSharedFolder::removeSharedFolder(const QString & account, const QString & path)
{
  QString accountFolder = d->vfs->root() + "/Mail/" + account + "/";
  if( !path.find(accountFolder) )
  {
    QString group = QString("/Aethera/Accounts/") + account + "/Kolab/";
    QString settingType = "/Temporary";
    QStringList folders = Settings::instance()->readListEntry
      (group + "SharedFolders" + settingType, '^');
    if( folders.remove( path.mid(accountFolder.length()) ) )
    {
      Settings::instance()->writeEntry
        (group + "SharedFolders" + settingType, folders, '^');
    }
    
    settingType = "/Calendar";
    folders = Settings::instance()->readListEntry
      (group + "SharedFolders" + settingType, '^');
    if( folders.remove( path.mid(accountFolder.length()) ) )
    {
      Settings::instance()->writeEntry
        (group + "SharedFolders" + settingType, folders, '^');
    }
    
    settingType = "/ToDo";
    folders = Settings::instance()->readListEntry
      (group + "SharedFolders" + settingType, '^');
    if( folders.remove( path.mid(accountFolder.length()) ) )
    {
      Settings::instance()->writeEntry
        (group + "SharedFolders" + settingType, folders, '^');
    }
    
    settingType = "/Contacts";
    folders = Settings::instance()->readListEntry
      (group + "SharedFolders" + settingType, '^');
    if( folders.remove( path.mid(accountFolder.length()) ) )
    {
      Settings::instance()->writeEntry
        (group + "SharedFolders" + settingType, folders, '^');
    }
    
    settingType = "/Notes";
    folders = Settings::instance()->readListEntry
      (group + "SharedFolders" + settingType, '^');
    if( folders.remove( path.mid(accountFolder.length()) ) )
    {
      Settings::instance()->writeEntry
        (group + "SharedFolders" + settingType, folders, '^');
    }
    
    settingType = "/Jabber";
    folders = Settings::instance()->readListEntry
      (group + "SharedFolders" + settingType, '^');
    if( folders.remove( path.mid(accountFolder.length()) ) )
    {
      Settings::instance()->writeEntry
        (group + "SharedFolders" + settingType, folders, '^');
    }
    
    return true;
  }
  
  return false;
}

QString KolabSharedFolder::settingsType(const QString & type) const
{
  QString settingType;
  if( type == "calendar" )
  {
    settingType = "/Calendar";
  }
  else if( type == "todo" )
  {
    settingType = "/ToDo";
  }
  else if( type == "contacts" )
  {
    settingType = "/Contacts";
  }
  else if( type == "notes" )
  {
    settingType = "/Notes";
  }
  else if( type == "jabber" )
  {
    settingType = "/Jabber";
  }
  return settingType;
}

void KolabSharedFolder::subfolders
        ( QStringList & folders,
          const QString & account,
          const QString & mailbox ) const
{
  QString emailFolder;
  bool emptyMailbox = false;
  if( mailbox.isEmpty() || mailbox == "\"\"" )
  {
    emailFolder = d->vfs->root() + "/Mail/" + account;
    emptyMailbox = true;
  }
  else
  {
    emailFolder = d->vfs->root() + "/Mail/" + account + "/" + mailbox;
    emptyMailbox = false;
  }
  
  VFolder * folder = d->vfs->folderRef( emailFolder );
  if( folder )
  {
    unsigned uLength = QString(d->vfs->root() + "/Mail/" + account + "/").length();
    QStringList folderList;
    folder->tree(folderList, -1);
    for(QStringList::Iterator it=folderList.begin(); it!=folderList.end(); ++it)
    {
      folders.append( (*it).mid(uLength) );
    }
  }
}
