/*
    Copyright (C) 2001-2004 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

/***************************************************************************
      kolabsharedfolder.h  -  utilities for kolab imap shared folders
                             -------------------
    begin                : jeu jun 3 2004
    copyright            : (C) 2004 by ATOL Conseils et dveloppements
                           <http://www.atolcd.com>
    copyright            : (C) 2004-2005 by theKompany.com
    
    author               : Fabien Armand <f.armand@atolcd.com>
    author               : Eugen Constantinescu <eug@thekompany.com>
 ***************************************************************************/


#ifndef __KOLABSharedFolder__
#define __KOLABSharedFolder__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <defines.h>
   
// qt classes
#include <qstring.h>
#include <qobject.h>

// Aethera classes
class VFSModule;
class ClientRMIModule;
class KolabSharedFolderPrivate;

class AETHERA_API KolabSharedFolder : public QObject
{
  Q_OBJECT

  public:
  
    /** Return a reference to the Kolab Shared Folder class.*/
    static KolabSharedFolder *ref();
    
    virtual ~KolabSharedFolder();

    /** public method*/
    virtual QString type(const QString &) const;
    virtual QString type(const QString & account, const QString & mailbox) const;
    
    virtual bool isKolabSharedFolder(const QString & path) const;
    
    virtual bool isKolabSharedFolder
            (const QString & account, const QString & mailbox) const;
    
    virtual bool isKolabFolder(const QString & path) const;
    
    virtual bool isKolabFolder(const QString & account, const QString & mailbox) const;
    
    virtual bool isKolabParentFolder(const QString & path) const;
    
    virtual void kolabFolders(QStringList & folders, const QString & account) const;
    
    virtual void kolabFolders
            ( QStringList & folders,
              const QString & account,
              const QString & type ) const;
    
    virtual void sharedFolders(QStringList & folders, const QString & account) const;
    
    virtual void sharedFolders
            ( QStringList & folders,
              const QString & account,
              const QString & type ) const;
    
    virtual bool isTempSharedFolder
      (const QString & account, const QString & mailbox) const;
    
    virtual bool isKolabAccount(const QString & account) const;
    
    virtual bool isCitadelAccount(const QString & account) const;
    
    virtual bool checkKolabFolders();
    
    virtual bool checkIMAPFolders(const QString &);
    
    virtual bool checkVFSFolders(const QString &);
    
    virtual bool addSharedFolder
            ( const QString & account,
              const QString & path,
              const QString & type );
    
    virtual bool removeSharedFolder(const QString & account, const QString & path);
    
    virtual void subfolders
            ( QStringList & folders,
              const QString & account,
              const QString & mailbox ) const;
    
  public slots:
  
    virtual void subscribeFolders(const QString & account);
    
    virtual void scannSharedFolders(const QString & account, const QString & defaultType);
    
    virtual void synchSharedFolders(const QString & account);
    
    virtual void createSharedFolders(const QString & account);
    
    virtual void createSharedCalendarFolders(const QString & path);
    
    virtual void createSharedContactFolders(const QString & path);
    
    virtual void createSharedNoteFolders(const QString & path);
    
    virtual void createSharedJabberFolders(const QString & path);
    
  private:
  
    KolabSharedFolder();
    
    bool createIMAPFolder(const QString &, const QString &);
    
    QString settingsType(const QString & type) const;

    // Attributes
    static KolabSharedFolder * thisInstance;
    
    KolabSharedFolderPrivate * d;

};
#endif
