/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          komaetheraplugin.cpp  -  the Aethera plug in interface
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include "komaetheraplugin.h"

// kore include
#include "korecomponents.h"

// Aethera manager
#include <komaetheraman.h>

#include <qmessagebox.h>
#include <qstringlist.h>

// The basic Aethera service
static const char* komaetheraservice_metainfo[][2]=
{
    { "name", "Kom Aethera Service" },
    { "type", "Plugin" },
    { "class", "KomAetheraService" },
    { "class/fqn", "KomAetheraService" },
//    { "factory", "uipart_factory" },
    { "description", AETHERA_SERVICE_DESC },
    { "version", "0.2.0" },
    { "version/major", "0" },
    { "version/minor", "2" },
    { "version/release", "0" },
    { "api/version", "0.2.0" },
    { "api/version/major", "0" },
    { "api/version/minor", "2" },
    { "api/version/release", "0" },
    { "service", AETHERA_SERVICE },
    { 0, 0 }
};

KomAetheraService::KomAetheraService()
  : kore::ComponentRef( new kore::ComponentProxy( new kore::MetaInfo( komaetheraservice_metainfo ) ) )
{
  // set the instance
  proxy()->setInstance( this );
}

void KomAetheraService::mergeMetaInfo(MetaInfo*)
{
  // not available right now
  // it should be provided by korelib
}

MetaInfo* KomAetheraService::componentInfo2pluginInfo(const ComponentRef&)
{
  // not available right now
  return new MetaInfo();
}

// The Config service
/*
static const char* komaetheraservice_metainfo[][2]=
{
    { "/name", "Kom Aethera Config Service" },
    { "/class", "KomConfigService" },
    { "/class/fqn", "KomConfigService" },
    { "/factory", "komconfigservice_factory" },
    { "/description", AETHERA_CONFIGSERVICE_DESC },
    { "/service", AETHERA_CONFIGSERVICE },
    { 0, 0 }
};
*/

KomConfigService::KomConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom Aethera Config Service");
  info->replaceAll("class", "KomConfigService");
  info->replaceAll("class/fqn", "KomConfigService");
//  info->replaceAll("factory", "komconfigservice_factory");
  info->replaceAll("description", AETHERA_CONFIGSERVICE_DESC);
  info->insert("service", AETHERA_CONFIGSERVICE);
}

// -------------- KOM UI Service -------------------------------
class KomUserInterfaceService;

KomUIPartPrivate::KomUIPartPrivate(UIPart* part)
  : UIPartPrivate(part)
{
}

KomUIPartPrivate::~KomUIPartPrivate()
{
  KomUserInterfaceService* part = (KomUserInterfaceService*) owner;

  for(QDictIterator<QWidget> it(part->widgetDict); it.current(); ++it)
  {
    QString temp=it.currentKey();
    unlinkWidget( temp );
    delete it.current();
  }
}

void KomUIPartPrivate::slot_activate(bool toggle)
{
  if(toggle)
  {
    slot_activate();
  }
}

void KomUIPartPrivate::slot_activate()
{
  KomUserInterfaceService* part = static_cast<KomUserInterfaceService*>(owner);
  if( part )
  {
    emit activate( part->mimeType() );
  }
}

  void
KomUIPartPrivate::slot_updateVInstances(const QStringList & l)
{
//  qDebug("KomUIPartPrivate::slot_updateVInstances");
  emit(updateVInstances(l));
}

  void
KomUIPartPrivate::slot_createVInstances(const QStringList & l)
{
//  qDebug("KomUIPartPrivate::slot_createVInstances");
  emit(createVInstances(l));
}

  void
KomUIPartPrivate::slot_deleteVInstances(const QStringList & l)
{
//  qDebug("KomUIPartPrivate::slot_deleteVInstances");
  emit(deleteVInstances(l));
}

  void
KomUIPartPrivate::slot_purgeVFolder(const QString & s)
{
//  qDebug("KomUIPartPrivate::slot_purgeVFolder");
  emit(purgeVFolder(s));
}

  void
KomUIPartPrivate::slot_updateHierarchy(const QString & s)
{
//  qDebug("KomUIPartPrivate::slot_updateHierarchy");
  emit(updateHierarchy(s));
}

  void
KomUIPartPrivate::slot_createVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type,
  const QString & subType,
  const QString & serverPath
)
{
//  qDebug("KomUIPartPrivate::slot_createVFolder");
  emit(createVFolder(parentFolderPath, folderName, type, subType, serverPath));
}

  void
KomUIPartPrivate::slot_deleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type
)
{
//  qDebug("KomUIPartPrivate::slot_deleteVFolder");
  emit(deleteVFolder(parentFolderPath, folderName, type));
}

    void
KomUIPartPrivate::slot_showProgress(const QString & s)
{
//  qDebug("KomUIPartPrivate::slot_showProgress(const QString & s)");
  emit(showProgress(s));
}

void KomUIPartPrivate::slot_cut(const QStringList &)
{
  // TODO
}

void KomUIPartPrivate::slot_copy(const QStringList &)
{
  // TODO
}

void KomUIPartPrivate::slot_paste(const QStringList &)
{
  // TODO
}

void KomUIPartPrivate::slot_delete(const QStringList &)
{
  // TODO
}

void KomUIPartPrivate::slot_new(const QString & type)
{
  emit(create(type));
}

// -----------------------------------------------------------------
// KomUserInterfaceService
// -----------------------------------------------------------------
/*
static const char* komuserinterfaceservice_metainfo[][2]=
{
    { "/name", "Kom UI Service" },
    { "/class", "KomUserInterfaceService" },
    { "/class/fqn", "KomUserInterfaceService" },
    { "/factory", "komuserinterfaceservice_factory" },
    { "/description", AETHERA_UISERVICE_DESC },
    { "/service", AETHERA_SERVICE },
    { "/service", AETHERA_UISERVICE },
    { 0, 0 }
};
*/

KomUserInterfaceService::KomUserInterfaceService()
  : toolBar(0L)
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom UI Service");
  info->replaceAll("class", "KomUserInterfaceService");
  info->replaceAll("class/fqn", "KomUserInterfaceService");
  info->replaceAll("description", AETHERA_UISERVICE_DESC);
  info->insert("service", AETHERA_SERVICE);
  info->insert("service", AETHERA_UISERVICE);
  
  KomAetheraMan * partManager = 0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_MANAGER );
    if( ref.isValid() )
      partManager = static_cast<KomAetheraMan*>(ref.instance());
  }

  Q_ASSERT(0 != partManager);
  mainWindow = partManager->mainWindow();
  Q_ASSERT(0 != mainWindow);

//  mainWindow = dynamic_cast<QMainWindow*>(qApp->mainWidget());
}

KomUserInterfaceService::~KomUserInterfaceService()
{
}

void KomUserInterfaceService::execAboutDialog(QWidget * parent) const
{
  // Default to very basic dialog. Normally we would add a pure virtual,
  // but I'm not sure of the source compatibility situation, and we might
  // as well have something to show.

  // TODO: i18n

  QMessageBox::about
    (
      parent,
      QString("About plugin: %1").arg(prettyName()),
      QString("This plugin handles data of type `%1'").arg(mimeType())
    );
}

QIconSet KomUserInterfaceService::iconSet() const
{
  return pluginIcon;
}

QString KomUserInterfaceService::mimeType() const
{
  return _mimeType;
}

QWidget* KomUserInterfaceService::getWidget(const QString & widgetCapability)
{
  return widgetDict.find(widgetCapability);
}

UIPartPrivate* KomUserInterfaceService::createQTHandler(void)
{
  return new KomUIPartPrivate((UIPart*)this);
}

void KomUserInterfaceService::createActions()
{
  if( !actionDict.size() || !actionDict.find( NEW_FOLDER_ID ) )
  {
  
    // Create the new folder action
    QAction *action = new QAction( QObject::tr("Create a new folder."),
                                   QIconLoader::instance()->iconSet( NEW_FOLDER_NORMAL ),
                                   QObject::tr("New Fol&der"),
                                   QKeySequence( QObject::tr("Ctrl+D", "File|New|Folder")),
                                   0 );
    actionDict.insert( NEW_FOLDER_ID , action);
    
    // Create the CUT action
    action = new QAction(  QObject::tr("Cut the selection."),
                           QIconLoader::instance()->iconSet( CUT_NORMAL ),
                           QObject::tr("&Cut"),
                           QKeySequence( QObject::tr("Ctrl+X", "Edit|Cut")),
                           0 );
    actionDict.insert( CUT_ID, action);
    
    // Create the COPY action
    action = new QAction(  QObject::tr("Copy the selection."),
                           QIconLoader::instance()->iconSet( COPY_NORMAL ),
                           QObject::tr("C&opy"),
                           QKeySequence( QObject::tr("Ctrl+C", "Edit|Copy")),
                           0 );
    actionDict.insert( COPY_ID, action);
    
    // Create the PASTE action
    action = new QAction(  QObject::tr("Paste from clipboard."),
                           QIconLoader::instance()->iconSet( PASTE_NORMAL ),
                           QObject::tr("&Paste"),
                           QKeySequence( QObject::tr("Ctrl+V", "Edit|Paste")),
                           0 );
    actionDict.insert( PASTE_ID, action);
    
    // Create the DELETE action
    action = new QAction(  QObject::tr("Delete the selection."),
                           QIconLoader::instance()->iconSet( DELETE_NORMAL ),
                           QObject::tr("&Delete"),
                           QKeySequence( QObject::tr("Key_Delete", "Edit|Delete")),
                           0 );
    actionDict.insert( DELETE_ID, action);
  }
}

QAction* KomUserInterfaceService::getAction(const QString& actionName)
{
  return actionDict.find(actionName);
}

bool KomUserInterfaceService::hasCapability(const QString& capabilityName)
{
  if( UIPart::hasCapability(capabilityName) )
    return true;
  else
  {
    if( capabilityName == AETHERA_INTERFACE )
      return true;
    if( capabilityName == TOOLBARS )
      return true;
    if( capabilityName == MENUBARS )
      return true;
    if( capabilityName == MIMETYPE )
      return true;
    if( capabilityName == ACTIVATE_BUTTON )
      return true;
  }

  return false;
}

// -------------- KomBackendService ------------------------------
/*
static const char* kombackendservice_metainfo[][2]=
{
    { "/name", "Kom Backend Service" },
    { "/class", "KomBackendService" },
    { "/class/fqn", "KomBackendService" },
    { "/factory", "kombackendservice_factory" },
    { "/description", AETHERA_BACKENDSERVICE_DESC },
    { "/service", AETHERA_BACKENDSERVICE },
    { 0, 0 }
};
*/

KomBackendService::KomBackendService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom Backend Service");
  info->replaceAll("class", "KomBackendService");
  info->replaceAll("class/fqn", "KomBackendService");
  info->replaceAll("description", AETHERA_BACKENDSERVICE_DESC );
  info->insert("service", AETHERA_BACKENDSERVICE );
  
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
    }
  }
  
  // set the type like vfs
  _mimeType = VFS_MIMETYPE;
}

KomBackendService::~KomBackendService()
{
}

QString KomBackendService::mimeType() const
{
  return _mimeType;
}

void KomBackendService::setMIMEType(const char* type)
{
  if( type )
    _mimeType=type;
}

QStringList KomBackendService::systemFolders() const
{
  QStringList nothing;
  
  return nothing;
}

QString KomBackendService::backendType() const
{
  return QString::null;
}
    
  QString
KomMainTreeInterface::path(KomTreeItem * item)
{
  if (0 == item)
    return QString::null;

  QString path(item->text(0));

  while (item->depth() > 0)
  {
    item = static_cast<KomTreeItem *>(item->parent());
    path.prepend(item->text(0) + "/");
  }

  path.prepend("/Aethera/VFS/");

  return path;
}


  KomTreeItem *
KomMainTreeInterface::selectItem(const QString & path)
{
  QString path_ = path;

  if( path_.isEmpty() ||
      -1 == path_.find("/Aethera/VFS/") )
  {
    return root();                                 
  }
  else
  {
    // remove /Aethera/VFS/ from path
    path_ = path_.mid(13);
  }
  
  QStringList splitPath = QStringList::split("/", path_);
  KomTreeItem * item =  static_cast<KomTreeItem *>( root() );
  if( !splitPath.isEmpty() && item )
  {
    for(QStringList::Iterator it=splitPath.begin(); it!=splitPath.end() && item; ++it)
    {
      // check for root
      if( it == splitPath.begin() )
      {
        if( (*it) != item->text(0) )
          item = 0L;
      }
      else
      {
        item = static_cast<KomTreeItem *>(  item->firstChild() );
        while( item && (*it) != item->text(0) )
          item = static_cast<KomTreeItem *>( item->nextSibling() );
      }

      if( item && item->childCount() )
      {
        item->setOpen(true);
      }
    }
  }

  parentTree()->setSelected(item, true);
  
  return item;
}

  void
KomMainTreeInterface::show(const QString & focus)
{
  root()->setOpen(true);
  createChildren( root() );
  if( focus.isEmpty() )
    parentTree()->setCurrentItem(0);
  else
    parentTree()->setCurrentItem( selectItem(focus) );
}

  KomTreeItem *
KomMainTreeInterface::currentItem()
{
  return static_cast<KomTreeItem *>(m_parentTree->currentItem());
}

KomLanguageService::KomLanguageService()
{
}

KomLanguageService::~KomLanguageService()
{
}

