/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          komaetheraplugin.cpp  -  the Aethera plug in interface
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#ifndef KOMAETHERAPLUGIN_H
#define KOMAETHERAPLUGIN_H

#include <defines.h>
#include <dirs.h>
#include <qiconloader.h>
#include <settings.h>
#include <about.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cassert>

// TINO include
#include <tino/uipart.h>
#include <tino/uipartprivate.h>

// Aethera CAPABILITY
#define AETHERA_INTERFACE       "AetheraInterface"
#define TOP_WIDGET              "TopWidget"
#define LEFT_WIDGET             "LeftWidget"
#define BANNER_WIDGET           "BannerWidget"
#define PLUGIN_ICONSET          "PluginIconSet"
#define TREE_INTERFACE          "TreeInterface"
#define ACTIVATE_BUTTON         "ActivateButton"
#define MIMETYPE                "MIMEType"

// QT
#include <qpixmap.h>
#include <qstring.h>
#include <qdict.h>
#include <qiconset.h>
#include <qaction.h>
#include <qkeysequence.h>
#include <qnamespace.h>
#include <qwidget.h>

class QToolBar;
class QMenuBar;
class QPopupMenu;

#include <kommaintreeitem.h>
#include <komcontainerwidget.h>
#include <korecomponents.h>
#include <vobject.h>
#include <vobjectfactory.h>
#include <vfsmodule.h>
#include <vfsmimetype.h>

#ifdef Q_WS_WIN
#if defined(Q_TEMPLATEDLL)
// MOC_SKIP_BEGIN
template class AETHERA_API QDict<QWidget>;
template class AETHERA_API QDict<QAction>;
template class AETHERA_API QPtrList<QWidget>;
// MOC_SKIP_END
#endif
#endif

typedef ComponentRef* PComponentRef;

#define _NO_SERVICE_ 0x00
class AETHERA_API KomAetheraService : public kore::ComponentRef, public kore::Interface
{
  public:
    KomAetheraService();
    virtual ~KomAetheraService() {}
    
    static void mergeMetaInfo(MetaInfo*);
    static MetaInfo* componentInfo2pluginInfo(const ComponentRef&);

  protected:
};

class AETHERA_API KomConfigService : public KomAetheraService
{
public:
  KomConfigService();
  virtual ~KomConfigService() {}

  virtual QPixmap icon() = 0;
  virtual QString caption() = 0;

  virtual void load() = 0;
  virtual void save() = 0;
  virtual void defaults() = 0;

  virtual bool modified() = 0;

  virtual QWidget * init(QWidget * parent) = 0;
};


class KomMainTreeInterface;

class AETHERA_API KomUIPartPrivate : public UIPartPrivate
{ Q_OBJECT
  public:
    KomUIPartPrivate(UIPart* part);
    virtual ~KomUIPartPrivate();
    virtual bool eventFilter(QObject*, QEvent*) { return false; }
  public slots:
    // SLOT methods for UI notifications
    virtual void slot_activate();
    virtual void slot_activate(bool);
    virtual void slot_updateVInstances(const QStringList &);
    virtual void slot_createVInstances(const QStringList &);
    virtual void slot_deleteVInstances(const QStringList &);
    virtual void slot_createVFolder(const QString &, const QString &, const QString &, const QString &, const QString &);
    virtual void slot_deleteVFolder(const QString &, const QString &, const QString &);
    virtual void slot_purgeVFolder(const QString &);
    virtual void slot_updateHierarchy(const QString &);
    virtual void slot_showProgress(const QString &);
    // SLOT methods for the common QActions
    // rikkus: Argh ! QStringList & ?
    virtual void slot_cut(const QStringList&);
    virtual void slot_copy(const QStringList&);
    virtual void slot_paste(const QStringList&);
    virtual void slot_delete(const QStringList&);
//    virtual void slot_moveTo(QStringList&, QString&) {}
//    virtual void slot_copyTo(QStringList&, QString&) {}
    /** This will be used for creating a new folder.*/
    virtual void slot_new(const QString & type);
    
  signals:
    virtual void activate(const QString &);
    virtual void create(const QString &);

    virtual void updateVInstances (const QStringList &);
    virtual void createVInstances (const QStringList &);
    virtual void deleteVInstances (const QStringList &);
    virtual void purgeVFolder     (const QString &);
    virtual void updateHierarchy  (const QString &);
    virtual void showProgress     (const QString &);

    virtual void createVFolder
      (
        const QString & parentFolderPath,
        const QString & folderName,
        const QString & type,
        const QString & subType,
        const QString & serverPath
      );

    virtual void deleteVFolder
      (
        const QString & parentFolderPath,
        const QString & folderName,
        const QString & type
      );
};

// The general background
#define PANEL_BACKGROUND        "PanelBackground.png"

// define the actions icons and string id.
// ICONS
#define CUT_NORMAL              "Cut.png"
#define CUT_ACTIVE              "Cut.png"
#define COPY_NORMAL             "Copy.png"
#define COPY_ACTIVE             "Copy.png"
#define PASTE_NORMAL            "Paste.png"
#define PASTE_ACTIVE            "Paste.png"
#define DELETE_NORMAL           "Delete.png"
#define DELETE_ACTIVE           "Delete.png"
#define HELP_NORMAL             "Help.png"
#define HELP_ACTIVE             "Help.png"
#define LEFT_NORMAL             "Left.png"
#define LEFT_ACTIVE             "Left.png"
#define RIGHT_NORMAL            "Right.png"
#define RIGHT_ACTIVE            "Right.png"
#define UP_NORMAL               "Up.png"
#define UP_ACTIVE               "Up.png"
#define DOWN_NORMAL             "Down.png"
#define DOWN_ACTIVE             "Down.png"
#define NEW_FOLDER_NORMAL       "new-folder-normal.png"
#define NEW_FOLDER_ACTIVE       "new-folder-focus.png"
// STRING ID.
#define CUT_ID                  "EDIT_CUT"
#define COPY_ID                 "EDIT_COPY"
#define PASTE_ID                "EDIT_PASTE"
#define DELETE_ID               "EDIT_DELETE"
#define NEW_FOLDER_ID           "FILE_NEW_FOLDER"

class AETHERA_API KomUserInterfaceService : public UIPart
{
  friend class KomUIPartPrivate;
  public:
    KomUserInterfaceService();
    virtual ~KomUserInterfaceService();

    /** The tree interface.*/
    virtual KomMainTreeInterface* getMainTreeInterface( QListView* ) { return NULL; }
    /** Get the icon set for building the plugin button.*/
    virtual QIconSet iconSet() const;
    /** Return the plugin mimetype string.*/
    virtual QString mimeType() const;

    /**
     * Used to identify this part to the user.
     */
    virtual QString prettyName() const { return QString::null; }

    // From UIPart
    virtual bool hasCapability(const QString& capname);
    /** Return the widget corresponding to the string.*/
    virtual QWidget *getWidget(const QString&);
    /** Create a new widget corresponding to the string.*/
    virtual void createWidget(const QString&) = 0;
    /** Show the part toolbars.*/
    virtual void showToolbars(QMainWindow *) = 0;
    /** Hide the part toolbars.*/
    virtual void hideToolbars(QMainWindow *) = 0;
    /** Show the part manus.*/
    virtual void showMenus(QMenuBar *) = 0;
    /** Hide the part menu.*/
    virtual void hideMenus(QMenuBar *) = 0;

    /** Return the action with the name 'actionName'.*/
    virtual QAction* getAction(const QString& actionName);

    virtual void setCurrentURL(QString) = 0;

    virtual void execAboutDialog(QWidget * parent) const;

    virtual void buildTree() {};
    
    virtual void removeTree() {};

  protected:
    /** Create the QT signal/slot handler.*/
    virtual UIPartPrivate *createQTHandler(void);
    /** Create the QActions here.*/
    virtual void createActions();

    /** The widgets dictionary sorted by the part capabilities.*/
    QDict<QWidget> widgetDict;
    /** The QAction dictionary sorted by string id.*/
    QDict<QAction> actionDict;
    /** Just the main Window.*/
    QMainWindow * mainWindow;
    /** The UIPart toolbar.*/
    QToolBar *toolBar;
    /** The QIconSet for the plugin button.*/
    QIconSet pluginIcon;
    /** The mimetype for the plugin.*/
    QString _mimeType;
};


#define _BACKEND_SERVICE_ 0x04
class AETHERA_API KomBackendService : public KomAetheraService
{
  public:
    /** Register the backend service.*/
    KomBackendService();
    virtual ~KomBackendService();

    /**
      * Return the VObjectFactory for VFS registration.
      * This factory is able to generate the VObject type objects.
      */
    virtual VObjectFactory* vobjectFactory() = 0;

    /** Return the VFS type, like a MIME type.*/
    virtual QString mimeType() const;
    /** Set the mime type.*/
    virtual void setMIMEType(const char* type=0);
    
    /** Some MUST to have system folders.*/
    virtual QStringList systemFolders() const;
    
    /** What type of backend it is using.*/
    virtual QString backendType() const;
    
    /**
      * Set the vfs module to be used by the component.
      * It should be used when we will have many VFS modules
      * (VFSModule), that's not the current situation.
      */
    void setVFS(VFSModule *externVFS)
    {
      vfs = externVFS;
    }

    VFSModule * vfsModule()
    {
      return vfs;
    }

  protected:
    /**
      * A pointer to the VFS module.
      * The vfs module is available via korelib too.
      * In time we should add more component specific things
      * to the VFS module interface. Right now we are using
      * the VFS module interface like component VFS interface.
      * That's the reason for keeping this public.
      */
    VFSModule* vfs;
    /** The mimetype of the objects.*/
    QString _mimeType;
};

/**

  * Implemented by plugins, used for showing a tree of objects,
  * usually in the list view on the left of the window.
  *
  * This is used instead of walking the VFS, so that context menus,
  * icons, etc. can be handled nicely. Or something like that.
  */
class AETHERA_API KomMainTreeInterface
{
  public:

    KomMainTreeInterface(QListView * parent)
      : m_parentTree(parent)
    {
      // rikkus: I removed the default (= 0) parameter, because I can't see a
      // reason for it, so this assert checks if anyone is actually calling
      // with parentTree == 0.

      assert(0 != m_parentTree);
    }

    virtual ~KomMainTreeInterface()
    {
      // Empty.
    }

    /**
      * @return the root of the tree, as seen by the plugin.
      */
    virtual KomTreeItem * root() = 0;

    /**
      * Create all the children for this item.
      */
    virtual void createChildren(KomTreeItem *) = 0;

    /**
      * @return true if this item is not a leaf.
      */
    virtual bool hasChild(KomTreeItem *) = 0;

    /**
      * @return A context menu appropriate for showing above the
      * given item.
      */

    virtual QPopupMenu * contextMenu(KomTreeItem *) = 0;


    QListView * parentTree()
    {
      return m_parentTree;
    }

    /** @return The item VFS path.
      * Ex: /Aethera/VFS/Mail/Inbox/MailId
      */
    virtual QString path(KomTreeItem *);
    virtual KomTreeItem * selectItem(const QString & path);
    
    KomTreeItem * currentItem();

    virtual void show() = 0;
    virtual void show(const QString & focus);
    
    virtual void hide() = 0;
    virtual QString name() const = 0;

  protected:

    KomItemTypeList & itemTypeList()
    {
      return m_itemTypeList;
    }

  private:

    KomItemTypeList   m_itemTypeList;
    QListView       * m_parentTree;
};


class AETHERA_API WidgetsFactory : public KomAetheraService
{
  public:
    WidgetsFactory(){}
    virtual ~WidgetsFactory(){}
    /** Create a new widget.*/
    virtual QWidget *newWidget(QWidget *parent=0, const char *name=0, QWidget::WFlags f = 0) = 0;
  protected:
    QPtrList<QWidget> widgetsList;
};

#define AETHERA_LANGUAGESERVICE_DESC "Language support"
#define AETHERA_LANGUAGESERVICE "Aethera/LanguageService"

class AETHERA_API KomLanguageService : public KomAetheraService
{
  public:
    KomLanguageService();
    virtual ~KomLanguageService();
    
    virtual QString translator() = 0;
};
#endif
