/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include <kommaintreeitem.h>


KomTreeItem::KomTreeItem( QListView * parent, KomItemType& treeItemType  )
:QListViewItem( parent ), KomItemType( treeItemType )
{
}

void KomTreeItem::setItemIcon( QPixmap *itemIcon )
{
  KomItemType::setItemIcon( itemIcon );
  setup();
  widthChanged( 0 );
  invalidateHeight();
  repaint();
}

void KomTreeItem::setItemText( QString *itemText )
{
  KomItemType::setItemText( itemText );
  setup();
  widthChanged( 0 );
  invalidateHeight();
  repaint();
}


const QPixmap* KomTreeItem::pixmap( int column ) const
{
  if ( column )
    return NULL;
  
  return m_itemIcon;
}

QString KomTreeItem::text( int column ) const
{
  if (!column)
  {
    if( m_itemText==NULL )
      return QString("");
    else
      return *m_itemText;
  }
  else
  {
    return QListViewItem::text(column);
  }
}


const KomItemType* KomItemTypeList::getItemOfType( const QString itemType )
{
  return find( itemType );
}


KomItemType::KomItemType( int itemType, QString* itemText, QPixmap *itemIcon,
             bool autoDeleteIcon, bool autoDeleteText)
{
  m_itemIcon = itemIcon;
  m_itemType = itemType;
  m_itemText = itemText;
  m_autoDeleteIcon = false;
  m_autoDeleteText = false;
  setAutoDelete( autoDeleteIcon, autoDeleteText );
}

KomItemType::KomItemType( KomItemType& komItemType )
{
  m_itemType = komItemType.m_itemType;
  m_itemIcon = komItemType.m_itemIcon;
  m_autoDeleteText = komItemType.m_autoDeleteText;
  m_autoDeleteIcon = false;
  if ( m_autoDeleteText && komItemType.m_itemText!=NULL )
     m_itemText = new QString( *komItemType.m_itemText );
  else m_itemText = komItemType.m_itemText;
}

KomItemType::~KomItemType( )
{
  if ( m_autoDeleteIcon && m_itemIcon )
  {
    delete m_itemIcon;
    m_itemIcon = 0L;
  }
  if ( m_autoDeleteText && m_itemText )
  {
    delete m_itemText;
    m_itemText = 0L;
  }
}


void KomItemType::setItemIcon( QPixmap *itemIcon )
{
  if ( m_autoDeleteIcon )
  {
    if( m_itemIcon )
    {
      delete m_itemIcon;
      m_itemIcon = 0L;
    }
  
    m_itemIcon = new QPixmap( *itemIcon );
  }
  else m_itemIcon = itemIcon;
}

void KomItemType::setItemText( QString *itemText )
{
  if ( m_autoDeleteText )
  {
    if( m_itemText )
    {
      delete m_itemText;
      m_itemText = 0L;
    }
    
    m_itemText = new QString( *itemText );
  }
  else m_itemText = itemText;
}

void KomItemType::setAutoDeleteIcon( bool autoDelete )
{
  if ( (!m_autoDeleteIcon) && autoDelete && m_itemIcon!=NULL )
    m_itemIcon = new QPixmap( *m_itemIcon );
  m_autoDeleteIcon = autoDelete;
}

void KomItemType::setAutoDeleteText( bool autoDelete )
{
  if ( (!m_autoDeleteText) && autoDelete && m_itemText!=NULL )
    m_itemText = new QString( *m_itemText );
  m_autoDeleteText = autoDelete;
}

void KomItemType::setAutoDelete( bool autoDeleteIcon, bool autoDeleteText )
{
  setAutoDeleteIcon( autoDeleteIcon );
  setAutoDeleteText( autoDeleteText );
}

