/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          mail.cpp  -  description
                             -------------------
    begin                : Tue Oct 1 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mail.h"

#include <qstring.h>
#include <qpixmap.h>

#include <qiconloader.h>

Mail * Mail::instance_ = 0L;
Mail::Private * Mail::d = 0L;

class Mail::Private
{
  public:
  
    Private();
    ~Private();
    
    QPixmap attachment;
    QPixmap noAttachment;
    QPixmap lowestPriority;
    QPixmap lowPriority;
    QPixmap highPriority;
    QPixmap highestPriority;
    QPixmap normalPriority;
    QPixmap forwardMail;
    QPixmap replyMail;
    QPixmap replyForwardMail;
    QPixmap normalMail;
    
  private:
  
};

Mail::Private::Private()
  : attachment( QIconLoader::instance()->image("attachment") ),
    noAttachment( QIconLoader::instance()->image("no-attachment") ),
    lowestPriority( QIconLoader::instance()->image("LowestPriority") ),
    lowPriority( QIconLoader::instance()->image("LowPriority") ),
    highPriority( QIconLoader::instance()->image("HighPriority") ),
    highestPriority( QIconLoader::instance()->image("HighestPriority") ),
    normalPriority( QIconLoader::instance()->image("NormalPriority") ),
    forwardMail( QIconLoader::instance()->image("forward-mail") ),
    replyMail( QIconLoader::instance()->image("reply-mail") ),
    replyForwardMail( QIconLoader::instance()->image("reply-forward-mail") ),
    normalMail( QIconLoader::instance()->image("normal-mail") )
{
}

Mail::Private::~Private()
{
}

Mail::Mail()
{
  d = new Private();
}

Mail::~Mail()
{
  if(d)
  {
    delete d;
    d = 0;
  }
}

Mail * Mail::instance()
{
  return instance_ ? instance_ : instance_ = new Mail();
}

QString Mail::toString(Mail::Attachment attach)
{
  QString text;
  
  switch( attach )
  {
    case HasAttachment:
      text = "attachment";
      break;
    case NoAttachment:
    default:
      break;
  }
  
  return text;
}

QPixmap Mail::toPixmap(Mail::Attachment attach)
{
  QPixmap pixmap;
  
  switch( attach )
  {
    case HasAttachment:
      pixmap = d->attachment;
      break;
    case NoAttachment:
    default:
      pixmap = d->noAttachment;
      break;
  }
  
  return pixmap;
}

Mail::Attachment Mail::toAttachment(const QString & text)
{
  if( "attachment" == text )
    return Mail::HasAttachment;
  
  return Mail::NoAttachment;
}

QString Mail::toString(Mail::Priority priority)
{
  QString text;
  
  switch( priority )
  {
    case Mail::Lowest :
      text = "Lowest";
      break;
    case Mail::Low :
      text = "Low";
      break;
    case Mail::High :
      text = "High";
      break;
    case Mail::Highest :
      text = "Highest";
      break;
    case Mail::Normal :
    default:
      text = "Normal";
      break;
  }
  
  return text;
}

QPixmap Mail::toPixmap(Mail::Priority priority)
{
  QPixmap pixmap;
    
  switch( priority )
  {
    case Mail::Lowest :
      pixmap = d->lowestPriority;
      break;
    case Mail::Low :
      pixmap = d->lowPriority;
      break;
    case Mail::High :
      pixmap = d->highPriority;
      break;
    case Mail::Highest :
      pixmap = d->highestPriority;
      break;
    case Mail::Normal :
    default:
      pixmap = d->normalPriority;
      break;
  }
  
  return pixmap;
}

Mail::Priority Mail::toPriority(const QString & text)
{
  Mail::Priority priority;
  char c = text[0].lower().latin1();
  
  switch(c)
  {
    case 'l':
      if( "low" == text.lower() )
        priority = Mail::Low;
      else if( "lowest" == text.lower() )
        priority = Mail::Lowest;
      else
        priority = Mail::Normal;
      break;
      
    case 'h':
      if( "high" == text.lower() )
        priority = Mail::High;
      else if( "highest" == text.lower() )
        priority = Mail::Highest;
      else
        priority = Mail::Normal;
      break;
    
    default:
      priority = Mail::Normal;
      break;
  }
  
  return priority;
}

QString Mail::toString(Mail::Status status)
{
  QString text;
  
  switch( status )
  {
    case Mail::Forwarded :
      text = "Forwarded";
      break;
    case Mail::Replyed :
      text = "Replyed";
      break;
    case Mail::ReplyedForwarded :
      text = "ReplyedForwarded";
      break;
    case Mail::Default :
    default:
      text = "Default";
      break;
  }
  
  return text;
}

QPixmap Mail::toPixmap(Mail::Status status)
{
  QPixmap pixmap;
  
  switch( status )
  {
    case Mail::Forwarded :
      pixmap = d->forwardMail;
      break;
    case Mail::Replyed :
      pixmap = d->replyMail;
      break;
    case Mail::ReplyedForwarded :
      pixmap = d->replyForwardMail;
      break;
    case Mail::Default :
    default:
      pixmap = d->normalMail;
      break;
  }
  
  return pixmap;
}

Mail::Status Mail::toStatus(const QString & text)
{
  Mail::Status status;
  char c = text[0].lower().latin1();
  
  switch(c)
  {
    case 'r':
      if( "replyed" == text.lower() )
        status = Mail::Replyed;
      else if( "replyedforwarded" == text.lower() )
        status = Mail::ReplyedForwarded;
      else
        status = Mail::Default;
      break;
      
    case 'f':
      if( "forwarded" == text.lower() )
        status = Mail::Forwarded;
      else
        status = Mail::Default;
      break;
    
    default:
      status = Mail::Default;
      break;
  }
  
  return status;
}

