/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include <metaformatcontrol.h>
#include <qsettings.h>
#include <qstringlist.h>
#include <qstring.h>
#include <stdio.h>

#include <settings.h>

MetaFormatControl *MetaFormatControl::thisInstance;

MetaFormatControl::MetaFormatControl()
{
	refresh();
}

MetaFormatControl::~MetaFormatControl()
{
	// save to config file
}

void MetaFormatControl::refresh()
{
  QStringList entryList = Settings::instance()->entryList("/Aethera/");

  if (entryList.end() == entryList.find("HTML Parser"))
	{
		qDebug("Non-existent group.");
		return;
	}

  QString prefix("/Aethera/HTML Parser");

	// reply data
  replyData = Settings::instance()->readListEntry(prefix + "Reply pattern", ',');

	enableReplyTags = enableBlockReplyTags = !replyData.isEmpty();

	// "Xxxxx wrote:"
	wroteData = Settings::instance()->readListEntry(prefix + "Wrote pattern", ',');
	enableWroteSequence=!wroteData.isEmpty();
#ifdef DEBUG_MFC
	qDebug("WROTE: %d, REPLY: %d", enableWroteSequence, enableReplyTags);
#endif

	QStringList activeRules =
    Settings::instance()->readListEntry(prefix + "Active rules", ',');

	enableAnySequence = !activeRules.isEmpty();

  QStringList::ConstIterator it;

	for (it = activeRules.begin(); it != activeRules.end(); ++it)
	{
		QString group=QString("HTML Rule ")+(*it);

		if (entryList.end() != entryList.find(group))
		{
      prefix = "/Aethera/" + group + "/";

			QString beginPattern  = Settings::instance()->readEntry(prefix + "Begin pattern");
			QString bodyPattern   = Settings::instance()->readEntry(prefix + "Body pattern");
			QString endPattern    = Settings::instance()->readEntry(prefix + "End pattern");

			if (
        beginPattern.isEmpty()
        && endPattern.isEmpty()
        && !bodyPattern.isEmpty()
      )
      {
				anyData.append( SeqTag(bodyPattern, (*it)) );
      }

			if (
        !beginPattern.isEmpty()
        && !endPattern.isEmpty()
        && !bodyPattern.isEmpty()
      )
      {
				anyData.append
          (
            SeqTag(beginPattern, bodyPattern, endPattern, (*it))
          );
      }
		}
	}
}

MetaFormatControl *MetaFormatControl::ref()
{
	if(!MetaFormatControl::thisInstance)
	{
		MetaFormatControl::thisInstance=new MetaFormatControl();
	}
	
	return MetaFormatControl::thisInstance;
}

SeqTag::SeqTag()
{
}

SeqTag::SeqTag(const QString &_beginPattern, const QString &_textPattern, const QString &_endPattern, const QString &_tag)
{
	beginPattern=_beginPattern;
	textPattern=_textPattern;
	endPattern=_endPattern;
	tag=_tag;
	strip=true;
}

SeqTag::SeqTag(const QString &_textPattern, const QString &_tag)
{
	textPattern=_textPattern;
	tag=_tag;
	strip=false;
}

SeqTag::~SeqTag()
{
}

bool SeqTag::operator==(const SeqTag & right) const
{
  if( beginPattern == right.beginPattern &&
    	textPattern  == right.textPattern &&
    	endPattern   == right.endPattern &&
    	tag          == right.tag &&
    	strip        == right.strip )
    return true;
    
  return false;
}

