/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <htmlprocessing.h>
#include <miscfunctions.h>
#include <texthtml.h>
#include <qcstring.h>
#include <qtextcodec.h>
#include <qfontmetrics.h>
#include <qwidget.h>
#include <vector>
#include <string>

#ifdef IDSTRING
  #undef IDSTRING
#endif

#define IDSTRING "miscfunctions :"

#define DEBUG_MISC_FUNCTIONS

QStringList split(const char *s, char c)
{
  QStringList t;
  QString token, k(s);
  if(c==0) c=',';
  int iLength = k.length();

  for(int i=0; i<iLength; i++)
  {
    if( s[i] == c )
    {
      token.stripWhiteSpace();
      t.append(token);
      token.truncate(0);
    }
    else
      token += k[i];
  }
  
  if(!token.isEmpty())
  {
    token.stripWhiteSpace();
    t.append(token);
  }
  
  return t;
}

/**
 *      EnhancedSplit - the same function as the previous split function,
 * but it ignores separators between "" and with escape sequences
 *
 */
QStringList enhancedSplit(QString source, char split, QStringList escapes)
{
        if(split==0) split=',';
        QStringList t;
        int beginChunk=-1;
        bool insideQuote=false;
        int i = 0;
#ifdef DEBUG_MISCFUNCTIONS
        qDebug("%s\n",(const char *)source);
#endif
        for(i=0;i<source.length();i++)
        {
                if(source[i]=='\"') insideQuote=!insideQuote;
                if((source[i]==split) && (insideQuote==false))
                {
                        // first i have to check for escapes...
                        bool hasEscape=false;
                        for(unsigned j=0;(hasEscape==false) && (j<escapes.count());j++)
                        {
                                if(i>=escapes[j].length() &&
                                        source.find(escapes[j],i-escapes[j].length(),false)==int(i-escapes[j].length()))
                                                hasEscape=true;
                        }
#ifdef DEBUG_MISCFUNCTIONS
                        for(unsigned k=0;k<i;k++) qDebug(" ");
#endif
                        if(!hasEscape)
                        {
                                t.append(source.mid(beginChunk+1,i-1-beginChunk).stripWhiteSpace());
                                beginChunk=i;
#ifdef DEBUG_MISCFUNCTIONS
                                qDebug("^\n");
#endif
                        }
#ifdef DEBUG_MISCFUNCTIONS
                        else
                                qDebug("*\n");
#endif
                }
        }
        if(unsigned(beginChunk)!=source.length()-1)
                t.append(source.mid(beginChunk+1,beginChunk-i-1));

        return t;
}

QString remove(QString source, QString what, bool sensitive)
{
  QString copy = source;
  copy.remove(what, sensitive);
/*  int pos = 0;
  while( pos != -1)
  {
    pos = copy.find(what, 0, sensitive);
    if(pos != -1)
      copy.remove(pos, what.length());
  }*/
  return copy;
}

QString toString(QStringList &s)
{
  return s.join(",");
/*  QString t;
  if(s.count())
    t.append(s[0]);
  for(unsigned i=1;i<s.count();i++)
  {
    t.append(", ");
    t.append(s[i]);
  }
  return t;*/
}

QString join(QStringList s, QString separator)
{
  return s.join(separator);
/*  QString t;
  if(s.count())
    t.append(s[0]);
  for(unsigned i=1;i<s.count();i++)
  {
    t.append(separator);
    t.append(s[i]);
  }
  return t;*/
}

void MIMEunfold(QString &source)
{
  int pos=0;
  int iterPos;
  while((pos=source.find('\n', pos)+1)!=0)
  {
    iterPos=pos;
    if(source[iterPos].isSpace())
      iterPos++;
    if(iterPos!=pos)
    {
      source.replace(pos-1, iterPos-pos+1, "");
#ifdef DEBUG_MISCFUNCTIONS
        qDebug(IDSTRING"unfold: pos=%d, iterpos=%d\n", pos, iterPos);
#endif
    }
  }
  source.stripWhiteSpace();
#ifdef DEBUG_MISCFUNCTIONS
    qDebug(IDSTRING"unfold: Result:\n%s---\n", (const char *)source);
#endif
}

void MIMEfold(QString &str, int maxLength)
{
  int count=0;
  int strLen=str.length();
  for(int i=0;i<strLen;i++)
  {
    count++;
    if(count==maxLength)
    {
      str.insert(i,"\n ");
      count=0;
      strLen++;
    }
    if(str[i]=='\n')
      count=0;
  }
}

void unsetFixedHeight(QWidget *w)
{
  w->setMinimumHeight(0);
  w->setMaximumHeight(QWIDGETSIZE_MAX);
}

void unsetFixedWidth(QWidget *w)
{
  w->setMinimumWidth(0);
  w->setMaximumWidth(QWIDGETSIZE_MAX);
}

void unsetFixedSize(QWidget *w)
{
  unsetFixedWidth(w);
  unsetFixedHeight(w);
}

void updateComboBox(QComboBox *box, QString item, bool addToBox)
{
  int j=-1;
  if(item.stripWhiteSpace().isEmpty())
    j=0;
  else
    for(int i=0;i<box->count();i++)
      if(box->text(i).find(item, 0, false)!=-1) j=i;
  if(j==-1 && addToBox)
  {
    qDebug("WARNING: NEW ITEM ADDED!");
    box->insertItem(item, 0);
    j=0;
  }
  if(j>=0)
    box->setCurrentItem(j);
}

