/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  qcolorbutton.h
  -  Color selection widget.
                             -------------------
    begin                : Thu Oct  4 22:59:52 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#ifndef QCOLOR_BUTTON_H
#define QCOLOR_BUTTON_H

#include <qframe.h>
#include <qcolor.h>

#include <defines.h>

/**
 * QColorButton provides a QButton which displays and may be used to select
 * a color. Pressing the button will create a modal QColorDialog from which
 * the user may choose a color. If a color is chosen, the front face of the
 * button will display that color.
 */
class AETHERA_API QColorButton : public QFrame
{
  Q_OBJECT
  Q_PROPERTY(QColor color READ color WRITE setColor);

  public:

    QColorButton(QWidget * parent = 0, const char * name = 0, WFlags f = 0);
    virtual ~QColorButton();

    /**
     * @return the stored color.
     */
    QColor color() const;

    QSize sizeHint() const;
    QSize minimumSizeHint() const;

  public slots:
    /**
     * Set the stored color and change the appearance of the front face.
     */
    void setColor(const QColor &);

  protected:

    /**
     * @reimp
     */
    virtual void drawContents(QPainter *);

    /**
     * @reimp
     */
    virtual void mousePressEvent(QMouseEvent *);

    /**
     * @reimp
     */
    virtual void mouseReleaseEvent(QMouseEvent *);

  signals:

    void colorChanged(const QColor &);

  private:

    QColor color_;
};

#endif
