/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  settingsdialog.cpp
  -  Container for settings widgets provided by plugins.
                             -------------------
    begin                : Tue Dec 18 15:06:07 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    modified             : Ghiciuc Ilie <ilie@eco-spaces.com.ro> - changed the entire UI
 ***************************************************************************/

//QT includes
#include <qcursor.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qmime.h>

//korelib includes
#include <korecomponents.h>

//own includes
#include "settingsdialog.h"
#include "komaetheraplugin.h"
#include "settingslistview.h"
#include "settingslistviewitem.h"

//tkcBase includes
#include <qiconloader.h>
#include <dirs.h>

/*
----------------------------------------------------------------------------------------
KomConfigPtrList class
----------------------------------------------------------------------------------------
*/

KomConfigPtrList::KomConfigPtrList()
  : QPtrList<KomConfigService>()
{
}

int KomConfigPtrList::compareItems( QPtrCollection::Item lhs, QPtrCollection::Item rhs )
{
  KomConfigService* lhsService = static_cast<KomConfigService*>( lhs );
  KomConfigService* rhsService = static_cast<KomConfigService*>( rhs );
  
  return QString::compare( lhsService->caption(), rhsService->caption() );
  
}

/*
----------------------------------------------------------------------------------------
SettingsDialog::Private class
----------------------------------------------------------------------------------------
*/

class SettingsDialog::Private
{
  public:
    Private( SettingsDialog* w);
    void handleServiceUI( KomConfigService* service );
    
    SettingsDialog* parent;


    QWidget* noSettingsWidget;
};

SettingsDialog::Private::Private(SettingsDialog* w)
  : parent(w),
    noSettingsWidget(0L)
{
}

void SettingsDialog::Private::handleServiceUI( KomConfigService* service )
{
    QWidget * serviceWidget = service->init( parent );
    parent->widgetStack->addWidget( serviceWidget );

    
    if( !noSettingsWidget)
      noSettingsWidget = new QWidget( parent );
    parent->widgetStack->addWidget( noSettingsWidget );
    
    /*
      a) Insert the service in the configuration tree. Figure out where to be inserted
      b) treeList holds the parts of the hieracity: "/Aethera/Plugins/Mail" --> "Aethera","Plugins","Mail"
      c) parentItem is the QListViewItem that will be parent to the configuration item
      d) Since only the last item in the tree has widget to display empty widgets will have to be put in the stack
    */
    
    QStringList treeList = QStringList::split( '/', service->caption() );
    if( treeList.count() > 1 )
    {
      SettingsListViewItem* tmpItem = static_cast<SettingsListViewItem*>( parent->settingsListView->firstChild() );
      
      SettingsListViewItem* parentItem = 0L;
      if( !tmpItem->findSettingsItem(treeList[0],0,tmpItem) )
      {
        parentItem = new SettingsListViewItem( service, parent->settingsListView );
        parentItem->setConfigWidget( noSettingsWidget );
        parentItem->setText( 0, treeList.first() );
        parentItem->setPixmap( 0, QIconLoader::instance()->loadIcon( "noSettings.png" ) ); 
        parentItem->setConfigService( 0L );
      }
      else
        parentItem = tmpItem;
      
      for( uint i=1; i<treeList.count()-1; i++ )
      {      
        SettingsListViewItem* childItem = static_cast<SettingsListViewItem*>( parentItem->firstChild() );
        if( !childItem )
        {
          SettingsListViewItem* item = new SettingsListViewItem( parentItem );
          item->setConfigWidget( noSettingsWidget );
          item->setConfigService( 0L );
          item->setText( 0, treeList[i] );
          item->setPixmap( 0, QIconLoader::instance()->loadIcon( "noSettings.png" ) ); 
          parentItem = item;
        }
        else
        {
          if( !childItem->findSettingsItem(treeList[i],0,childItem) )
          {
            SettingsListViewItem* item = new SettingsListViewItem( parentItem );
            item->setConfigWidget( noSettingsWidget );
            item->setConfigService( 0L );
            item->setText( 0, treeList[i] );
            item->setPixmap( 0, QIconLoader::instance()->loadIcon( "noSettings.png" ) ); 
            parentItem = item;
          }
          else
            parentItem = childItem;
          
        }
      }
      
      SettingsListViewItem* item = new SettingsListViewItem( service, parentItem );
      item->setConfigWidget( serviceWidget );
      item->setText( 0, treeList.last() );
      item->setPixmap( 0, service->icon() ); 
    }
    else //if the first node in the tree has config
    {
      SettingsListViewItem* item = new SettingsListViewItem( service, parent->settingsListView );
      item->setConfigWidget( serviceWidget );
      item->setText( 0, treeList.last() );
      item->setPixmap( 0, service->icon() );
    }

}
/*
----------------------------------------------------------------------------------------
SettingsDialog class
----------------------------------------------------------------------------------------
*/
SettingsDialog::SettingsDialog(QWidget * parent, const char * name)
  : SettingsDialogBase(parent, name, true)
{
  d = new Private( this );

  connect( settingsListView, SIGNAL( currentChanged(QListViewItem*) ),
           this, SLOT( slotCurrentChanged(QListViewItem*) ) 
         );             
  connect( okButton, SIGNAL( clicked() ),
           this, SLOT( slotOkClicked() ) 
         );             
  connect( applyButton, SIGNAL( clicked() ),
           this, SLOT( slotApply() ) 
         );             

  connect( defaultButton, SIGNAL( clicked() ),
           this, SLOT( slotDefault() ) 

         );             
    
  loadServices();
  settingsListView->setSorting( 0 );
}

SettingsDialog::~SettingsDialog()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

void SettingsDialog::languageChange()
{
  SettingsDialogBase::languageChange();
}

void SettingsDialog::init()
{
  SettingsDialogBase::init();
}

void SettingsDialog::loadServices()
{
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

  // rikkus: Is the kore API meant to be a joke ?
  // It doesn't use the STL where it should use it, and where it does use it,
  // it uses the non-standard SGI extensions.

  ComponentRef *references=0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
    references = trader->registeredComponents( AETHERA_CONFIGSERVICE );
  else
  {
    qDebug("No component trader available!");
    return;
  }
  

  if ( !references[0].isValid() )
    qDebug("No service providers registered for %s", AETHERA_CONFIGSERVICE);

  uint i = 0;

  KomConfigPtrList serviceList;
  
  kore::ComponentRef provider;
  for (; (provider = references[i]).isValid(); i++)
  {
    KomConfigService * service = dynamic_cast<KomConfigService *>(provider.instance());

    if (0 == service)
    {
      qDebug("Can't dynamic_cast to KomConfigService");
      continue;
    }
    
    serviceList.inSort( service );

    qApp->processEvents();
  }
  
  KomConfigService* service;
  for( service = serviceList.first(); service; service=serviceList.next() )  
    d->handleServiceUI( service );

  delete [] references;
  SettingsListViewItem* item = static_cast<SettingsListViewItem*>( settingsListView->firstChild() );
  settingsListView->setSelected( item, true );
  slotAboutToShow();

  QApplication::restoreOverrideCursor();
}

void SettingsDialog::slotAboutToShow()
{
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

  QListViewItemIterator it( settingsListView );
  while( it.current() )
  {
    SettingsListViewItem* item = static_cast<SettingsListViewItem*>( it.current() );

    KomConfigService* service = item->getConfigService();
    if( service )
      service->load();
      
    // XXX: Really need to speed up dialog loading instead.
    qApp->processEvents();
    ++it;
  }

  QApplication::restoreOverrideCursor();
}

void SettingsDialog::slotDefault()
{
  SettingsListViewItem* item = static_cast<SettingsListViewItem*>( settingsListView->currentItem() );
  KomConfigService* service = item->getConfigService();
  if( service )
    service->defaults();
}


void SettingsDialog::slotApply()
{
  static bool warningDisplayed = false;

  if( !warningDisplayed )
  {
    QMessageBox::information( this,
                              tr( "Warning"),
                              tr( "Some settings will not be applied until after\n"
                                  "you restart Aethera"
                                )
                             );
    warningDisplayed = true;
  }
  
  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
  
  QListViewItemIterator it( settingsListView );
  while( it.current() )
  {
    SettingsListViewItem* item = static_cast<SettingsListViewItem*>( it.current() );
    KomConfigService* service = item->getConfigService();
  
    if( service )
      service->save();
    ++it;
    // XXX: Really need to speed up dialog loading instead.
    qApp->processEvents();
  }

  QApplication::restoreOverrideCursor();  
}

void SettingsDialog::slotCurrentChanged(QListViewItem* itm)
{
  SettingsListViewItem* item = dynamic_cast<SettingsListViewItem*>( itm );
  Q_CHECK_PTR(item);
  
  widgetStack->raiseWidget( item->getConfigWidget() );
}

void SettingsDialog::slotOkClicked()
{
  /*
    we need this slot to make sure that the settings are saved before the dialog is closed.
  */
  
  slotApply();
  accept();
}


