/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include <sortingwidget.h>
#include <sortingwidgetitem.h>
#include <clipboard.h>
#include <komaetheratools.h>

#include <qapplication.h>
#include <qclipboard.h>
#include <qcursor.h>
#include <qtimer.h>

#define BARCOLOR QColor(0xd6,0xd2,0xc7)

SortingWidget::SortingWidget(VFSModule* vfsModule, QWidget *parent, const char *name, WFlags f)
  :QIconView(parent, name, f/* | WRepaintNoErase*/), DataHandler(vfsModule)
{
  mainPopup=0;
  sortPopup=0;
  childPopup=0;
	
  // Is this a Qt bug ? If this next line is enabled, repaints are broken.
//  viewport()->setBackgroundColor(QColor("white"));
    	
  allwaysPopup=true;
  showSort=true;

  setAcceptDrops(false);
  
  connect(this, SIGNAL(rightButtonPressed(QIconViewItem*, const QPoint&)),
          this, SLOT(slotRightButtonPressed(QIconViewItem*, const QPoint&)));
  
//  connect(this, SIGNAL(rightButtonPressed(QIconViewItem*, const QPoint&)), this, slotRightButtonPressed(QIconViewItem*, const QPoint&));
}

SortingWidget::~SortingWidget()
{
}

void SortingWidget::popupGlobal()
{
  if(mainPopup)
    mainPopup->popup(QCursor::pos());
}

void SortingWidget::popupChild()
{
  if(childPopup && !isSelectionEmpty())
    childPopup->popup(QCursor::pos());
  else
    if(allwaysPopup)
      popupGlobal();
}

void SortingWidget::setAllwaysShowPopup(bool t)
{
  allwaysPopup=t;
}
	
bool SortingWidget::allwaysShowPopup()
{
  return allwaysPopup;
}

QStringList SortingWidget::getSelection()
{
  QStringList selection;
  
  if( !isSelectionEmpty() && SORTING_WIDGET_ITEM_ID == currentItem()->rtti() )
  {
    for(SortingWidgetItem *item=(SortingWidgetItem*)this->firstItem(); item; item=(SortingWidgetItem*)item->nextItem())
      if( item->isSelected() )
      {
        if( !item->path().isEmpty() )
          selection.append( item->path() );
      }
  }
  
  return selection;
}

void SortingWidget::selectAll()
{
  QIconView::selectAll(true);
}

void SortingWidget::unselectAll()
{
  QIconView::selectAll(false);
}

bool SortingWidget::isSelectionEmpty()
{
  if( 0 == currentItem() )
    return true;
  else
    return false;
}

bool SortingWidget::isSelectionSingle()
{
  QIconView::SelectionMode mode=selectionMode();
  if( mode == QIconView::Single )
    return true;
  else
    return false;
}

bool SortingWidget::isSelectionMultiple()
{
  QIconView::SelectionMode mode=selectionMode();
  if( mode == QIconView::Multi ||
      mode == QIconView::Extended )
    return true;
  else
    return false;
}

void SortingWidget::slotRightButtonPressed(QIconViewItem*, const QPoint&)
{
  popupChild();
}




