/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#ifndef __SortingWidget__
#define __SortingWidget__

#include <qwidget.h>
#include <qstring.h>
#include <qlist.h>
#include <qpopupmenu.h>
#include <qevent.h>
#include <qiconview.h>

#include <datahandler.h>
#include <vfsmodule.h>
#include <vfolder.h>
#include <vobject.h>
#include <defines.h>

class AETHERA_API SortingWidget : public QIconView, public DataHandler
{
  Q_OBJECT
  public:
    // Creates a SortingWidget object.
    SortingWidget(VFSModule *vfsModule, QWidget *parent=0, const char *name=0, WFlags f=0);

    // Destroys a SortingWidget object.
    ~SortingWidget();

    // Allows the SortingWidget to display the main popup when a child is right-clicked when childPopup doesn't exist.
    virtual void setAllwaysShowPopup(bool t=true);

    // Returns the allwaysPopup property.
    virtual bool allwaysShowPopup();

    // Enables the sort popup, if the parameter is true.
    virtual void setShowSorting(bool = true) {}

    // Returns true is the sort popup is enabled.
    virtual bool showSorting() { return showSort; }

    // Shows the global popup.
    virtual void popupGlobal();

    // Shows the child popup (or the global, the child popup is null, the main popup exists and allwaysPopup is true).
    virtual void popupChild();
    
    // From DataHandler
    virtual QStringList getSelection();
    virtual void selectAll();
    virtual void unselectAll();
    virtual bool isSelectionEmpty();
    virtual bool isSelectionSingle();
    virtual bool isSelectionMultiple();
    
  public slots:
    void slotRightButtonPressed(QIconViewItem*, const QPoint&);

  protected:
    // Makes the global popup.
    virtual void makeGlobalPopup()=0;
    // Makes the child popup.
    virtual void makeChildPopup()=0;

    QPopupMenu *mainPopup;
    QPopupMenu *sortPopup;
    QPopupMenu *childPopup;
    bool allwaysPopup;
    bool showSort;
};

#endif
