/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

//#include <ctype.h>
#include <htmlprocessing.h>
#include <texthtml.h>
#include <metaformatcontrol.h>
#include <textprocessor.h>

#include <qdict.h>
#include <qsettings.h>

#include <settings.h>

#define IDSTRING "TextHtmlConvertor: "

const char * const configGroup = "/Aethera/HTML Parser/";

TextHtmlConvertor* TextHtmlConvertor::thisInstance=0;

TextHtmlConvertor* TextHtmlConvertor::ref()
{
  if( !thisInstance )
    thisInstance = new TextHtmlConvertor();
  return thisInstance;
}

TextHtmlConvertor::TextHtmlConvertor()
{
	startPagesDict.setAutoDelete(true);
	endPagesDict.setAutoDelete(true);
	cssDict.setAutoDelete(true);
	beginHTMLDict.setAutoDelete(true);
	endHTMLDict.setAutoDelete(true);
	init();
}
void TextHtmlConvertor::init()
{
  QString group(configGroup);

	seeText=
    Settings::instance()->readBoolEntry(group + "Text only");

	addressDetection=
    Settings::instance()->readBoolEntry(group + "Web address detection");

  maxTextLength=
    Settings::instance()->readNumEntry(group + "Max parsing size", 8192);

	readReply();
	readWrote();
	readPages();
	readReplacements();
}

void TextHtmlConvertor::readReply()
{
#ifdef DEBUG_THC
	printf(IDSTRING"Reading reply tags...\n");
#endif
  QString group(configGroup);
	replyBeginTag=Settings::instance()->readEntry(group + "Reply begin tag");
	replyEndTag=Settings::instance()->readEntry(group + "Reply end tag");
	replyMiddleTag=Settings::instance()->readEntry(group + "Reply middle tag");
	replyTabulator=Settings::instance()->readEntry(group + "Reply tabulator");
	replyDuplicateTabulator=
    Settings::instance()->readBoolEntry
    (group + "Reply duplicate tabulator");

#ifdef DEBUG_THC
	printf("replyBeginTag : %s\n",(const char *)replyBeginTag);
	printf("replyEndTag : %s\n",(const char *)replyEndTag);
	printf("replyMiddleTag : %s\n",(const char *)replyMiddleTag);
	printf("replyTabulator : %s\n",(const char *)replyTabulator);
	printf("replyDuplicateTabulator : %d\n",replyDuplicateTabulator);
#endif
}

void TextHtmlConvertor::readWrote()
{
#ifdef DEBUG_THC
	printf(IDSTRING"Reading \"wrote:\" tags...\n");
#endif
  QString group(configGroup);
	wroteBeginTag=Settings::instance()->readEntry(group + "Wrote begin tag");
	wroteEndTag=Settings::instance()->readEntry(group + "Wrote end tag");
}

void TextHtmlConvertor::readPages()
{
#ifdef DEBUG_THC
	printf(IDSTRING"Reading pages...\n");
#endif
  QString group(configGroup);
	// set default page...
	QStringList templatePagesList=
    Settings::instance()->readListEntry(group + "Template pages list", ',');
	// clear the old stuff
	startPagesDict.clear();
	endPagesDict.clear();

	if(templatePagesList.end() == templatePagesList.find("default"))
	{
		startPagesDict.insert("default", new QString("<html><head>"
			"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">"
			"<style type=\"text/css\"><!-- CSS --></style>"
			"</head>"
			"<body bgcolor=\"white\"><table width=\"100%\"><tr><td>"));
		endPagesDict.insert("default", new QString("</td></tr></table></body></html>"));
	}

  QStringList::ConstIterator it;

	for(it = templatePagesList.begin(); it != templatePagesList.end(); ++it)
	{
		QString name=QString("Template page ")+(*it);

    QStringList entryList(Settings::instance()->entryList("/Aethera/"));

		if (entryList.end() != entryList.find(name))
		{
      startPagesDict.insert
        (
          name,
          new QString(Settings::instance()->readEntry(group + name + "/Begin HTML"))
        );

			endPagesDict.insert
        (
          name,
          new QString(Settings::instance()->readEntry(group + name + "/End HTML")));
		}
	}
}

void TextHtmlConvertor::readReplacements()
{
#ifdef DEBUG_THC
	printf(IDSTRING"Reading replacements tags...\n");
#endif
  QString group(configGroup);

  activeRules = Settings::instance()->readListEntry(group + "Active rules", ',');
#ifdef DEBUG_THC
	printf("ACTIVE RULES: %d\n",activeRules.count());
#endif

  QStringList entryList(Settings::instance()->entryList("/Aethera/"));

  QStringList::ConstIterator it;

	for(it = activeRules.begin(); it != activeRules.end(); ++it)
	{
		QString name=QString("HTML Rule ")+(*it);
#ifdef DEBUG_THC
		printf("Gathering information for rule: [%s]\n",(const char *)name);
#endif

		if(entryList.end() != entryList.find(name))
		{
      beginHTMLDict.insert
        (
          (*it),
          new QString
          (Settings::instance()->readEntry("/Aethera/" + name + "/Begin HTML"))
        );

			endHTMLDict.insert
        (
          (*it),
          new QString
          (Settings::instance()->readEntry("/Aethera/" + name + "/End HTML"))
        );
		}
	}
}

void TextHtmlConvertor::updateReply(QString &buffer)
{
	QString chunk;
	int pos=0;
	while((pos=buffer.find("<reply",pos))!=-1)
	{
		int mark=pos;
		int p1=buffer.find(">",pos+6);
		int level=buffer.mid(pos+7, p1-pos-7).toInt();
		chunk=replyBeginTag;
		for(int i=0;i<level;i++)
			chunk +=replyTabulator;
		chunk += replyMiddleTag;
		pos=p1+1;
		p1=buffer.find("</reply>");
		chunk += buffer.mid(pos, p1-pos) + replyEndTag;
#ifdef DEBUG_THC
		printf("Reply : %d\n",level);
#endif
		pos=p1;
		buffer.replace(mark, pos+8-mark, chunk);
	}
}

void TextHtmlConvertor::updateGeneric(QString &buffer, QString tag, QString r1, QString r2)
{
	int pos=0;
	QString btag=QString("<")+tag+">";
	QString etag=QString("</")+tag+">";
#ifdef DEBUG_THC
	printf("Finding: %s and %s\n", (const char *)btag, (const char *)etag);
#endif
	while((pos=buffer.find(btag,pos))!=-1)
	{
		buffer.replace(pos, btag.length(), r1);
		pos=buffer.find(etag,pos+r1.length());
		if(pos!=-1)
			buffer.replace(pos, etag.length(), r2);
	}
}

void TextHtmlConvertor::updateWrote(QString &buffer)
{
	updateGeneric(buffer, "wrote", wroteBeginTag, wroteEndTag);
}

void TextHtmlConvertor::updateCustom(QString &buffer)
{
#ifdef DEBUG_THC
	printf(" Updating custom...\n");
	for(QStrListIterator it(activeRules); it.current(); ++it)
		printf("Active rules: [%s]\n", (const char *)(*it));
	printf("%d-----\n", activeRules.count());
#endif
	QString *b, *e;

  QStringList::ConstIterator it;
	for(it = activeRules.begin(); it != activeRules.end(); ++it)
	{
		b=beginHTMLDict[(*it)];
		e=endHTMLDict[(*it)];
#ifdef DEBUG_THC
		printf(" Updating <%s> %p %p\n", (const char *)(*it), b, e);
#endif
		if(b && e)
			updateGeneric(buffer, (*it), *b, *e);
	}
}


void TextHtmlConvertor::text2html(QCString &str, QString tmpl)
{
  QString temp=str;
  text2html(temp,tmpl);
  str=temp;
}

void TextHtmlConvertor::text2html(QString &str, QString tmpl)
{
	QString name=(startPagesDict[tmpl]==0)?QString("default"):tmpl;
	/*
	 *	text.length()<=maxTextLength is a dumb hack for skipping html conversion
	 *	of the text for large messages.
	 */
	if(str.length()<=unsigned(maxTextLength))
	{
		// first replace html tags...
		HtmlProcessor::thisInstance()->replaceHTMLTags(str);
		// metaformat what's left
		str=TextProcessor::metaFormat(str, *(MetaFormatControl::ref()));
	}

	
	if(str.length()<=unsigned(maxTextLength))
	{
		// update the formatting
		if(MetaFormatControl::ref()->enableReplyTags || MetaFormatControl::ref()->enableBlockReplyTags)
			updateReply(str);
		if(MetaFormatControl::ref()->enableWroteSequence)
			updateWrote(str);
		if(MetaFormatControl::ref()->enableAnySequence)
			updateCustom(str);
	}
	str=str.simplifyWhiteSpace();
	// make the links
	if(addressDetection)
	{
		HtmlProcessor::thisInstance()->replaceURLs("http://",str);
		HtmlProcessor::thisInstance()->replaceURLs("ftp://",str);
		HtmlProcessor::thisInstance()->replaceURLs("ftp",str,"ftp://");
		HtmlProcessor::thisInstance()->replaceURLs("www",str,"http://");
		HtmlProcessor::thisInstance()->replaceURLs("mailto:", str, "", true);
		HtmlProcessor::thisInstance()->replaceEmails(str);
	}

	// do a preformatted?
	if(seeText || str.length()>unsigned(maxTextLength))
		str.prepend("<pre>").append("</pre>");
	// put in the html tags
	str.prepend(*startPagesDict[name]).append(*endPagesDict[name]);
}

void TextHtmlConvertor::html2text(QCString &str)
{
  QString temp=str;
  html2text(temp);
  str=temp;
}

void TextHtmlConvertor::html2text(QString &str)
{
	HtmlProcessor::thisInstance()->eliminateComments(str);
	HtmlProcessor::thisInstance()->correctHTML(str);
	for(int i=0;i!=-1;)
	{
		i=str.find("<p>",i,false);
		if(i!=-1) str.replace(i,3,"\n\t");
	}
	{
	for(int i=0;i!=-1;)
	{
		i=str.find("<br>",i,false);
		if(i!=-1) str.replace(i,4,"\n");
	}
	}
	HtmlProcessor::thisInstance()->removeTags(str);
	HtmlProcessor::thisInstance()->removeHTMLTags(str);
}

void TextHtmlConvertor::html2html(QCString &str)
{
  QString temp=str;
  html2html(temp);
  str=temp;
}

void TextHtmlConvertor::html2html(QString &str)
{
	if(addressDetection && str.length()<=unsigned(maxTextLength))
	{
		HtmlProcessor::thisInstance()->replaceURLs("http://",str);
		HtmlProcessor::thisInstance()->replaceURLs("ftp://",str);
		HtmlProcessor::thisInstance()->replaceURLs("ftp",str,"ftp://");
		HtmlProcessor::thisInstance()->replaceURLs("www",str,"http://");
		HtmlProcessor::thisInstance()->replaceURLs("mailto:",str);
		HtmlProcessor::thisInstance()->replaceEmails(str);
	}
}
