/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/**
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#ifndef HAS_TOOLBOX_H
#define HAS_TOOLBOX_H
	
#include <qdatetime.h>
#include <qstring.h>

#include <defines.h>

//#warning Someone to verify if the times are correct.
//#warning All of them are in seconds.

#define TOOLBOX_SECOND		1
#define TOOLBOX_MINUTE		60
#define TOOLBOX_HOUR			3600
#define TOOLBOX_DAY				86400
#define TOOLBOX_WEEK			604800
#define TOOLBOX_YEAR			31536000

typedef struct AETHERA_API Period
{
	QDateTime	start;
	long length;
	bool isFinished;
} Period;

/**
 *	This class is a collection of various methods (from miscfunctions)
 */
class AETHERA_API Toolbox
{
public:
	enum IntervalType { DefaultTime=0, BeforeTime, AfterTime };
	
	enum Method { Publish=0, Request, Reply, Add, 
		Cancel, Refresh, Counter, DeclineCounter };
		
	enum TimeZone { UnknownTimeZone=0,
			UTCTimeZone };
			
	static const TimeZone GMTTimeZone;
	static const TimeZone ZULUTimeZone;

	QDateTime getDateTime(const QString &str, TimeZone &tz);
	QDateTime getDateTime(const QString &datetime, QString &tz);
	//QDateTime getDateTime(QString str);
	long getDateTimeOffset(const QString &str,IntervalType &offsetType);
	long getTimeOffset(const QString &str);
	QString extractNumbers(const QString &str, int pos);
	int getDays(long seconds);
	int getHours(long seconds);
	int getMinutes(long seconds);
	int getSeconds(long seconds);
	//set (converts xxxxx in seconds
	long setDays(int days);
	long setHours(int hours);
	long setMinutes(int minutes);
	long setSeconds(int seconds);
	Period parseString(const QString &period);
};

#ifdef Q_WS_WIN
#ifndef AETHERA_EXPORTS
    AETHERA_API Toolbox tb;
#endif
#else
extern Toolbox tb;
#endif

#endif
