/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Changed by Eugen C. for speeding reasons.
 */

#include <cassert>
#include <vfolder.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <vobject.h>
#include <vcacheobject.h>

VFSModule * VFolder::vfsModule_ = 0L;

VFolder::VFolder(const QString & path)
  : VReference    (),
    folderPath_   (path),
    objectDict_   (599),
    folderCache_  (0L)
{
  if( !vfsModule_ )
  {
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfsModule_ = static_cast<VFSModule*>( ref.instance() );
      else
      {
        qDebug("VFolder: Can't find VFS module");
      }
    }  
  
    assert(0 != vfsModule_);
  }

  objectDict_.setAutoDelete(false);
  objectList_.setAutoDelete(false);
  folderList_.setAutoDelete(false);
}

VFolder::~VFolder()
{
  if( objectList_.count() )
  {
    objectDict_.clear();
    objectList_.setAutoDelete(true);
    objectList_.clear();
  }
//  if( folderList_.count() )
//  {
//    // the subfolders has to be deleted too
//    folderList_.setAutoDelete(true);
//    folderList_.clear();
//  }
}

QString VFolder::name() const
{
  int lastSlash = folderPath_.findRev("/");

  if (lastSlash != -1)
    return folderPath_.mid(lastSlash + 1);

  return QString::null;
}

QString VFolder::url() const
{
  return folderType_ + ":/" + folderPath_;
}

QString VFolder::type() const
{
  return folderType_;
}

QString VFolder::viewType() const
{
  return folderViewType_;
}

QString VFolder::mimeType() const
{
  return folderMimeType_;
}

QString VFolder::binding() const
{
  return folderBinding_;
}

QString VFolder::path() const
{
  return folderPath_;
}

QString VFolder::serverPath() const
{
  return folderSPath_;
}

void VFolder::setBinding(const QString & t)
{
  folderBinding_ = t;
}

void VFolder::setServerPath(const QString & t)
{
  folderSPath_ = t;
}

void VFolder::setType(const QString & t)
{
  folderType_ = t;
}

void VFolder::setViewType(const QString & t)
{
  folderViewType_ = t;
}

void VFolder::setMimeType(const QString & t)
{
  folderMimeType_ = t;
}

QPtrList<VObject> & VFolder::objectList()
{
  return objectList_;
}

QDict<VObject> & VFolder::objectDict()
{
  return objectDict_;
}

bool VFolder::isSimpleObject(const QString & objectName) const
{
  return ( "attr" != objectName.right(4) && "cache" != objectName.right(5) );
}

bool VFolder::addObject(VObject * obj)
{
  if (!objectDict_.find(obj->name()))
  {
    objectDict_.insert(obj->name(), obj);
    objectList_.append(obj);

    if ( "dir"+QString(ATTR_SEP)+"cache" == obj->name())
      folderCache_ = dynamic_cast<VCacheObject *>(obj);

    return true;
  }
  else
  {
    return false;
  }
}

bool VFolder::addObject(const QString & id)
{
  if (!objectDict_.find(id))
  {
    return 0 != (vfsModule_->_fact_new(folderPath_ + "/" + id, viewType()));
  }

  return false;
}

bool VFolder::removeObject(VObject * obj)
{
  if (obj == folderCache_)
    folderCache_ = 0;

  delete obj;
  obj = 0L;
  
//  objectDict_.remove(obj->name());
//  objectList_.setAutoDelete(true);
//  objectList_.removeRef(obj);
//  objectList_.setAutoDelete(false);
  
  return true;
}

bool VFolder::removeObjectRef(VObject * obj)
{
  if (obj == folderCache_)
    folderCache_ = 0L;
 
  objectDict_.remove(obj->name());
  objectList_.removeRef(obj);
  
  return true;
}

bool VFolder::removeObject(const QString & id)
{
  VObject * obj = objectDict_.find(id);

  if( !obj )
    return false;
  else
    return removeObject(obj);
}

VCacheObject * VFolder::entryCacheBlock()
{
  return folderCache_;
}

QString VFolder::flags() const
{
  return folderFlags_;
}

void VFolder::setFlags(const QString & newFlags)
{
  folderFlags_ = newFlags;
}

void VFolder::setFolderPath(const QString & newPath)
{
  folderPath_ = newPath;
}

void VFolder::setParentFolder(VFolder * folder)
{
  parent=folder;
}

bool VFolder::addFolder(VFolder * folder)
{
  folderList_.append(folder);
  return true;
}

bool VFolder::removeFolder(VFolder * folder)
{
  return (folderList_.removeRef(folder));
}

void VFolder::tree(QStringList & urlTree, int depth)
{
  if( depth )
  {
//    qDebug("print tree: %s", path().latin1());

    for(VFolderListIterator it(folderList_); it.current(); ++it)
    {
      urlTree.append( it.current()->path() );
      if( depth == -1 )
        it.current()->tree(urlTree, -1);
      else
        it.current()->tree(urlTree, depth-1);
    }

//    for(QStringList::Iterator itP=urlTree.begin(); itP!=urlTree.end(); ++itP)
//      qDebug("subfolder: %s", (*itP).latin1());
  }
}

void VFolder::tree(VFolderList & vTree, int depth)
{
  if( depth )
  {
    for(VFolderListIterator it(folderList_); it.current(); ++it)
    {
      vTree.append( it.current() );
      if( depth == -1 )
        it.current()->tree(vTree, -1);
      else
        it.current()->tree(vTree, depth-1);
    }
  }
}

VFolderList & VFolder::folderList()
{
  return folderList_;
}

VFolder* VFolder::parentFolder()
{
  return parent;
}

bool VFolder::freeObjects()
{
  QPtrList<VObject> objectsDeleted;
  objectsDeleted.setAutoDelete(true);
  
  if( objectList_.count() )
  {
    for(QPtrListIterator<VObject> it(objectList_); it.current(); ++it)
    {
      if( isSimpleObject( it.current()->name() ) )
      {
        objectsDeleted.append( it.current() );
      }
    }
    
    objectsDeleted.clear();
    objectDict_.resize(599);
  }
  
  return true;
}

bool VFolder::loadObjects()
{
  return vfsModule_->createVInstances(this);
}

