/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Changed by Eugen C. for speeding reasons.
 */

#ifndef __VirtualFolder__
#define __VirtualFolder__

#include <qstring.h>
#include <qptrlist.h>
#include <qdict.h>
#include <qstringlist.h>

#include <vreference.h>
#include <defines.h>
#include <vfsmodule.h>

class VObject;
class VCacheObject;
class VFolder;

#ifdef Q_WS_WIN
#if defined(Q_TEMPLATEDLL)
// MOC_SKIP_BEGIN
template class AETHERA_API QDict<VObject>;
template class AETHERA_API QPtrList<VObject>;
template class AETHERA_API QPtrList<VFolder>;
// MOC_SKIP_END
#endif
#endif

typedef QPtrList<VFolder> VFolderList;
typedef QPtrListIterator<VFolder> VFolderListIterator;

/**
 * This class contains the definition of a folder, as seen by an Aethera client.
 * It contains three kinds of information:
 *
 * 1) The folder URL, which holds a protocol, binding and client folder path
 * (the server path is separate.)
 *
 * 2) The main view pointer (whose existence is synced with the folder) and the
 * alternative views.
 *
 * 3) The main tree widget pointer and the alternative tree widgets.
 *
 * 4) A map of child objects.
 */
class AETHERA_API VFolder : public VReference
{
  friend class ClientVFS;

  public:

    virtual ~VFolder();

    QString name()        const;

    /**
     * URL looks like `type://path'.
     */
    QString url()         const;

    QString type()        const;
    QString viewType()    const;
    QString mimeType()    const;

    /**
     * Binding ("thisInstance", QString::null for local folders or
     * an account name).
     */
    QString binding()     const;

    QString path()        const;

    /**
     * Server (remote) path.
     */
    QString serverPath()  const;

    QString flags()       const;

    bool isSimpleObject(const QString & objectName) const;

    void setFlags(const QString &);

    /**
     *(Re)sets the folder type.
     */
    void setType(const QString &);

    void setViewType(const QString &);

    void setMimeType(const QString &);

    /**
     * (Re)sets the folder binding (exclusive for Aethera servers,
     * see commbinder.h)
     */
    void setBinding(const QString &);

    /**
     * (Re)sets the server path.
     */
    void setServerPath(const QString &);
    void setFolderPath(const QString &);

    /**
     * Child objects as a QDict.
     */
    QDict<VObject> & objectDict();

    /**
     * Child objects as a QPtrList.
     */
    QPtrList<VObject> & objectList();		

    bool addObject(const QString &);
    bool addObject(VObject *);
    bool removeObject(const QString &);
    bool removeObject(VObject *);
    bool removeObjectRef(VObject *);
    virtual bool freeObjects();
    virtual bool loadObjects();

    VCacheObject * entryCacheBlock();
    
    /** The sorted urls of the VFS.*/
    void tree(QStringList & urlTree,int depth);
    /** VFolder tree.*/
    void tree(VFolderList & vTree,int depth);
    /** The subfolders list sorted by name.*/
    VFolderList & folderList();
    /** Get the parent folder.*/
    VFolder* parentFolder();

  protected:

    VFolder(const QString & path);
    
  private:

    static VFSModule * vfsModule_;
    
    QString folderPath_;
    QString folderType_;
    QString folderViewType_;
    QString folderMimeType_;
    QString folderBinding_;
    QString folderSPath_;
    QString folderFlags_;

    /**
     * The list of objects
     */
    QPtrList<VObject> objectList_;

    /**
     * The list of objects sorted by name, useful for checking.
     */
    QDict<VObject> objectDict_;

    VCacheObject * folderCache_;
    
    // ClientVFS related
    void setParentFolder(VFolder*);
    VFolder *parent;
    
    /** Subfolders.*/
    QPtrList<VFolder> folderList_;
    /** Add a folder.
      * Only ClientVFS can do this.
      */
    bool addFolder(VFolder*);
    /** Remove a folder.
      * Only ClientVFS can do this.
      */
    bool removeFolder(VFolder*);
};

#endif

