/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

//Aethera includes
#include "vfsobjectdialog.h"
#include "folderlistwidget.h"

//korelib includes
#include <kore/componenttrader.h>
using namespace kore;

//tkcBase includes
#include <qiconloader.h>

//QT includes
#include <qlayout.h>
#include <qsplitter.h>
#include <qtextbrowser.h>
#include <qvaluelist.h>
#include <qhbuttongroup.h>

class VFSObjectDialog::Private
{
  public:
  
    Private(VFSObjectDialog*);
    ~Private();
    
    FolderListWidget* folderList;
    ObjectListFactory* objectListFactory;
    VFSObjectList* objectList;
    QWidgetStack* objectListViewStack;
    VFSObjectListView* objectListView;
    VFSObjectIconView* objectIconView;
    VFSObjectPreview* objectPreview;
    QPushButton* previewToggleButton;
    QPushButton* listViewButton;
    QPushButton* iconViewButton;
    QButtonGroup* viewButtonGroup;
    QLineEdit* urlLineEdit;
    QPushButton* okButton;
    QPushButton* cancelButton;
  
  private:
  
    VFSObjectDialog * parent_;
};

VFSObjectDialog::Private::Private(VFSObjectDialog * parent)
  : folderList(0L),
    objectListFactory(0L),
    objectList(0L),
    objectListViewStack(0L),
    objectListView(0L),
    objectIconView(0L),
    objectPreview(0L),
    previewToggleButton(0L),
    listViewButton(0L),
    iconViewButton(0L),
    viewButtonGroup(0L),
    urlLineEdit(0L),
    okButton(0L),
    cancelButton(0L),
    parent_(parent)
{
}

VFSObjectDialog::Private::~Private()
{
  if( objectList )
  {
    delete objectList;
    objectList = 0L;
  }

//  delete folderList;
//  folderList=0L;
//
//  delete objectListView;
//  objectListView=0L;
//
//  delete objectIconView;
//  objectIconView=0L;
//
//  delete objectPreview;
//  objectPreview=0L;
}


VFSObjectDialog::VFSObjectDialog(const QString& fileListComponent_ID,
                             const QString& root,
                             QWidget *parent,
                             const char *name)
  : QDialog(parent, name, true)
{
  d = new Private(this);
  
  setName( "Object Dialog" );
  setMinimumSize( 240, 240 );
  resize( 640, 480 );
  
  QIconLoader* iconLoader = QIconLoader::instance();
  
  QVBoxLayout* dialogLayout = new QVBoxLayout( this, 5, 5, "ObjectDialogLayout" );

  QHBoxLayout* topButtonLayout = new QHBoxLayout( 0L, 1, 1, "topButtonLayout" );
  if( !d->previewToggleButton )
  {
    d->previewToggleButton = new QPushButton( this, "previewToggleButton" );
    d->previewToggleButton->setToggleButton( true );
    d->previewToggleButton->setIconSet( iconLoader->barIcon("thumbnail") );
    d->previewToggleButton->setDisabled( true );
    topButtonLayout->addWidget( d->previewToggleButton );
  }
  if( !d->viewButtonGroup )
  {
    d->viewButtonGroup = new QHButtonGroup( this, "viewButtonGroup" );
    d->viewButtonGroup->setExclusive( true );
    topButtonLayout->addWidget( d->viewButtonGroup );
  }
  if( !d->listViewButton )
  {
    d->listViewButton = new QPushButton( d->viewButtonGroup, "listViewButton" );
    d->listViewButton->setToggleButton( true );
    d->listViewButton->setIconSet( iconLoader->iconSet("view_detailed") );
    d->listViewButton->setOn( false );
    d->listViewButton->setDisabled( true );
  }
  if( !d->iconViewButton )
  {
    d->iconViewButton = new QPushButton( d->viewButtonGroup, "iconViewButton" );
    d->iconViewButton->setToggleButton( true );
    d->iconViewButton->setIconSet( iconLoader->iconSet("view_icon") );
    d->iconViewButton->setOn( false );
    d->iconViewButton->setDisabled( true );
  }
  if( !d->urlLineEdit)
  {
    d->urlLineEdit = new QLineEdit( this, "urlLineEdit" );
    d->urlLineEdit->setText( "/Aethera/VFS/" );
    topButtonLayout->addWidget( d->urlLineEdit );
  }
  dialogLayout->addLayout( topButtonLayout );

  QSplitter* folderFileSplitter = new QSplitter(Qt::Horizontal, this, "folderFileSplitter");
  
  if(!d->objectListFactory)
  {
      ComponentTrader *trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
      if( trader )
      {
          ComponentRef ref = trader->registeredComponent(fileListComponent_ID);
          if( ref.isValid() )
          {
            d->objectListFactory = static_cast<ObjectListFactory*>(ref.instance());
          }
      }
  }
  
  if( d->objectListFactory )
  {
    if( !d->objectList )
    {
      d->objectList = dynamic_cast<VFSObjectList*>
                      ( d->objectListFactory->newObjectList() );
    }
  }

  if( !d->objectList )
  {
    return;
  }

  if( !d->folderList )
    d->folderList = new FolderListWidget(root, folderFileSplitter, "d->folderList");
  
  if(d->folderList)
  {
    connect( d->folderList,
             SIGNAL( fullPathText(const QString&) ),
             d->urlLineEdit,
             SLOT( setText(const QString&) )
           );
  }

  QSplitter* viewPreviewSplitter = new QSplitter(Qt::Vertical, folderFileSplitter, "viewPreviewSplitter");

  if(!d->objectListViewStack)
    d->objectListViewStack = new QWidgetStack(viewPreviewSplitter, "d->objectListViewStack");

  if( d->objectList )
  {
    if( d->objectList->hasCapability(OBJECT_VIEW) )
    {
      if( d->objectList->hasMultipleView() )
      {
        if( d->objectList->hasView( VFSObjectList::ListView ) )
        {
          d->listViewButton->setEnabled( true );
          d->listViewButton->setOn( true );
          
          d->objectListView = d->objectList->newObjectView(this, "objectListView");
          d->objectListView->slotSetFolder( root );

          d->objectListViewStack->addWidget( d->objectListView );
          d->objectListViewStack->raiseWidget( d->objectListView );

          //connect the folder tree to the list view
          connect( d->folderList,
                   SIGNAL( fullPathText(const QString&) ),
                   d->objectListView,
                   SLOT( slotSetFolder(const QString&) )
                 );
                 
          d->viewButtonGroup->insert( d->listViewButton, d->objectListViewStack->id( d->objectListView ) );
          d->viewButtonGroup->setButton( d->objectListViewStack->id( d->objectListView ) );
          connect( d->viewButtonGroup,
                   SIGNAL( clicked(int) ),
                   d->objectListViewStack,
                   SLOT( raiseWidget(int) )
                 );
        }
        if( d->objectList->hasView(VFSObjectList::IconView) )
        {
          d->iconViewButton->setEnabled( true );
          
          d->objectIconView = d->objectList->newObjectIconView(this, "objectListView");
          d->objectIconView->slotSetFolder( "/Aethera/VFS" + root );
                                         
          d->objectListViewStack->addWidget( d->objectIconView );

          connect( d->folderList,
                   SIGNAL( fullPathText(const QString&) ),
                   d->objectIconView,
                   SLOT( slotSetFolder(const QString&) )
                 );

          d->viewButtonGroup->insert( d->iconViewButton, d->objectListViewStack->id(d->objectIconView) );
          connect( d->viewButtonGroup,
                   SIGNAL( clicked(int) ),
                   d->objectListViewStack,
                   SLOT( raiseWidget(int) )
                 );

        }
      }
      else //no multiple view, only a list view
      {
        d->objectListView = d->objectList->newObjectView(this, "objectListView");
        d->objectListView->slotSetFolder( "/Aethera/VFS" + root );

        d->objectListViewStack->addWidget( d->objectListView );
        d->objectListViewStack->raiseWidget( d->objectListView );

        d->viewButtonGroup->insert( d->listViewButton, d->objectListViewStack->id(d->objectListView) );
        d->viewButtonGroup->insert( d->iconViewButton, d->objectListViewStack->id(d->objectIconView) );

        //connect the folder tree to the list view
        connect( d->folderList,
                 SIGNAL( fullPathText(const QString&) ),
                 d->objectListView,
                 SLOT( slotSetFolder(const QString&) )
               );
      }
    }
    if( d->objectList->hasCapability(OBJECT_PREVIEW) )
    {
      d->previewToggleButton->setEnabled( true );
      d->previewToggleButton->setOn( true );
      
      if(!d->objectPreview)
        d->objectPreview = d->objectList->newObjectPreview(viewPreviewSplitter, "d->objectPreview");

      connect( d->previewToggleButton,
               SIGNAL( toggled(bool) ),
               d->objectPreview,
               SLOT( slotToggleVisibility(bool) )
             );

      //if we have list view connect the list view to the preview
      if( d->objectListView )
      {
        connect( d->objectListView,
                 SIGNAL( objectSelected(const QString&) ),
                 d->objectPreview,
                 SLOT( slotSetObject(const QString&) )
               );
      }

      //if we have icon view connect the icon view to the preview
      if( d->objectIconView )
      {
        connect( d->objectIconView,
                 SIGNAL( objectSelected(const QString&) ),
                 d->objectPreview,
                 SLOT( slotSetObject(const QString&) )
               );
      }

    }
  }
  dialogLayout->addWidget( folderFileSplitter );

  QHBoxLayout* buttomButtonLayout = new QHBoxLayout( 0L, 3, 3, "buttomButtonLayout" );
  QSpacerItem* leftSpace = new QSpacerItem( 250, 20 );
  buttomButtonLayout->addItem( leftSpace );
  if( !d->okButton )
  {
    d->okButton = new QPushButton( "Ok", this, "okButton");
    connect( d->okButton,
             SIGNAL( pressed() ),
             this,
             SLOT( accept() )
           );
    buttomButtonLayout->addWidget(d->okButton);
  }
  if( !d->cancelButton )
  {
    d->cancelButton = new QPushButton( "Cancel", this, "cancelButton");
    connect( d->cancelButton,
             SIGNAL( pressed() ),
             this,
             SLOT( reject() )
           );
    buttomButtonLayout->addWidget(d->cancelButton);
  }

  QSpacerItem* rightSpace = new QSpacerItem( 250, 20 );
  buttomButtonLayout->addItem( rightSpace );
  dialogLayout->addLayout( buttomButtonLayout );

  //set the proportins of the splitters
  QValueList<int> splittProportion;
  splittProportion.append( width() / 4 );
  splittProportion.append( 3*width()/ 4 );
  //set the vertical splitter proportion
  folderFileSplitter->setSizes( splittProportion );
  splittProportion.clear();

  splittProportion.append( folderFileSplitter->height() / 3 );
  splittProportion.append( 2 * folderFileSplitter->height() / 3 );
  //set the horizontal splitter proportion
  viewPreviewSplitter->setSizes( splittProportion );

  //set the tab order
  setTabOrder( d->previewToggleButton, d->viewButtonGroup );
  setTabOrder( d->viewButtonGroup, d->folderList );
  setTabOrder( d->folderList, d->objectListViewStack );
  if( d->objectPreview )
  {
    setTabOrder( d->objectListViewStack, d->objectPreview );
    setTabOrder( d->objectPreview, d->okButton );
  }
  else
  {
    setTabOrder( d->objectListViewStack, d->okButton );
  }
  setTabOrder( d->okButton, d->cancelButton );
}


VFSObjectDialog::~VFSObjectDialog()
{
}

QString VFSObjectDialog::selectedObject()
{
  // if we have only one file view
  if( !d->objectList->hasMultipleView() )
  {
    return d->objectListView->getSelectedObject();
  }
  else
  {
    // if we have multiple views, try to figure out which one is on top
    if( d->viewButtonGroup->selected() == d->listViewButton )
    {
      return d->objectListView->getSelectedObject();
    }
    else if( d->viewButtonGroup->selected() == d->iconViewButton )
    {
      return d->objectIconView->getSelectedObject();
    }
  }
  
  return QString::null;
}

QStringList VFSObjectDialog::selectedObjects()
{
  QStringList urls;
  
  urls += selectedObject();
  
  return urls;
}

QString VFSObjectDialog::getOpenFile(const QString& component_Id, const QString& folder)
{
  VFSObjectDialog fileDialog( component_Id, folder, 0L, "fileDialog" );

  if( fileDialog.exec() == QDialog::Accepted )
  {
  }

  return QString::null;
}
