/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.

  ______________________________________________________________________________
  DATE        : 15 April 2003
  COPYRIGHT   : (C) 2003 by theKompany.com
  AUTHOR      : Eugen Constantinescu <eug@thekompany.com>
  AUTHOR      : Iulian Ghiciuc <ilie@eco-spaces.com.ro>
  USAGE       : just include this file whenever you need to create a object of
                this class
  BUGS        : 1) the iconview and listview pixmaps don't show when we open a file
                dialog for contacts but do show when we open a file dialog for
                notes
                2) a possible memory leak in the static methos getOpenFile.
                If i delete the objects i create i get a segmentation fault
                after i close the application
  DESCRIPTION : with this class we provide a object open dialog of the
                objects saved in the Aethera VFS. The dialog provides multiple
                views (listView and iconView) and preview.
  ______________________________________________________________________________
*/

#ifndef VFS_OBJECT_DIALOG_H
#define VFS_OBJECT_DIALOG_H

//Aethera includes
#include "vfsobjectlistglobals.h"
#include "vfsobjectlist.h"
#include "vfsobjectlistview.h"
#include "vfsobjecticonview.h"
#include "folderlistwidget.h"
#include "komaetheraplugin.h"

//QT includes
#include <qdialog.h>
#include <qwidgetstack.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlineedit.h>

class AETHERA_API VFSObjectDialog : public QDialog
{
    Q_OBJECT

    public:
      /*
      PURPOSE    : constructs a VFSObjectDialog object
      PARAMETERS :
        fileListComponent_ID - this string is used by the file dialog to request
                               a file list component exported by each plugin from
                               the korelib. The complete list of components that
                               one can load can be found in the
                               vfsobjectlistviewglobals.h file. That component will
                               provide the preview widget and the file view widgets
                               that the dialog will display. A list view will always
                               be availabile.
        root - this is the folder that we what the dialog to open in. Because
               certain widgets were designed before this dialog and we can't easily
               change then you are required to pass "/Contacts", "/Notes",....
               to this parameter
        parent - the parent of the dialog
        name - internal id name
      */
      VFSObjectDialog(const QString& fileListComponent_ID,
                    const QString& root=QString::null,
                    QWidget *parent=0,
                    const char *name=0
                   );
               
      /*
      PURPOSE    : destruct a VFSObjectDialog object
      PARAMETERS : none
      */
      virtual ~VFSObjectDialog();
  
      /*
      PURPOSE    : use this static method when you whant to open a file in the Aethera
                   VFS. This method will return the full VFS path of the object you
                   selected in the object view or string null in any other case.
      PARAMETERS : component_Id - the dialog will query the korelib for the component
                                  identified by component_Id
                   folder - the root folder loaded by the dialog. This parameter should be
                            something in the form of "/Notes", "/Contacts", etc. Asking for a
                            folder that doesn't exist will not give nice results because we don't
                            check for existence
      */
      static QString getOpenFile( const QString& component_Id, const QString& folder );

    protected:
    
      /** Return the VFS url for selected object.*/
      QString selectedObject();
      
      /** Return the VFS url-s for selected objects.*/
      QStringList selectedObjects();
      
    private:

      class Private;
      Private * d;
};

#endif
