/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.

    ______________________________________________________________________________
    DATE        : 15 April 2003
    COPYRIGHT   : (C) 2003 by theKompany.com
    AUTHOR      : Eugen Constantinescu <eug@thekompany.com>
    AUTHOR      : Iulian Ghiciuc (ilie@eco-spaces.com.ro)
    BUGS        : none that I know of :)
    DESCRIPTION : this abstract class is provied for interface with the listView,
                  iconView, preview widgets. I can access those widgets directlly
                  but it is better to do it from here. This interface is used by the
                  VFSFileDialog. All plugins must reimplement this class to provide
                  functionality
    ______________________________________________________________________________
*/

#ifndef VFS_OBJECT_LIST_H
#define VFS_OBJECT_LIST_H

//own includes
#include "vfsobjectlistglobals.h"
#include "vfsobjectlistview.h"
#include "vfsobjecticonview.h"
#include "vfsobjectpreview.h"

//Aethera includes
#include "komaetheraplugin.h"

//QT includes
#include <qstring.h>
#include <qwidget.h>

class AETHERA_API VFSObjectList
{
  public:
  
    /*
    PURPOSE : the type of view that this class can provide
    */
    enum ViewType
    {
      ListView=0,
      IconView=1
    };

    /*
    PURPOSE    : construct a VFSObjectList object
    */  
    VFSObjectList();

    /*
    PURPOSE    : destruct a VFSObjectList object
    PARAMETERS : none
    */
    virtual ~VFSObjectList() {};

    /*
    PURPOSE    : return true if the component provides multiple view (listView
                 and iconView). By default this method return false but all
                 plugins that whant to provide multiple views must reimplement
    PARAMETERS : none
    */
    virtual bool hasMultipleView();

    
    /*
    PURPOSE    : query this class if it provides a certain type of view
    PARAMETERS : t - the type of view we whant to find out about
    */
    virtual bool hasView(ViewType);

    /*
    PURPOSE    : return a pointer to a VFSObjectIconView object. By default this return the null
                 pointer. Plugins must reimplement to provide an iconView. God help you if
                 you call this before makeing sure that the class has the view you ask for
    PARAMETERS : parent - parent
                 name - name
    */
    virtual VFSObjectIconView* newObjectIconView(QWidget* parent=0, const char* name=0);

    /*
    PURPOSE    : return true is the component has the capability you ask for and false in any
                 other case. The list of capabilities can be found in the
                 vfsobjectlistviewglobals.h
    PARAMETERS : capa - a string reprezenting the capability. Should be one found in the
                        mentioned file
    */
    virtual bool hasCapability(const QString& capa) = 0;

    /*
    PURPOSE    : return a pointer to a VFSObjectListView object. This is the minimum a component
                 has to provide
    PARAMETERS : parent - parent
                 name - name
    */
    virtual VFSObjectListView* newObjectView(QWidget* parent=0, const char* name=0) = 0;


    /*
    PURPOSE    : return a pointer to a VFSObjectPreview object. Should return null if the
                 plugin does not provide one
    PARAMETERS : parent - parent
                 name - name
    */
    virtual VFSObjectPreview* newObjectPreview(QWidget* parent=0, const char* name=0) = 0;
};


/*
PURPOSE : this class is the ObjectList factory. Like the ObjectList class you must inherit this
          class in order to create objects of ObjectList type.
*/
class ObjectListFactory : public KomAetheraService
{
  public:
    ObjectListFactory(){}
    ~ObjectListFactory(){}

    virtual VFSObjectList* newObjectList()=0;
};

#endif
