/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.

    ______________________________________________________________________________
    DATE        : 15 April 2003
    COPYRIGHT   : (C) 2003 by theKompany.com
    AUTHOR      : Eugen Constantinescu <eug@thekompany.com>
    AUTHOR      : Iulian Ghiciuc <ilie@eco-spaces.com.ro>
    BUGS        : none that i know of :)
    DESCRIPTION : the listView widget diplayed in the VFSFileDialog.
                  The VFSObjectListViewItem (a name a bit too long) is reimplemented
                  to provide altenative coloration
    ______________________________________________________________________________
*/

#ifndef VFS_LIST_OBJECT_VIEW_H
#define VFS_LIST_OBJECT_VIEW_H

//QT includes
#include <qlistview.h>
#include <qpainter.h>
#include <qpalette.h>

#include <defines.h>

class AETHERA_API VFSObjectListView : public QListView
{
  Q_OBJECT
  
  public:
  
    /*
    PURPOSE    : construct a VFSObjectListView object
    PARAMETERS : parent - parent
                 name - internal name
    */
    VFSObjectListView(QWidget* parent=0, const char* name=0);

    
    /*
    PURPOSE    : deconstruct a VFSObjectListView object
    PARAMETERS : none
    */
    virtual ~VFSObjectListView(){}

    /*
    PURPOSE    : return the object that is selected now in the listView
    PARAMETERS : none
    */
    virtual QString getSelectedObject()=0;

  public slots:
  
    /*
    PURPOSE    : the listView connects to this slot in order to emit the
                 objectSelected signal with the full VFS path
    PARAMETERS : itm - the currently selected item
    */
    virtual void slotObjectSelected(QListViewItem* itm )=0;

    /*
    PURPOSE    : external widgets connect to this slot to instruct the listView
                 to load the objects in the fld folder
    PARAMETERS : fld - the folder it has to load
    */
    virtual void slotSetFolder(const QString& fld)=0;

};


class AETHERA_API VFSObjectListViewItem : public QListViewItem
{
  public:
  
    /*
    PURPOSE    : construct a VFSObjectListViewItem
    PARAMETERS : parent - parent
                 name - name
    */
    VFSObjectListViewItem(VFSObjectListView* parent=0, const char* name=0);

    
    /*
    PURPOSE    : destruct a VFSObjectListViewItem
    PARAMETERS : none
    */
    ~VFSObjectListViewItem();

    /*
    PURPOSE    : this method is reimplemented to provide alternative coloration
    PARAMETERS : see QT documenation for description on parameters
    */
    virtual void paintCell( QPainter *p,
                            const QColorGroup &cg,
                            int column,
                            int width,
                            int alignment
                          );

    /*
    PURPOSE    : return true is the current item has alternante color
    PARAMETERS : none
    */
    bool isAlternante();

  private:
  
    class Private;
    Private * d;
    
};

#endif
