/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 /** Changed by Eugen C. <eug@thekompany.com>
   * Date: 28 Aug 2001
   */

#include <cassert>
#include <vobject.h>
#include <vfolder.h>
#include <vreference.h>
#include <vfsmodule.h>
#include <any.h>

VFSModule * VObject::vfsModule_ = 0;

VObject::VObject(const QString & path)
  : VReference  (),
    folder_     (0)
{
  int lastSlash = path.findRev('/');
 
  if (lastSlash > -1)
  {
    id_ = path.mid(lastSlash + 1);
  }
  else
  {
    id_ = path;
  }
 
  folder_ = vfsModule()->folderRef(path.left(lastSlash));

  if (0 != folder_)
  {
    folder_->addObject(this);
  }
}

VObject::VObject(const QString & path, const QByteArray & /* data */)
  : VReference  (),
    folder_     (0)
{
  if( !vfsModule_ )
  {
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfsModule_ = static_cast<VFSModule*>( ref.instance() );
      else
        qDebug("VObject: Can't find VFS module");
    }
  }
  
  int lastSlash = path.findRev('/');
  
  if (lastSlash > -1)
  {
    id_ = path.mid(lastSlash + 1);
  }
  else
  {
    id_ = path;
  }
  
  folder_ = vfsModule_->folderRef(path.left(lastSlash));
  
  if (0 != folder_)
  {
    folder_->addObject(this);
  }
}

VObject::~VObject()
{
  if (0 != folder_)
    folder_->removeObjectRef(this);
    
//  qDebug("VObject destructor! - %s", name().latin1());
}

QString VObject::path() const
{
  if (0 != folder_)
  {
    return folder_->path() + "/" + id_;
  }
  else
  {
    return id_;
  }
}

QString VObject::name() const
{
  return id_;
}

QString VObject::url() const
{
  return type() + ":/" + path();
}

QString VObject::type() const
{
  if (0 != folder_)
  {
    return folder_->type();
  }
  else
  {
    return QString("unknown");
  }
}

QString VObject::serverPath() const
{
  if (0 != folder_)
  {
    return folder_->serverPath() + "/" + id_;
  }
  else
  {
    return QString::null;
  }
}

QString VObject::binding() const
{
  if (0 != folder_)
  {
    return folder_->binding();
  }
  else
  {
    return QString::null;
  }
}

QByteArray VObject::data() const
{
  QByteArray ret;

  QString _path = path();

  if (!_path.isNull())
  {
    bool res = vfsModule_->getObjectData(_path, ret);
    
    if (!res)
    {
      qDebug
        (
          "VObject: getData failed, reason given by server: %s",
          vfsModule_->regetLastError().ascii()
        );
    }   
  }

  return ret;
}

bool VObject::setData(const QByteArray & newData)
{
  bool res = false;

  QString _path = path();
  
  if (!_path.isNull())
  {
    // Temporary.

    res = vfsModule_->setObjectData(_path, newData);
    
    if (!res)
    {
      qDebug
        (
          "vobject: setData failed, reason given by server: %s",
          vfsModule_->regetLastError().ascii()
        );
    }
  }
  
  return res;
}

VFolder * VObject::parentFolder()
{
  return folder_;
}

QString VObject::parentObject() const
{
  return vfsModule_->parent(path());
}

bool VObject::setParentObject(QString parentObj)
{
  return vfsModule_->setParent(path(), parentObj);
}

QStringList VObject::relatedList() const
{
  return vfsModule_->related(path());
}

bool VObject::setRelatedList(const QStringList & rList)
{
  return vfsModule_->setRelated(path(), rList);
}

bool VObject::save()
{
  // rikkus: Should be const, but it calls setData. Why ?
  return setData(_stream());
}

bool VObject::sync()
{
  _narrow(data());
  
  // XXX: Here we should check the BrokerInterface for errors.
  
  return true;
}

QueryProcessor::QueryType VObject::queryType(const QString & /* query */)
{
  // STUB - to be implemented by derived classes.
  return QueryProcessor::InvalidQuery;
}

VFSModule * VObject::vfsModule()
{
  if( !vfsModule_ )
  {
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfsModule_ = static_cast<VFSModule*>( ref.instance() );
      else
        qDebug("VObject: Can't find VFS module");
    }
  
    assert(0 != vfsModule_);
  }
  
  return vfsModule_;
}

