/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <brokeriface.h>
#include <qstring.h>
#include <qstringlist.h>
#include <magellan.h>
#include <commands.h>
#include <statuscodes.h>
#include <commframe.h>
#include <commbinder.h>
#include <qcstring.h>
#include <qdatastream.h>
#include <conversions.h>
#include <queryprocessor.h>
#include <clientvfs.h>
#include <vobject.h>
#include <vfolder.h>
#include <vobjectfactory.h>

static BrokerInterface *inst;

class BrokerInterface::Private
{
  public:
  
    Private();
    ~Private();
    
    bool copy(const QStringList & urls, QString parent_url);
    bool move(const QStringList & urls, QString parent_url);

  private:
};

BrokerInterface::Private::Private()
{
}

BrokerInterface::Private::~Private()
{
}

bool BrokerInterface::Private::move(const QStringList & urls, QString parent_url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::MoveObjects;
  
  QDataStream s(f.extraData, IO_WriteOnly);
  s <<  ClientVFS::thisInstance()->map(urls)
    <<  ClientVFS::thisInstance()->map(parent_url);
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  }
  
  return true;
}

bool BrokerInterface::Private::copy(const QStringList & urls, QString parent_url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::CopyObjects;
  
  QDataStream s(f.extraData, IO_WriteOnly);
  s <<  ClientVFS::thisInstance()->map(urls)
    <<  ClientVFS::thisInstance()->map(parent_url);
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  }
  
  return true;
}

BrokerInterface::BrokerInterface()
{
  // test the client-server loopback link
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::RcptOk;
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return;
  }
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    qDebug("Warning: the server did not complete the check request");
    return;
  }
  
  // everything went fine
  // qDebug("BrokerInterface: loopback client/server link initiated successfully");
  
  d = new Private();
}

BrokerInterface::~BrokerInterface()
{
  if(d)
  {
    delete d;
    d = 0;
  }
}

BrokerInterface *BrokerInterface::thisInstance()
{
  return inst==0?inst=new BrokerInterface():inst;
}

bool BrokerInterface::isFolder(QString url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::UrlIsFolder;
  f.extraData=_stream(url);
  
  // debug
  // qDebug("param: %s, from stream: %s", (const char *)url, (const char *)_narrowQString(f.extraData));
      
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return _narrowBool(r.extraData);
}

bool BrokerInterface::exists(QString url)
{
  if(url.isEmpty()) return false;

  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::UrlExists;

  // debug
  // qDebug("brokeriface: sending exists(%s), mapped from %s", (const char *)ClientVFS::thisInstance()->map(url), (const char *)url);

  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return _narrowBool(r.extraData);
}

QString BrokerInterface::parent(QString url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetParent;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  } 
  
  return ClientVFS::thisInstance()->reverseMap(_narrowQString(r.extraData));
}

QString BrokerInterface::folder(QString url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetFolder;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  } 
  
  return ClientVFS::thisInstance()->reverseMap(_narrowQString(r.extraData));
}

QStringList BrokerInterface::children(QString url)
{
  QStringList eList;
  
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetChildren;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  // debug
//  qDebug("brokerinterface: [children] url: %s, mapping %s", (const char *)url, (const char *)ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return eList;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return eList;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return eList;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return eList;
  } 
  
  //debug
  // qDebug("brokeriface: entrylist query: Completed_Yes");

  return ClientVFS::thisInstance()->reverseMap(_narrowQStringList(r.extraData));
}

QStringList BrokerInterface::subfolders(QString url)
{
  QStringList eList;
  
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetSubfolders;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return eList;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return eList;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return eList;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return eList;
  } 
  
  return ClientVFS::thisInstance()->reverseMap(_narrowQStringList(r.extraData));
}

QStringList BrokerInterface::related(QString url)
{
  QStringList eList;
  
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetRelated;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return eList;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return eList;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return eList;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return eList;
  } 
  
  return _narrowQStringList(r.extraData);
}

int BrokerInterface::type(QString url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetType;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return Magellan::Unknown;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return Magellan::Unknown;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return Magellan::Unknown;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return Magellan::Unknown;
  } 
  
  return _narrowInt(r.extraData);
}

int BrokerInterface::subtype(QString url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetSubtype;
  f.extraData=_stream(url);
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return Magellan::Unknown;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return Magellan::Unknown;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return Magellan::Unknown;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return Magellan::Unknown;
  } 
  
  return _narrowInt(r.extraData);
}


bool BrokerInterface::setRelated(QString url, QStringList rList)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::SetRelated;
  QDataStream s(f.extraData, IO_WriteOnly);
  s<<ClientVFS::thisInstance()->map(url)<<rList;
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }

  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return true;
}

bool BrokerInterface::setParent(QString url, QString parentUrl)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::SetParent;
  QDataStream s(f.extraData, IO_WriteOnly);
  s<<ClientVFS::thisInstance()->map(url)<<ClientVFS::thisInstance()->map(parentUrl);
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return true;
}

bool BrokerInterface::getObjectData(QString url, QByteArray &data)
{
  // query check
  if(url.find("?")>-1)
  {
    bool res=QueryProcessor::thisInstance()->getData(url, data);
    err=QueryProcessor::thisInstance()->lastError();
    return res;
  }
  
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetData;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));

  // debug
  // qDebug("brokeriface: sending getData(%s), mapped from %s", (const char *)ClientVFS::thisInstance()->map(url), (const char *)url);
    
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }

  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 

  data = r.extraData;
  r.extraData.detach();
  
  return true;
}
    
bool BrokerInterface::getServerObjectsData(const QStringList &urls, QByteArray &data)
{
// query check - unavailable
// TODO: query for a group of objects
  
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetObjectsData;
  f.extraData=_stream(urls);
  
  // debug
  // qDebug("brokeriface: sending getData(%s), mapped from %s", (const char *)ClientVFS::thisInstance()->map(url), (const char *)url);
    
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
//    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }

  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  data = r.extraData;
  r.extraData.detach();
  
  return true;
}

bool BrokerInterface::getObjectData(const QStringList &urls, QByteArray &data)
{
// query check - unavailable
// TODO: query for a group of objects
  
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetObjectsData;
  f.extraData=_stream(ClientVFS::thisInstance()->map(urls));
  
  // debug
  // qDebug("brokeriface: sending getData(%s), mapped from %s", (const char *)ClientVFS::thisInstance()->map(url), (const char *)url);
    
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }

  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 

  data = r.extraData;
  r.extraData.detach();
  
  return true;
}

bool BrokerInterface::getChildrenData(const QString & url, QByteArray &data)
{
// query check - unavailable
// TODO: query for a group of objects
  
  CommFrame f, r;
  QString serverUrl;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::GetChildrenData;
  serverUrl=ClientVFS::thisInstance()->map(url);
  fflush(stdout);
  if( serverUrl!=QString::null )
    f.extraData=_stream(serverUrl);
  else
  {
    qDebug("No map entry for %s url.",(const char *)url);
    return false;
  }

  // debug
//  qDebug("brokeriface: sending getChildrenData(%s), mapped from %s", (const char *)ClientVFS::thisInstance()->map(url), (const char *)url);
    
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }

  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 

  data = r.extraData;
  r.extraData.detach();
  
  return true;
}

QByteArray BrokerInterface::getSimpleData(QString url)
{
  // this is just a wrapper over getObjectData so that we can use _narrow directly
  // it makes sense only when we receive one data structure, such as a bool or QString
  QByteArray data;
  getObjectData(url, data);
  return data.copy();
}

unsigned BrokerInterface::getFolderRecentData(const QString & folderURL)
{
  CommFrame f, r;
  QString serverUrl;
  f.majorRevNo = 1;
  f.minorRevNo = 1;
  f.requestFrame = true;
  f.clientFrame = true;
  f.requestsReply = true;
  f.commandCode = MagellanComm::GetFolderRecentData;
  serverUrl = ClientVFS::thisInstance()->map( folderURL );
  
  if( serverUrl!=QString::null )
    f.extraData=_stream(serverUrl);
  else
  {
    qDebug("No map entry for %s url.", folderURL.latin1());
    return false;
  }

  // debug
//  qDebug("brokeriface: sending getFolderRecentData(%s), mapped from %s",
//    ClientVFS::thisInstance()->map( folderURL ).latin1(),
//    folderURL.latin1());

  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
 
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  }

  return _narrowInt(r.extraData);
}

bool BrokerInterface::setObjectData(QString url, const QByteArray &data)
{
  // query check
  if(url.find("?")>-1)
  {
    bool res=QueryProcessor::thisInstance()->setData(url, data);
    err=QueryProcessor::thisInstance()->lastError();
    return res;
  }
  
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::SetData;
  QDataStream s(f.extraData, IO_WriteOnly);
  s<<ClientVFS::thisInstance()->map(url)<<data;
  // debug
  // qDebug("brokeriface: sending setData(%s) to %s, mapped from %s", (const char *)ClientVFS::thisInstance()->map(url), (const char *)_narrowQString(data), (const char *)url);
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return true;
}

QString BrokerInterface::createObject(QString parent_url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::CreateObject;
  f.extraData=_stream(ClientVFS::thisInstance()->map(parent_url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  } 
  
  return ClientVFS::thisInstance()->reverseMap(_narrowQString(r.extraData));
}

QString BrokerInterface::createObject(QString parent_url, QByteArray &data)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::CreateObjectUsingData;
  QDataStream s(f.extraData, IO_WriteOnly);
  s << ClientVFS::thisInstance()->map(parent_url) << data;

  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }

  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  }

  return ClientVFS::thisInstance()->reverseMap(_narrowQString(r.extraData));
}

QString BrokerInterface::createFolder(QStringList &paramList)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::CreateFolder;
  f.extraData=_stream(paramList);

  // debug
  // qDebug("brokerinterface: sending CreateFolder(%s)", (const char *)_narrowQString(f.extraData));
    
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  } 
  
  return _narrowQString(r.extraData);
}

QString BrokerInterface::copy(QString url, QString dest_parent_url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::CopyUrl;
  QDataStream s(f.extraData, IO_WriteOnly);
  s<<ClientVFS::thisInstance()->map(url)<<ClientVFS::thisInstance()->map(dest_parent_url);
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  } 
  
  return ClientVFS::thisInstance()->reverseMap(_narrowQString(r.extraData));
}

QString BrokerInterface::move(QString url, QString dest_parent_url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::MoveUrl;
  QDataStream s(f.extraData, IO_WriteOnly);
  s<<ClientVFS::thisInstance()->map(url)<<ClientVFS::thisInstance()->map(dest_parent_url);
  
  // debug
  // qDebug("brokerinterface: trying to move %s->%s...", (const char *)ClientVFS::thisInstance()->map(url), (const char *)ClientVFS::thisInstance()->map(dest_parent_url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  } 
  
  return ClientVFS::thisInstance()->reverseMap(_narrowQString(r.extraData));
}

bool BrokerInterface::send(QString url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::SendUrl;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return true;
}

bool BrokerInterface::remove(QString url)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::RemoveUrl;
  f.extraData=_stream(ClientVFS::thisInstance()->map(url));
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return true;
}

bool BrokerInterface::removeFolder(QStringList &paramList)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::RemoveFolder;
  f.extraData=_stream(paramList);
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  } 
  
  return true;
}

QString BrokerInterface::regetLastError()
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::RegetLastError;
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return QString::null;
  }
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return QString::null;
  }
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return QString::null;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return QString::null;
  } 
  
  return _narrowQString(r.extraData);
}

QString BrokerInterface::lastError()
{
  return err;
}

bool BrokerInterface::copy(const QStringList & urls, QString parent_url)
{
  return d->copy(urls, parent_url);
/*  QStringList temp;
  int i = 1;
  bool bRet=true;
  for(QStringList::ConstIterator it=urls.begin(); it!=urls.end() && bRet; ++it)
  {
    temp.append(*it);
    if( !(i%500) )
    {
      bRet = d->copy(temp, parent_url);
      temp.clear();
      i = 1;
    }
    i++;
  }
  
  if(temp.count() && bRet)
  {
    bRet = d->copy(temp, parent_url);
    temp.clear();
  }
  
  return bRet;*/
}

bool BrokerInterface::move(const QStringList & urls, QString parent_url)
{
  return d->move(urls, parent_url);
/*  QStringList temp;
  int i = 1;
  bool bRet=true;
  for(QStringList::ConstIterator it=urls.begin(); it!=urls.end() && bRet; ++it)
  {
    temp.append(*it);
    if( !(i%500) )
    {
      bRet = d->move(temp, parent_url);
      temp.clear();
      i = 1;
    }
    i++;
  }
  
  if(temp.count() && bRet)
  {
    bRet = d->move(temp, parent_url);
    temp.clear();
  }
  
  return bRet;*/
}

bool BrokerInterface::remove(const QStringList & urls)
{
  CommFrame f, r;
  f.majorRevNo=1;
  f.minorRevNo=1;
  f.requestFrame=true;
  f.clientFrame=true;
  f.requestsReply=true;
  f.commandCode=MagellanComm::RemoveObjects;
  
  f.extraData = _stream( ClientVFS::thisInstance()->map(urls) );
  
  if(!CommBinder::thisInstance()->send("thisInstance", f, r))
  {
    qDebug("Warning: no link could be established with the server, CommBinder returned an error");
    return false;
  }
  
  if(!r.hasStatus)
  {
    qDebug("Warning: no link could be established with the server, no status was retrieved");
    return false;
  }
  
  if(r.statusCode!=StatusInfo::Ok)
  {
    qDebug("Warning: no link could be established with the server, an invalid status was received");
    return false;
  }
  
  if(r.completionStatus!=StatusInfo::Completed_Yes)
  {
    return false;
  }
  
  return true;
}

