/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#ifndef __BrokerInterface__
#define __BrokerInterface__

#include <qstring.h>
#include <qstringlist.h>

/*
This class is part of the client core implementation.
It defines the client object broker, which is an interface to an arbitrary
collection of server ORBs.
The client-server communication is defined in CommBinder (see commbinder.h)
which acts as a transport agent for the BrokerInterface and KernelScriptEngine.
This class is unique in an Aethera client.
*/

class BrokerInterface
{
  public:
  
    BrokerInterface();
    ~BrokerInterface();
    
    static BrokerInterface *thisInstance();

    // URL-centered methods
    bool isFolder(QString url);          // true if the url is an existent folder
    bool exists(QString url);            // true if the url is an existent folder or object
    QString parent(QString url);         // returns the logical parent of a folder or object
    QString folder(QString url);         // returns the folder parent of a folder or object
    QStringList children(QString url);   // returns the list of child objects
    QStringList subfolders(QString url); // returns the list of child folders
    QStringList related(QString url);   // returns the list of related objects
    int type(QString url);               // returns the object major type, based on its url
    int subtype(QString url);            // returns the object fine type

    // set methods
    bool setRelated(QString url, QStringList rList); // warning: this works on server URLs
    bool setParent(QString url, QString parentUrl);

    // object-centered methods
    bool getObjectData(QString url, QByteArray &data);
    /** Get data for many objects.*/
    bool getObjectData(const QStringList & urls, QByteArray & data);
    /** Get data for many objects.*/
    bool getServerObjectsData(const QStringList & urls, QByteArray & data);
    /** Get all folder's children data.*/
    bool getChildrenData(const QString & url, QByteArray & data);
    QByteArray getSimpleData(QString url);
    /** Get the recent mails number from a mails folder.*/
    unsigned getFolderRecentData(const QString & folderURL);
    
    bool setObjectData(QString url, const QByteArray &data);
    
    /** Create a new object on the server.*/
    QString createObject(QString parent_url);
    QString createObject(QString parent_url, QByteArray &data);
    bool send(QString url);
    QString copy(QString url, QString parent_url);
    QString move(QString url, QString parent_url);
    /** Remove the object.*/
    bool remove(QString url);
    
    // multiple objects operations
    bool copy(const QStringList & urls, QString parent_url);
    bool move(const QStringList & urls, QString parent_url);
    bool remove(const QStringList & urls);
    
    /** Create a new folder on the server.*/
    QString createFolder(QStringList& paramList);
    /** Remove the folder.*/
    bool removeFolder(QStringList& paramList);

    // error handling
    QString lastError();
    QString regetLastError();
    
  private:
  
    class Private;
    Private * d;
    
    QString err;
};

#endif


