/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <clientrmi.h>
#include <any.h>
#include <commands.h>
#include <rmicalldata.h>
#include <commframe.h>
#include <qstring.h>
#include <qstringlist.h>
#include <commbinder.h>
#include <conversions.h>
#include <clientvfs.h>
#include <qdatastream.h>
#include <vobject.h>
#include <vfolder.h>
#include <vobjectfactory.h>

ClientRMI *ClientRMI::thisInstance = 0;

ClientRMI *ClientRMI::ref()
{
  if( !thisInstance )
    thisInstance = new ClientRMI();
  return thisInstance;
}

ClientRMI::~ClientRMI()
{
}

void ClientRMI::__RMICall(const RMICallData &data)
{
  CommFrame inFrame;
  
  inFrame.majorRevNo=1;
  inFrame.minorRevNo=1;
  inFrame.requestFrame=true;
  inFrame.clientFrame=true;
  inFrame.deniesReply=true;
  inFrame.hasData=true;
  inFrame.commandCode=MagellanComm::RMICall;
  
  inFrame.extraData=_stream(data);
  
  CommBinder::thisInstance()->send("thisInstance", inFrame);
}

Any ClientRMI::__RMICallback(const RMICallData &data)
{
  CommFrame inFrame, outFrame;
  
  inFrame.majorRevNo=1;
  inFrame.minorRevNo=1;
  inFrame.requestFrame=true;
  inFrame.clientFrame=true;
  inFrame.requestsReply=true;
  inFrame.hasData=true;
  inFrame.commandCode=MagellanComm::RMICall;
  
  inFrame.extraData=_stream(data);
  
  CommBinder::thisInstance()->send("thisInstance", inFrame, outFrame);
  
  return _narrowAny(outFrame.extraData);
}

void ClientRMI::__process(const CommFrame &inFrame)
{
  RMICallData data=_narrowRMICallData(inFrame.extraData);
  
  if(data.method=="sendPOP3Progress(QString)")
  {
    sendPOP3Progress(data.param[0].toString());
  }
  else if(data.method=="sendSMTPProgress(QString)")
  {
    sendSMTPProgress(data.param[0].toString());
  }
}

void ClientRMI::__process(const CommFrame &inFrame, CommFrame &outFrame)
{
  RMICallData data=_narrowRMICallData(inFrame.extraData);
  
  if(data.method=="getSMTPAction(QString,QString,int)")
  {
    outFrame.extraData=_stream(Any(getSMTPAction(data.param[0].toString(), data.param[1].toString(), (int)data.param[2])));
  }
  if(data.method=="getMailAccountPassword(QString)")
  {
    outFrame.extraData=_stream(Any(getMailAccountPassword(data.param[0].toString())));
  }
}

QStringList ClientRMI::getAccountNames(int accountType)
{
  RMICallData data;
  data.method="getAccountNames(int)";
  data.pcount=1;
  data.param.append(Any(accountType));
  return __RMICallback(data).toStringList();
}

QString ClientRMI::getDefaultAccount()
{
  RMICallData data;
  data.method="getDefaultAccount()";
  data.pcount=0;
  return __RMICallback(data).toString();
}

bool ClientRMI::isAccountActive(QString account)
{
  RMICallData data;
  data.method="isAccountActive(QString)";
  data.pcount=1;
  data.param.append(Any(account));
  return (int)__RMICallback(data);
}

QString ClientRMI::getAccountFromAddress(QString account)
{
  RMICallData data;
  data.method="getAccountFromAddress(QString)";
  data.pcount=1;
  data.param.append(Any(account));
  return __RMICallback(data).toString();
}

QString ClientRMI::getAccountReplyToAddress(QString account)
{
  RMICallData data;
  data.method="getAccountReplyToAddress(QString)";
  data.pcount=1;
  data.param.append(Any(account));
  return __RMICallback(data).toString();
}

void ClientRMI::startSendReceive(QString account)
{
  RMICallData data;
  data.method="startSendReceive(QString)";
  data.pcount=1;
  data.param.append(Any(account));
  __RMICall(data);
}

void ClientRMI::startSend(QString account)
{
  RMICallData data;
  data.method="startSend(QString)";
  data.pcount=1;
  data.param.append(Any(account));
  __RMICall(data);
}

void ClientRMI::startReceive(QString account)
{
  RMICallData data;
  data.method="startReceive(QString)";
  data.pcount=1;
  data.param.append(Any(account));
  __RMICall(data);
}

void ClientRMI::stopPOP3Delivery()
{
  RMICallData data;
  data.method="stopPOP3Delivery()";
  data.pcount=0;
  __RMICall(data);
}

void ClientRMI::setAutoExpunge(bool mode)
{
  RMICallData data;
  data.method="setAutoExpunge(bool)";
  data.pcount=1;
  data.param.append(Any(mode));
  __RMICall(data);
}

bool ClientRMI::mount(QString url)
{
  RMICallData data;
  data.method="mount(QString)";
  data.pcount=1;
  data.param.append(Any(ClientVFS::thisInstance()->map(url)));
  return __RMICallback(data).toInt();
}

bool ClientRMI::unmount(QString url)
{
  RMICallData data;
  data.method="unmount(QString)";
  data.pcount=1;
  data.param.append(Any(ClientVFS::thisInstance()->map(url)));
  return __RMICallback(data).toInt();
}

bool ClientRMI::subfolders(QString accountName, QString mailboxName, QString type)
{
  RMICallData data;
  data.method="subfolders(QString, QString, QString)";
  data.pcount=3;
  data.param.append( Any(accountName) );
  data.param.append( Any(mailboxName) );
  data.param.append( Any(type) );
  
  return __RMICallback(data).toInt();
}

bool ClientRMI::runKolabJob(
  const QString & accountName,
  const QString & mailboxName,
  const QString & state )
{
  RMICallData data;
  data.method="runKolabJob(QString, QString, QString)";
  data.pcount=3;
  data.param.append( Any(accountName) );
  data.param.append( Any(mailboxName) );
  data.param.append( Any(state) );
  
  return __RMICallback(data).toInt();
}

bool ClientRMI::runKolabSharedJob(
  const QString & accountName,
  const QString & mailboxName,
  const QString & state )
{
  RMICallData data;
  data.method="runKolabSharedJob(QString, QString, QString)";
  data.pcount=3;
  data.param.append( Any(accountName) );
  data.param.append( Any(mailboxName) );
  data.param.append( Any(state) );
  
  return __RMICallback(data).toInt();
}

bool ClientRMI::expunge(const QString & url)
{
  RMICallData data;
  
  data.method = "expunge(QString)";
  data.pcount = 1;
  data.param.append( Any(ClientVFS::thisInstance()->map(url)) );
  
  return __RMICallback(data).toInt();
}

bool ClientRMI::rename(const QString & url, const QString & newName)
{
  RMICallData data;
  
  data.method="rename(QString, QString)";
  data.pcount=2;
  data.param.append( Any(ClientVFS::thisInstance()->map(url)) );
  data.param.append( Any(newName) );
  
  return __RMICallback(data).toInt();
}


  void
ClientRMI::loadAccounts()
{
  RMICallData data;
  data.method="loadAccounts()";
  data.pcount=0;

  __RMICall(data);
}

  bool
ClientRMI::addAccount(const Account& account)
{
  QByteArray a;
  QDataStream str(a, IO_WriteOnly);
  str << account;

  RMICallData data;
  data.method="addAccount(QByteArray)";
  data.pcount=1;
  data.param.append(Any(a));
  
  return __RMICallback(data).toInt();
} 

  bool
ClientRMI::deleteAccount(QString accountName)
{
  RMICallData data;
  data.method="deleteAccount(QString)";
  data.pcount=1;
  data.param.append(Any(accountName));
  
  return __RMICallback(data).toInt();
}

  void
ClientRMI::setDefaultAccount(QString accountName)
{
  RMICallData data;
  data.method="setDefaultAccount(QString)";
  data.pcount=1;
  data.param.append(Any(accountName));
  
  __RMICall(data);
}

  Account
ClientRMI::getAccount(QString accountName)
{
  RMICallData data;
  data.method="getAccount(QString)";
  data.pcount=1;
  data.param.append(Any(accountName));

  QByteArray a = __RMICallback(data).toByteArray();

  Account account;

  QDataStream str(a, IO_ReadOnly);

  str >> account;

  return account;
}

  void
ClientRMI::toggleAccountActive(QString accountName)
{
  RMICallData data;
  data.method="toggleAccountActive(QString)";
  data.pcount=1;
  data.param.append(Any(accountName));

  __RMICall(data);
}

  void
ClientRMI::saveAccount(QString accountName, const Account& account)
{
  QByteArray a;
  QDataStream str(a, IO_WriteOnly);
  str << account;

  RMICallData data;
  data.method="saveAccount(QString, QByteArray)";
  data.pcount=2;
  data.param.append(Any(accountName));
  data.param.append(Any(a));
  
  __RMICall(data);
}

  void
ClientRMI::saveAccounts()
{
  RMICallData data;
  data.method="saveAccounts()";
  data.pcount=0;

  __RMICall(data);
}

bool ClientRMI::importMessages(QString toFolder)
{
  RMICallData data;
  data.method="importMessages(QString)";
  data.pcount=1;
  data.param.append( Any(toFolder) );
  
  return __RMICallback(data).toInt();
}


/*
static const char* homeuiservice_metainfo[][2]=
{
    { "/name", "Client RMI Component" },
    { "/class", "ClientRMIComponent" },
    { "/class/fqn", "ClientRMIComponent" },
    { "/description", AETHERA_RMI_CLIENT_DESC },
    { "/factory", "clientrmicomponent_factory" },
    { "/service", AETHERA_RMI_CLIENT },
    { 0, 0 }
};
*/

kore::Interface* clientrmicomponent_factory()
{
  return new ClientRMIComponent();
}
    
ClientRMIModule::ClientRMIModule()
{
}

ClientRMIModule::~ClientRMIModule()
{
}

ClientRMIComponent::ClientRMIComponent()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Client RMI Component");
  info->replaceAll("class", "ClientRMIComponent");
  info->replaceAll("class/fqn", "ClientRMIComponent");
  info->replaceAll("description", AETHERA_RMI_CLIENT_DESC );
  info->replaceAll("factory", "clientrmicomponent_factory");
  info->insert("service", AETHERA_RMI_CLIENT );
  
  ClientRMI::thisInstance = (ClientRMI*)this;
  
  registerRMIModule();
}

void ClientRMIComponent::registerRMIModule()
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    trader->registerComponent( *this );
  }
}


