/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */
#ifndef __ClientRMI__
#define __ClientRMI__

#include <komaetheraplugin.h>

#include <clientrmistub.h>
#include <qstring.h>
#include <accounts.h>


#define AETHERA_RMI_CLIENT             "/Aethera/RMI/Client"
#define AETHERA_RMI_CLIENT_DESC        "The RMI component from the client side."

class QStringList;
class CommFrame;
class RMICallData;
class Any;

using namespace kore;

extern "C"
{
    kore::Interface* clientrmicomponent_factory();
};

/**
  * Remote method invocation.
  * Provides proxies for methods on server objects, through a facade.
  * Client code may use this stuff to talk to the server objects.
  * Data is (de)marshalled and in future should go through a wire,
  * but currently is just demarshalled again in-process.
  */
class ClientRMI : public ClientRMIStub
{
  friend class CommBinder;
  public:
    static ClientRMI *ref();
    virtual ~ClientRMI();

    // server invocations

    // Proxies for AccountManager.
    virtual QStringList getAccountNames(int accountType=0/*Account::UNDEFINE*/);
    virtual QString getDefaultAccount();
    virtual bool isAccountActive(QString);
    virtual QString getAccountFromAddress(QString);
    virtual QString getAccountReplyToAddress(QString);
    virtual void loadAccounts();
    virtual bool addAccount(const Account&);
    virtual bool deleteAccount(QString);
    virtual void setDefaultAccount(QString);
    virtual Account getAccount(QString);
    virtual void toggleAccountActive(QString);
    virtual void saveAccount(QString, const Account&);
    virtual void saveAccounts();
    
    virtual void startSendReceive(QString account=QString::null);
    virtual void startReceive(QString account=QString::null);
    virtual void startSend(QString account=QString::null);
    virtual void stopPOP3Delivery();

    virtual void setAutoExpunge(bool);

    // methods used by IMAP folders
    /**
      * Mount and unmount some special folders.
      * This feature is used only for the remote folders.
      * IMAP folders needs a mount action for starting the connection
      * and an unmount action for closing the connection.
      * In this way the connection will be open only when you are seeing
      * the IMAP folder.
      * A mount acction will read the folder content too.
      */
    virtual bool mount(QString url);
    /** Unmount a remote folder, see mount.*/
    virtual bool unmount(QString url);
    /** Create all the subfolders.*/
    virtual bool subfolders(QString accountName, QString mailboxName, QString type = QString("genmail"));
    /** Rename.*/
    virtual bool rename(const QString& url, const QString& newName);
    /** Expunge.*/
    virtual bool expunge(const QString& url);
    /**Import messages to folder*/
    virtual bool importMessages(QString toFolder);
    
    // Kolab stuff
    virtual bool runKolabJob(const QString &, const QString&, const QString&);
    virtual bool runKolabSharedJob(const QString &, const QString&, const QString&);

  protected:
    static ClientRMI *thisInstance;
  private:
    virtual void __RMICall(const RMICallData &);
    virtual Any __RMICallback(const RMICallData &);

    virtual void __process(const CommFrame &);
    virtual void __process(const CommFrame &, CommFrame &);
    

};

class ClientRMIModule : public ClientRMI, public KomAetheraService
{
  public:
    ClientRMIModule();
    virtual ~ClientRMIModule();
    /** Register the module to kore.*/
    virtual void registerRMIModule() = 0;
};

class ClientRMIComponent : public ClientRMIModule
{
  public:
    ClientRMIComponent();
    virtual ~ClientRMIComponent() {}
    /** Register the module to kore.*/
    virtual void registerRMIModule();
};

#endif

