/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 /** Changed by Eugen C. <eug@thekompany.com>
   * Date: 28 Aug 2001
   */

#include "clientvfs.h"

#include <vobject.h>
#include <vfolder.h>
#include <vobjectfactory.h>
#include <brokeriface.h>
#include <vcacheobject.h>
#include <komaetheratools.h>
#include <vfsmimetype.h>
#include <settings.h>

#include <qmessagebox.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdict.h>

ClientVFS * ClientVFS::inst = 0;

ClientVFS::ClientVFS()
  : factoryDict(23),
    folders(499)
{
  useFact=false;
  autogenTemp=false;
  genVInstances=false;
  genNonRegistered=false;
  currentPath=root();
  
  qDebug("ClientVFS activated.");
}

ClientVFS::~ClientVFS()
{
  factoryDict.setAutoDelete(true);
  factoryDict.clear();
  folders.setAutoDelete(true);
  folders.clear();
  
  qDebug("ClientVFS deactivated.");
}

ClientVFS *ClientVFS::thisInstance()
{
  return inst ? inst : inst = new ClientVFS();
}

bool ClientVFS::initHierarchy()
{
  // read the folder hierarchy from the configuration object

  // register root
  _new(root(), "root", "root", "vfs", QString::null, QString::null);
      
  QStringList topLevel =
    Settings::instance()->readListEntry(root()+"/TopLevel", ',');

  if(topLevel.isEmpty())
  {
    qWarning
      (
        "ClientVFS: no hierarchy could be retrieved from the config"
        " object, bailing out"
      );

    return false;
  }

  QStringList::ConstIterator it;
  for(it = topLevel.begin(); it != topLevel.end(); ++it)
  {
    QString folderName=(*it);
    if(!makeBranch(folderName, root()))
    {
      qWarning
        (
          "ClientVFS: makeBranch() returned false when true was expected,"
          " bailing out"
        );

      return false;
    }
  }
  return true;
}

bool ClientVFS::makeBranch(const QString & _folderName, const QString & _path)
{
  int slashPos = _folderName.findRev('/');

  QString folderName = _folderName.mid(slashPos + 1);

  QString group = _path + "/" + folderName + "/";
  QString path  = _path + "/" + folderName;

  // read the folder type
  QString type = Settings::instance()->readEntry(group + "Type");

  if (type.isEmpty())
    type = QString::null;

  // view type associated with folder
  QString viewType = Settings::instance()->readEntry(group + "View type");

  if (viewType.isEmpty())
    viewType = type;

  // MIME type associated with folder
  QString mimeType = Settings::instance()->readEntry(group + "MIME type");

  // read the folder binding
  QString binding = Settings::instance()->readEntry(group + "Binding");

  // read the server path
  QString sPath = Settings::instance()->readEntry(group + "Server Path");

  // read the folder flags
  QString flags = Settings::instance()->readEntry(group + "Flags");

  if (flags.isEmpty())
    flags = "c";

  // debug message
  qDebug("Creating new VFolder: %s...", (type + ":" + path).ascii());

  VFolder * tFolder = _new(path, type, viewType, mimeType, binding, sPath);

  if (0 == tFolder)
  {
    qWarning
      (
        "folder %s could not be created, reverting to null binding",
        path.ascii()
      );

    tFolder = _new
      (path, type, viewType, mimeType, QString::null, QString::null);
  }

  if (0 != tFolder)
    tFolder->setFlags(flags);

  // create VObject instances, if necessary

  if (genVInstances)
  {
    qDebug("Create VObjects");
    
    if (!createVInstances(tFolder))
    {
      qWarning
        (
          "ClientVFS: createVInstances(VFolder *) returned false"
          " when true was expected, bailing out"
        );

      return false;
    }
  }

  // read the subfolder list
  QStringList subFolders = Settings::instance()->readListEntry(group + "Subfolders", ',');

  // process every subfolder

  QStringList::ConstIterator it;

  for (it = subFolders.begin(); it != subFolders.end(); ++it)
  {
    if (!makeBranch((*it), path))
    {
      qWarning
        (
          "ClientVFS: makeBranch() returned false when true was"
          " expected, bailing out"
        );

      return false;
    }
  }

  return true;
}

  VFolder *
ClientVFS::_new
(
  const QString & path,
  const QString & type,
  const QString & viewType,
  const QString & mimeType,
  const QString & binding,
  const QString & spath
)
{
#if 0
  qDebug(
    "VFolder::_new(%s, %s, %s, %s)",
    path.ascii(),
    type.ascii(),
    binding.ascii(),
    spath.ascii()
  );
#endif

  VFolder * test = folderRef(path);
  if( test )
  {
    qWarning("VFolder *_new() was called on an already registered path \n%s || %s || %s",
      path.latin1(), test->path().latin1(), test->serverPath().latin1());

    return 0;
  }
  else
  {
    int slashPos = path.findRev('/');
    VFolder * parent = folderRef( path.left(slashPos) );
    if( !parent )
      parent = folderRef( root() );

    VFolder * f = new VFolder(path);

    f->setType(type);
    f->setViewType(viewType);
    f->setMimeType(mimeType);
    f->setBinding(binding);
    f->setServerPath(spath);
    // add it to map
    if( !spath.isEmpty() )
      s2cMap[spath]=path;

    folders.insert(path, f);
    if( parent )
    {
      // add the new folder to the parent
      parent->addFolder(f);
      // set the parent folder for the new folder
      f->setParentFolder(parent);
    }

    return f;
  }
}

VObject * ClientVFS::_fact_new(const QString& path, const QString& _type)
{
  QString type = _type;
  
  if( type.isEmpty() )
  {
    QString folderPath = path.left( path.findRev('/') );
    VFolder * folder = folderRef(folderPath);
  
    if(folder)
    {
      type = folder->type();
    }
    else
      return 0L;
  }
  
  VObjectFactory * fact = factoryDict[type];
 
  if( !path.isEmpty() && fact )
    return fact->_new(path);

  qWarning("ClientVFS::_fact_new could not create object %s", path.ascii());
  qWarning("Reason(s):");

  if (!fact)
    qWarning("  -- no factory was associated with type \"%s\"", type.ascii());

  return 0L;
}

VObject * ClientVFS::_fact_new(const QString& path, const QByteArray &_data, const QString& _type)
{
  QString type = _type;
  
  if( type.isEmpty() )
  {
    QString folderPath = path.left( path.findRev('/') );
    VFolder * folder = folderRef(folderPath);
  
    if(folder)
    {
      type = folder->type();
    }
    else
      return 0L;
  }
  
  VObjectFactory * fact = factoryDict[type];
 
  if( !path.isEmpty() && fact )
    return fact->_new(path, _data);

  qWarning("ClientVFS::_fact_new could not create object %s", path.ascii());
  qWarning("Reason(s):");

  if (!fact)
    qWarning("  -- no factory was associated with type \"%s\"", type.ascii());

  return 0L;
}

VObject * ClientVFS::_fact_ref(const QString& path, const QString& _type)
{
  if (!path.isEmpty())
  {
    VObject * r = objectRef(path);

    if (0 != r)
      return r;
  }

  return _fact_new(path, _type);
}

VObject * ClientVFS::_fact_ref(const QString& path, const QByteArray &_data, const QString& _type)
{
  if (!path.isEmpty())
  {
    VObject * r = objectRef(path);

    if (0 != r)
    {
      r->_narrow(_data);
      return r;
    }
  }

  return _fact_new(path, _type);
}

bool ClientVFS::createVInstances(VFolder * folder)
{
  QByteArray childrenInfo, childInfo;
  QStringList childrenList;

  VObjectFactory * fact = factoryDict[folder->type()];

  if( !BrokerInterface::thisInstance()->getChildrenData(folder->path(), childrenInfo) ||
      !childrenInfo.size() )
  {
    qDebug("Nothing to create!");
    return true;
  }

  // Read the list of children.
  QDataStream infoStream(childrenInfo, IO_ReadOnly);

  // Read the number of objects.

  Q_UINT32 uObjectsNo = 0;
  infoStream >> uObjectsNo;

  if (!uObjectsNo)
  {
    qDebug("Nothing to create!");
    return true;
  }

//  qDebug("The number of children to create %u !", uObjectsNo);

  // Read the objects list.
  infoStream >> childrenList;

  // Get the client urls.
  childrenList = reverseMap(childrenList);

  // Build all the children.

  bool bRet = true;

  for (
    QStringList::Iterator child = childrenList.begin();
    bRet && child != childrenList.end();
    ++child
  )
  {
    infoStream >> childInfo;

    // Build the new object using the childInfo.

    if( !useFact ||
        !fact ||
        !fact->_new((*child), childInfo) )
      bRet = false;
//    else if(genNonRegistered)
//      bRet=_new<VObject>((*child), childInfo);
  }

  childrenInfo.truncate(0);
  childInfo.truncate(0);
  childrenList.clear();
  
  return bRet;
}

bool ClientVFS::createVInstances(const QStringList & paths, bool bServerUrl)
{
  QByteArray objectsInfo, objectInfo;
  QStringList objectsList;
  bool bRet=true;

  if( bServerUrl )
    bRet = BrokerInterface::thisInstance()->getServerObjectsData(paths, objectsInfo);
  else
    bRet = BrokerInterface::thisInstance()->getObjectData(paths, objectsInfo);

  if( !bRet || !objectsInfo.size())
  {
    qDebug("ClientVFS::createVInstances : empty buffer");
    return false;
  }

  // Read the list of children.
  QDataStream infoStream(objectsInfo, IO_ReadOnly);

  // Read the number of objects.

  Q_UINT32 uObjectsNo = 0;

  infoStream >> uObjectsNo;

  if (!uObjectsNo)
  {
    qDebug("Nothing to create!");
    return true;
  }

//  qDebug("The number of children to create %u !", uObjectsNo);
  
  // Read the objects list
  infoStream >> objectsList;

  // get the client urls
  objectsList = reverseMap(objectsList);

  // build all the children

  QString lastFolder;
  QString folderPath;
  QString obj;
  VFolder * folder = 0L;
  VObjectFactory * fact = 0L;
  VObject * object = 0L;
  
  for (
    QStringList::ConstIterator objectIt = objectsList.begin();
    bRet && objectIt != objectsList.end();
    ++objectIt
  )
  {
    obj = (*objectIt).mid( (*objectIt).findRev('/') + 1 );
    if (!obj.contains(ATTR_SEP))
    {
      folderPath = (*objectIt).left((*objectIt).findRev('/'));
      if( lastFolder != folderPath )
      {
        folder = folderRef(folderPath);

        // Get the factory.
        if( folder )
          fact = factoryDict[folder->type()];
        else
          fact = 0L;
      }
      lastFolder = folderPath;

      if (0 != folder)
      {
        infoStream >> objectInfo;

        // build the new object using the childInfo
        if( useFact && fact )
        {
          object = objectRef(*objectIt);
          if( object )
            object->_narrow(objectInfo);
          else
            object = fact->_new((*objectIt), objectInfo);
          if( !object )
            bRet = false;
        }
        else
          bRet = false;
      }
    }
  }
  
  objectsInfo.truncate(0);
  objectInfo.truncate(0);
  objectsList.clear();
  
  return bRet;
}

bool ClientVFS::createVInstances(const QString& type)
{
  // Iterate over folder collection.

  QDictIterator<VFolder> it(folders);

  VFolder * tFolder;

  while((tFolder=it.current()))
  {
    if( type.isEmpty() || type == tFolder->type() )
    {
      if (!createVInstances(tFolder))
      {
        qWarning(
          "ClientVFS: createVInstances(VFolder *) returned false when true"
          " was expected, bailing out"
        );
  
        return false;
      }
    }
    ++it;
  }
  return true;
}

bool ClientVFS::rehashHierarchy()
{
  folders.clear();
  s2cMap.clear();
  currentPath=root();
  pStack.clear();
  return initHierarchy();
}

bool ClientVFS::rehashVInstances()
{
  return createVInstances();
}

QDict<VFolder> ClientVFS::folderDict()
{
  return folders;
}

QString ClientVFS::parentFolder(const QString & path)
{
  int depth = path.contains('/');
  
  // Test whether the root was given here, and if so, we keep it.

  if (depth < 3)
    return root();

  return path.left(path.findRev('/'));
}

VFolder * ClientVFS::parentFolder(VFolder * folder)
{
  VFolder * parent=folder->parentFolder();
  if( parent )
    return parent;
  else
    return folderRef( root() );
}

QStringList ClientVFS::folderList(const QString & folder)
{
  QStringList fList;
  VFolder * topFolder = folderRef( folder );
  if( topFolder )
  {
    /** depth = 1 */
    topFolder->tree(fList, 1);
    fList.sort();
  }
  
  return fList;
}

void ClientVFS::autogenTemporary(bool temp)
{
  autogenTemp = temp;
}

void ClientVFS::useFactories(bool fact)
{
  useFact = fact;
}

void ClientVFS::autogenVInstances(bool vinst, bool nonreg)
{
  genVInstances     = vinst;
  genNonRegistered  = nonreg;
}

VFolder * ClientVFS::folderRef(const QString & folder)
{
  if( folder.isEmpty() )
    return 0L;
  
  VFolder * ref = folders[folder];

  if (0 != ref)
    return ref;

  // TODO Fix this part using root()

  if (autogenTemp)
  {
    if (folder.contains('/') == 2 && folder.left(8) == "/Aethera")
    {
      // A valid non-existent folder like "/Aethera/abcd" was given.
      return _new(folder, "temporary", "", "");
    }
    else if (!folder.contains('/') && !folders["/Aethera/" + folder])
    {
      // A folder like "abcd" was given, and "/Aethera/abcd" does not exist.
      return _new("/Aethera/" + folder, "temporary", "", "");
    }
    else
    {
      // Only "temporary" was given, or some wrong name.
      return _new(genTempFolderName(), "temporary", "", "");
    }
  }
  
  return 0;
}

VFolder * ClientVFS::folderRef(const QString & parent, const QString & name)
{
  return folderRef(parent + "/" + name);
}

VFolder * ClientVFS::folderRef(VFolder * parent, const QString & name)
{
  if (0 == parent)
    return folderRef(name);

  return folderRef(parent->path() + "/" + name);
}

VObject * ClientVFS::objectRef(const QString & object)
{
  // If the object is null.
  if (object.isEmpty())
    return 0;

  int iFindRev = object.findRev('/');
  return objectRef(folderRef(object.left(iFindRev)), object.mid(iFindRev + 1));
}

VObject * ClientVFS::objectRef(VFolder * parent, const QString & id)
{
  if (0 == parent || id.isEmpty())
    return 0;

  return parent->objectDict()[id];
}

VObject * ClientVFS::objectRef(const QString & folder, const QString & id)
{
  if (folder.isEmpty() || id.isEmpty())
    return 0;

  return objectRef(folderRef(folder), id);
}

VReference * ClientVFS::reference(const QString & url)
{
  if (url.isEmpty())
    return 0;

  VReference * ref = objectRef(url);

  if (0 != ref)
    return ref;

  return folderRef(url);
}

void ClientVFS::registerFactory(const QString & type, VObjectFactory * factory)
{
  if (type == "temporary" || !type || 0 == factory)
    return;

  factoryDict.insert(type, factory);
}

bool ClientVFS::cd(const QString & path)
{
  // Test to see if this path is the root, which is not included in the folder
  // dict.

  if (path == root())
  {
    currentPath = path;
    return true;
  }
  
  // Test to see if this is an absolute path.

  if (folders[path])
  {
    currentPath = path;
    return true;
  }

  // Test to see if this is a relative path.

  QString tgtPath = currentPath + "/" + path;

  if (folders[tgtPath])
  {
    currentPath = tgtPath;
    return true;
  }
  
  // Path is not absolute nor relative.
  return false;
}

QStringList ClientVFS::dir(const QString & params)
{
  QStringList dirList(folderList(currentPath));

  if (params.contains("-l"))
    return dirList;

  // Short dir requested, therefore strip all the paths.

  QStringList sDirList;

  for (
    QStringList::ConstIterator it = dirList.begin();
    it != dirList.end();
    ++it
  )
  {
    sDirList.append((*it).mid((*it).findRev('/') + 1));
  }

  sDirList.sort();

  return sDirList;
}

void ClientVFS::cdUp()
{
  cd(parentFolder(currentPath));
}

void ClientVFS::cdRoot()
{
  cd(root());
}

QString ClientVFS::pwd() const
{
  return currentPath;
}

QString ClientVFS::root() const
{
  return "/Aethera/VFS";
}

void ClientVFS::pushPath()
{
  pStack.append(currentPath);
}

void ClientVFS::popPath()
{
  if (pStack.isEmpty())
    return;

  currentPath = *pStack.last();

  pStack.remove(pStack.fromLast());
}

QString ClientVFS::genTempFolderName()
{
  QStringList fList = folderList(root());

  int k = 0;

  QString tName;

  tName.sprintf("/Aethera/VFS/Temp%d", k++);

  while (fList.find(tName) != fList.end())
    tName.sprintf("/Aethera/VFS/Temp%d", k++);

  return tName;
}

void ClientVFS::saveStructure()
{
  QStringList fList=folderList(root());
  QStringList sList;
  QString type;
  VFolder *vFolder;
  QString subFolder;
  
  for(QStringList::Iterator it=fList.begin(); it!=fList.end(); ++it)
  {
    vFolder=folderRef( (*it) );
    type=vFolder->type();
    if( type != QString::null && 
        type != "temporary")
    {
      subFolder = vFolder->path();
      // adding to TopLevel
      sList.append( subFolder.mid(subFolder.findRev('/')+1) );
      // saving the subfolder
      saveFolderBranch( vFolder );
    }
  }
  
  Settings::instance()->writeEntry(root()+"/TopLevel", sList, ',');
  ClientNotifier::thisInstance()->callSaveSettings();
//  Settings::forceWrite();
}

void ClientVFS::saveFolderBranch(VFolder *folder)
{
  saveFolder(folder);

  QStringList fList=folderList(folder->path());
  QString type;
  VFolder *vFolder;
  
  for(QStringList::Iterator it=fList.begin(); it!=fList.end(); ++it)
  {
    vFolder=folderRef( (*it) );
    type=vFolder->type();
    if( type != QString::null && 
        type != "temporary")
    {
      // saving the subfolder
      saveFolderBranch( vFolder );
    }
  }
}

void ClientVFS::saveFolder(VFolder *folder)
{
  QString group=folder->path()+"/";
  
//  qDebug("saveFolder = %s", group.latin1());
  
  Settings::instance()->writeEntry(group + "Type", folder->type());
  Settings::instance()->writeEntry(group + "View type", folder->viewType());
  Settings::instance()->writeEntry(group + "MIME type", folder->mimeType());
  Settings::instance()->writeEntry(group + "Binding", folder->binding());
  Settings::instance()->writeEntry(group + "Server Path", folder->serverPath());
  Settings::instance()->writeEntry(group + "Flags", folder->flags());

  QStringList sList;
  folder->tree(sList, 1);

  Settings::instance()->writeEntry(group + "Subfolders", sList, ',');
}

QString ClientVFS::reverseMap(const QString & path)
{
  if(path.isEmpty())
    return QString::null;

  // check to see if this is a folder
  QStringMapIterator it = s2cMap.find(path);
  
  if( it != s2cMap.end() )
    return it.data();
  else
  {
    // check to see if it's a simple object
    if( -1 == path.find("/") )
      return path;

    // maybe it's an object
    // ids are the same for the client and server
    QString id = path.mid(path.findRev("/")+1);
    QString sPath = path.left(path.findRev("/"));

    it = s2cMap.find(sPath);
    if( it != s2cMap.end() )
      return it.data()+"/"+id;
  }
  
  // a reverse mapping was not found
  return QString::null;
}

QStringList ClientVFS::reverseMap(const QStringList & list)
{
  QStringList ret;

  for (QStringList::ConstIterator it(list.begin()); it != list.end(); ++it)
    ret << reverseMap(*it);

  return ret;
}

QString ClientVFS::map(const QString & url)
{
  if( !url || url.isEmpty())
    return QString::null;

  VFolder *folder = folderRef(url);
  if( folder )
    return folder->serverPath();
  else
  {
    QString objectId = url.mid(url.findRev("/")+1);
    QString folderName = url.left(url.findRev("/"));
    
    // try to get the reference
    folder = folderRef( folderName );
    if( folder )
    {
      if( !folder->binding().isNull() )
        return folder->serverPath()+"/"+objectId;
    }
  }

  // no server mapping found
  return QString::null;
}

QStringList ClientVFS::map(const QStringList & urlList)
{
  QStringList ret;

  for (
    QStringList::ConstIterator it(urlList.begin());
    it != urlList.end();
    ++it
  )
    ret << map(*it);

  return ret;
}


bool ClientVFS::canDefine(const QString & url)
{
  if(BrokerInterface::thisInstance()->exists(url))
    return true;
    
//  VFolder *f=folderRef(url.left(url.findRev('/')));
//  if(f && f->type()=="temporary")
//    return true;
  
  return false;
}

void ClientVFS::updateVInstances(const QStringList & paths)
{
  if( 1==paths.size() )
  {
    updateVInstance( paths[0] );
  }
  else if( paths.size() )
  {
    QByteArray objectsInfo, objectInfo;
    QStringList objectsPaths;
    
    if(  !BrokerInterface::thisInstance()->getObjectData(paths, objectsInfo) ||
        !objectsInfo.size() )
      return;

    // read the list of childs
    QDataStream infoStream(objectsInfo, IO_ReadOnly);
    // Read the number of objects
    unsigned uObjectsNo=0;
    infoStream >> uObjectsNo;
    if( !uObjectsNo )
    {
      printf("\nNothing to update!\n");
      fflush(stdout);
      return;
    }
    
//    printf("\nThe number of children to update %u !\n",uObjectsNo);
//    fflush(stdout);
    
    // Read the objects list
    infoStream >> objectsPaths;

    // get the client urls
    objectsPaths=reverseMap(objectsPaths);
    
    // Update all the children
    for(QStringList::Iterator it=objectsPaths.begin(); it!=objectsPaths.end(); ++it)
    {
      infoStream >> objectInfo;
      VObject *obj = objectRef(*it);
      if(obj)
        obj->_narrow( objectInfo );
//        else
//          qWarning("Error at updateVInstances(QStringList paths): Object %s wasn't found", (*it).latin1());
    }
  }
}

void ClientVFS::updateVInstance(const QString & path)
{
  // printf("Notify: object %s was changed on server, updating\n", (const char *)path);

  QByteArray objectsInfo, objectInfo;
  QString vPath=reverseMap(path);
  VFolder *folder = folderRef(vPath);
  if( folder )
  {
    QStringList childrenPaths;
    
    if( !BrokerInterface::thisInstance()->getChildrenData(vPath, objectsInfo) ||
       !objectsInfo.size() )
      return;

    // read the list of childs
    QDataStream infoStream(objectsInfo, IO_ReadOnly);
    // Read the number of objects
    unsigned uObjectsNo=0;
    infoStream >> uObjectsNo;
    if( !uObjectsNo )
    {
      printf("\nNothing to update!\n");
      fflush(stdout);
      return;
    }
    
//    printf("\nThe number of children to update %u !\n",uObjectsNo);
//    fflush(stdout);
    
    // Read the objects list
    infoStream >> childrenPaths;

    // get the client urls
    childrenPaths=reverseMap(childrenPaths);
    
    // Update all the children
    for(QStringList::Iterator it=childrenPaths.begin(); it!=childrenPaths.end(); ++it)
    {
      infoStream >> objectInfo;
      VObject *obj = objectRef(*it);
      if(obj)
        obj->_narrow( objectInfo );
//      else
//        qWarning("Error at updateVInstance(QString path): Object %s wasn't found", (*it).latin1());
    }
  }
  else
  {
    VObject *obj = objectRef(vPath);
    if(obj)
      obj->sync();
//    else
//      qWarning("Error at updateVInstance(QString path): Object %s wasn't found", vPath.latin1());
  }
}

void ClientVFS::deleteVInstance(const QString & path, bool bServerUrl)
{
  // debug
  // printf("Notify: object %s was deleted on server, deleting\n", (const char *)path);

  QString vfsPath;
  
  if( bServerUrl )
    vfsPath = reverseMap(path);
  else
    vfsPath = path;
    
  VObject *obj = objectRef(vfsPath);
  if(obj)
  {
    // debug
    // printf("Notify: object %s is about to be deleted\n", (const char *)localPath);
    
    delete obj;
  }
}

void ClientVFS::deleteVInstances(const QStringList & paths, bool bServerUrl)
{
  QStringList vfsPaths;
  
  if( bServerUrl )
    vfsPaths = reverseMap(paths);
  else
    vfsPaths = paths;
  
  for(QStringList::Iterator it = vfsPaths.begin(); it != vfsPaths.end(); ++it)
  {
    VObject *obj = objectRef(*it);
    if(obj)
    {
      // debug
//      qDebug("Notify: VObject %s is deleted\n", (*it).latin1());
      
      delete obj;
    }
  }
}

void ClientVFS::createVInstance(const QString & path, bool)
{
  if( -1 == path.find(ATTR_SEP) )
  {
    QString localPath=reverseMap(path);

    // debug
    // printf("Notify: object %s is about to be created\n", (const char *)localPath);

    QString folderPath=localPath.left(localPath.findRev("/"));
    VFolder *folder=folderRef(folderPath);

    if(folder)
    {
      if(useFact)
      {
        _fact_ref(localPath, folder->type());
      }
      else
      {
        printf("Warning: cannot use factories, local instance was not created\n");
      }
    }
    else
    {
      printf("Warning: cannot find parent folder (hierarchy failure?), local instance not created\n");
    }
  }
  else
  {
    printf("Warning: cannot create attributes or subobjects automatically, discarding\n");
  }
}

void ClientVFS::initFolderCache(const QString &type)
{
  QDictIterator<VFolder> it(folders);
  VFolder *folder;
  while((folder=it.current()))
  {
    if(folder->type()==type)
    {
      if ( !objectRef(folder->path()+"/"+"dir"+ATTR_SEP+"cache") )
        new VCacheObject(folder->path()+"/"+"dir"+ATTR_SEP+"cache");
    }
    
    ++it;
  }
}

void ClientVFS::discardFolderCache(const QString &type)
{
  QDictIterator<VFolder> it(folders);
  VFolder *folder;
  while((folder=it.current()))
  {
    if(folder->type()==type)
    {
      VCacheObject *cacheObj=(VCacheObject *)objectRef(folder, "dir^cache");
      if(cacheObj) delete cacheObj;
    }
    ++it;
  }
}

// create only a virtual folder
void ClientVFS::createFolder(const QString &parent, const QString &name, const QString &type, const QString &viewType, const QString &serverPath)
{
  QString parentFolder;
  // Account -> /Aethera/VFS/Mail/Account
  if( type=="imap" )
    parentFolder=root()+"/Mail/"+parent;
  else
    parentFolder=parent;

  VFolder* vParent=folderRef(parentFolder);
  if( !vParent )
    // debug
    printf("clientvfs: warning: could not create the VFS %s folder\n", name.latin1());
  else
  {
    QString type=vParent->type();
    QString folderPath=parentFolder+"/"+name;

    if(!serverPath.isEmpty() || viewType=="genmail")
    {
      // building the view type
      QString vType;
      QString mimeType;
      
      if( viewType.isNull() || viewType.isEmpty() )
        vType=type;
      else
        vType=viewType;
      
      mimeType = VFSMimeType::vfs2mime(vType);
      
      // Create the folder
      VFolder *newFolder=_new(folderPath, type, vType, mimeType, serverPath.isEmpty()?"":"thisInstance", serverPath);
      if( newFolder )
      {
        newFolder->setFlags("c");
//        saveStructure();
        saveFolderBranch(newFolder->parentFolder());
        ClientNotifier::thisInstance()->callSaveSettings();
      }
      else
        // debug
        printf("clientvfs: warning: could not create folder [spath:%s, type:%s]\n", (const char *)serverPath, (const char *)type);
    }
    else
      // debug
      printf("clientvfs: warning: could not create folder [spath:%s, type:%s], device expected\n", (const char *)serverPath, (const char *)type);
  }
}

VFolder *ClientVFS::createFolder(VFolder *parent, const QString &name, const QString &type, const QString &viewType)
{
  // a list of parameter for creating a new folder on the server
  QStringList paramList;
  QString folderPath = parent ? parent->path()+"/"+name : root()+"/"+name;
  QString serverPath;
  
  // type and viewType for knowing the server folder type
  paramList.append(type);
  paramList.append(viewType);
  if( type=="imap" )
  {
    int iFind1=0, iFind2=0;
    QString mailRoot = root()+"/Mail/";
    
    //  folderPath=/Aethera/VFS/Mail/Account/Mailbox
    if( -1 == (iFind1=folderPath.find( mailRoot )) )
      return 0;
    iFind1+=mailRoot.length();
    iFind2=folderPath.find('/', iFind1);
    if( iFind2==-1 )
      return 0;

    paramList.append(folderPath.mid( iFind1, iFind2 - iFind1 ));
    paramList.append(folderPath.right( folderPath.length() - (iFind2+1) ));
  }
  
  // debug
  // printf("clientvfs: asking brokerinterface to create a [%s] device\n", (const char *)devType);
  
  if( type!="genmail" ) serverPath=BrokerInterface::thisInstance()->createFolder(paramList);
  else printf("clientvfs: warning: new folder [%s] has no associated device\n", (const char *)name);
  
  // For IMAP the design is different
  // The folder will be created after the IMAP server will be ready
  if( type!="imap" )
  {
    if(!serverPath.isEmpty() || type=="genmail")
    {
      // building the view type
      QString vType;
      QString mimeType;
      
      if( viewType.isNull() || viewType.isEmpty() )
        vType=type;
      else
        vType=viewType;
      
      if(vType=="mail") mimeType="text/rfc822";
      else if(vType=="genmail") mimeType="";
      else if(vType=="notes") mimeType="application/x-notes";
      else if(vType=="contacts") mimeType="text/directory";
      else if(vType=="groups") mimeType="application/x-groups";
      
      // Create the folder
      VFolder * newFolder = _new(folderPath, type, vType, mimeType, serverPath.isEmpty()?"":"thisInstance", serverPath);
      if( newFolder )
      {
        newFolder->setFlags("c");
//        saveStructure();
        saveFolderBranch(newFolder->parentFolder());
        ClientNotifier::thisInstance()->callSaveSettings();
        
        // ask Eug about the signal
        ClientNotifier::thisInstance()->emitCreateVFolder
        (
          newFolder->parentFolder()->path(),
          newFolder->name(),
          newFolder->type(),
          newFolder->viewType(),
          newFolder->serverPath()
        );

        return newFolder;
      }
      else
      {
        // debug
        printf("clientvfs: warning: could not create folder [spath:%s, type:%s]\n", (const char *)serverPath, (const char *)type);
      }
    }
    else
    {
      // debug
      printf("clientvfs: warning: could not create folder [spath:%s, type:%s], device expected\n", (const char *)serverPath, (const char *)type);
    }
  }
  
  return 0;
}

void ClientVFS::deleteFolder(const QString &parent, const QString &folderName, const QString &type)
{
  QString folderPath;
  // Account -> /Aethera/VFS/Mail/Account
  if( type=="imap" )
  {
    if( folderName.isEmpty() )
      folderPath=root()+"/Mail/"+parent;
    else
      folderPath=root()+"/Mail/"+parent+"/"+folderName;
  }
  else
    folderPath=parent+"/"+folderName;

  deleteVFolder(folderPath);
}

bool ClientVFS::deleteFolder(const QString &name)
{
  QStringList subFolders=folderList(name);
  
  for(uint i=0;i<subFolders.count();i++)
    if(!deleteFolder(subFolders[i]))
      return false;
  
  VFolder *folder=folderRef(name);
  if( folder )
  {
    QString folderPath=folder->path();
    QStringList paramList;
    QString serverPath=folder->serverPath();
    
    // type and viewType for knowing the server folder type
    paramList.append(folder->type());
    paramList.append(folder->viewType());
    if( serverPath.isEmpty() )
      paramList.append("default");
    else
      paramList.append(serverPath);
    
    if( folder->type()=="imap" )
    {
      int iFind1=0, iFind2=0;
      QString mailRoot = root()+"/Mail/";
      
      //  folderPath=/Aethera/VFS/Mail/Account/Mailbox
      if( -1 == (iFind1=folderPath.find( mailRoot )) )
        return 0;
        
      iFind1+=mailRoot.length();
      iFind2=folderPath.find('/', iFind1);
      if( iFind2 != -1 )
      {
        paramList.append(folderPath.mid( iFind1, iFind2 - iFind1 ));
        paramList.append(folderPath.right( folderPath.length() - (iFind2+1) ));
      }
      else
      {
        paramList.append(folderPath.mid(iFind1));
        paramList.append(QString());
      }
      
      BrokerInterface::thisInstance()->removeFolder(paramList);
    }
    else
    {
      if( !folder->binding().isEmpty() )
        BrokerInterface::thisInstance()->removeFolder(paramList);
      deleteVFolder(folder);
    }

    return true;
  }
  else
    return false;
}

void ClientVFS::deleteVFolder(const QString &name)
{
  VFolder *folder = folderRef(name);

  deleteVFolder(folder);
}

void ClientVFS::deleteVFolder(VFolder *folder)
{
  if( folder )
  {
    VFolder * parent = folder->parentFolder();
    if( parent )
    {
      // remove it from the parent list
      parent->removeFolder(folder);
      // remove it from the global index
      folders.remove(folder->path());
      // remove it from the map
      s2cMap.remove(folder->serverPath());
      // remove it from settings
      Settings::instance()->removeGroup( folder->path() );
      // free the VFolder
      delete folder;
      folder = 0;
      // save the VFS
//      saveStructure();
      saveFolderBranch(parent);
      ClientNotifier::thisInstance()->callSaveSettings();
    }
  }
}

bool ClientVFS::moveFolder(const QString &path, const QString &newPath)
{
  VFolder * oldFolder = folderRef( path );
  VFolder * newParent = folderRef( parentFolder(newPath) );
  
  if( oldFolder && newParent )
  {
    // remove it from the parent list
    oldFolder->parentFolder()->removeFolder(oldFolder);
    // add it to the new place
    oldFolder->setParentFolder(newParent);
    newParent->addFolder(oldFolder);
    
    // remove it from general index
    folders.remove(path);
    // add it to general index
    oldFolder->setFolderPath(newPath);
    folders.insert(newPath, oldFolder);
    
    // remove it from the map
    s2cMap.remove(oldFolder->serverPath());
    // add it to map
    if( !oldFolder->serverPath().isEmpty() )
      s2cMap[oldFolder->serverPath()] = newPath;

    // get the moving tree
    QStringList folderList;
    oldFolder->tree(folderList, 1);

    QString oldFolderPath;
    QString newFolderPath;
    for(uint i=0;i<folderList.count();i++)
    {
      oldFolderPath = folderList[i];
      newFolderPath = newPath + "/" + oldFolderPath.mid( path.length() + 1 );
      moveFolder(oldFolderPath, newFolderPath);
    }
    
    // remove the old settings
    Settings::instance()->removeGroup( path );
    // save the new VFS tree in settings
    saveStructure();
    return true;
  }
  else
  {
    return false;
  }
}


