/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 /** Changed by Eugen C. <eug@thekompany.com>
   * Date: 28 Aug 2001
   */

#ifndef __ClientVFS__
#define __ClientVFS__

#include <vfsmodule.h>

typedef QMap<QString, QString> QStringMap;
typedef QStringMap::Iterator QStringMapIterator;

#include <defines.h>

class ClientVFS
{
  friend class ClientNotifier;

  public:

    virtual ~ClientVFS();

    static ClientVFS * thisInstance();

    bool initHierarchy();
    bool createVInstances(const QString & type=QString::null);
    bool rehashHierarchy();
    bool rehashVInstances();

    QDict<VFolder> folderDict();

    QString   parentFolder(const QString & url);
    VFolder * parentFolder(VFolder *);

    /**
     * @return a list of folder names.  This implementation guarantees the
     * existence of "/Aethera".
     */
    QStringList folderList(const QString & folder);

    /**
     * Methods controlling the behaviour of the ClientVFS object.
     */
    void autogenTemporary(bool temp);
    void useFactories(bool fact);
    void autogenVInstances(bool vinst, bool nonreg);

    /** References - all VReferences are always valid if they exist somewhere
     * in Aethera since they register automatically with ClientVFS.
     *
     * A "temporary://" (always local!) virtual folder will be automatically
     * generated if a reference is requested and the folder doesn't exist,
     * unless disabled by autogenTemporary().
     *
     * This is the only situation where it is allowed to use an url (otherwise
     * ClientVFS wouldn't know a temp folder was requested.) If only
     * "temporary" is passed as a parameter a unique name will be automatically
     * generated.
     *
     * Temporary folders are registered under the root folder (!), but they
     * have no associated binding nor server path.
     *
     * Temporary folders can have only VTemporary type entries, and they are
     * NOT handled by BrokerInterface (VTemporary objects are).
     * rikkus: Self-contradictory sentence ?
     */
    VFolder * folderRef(const QString & folder);
    VFolder * folderRef(const QString & parent, const QString & name);
    VFolder * folderRef(VFolder * parent, const QString & name);

    /**
     * Object references work here exclusively on folder/object relationships
     * don't use logical parents, subobjects, attributes, related lists etc. as
     * the VFS map doesn't include such run-time generated data use the
     * BrokerInteface instead, if needed.
     *
     * March 13,2000 - disregard the previous three lines; in order to optimize
     * the query processor I've added object registration for all VObjects
     */

    VObject * objectRef(const QString & object);
    VObject * objectRef(VFolder * parent, const QString & id);
    VObject * objectRef(const QString & folder, const QString & id);

    /**
     * General reference
     */
    VReference * reference(const QString & url);

    void initFolderCache    (const QString & type);
    void discardFolderCache (const QString & type);

    /**
     * Register VObject factories
     */
    void registerFactory(const QString & type, VObjectFactory * factory);

    /**
     * This makes a new folder.
     */
    VFolder * _new
      (
        const QString & path,
        const QString & type,
        const QString & viewType,
        const QString & mimeType,
        const QString & binding     = QString::null,
        const QString & serverPath  = QString::null
      );

    VObject * _fact_new(const QString& path, const QString& _type=QString::null);
    VObject * _fact_new(const QString& path, const QByteArray &_data, const QString& _type=QString::null);
    VObject * _fact_ref(const QString& path, const QString& _type=QString::null);
    VObject * _fact_ref(const QString& path, const QByteArray &_data, const QString& _type=QString::null);

    bool canDefine(const QString & url);

#ifndef _QTWIN_
    /**
     * template VObject factories use _new to create a new VObject; this
     * returns 0 if you pass an undefinable path (i.e. which is not defined on
     * the server and it is not temporary) or if an object with that address
     * already exists
     */
    template <class T>
      T * _new(const QString & url)
      {
        if ( !objectRef(url))
          return new T(url);

        return 0;
      }

    /**
     * template VObject factories use _new to create a new VObject; this
     * returns 0 if you pass an undefinable path (i.e. which is not defined on
     * the server and it is not temporary) or if an object with that address
     * already exists
     */
    template <class T>
      T * _new(const QString & url, const QByteArray & data)
      {
        return new T(url, data);
      }

    /**
     * This works like _new, but it will attempt to find and return an existing
     * instance first.
     */
    template <class T>
      T * _ref(const QString & url)
      {
        if (VObject * r = objectRef(url))
          return static_cast<T *>(r);

          return new T(url);

        return 0;
      }

    /**
     * This works like _new, but it will attempt to find and return an existing
     * instance first.
     */
    template <class T>
      T * _ref(const QString & url,  const QByteArray & data)
      {
        if (VObject * r = objectRef(url))
        {
          r->_narrow(data);
          return static_cast<T *>(r);
        }

          return new T(url, data);

        return 0;
      }
#endif

    /**
     * Generate temporary folder name.
     */
    QString genTempFolderName();

    /**
     * Navigation.
     */
    QString pwd() const;

    QStringList dir(const QString & params = QString::null);

    bool cd(const QString & path);
    void cdUp();
    void cdRoot();
    void pushPath();
    void popPath();

    QString root() const;

    /**
     * Use this with caution since it can cause major structure disruption;
     * normally, it shouldn't be called manually.
     */
    bool createVInstances(VFolder * folder);

    void saveStructure();
    void saveFolder(VFolder * folder);

    /**
     * Map a server address to a client address. @return QString::null if
     * such a mapping does not exist.
     */
    QString reverseMap(const QString & path);

    /**
     * Map server addresses to client addresses. QString::null is used where
     * mappings do not exist. XXX Check this.
     */
    QStringList reverseMap(const QStringList & list);

    /**
     * Map a client address to a server address.
     */
    QString map(const QString & url);

    QStringList map(const QStringList & urlList);

    /**
     * Create a folder from the client.
     */
    VFolder * createFolder
      (
        VFolder * parent,
        const QString & name,
        const QString & type,
        const QString & viewType
      );

    /**
     * Create a new folder notified by the server.
     */
    void createFolder
      (
        const QString & parentFolder,
        const QString & name,
        const QString & type,
        const QString & viewType,
        const QString & serverPath = QString::null
      );

    bool deleteFolder(const QString & name);

    /**
     * Delete a folder notified by the server.
     */
    void deleteFolder
      (
        const QString & parent,
        const QString & folderName,
        const QString & type
      );

    /**
     * Remove the VFS folder.
     */
    void deleteVFolder(const QString & name);

    /**
     * Remove the VFS folder.
     */
    void deleteVFolder(VFolder * folder);

    /**
     * Move the folder.
     */
    bool moveFolder(const QString & path, const QString & newPath);

    /** Update the VObject.*/
    void updateVInstance    (const QString & path);
    /** Update the VObjects.*/
    void updateVInstances   (const QStringList & pathList);
    void deleteVInstance    (const QString & path, bool bServerUrl=true);
    void deleteVInstances   (const QStringList & paths, bool bServerUrl=true);
    void createVInstance    (const QString & path, bool bServerUrl=true);
    bool createVInstances   (const QStringList & pathList, bool bServerUrl=true);

  private:

    /** It's a singleton class.*/
    ClientVFS();
    static ClientVFS * inst;
    
    bool makeBranch(const QString & folderName, const QString & path);

    void saveFolderBranch(VFolder * folder);

    /** The factories for VObjects.*/
    QDict<VObjectFactory> factoryDict;
    /** The general index for VFolders.*/
    QDict<VFolder>        folders;
    /** The map for converting server URL to client URL.*/
    QStringMap            s2cMap;
    QString               currentPath;
    QStringList           pStack;
    bool                  autogenTemp;
    bool                  useFact;
    bool                  genVInstances;
    bool                  genNonRegistered;
};

#endif
















