/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                vfscomponent.cpp  -  the VFS korelib component
                             -------------------
    begin                : Wed Aug 29 14:02:00 EET 2001
    copyright            : (C) 2001 by theKompany.com (www.thekompany.com>
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#include <vfscomponent.h>
#include <komaetheratools.h>
#include <vobjectfactory.h>
#include <vfolder.h>

/*
static const char* vfscomponent_metainfo[][2]=
{
    { "/name", "VFS Component" },
    { "/class", "VFSComponent" },
    { "/class/fqn", "VFSComponent" },
    { "/factory", "vfscomponent_factory" },
    { 0, 0 }
};
*/

kore::Interface* vfscomponent_factory()
{
  return new VFSComponent();
}

VFSComponent::VFSComponent()
{
  // add the factory to the component meta info
  MetaInfo *info = metaInfo();
  info->replaceAll("name", "VFS Component");
  info->replaceAll("class", "VFSComponent");
  info->replaceAll("class/fqn", "VFSComponent");
  info->replaceAll("factory", "vfscomponent_factory");
  
  clientVFS=ClientVFS::thisInstance();
  brokerInterface=BrokerInterface::thisInstance();
  if( clientVFS==NULL || brokerInterface==NULL )
    DEBUG_MSG("VFSComponent Error: ClientVFS or BrokerInterface cannot be built.");
}

VFSComponent::~VFSComponent()
{
  if( clientVFS!=NULL )
  {
    delete clientVFS;
    clientVFS=NULL;
  }
  if( brokerInterface!=NULL )
  {
    delete brokerInterface;
    brokerInterface=NULL;
  }
}

bool VFSComponent::initHierarchy()
{
  return clientVFS->initHierarchy();
}

bool VFSComponent::createVInstances(const QString & type)
{
  return clientVFS->createVInstances(type);
}

bool VFSComponent::createVInstances(VFolder *folder)
{
  return clientVFS->createVInstances(folder);
}

bool VFSComponent::rehashHierarchy()
{
  return clientVFS->rehashHierarchy();
}

bool VFSComponent::rehashVInstances()
{
  return clientVFS->rehashVInstances();
}


QDict<VFolder> VFSComponent::folderDict()
{
  return clientVFS->folderDict();
}

QString VFSComponent::parentFolder(QString url)
{
  return clientVFS->parentFolder(url);
}

VFolder* VFSComponent::parentFolder(VFolder *folder)
{
  return clientVFS->parentFolder(folder);
}

QStringList VFSComponent::folderList(QString folder)
{
  return clientVFS->folderList(folder);
}

void VFSComponent::autogenTemporary(bool temp)
{
  clientVFS->autogenTemporary(temp);
}

void VFSComponent::useFactories(bool fact)
{
  clientVFS->useFactories(fact);
}

void VFSComponent::autogenVInstances(bool vinst, bool nonreg)
{
  clientVFS->autogenVInstances(vinst, nonreg);
}

VFolder* VFSComponent::folderRef(QString folder)
{
  return clientVFS->folderRef(folder);
}

VFolder* VFSComponent::folderRef(QString parent, QString name)
{
  return clientVFS->folderRef(parent, name);
}

VFolder* VFSComponent::folderRef(VFolder *parent, QString name)
{
  return clientVFS->folderRef(parent, name);
}

VObject* VFSComponent::objectRef(QString object)
{
  return clientVFS->objectRef(object);
}

VObject* VFSComponent::objectRef(VFolder *parent, QString id)
{
  return clientVFS->objectRef(parent, id);
}

VObject* VFSComponent::objectRef(QString folder, QString id)
{
  return clientVFS->objectRef(folder, id);
}

VReference* VFSComponent::reference(QString url)
{
  return clientVFS->reference(url);
}

void VFSComponent::initFolderCache(const QString &type)
{
  clientVFS->initFolderCache(type);
}

void VFSComponent::discardFolderCache(const QString &type)
{
  clientVFS->discardFolderCache(type);
}

void VFSComponent::registerFactory(QString type, VObjectFactory *factory)
{
  clientVFS->registerFactory(type, factory);
}

VFolder* VFSComponent::_new(const QString& path,
                            const QString& type,
                            const QString& viewType,
                            const QString& mimeType,
                            const QString& binding,
                            const QString& spath)
{
  return clientVFS->_new(path, type, viewType, mimeType, binding, spath);
}

VObject* VFSComponent::_fact_new(const QString& path, const QString& type)
{
  return clientVFS->_fact_new(path, type);
}

VObject* VFSComponent::_fact_ref(const QString& path, const QString& type)
{
  return clientVFS->_fact_ref(path, type);
}

bool VFSComponent::canDefine(QString url)
{
  return clientVFS->canDefine(url);
}

#ifndef _QTWIN_
template <class T>
T* VFSComponent::_new(QString url)
{
  return clientVFS->template _new<T>(url);
}

template <class T>
T* VFSComponent::_new(QString url, QByteArray &data)
{
  return clientVFS->template _new<T>(url, data);
}

template <class T>
T* VFSComponent::_ref(QString url)
{
  return clientVFS->template _ref<T>(url);
}
#endif

QString VFSComponent::genTempFolderName()
{
  return clientVFS->genTempFolderName();
}

QString VFSComponent::pwd()
{
  return clientVFS->pwd();
}

QStringList VFSComponent::dir(QString params)
{
  return clientVFS->dir(params);
}

bool VFSComponent::cd(QString path)
{
  return clientVFS->cd(path);
}

void VFSComponent::cdUp()
{
  clientVFS->cdUp();
}

void VFSComponent::cdRoot()
{
  clientVFS->cdRoot();
}

void VFSComponent::pushPath()
{
  clientVFS->pushPath();
}

void VFSComponent::popPath()
{
  clientVFS->popPath();
}

QString VFSComponent::root()
{
  return clientVFS->root();
}

void VFSComponent::saveStructure()
{
  clientVFS->saveStructure();
}

void VFSComponent::saveFolder(VFolder *folder)
{
  clientVFS->saveFolder(folder);
}
		
QString VFSComponent::reverseMap(QString path)
{
  return clientVFS->reverseMap(path);
}

QStringList VFSComponent::reverseMap(QStringList list)
{
  return clientVFS->reverseMap(list);
}

QString VFSComponent::map(QString url)
{
  return clientVFS->map(url);
}

QStringList VFSComponent::map(QStringList urlList)
{
  return clientVFS->map(urlList);
}

VFolder* VFSComponent::createFolder(VFolder *parent, const QString &name, const QString &type, const QString &viewType)
{
  return clientVFS->createFolder(parent, name, type, viewType);
}

void VFSComponent::createFolder(const QString &parentFolder, const QString &name, const QString &type, const QString &viewType, const QString &serverPath)
{
  clientVFS->createFolder(parentFolder, name, type, viewType, serverPath);
}

bool VFSComponent::deleteFolder(const QString &name)
{
  return clientVFS->deleteFolder(name);
}

void VFSComponent::deleteFolder(const QString &parent, const QString &folderName, const QString &type)
{
  clientVFS->deleteFolder(parent, folderName, type);
}

void VFSComponent::deleteVFolder(const QString &name)
{
  clientVFS->deleteVFolder(name);
}

void VFSComponent::deleteVFolder(VFolder *folder)
{
  clientVFS->deleteVFolder(folder);
}

bool VFSComponent::moveFolder(const QString &path, const QString &newPath)
{
  qDebug("VFSComponent::moveFolder %s - %s", path.latin1(), newPath.latin1());
  
  VFolder * folder = folderRef(path);
  if( !folder )
    return false;
    
  QString oldParentPath = folder->parentFolder()->path();
  QString oldName = folder->name();
  QString oldType = folder->type();
  
  if( clientVFS->moveFolder(path, newPath) &&
      (folder = folderRef(newPath)) )
  {
    qDebug("Remove %s folder.", (oldParentPath + "/" + oldName).latin1());
    // remove the old folder from tree
    emitDeleteVFolder(oldParentPath, oldName, oldType);
    
    qDebug("Create %s folder.", folder->path().latin1());
    // add the new tree
    emitCreateVFolder(folder->parentFolder()->path(),
                      folder->name(),
                      folder->type(),
                      folder->viewType(),
                      folder->serverPath());
    
    QStringList foldersTree;
    folder->tree(foldersTree, -1);
    
    for(QStringList::const_iterator it=foldersTree.begin(); it!=foldersTree.end(); ++it)
    {
      folder = folderRef(*it);
      
      if( folder )
      {
        qDebug("Create %s folder.", folder->path().latin1());
        // add the new tree
        emitCreateVFolder(folder->parentFolder()->path(),
                          folder->name(),
                          folder->type(),
                          folder->viewType(),
                          folder->serverPath());
      }
    }
    
    return true;
  }
  else
    return false;
}

void VFSComponent::updateVInstance(const QString & path)
{
  clientVFS->updateVInstance(path);
}

void VFSComponent::updateVInstances(const QStringList & pathList)
{
  clientVFS->updateVInstances(pathList);
}

void VFSComponent::deleteVInstance(const QString & path, bool bServerUrl)
{
  clientVFS->deleteVInstance(path, bServerUrl);
}

void VFSComponent::deleteVInstances(const QStringList & paths, bool bServerUrl)
{
  clientVFS->deleteVInstances(paths, bServerUrl);
}

void VFSComponent::createVInstance(const QString & path, bool bServerUrl)
{
  clientVFS->createVInstance(path, bServerUrl);
}

bool VFSComponent::createVInstances(const QStringList & pathList, bool bServerUrl)
{
  return clientVFS->createVInstances(pathList, bServerUrl);
}


bool VFSComponent::isFolder(QString url)
{
  return brokerInterface->isFolder(url);
}

bool VFSComponent::exists(QString url)
{
  return brokerInterface->exists(url);
}

QString VFSComponent::parent(QString url)
{
  return brokerInterface->parent(url);
}

QString VFSComponent::folder(QString url)
{
  return brokerInterface->folder(url);
}

QStringList VFSComponent::children(QString url)
{
  return brokerInterface->children(url);
}

QStringList VFSComponent::subfolders(QString url)
{
  return brokerInterface->subfolders(url);
}

QStringList VFSComponent::related(QString url)
{
  return brokerInterface->related(url);
}

int VFSComponent::type(QString url)
{
  return brokerInterface->type(url);
}

int VFSComponent::subtype(QString url)
{
  return brokerInterface->subtype(url);
}
		
bool VFSComponent::setRelated(QString url, QStringList rList)
{
  return brokerInterface->setRelated(url, rList);
}

bool VFSComponent::setParent(QString url, QString parentUrl)
{
  return brokerInterface->setParent(url, parentUrl);
}

bool VFSComponent::getObjectData(QString url, QByteArray &data)
{
  return brokerInterface->getObjectData(url, data);
}

bool VFSComponent::getObjectData(QStringList &urls, QByteArray &data)
{
  return brokerInterface->getObjectData(urls, data);
}

bool VFSComponent::getServerObjectsData(QStringList &urls, QByteArray &data)
{
  return brokerInterface->getServerObjectsData(urls, data);
}

bool VFSComponent::getChildrenData(QString url, QByteArray &data)
{
  return brokerInterface->getChildrenData(url, data);
}

QByteArray VFSComponent::getSimpleData(QString url)
{
  return brokerInterface->getSimpleData(url);
}

unsigned VFSComponent::getFolderRecentData(const QString & folderURL)
{
  return brokerInterface->getFolderRecentData( folderURL );
}

bool VFSComponent::setObjectData(QString url, const QByteArray &data)
{
  return brokerInterface->setObjectData(url, data);
}

QString VFSComponent::createObject(QString parent_url)
{
  return brokerInterface->createObject(parent_url);
}

QString VFSComponent::createObject(QString parent_url, QByteArray &data)
{
  return brokerInterface->createObject(parent_url, data);
}

QString VFSComponent::createFolder(QStringList& paramList)
{
  return brokerInterface->createFolder(paramList);
}

QString VFSComponent::copy(QString url, QString parent_url)
{
  return brokerInterface->copy(url, parent_url);
}

QString VFSComponent::move(QString url, QString parent_url)
{
  return brokerInterface->move(url, parent_url);
}

bool VFSComponent::remove(QString url)
{
  return brokerInterface->remove(url);
}

bool VFSComponent::copy(const QStringList & urls, QString parent_url)
{
  return brokerInterface->copy(urls, parent_url);
}

bool VFSComponent::move(const QStringList & urls, QString parent_url)
{
  return brokerInterface->move(urls, parent_url);
}

bool VFSComponent::remove(const QStringList & urls)
{
  return brokerInterface->remove(urls);
}

bool VFSComponent::removeFolder(QStringList& paramList)
{
  return brokerInterface->removeFolder(paramList);
}

bool VFSComponent::send(QString url)
{
  return brokerInterface->send(url);
}

QString VFSComponent::lastError()
{
  return brokerInterface->lastError();
}

QString VFSComponent::regetLastError()
{
  return brokerInterface->regetLastError();
}


