/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                vfscomponent.h  -  the VFS korelib component
                             -------------------
    begin                : Wed Aug 29 14:02:00 EET 2001
    copyright            : (C) 2001 by theKompany.com (www.thekompany.com>
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#ifndef VFSCOMPONENT_H
#define VFSCOMPONENT_H

#include <vfsmodule.h>
#include <clientvfs.h>
#include <brokeriface.h>

extern "C"
{
    kore::Interface* vfscomponent_factory();
};

class VFSComponent : public VFSModule
{
  public:
    VFSComponent();
    virtual ~VFSComponent();

    // ----------------- ClientVFS API --------------------------------
    // Methods for building the VFS
    /** Initialise the VFS hierarchy.*/
    virtual bool initHierarchy();
    /** Create all the entries from VFS.*/
    virtual bool createVInstances(const QString& type=QString::null);
    /** Create all the entries from VFS for the folder 'folder'.*/
    virtual bool createVInstances(VFolder *folder);
    /** Rebuilding the VFS hierarchy.*/
    virtual bool rehashHierarchy();
    /** Rebuilding the VFS.*/
    virtual bool rehashVInstances();

    // Methods for browsing the VFS folders
    /** Return a sorted folders list.*/
    virtual QDict<VFolder> folderDict();
    /** Return the name of the parent folder.*/
    virtual QString parentFolder(QString url);
    /** Return the parent folder.*/
    virtual VFolder *parentFolder(VFolder *);
    /** Return a list of folders.*/
    virtual QStringList folderList(QString folder);

    // Methods controlling the behaviour of the ClientVFS object
    /** If the VFS will create the temporar entries automaticaly.*/
    virtual void autogenTemporary(bool temp);
    /** If the VFS is using the registered factories.*/
    virtual void useFactories(bool fact);
    /** VFS will create entries even if there aren't factories for it.*/
    virtual void autogenVInstances(bool vinst, bool nonreg);

    /** Return the folder reference with the 'folder' name.*/
    virtual VFolder *folderRef(QString folder);
    /** Return the folder reference which has the parent name 'parent' and the name 'name'.*/
    virtual VFolder *folderRef(QString parent, QString name);
    /** Return the folder reference which has the parent 'parent' and the name 'name'.*/
    virtual VFolder *folderRef(VFolder *parent, QString name);
    /** Return the object reference with the url 'object'.*/
    virtual VObject *objectRef(QString object);
    /** Return the object reference which has the parent and the id.*/
    virtual VObject *objectRef(VFolder *parent, QString id);
    /** Return the object reference which has the parent and the id.*/
    virtual VObject *objectRef(QString folder, QString id);		
    /** Return the reference for an VFS entry using its url.*/
    virtual VReference *reference(QString url);

    // init folder cache
    /** Initialize the cache.*/
    virtual void initFolderCache(const QString &type);
    /** Free the cache.*/
    virtual void discardFolderCache(const QString &type);

    // register VObject factories
    /** Register a new factory for creating VObjects.*/
    virtual void registerFactory(QString type, VObjectFactory *factory);

    // this makes a new folder
    /** Create a new folder in VFS.*/
    virtual VFolder *_new(const QString& path,
                          const QString& type,
                          const QString& viewType,
                          const QString& mimeType,
                          const QString& binding=QString::null,
                          const QString& spath=QString::null);

    /** Create a new VFS entry using a factory.*/
    virtual VObject *_fact_new(const QString& path, const QString& _type=QString::null);
    /** Try to find the object and if it isn't it will be created.*/
    virtual VObject *_fact_ref(const QString& path, const QString& _type=QString::null);

    /** If there are support for this kind of url.*/
    virtual bool canDefine(QString url);

#ifndef _QTWIN_
    /** Template for creating a new VFS entry.*/
    template <class T>
    T *_new(QString url);

    /** Template for creating a new VFS entry with the attributes from 'data'.*/
    template <class T>
    T *_new(QString url, QByteArray &data);

    /** Try to find the url and if it isn't it will be created.*/
    template <class T>
    T *_ref(QString url);
#endif

    /** Return the name of the temporary folder.*/
    virtual QString genTempFolderName();

    // Shell methods
    /** Return the current folder.*/
    virtual QString pwd();
    /** Return the subfolders from the current folder.*/
    virtual QStringList dir(QString params=QString::null);
    /** Change the folder to 'path'.*/
    virtual bool cd(QString path);
    /** Change the folder to the parent folder.
      * Go up in the VFS hierarchy.
      */
    virtual void cdUp();
    /** The root folder will be the current folder.*/
    virtual void cdRoot();
    /** Push the current folder in the VFS stack.*/
    virtual void pushPath();
    /** Pop the current folder from the VFS stack.*/
    virtual void popPath();
    /** Return the root folder name.
      * The name should be Aethera :)
      */
    virtual QString root();

    // Save the VFS into a file.
    /** Save the whole VFS structure into the configuration file.*/
    virtual void saveStructure();
    /** Save the folder into the configuration folder.*/
    virtual void saveFolder(VFolder *folder);

    // Maps a server address on a client address, or returns QString::null
    // if such a mapping does not exist
    /** sever url -> client url.*/
    virtual QString reverseMap(QString path);
    /** sever urls -> client urls.*/
    virtual QStringList reverseMap(QStringList list);
    // Maps a client address on a server address
    /** client url -> server url.*/
    virtual QString map(QString url);
    /** client urls -> server urls.*/
    virtual QStringList map(QStringList urlList);

    // create and delete folders methods
    /** Create a folder from the client.*/
    virtual VFolder *createFolder(VFolder *parent, const QString &name, const QString &type, const QString &viewType);
    /** Create a new folder notified by the server.*/
    virtual void createFolder(const QString &parentFolder, const QString &name, const QString &type, const QString &viewType, const QString &serverPath=QString::null);
    /** Delete the folder.*/
    virtual bool deleteFolder(const QString &name);
    /** Delete a folder notified by the server.*/
    virtual void deleteFolder(const QString &parent, const QString &folderName, const QString &type);
    /** Remove the VFS folder.*/
    virtual void deleteVFolder(const QString &name);
    /** Remove the VFS folder.*/
    virtual void deleteVFolder(VFolder *folder);
    /** Move the folder.*/
    virtual bool moveFolder(const QString &path, const QString &newPath);
    /** Update the VObject.*/
    virtual void updateVInstance    (const QString & path);
    /** Update the VObjects.*/
    virtual void updateVInstances   (const QStringList & pathList);
    virtual void deleteVInstance    (const QString & path, bool bServerUrl=true);
    virtual void deleteVInstances   (const QStringList & paths, bool bServerUrl=true);
    virtual void createVInstance    (const QString & path, bool bServerUrl=true);
    virtual bool createVInstances   (const QStringList & pathList, bool bServerUrl=true);

    // ----------- Broker Interface API ----------------------------------
    /** Check if the url is a folder.*/
    virtual bool isFolder(QString url);
    /** Check if there is the folder with the url.*/
    virtual bool exists(QString url);
    /** Return the parent for the url.*/
    virtual QString parent(QString url);
    /** Return the folder name from the server.*/
    virtual QString folder(QString url);
    /** Return the list of childrens from the folder with the url using the server.*/
    virtual QStringList children(QString url);
    /** Return the list of subfolders from the folder with the url using the server.*/
    virtual QStringList subfolders(QString url);
    /** Not implemented yet.*/
    virtual QStringList related(QString url);
    /** Return the url type like a number.*/
    virtual int type(QString url);
    /** Return the url subtype like a number.*/
    virtual int subtype(QString url);

    // set methods
    /** Set related entries to the url.*/
    virtual bool setRelated(QString url, QStringList rList);
    /** Set the parent for the url.*/
    virtual bool setParent(QString url, QString parentUrl);

    // Methods for getting and setting the objects attributes
    /** Get the url object data.*/
    virtual bool getObjectData(QString url, QByteArray &data);
    /** Get data for many objects.*/
    virtual bool getObjectData(QStringList &urls, QByteArray &data);
    /** Get data for many objects.*/
    virtual bool getServerObjectsData(QStringList &urls, QByteArray &data);
    /** Get all folder's children data.*/
    virtual bool getChildrenData(QString url, QByteArray &data);
    /** Get the object data like an bytes array.*/
    virtual QByteArray getSimpleData(QString url);
    /** Set the url object data.*/
    virtual bool setObjectData(QString url, const QByteArray &data);
    /** Get the recent mails number from a mails folder.*/
    virtual unsigned getFolderRecentData(const QString & folderURL);

    // Shell methods for the server
    /** Create a new object on the server.*/
    virtual QString createObject(QString parent_url);
    virtual QString createObject(QString parent_url, QByteArray &data);
    /** Create a new folder on the server.*/
    virtual QString createFolder(QStringList& paramList);
    /** Copy an url object to other url object.*/
    virtual QString copy(QString url, QString parent_url);
    /** Move an url object to other url object.*/
    virtual QString move(QString url, QString parent_url);
    /** Remove the object from the server.*/
    virtual bool remove(QString url);
    /** Remove the folder from the server.*/
    virtual bool removeFolder(QStringList& paramList);
    /** Send an url to the server.*/
    virtual bool send(QString url);
    
    // multiple objects operations
    virtual bool copy(const QStringList & urls, QString parent_url);
    virtual bool move(const QStringList & urls, QString parent_url);
    virtual bool remove(const QStringList & urls);
    
    /** Error handling.*/
    virtual QString lastError();
    /** Error handling.*/
    virtual QString regetLastError();

  private:
    ClientVFS *clientVFS;
    BrokerInterface *brokerInterface;

};

#endif




