/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef KOMAETHERAMAN_H
#define KOMAETHERAMAN_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//Qt include
#include <qlabel.h>
#include <qdict.h>
#include <qpushbutton.h>

// TINO include
#include <tino/tino.h>

// TKWidgets include
#include <tkdockwidget.h>
#include <tkdockmanager.h>
#include <tktabbar.h>

// KOM include
#include <komaetheraplugin.h>
#include <komaetheratools.h>
#include <komcontainerwidget.h>
#include <komuipartmanagerprivate.h>

class KomMainTree;
class SettingsDialog;
class QButtonGroup;
class QAction;
class QPopupMenu;

#define _DEBUG_MODE_

#define APP_KEY                   "/Aethera/Geometry/TKWidgets"

extern "C"
{
    kore::Interface* komaetheraman_factory();
};

/** Aethera Application Manager.*/
class KomAetheraMan : public UIPartManager
{
public:
  KomAetheraMan(QMainWindow*);
  virtual ~KomAetheraMan();
   bool init();
  /** A slot for the plugins tollbar click.*/
  void pluginToolbarButtonClicked( const QString & type );

  void treeURLChanged(const QString &);
  void showSettingsDialog();

  /**
   * Show the help window.
   */
  void helpIndex();

  /**
   * Show the about dialog for the currently active plugin.
   */
  void aboutActivePlugin();

  /**
   * Show the about dialog for the app.
   */
  void about();

  /**
   * Shop for plugins.
   */
  void shop();
  
  /** Hide or show the left widget.*/
  void toggleLeftDockWidget(bool);
  /** Hide or show the banner.*/
  void toggleBannerDockWidget(bool);
  /** Set the status of hide/show widgets actions.*/
  void setToggleActions();
  
  /** Return the main widget.*/
  virtual QWidget * mainWidget();
  /** Return the main tree.*/
  virtual KomMainTree * foldersTree();

protected:

  /**
   * Used to remove help menu before creating plugin's menus
   */
  void removeHelpMenu();

  /**
   * Used to re-add help menu before creating plugin's menus.
   */
  void addHelpMenu();

  /** Create the main QActions.*/
  void createMainQActions();
  
  // virtual methods (some are from TINO)
  /** Add a new part.*/
  virtual void addPart(UIPart *);
  /** Remove a part.*/
  virtual void removePart(UIPart *);
  /** Enable the part UI knowing its id.*/
  virtual void activatePart(int id);
  /** Enable the part UI knowing its type.*/
  virtual void activatePart(const QString & type);
  /** Enable the part UI.*/
  virtual void activatePart(UIPart *);
  /** Disable the part UI.*/
  virtual void deactivatePart(UIPart *);
  /** Create the QT signals/slots handler.*/
  virtual UIPartManagerPrivate *createQTHandler(void);

  // the DockWidgets
  /** The dock widgets manager from TKWidgets.*/
  TKDockManager *dockManager;
  /** The main dock widget.*/
  TKDockWidget *mainDockWidget;
  /** The left dock widget.*/
  TKDockWidget *leftDockWidget;
  /** The widget container for the left widget.*/
  KomContainerWidget *leftWidget;
  /** The banner dock widget.*/
  TKDockWidget *bannerDockWidget;
  /** The widget container for the banner widget.*/
  KomContainerWidget *bannerWidget;
  /** The widget container for the top widget.*/
  KomContainerWidget *topWidget;

  /** The tree interface.*/
  KomMainTree *mainTree;

private:
  /** Add translators.*/
  void InitializingTranslators();
  /** Initialize the UI for the components.*/
  void InitializingUIServices();
  /** Initialize the backend service for the components.*/
  void InitializingBackendServices();
  /** A dictionary for the UI components using the index number.*/
  QDict<UIPart> idxPartDict;
  /** A dictionary for the UI components using the mimetype string.*/
  QDict<UIPart> typePartDict;

  /** The plug ins toolbar.*/
  QToolBar *pluginToolbar;
  /** The type of the last active button.*/
  QString lastToolbarButtonClicked;

  SettingsDialog * m_settingsDialog;

  QButtonGroup * m_pluginToolbarButtonGroup;

  int m_helpMenuId;
  QPopupMenu * m_helpPopup;

  QAction * m_helpIndexAction;
  QAction * m_aboutPluginAction;
  QAction * m_aboutAction;
  QAction * m_shopAction;
  QAction * m_toggleLeftWidgetAction;
  QAction * m_toggleBannerWidgetAction;
  QActionGroup * m_toggleWidgetAction;
  
  QLabel * emptyLabel;
  bool bLeftVisible;
};

#endif
