/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include <komaetheraxchg.h>


KomAetheraXchg::KomPluginConnectionPoint::KomPluginConnectionPoint( )
{
  m_pluginInterface = NULL;
}

KomAetheraXchg::KomPluginConnectionPoint::KomPluginConnectionPoint( const QString &pluginName, KomXchgInterface *pluginInterface)
{
  m_pluginInterface = pluginInterface;
  m_pluginName = pluginName;
}

KomAetheraXchg::KomPluginConnectionPoint::~KomPluginConnectionPoint()
{
  DELETE_PTR( m_pluginInterface );
}

KomAetheraXchg::KomPluginConnectionPoint& KomAetheraXchg::KomPluginConnectionPoint::operator=( KomPluginConnectionPoint& r )
{
  DELETE_PTR( m_pluginInterface );
  m_pluginInterface = new KomXchgInterface( *(r.m_pluginInterface) );
  return *this;
}

void KomAetheraXchg::KomPluginConnectionPoint::setInterface( KomXchgInterface* pluginInterface )
{
  DELETE_PTR( m_pluginInterface );
  m_pluginInterface = pluginInterface;
}




KomAetheraXchg::KomAetheraXchg( int pluginConnectionsCount )
{
  m_pluginConnections = NULL;
  m_pluginConnectionsCount = pluginConnectionsCount;
  if ( m_pluginConnectionsCount>0 )
  {
    m_pluginConnections = new PKomPluginConnectionPoint[ m_pluginConnectionsCount ];
    for ( int i=0; i<m_pluginConnectionsCount; ++i )
    {
      m_pluginConnections[i] = NULL;
    }
  }
}
KomAetheraXchg::~KomAetheraXchg()
{
  DELETEPARRAY_PTR( m_pluginConnections, m_pluginConnectionsCount );
}

KomXchgInterface* KomAetheraXchg::getXchgInterface( int pluginId )
{
  if ( m_pluginConnections==NULL || pluginId<0 || pluginId>=m_pluginConnectionsCount )
    return NULL;
  return m_pluginConnections[ pluginId ]->getInterface();
}

KomXchgInterface* KomAetheraXchg::getXchgInterface( QString pluginName )
{
  if ( m_pluginConnections==NULL ) return NULL;
  for ( int i=0; i<m_pluginConnectionsCount; ++i )
    if ( m_pluginConnections[i]!=NULL )
    {
      if ( (*m_pluginConnections[i]) == pluginName )
        return m_pluginConnections[i]->getInterface();
    }
  return NULL;
}

int KomAetheraXchg::getPluginId( QString pluginName )
{
  if ( m_pluginConnections==NULL ) return KOMFAILED;
  for ( int i=0; i<m_pluginConnectionsCount; ++i )
    if ( m_pluginConnections[i]!=NULL )
    {
      if ( (*m_pluginConnections[i]) == pluginName )
        return i;
    }
  return KOMFAILED;
}

int KomAetheraXchg::connectPlugin( QString pluginName )
{
  if ( m_pluginConnections==NULL ) return KOMFAILED;
  int id =0;
  for ( id=0; id<m_pluginConnectionsCount; ++id )
    if ( m_pluginConnections[id]!=NULL )
    {
      m_pluginConnections[id] = new KomPluginConnectionPoint( pluginName,
                                    new KomXchgInterface );
      return id;
    }
  return KOMFAILED;
}

int KomAetheraXchg::disconnectPlugin( QString pluginName )
{
  int id = getPluginId( pluginName );
  if ( id>=0 ) return disconnectPlugin( id );
  return KOMFAILED;
}

int KomAetheraXchg::disconnectPlugin( int pluginId )
{
  if ( m_pluginConnections==NULL  || pluginId<0 || pluginId>=m_pluginConnectionsCount )
    return KOMFAILED;
  if ( m_pluginConnections[pluginId]==NULL ) return KOMFAILED;
  delete m_pluginConnections[pluginId];
  m_pluginConnections[pluginId] = NULL;
  return OK;
}




KomXchgInterface::KomXchgInterface( unsigned int reserveActionCount )
{
  m_registeredActions = NULL;
  m_registeredActionsCount = 0;
  m_reservedActionCount = reserveActionCount;
  if ( m_reservedActionCount>0 )
  {
    m_registeredActions = new KomXchgAction[m_reservedActionCount];
    for (int i=0; i<m_reservedActionCount; ++i )
    {
      m_registeredActions[i] = NULL;
    }
  }
}

KomXchgInterface::~KomXchgInterface()
{
  DELETEARRAY_PTR( m_registeredActions );
}


int KomXchgInterface::registerAction( KomXchgAction actionPtr, stdActions actionId )
{
  if ( m_registeredActions==NULL || m_registeredActionsCount>=m_reservedActionCount )
    return KOMFAILED;
  if ( actionId!=stdNone )
  {
    if ( m_registeredActions[actionId]!=NULL )
    {
      m_registeredActions[actionId]=actionPtr;
      return OK;
    }
    else return KOMFAILED;
  }
  else
  {
    for (int i=0; i<m_reservedActionCount; ++i )
      if ( m_registeredActions[i] != NULL )
      {
        m_registeredActions[actionId]=actionPtr;
        return i;
      }
  }
  return KOMFAILED;
}

int KomXchgInterface::execute( stdActions actionId, KomXchgObject& o )
{
  if ( m_registeredActions==NULL ) return KOMFAILED;
  if ( m_registeredActions[actionId]==NULL ) return KOMFAILED;
  return (*m_registeredActions[actionId])( o );
}

int KomXchgInterface::execute( int actionId, KomXchgObject& o )
{
  if ( m_registeredActions==NULL ) return KOMFAILED;
  if ( m_registeredActions[actionId]==NULL ) return KOMFAILED;
  return (*m_registeredActions[actionId])( o );
}



