/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    notesmainwidget.cpp
                             -------------------
    begin                : Sun Nov 25 16:09:14 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <vdragobject.h>
#include <qtimer.h>
#include <qiconview.h>
#include <qwindowsstyle.h>
#include <kommaintree.h>
#include <qpopupmenu.h>
#include <qheader.h>

static const int autoOpenTime = 500;

KomMainTree::KomMainTree(QWidget * parent, const char * name)
  : QListView           (parent, name),
    m_currentInterface  (0L),
    m_autoOpenTimer     (0L),
    m_lastCurrentItem   (0L),
    m_dropItem          (0L)
{
  setSelectionMode(QListView::Single);

  setTreeStepSize(20);

  setRootIsDecorated(true);

  addColumn(""); // Hidden - we just want to be able to see stuff ;)

  header()->hide();

  connect
    (
      this,
      SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)),
      SLOT(slot_contextMenuRequested(QListViewItem *, const QPoint &, int))
    );

  connect
    (
      this,
      SIGNAL(currentChanged(QListViewItem *)),
      SLOT(slot_currentChanged(QListViewItem *))
    );

  m_autoOpenTimer = new QTimer(this);

  connect
    (
      m_autoOpenTimer,
      SIGNAL(timeout()),
      SLOT(slotOpenBranch())
    );

  setAcceptDrops(true);
  viewport()->setAcceptDrops(true);
}

KomMainTree::~KomMainTree( )
{
}

  void
KomMainTree::addInterface(KomMainTreeInterface * iface)
{
  if( !iface )
  {
    printf("/nNo interface !!!/n");
    fflush(stdout);
    return;
  }

  m_nameToInterfaceMap[iface->name()] = iface;

  if (0 == m_currentInterface)
    m_currentInterface = iface;
}

  void
KomMainTree::delInterface(KomMainTreeInterface * iface)
{
  QMap<QString, KomMainTreeInterface *>::Iterator it;

  for
    (
      it = m_nameToInterfaceMap.begin();
      it != m_nameToInterfaceMap.end();
      ++it
    )
  {
    if (it.data() == iface)
    {
      m_nameToInterfaceMap.remove(it.key());
      return;
    }
  }
}

  void
KomMainTree::slot_contextMenuRequested
(
  QListViewItem * _item,
  const QPoint  & pos,
  int             column
)
{
  DEBUG_MSG("slot_contextMenuRequested");

  if (-1 == column)


    return;

  KomTreeItem * item = static_cast<KomTreeItem *>(_item);

  KomMainTreeInterface * iface = findInterface(item);

  if (0 == iface)
  {
    DEBUG_MSG("Hmm, can't find KomMainTreeInterface for this KomTreeItem");
    return;
  }

  iface->contextMenu(item)->popup(pos);
}

  void
KomMainTree::slot_currentChanged(QListViewItem * _item)
{
//  qDebug("slot_currentChanged(%s)", _item->text(0).ascii());

  if (0 == _item)
    return;

  KomTreeItem * item = static_cast<KomTreeItem *>(_item);

  KomMainTreeInterface * iface = findInterface(item);

  if (0 == iface)
  {
    qDebug("No interface for %s ?", item->text(0).ascii());
    return;
  }

  if (m_currentInterface == iface)
  {
//    qDebug("No need to change interface");
  }
  else
  {
    m_currentInterface = iface;
 
    KomTreeItem * rootItem = findRootOf(item);

    if (0 == item)
    {
      qDebug("Hmm, no root ?");
      return;
    }

    QString rootName = rootItem->text(0).lower();

    emit(changePart(rootName));
  }

  QString s = path(item);

  emit(currentURLChanged(path(item)));
}

  KomTreeItem *
KomMainTree::findRootOf(KomTreeItem * item)
{
  if (0 == item)
  {
    DEBUG_MSG("findRootOf 0 ? heh, no thanks.");
    return 0;
  }

  while (true)
  {
    KomTreeItem * parent = static_cast<KomTreeItem *>(item->parent());

    if (0 == parent)
      return item;
    else
      item = parent;
  }
}

  KomMainTreeInterface *
KomMainTree::findInterface(KomTreeItem * item)
{
  if (0 == item)
  {
    DEBUG_MSG("findInterface of 0 ? No thanks.");
    return 0;
  }

  KomTreeItem * rootItem = findRootOf(item);

  if (0 == rootItem)
    return 0;

  return m_nameToInterfaceMap[rootItem->text(0)];
}

  QString
KomMainTree::path(KomTreeItem * item)
{
  if (0 == item)
    return QString::null;

  QString path(item->text(0));

  while (item->depth() > 0)
  {
    item = static_cast<KomTreeItem *>(item->parent());
    path.prepend(item->text(0) + "/");
  }

  path.prepend("/Aethera/VFS/");

  return path;
}

  QDragObject *
KomMainTree::dragObject()
{
  QListViewItem * i = selectedItem();

  if (0 == i)
    return 0;

  // STUB
  return 0;
}

  void
KomMainTree::contentsDragEnterEvent(QDragEnterEvent * e)
{
  if (!VDragObject::canDecode(e))
  {
    e->ignore();
    return;
  }

  m_lastCurrentItem = currentItem();

  QListViewItem * i = itemAt(contentsToViewport(e->pos()));

  if (0 == i)
    return;

  m_dropItem = i;
  m_autoOpenTimer->start(autoOpenTime);
}

  void
KomMainTree::contentsDragLeaveEvent(QDragLeaveEvent *)
{
  m_autoOpenTimer->stop();
  m_dropItem = 0;

  blockSignals(true);
  setCurrentItem(m_lastCurrentItem);
  setSelected(m_lastCurrentItem, true);
  blockSignals(false);
}

  void
KomMainTree::contentsDragMoveEvent(QDragMoveEvent * e)
{
  if (!VDragObject::canDecode(e))
  {
    e->ignore();
    return;
  }

  QListViewItem * i = itemAt(contentsToViewport(e->pos()));

  if (0 == i)
  {
    e->ignore();
    m_autoOpenTimer->stop();
    m_dropItem = 0;
    return;
  }

  blockSignals(true);
  setSelected(i, true);
  blockSignals(false);

  // Check that item being dropped is of same type as drop site.

  QString dropPath = path(static_cast<KomTreeItem *>(i));

  QString dropType = QStringList::split('/', dropPath)[2];

  QString dragPath = VDragObject::decode(e).first();

  QString dragType = QStringList::split('/', dragPath)[2];

  if (dropType != dragType)
  {
    // Type mismatch.
    e->ignore();
    return;
  }

  if (i != m_dropItem)
  {
    m_autoOpenTimer->stop();
    m_dropItem = i;

    m_autoOpenTimer->start(autoOpenTime);
  }

  e->accept();

  switch (e->action())
  {
    case QDropEvent::Copy:
    case QDropEvent::Move:
      e->acceptAction();
      break;
    default:
      break;
  }
}

  void
KomMainTree::contentsDropEvent(QDropEvent * e)
{
  m_autoOpenTimer->stop();

  blockSignals(true);
  setCurrentItem(m_lastCurrentItem);
  setSelected(m_lastCurrentItem, true);
  blockSignals(false);

  if (0 == m_dropItem)
  {
    qDebug("KomMainTree::contentsDropEvent: m_dropItem is 0");
    e->ignore();
    return;
  }

  QString dropPath = path(static_cast<KomTreeItem *>(m_dropItem));

  m_dropItem = 0;

  if (!VDragObject::canDecode(e))
  {
    e->ignore();
    return;
  }

  QListViewItem * i = itemAt(contentsToViewport(e->pos()));

  if (0 == i)
  {
    e->ignore();
    return;
  }

  QStringList l = VDragObject::decode(e);

  if (l.isEmpty())
    return;

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = dynamic_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qDebug("KomMainTree::contentsDropEvent: VFS is 0 ?");
    return;
  }

  switch (e->action())
  {
    case QDropEvent::Copy:
      qDebug("Copying to %s", dropPath.ascii());
      
      vfs->copy(l, dropPath);
      break;

    case QDropEvent::Move:
      qDebug("Moving to %s", dropPath.ascii());
      
      vfs->move(l, dropPath);
      break;

    default:
      break;
  }
}

  QListViewItem *
KomMainTree::selectedItem()
{
  QListViewItemIterator it(this);

  while (it.current())
  {
    if (it.current()->isSelected())
      return it.current();

    ++it;
  }

  return 0;
}

  void
KomMainTree::slotOpenBranch()
{
  m_autoOpenTimer->stop();

  if (0 == m_dropItem)
    return;

  if (!m_dropItem->isOpen())
  {
    m_dropItem->setOpen(true);
    m_dropItem->repaint();
  }
}

