/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  homeconfigservice.cpp  -  the configuration service for the Home plug in
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include "homeconfigservice.h"
#include "homeconfighandler.h"

#include <urllist.h>
#include <settings.h>
#include <kurllabel.h>

//QT includes
#include <qapplication.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qspinbox.h>
#include <qtabwidget.h>

//tkcBase includes
#include <qiconloader.h>

class HomeConfigService::Private
{
  public:
  
    Private(HomeConfigService * parent_);

    virtual ~Private();
    
    QWidget * widget;
    URLList * rssWidget;
    URLList * weatherWidget;
    QCheckBox * metric;
    HomeConfigHandler * qHandler;
    
    QString rssGroup;
    QString weatherGroup;
    
  private:
  
    HomeConfigService * parent_;
};

HomeConfigService::Private::Private(HomeConfigService * parent)
  : widget(0L),
    rssWidget(0L),
    weatherWidget(0L),
    metric(0L),
    qHandler(0L),
    rssGroup(Settings::instance()->name() + "/RSS/"),
    weatherGroup(Settings::instance()->name() + "/Weather/"),
    parent_( parent )
{
  qHandler = new HomeConfigHandler( parent_ );
}

HomeConfigService::Private::~Private()
{
  if( widget )
  {
    delete widget;
    widget = 0L;
  }
  
  if( qHandler )
  {
    delete qHandler;
    qHandler = 0L;
  }
}

/*
static const char* homeconfigservice_metainfo[][2]=
{
    { "/name", "Home Config Service" },
    { "/class", "HomeConfigService" },
    { "/class/fqn", "HomeConfigService" },
    { "/factory", "homeconfigservice_factory" },
    { 0, 0 }
};
*/

kore::Interface* homeconfigservice_factory()
{
  return new HomeConfigService();
}
    
HomeConfigService::HomeConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Home Config Service");
  info->replaceAll("class", "HomeConfigService");
  info->replaceAll("class/fqn", "HomeConfigService");
  info->replaceAll("factory", "homeconfigservice_factory");
  
  d = new Private(this);
}

HomeConfigService::~HomeConfigService()
{
  delete d;
}

  QWidget *
HomeConfigService::init(QWidget * parent)
{
  d->widget = new QWidget(parent, "HomeConfigService widget");
   
  QVBoxLayout * generalLayout = new QVBoxLayout( d->widget, 6, 6, "GeneralLayout");
  QTabWidget* tabWidget = new QTabWidget( d->widget, "HomeConfigService tabWidget");
  generalLayout->addWidget( tabWidget );

  //RSS tab  
  QGroupBox * rssGroupBox = new QGroupBox( d->widget, "rssGroupBox" );
  rssGroupBox->setTitle( QObject::trUtf8( "RSS Configuration" ) );
  rssGroupBox->setColumnLayout(0, Qt::Vertical );
  rssGroupBox->layout()->setSpacing( 6 );
  rssGroupBox->layout()->setMargin( 6 );
  QVBoxLayout * rssGroupBoxLayout = new QVBoxLayout( rssGroupBox->layout() );
  d->rssWidget = new URLList( rssGroupBox );
  rssGroupBoxLayout->addWidget( d->rssWidget );
  KURLLabel * rssLabel = 
    new KURLLabel("http://webreference.com/services/news/", "Get some RSS links.", rssGroupBox );
  rssGroupBoxLayout->addWidget( rssLabel );
  tabWidget->addTab( rssGroupBox, "RSS" );
  
  
  //weather
  QGroupBox * weatherGroupBox = new QGroupBox( d->widget, "weatherGroupBox" );
  weatherGroupBox->setTitle( QObject::trUtf8( "Weather Configuration" ) );
  weatherGroupBox->setColumnLayout(0, Qt::Vertical );
  weatherGroupBox->layout()->setSpacing( 6 );
  weatherGroupBox->layout()->setMargin( 6 );
  QVBoxLayout * weatherGroupBoxLayout = new QVBoxLayout( weatherGroupBox->layout() );
  d->weatherWidget = new URLList( weatherGroupBox );
  weatherGroupBoxLayout->addWidget( d->weatherWidget );
  d->metric = new QCheckBox( QObject::trUtf8("convert to metric"), weatherGroupBox);
  weatherGroupBoxLayout->addWidget( d->metric );
  KURLLabel * weatherLabel = 
    new KURLLabel("http://www.nws.noaa.gov/tg/siteloc.shtml", "Get the ICAO code.", weatherGroupBox );
  weatherGroupBoxLayout->addWidget( weatherLabel );
  tabWidget->addTab( weatherGroupBox, "Weather" );
  
  
  QObject::connect( rssLabel, SIGNAL(leftClickedURL(const QString&)),
                    d->qHandler, SLOT(launchUrl(const QString&)) );
  QObject::connect( rssLabel, SIGNAL(rightClickedURL(const QString&)),
                    d->qHandler, SLOT(launchUrl(const QString&)) );
  
  QObject::connect( weatherLabel, SIGNAL(leftClickedURL(const QString&)),
                    d->qHandler, SLOT(slotICAO(const QString&)) );
  QObject::connect( weatherLabel, SIGNAL(rightClickedURL(const QString&)),
                    d->qHandler, SLOT(slotICAO(const QString&)) );
  
//  QObject::connect( weatherLabel, SIGNAL(leftClickedURL(const QString&)),
//                    d->qHandler, SLOT(launchUrl(const QString&)) );
//  QObject::connect( weatherLabel, SIGNAL(rightClickedURL(const QString&)),
//                    d->qHandler, SLOT(launchUrl(const QString&)) );
  
  return d->widget;
}

  void
HomeConfigService::load()
{
  if( d->widget )
  {
    // RSS
    QStringList rssURL = Settings::instance()->readListEntry(d->rssGroup + "URL",'^');
    if( rssURL.isEmpty() )
    {
      rssURL += "http://slashdot.org/slashdot.rss";
      rssURL += "http://www.progrockrecords.com/shop/prr.rdf";
      rssURL += "http://www.mindawn.com/feeds/news.rss";
      rssURL += "http://www.thekompany.com/about/rss/tkc.rdf";
    }

    d->rssWidget->urlListBox->clear();
    if( !rssURL.isEmpty() )
    {
      d->rssWidget->urlListBox->insertStringList( rssURL );
    }

    bool rssTimer = Settings::instance()->readBoolEntry(d->rssGroup + "Timer", true);
    d->rssWidget->timerCheckBox->setChecked( rssTimer );

    unsigned rssInterval = Settings::instance()->readNumEntry(d->rssGroup + "Interval", 30);
    d->rssWidget->timerSpinBox->setValue( rssInterval );

    // Weather
    QStringList weatherURL = Settings::instance()->readListEntry(d->weatherGroup + "URL",'^');

    d->weatherWidget->urlListBox->clear();
    if( !weatherURL.isEmpty() )
    {
      d->weatherWidget->urlListBox->insertStringList( weatherURL );
    }

    bool weatherTimer = Settings::instance()->readBoolEntry(d->weatherGroup + "Timer", true);
    d->weatherWidget->timerCheckBox->setChecked( weatherTimer );

    unsigned weatherInterval = Settings::instance()->readNumEntry(d->weatherGroup + "Interval", 30);
    d->weatherWidget->timerSpinBox->setValue( weatherInterval );

    bool weatherMetric = Settings::instance()->readBoolEntry(d->weatherGroup + "Metric", true);
    d->metric->setChecked( weatherMetric );
  }
  else
  {
    qDebug("HomeConfigService::load() : main widget isn't available");
  }
}

  void
HomeConfigService::save()
{
  if( d->widget )
  {
    // RSS
    QStringList rssURL;
    QListBoxItem * item = d->rssWidget->urlListBox->firstItem();

    while( item )
    {
      rssURL += item->text();

      item = item->next();
    }

    if( !rssURL.isEmpty() )
      Settings::instance()->writeEntry(d->rssGroup + "URL", rssURL, '^');
    else
      Settings::instance()->removeEntry(d->rssGroup + "URL");    

    Settings::instance()->writeEntry(d->rssGroup + "Timer", d->rssWidget->timerCheckBox->isChecked());
    if( d->rssWidget->timerCheckBox->isChecked() )
      Settings::instance()->writeEntry(d->rssGroup + "Interval", d->rssWidget->timerSpinBox->value());

    // Weather
    QStringList weatherURL;
    item = d->weatherWidget->urlListBox->firstItem();

    while( item )
    {
      weatherURL += item->text();

      item = item->next();
    }

    if( !weatherURL.isEmpty() )
      Settings::instance()->writeEntry(d->weatherGroup + "URL", weatherURL, '^');
    else
      Settings::instance()->removeEntry(d->weatherGroup + "URL");    
    
    Settings::instance()->writeEntry(d->weatherGroup + "Timer", 
      d->weatherWidget->timerCheckBox->isChecked());
    if( d->weatherWidget->timerCheckBox->isChecked() )
      Settings::instance()->writeEntry(d->weatherGroup + "Interval", 
        d->weatherWidget->timerSpinBox->value());
    Settings::instance()->writeEntry(d->weatherGroup + "Metric", d->metric->isChecked());
  }
  else
  {
    qDebug("HomeConfigService::save() : main widget isn't available");
  }
}

  void
HomeConfigService::defaults()
{
  if( d->widget )
  {
    // RSS
    QStringList rssURL;
    rssURL += "http://slashdot.org/slashdot.rss";
    rssURL += "http://www.davidjanes.com/blogosphere/feeds/0000/000092.feed.rss";
    
    d->rssWidget->urlListBox->clear();
    if( !rssURL.isEmpty() )
    {
      d->rssWidget->urlListBox->insertStringList( rssURL );
    }
    
    d->rssWidget->timerCheckBox->setChecked( true );
    d->rssWidget->timerSpinBox->setValue( 30 );
    
    // Weather
    d->weatherWidget->urlListBox->clear();
    d->weatherWidget->timerCheckBox->setChecked( true );
    d->weatherWidget->timerSpinBox->setValue( 30 );
    d->metric->setChecked( true );
  }
  else
  {
    qDebug("HomeConfigService::defaults() : main widget isn't available");
  }
}

  QPixmap
HomeConfigService::icon()
{
  return QIconLoader::instance()->loadIcon( "Home_Settings.png" );
}

  QString
HomeConfigService::caption()
{
  return QObject::tr("Home");
}

  bool
HomeConfigService::modified()
{
  return false;
}

  void
HomeConfigService::setICAO(const QString & code)
{
  if( d->weatherWidget )
  {
    d->weatherWidget->urlLineEdit->setText(code);
  }
}

// vim:ts=2:sw=2:tw=78:noet
