/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          homeuiservice.cpp  -  the UI service from the Home plug in
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
 ***************************************************************************/
/*
Modified by Ilie to save user settings on June 03, 2003
*/
#include <homeuiservice.h>
#include "homeuihandler.h"

#include <rssviewer.h>
#include <weatherviewer.h>

// QT includes
#include <qiconset.h>
#include <qpixmap.h>
#include <qsplitter.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qiconloader.h>
#include <qmessagebox.h>
#include <qvaluelist.h>

//tksBase includes
#include <settings.h>
#include <qsettingsextra.h>

/*
static const char* homeuiservice_metainfo[][2]=
{
    { "/name", "Home UI Service" },
    { "/class", "HomeUIService" },
    { "/class/fqn", "HomeUIService" },
    { "/factory", "homeuiservice_factory" },
    { 0, 0 }
};
*/

kore::Interface* homeuiservice_factory()
{
  return new HomeUIService();
}
    
HomeUIService::HomeUIService()
  : leftWidget(0L),
    topWidget(0L),
    topBanner(0L),
    leftBanner(0L),
    reportsWidget(0L),
    rssViewer(0L),
    weatherViewer(0L),
    isRSSOn( true ),
    isWeatherOn( true ),
    splitterPossition( 200 )
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Home UI Service");
  info->replaceAll("class", "HomeUIService");
  info->replaceAll("class/fqn", "HomeUIService");
  info->replaceAll("factory", "homeuiservice_factory");
  
  pluginIcon.setPixmap
    (
      BarIcon(HOME_ON),
      QIconSet::Automatic,
      QIconSet::Active,
      QIconSet::Off
    );

  pluginIcon.setPixmap
    (
      BarIcon(HOME_OFF),
      QIconSet::Automatic,
      QIconSet::Normal,
      QIconSet::Off
    );

  _mimeType = VFS_HOME_MIMETYPE;

  
  readSettings();
}

HomeUIService::~HomeUIService()
{
  saveSettings();
}

bool HomeUIService::hasCapability(const QString& capabilityName)
{
  if
    (
          capabilityName == TOP_WIDGET
      ||  capabilityName == BANNER_WIDGET
      ||  capabilityName == LEFT_WIDGET
      ||  capabilityName == PLUGIN_ICONSET
    )
      return true;

  return KomUserInterfaceService::hasCapability(capabilityName);
}

void HomeUIService::createWidget(const QString& capabilityName)
{
  if( capabilityName==TOP_WIDGET )
  {
    if ( topWidget==NULL )
    {
      topWidget = new KomContainerWidget(0, "HomeTopWidget");
      
      reportsWidget = new QWidget( topWidget );
      topWidget->addChildWidget( reportsWidget );
      topWidget->showChild( reportsWidget );
      
      QVBoxLayout * reportsLayout = new QVBoxLayout(reportsWidget);
      reportsSplitter = new QSplitter(Qt::Vertical, reportsWidget);
      rssViewer = new RSSViewer(reportsWidget, "RSSViewer");
      weatherViewer = new WeatherViewer(reportsWidget, "WeatherViewer");
      
      rssViewer->widget(reportsSplitter, "RSSWidget");
      rssViewer->slotUpdate();
      weatherViewer->widget(reportsSplitter, "WeatherWidget");
      weatherViewer->slotUpdate();
      
      reportsLayout->addWidget(reportsSplitter);

      //restore las session settings
      if( !isRSSOn )
        rssViewer->hideWidget();
      if( !isWeatherOn )
        weatherViewer->hideWidget();
      QValueList<int> sizeList;
      
      // avoid math exception and set the splitter in the middle
      if( !splitterPossition )
        splitterPossition = 20;
      
      // '10 *' for a better aproximation
      int iTemp = reportsWidget->height() * 10 / splitterPossition;
      sizeList << iTemp << reportsWidget->height() - iTemp;
      reportsSplitter->setSizes(sizeList);
  
      widgetDict.insert( TOP_WIDGET, topWidget);
    }
  }
  if( capabilityName==LEFT_WIDGET )
  {
    if ( leftWidget==NULL )
    {
      leftWidget=new KomContainerWidget(0, "HomeLeftWidget");
      if ( leftBanner==NULL )
        leftBanner = new QLabel( leftWidget );
      leftWidget->addChildWidget( leftBanner );
      leftBanner->setPixmap( BarIcon(HOME_LEFT_BANNER) ) ;
      leftWidget->showChild( leftBanner );
      
      widgetDict.insert( LEFT_WIDGET, leftWidget);
    }
  }
  else if( capabilityName==BANNER_WIDGET )
  {
    if ( topBanner==NULL )
    {
      topBanner = new QLabel(0);
      topBanner->setPixmap( BarIcon(HOME_TOP_BANNER) ) ;

      // Add the banner widget to the plug in widgets dict.
      widgetDict.insert( BANNER_WIDGET, topBanner );
    }
  }
}

void HomeUIService::showToolbars(QMainWindow *mainWindow)
{
  if( toolBar==NULL )
  {
    HomeUIHandler* qtHandler = new HomeUIHandler( this );
    assert( qtHandler != 0L );

    toolBar = new QToolBar(mainWindow, "HomeToolBar");
    
    QAction *refreshAction;
    refreshAction = new QAction( QObject::tr("Reload"),
                          QIconLoader::instance()->iconSet( "Reload.png" ),
                          QObject::tr("&Reload"),
                          QKeySequence( /** may be F5 */ ),
                          (QObject*)topWidget );
    QObject::connect(refreshAction, SIGNAL(activated()), qtHandler, SLOT(slotUpdateRSS()));
    QObject::connect(refreshAction, SIGNAL(activated()), qtHandler, SLOT(slotUpdateWeather()));
    refreshAction->addTo(toolBar);

    QAction* toggleRSSAction = 0L;
    QIconSet iconSetRSS;
    iconSetRSS.setPixmap(BarIcon("rss_show.png"), QIconSet::Automatic, QIconSet::Normal, QIconSet::On);
    iconSetRSS.setPixmap(BarIcon("rss_hide.png"), QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
    
    toggleRSSAction = new QAction( QObject::tr("RSS Toggle"),
                          iconSetRSS,
                          QObject::tr("&RSS Toggle"),
                          QKeySequence( /** may be F5 */ ),
                          (QObject*)topWidget );
    toggleRSSAction->setToggleAction( true );
    toggleRSSAction->setOn( isRSSOn );
    QObject::connect( toggleRSSAction, SIGNAL(toggled(bool)),
                      qtHandler, SLOT(slotToggleRSS(bool))
                    );
    toggleRSSAction->addTo(toolBar);
    
    QAction* toggleWeatherAction = 0L;
    QIconSet iconSetWeather;
    iconSetWeather.setPixmap(BarIcon("weather_show.png"), QIconSet::Automatic, QIconSet::Normal, QIconSet::On);
    iconSetWeather.setPixmap(BarIcon("weather_hide.png"), QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
    
    toggleWeatherAction = new QAction( QObject::tr("Weather Toggle"),
                              iconSetWeather,
                              QObject::tr("&Weather Toggle"),
                              QKeySequence( /** may be F5 */ ),
                              (QObject*)topWidget );
    toggleWeatherAction->setToggleAction( true );
    toggleWeatherAction->setOn( isWeatherOn );
    QObject::connect( toggleWeatherAction, SIGNAL(toggled(bool)),
                      qtHandler, SLOT(slotToggleWeather(bool))
                    );
    toggleWeatherAction->addTo(toolBar);
  }
  else
  {
    toolBar->show();
  }
}

void HomeUIService::hideToolbars(QMainWindow *)
{
  if( toolBar!=NULL )
    toolBar->hide();
}

void HomeUIService::showMenus(QMenuBar *mainMenu)
{
  if( mainMenu!=NULL )
  {
    // TODO
    // Add the QPopupMenu-s and insertItem() here, with all associated QAcctions
  }
}

void HomeUIService::hideMenus(QMenuBar *mainMenu)
{
  if( mainMenu!=NULL )
  {
    // TODO
    // Clear all the menu entries
  }
}

QString HomeUIService::prettyName() const
{
  // XXX tr ?
  return "Home";
}

  void
HomeUIService::execAboutDialog(QWidget * parent) const
{
  QMessageBox::about
    (
      parent,
      QString("About plugin: %1").arg(prettyName()),
      QString("RSS and Weather Reports.\nIt uses the weather library from KDE.")
    );
}


void HomeUIService::readSettings()
{
  isRSSOn = Settings::instance()->readBoolEntry
            (Settings::instance()->name() + "/RSS/IsOn", isRSSOn);
  isWeatherOn = Settings::instance()->readBoolEntry
                (Settings::instance()->name() + "/Weather/IsOn", isWeatherOn);
  splitterPossition = QSettingsExtra::readUnsignedNumEntry
                      ( Settings::instance(),
                        Settings::instance()->name() + "/Home/SplitterPosstion",
                        splitterPossition
                      );
}


void HomeUIService::saveSettings()
{
  Settings::instance()->writeEntry(Settings::instance()->name() + "/RSS/IsOn", isRSSOn);
  Settings::instance()->writeEntry(Settings::instance()->name() + "/Weather/IsOn", isWeatherOn);
  if(isRSSOn && isWeatherOn)
  {
    QValueList<int> sizeList = reportsSplitter->sizes();
    
    // avoid math exception and set the splitter in the middle
    if( !sizeList[0] )
      sizeList[0] = reportsWidget->height() / 2;
    
    // '10 *' for a better aproximation
    splitterPossition = reportsWidget->height() * 10 / sizeList[0];
    QSettingsExtra::writeEntry
      (Settings::instance(), Settings::instance()->name() + "/Home/SplitterPosstion", splitterPossition  );
  }
}


void HomeUIService::slotToggleRSS( bool value )
{
  if( value )
  {
    rssViewer->showWidget();
  }
  else
  {
    rssViewer->hideWidget();
  }
  isRSSOn = value;
}


void HomeUIService::slotToggleWeather( bool value )
{
  if( value )
  {
    weatherViewer->showWidget();
  }
  else
  {
    weatherViewer->hideWidget();
  }
  isWeatherOn = value;
}

void HomeUIService::slotUpdateRSS()
{
  if( isRSSOn )
    rssViewer->slotUpdate();
}

void HomeUIService::slotUpdateWeather()
{
  if( isWeatherOn )
    weatherViewer->slotUpdate();
}

