/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          homeuiservice.cpp  -  the UI service from the Home plug in
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
 ***************************************************************************/
/*
Modified by Ilie to save user settings on June 03, 2003
*/
#ifndef HOMEUISERVICE_H
#define HOMEUISERVICE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <komaetheraplugin.h>

#include <qlabel.h>
#include <qsplitter.h>

#define HOME_TOP_BANNER         "aethera-banner-welcome.png"
#define HOME_LEFT_BANNER        "aethera-home.png"

#define HOME_ON                 "Home2_active.png"
#define HOME_OFF                "Home2_inactive.png"
#define HOME_TOP_BANNER_LEFT    "Home_Type.png"
#define HOME_TOP_BANNER_RIGHT   "Home_Banner3.png"

class RSSViewer;
class WeatherViewer;

extern "C"
{
    HOME_PLUGIN_API kore::Interface* homeuiservice_factory();
};

class HOME_PLUGIN_API HomeUIService : public KomUserInterfaceService
{
  public:
    HomeUIService();
    virtual ~HomeUIService();

    // From UIPart
    /** Query for capabilities.*/
    virtual bool hasCapability(const QString&);
    /** Create a new widget corresponding to the string.*/
    virtual void createWidget(const QString&);
    /** Show the part toolbars.*/
    virtual void showToolbars(QMainWindow *);
    /** Hide the part toolbars.*/
    virtual void hideToolbars(QMainWindow *);
    /** Show the part manus.*/
    virtual void showMenus(QMenuBar *);
    /** Hide the part menu.*/
    virtual void hideMenus(QMenuBar *);

    virtual void setCurrentURL(QString) {}

    virtual QString prettyName() const;
    
    virtual void execAboutDialog(QWidget * parent) const;

    void slotToggleRSS( bool );
    void slotToggleWeather( bool );
    void slotUpdateRSS();
    void slotUpdateWeather();

  protected:
    /** Left widget.*/
    KomContainerWidget *leftWidget;
    /** Top widget.*/
    KomContainerWidget *topWidget;
    /** The top banner widget.*/
    QLabel *topBanner;
    /** The left banner from the left widget.*/
    QLabel *leftBanner;
    /** The RSS and Weather widget.*/
    QWidget *reportsWidget;
    RSSViewer * rssViewer;
    WeatherViewer * weatherViewer;
    QSplitter* reportsSplitter;
    
  private:
    void readSettings();
    void saveSettings();
    
    bool isRSSOn;
    bool isWeatherOn;
    unsigned int splitterPossition;
};


#endif


