/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          icaodialog.h  -  description
                             -------------------
    begin                : Fri Jun 13 2003
    copyright            : (C) 2003 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#include "icaodialog.h"

#include <qlineedit.h>
#include <qlistview.h>
#include <qfile.h>

#include <dirs.h>

ICAODialog::ICAODialog(QWidget *parent, const char *name ) 
  : ICAODialogBase(parent,name)
{
  // remove the default info added by designer
  icaoListView->clear();
  icaoListView->removeColumn(0);
  icaoListView->setSelectionMode( QListView::NoSelection );
  
  // add the columns
  icaoListView->addColumn( tr("ICAO code"), 70 );
  icaoListView->addColumn( tr("Country"), 150 );
  icaoListView->addColumn( tr("Details"), 200 );
  
  icaoListView->setSorting(-1);
  init();
  icaoListView->setSorting(1, true);
  icaoListView->setShowSortIndicator(true);
  
  slotChangeSelection( icaoListView->firstChild() );
}

ICAODialog::~ICAODialog()
{
}

void ICAODialog::init()
{
  // fill the list view
  QFile locationsFile( Dirs::locate("data/nsd_cccc.txt") );
  
  if( locationsFile.open(IO_ReadOnly) )
  {
    QListViewItem * item = 0L;
    QString line;
    QTextStream stream( &locationsFile );
    int iFind1 = -1;
    int iFind2 = -1;
    
    while( !(line = stream.readLine()).isEmpty() )
    {
      QString code("NONE");
      QString country("UNKNOWN");
      QString details("NONE");
      QString temp;
      
      iFind1 = line.find(';');
      if(-1 != iFind1)
      {
        code = line.left(iFind1);
      }
      
      // pass over unused fields
      iFind1 = line.find(';', iFind1+1);
        
      // get the info
      iFind1 = line.find(';', iFind1+1);
      iFind2 = line.find(';', iFind1+1);
        
      if(-1 != iFind1 && -1 != iFind2)
      {
        temp = line.mid(iFind1+1, iFind2-iFind1-1);
        if( !temp.isEmpty() )
          details = temp;
          
        iFind1 = line.find(';', iFind2+1);
        iFind2 = line.find(';', iFind1+1);
        
        if(-1 != iFind1 && -1 != iFind2)
        {
          temp = line.mid(iFind1+1, iFind2-iFind1-1);
          if( !temp.isEmpty() )
          {
            country = temp;
          }
        }
      }
      
      // build the new entry
      item = new QListViewItem(icaoListView, code, country, details);
      if(!item)
      {
        qDebug("ICAODialog::init : cannot build a new list view item");
        break;
      }
    } // end of while
    
    locationsFile.close();
  }
  else
  {
    qDebug("ICAODialog::init : cannot open the data/nsd_cccc.txt file.");
  }
}

bool ICAODialog::isSelected()
{
  return (!selectedLineEdit->text().isEmpty());
}

QString ICAODialog::selected() const
{
  return selectedLineEdit->text();
}
    
