/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          rssviewer.cpp  -  description
                             -------------------
    begin                : Thu Jan 30 2003
    copyright            : (C) 2003 by theKompany.com
    email                : eug@thekompany.com
 ***************************************************************************/

#include "rssviewer.h"

#include <atextbrowser.h>
#include <settings.h>

//#include <qtextbrowser.h>
#include <qfile.h>
#include <qurl.h>
#include <qdom.h>

class RSSViewer::Private
{
  public:
  
    Private(RSSViewer *);
    ~Private();
    
    QTextBrowser * widget;
    QString buffer;
  
  private:
  
    RSSViewer * parent_;
};

RSSViewer::Private::Private(RSSViewer * parent)
  : widget(0),
    parent_(parent)
{
}

RSSViewer::Private::~Private()
{
}

RSSViewer::RSSViewer(QObject *parent, const char *name )
  : ReportViewer(parent,name)
{
  d = new Private(this);
  
  group = Settings::instance()->name() + "/RSS/";
  
  connect(&timer, SIGNAL(timeout()), this, SLOT(slotUpdate()));
  
  int timerValue = Settings::instance()->readNumEntry(group + "Interval", 30);
  bool bTimer = Settings::instance()->readBoolEntry(group + "Timer", true);
  
  // huh
  fileName = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  fileName += "/RSS.temp";
  
  if( bTimer )
    timer.start(timerValue * 60 * 1000, false);
  else
    timer.stop();
}


RSSViewer::~RSSViewer()
{
  delete d;
  d = 0;
}

QWidget * RSSViewer::widget(QWidget *parent, const char *name)
{
  if( !d->widget )
  {
    d->widget = new ATextBrowser(parent, name);
    
    disconnect( d->widget, SIGNAL(linkClicked(const QString&)) , 0, 0 );
    disconnect( d->widget, SIGNAL(linkClicked(const QString&)) ,
                d->widget, SLOT(setSource(const QString&)) );
    connect( d->widget, SIGNAL(linkClicked(const QString &)),
             this, SLOT(slotOpenLink(const QString &)));
    
    d->widget->setText("Waiting for data...");
  }
  
  return d->widget;
}

void RSSViewer::slotUpdate()
{
//  qDebug("RSSViewer::slotUpdate");
  
  ReportViewer::slotUpdate();
  
  urlList = Settings::instance()->readListEntry(group + "URL", '^');

  if( !urlList.isEmpty() )
    nextURL();
  else
    clear();
}

bool RSSViewer::parseData(QIODevice * dev)
{
  if( dev->open(IO_ReadOnly) )
  {
    QTextStream stream(dev);
    QString text( stream.read() );
    dev->close();
    
    if( text.isEmpty() )
    {
      return false;
    }
    
    QDomDocument doc( "RSS" );
    if( !doc.setContent( dev ) )
    {
      return false;
    }
  
    QDomElement docElem = doc.documentElement();
    
    d->buffer = QString::null;
    d->buffer += "<html><body>";
    
    QDomNode n = docElem.firstChild();
    QDomElement e;
    
    // get the title
    while( !n.isNull() )
    {
      // try to convert the node to an element.
      e = n.toElement();
        
      if( !e.isNull() && 
          "channel" == e.tagName().lower() )
      {
        QString a( QString("<h4>News <a href=\"%1\">%2</a></h4>") );
        a = a.arg(e.namedItem("link").toElement().text()).arg(e.namedItem("title").toElement().text());
        d->buffer += a;
        break;
      }
        
      n = n.nextSibling();
    }
      
    // old RSS format (ver < 1.0)
    if( "rss" == docElem.tagName().lower() )
    {
      n = e.firstChild();
    }
    
    // get the items
    d->buffer += "<ul>";
    while( !n.isNull() )
    {
      // try to convert the node to an element.
      e = n.toElement();
        
      if( !e.isNull() && 
          "item" == e.tagName().lower() )
      {
        QString a( "<li><a href=\"%1\">%2</a></li>" );
        a = a.arg(e.namedItem("link").toElement().text()).arg(e.namedItem("title").toElement().text());
        d->buffer += a;
      }
      n = n.nextSibling();
    }
    d->buffer +="</ul></body></html>";
  
    dev->close();
    return true;
  }
  else
  {
    d->buffer = QString::null;
    qDebug("RSSViewer::parseData : cannot open the IO device.");
  }
  
  return false;
}

bool RSSViewer::parseData(const QByteArray&)
{
  // TODO: it's not needed for the moment
  
  return false;
}

void RSSViewer::show()
{
  // clean the widget before showing the new data
  if( refresh )
  {
    refresh = false;
    clear();
  }
      
  d->widget->append( d->buffer );
}

void RSSViewer::hideWidget()
{
  d->widget->hide();
}

void RSSViewer::showWidget()
{
  d->widget->show();
}

void RSSViewer::clear()
{
  d->widget->clear();
}

void RSSViewer::nextURL()
{
  if( !urlList.isEmpty() )
  {
    qDebug("RSSViewer::nextURL : %s", urlList.first().latin1());
    
    QUrl url( urlList.first() );
    
    urlList.pop_front();
    
    fileDownload->download(url, fileName);
  }
  else
  {
    // remove the temporary file
    QFile tempFile(fileName);
    tempFile.remove();
  }
}

void RSSViewer::startTimer()
{
  int timerValue = Settings::instance()->readNumEntry(group + "Interval", 30);
  
  timer.start(timerValue * 60 * 1000, false);
  
  slotUpdate();
}

