/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
			weatherlib.cpp  -  description
			-------------------
begin                : Wed Jul 5 2000
copyright            : (C) 2000 by Ian Reinhart Geiser
email                : geiseri@msoe.edu
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

//
// Changed by Eugen C. <eug@thekompany.com> for Aethera
// 

#include "weatherlib.h"

#include <qfile.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <math.h>

class WeatherLib::Data
{
	public:
		Data()
		{
			theWeatherType = WeatherLib::NoWeatherType;
      theWeatherExtraType = WeatherLib::NoWeatherExtraType;
			clouds = 0;
			windMPH = 0;
			tempF = 0;
			qsCoverList.clear();
			qsCurrentList.clear();
			qsDate = QDate::currentDate();
			qsPressure = QString::null;
			qsTemperature = QString::null;
			qsDewPoint = QString::null;
			qsRelHumidity = QString::null;
			qsTime = QTime::currentTime();
			qsVisibility = QString::null;
			qsWindSpeed = QString::null;
			qsWindChill = QString::null;
			qsHeatIndex = QString::null;
			qsWindDirection = QString::null;
			reportLocation = QString::null;
			age = QDateTime::currentDateTime();
		};
		~Data(){};

		void clear();

		/** The current weather state outside */
		WeatherLib::WeatherType theWeatherType;
		WeatherLib::WeatherExtraType theWeatherExtraType;
    
		int clouds;
		int windMPH;
		float tempF;
		QStringList qsCoverList;
		QStringList qsCurrentList;
		QDate qsDate;
		QString qsPressure;
		QString qsTemperature;
		QString qsDewPoint;
		QString qsRelHumidity;
		QTime qsTime;
		QString qsVisibility;
		QString qsWindSpeed;
		QString qsWindChill;
		QString qsHeatIndex;
		QString qsWindDirection;
		QString reportLocation;
		QDateTime age;
		bool downloading;
		bool updated;
};

void WeatherLib::Data::clear()
{
	theWeatherType = WeatherLib::NoWeatherType;
  theWeatherExtraType = WeatherLib::NoWeatherExtraType;
	clouds = 0;
	windMPH = 0;
	tempF = 0;
	qsCoverList.clear();
	qsCurrentList.clear();
	qsDate = QDate::currentDate();
	qsPressure = "";
	qsTemperature = "";
	qsDewPoint = "";
	qsRelHumidity = "";
	qsTime = QTime::currentTime();
	qsVisibility = "";
	qsWindSpeed = "";
	qsWindChill = "";
	qsHeatIndex = "";
	qsWindDirection = "";
	age = QDateTime::currentDateTime();
}

WeatherLib::WeatherLib(QObject *parent, const char *name, bool bMetric)
  : QObject (parent, name),
    MetricMode (bMetric)
{
	CoverRegExp    = QRegExp("^(FEW|SCT|BKN|OVC|SKC|CLR|CAVOK)([0-9]{3})?(?:TCU|CB)?$");
	CurrentRegExp  = QRegExp("^(\\+|-|VC)?([A-Z]{2,4})$");
	WindRegExp     = QRegExp("^([0-9]{3}|VRB)([0-9]{2,3})(?:G([0-9]{2,3}))?(KT|KMH|MPS)$");
	// VisRegExp does not work with fractions...yet
	VisRegExp      = QRegExp("^([0-9]{1,2})SM$");
	TempRegExp     = QRegExp("^(M)?([0-9]{2})/(?:(M)?([0-9]{2}))?$");
	TimeRegExp     = QRegExp("^([0-9]{2}:[0-9]{2})$");
	DateRegExp     = QRegExp("^([0-9]{4}/[0-9]{2}/[0-9]{2})$");
	PressRegExp    = QRegExp("^([AQ])([0-9]{4})$");
  
  weatherData = new Data();
}

WeatherLib::~WeatherLib()
{
  if( weatherData )
  {
    delete weatherData;
    weatherData = 0L;
  }
}

/** Parse the current cover type */
bool WeatherLib::parseCover(QString s, Data *d){
	if (CoverRegExp.search(s) > -1)
	{
		QString sCode = CoverRegExp.cap(1);
		float fFeet = CoverRegExp.cap(2).toFloat();
		QString sClouds;
		QString skycondition;

		if ( MetricMode )
		{
			fFeet = fFeet * 0.3048;  // Check this!
		}
		
		if (fFeet >= 10)
		{
			fFeet /= 10;
			if ( MetricMode )
				sClouds = QString("%1 %2").arg(fFeet,0,'f',1).arg(trUtf8(" kilometers"));
			else
				sClouds = QString("%1 %2").arg(fFeet,0,'f',1).arg(trUtf8(" thousand feet"));
		}
		else
		{
			fFeet *= 100;
			if ( MetricMode )
				sClouds = QString("%1 %2").arg(fFeet,0,'f',0).arg(trUtf8("meters"));
			else
				sClouds = QString("%1 %2").arg(fFeet).arg(trUtf8("feet"));
		}

		if (sCode == "FEW")
		{
			skycondition = trUtf8( "Few clouds at %1." ).arg(sClouds);
			d->clouds += 2;
		}
		else if (sCode == "SCT")
		{
			skycondition = trUtf8( "Scattered clouds at %1." ).arg(sClouds);
			d->clouds += 4;
		}
		else if (sCode == "BKN")
		{
			skycondition = trUtf8( "Broken clouds at %1." ).arg(sClouds);
			d->clouds += 8;
		}
		else if (sCode == "OVC")
		{
			skycondition = trUtf8( "Overcast clouds at %1." ).arg(sClouds);
			d->clouds += 16;
		}
		else if ((sCode == "CLR") || (sCode == "SKC") || (sCode == "CAVOK"))
		{
			skycondition = trUtf8("Clear skies.");
		}

		d->qsCoverList << skycondition;

		return true;
	}
	return false;
}

/** Parse the current weather conditions */
bool WeatherLib::parseCurrent(QString s, Data *d){
	if (CurrentRegExp.search(s) > -1)
	{
		QString sIntensity = CurrentRegExp.cap(1);
		QString sCode = CurrentRegExp.cap(2);
		QString currentWeather = "";

		// Decode the intensity
		if (sIntensity == "+")
			currentWeather = trUtf8("Heavy");
		else if (sIntensity == "-")
			currentWeather = trUtf8("Light");

		// Decode the descriptor
		if (sCode.contains("MI"))
			currentWeather += trUtf8(" Shallow");
		else if (sCode.contains("PR"))
			currentWeather += trUtf8(" Partial");
		else if (sCode.contains("BC"))
			currentWeather += trUtf8(" Patches");
		else if (sCode.contains("DR"))
			currentWeather += trUtf8(" Low Drifting");
		else if (sCode.contains("BL"))
    {
			currentWeather += trUtf8(" Blowing");
      d->theWeatherExtraType = WeatherLib::Wind;
    }
		else if (sCode.contains("SH"))
		{
			currentWeather += trUtf8(" Showers");
			d->theWeatherType = WeatherLib::Shower;
		}
		else if (sCode.contains("TS"))
		{
			currentWeather += trUtf8(" Thunder Storm");
			d->theWeatherExtraType = WeatherLib::ThunderStorm;
		}
		else if (sCode.contains("FZ"))
		{
			currentWeather += trUtf8(" Freezing");
			d->theWeatherExtraType = WeatherLib::Freezing;
		}

		// Decode weather phenomena
		if (sCode.contains("DZ"))
		{
			currentWeather += trUtf8(" Drizzle");
			d->theWeatherType = WeatherLib::Shower1;
		}
		else if (sCode.contains("RA"))
		{
			currentWeather += trUtf8(" Rain");
			d->theWeatherType = WeatherLib::Shower;
		}
		else if (sCode.contains("SN"))
		{
			currentWeather += trUtf8(" Snow");
			d->theWeatherType = WeatherLib::Snow;
		}
		else if (sCode.contains("SG"))
		{
			currentWeather += trUtf8(" Snow Grains");
			d->theWeatherType = WeatherLib::Snow5;
		}
		else if (sCode.contains("IC"))
		{
			currentWeather += trUtf8(" Ice Crystals");
			d->theWeatherType = WeatherLib::Snow;
			d->theWeatherExtraType = WeatherLib::Freezing;
		}
		else if (sCode.contains("PE"))
		{
			currentWeather += trUtf8(" Ice Pellets");
			d->theWeatherType = WeatherLib::Shower;
			d->theWeatherExtraType = WeatherLib::Hail;
		}
		else if (s.contains("GR"))
		{
			currentWeather += trUtf8(" Hail");
			d->theWeatherType = WeatherLib::Shower;
			d->theWeatherExtraType = WeatherLib::Hail;
		}
		else if (sCode.contains("GS"))
		{
			currentWeather += trUtf8(" Small Hail Pellets");
			d->theWeatherType = WeatherLib::Shower;
			d->theWeatherExtraType = WeatherLib::Hail;
		}
		else if (s.contains("UP"))
		{
			currentWeather += trUtf8(" Unknown Precipitation");
			d->theWeatherType = WeatherLib::Shower1;
		}
		else if (sCode.contains("BR"))
		{
			currentWeather += trUtf8(" Mist");
			d->theWeatherExtraType = WeatherLib::Mist;
		}
		else if (sCode.contains("FG"))
		{
			currentWeather += trUtf8(" Fog");
			d->theWeatherExtraType = WeatherLib::Fog;
		}
		else if (sCode.contains("FU"))
			currentWeather += trUtf8(" Smoke");
		else if (sCode.contains("VA"))
			currentWeather += trUtf8(" Volcanic Ash");
		else if (sCode.contains("DU"))
			currentWeather += trUtf8(" Widespread Dust");
		else if (sCode.contains("SA"))
			currentWeather += trUtf8(" Sand");
		else if (sCode.contains("HZ"))
			currentWeather += trUtf8(" Haze");
		else if (sCode.contains("PY"))
			currentWeather += trUtf8(" Spray");
		else if (sCode.contains("PO"))
			currentWeather += trUtf8(" Dust/Sand Swirls");
		else if (sCode.contains("SQ"))
			currentWeather += trUtf8(" Sudden Winds");
		else if (sCode.contains("FC"))
		{
			if (sIntensity == "+")
      {
  			d->theWeatherExtraType = WeatherLib::Tornado;
				currentWeather = trUtf8("Tornado");
      }
			else
      {
  			d->theWeatherExtraType = WeatherLib::FunnelCloud;
				currentWeather += trUtf8(" Funnel Cloud");
      }
		}
		else if (sCode.contains("SS"))
    {
			d->theWeatherExtraType = WeatherLib::SandStorm;
			currentWeather += trUtf8(" Sand Storm");
    }
		else if (sCode.contains("DS"))
    {
			d->theWeatherExtraType = WeatherLib::DustStorm;
			currentWeather += trUtf8(" Dust Storm");
    }

		if (!currentWeather.isEmpty())
		d->qsCurrentList << currentWeather;

		return true;
		}
	return false;
}

/** Parse out the current temperature */
bool WeatherLib::parseTemperature(QString s, Data *d){
	if (TempRegExp.search(s) > -1)
	{
		QString unit;

		float fTemp = TempRegExp.cap(2).toFloat();
		if (TempRegExp.cap(1) == "M")
			fTemp *= -1;
		float fDew = TempRegExp.cap(4).toFloat();
		if (TempRegExp.cap(3) == "M")
			fDew *= -1;

		#define E(t) ::pow(10, 7.5*t/(237.7+t))
		float fRelHumidity = E(fDew)/E(fTemp) * 100;
		d->qsRelHumidity.setNum(fRelHumidity, 'f', 1);
		d->qsRelHumidity += "%";

		float fHeatIndex = 0;
		d->tempF = (fTemp*9/5)+32;
		if (d->tempF >= 80)
		{
			#define SQR(a)  ((a)*(a))
			fHeatIndex = -42.379 + (2.04901523*d->tempF) + (10.14333127*fRelHumidity)
				- (0.22475541*d->tempF*fRelHumidity) - (0.00683783*SQR(d->tempF))
				- (0.05481717*SQR(fRelHumidity)) + (0.00122874*SQR(d->tempF)*fRelHumidity)
				+ (0.00085282*d->tempF*SQR(fRelHumidity)) 
				- (0.00000199*SQR(d->tempF)*SQR(fRelHumidity));

			if ( fHeatIndex <= d->tempF )
				fHeatIndex = 0;
		}

		if ( MetricMode )
		{
			unit = trUtf8("°C");
			d->qsTemperature.setNum(fTemp);
			d->qsDewPoint.setNum(fDew);
			if (fHeatIndex >= 80)
				d->qsHeatIndex.setNum((fHeatIndex-32)*5/9, 'f', 1);
		}
		else
		{
			unit = trUtf8("°F");
			d->qsTemperature.setNum((fTemp*9/5) + 32);
			d->qsDewPoint.setNum((fDew*9/5) + 32);
			if (fHeatIndex >= 80)
				d->qsHeatIndex.setNum(fHeatIndex, 'f', 1);
		}
		d->qsTemperature += unit;
		d->qsDewPoint += unit;
		if (!d->qsHeatIndex.isEmpty())
			d->qsHeatIndex += unit;

		return true;
	}
	return false;
}

/** Parse out the current date. */
bool WeatherLib::parseDate(QString s, Data *d){
	if (DateRegExp.search(s) > -1)
	{
		QString dateString = DateRegExp.cap(1);
		QString day, month, year;

		day = dateString.mid(8,2);
		month = dateString.mid(5,2);
		year = dateString.mid(0,4);
		
		QDate theDate(year.toInt(), month.toInt(), day.toInt());

		
		d->qsDate = theDate;
		return true;
	}
	return false;
}

/** Parse out the current time. */
bool WeatherLib::parseTime(QString s, Data *d){
	if (TimeRegExp.search(s) > -1)
	{
		QString hour, minute, dateString;

		dateString = TimeRegExp.cap(1);
		hour = dateString.mid(0,2);
		minute = dateString.mid(3,2);
		QTime theTime(hour.toInt(), minute.toInt());

		d->qsTime = theTime;
		return true;
	}
	return false;
}

/** Parse out the current visibility */
bool WeatherLib::parseVisibility( QString s, Data *d){
	if (VisRegExp.search(s) > -1)
	{
		float fVisibility = VisRegExp.cap(1).toFloat();

		if ( MetricMode )
		{
			fVisibility *= 1.6;
			d->qsVisibility.setNum(fVisibility);
			d->qsVisibility += trUtf8("km");
		}
		else
		{
			d->qsVisibility.setNum(fVisibility);
			d->qsVisibility += trUtf8("m");
		}
		return true;
	}
	return false;
}

/** Parse out the current pressure. */
bool WeatherLib::parsePressure( QString s, Data *d){
	if (PressRegExp.search(s) > -1)
	{
		QString type = PressRegExp.cap(1);
		float fPressure = PressRegExp.cap(2).toFloat();

		if ( MetricMode )
		{
			if (type == "A")
				fPressure *= (33.8639/100);
			d->qsPressure.setNum(fPressure, 'f', 0);
			d->qsPressure += trUtf8(" hPa");
		}
		else
		{
			if (type == "Q")
				fPressure /= 33.8639;
			else
				fPressure /= 100;
			d->qsPressure.setNum(fPressure, 'f', 2);
			d->qsPressure += trUtf8("\" Hg");
		}
		return true;
	}
	return false;
}

/** Parse the wind speed */
bool WeatherLib::parseWindSpeed(QString s, Data *d)
{
	if (WindRegExp.search(s) > -1)
	{
		int iDirection = WindRegExp.cap(1).toInt();
		int iWindSpeed = WindRegExp.cap(2).toInt();
		int iGustSpeed = WindRegExp.cap(3).toInt();
		QString sWindUnit = WindRegExp.cap(4);

		if ( MetricMode )
		{
			if (sWindUnit == "KT")
			{
				iWindSpeed = (int) (iWindSpeed * 3.6/1.94);
				iGustSpeed = (int) (iGustSpeed * 3.6/1.94);
			}
			else if (sWindUnit == "MPS")
			{
				iWindSpeed = (int) (iWindSpeed * 3.6);
				iGustSpeed = (int) (iGustSpeed * 3.6);
			}
			d->windMPH = (int) (iWindSpeed/1.61);
			sWindUnit = trUtf8(" km/h");
			d->qsWindSpeed.setNum(iWindSpeed);
		}
		else
		{
			if (sWindUnit == "KT")
			{
				iWindSpeed = (int) (iWindSpeed * 2.24/1.94);
				iGustSpeed = (int) (iGustSpeed * 2.24/1.94);
			}
			else if (sWindUnit == "KMH")
			{
				iWindSpeed = (int) (iWindSpeed / 1.61);
				iGustSpeed = (int) (iGustSpeed / 1.61);
			}
			else if (sWindUnit == "MPS")
			{
				iWindSpeed = (int) (iWindSpeed * 2.24);
				iGustSpeed = (int) (iGustSpeed * 2.24);
			}
			d->windMPH = iWindSpeed;
			sWindUnit = trUtf8(" MPH");
			d->qsWindSpeed.setNum(iWindSpeed);
		}
		d->qsWindSpeed += sWindUnit;

		if (iGustSpeed > 0)
			d->qsCurrentList << trUtf8("Wind gusts up to %1 %2").arg(iGustSpeed).arg(sWindUnit);

		if ((WindRegExp.cap(1) != "VRB") && (iWindSpeed > 0))
		{
			if ((iDirection >=0 && iDirection <= 11) || ( iDirection >= 349 && iDirection <= 360))
				d->qsWindDirection = trUtf8("N");
			else if (iDirection >= 12 && iDirection <= 32)
				d->qsWindDirection = trUtf8("NNE");
			else if (iDirection >= 33 && iDirection <= 56)
				d->qsWindDirection = trUtf8("NE");
			else if (iDirection >= 57 && iDirection <= 78)
				d->qsWindDirection = trUtf8("ENE");
			else if (iDirection >= 79 && iDirection <= 101)
				d->qsWindDirection = trUtf8("E");
			else if (iDirection >= 102 && iDirection <= 123)
				d->qsWindDirection = trUtf8("ESE");
			else if (iDirection >= 124 && iDirection <= 146)
				d->qsWindDirection = trUtf8("SE");
			else if (iDirection >= 147 && iDirection <= 168)
				d->qsWindDirection = trUtf8("SSE");
			else if (iDirection >= 169 && iDirection <= 191)
				d->qsWindDirection = trUtf8("S");
			else if (iDirection >= 192 && iDirection <= 213)
				d->qsWindDirection = trUtf8("SSW");
			else if (iDirection >= 214 && iDirection <= 236)
				d->qsWindDirection = trUtf8("SW");
			else if (iDirection >= 237 && iDirection <= 258)
				d->qsWindDirection = trUtf8("WSW");
			else if (iDirection >= 259 && iDirection <= 281)
				d->qsWindDirection = trUtf8("W");
			else if (iDirection >= 282 && iDirection <= 303)
				d->qsWindDirection = trUtf8("WNW");
			else if (iDirection >= 304 && iDirection <= 326)
				d->qsWindDirection = trUtf8("NW");
			else if (iDirection >= 327 && iDirection <= 348)
				d->qsWindDirection = trUtf8("NNW");
		}
		return true;
	}
	return false;
}

void WeatherLib::calcWeatherType(Data *d){
  if ( WeatherLib::NoWeatherType == d->theWeatherType ||
       WeatherLib::Cloudy == d->theWeatherType )
  {
    if( WeatherLib::Cloudy != d->theWeatherType &&
        (WeatherLib::Fog == d->theWeatherExtraType ||
        WeatherLib::Mist == d->theWeatherExtraType) )
    {
      d->theWeatherType = WeatherLib::Cloudy;
    }
    
    if( d->clouds == 0 && WeatherLib::Cloudy != d->theWeatherType )
    {
      if( d->tempF == 0 && d->qsTemperature.isEmpty() )
        d->theWeatherType = WeatherLib::Sunny;
      // t < 0 C
      else if( d->tempF <= 32 )
        d->theWeatherType = WeatherLib::Sunny1;
      // 0 C < t < 15 C
      else if( d->tempF > 32 && d->tempF <= 59 )
        d->theWeatherType = WeatherLib::Sunny2;
      // 15 C < t < 25 C
      else if( d->tempF > 59 && d->tempF <= 77 )
        d->theWeatherType = WeatherLib::Sunny3;
      // 25 C < t < 35 C
      else if( d->tempF > 77 && d->tempF <= 95 )
        d->theWeatherType = WeatherLib::Sunny4;
      // 35 C < t
      else
        d->theWeatherType = WeatherLib::Sunny5;
    }
		else if (d->clouds >= 0 && d->clouds <= 2)
			d->theWeatherType = WeatherLib::Cloudy1;
		else if ( d->clouds > 2 && d->clouds <= 4)
			d->theWeatherType = WeatherLib::Cloudy2;
		else if ( d->clouds > 4 && d->clouds <= 8)
			d->theWeatherType = WeatherLib::Cloudy3;
		else if ( d->clouds > 8 && d->clouds <= 16)
			d->theWeatherType = WeatherLib::Cloudy4;
		else
			d->theWeatherType = WeatherLib::Cloudy5;
	}
	else if ( d->theWeatherType == WeatherLib::Shower ||
            d->theWeatherExtraType == WeatherLib::Hail ||
            d->theWeatherExtraType == WeatherLib::Tornado ||
            d->theWeatherExtraType == WeatherLib::FunnelCloud ||
            d->theWeatherExtraType == WeatherLib::ThunderStorm ||
            d->theWeatherExtraType == WeatherLib::Storm )
	{
		if (d->clouds >= 0 && d->clouds <= 2)
			d->theWeatherType = WeatherLib::Shower;
		else if (d->clouds > 2 && d->clouds <= 4)
			d->theWeatherType = WeatherLib::Shower1;
		else if ( d->clouds > 4  && d->clouds <= 8)
			d->theWeatherType = WeatherLib::Shower2;
		else if ( d->clouds > 8 && d->clouds <= 12)
			d->theWeatherType = WeatherLib::Shower3;
		else if ( d->clouds > 12 && d->clouds <= 16)
			d->theWeatherType = WeatherLib::Shower4;
		else
			d->theWeatherType = WeatherLib::Shower5;
    
    QStringList findLight = d->qsCurrentList.grep("Light Rain", false);
    if( !findLight.isEmpty() &&
        (d->theWeatherType == WeatherLib::Shower4 ||
        d->theWeatherType == WeatherLib::Shower5) )
      d->theWeatherType = WeatherLib::Shower3;
	}
	else if ( d->theWeatherType == WeatherLib::Snow ||
            d->theWeatherExtraType == WeatherLib::Freezing )
	{
		if (d->clouds >= 0 && d->clouds <= 2)
			d->theWeatherType = WeatherLib::Snow;
		else if (d->clouds > 2 && d->clouds <= 4)
			d->theWeatherType = WeatherLib::Snow1;
		else if ( d->clouds > 4 && d->clouds <= 8)
			d->theWeatherType = WeatherLib::Snow2;
		else if ( d->clouds > 8 && d->clouds <= 12)
			d->theWeatherType = WeatherLib::Snow3;
		else if ( d->clouds > 12 && d->clouds <= 16)
			d->theWeatherType = WeatherLib::Snow4;
		else
			d->theWeatherType = WeatherLib::Snow5;
   
    QStringList findLight = d->qsCurrentList.grep("Light Snow", false);
    if( !findLight.isEmpty() &&
        (d->theWeatherType == WeatherLib::Snow4 ||
        d->theWeatherType == WeatherLib::Snow5) )
      d->theWeatherType = WeatherLib::Snow3;
	}
  
  if( d->theWeatherExtraType == WeatherLib::Wind ||
      d->theWeatherExtraType == WeatherLib::NoWeatherExtraType )
  {
    if( d->windMPH == 0 && d->qsWindSpeed.isEmpty() )
      d->theWeatherExtraType = WeatherLib::Wind;
    else if( d->windMPH >= 0 && d->windMPH <= 6 )
      d->theWeatherExtraType = WeatherLib::Wind0;
    else if( d->windMPH > 6 && d->windMPH <= 15 )
      d->theWeatherExtraType = WeatherLib::Wind1;
    else if( d->windMPH > 15 && d->windMPH <= 27 )
      d->theWeatherExtraType = WeatherLib::Wind2;
    else if( d->windMPH > 27 && d->windMPH <= 39 )
      d->theWeatherExtraType = WeatherLib::Wind3;
    else if( d->windMPH > 39 && d->windMPH <= 52 )
      d->theWeatherExtraType = WeatherLib::Wind4;
    else if( d->windMPH > 52 )
      d->theWeatherExtraType = WeatherLib::Wind5;
  }
}

void WeatherLib::calcWindChill(Data *d){
	float windChill = 35.74 + (0.6215*d->tempF) - (35.75*::pow(d->windMPH, 0.16))
		+ (0.4275*d->tempF*::pow(d->windMPH, 0.16));

	if (windChill < 48)
	{
		if ( MetricMode )
		{
			d->qsWindChill.setNum((windChill-32)*5/9, 'f', 1);
			d->qsWindChill += trUtf8("°C");
		}
		else
		{
			d->qsWindChill.setNum(windChill, 'f', 1);
			d->qsWindChill += trUtf8("°F");
		}
	}
}

void WeatherLib::processData(QIODevice * device)
{
  if( device->open(IO_ReadOnly) )
  {
    QString text;
    QTextStream stream(device);
    stream >> text;
    
    processData(text);
    
    device->close();
  }
}

void WeatherLib::processData(const QString & metar)
{
  processData(metar, weatherData);
}

void WeatherLib::processData(QString metar, Data *d){
  d->clear();
  
  if(metar.isEmpty())
  {
    qDebug("WeatherLib::processData : the data file is empty");
    return;
  }

	// Split at whitespace into tokens
	QStringList dataList = QStringList::split(QRegExp("\\s+"), metar);
	bool found = false;

	for (QStringList::ConstIterator it = dataList.begin(); it != dataList.end(); ++it)
	{
		// Abort if ReMarK code
		if (*it == "RMK")
			break;

		// Don't try to parse the ICAO location code
		if ((!found) && (*it == d->reportLocation))
		{
			found = true;
			continue;
		}

		if (parseDate(*it, d))
			continue;
		if (parseTime(*it, d))
			continue;
		if (parseWindSpeed(*it, d))
			continue;
		if (parseVisibility(*it, d))
			continue;
		if (parseTemperature(*it, d))
			continue;
		if (parsePressure(*it, d))
			continue;
		if (parseCover(*it, d))
			continue;
		if (parseCurrent(*it, d))
			continue;
	}

	calcWindChill(d);
	calcWeatherType(d);
}

QString WeatherLib::temperature(){
	return weatherData->qsTemperature;
}

QString WeatherLib::pressure(){
	return weatherData->qsPressure;
}

QString WeatherLib::wind(){
	return (weatherData->qsWindSpeed + " " + weatherData->qsWindDirection);
}

/**  */
QString WeatherLib::dewPoint(){
	return weatherData->qsDewPoint;
}

QString WeatherLib::relHumidity(){
	return weatherData->qsRelHumidity;
}

QString WeatherLib::heatIndex(){
	return weatherData->qsHeatIndex;
}

QString WeatherLib::windChill(){
	return weatherData->qsWindChill;
}

WeatherLib::WeatherType WeatherLib::type()
{
  return weatherData->theWeatherType;
}

WeatherLib::WeatherExtraType WeatherLib::extraType()
{
  return weatherData->theWeatherExtraType;
}

QString WeatherLib::date(){
  QDateTime gmtDateTime(weatherData->qsDate, weatherData->qsTime);
  
  // convert it to local time
  int iSec = QDateTime::currentDateTime(Qt::UTC).secsTo( QDateTime::currentDateTime(Qt::LocalTime) );
  gmtDateTime = gmtDateTime.addSecs(iSec);
  
  return gmtDateTime.toString();
}

/** Returns the current cover */
QStringList WeatherLib::cover(){
	return weatherData->qsCoverList;
}

/** return the visibility */
QString WeatherLib::visibility(){
	return weatherData->qsVisibility;
}

/** return the weather text */
QStringList WeatherLib::weather(){
	return weatherData->qsCurrentList;
}

void WeatherLib::setMetric(bool bMetric)
{
  MetricMode = bMetric;
}
