/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          weatherviewer.cpp  -  description
                             -------------------
    begin                : Thu Jan 30 2003
    copyright            : (C) 2003 by theKompany.com
    email                : eug@thekompany.com
 ***************************************************************************/

#include "weatherviewer.h"
#include "weatherlib.h"

#include <settings.h>
#include <qiconloader.h>
#include <dirs.h>

#include <qtextbrowser.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qurl.h>
#include <qcolor.h>
#include <qmime.h>

class WeatherViewer::Private
{
  public:
  
    Private(WeatherViewer *);
    ~Private();
    
    QTextBrowser * widget;
    WeatherLib * parser;
    QString buffer;
  
  private:
  
    WeatherViewer * parent_;
};

WeatherViewer::Private::Private(WeatherViewer * parent)
  : widget(0L),
    parser(0L),
    parent_(parent)
{
}

WeatherViewer::Private::~Private()
{
}

WeatherViewer::WeatherViewer(QObject *parent, const char *name )
  : ReportViewer(parent,name)
{
  d = new Private(this);
  
  group = Settings::instance()->name() + "/Weather/";
  
  bool bMetric = Settings::instance()->readBoolEntry(group + "Metric", true);
  d->parser = new WeatherLib(this, "WeatherParser", bMetric);
  
  connect(&timer, SIGNAL(timeout()), this, SLOT(slotUpdate()));
  
  int timerValue = Settings::instance()->readNumEntry(group + "Interval", 30);
  bool bTimer = Settings::instance()->readBoolEntry(group + "Timer", true);
  
  // huh
  fileName = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  fileName += "/Weather.temp";
  
  if( bTimer )
    timer.start(timerValue * 60 * 1000, false);
  else
    timer.stop();
}

WeatherViewer::~WeatherViewer()
{
  if(d)
  {
    delete d;
    d = 0L;
  }
}

QWidget * WeatherViewer::widget(QWidget *parent, const char *name)
{
  if( !d->widget )
  {
    d->widget = new QTextBrowser(parent, name);
    
    disconnect( d->widget, SIGNAL(linkClicked(const QString&)) , 0 , 0 );
    connect( d->widget, SIGNAL(linkClicked(const QString &)),
             this, SLOT(slotOpenLink(const QString &)));
    
    d->widget->setText("Waiting for data...");
  }
  
  return d->widget;
}

void WeatherViewer::slotUpdate()
{
  ReportViewer::slotUpdate();
  
  bool bMetric = Settings::instance()->readBoolEntry(group + "Metric", true);
  d->parser->setMetric( bMetric );
  
  urlList = Settings::instance()->readListEntry(group + "URL", '^');

  if( !urlList.isEmpty() )
    nextURL();
  else
    clear();}

bool WeatherViewer::parseData(QIODevice * dev)
{
  if( dev->open(IO_ReadOnly) )
  {
    d->buffer = QString::null;
    
    QTextStream stream(dev);
    QString text( stream.read() );
    dev->close();
    
    if( !text.isEmpty() )
    {
      d->parser->processData(text);
      
      // Title 
      QString locationCode = currentURL.mid(currentURL.findRev('/') + 1);
      locationCode = locationCode.left( locationCode.findRev('.') );
      locationCode = findLocation( locationCode );
      
      WeatherLib::WeatherType type = d->parser->type();
      WeatherLib::WeatherExtraType extraType = d->parser->extraType();
      
      QString weatherIcon;
      switch( type )
      {
        case WeatherLib::Sunny:
          weatherIcon = "Sunny.png";
          break;
        case WeatherLib::Sunny1:
          weatherIcon = "Sunny1.png";
          break;
        case WeatherLib::Sunny2:
          weatherIcon = "Sunny2.png";
          break;
        case WeatherLib::Sunny3:
          weatherIcon = "Sunny3.png";
          break;
        case WeatherLib::Sunny4:
          weatherIcon = "Sunny4.png";
          break;
        case WeatherLib::Sunny5:
          weatherIcon = "Sunny5.png";
          break;
        
        case WeatherLib::Cloudy:
          weatherIcon = "Cloudy.png";
          break;
        case WeatherLib::Cloudy1:
          weatherIcon = "Cloudy1.png";
          break;
        case WeatherLib::Cloudy2:
          weatherIcon = "Cloudy2.png";
          break;
        case WeatherLib::Cloudy3:
          weatherIcon = "Cloudy3.png";
          break;
        case WeatherLib::Cloudy4:
          weatherIcon = "Cloudy4.png";
          break;
        case WeatherLib::Cloudy5:
          weatherIcon = "Cloudy5.png";
          break;
        
        case WeatherLib::Shower:
          weatherIcon = "Shower.png";
          break;
        case WeatherLib::Shower1:
          weatherIcon = "Shower1.png";
          break;
        case WeatherLib::Shower2:
          weatherIcon = "Shower2.png";
          break;
        case WeatherLib::Shower3:
          weatherIcon = "Shower3.png";
          break;
        case WeatherLib::Shower4:
          weatherIcon = "Shower4.png";
          break;
        case WeatherLib::Shower5:
          weatherIcon = "Shower5.png";
          break;
        
        case WeatherLib::Snow:
          weatherIcon = "Snow.png";
          break;
        case WeatherLib::Snow1:
          weatherIcon = "Snow1.png";
          break;
        case WeatherLib::Snow2:
          weatherIcon = "Snow2.png";
          break;
        case WeatherLib::Snow3:
          weatherIcon = "Snow3.png";
          break;
        case WeatherLib::Snow4:
          weatherIcon = "Snow4.png";
          break;
        case WeatherLib::Snow5:
          weatherIcon = "Snow5.png";
          break;
        
        default:
          break;
      }
      
      QString extraIcon;
      switch( extraType )
      {
        case WeatherLib::Wind:
          extraIcon = "Wind.png";
          break;
        case WeatherLib::Wind0:
          extraIcon = "Wind0.png";
          break;
        case WeatherLib::Wind1:
          extraIcon = "Wind1.png";
          break;
        case WeatherLib::Wind2:
          extraIcon = "Wind2.png";
          break;
        case WeatherLib::Wind3:
          extraIcon = "Wind3.png";
          break;
        case WeatherLib::Wind4:
          extraIcon = "Wind4.png";
          break;
        case WeatherLib::Wind5:
          extraIcon = "Wind5.png";
          break;
        
        case WeatherLib::Storm:
          extraIcon = "Storm.png";
          break;
        case WeatherLib::ThunderStorm:
          extraIcon = "ThunderStorm.png";
          break;
        case WeatherLib::Tornado:
          extraIcon = "Tornado.png";
          break;
        case WeatherLib::FunnelCloud:
          extraIcon = "FunnelCloud.png";
          break;
        case WeatherLib::SandStorm:
          extraIcon = "SandStorm.png";
          break;
        case WeatherLib::DustStorm:
          extraIcon = "DustStorm.png";
          break;
        case WeatherLib::Fog:
          extraIcon = "Fog.png";
          break;
        case WeatherLib::Mist:
          extraIcon = "Mist.png";
          break;
        case WeatherLib::Freezing:
          extraIcon = "Freezing.png";
          break;
        case WeatherLib::Hail:
          weatherIcon = "Hail.png";
          break;
        default:
          break;
      }
      
#ifdef _QTWIN_
      if( !weatherIcon.isEmpty() )
      {
      	QPixmap weatherImage = QIconLoader::instance()->barIcon(weatherIcon);
        weatherIcon = "weather_" + weatherIcon;
        QMimeSourceFactory::defaultFactory()->setPixmap(weatherIcon, weatherImage);
      }
      if( !extraIcon.isEmpty() )
      {
      	QPixmap extraImage = QIconLoader::instance()->barIcon(extraIcon);
        extraIcon = "weather_" + extraIcon;
        QMimeSourceFactory::defaultFactory()->setPixmap(extraIcon, extraImage);
      }
#else
      if( !weatherIcon.isEmpty() )
        weatherIcon = QIconLoader::instance()->iconPath( weatherIcon );
      if( !extraIcon.isEmpty() )
        extraIcon = QIconLoader::instance()->iconPath( extraIcon );
#endif
    
      QStringList cover = d->parser->cover();
      QStringList weather = d->parser->weather();
      
      QString weatherText = "<ul>";
      {
      for (QStringList::const_iterator it = cover.begin(); it != cover.end(); ++it)
          weatherText += "<li nowrap=\"nowrap\" >" + *it + "</li>";
      }
  
      {
      for (QStringList::const_iterator it = weather.begin(); it != weather.end(); ++it)
          weatherText += "<li nowrap=\"nowrap\" >" + *it + "</li>";
      }
  
      weatherText += "</ul>";
  
      d->buffer = "<h4><b>" + trUtf8( "Weather Report " )+ locationCode +
      QString( trUtf8( "  ---  Last updated on %1</b></h4>" ) ).arg(d->parser->date()) +
      "<table><tr>";
      
      if( !weatherIcon.isEmpty() )
      {
        d->buffer += QString("<td><img width=\"32\" height=\"32\" src=\"%1\" /></td>").arg(weatherIcon);
      }
      if( !extraIcon.isEmpty() )
      {
        d->buffer += QString("<td><img width=\"32\" height=\"32\" src=\"%1\" /></td>").arg(extraIcon);
      }
      d->buffer += QString( "<td style=\"vertical-align: top\">%1</td></tr></table>" ).arg(weatherText);
      
      d->buffer += "<table><tr>" +
      QString("<th nowrap=\"nowrap\" style=\"text-align: right\">" + trUtf8( "Temperature:" ) + "</th><td>%1</td>"
      "<th nowrap=\"nowrap\" style=\"text-align: right\">" + trUtf8( "Dew Point:" ) + "</th><td>%2</td>"
      "<th nowrap=\"nowrap\" style=\"text-align: right\">" + trUtf8( "Air Pressure:" ) + "</th><td>%3</td>"
      "<th nowrap=\"nowrap\" style=\"text-align: right\">" + trUtf8( "Rel. Humidity:" ) + "</th><td>%4</td>"
      "<th nowrap=\"nowrap\" style=\"text-align: right\">" + trUtf8( "Wind Speed:" ) + "</th><td>%5</td>")
      .arg(d->parser->temperature()).arg(d->parser->dewPoint()).arg(d->parser->pressure())
      .arg(d->parser->relHumidity()).arg(d->parser->wind());
      
      if (!d->parser->heatIndex().isEmpty())
          d->buffer += QString("<th nowrap=\"nowrap\" style=\"text-align: right\">" + trUtf8( "Heat Index:" ) +
          "</th><td>%1</td>").arg(d->parser->heatIndex());
      else if (!d->parser->windChill().isEmpty())
          d->buffer += QString("<th nowrap=\"nowrap\" style=\"text-align: right\">" + trUtf8( "Wind Chill:" ) +
          "</th><td>%1</td>").arg(d->parser->windChill());
      else
          d->buffer += "<td >&nbsp;</td><td>&nbsp;</td>";
      d->buffer += "</tr></table>"/*</body></html>*/;
    
      return true;
    }
    else
    {
      qDebug("WeatherViewer::parseData : no data available");
      dev->close();
    }
  }
  else
  {
    d->buffer = QString::null;
    qDebug("RSSViewer::parseData : cannot open the IO device.");
  }
  
  return false;
}

bool WeatherViewer::parseData(const QByteArray&)
{
  // TODO: it's not needed for the moment
  
  return false;
}

void WeatherViewer::show()
{
  // clean the widget before showing the new data
  if( refresh )
  {
    refresh = false;
    clear();
  }
      
  d->widget->append( d->buffer );
}

void WeatherViewer::hideWidget()
{
  d->widget->hide();
}

void WeatherViewer::showWidget()
{
  d->widget->show();
}

void WeatherViewer::clear()
{
  d->widget->clear();
}

void WeatherViewer::nextURL()
{
  if( !urlList.isEmpty() )
  {
    QString next = urlList.first();
    
    if( -1 == next.find('/') )
    {
      next = next.mid( next.findRev('/') + 1 );
      
      if( -1 == next.find('.') )
        next = "http://weather.noaa.gov/pub/data/observations/metar/stations/" + next.upper() + ".TXT";
      else
        next = "http://weather.noaa.gov/pub/data/observations/metar/stations/" + next.upper();
    }
    
    QUrl url(next);
    
    urlList.pop_front();
    
    qDebug("WeatherViewer: URL=%s", url.toString().latin1());
    
    fileDownload->download(url, fileName);
  }
  else
  {
    // remove the temporary file
    QFile tempFile(fileName);
    tempFile.remove();
  }
}

void WeatherViewer::startTimer()
{
  int timerValue = Settings::instance()->readNumEntry(group + "Interval", 30);
  
  timer.start(timerValue * 60 * 1000, false);
  
  slotUpdate();
}

QString WeatherViewer::findLocation(const QString& locationCode)
{
  QFile locationsFile( Dirs::locate("data/nsd_cccc.txt") );
  QString location( locationCode );
  
  if( locationsFile.open(IO_ReadOnly) )
  {
    QString line;
    QTextStream stream( &locationsFile );
    int iFind1 = -1;
    int iFind2 = -1;
    
    while( !(line = stream.readLine()).isEmpty() )
    {
      iFind1 = line.find( locationCode.upper() );
      if( -1 != iFind1 )
      {
        // pass over unused fields
        iFind1 = line.find(';', iFind1+1);
        iFind1 = line.find(';', iFind1+1);
        
        // get the info
        iFind1 = line.find(';', iFind1+1);
        iFind2 = line.find(';', iFind1+1);
        
        if(-1 != iFind1 && -1 != iFind2)
        {
          QString city = line.mid(iFind1+1, iFind2-iFind1-1);
          if( !city.isEmpty() )
          {
            location = city + " (" + locationCode + ") ";
          }
          
          iFind1 = line.find(';', iFind2+1);
          iFind2 = line.find(';', iFind1+1);
        
          if(-1 != iFind1 && -1 != iFind2)
          {
            QString country = line.mid(iFind1+1, iFind2-iFind1-1);
            if( !country.isEmpty() )
            {
              location += country;
            }
          }
        }
        
        break;
      }
    }
    
    locationsFile.close();
  }
  else
  {
    qDebug("WeatherViewer::findLocation : cannot open the data/nsd_cccc.txt file.");
  }
  
//  QFile locationsFile( Dirs::locate("data/nsd_bbsss.txt") );
//  QString location( locationCode );
//  
//  if( locationsFile.open(IO_ReadOnly) )
//  {
//    QString line;
//    QTextStream stream( &locationsFile );
//    int iFind1 = -1;
//    int iFind2 = -1;
//    
//    while( !(line = stream.readLine()).isEmpty() )
//    {
//      iFind1 = line.find( locationCode.upper() );
//      if( -1 != iFind1 )
//      {
//        iFind1 = line.find(';', iFind1+1);
//        iFind2 = line.find(';', iFind1+1);
//        
//        if(-1 != iFind1 && -1 != iFind2)
//        {
//          QString city = line.mid(iFind1+1, iFind2-iFind1-1);
//          if( !city.isEmpty() )
//          {
//            location = city + " (" + locationCode + ") ";
//          }
//          
//          iFind1 = line.find(';', iFind2+1);
//          iFind2 = line.find(';', iFind1+1);
//        
//          if(-1 != iFind1 && -1 != iFind2)
//          {
//            QString country = line.mid(iFind1+1, iFind2-iFind1-1);
//            if( !country.isEmpty() )
//            {
//              location += country;
//            }
//          }
//        }
//        
//        break;
//      }
//    }
//    
//    locationsFile.close();
//  }
//  else
//  {
//    qDebug("WeatherViewer::findLocation : cannot open the data/nsd_bbsss.txt file.");
//  }
  
  return location;
}
