/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <addressclass.h>
#include <addressbookview.h>
#include <ataddressbook.h>
#include "contactsearchservice.h"
#include <vfsmodule.h>

#include <qiconloader.h>
#include <qstringlist.h>
#include <qpixmap.h>
#include <qcstring.h>
#include <qstring.h>
#include <qheader.h>

#define IDSTRING "AddressBookView: "

//#define DEBUG_ABW

class AddressBookView::Private
{
  public:
  
    Private();
    ~Private();
    
    QString getKey(VContactObject*);
    
    int iMode;
    
  private:
};

AddressBookView::Private::Private()
  : iMode( AddressBookViewInterface::Contacts )
{
}

AddressBookView::Private::~Private()
{
}

  QString
AddressBookView::Private::getKey(VContactObject * contact)
{
  if(iMode & AddressBook::Contacts)
  {
    return QString( contact->n()[0] + " " + contact->n()[1] +
                    " <" + contact->emails()[0] + ">" );
  }
  else if(iMode & AddressBookViewInterface::SIP)
  {
    return QString( contact->n()[0] + " " + contact->n()[1] +
                    " <" + contact->phone("sip") + ">" );
  }

  return QString::null;
}

AddressBookViewInterface::AddressBookViewInterface( QWidget *parent, const char *name)
  : QListView(parent, name)
{
}

AddressBookView::AddressBookView( QWidget *parent, const char *name)
  : AddressBookViewInterface(parent, name)
{
  d = new Private();
  
  setSelectionMode(QListView::Extended);
  setShowSortIndicator(true);
  setRootIsDecorated(true);
  setAllColumnsShowFocus(true);
}

AddressBookView::~AddressBookView()
{
  delete d;
  d = 0;
}

void AddressBookView::initMode(int mode, QString match, bool showMatched)
{
  AddressBook *instance = AddressBook::thisInstance();
  
  QStringList list;
  bool _show=true;
  int iColumn = 0;
  
  d->iMode = mode;
  
  VFSModule* vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
    else
      qDebug("ContactSearchService: Can't find VFS module");
  }
  Q_CHECK_PTR(vfs);
    
  // UI - showing the header..?
  if(mode & AddressBook::ShowHeader)
    header()->show();
  else
    header()->hide();
  // UI - as list? as tree?
  setRootIsDecorated(!(mode & AddressBook::ListOnly));
  if(mode & AddressBook::Groups)
  {
    iColumn = addColumn("Names", 200);
    setColumnWidthMode(iColumn, QListView::Manual);
    iColumn = addColumn("E-mail");
    setColumnWidthMode(iColumn, QListView::Maximum);
    setSorting(0);
  
    list=instance->completeNames("", AddressBook::Groups);
    for(uint i=0;i<list.count();i++)
    {
      if(match==QString::null)
      {
        _show=true;
      }
      else
      {
        if (showMatched)
          if(isMatched(instance->getEntry(list[i]),match))
            _show=true;
          else
            _show=false;
        else
          if(!isMatched(instance->getEntry(list[i]),match))
            _show=true;
          else
            _show=false;
      }
      if(_show)
      {
        QListViewItem *item = new QListViewItem(this);
        //QCheckListItem *item=new QCheckListItem(this, "test");
        
        if( item )
        {
          item->setText(0,list[i]);
          item->setPixmap(0,BarIcon("Groups"));
          item->setText(2,instance->getEntry(list[i]).join(","));
          insertItem(item);
          QStringList members=instance->getEntry(list[i]);
          if(members.count()==0 || (mode & AddressBook::ListOnly))
            item->setExpandable(false);
          else
            item->setExpandable(true);
          for(uint i=0;i<members.count() && (!(mode & AddressBook::ListOnly));i++)
          {
            QListViewItem *subItem = new QListViewItem(item);
            AddressClass *ac = new AddressClass( members[i] );
            
            if(ac && ac->getTranslated_name().isEmpty())
            {
              delete subItem;
              subItem = 0;
            }
            else
            {
              if( subItem )
              {
                subItem->setText(0, ac->getTranslated_name() );
                subItem->setText(1, QString(ac->email) );
                subItem->setText(2, members[i]);
                subItem->setPixmap(0, BarIcon("Contacts"));
              }
            }
            
            delete ac;
            ac = 0;
          }
        }
      }
    }
  }
  if(mode & AddressBook::Contacts)
  {
#if 0
    iColumn = addColumn("Names", 200);
    setColumnWidthMode(iColumn, QListView::Manual);
    iColumn = addColumn("E-mail");
    setColumnWidthMode(iColumn, QListView::Maximum);
    setSorting(0);
  
    // here is a list containing all the unique names...
//    QStringList uniq;
//    QString elem;
    list=instance->completeNames("", AddressBook::Contacts);
//    for(QStringList::ConstIterator it=list.begin(); it!=list.end(); ++it)
//    {
//      elem = instance->getDefaultName((*it));
//      if( -1 == uniq.findIndex(elem) )
//        uniq.append(elem);
//    }
//    
//    list = uniq;
    
    list=instance->getEntries(list);
    
  {
    for(QStringList::ConstIterator it=list.begin(); it!=list.end(); ++it)
    {
      QListViewItem *item=new QListViewItem(this);
      QStringList l=QStringList::split(',',(*it));
      
      if(l.count()<=1)
        item->setExpandable(false);
      else
        item->setExpandable(true);
      
      AddressClass *ac=new AddressClass(l[0]);
      if(ac->getTranslated_name().simplifyWhiteSpace().isEmpty())
        delete item;
      else
      {
        item->setText(0, ac->getTranslated_name());
        item->setText(1, QString(ac->email) );
        item->setText(2, l[0]);
        item->setPixmap(0,BarIcon("Contacts"));
      }
      delete ac;
      
      if(l.count()>1)
        for(uint j=1;j<l.count();j++)
        {
          QListViewItem *subItem=new QListViewItem(item);
          AddressClass *ac=new AddressClass( l[j] );
          if(ac->getTranslated_name().isEmpty())
            delete subItem;
          else
          {
            subItem->setText(0, ac->getTranslated_name() );
            subItem->setText(1, QString(ac->email) );
            subItem->setText(2, l[j]);
            subItem->setPixmap(0,BarIcon("Contacts"));
          }
          delete ac;
        }
    }
  }
#endif
    iColumn = addColumn("Names", 200);
    setColumnWidthMode(iColumn, QListView::Manual);
    iColumn = addColumn("E-mail");
    setColumnWidthMode(iColumn, QListView::Maximum);
    setSorting(0);
  
    QStringList contactLst = ContactSearchService::instance()->getAllContactsList();
    QString key;
    QDict<QListViewItem> index;
    QListViewItem * item = 0;
    for(QStringList::Iterator it=contactLst.begin(); it!=contactLst.end(); ++it)
    {
      VContactObject* contact = static_cast<VContactObject*>
        (vfs->_fact_ref((*it), VFS_CONTACTS_MIMETYPE ));
      if( contact )
      {
        if( !contact->emails().isEmpty() && !(contact->emails()[0]).isEmpty() )
        {
          key = d->getKey(contact);
          if( !index[key] )
          {
            item = new QListViewItem(this);
            if(item)
            {
              index.insert(key, item);
              
              item->setText(0, contact->n()[0] + " " + contact->n()[1] );
              item->setText(1, contact->emails()[0]);
              item->setText(2, key);
            }
          }
        }
      }
    }
/*    ContactSearchService::instance()->freeContacts();*/
  }

  if(mode & AddressBook::History)
  {
    iColumn = addColumn("Names", 200);
    setColumnWidthMode(iColumn, QListView::Manual);
    iColumn = addColumn("E-mail");
    setColumnWidthMode(iColumn, QListView::Maximum);
    setSorting(0);
  
    list=instance->completeNames("", AddressBook::History);
#ifdef DEBUG_ABW
    printf("History: %d\n", list.count());
#endif
    for(uint i=0;i<list.count();i++)
    {
      QListViewItem *item = new QListViewItem(this);
      item->setText(0,list[i]);
      item->setPixmap(0,BarIcon("Contacts"));
      insertItem(item);
    }
  }
  if(mode & AddressBookViewInterface::SIP)
  {
    iColumn = addColumn("Names", 200);
    setColumnWidthMode(iColumn, QListView::Manual);
    iColumn = addColumn("SIP");
    setColumnWidthMode(iColumn, QListView::Maximum);
    setSorting(0);
  
    QStringList contactLst = ContactSearchService::instance()->getAllContactsList();
    QDict<QListViewItem> index;
    QString key;
    QListViewItem * item = 0;
    for(QStringList::Iterator it=contactLst.begin(); it!=contactLst.end(); ++it )
    {
      VContactObject* contact = static_cast<VContactObject*>
        (vfs->_fact_ref((*it), VFS_CONTACTS_MIMETYPE ));
      if( contact )
      {
        if( !contact->phone("sip").isEmpty() )
        {
          key = d->getKey(contact);
          if( !index[key] )
          {
            item = new QListViewItem(this);
            if(item)
            {
              index.insert(key, item);
              
              item->setText(0, contact->n()[0] + " " + contact->n()[1] );
              item->setText(1, contact->phone("sip"));
            }
          }
        }
      }
    }
/*    ContactSearchService::instance()->freeContacts();*/
  }
}

void AddressBookView::getSelectedEmailList(QStringList &list)
{
  list.clear();
  QListViewItemIterator it(this);
  for ( ; it.current(); ++it )
  {
    if (it.current()->isSelected() && !it.current()->text(0).isEmpty())
    {
      list+=QStringList::split(',',it.current()->text(2) );
    }
  }
}

bool AddressBookView::isMatched(QStringList l, QString check)
{
  AddressClass *checkAc = new AddressClass( check.lower() );
  bool result=false;
  for(uint i=0;i<l.count();i++)
  {
    AddressClass *ac= new AddressClass( QCString(l[i]) );
    result =  ac->getTranslated_name().lower()==checkAc->getTranslated_name().lower() ||
              ac->email.lower()==QCString(checkAc->getTranslated_name()).lower();
    delete ac;
    if(result)
    {
      delete checkAc;
      return result;
    }
  }
  delete checkAc;
  return result;
}

/*
static const char* addressbookviewfactory_metainfo[][2]=
{
    { "/name", "Addressbook View Factory" },
    { "/class", "AddressBookViewFactory" },
    { "/class/fqn", "AddressBookViewFactory" },
    { "/factory", "addressbookviewfactory_factory" },
    { "/description", AETHERA_ADDRESSBOOK_VIEW_FACTORY_DESC },
    { "/service", AETHERA_ADDRESSBOOK_VIEW_FACTORY },
    { 0, 0 }
};
*/

kore::Interface* addressbookviewfactory_factory()
{
  return new AddressBookViewFactory();
}

AddressBookViewFactory::AddressBookViewFactory()
{
  // set the meta information
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Addressbook View Factory");
  info->replaceAll("class", "AddressBookViewFactory");
  info->replaceAll("class/fqn", "AddressBookViewFactory");
  info->replaceAll("factory", "addressbookviewfactory_factory");
  info->replaceAll("description", AETHERA_ADDRESSBOOK_VIEW_FACTORY_DESC);
  info->insert("service", AETHERA_ADDRESSBOOK_VIEW_FACTORY);
}

AddressBookViewFactory::~AddressBookViewFactory()
{
}

QWidget* AddressBookViewFactory::newWidget(QWidget *parent, const char *name, QWidget::WFlags)
{
  AddressBookView *newAddressBookView = new AddressBookView(parent, name);
  widgetsList.append(newAddressBookView);
  return newAddressBookView;
}

