/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <qstring.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qcache.h>
#include <qdict.h>
#include <qsettings.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qptrlist.h>

#include <stdio.h>

#include <ataddressbook.h>
#include <addressclass.h>
#include <vfolder.h>
#include <vobject.h>
#include <vcontactobject.h>
#include <groupobj.h>
#include <qsettings.h>
#include <vfsmimetype.h>

#define IDSTRING "AddressBook: "

//#define DEBUG_ADDRESSBOOK

AddressBook *AddressBook::item=0;

#include <settings.h>

AddressBook *AddressBook::thisInstance()
{
  if(!item)
    item=new AddressBook;
  
  return item;
}


AddressBook::AddressBook()
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  Q_ASSERT(0 != vfs);

// this is the default history size. should be customizable
// TODO QSettings stuff
//  Settings::instance()->setGroup("Address book");
//  if(Settings::instance()->hasKey("Cache size"))
//    historySize=Settings::instance()->readEntry("Cache size").toInt();
//  else

  historySize=100;
  entries.setAutoDelete(true);
  reverse.setAutoDelete(true);
  aliasesDict.setAutoDelete(true);
  go.setAutoDelete(false);
 
  // groups before contacts to avoid skipping a group due identical name with
  // an ordinary contact
  scanGroups();
  scanContacts();
//  loadHistory();
  
  changed_ = false;
}

void AddressBook::setChanged()
{
  changed_ = true;
}

void AddressBook::update()
{
  if( changed_ )
  {
    changed_ = false;
    
    historySize=100;
    entries.clear();
    reverse.clear();
    aliasesDict.clear();
    go.clear();
    contacts.clear();
    groups.clear();
    historyAddresses.clear();
   
    scanGroups();
    scanContacts();
//    loadHistory();
  }
}

void AddressBook::scanContacts()
{
  VFolder *folder = 0L;
  VFolderList folderList;
  
  folder = vfs->folderRef( vfs->root() + "/" + VFS_CONTACTS_ROOT );
  if(!folder)
  {
    qDebug("AddressBook::scanContacts : no root folder");
    return;
  }
  
  // scan the contacts root folder
  if( "contacts" == folder->type() )
  {
    QPtrList<VObject> & entries = folder->objectList();
    for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
    {
      if( folder->isSimpleObject( it1.current()->name() ) )
      {
        VContactObject * contact = dynamic_cast<VContactObject*>(it1.current());
        if(contact)
          addContact(contact);
      }
    }
  }
    
  // scan the contacts folders tree
  folder->tree(folderList, -1);
  
  for(VFolderListIterator it(folderList); it.current(); ++it)
  {
    folder = it.current();
    
    if( "contacts" == folder->type() )
    {
      QPtrList<VObject> & entries = folder->objectList();
      for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
      {
        if( folder->isSimpleObject( it1.current()->name() ) )
        {
          VContactObject * contact = dynamic_cast<VContactObject*>(it1.current());
          if(contact)
            addContact(contact);
        }
      }
    }
  }
}

void AddressBook::scanGroups()
{
  VFolder *folder = 0L;
  VFolderList folderList;
  
  folder = vfs->folderRef( vfs->root() + "/" + VFS_GROUPS_ROOT );
  if(!folder)
  {
    qDebug("AddressBook::scanGroups : no root folder");
    return;
  }
  
  // scan the root folder of groups
  if( "groups" == folder->type() )
  {
    QPtrList<VObject> & entries = folder->objectList();
    for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
    {
      if( folder->isSimpleObject( it1.current()->name() ) )
      {
        GroupObject * group = dynamic_cast<GroupObject*>(it1.current());
        if( group )
          addGroup( group );
      }
    }
  }
  
  // scan the group tree
  folder->tree(folderList, -1);
  
  for(VFolderListIterator it(folderList); it.current(); ++it)
  {
    folder = it.current();
    
    if( "groups" == folder->type() )
    {
      QPtrList<VObject> & entries = folder->objectList();
      for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
      {
        if( folder->isSimpleObject( it1.current()->name() ) )
        {
          GroupObject * group = dynamic_cast<GroupObject*>(it1.current());
          if( group )
            addGroup( group );
        }
      }
    }
  }
}

void AddressBook::removeHistoryItem(QString str)
{
  entries.remove(str);
  aliasesDict.remove(str);
  reverse.remove(str);
  historyAddresses.remove(str);
}

void AddressBook::loadHistory()
{
  QString mdir = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  QFile f(mdir+"/internal/addrcache");
  if(f.open(IO_ReadOnly))
  {
    QTextStream txt(&f);
    int count=0;
    while(!txt.atEnd() && count<historySize)
    {
      QString addr=txt.readLine();
      historyAddresses.append(addr);
//      if(entries[addr]==0)
//        entries.insert(addr,new QString(addr));
      if(aliasesDict[addr]==0)
        aliasesDict.insert(addr,new QStringList(addr));
      if(reverse[addr]==0)
        reverse.insert(addr,new QStringList(addr));
    }
  }
}

void AddressBook::saveHistory()
{
  QString mdir = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  QFile f(mdir+"/internal/addrcache");
  
  if(f.open(IO_WriteOnly))
  {
    QTextStream txt(&f);
    
    for(QStringList::ConstIterator it=historyAddresses.begin(); it!=historyAddresses.end(); ++it)
      txt << (*it) << "\n";
  }
}

QString AddressBook::makeProper(VContactObject *contact, QString address)
{
  if(isProper(address))
    return address;
  bool proper=false;
  QString name = contact->n().join(" ");
  int l,r,at;
  l =address.find("<");
  r =address.find(">");
  at=address.find("@");
  if(l!=-1 && r!=-1 && at!=-1 && l<at && at<r) proper=true;
  if(l==0)
  {
    proper=false;
    address.remove(0,1);
    address.remove(r-1,1);
  }
  
  if( address.isEmpty() ||
      at == -1 )
    return QString::null;
  
  if(!proper)
    address=QString("\""+name+"\" <") + address + QString(">");
  return address;
}

bool AddressBook::isProper(QString &addr)
{
  // check for the first quote
  int count=addr.find("\"");
  if(count==-1) return false;

  // and for the second
  count=addr.find("\"",count+1);
  if(count==-1) return false;
  int count2=count;
  // check for "<"
  count=addr.find("<",count);
  if(count==-1)
  {
    // try with []
    count=addr.find("[",count2);
    if(count==-1) return false;
    addr.replace(count,1,"<");
    count=addr.find("]",count);
    if(count==-1) return false;
    addr.replace(count,1,">");
    return true;
  }
  //check for @
  count=addr.find("@",count);
  if(count==-1) return false;
  // check for ">"
  count=addr.find(">",count);
  if(count==-1) return false;
  return true;
}

void AddressBook::addContact(VContactObject *contact)
{
  //first look in the entries to see if that already existed
  if(entries[contact->url()]!=0)
  {
    // remove all the occurences from the dicts & lists
    removeContact(contact->url());
  }
  if( !contact->emails().count() )
  {
    qDebug(IDSTRING"No e-mails for [%s]. Not adding into addressbook.",
      (const char *)contact->fn());
    return;
  }
  
  // It is possible to have 2 empty emails
  // :(, this has to be changed
  
  // building the list of emails
  QStringList emails;
  QString emailProper;
  int iDefault = contact->defaultEmail();
  
  // we have a default email, so we will show only that email
  if( -1 != iDefault &&
      (int)(contact->emails().count()) > iDefault &&
      !contact->emails()[iDefault].simplifyWhiteSpace().isEmpty() )
  {
    emailProper = makeProper(contact, 
      contact->emails()[iDefault].simplifyWhiteSpace());
    
    if( isProper(emailProper) )
      emails.append(emailProper);
  }
  else
  {
    for(uint i=0;i<contact->emails().count();i++)
    {
      if( !contact->emails()[i].simplifyWhiteSpace().isEmpty() )
      {
        emailProper = makeProper(contact, contact->emails()[i].simplifyWhiteSpace());
        if( isProper(emailProper) )
          emails.append(emailProper);
      }
    }
  }
  if( !emails.count() )
  {
    qDebug(IDSTRING"No e-mails for [%s]. Not adding into addressbook.",
      (const char *)contact->fn());
    return;
  }
  
  // done building the list of e-mails
  QStringList aliases; // for reverse
  QStringList temp;
  /*
   * adding:
   *    the FN as defined by the contact label
   *    the default e-mail
   *    the proper address
   *    the full name
   *    the first name
   *    the surname
   *    the nicknames
   */
  aliases.append(contact->fn());
  aliases.append(emails[0]);
  aliases.append(contact->n()[0]+" "+contact->n()[1]);
  aliases.append(contact->n()[0].simplifyWhiteSpace());
  aliases.append(contact->n()[1].simplifyWhiteSpace());
  QStringList nicks=QStringList::split(',',contact->nick());
  {
  for(uint i=0;i<nicks.count();i++)
  {
    aliases.append(nicks[i].simplifyWhiteSpace());
  }
  }
  
  /*
   * add the contact to the entries dict
   */
  entries.insert(contact->url(), new QString(emails.join(",")));
  
  /*
   * add the link to the reverse dict (for easy removing purposes)
   */
  reverse.insert(contact->url(), new QStringList(aliases));
  
  {
  QString tempAlias;
  for(uint i=0;i<aliases.count();i++)
  {
    tempAlias = aliases[i];
    if( !tempAlias.isEmpty() )
    {
      contacts.append(tempAlias);
      /*
       * If there is already the entry, just add the url
       * otherwise, just create the entry
       */
      if(aliasesDict[tempAlias])
      {
        aliasesDict[tempAlias]->append(contact->url());
      }
      else
      {
        aliasesDict.insert(tempAlias, new QStringList(contact->url()));
      }
    }
  }
  }
}

void AddressBook::removeContact(QString url)
{
  QStringList *keys;
  keys=reverse[url];
  if(!keys) return;
  // remove
  for(uint i=0;i<keys->count();i++)
  {
    // removes all the aliases for the contacts list & dict
    if(aliasesDict[(*keys)[i]])
    {
      aliasesDict[(*keys)[i]]->remove(url);
      if(aliasesDict[(*keys)[i]]->count()==0)
        aliasesDict.remove((*keys)[i]);
    }

    contacts.remove((*keys)[i]);
  }
  reverse.remove(url);
}

void AddressBook::addGroup(GroupObject *group)

{
  //first look in the entries to see if that already existed
  if(entries[group->url()]!=0)
  {
    // remove all the occurences from the dicts & lists
    removeGroup(group->url());
  }
  /*
   * The group's aliases (various classifications pending)
   *  for now, just the name...
   */
  QStringList aliases;
  aliases.append(group->groupName());
  /*
   * add the contact to the entries dict
   */
//  entries.insert(group->url(), new QString(group->members().join(",")));
  /*
   * add the link to the reverse dict (for easy removing purposes)
   */
  reverse.insert(group->url(), new QStringList(aliases));
  for(uint i=0;i<aliases.count();i++)
  {
    groups.append(aliases[i]);
    go.insert(aliases[i],group);
    if(aliasesDict[aliases[i]])
    {
      aliasesDict[aliases[i]]->append(group->url());
    }
    else
    {
      aliasesDict.insert(aliases[i],new QStringList(group->url()));
    }
  }
}

void AddressBook::removeGroup(QString url)
{
  QStringList *keys;
  keys=reverse[url];
  // remove
  for(uint i=0;i<keys->count();i++)
  {
    // removes all the aliases for the contacts list & dict
    if(aliasesDict[(*keys)[i]])
    {
      aliasesDict[(*keys)[i]]->remove(url);
      if(aliasesDict[(*keys)[i]]->count()==0)
        aliasesDict.remove((*keys)[i]);
    }
    groups.remove((*keys)[i]);
    go.remove((*keys)[i]);
  }
  reverse.remove(url);
}


void AddressBook::addHistoryAddress(QString address)
{
  if(historyAddresses.findIndex(address)==-1)
  {
    qDebug(IDSTRING"Element not found");
    historyAddresses.prepend(address);
    if(historyAddresses.count()>historySize)
    {
      QString oldAddress=historyAddresses[historyAddresses.count()-1];
      entries.remove(oldAddress);
      reverse.remove(oldAddress);
      aliasesDict.remove(oldAddress);
//      entries.insert(address,new QString(address));
      reverse.insert(address,new QStringList(address));
      aliasesDict.insert(address,new QStringList(address));
      historyAddresses.remove(historyAddresses.at(historyAddresses.count()-1));
    }
  }
  else
  {
    qDebug(IDSTRING"Element found. Move on top.");
    historyAddresses.remove(address);
    historyAddresses.prepend(address);
  }
//  saveHistory();
}


void AddressBook::addUnique(QStringList &list, QString entry)
{
  QString tmp = entry.stripWhiteSpace().simplifyWhiteSpace();
  
  if( !tmp.isEmpty() && 

      -1 == list.findIndex(tmp) )
  {
    list.append(tmp);
    list.sort();
  }
}

QStringList AddressBook::completeNamesContacts(QString key)
{
  QStringList result;
  
  for(QStringList::ConstIterator it=contacts.begin(); it!=contacts.end(); ++it)
  {
    if( key.lower() == (*it).left(key.length()).lower() &&
        !(*it).simplifyWhiteSpace().isEmpty() )
    {
      result.append(*it);
    }
  }
  
  return result;
}

QStringList AddressBook::completeNamesGroups(QString key)
{

#ifdef DEBUG_ADDRESSBOOK
  printf(IDSTRING"completeNamesGroups (%s)\n", (const char *)key);
#endif
  QStringList result;
  for(uint i=0;i<groups.count();i++)
  {
    if(key.lower()==groups[i].left(key.length()).lower() &&
          !groups[i].simplifyWhiteSpace().isEmpty())
    {
#ifdef DEBUG_ADDRESSBOOK
      printf(IDSTRING"\t added: |%s|\n", (const char *)contacts[i]);
#endif
      result.append(groups[i]);
    }
  }

#ifdef DEBUG_ADDRESSBOOK
  printf(IDSTRING"completeNamesGroups. %d of %d\n", result.count(), groups.count());
#endif
  return result;
}

QStringList AddressBook::completeNamesHistory(QString key)
{
#ifdef DEBUG_ADDRESSBOOK
  printf(IDSTRING"completeNamesHistory (%s)\n", (const char *)key);
#endif
  QStringList result;
  for(uint i=0;i<historyAddresses.count();i++)
  {
    if(key.lower()==historyAddresses[i].left(key.length()).lower() &&
          !historyAddresses[i].simplifyWhiteSpace().isEmpty())
    {

#ifdef DEBUG_ADDRESSBOOK
      printf(IDSTRING"\t added: |%s|\n", (const char *)contacts[i]);
#endif
      result.append(historyAddresses[i]);
    }
  }
#ifdef DEBUG_ADDRESSBOOK
  printf(IDSTRING"completeNamesHistory. %d\n", result.count());
#endif
  return result;
}

/*
 * Gets the *names* and **NOT** the e-mail addresses
 */
QStringList AddressBook::completeNames(QString key, int mode)
{
  update();
  
  QStringList result;
  
  if(mode & Contacts)
    result += completeNamesContacts(key);
  if(mode & Groups)
    result += completeNamesGroups(key);
  if(mode & History)
    result += completeNamesHistory(key);
  return result;
}

/*
 *  Gets the e-mails addresses based on "completeNames"
 */
QStringList AddressBook::complete(QString key)
{
  update();
  
  QStringList result;
  QStringList names=completeNames(key);
  for(uint i=0;i<names.count();i++)
  {
    QStringList *temp=aliasesDict[names[i]];
    if(temp)
      for(uint i=0;i<temp->count();i++)
      {
        QString *name=entries[(*temp)[i]];
        if(name) addUnique(result, *name);
      }
  }
  return result;
}


/*
 *  Gets the e-mail addresses given a key. if it's a group then splits them
 * Otherwise, it just returns a list of e-mails.
 *  WARNING: The key MAY have multiple significances!
 *    It will return ALL the entries corresponding to "John"...
 */
QStringList AddressBook::getEntry(QString key)
{
  QStringList result, res;
  QStringList *aliasesKey = aliasesDict[key.simplifyWhiteSpace()];
  
  if(!aliasesKey || key.simplifyWhiteSpace().isEmpty())
  {
    return result;
  }
  
  for(QStringList::ConstIterator it=aliasesKey->begin(); it!=aliasesKey->end(); ++it)
  {
    QString *tmp=entries[(*it)];
    if(!tmp)
    {
      return result;
    }
    
    if(contacts.findIndex(key)!=-1)
      result=*tmp;
    else
        result=QStringList::split(',',*tmp);
    
    for(uint i=0;i<result.count();i++)
    {
      QString str=result[i].simplifyWhiteSpace();
      if(res.findIndex(str)==-1)
        res+=str;
    }
  }
  return res;
  
}

/*
 *  returns a full length list for the contact as well.
 */
QStringList AddressBook::getRealEntry(QString key)
{
  QStringList result, res;
  QStringList *aliasesKey = aliasesDict[key.simplifyWhiteSpace()];
  
  // if there isn't an alias, just copy it to output
  if(!aliasesKey)
  {
    if( !key.simplifyWhiteSpace().isEmpty() )
    {
      result.append( AddressClass(key).fullAddress() );
    }
    
    return result;
  }
  
  for(QStringList::ConstIterator it=aliasesKey->begin(); it!=aliasesKey->end(); ++it)
  {
    QString *tmp = entries[(*it)];
    
    if(!tmp)
    {
      return result;
    }
    
    if( -1 != contacts.findIndex(key) )
    {
      if( -1 != key.find("@") )
        result=QStringList::split(',',*tmp);
      else
        result=QStringList::split(',',*tmp)[0];
    }
    else
      result=QStringList::split(',',*tmp);
    
    for(uint j=0;j<result.count();j++)
    {
      QString str=result[j].simplifyWhiteSpace();
      
      if( -1 == res.findIndex(str) )
        res += str;
    }
  }
  return res;
}

/*
 *  Get Default Name
 */
QString AddressBook::getDefaultName(QString contact)
{
  QStringList myURLs;
  QStringList iterURLs;
  
  myURLs = getEntry(contact);
  
  for(QStringList::ConstIterator it=contacts.begin(); it!=contacts.end(); ++it)
  {
    iterURLs = getEntry(*it);
    
    for(QStringList::ConstIterator it1=myURLs.begin(); it1!=myURLs.end(); ++it1)
      if( -1 != iterURLs.findIndex(*it1) )
        return (*it);
  }
  
  return QString::null;
}

/*
 *  A wrapper over getEntry
 */
QStringList AddressBook::getEntries(QStringList keys)
{
  QStringList result;
  
  for(QStringList::ConstIterator it=keys.begin(); it!=keys.end(); ++it)
  {
    QStringList r = getRealEntry((*it));
    
    for(uint j=0;j<r.count();j++)
      addUnique(result, r[j]);
  }
  
  return result;
}

