/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#include <qtimer.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qfont.h>
#include <qlist.h>
#include <qfontmetrics.h>
#include <carddialog.h>
#include <carddialogwidget.h>
#include <etoolbarbutton.h>
#include <toolbarseparator.h>
#include <ebutton.h>
#include <pixlibrary.h>
#include <miscfunctions.h>
#include <qmessagebox.h>

// #define DEBUG_CARDDIALOG


CardDialog::CardDialog
(
  VContactObject  * contact,
  QWidget         * parent,
  bool            * /* saved */,
  const char      * name,
  IfUnedited        ifUnedited
)
  : QMainWindow       (parent, name),
    _tile             (0),
    _fields           (0),
    _newCard          (false),
    _originalContact  (*contact),
    _ifUnedited       (ifUnedited)
{
	initToolBar();

	// buttons enable/disable
//	upArrowButton->setDisabled(true);
//	downArrowButton->setDisabled(true);
//	clearButton->setDisabled(true);
//	helpButton->setDisabled(true);
	sendMailButton->setDisabled(true);
  // rikkus: It doesn't even do anything !
//	printButton->setDisabled(true);
	saveButton->setDisabled(false);
//	addToGroupButton->setDisabled(true);
	// the widget
	dialogWidget = new CardDialogWidget(contact, this);
	if(contact)
		setCaption(contact->fn());
	setMenu();
	// disabling by default some of them
	setToolbar();
	
	setCentralWidget(dialogWidget);
	layout()->setResizeMode(QLayout::FreeResize);
	connect(dialogWidget, SIGNAL(newHeight(int)), this, SLOT(setNewHeight(int)));
}

void CardDialog::setCaption(QString str, bool modified)
{
	if(modified!=_modified || str!=_str)
	{
		QMainWindow::setCaption(QString("Contact: ")+str + ((modified)?" (*)":""));
		_modified=modified;
		_str=str;
	}
}


void CardDialog::setMenu()
{
	QPopupMenu * file = new QPopupMenu(this);

	file->insertItem
    ("&Save and Close", dialogWidget, SLOT(saveCloseCard()), CTRL+Key_W);

	file->insertItem("S&ave", dialogWidget, SLOT(saveCard()), CTRL+Key_S);
	file->insertItem("&Clear", dialogWidget, SLOT(clearCard()));

	file->insertSeparator();

//	file->insertItem("&Print", dialogWidget, SLOT(printCard()));

//	file->insertSeparator();

	file->insertItem("C&lose", this, SLOT(slotClose()));

//	QPopupMenu *tools = new QPopupMenu(this);
//	tools->insertItem("Send E-mail to...", dialogWidget, SLOT(sendCard()));
//	tools->insertItem("Add to group", dialogWidget, SLOT(groupCard()));
//	tools->insertSeparator();
//	tools->insertItem("Previous", dialogWidget, SLOT(prevCard()));
//	tools->insertItem("Next", dialogWidget, SLOT(nextCard()));
//	QPopupMenu *help = new QPopupMenu(this);
//	help->insertItem("Index", dialogWidget, SLOT(helpCard()));

	menuBar()->insertItem("Card", file);
//	bar->insertItem("Tools",tools);
//	bar->insertSeparator();
//	bar->insertItem("Help",help);
}

void CardDialog::initToolBar()
{
	QFontMetrics fm(QFont("Helvetica",10,QFont::Normal));
  toolBar = new QToolBar("Main toolbar", this);
	EToolBarButton *b = new  EToolBarButton(PixLibrary::thisInstance()->pixmap(PixLibrary::SaveIcon), toolBar);
	saveButton = b;
	b->setText("Save");

  // rikkus: new clears ? No thanks.
//	b = new  EToolBarButton(PixLibrary::thisInstance()->pixmap(PixLibrary::NewIcon), toolBar);
//	clearButton = b;
	clearButton = 0;
//	b->setText("New");
//  qDebug("CardDialog::initToolBar: Need to insert widget in toolbar here");
//	toolBar->insertWidget(2, 30 + fm.width("New"), b);

//	b = new  EToolBarButton(PixLibrary::thisInstance()->pixmap(PixLibrary::PrintIcon), toolBar);
//	printButton = b;
	printButton = 0;
//	b->setText("Print");

	b = new  EToolBarButton(PixLibrary::thisInstance()->pixmap(PixLibrary::SendMailIcon), toolBar);
	sendMailButton = b;
	b->setText("Send mail to...");

#if 0
	b = new  EToolBarButton(PixLibrary::thisInstance()->pixmap(PixLibrary::GroupIcon), toolBar);
	addToGroupButton = b;
	b->setText("Add to group");
#endif

	// ToolBarSeparator *ts = new ToolBarSeparator(toolBar);

#if 0
	b = new  EToolBarButton(toolBar);
	downArrowButton = b;
	b->setNormalPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::ArrowDownNormalIcon));
	b->setFocusPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::ArrowDownFocusIcon));
	b->setClickedPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::ArrowDownFocusIcon));

	b = new  EToolBarButton(toolBar);
	upArrowButton = b;
	b->setNormalPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::ArrowUpNormalIcon));
	b->setFocusPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::ArrowUpFocusIcon));
	b->setClickedPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::ArrowUpFocusIcon));

	ts = new ToolBarSeparator(toolBar);

	b = new  EToolBarButton(toolBar);
	helpButton = b;
	b->setNormalPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::HelpNormalIcon));
	b->setFocusPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::HelpFocusIcon));
	b->setClickedPixmap(PixLibrary::thisInstance()->pixmap(PixLibrary::HelpFocusIcon));
	b->setFixedWidth(b->width());
#endif
}

void CardDialog::setToolbar()
{
	connect(saveButton, SIGNAL(clicked()), dialogWidget, SLOT(saveCard()));
//	connect(clearButton, SIGNAL(clicked()), dialogWidget, SLOT(newCard()));
//	connect(printButton, SIGNAL(clicked()), dialogWidget, SLOT(printCard()));
	connect(sendMailButton, SIGNAL(clicked()), dialogWidget, SLOT(sendCard()));
//	connect(addToGroupButton, SIGNAL(clicked()), dialogWidget, SLOT(groupCard()));
#if 0
	connect(downArrowButton, SIGNAL(clicked()), dialogWidget, SLOT(nextCard()));
	connect(upArrowButton, SIGNAL(clicked()), dialogWidget, SLOT(prevCard()));
	connect(helpButton, SIGNAL(clicked()), dialogWidget, SLOT(helpCard()));
#endif
}

void CardDialog::setNewHeight(int h)
{
	resize(dialogWidget->width(), h+ toolBar->height());
#ifdef DEBUG_CARDDIALOG
	printf("Height: %d, real: %d\n", h, height());
#endif
}

void CardDialog::setTile(CardTileView *item)
{
	_tile=item;
}

void CardDialog::setFields(QStringList *item)
{
	_fields=item;
}
void CardDialog::setNewCard(bool nc)
{
	_newCard=nc;
}

CardTileView *CardDialog::tile() { return _tile; }
QStringList *CardDialog::fields() { return _fields; }
bool CardDialog::newCard() { return _newCard; }

CardDialog::~CardDialog()
{
  qDebug("CardDialog::~CardDialog");
}

void CardDialog::slotClose()
{
  qDebug("Getting card");
  VContactObject * card = dialogWidget->card();

  qDebug("Getting card path");

  QString path = card->path();

  if
    (
      (_ifUnedited == RemoveIfUnedited)
      &&
      (_originalContact == *dialogWidget->card())
    )
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
    }

    Q_ASSERT(0 != vfs);

    qDebug("Removing card, unedited");
    vfs->remove(dialogWidget->card()->path());
    delete dialogWidget->card();
  }
  else
  {
    dialogWidget->card()->save();
  }

  close();
  qDebug("Emitting dialogClosed");
	emit(dialogClosed(path));

  qDebug("Watch out, card dialog deleting self");
  delete this;
}

void CardDialog::closeEvent(QCloseEvent *)
{
  // Delayed self-destruct.
  QTimer * t = new QTimer(this);
  connect(t, SIGNAL(timeout()), SLOT(slotClose()));
  t->start(0, true);
}
