/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include <qlineedit.h>
#include <qtextedit.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qseparator.h>

#include <contacteditdialog.h>

#include <vcontactobject.h>
#include <defines.h>
#include <vfsmimetype.h>
#include <vfsmodule.h>

// FIXME: Use designer.
// FIXME: Fix designer.

ContactEditDialog::ContactEditDialog
(
  VContactObject  * contact,
  QWidget         * parent,
  const char      * name,
  IfUnedited        ifUnedited
)
  : QTabDialog    (parent, name, true /* modal */),
    path_         (contact->path()),
    ifUnedited_   (ifUnedited)
{
  setCaption(tr("Edit Contact - %1").arg(AppName));
  QSize size_ = size();
  size_.setWidth(400);
  resize(size_);

  namePage_     = new QWidget(this, "Name page");
  homeAddressPage_  = new QWidget(this, "Home address page");
  workAddressPage_  = new QWidget(this, "Work address page");
  otherPage_    = new QWidget(this, "Other page");

  addTab(namePage_,         tr("&Name"));
  addTab(homeAddressPage_,  tr("&Home"));
  addTab(workAddressPage_,  tr("&Work"));
  addTab(otherPage_,        tr("O&ther"));

  setOkButton();
  setCancelButton();

  connect(this, SIGNAL(applyButtonPressed()), SLOT(slotApplyButtonPressed()));
  connect(this, SIGNAL(cancelButtonPressed()), SLOT(slotCancelButtonPressed()));

  // Name page.

  // Name box.

  QLabel * firstNameLabel = new QLabel(tr("&First:"), namePage_);
  QLabel * lastNameLabel = new QLabel(tr("&Last:"), namePage_);
  QLabel * displayNameLabel = new QLabel(tr("&Display:"), namePage_);
  QLabel * nickNameLabel = new QLabel(tr("Nic&kname:"), namePage_);

  firstNameLineEdit_    = new QLineEdit(namePage_);
  lastNameLineEdit_     = new QLineEdit(namePage_);
  displayNameLineEdit_  = new QLineEdit(namePage_);
  nickNameLineEdit_     = new QLineEdit(namePage_);

  // Email box.

  QLabel * emailLabel = new QLabel(tr("&Email:"), namePage_);
  QLabel * additionalEmailLabel = new QLabel(tr("Addi&tional Email:"), namePage_);

  emailLineEdit_            = new QLineEdit(namePage_);
  additionalEmailLineEdit_  = new QLineEdit(namePage_);

  // Tel box.

  QLabel * homeTelLabel = new QLabel(tr("&Home:"), namePage_);
  QLabel * workTelLabel = new QLabel(tr("&Work:"), namePage_);
  QLabel * mobileTelLabel = new QLabel(tr("&Mobile:"), namePage_);
  QLabel * faxTelLabel = new QLabel(tr("Fa&x:"), namePage_);
  QLabel * pagerTelLabel = new QLabel(tr("&Pager:"), namePage_);
  QLabel * sipTelLabel = new QLabel(tr("&SIP:"), namePage_);

  homeTelLineEdit_    = new QLineEdit(namePage_);
  workTelLineEdit_    = new QLineEdit(namePage_);
  mobileTelLineEdit_  = new QLineEdit(namePage_);
  faxTelLineEdit_     = new QLineEdit(namePage_);
  pagerTelLineEdit_   = new QLineEdit(namePage_);
  sipTelLineEdit_     = new QLineEdit(namePage_);

  QGridLayout * namePageLayout = new QGridLayout(namePage_, 15, 2, 6, 6);

  namePageLayout->addMultiCellWidget(new QSeparator(tr("Name"), namePage_), 0, 0, 0, 1);
  namePageLayout->addMultiCellWidget(new QSeparator(tr("Email"), namePage_), 5, 5, 0, 1);
  namePageLayout->addMultiCellWidget(new QSeparator(tr("Telephone"), namePage_), 8, 8, 0, 1);

  namePageLayout->addWidget(firstNameLabel,       1, 0);
  namePageLayout->addWidget(lastNameLabel,        2, 0);
  namePageLayout->addWidget(displayNameLabel,     3, 0);
  namePageLayout->addWidget(nickNameLabel,        4, 0);
  namePageLayout->addWidget(emailLabel,           6, 0);
  namePageLayout->addWidget(additionalEmailLabel, 7, 0);
  namePageLayout->addWidget(homeTelLabel,         9, 0);
  namePageLayout->addWidget(workTelLabel,        10, 0);
  namePageLayout->addWidget(mobileTelLabel,      11, 0);
  namePageLayout->addWidget(faxTelLabel,         12, 0);
  namePageLayout->addWidget(pagerTelLabel,       13, 0);
  namePageLayout->addWidget(sipTelLabel,         14, 0);

  namePageLayout->addWidget(firstNameLineEdit_,       1, 1);
  namePageLayout->addWidget(lastNameLineEdit_,        2, 1);
  namePageLayout->addWidget(displayNameLineEdit_,     3, 1);
  namePageLayout->addWidget(nickNameLineEdit_,        4, 1);
  namePageLayout->addWidget(emailLineEdit_,           6, 1);
  namePageLayout->addWidget(additionalEmailLineEdit_, 7, 1);
  namePageLayout->addWidget(homeTelLineEdit_,         9, 1);
  namePageLayout->addWidget(workTelLineEdit_,        10, 1);
  namePageLayout->addWidget(mobileTelLineEdit_,      11, 1);
  namePageLayout->addWidget(faxTelLineEdit_,         12, 1);
  namePageLayout->addWidget(pagerTelLineEdit_,       13, 1);
  namePageLayout->addWidget(sipTelLineEdit_,         14, 1);

  // Home address page.

  QGridLayout * homeAddressPageLayout =
    new QGridLayout(homeAddressPage_, 8, 2, 6, 6);

  QLabel * homeAddressLabel = new QLabel(tr("&Address:"), homeAddressPage_);
  QLabel * homeCityLabel = new QLabel(tr("C&ity:"), homeAddressPage_);
  QLabel * homeStateLabel = new QLabel(tr("&State/Area:"), homeAddressPage_);
  QLabel * homeZipLabel = new QLabel(tr("&Zip/Postal Code:"), homeAddressPage_);
  QLabel * homeCountryLabel = new QLabel(tr("Co&untry:"), homeAddressPage_);
  QLabel * homeWebsiteLabel = new QLabel(tr("&Website:"), homeAddressPage_);

  homeAddressLineEdit_    = new QLineEdit(homeAddressPage_);
  homeAddress2LineEdit_   = new QLineEdit(homeAddressPage_);
  homeCityLineEdit_       = new QLineEdit(homeAddressPage_);
  homeStateLineEdit_      = new QLineEdit(homeAddressPage_);
  homeZipLineEdit_        = new QLineEdit(homeAddressPage_);
  homeCountryLineEdit_    = new QLineEdit(homeAddressPage_);
  homeWebsiteLineEdit_    = new QLineEdit(homeAddressPage_);

  homeAddressPageLayout->addWidget(homeAddressLabel,        0, 0);
  homeAddressPageLayout->addWidget(homeCityLabel,           2, 0);
  homeAddressPageLayout->addWidget(homeStateLabel,          3, 0);
  homeAddressPageLayout->addWidget(homeZipLabel,            4, 0);
  homeAddressPageLayout->addWidget(homeCountryLabel,        5, 0);
  homeAddressPageLayout->addWidget(homeWebsiteLabel,        6, 0);

  homeAddressPageLayout->addWidget(homeAddressLineEdit_,        0, 1);
  homeAddressPageLayout->addWidget(homeAddress2LineEdit_,       1, 1);
  homeAddressPageLayout->addWidget(homeCityLineEdit_,           2, 1);
  homeAddressPageLayout->addWidget(homeStateLineEdit_,          3, 1);
  homeAddressPageLayout->addWidget(homeZipLineEdit_,            4, 1);
  homeAddressPageLayout->addWidget(homeCountryLineEdit_,        5, 1);
  homeAddressPageLayout->addWidget(homeWebsiteLineEdit_,        6, 1);

  // Work address page.

  QGridLayout * workAddressPageLayout =
    new QGridLayout(workAddressPage_, 12, 2, 6, 6);

  QLabel * workTitleLabel = new QLabel(tr("&Title:"), workAddressPage_);
  QLabel * workDepartmentLabel = new QLabel(tr("&Department:"), workAddressPage_);
  QLabel * workOrganizationLabel = new QLabel(tr("O&rganization"), workAddressPage_);
  QLabel * workAddressLabel = new QLabel(tr("&Address:"), workAddressPage_);
  QLabel * workCityLabel = new QLabel(tr("C&ity:"), workAddressPage_);
  QLabel * workStateLabel = new QLabel(tr("&State/Area:"), workAddressPage_);
  QLabel * workZipLabel = new QLabel(tr("&Zip/Postal Code:"), workAddressPage_);
  QLabel * workCountryLabel = new QLabel(tr("Co&untry:"), workAddressPage_);
  QLabel * workWebsiteLabel = new QLabel(tr("&Website:"), workAddressPage_);

  workTitleLineEdit_        = new QLineEdit(workAddressPage_);
  workDepartmentLineEdit_   = new QLineEdit(workAddressPage_);
  workOrganizationLineEdit_ = new QLineEdit(workAddressPage_);
  workAddressLineEdit_    = new QLineEdit(workAddressPage_);
  workAddress2LineEdit_   = new QLineEdit(workAddressPage_);
  workCityLineEdit_       = new QLineEdit(workAddressPage_);
  workStateLineEdit_      = new QLineEdit(workAddressPage_);
  workZipLineEdit_        = new QLineEdit(workAddressPage_);
  workCountryLineEdit_    = new QLineEdit(workAddressPage_);
  workWebsiteLineEdit_    = new QLineEdit(workAddressPage_);

  workAddressPageLayout->addMultiCellWidget
    (new QSeparator(QString::null, workAddressPage_), 3, 3, 0, 1);

  workAddressPageLayout->addWidget(workTitleLabel,          0, 0);
  workAddressPageLayout->addWidget(workDepartmentLabel,     1, 0);
  workAddressPageLayout->addWidget(workOrganizationLabel,   2, 0);
  workAddressPageLayout->addWidget(workAddressLabel,        4, 0);
  workAddressPageLayout->addWidget(workCityLabel,           6, 0);
  workAddressPageLayout->addWidget(workStateLabel,          7, 0);
  workAddressPageLayout->addWidget(workZipLabel,            8, 0);
  workAddressPageLayout->addWidget(workCountryLabel,        9, 0);
  workAddressPageLayout->addWidget(workWebsiteLabel,       10, 0);

  workAddressPageLayout->addWidget(workTitleLineEdit_,          0, 1);
  workAddressPageLayout->addWidget(workDepartmentLineEdit_,     1, 1);
  workAddressPageLayout->addWidget(workOrganizationLineEdit_,   2, 1);
  workAddressPageLayout->addWidget(workAddressLineEdit_,        4, 1);
  workAddressPageLayout->addWidget(workAddress2LineEdit_,       5, 1);
  workAddressPageLayout->addWidget(workCityLineEdit_,           6, 1);
  workAddressPageLayout->addWidget(workStateLineEdit_,          7, 1);
  workAddressPageLayout->addWidget(workZipLineEdit_,            8, 1);
  workAddressPageLayout->addWidget(workCountryLineEdit_,        9, 1);
  workAddressPageLayout->addWidget(workWebsiteLineEdit_,       10, 1);

  // Other page.

  // Top bit.

  QGridLayout * otherPageLayout = new QGridLayout(otherPage_, 3, 2, 6, 6);

  otherPageLayout->addMultiCellWidget
    (new QSeparator(tr("Notes"), otherPage_), 0, 0, 0, 1);

  notesTextEdit_ = new QTextEdit(otherPage_);

  otherPageLayout->addMultiCellWidget(notesTextEdit_, 1, 1, 0, 1);

  // Buddies.

  firstNameLabel->setBuddy(firstNameLineEdit_);
  lastNameLabel->setBuddy(lastNameLineEdit_);
  displayNameLabel->setBuddy(displayNameLineEdit_);
  nickNameLabel->setBuddy(nickNameLineEdit_);

  emailLabel->setBuddy(emailLineEdit_);
  additionalEmailLabel->setBuddy(additionalEmailLineEdit_);

  homeTelLabel->setBuddy(homeTelLineEdit_);
  workTelLabel->setBuddy(workTelLineEdit_);
  mobileTelLabel->setBuddy(mobileTelLineEdit_);
  faxTelLabel->setBuddy(faxTelLineEdit_);
  pagerTelLabel->setBuddy(pagerTelLineEdit_);
  sipTelLabel->setBuddy(sipTelLineEdit_);

  homeAddressLabel->setBuddy(homeAddressLineEdit_);
  homeCityLabel->setBuddy(homeCityLineEdit_);
  homeStateLabel->setBuddy(homeStateLineEdit_);
  homeZipLabel->setBuddy(homeZipLineEdit_);
  homeCountryLabel->setBuddy(homeCountryLineEdit_);
  homeWebsiteLabel->setBuddy(homeWebsiteLineEdit_);

  workTitleLabel->setBuddy(workTitleLineEdit_);
  workDepartmentLabel->setBuddy(workDepartmentLineEdit_);
  workOrganizationLabel->setBuddy(workOrganizationLineEdit_);

  workAddressLabel->setBuddy(workAddressLineEdit_);
  workCityLabel->setBuddy(workCityLineEdit_);
  workStateLabel->setBuddy(workStateLineEdit_);
  workZipLabel->setBuddy(workZipLineEdit_);
  workCountryLabel->setBuddy(workCountryLineEdit_);
  workWebsiteLabel->setBuddy(workWebsiteLineEdit_);

  load();
}

ContactEditDialog::~ContactEditDialog()
{
  // Empty.
}

  void
ContactEditDialog::slotApplyButtonPressed()
{
  save();
}

  void
ContactEditDialog::slotCancelButtonPressed()
{
  close();
}

  void
ContactEditDialog::load()
{
  VContactObject * contact_ = contact();
  if( contact_ )
  {
    firstNameLineEdit_        ->setText(contact_->n()[0]);
    lastNameLineEdit_         ->setText(contact_->n()[1]);
    displayNameLineEdit_      ->setText(contact_->fn());
    nickNameLineEdit_         ->setText(contact_->nick());
    emailLineEdit_            ->setText(contact_->emails()[0]);
    additionalEmailLineEdit_  ->setText(contact_->emails()[1]);
  
    homeTelLineEdit_    ->setText(contact_->phone("home"));
    workTelLineEdit_    ->setText(contact_->phone("work"));
    mobileTelLineEdit_  ->setText(contact_->phone("mobile"));
    faxTelLineEdit_     ->setText(contact_->phone("fax"));
    pagerTelLineEdit_   ->setText(contact_->phone("pager"));
    sipTelLineEdit_   ->setText(contact_->phone("sip"));
  
    homeAddressLineEdit_      ->setText(contact_->homeAdr()[2]);
    homeAddress2LineEdit_     ->setText(contact_->homeAdr()[1]);
    homeCityLineEdit_         ->setText(contact_->homeAdr()[3]);
    homeStateLineEdit_        ->setText(contact_->homeAdr()[4]);
    homeZipLineEdit_          ->setText(contact_->homeAdr()[5]);
    homeCountryLineEdit_      ->setText(contact_->homeAdr()[6]);
    
    workTitleLineEdit_        ->setText(contact_->title());
    workDepartmentLineEdit_   ->setText(contact_->role());
    workOrganizationLineEdit_ ->setText(contact_->org());
  
    workAddressLineEdit_      ->setText(contact_->workAdr()[2]);
    workAddress2LineEdit_     ->setText(contact_->workAdr()[1]);
    workCityLineEdit_         ->setText(contact_->workAdr()[3]);
    workStateLineEdit_        ->setText(contact_->workAdr()[4]);
    workZipLineEdit_          ->setText(contact_->workAdr()[5]);
    workCountryLineEdit_      ->setText(contact_->workAdr()[6]);
  
    homeWebsiteLineEdit_      ->setText(contact_->URLs()[0]);
    workWebsiteLineEdit_      ->setText(contact_->URLs()[1]);
  
    notesTextEdit_            ->setText(contact_->notes());
  
    QStringList urls = contact_->URLs();
  
    while (urls.count() < 2)
      urls += "";
  
    homeWebsiteLineEdit_->setText(urls[0]);
    workWebsiteLineEdit_->setText(urls[1]);
  }
}

  void
ContactEditDialog::save()
{
  VContactObject * contact_ = contact();
  if( contact_ )
  {
    QStringList n = contact_->n();
    while (n.count() < 2)
      n += "";
    n[0] = firstNameLineEdit_->text();
    n[1] = lastNameLineEdit_->text();
    contact_->setN(n);
  
    contact_->setFn(displayNameLineEdit_->text());
  
    contact_->setNick(nickNameLineEdit_->text());
  
    QStringList emails = contact_->emails();
    while (emails.count() < 2)
      emails += "";
    emails[0] = emailLineEdit_->text();
    emails[1] = additionalEmailLineEdit_->text();
    contact_->setEmails(emails);
  
    contact_->setPhone("home",    homeTelLineEdit_    ->text());
    contact_->setPhone("work",    workTelLineEdit_    ->text());
    contact_->setPhone("mobile",  mobileTelLineEdit_  ->text());
    contact_->setPhone("fax",     faxTelLineEdit_     ->text());
    contact_->setPhone("pager",   pagerTelLineEdit_   ->text());
    contact_->setPhone("sip",     sipTelLineEdit_     ->text());
  
    QStringList homeAdr = contact_->homeAdr();
  
    while (homeAdr.count() < 7)
      homeAdr += "";
  
    // Post Office Address (0)
    // Extended Address (note swapping)
    // Street
    // Locality
    // Region
    // Postal Code
    // Country
    homeAdr[1] = homeAddress2LineEdit_->text();
    homeAdr[2] = homeAddressLineEdit_->text();
    homeAdr[3] = homeCityLineEdit_->text();
    homeAdr[4] = homeStateLineEdit_->text();
    homeAdr[5] = homeZipLineEdit_->text();
    homeAdr[6] = homeCountryLineEdit_->text();
  
    contact_->setHomeAdr(homeAdr);
  
    contact_->setTitle(workTitleLineEdit_->text());
    contact_->setRole(workDepartmentLineEdit_->text());
    contact_->setOrg(workOrganizationLineEdit_->text());
  
    QStringList workAdr = contact_->workAdr();
  
    while (workAdr.count() < 7)
      workAdr += "";
  
    workAdr[1] = workAddress2LineEdit_->text();
    workAdr[2] = workAddressLineEdit_->text();
    workAdr[3] = workCityLineEdit_->text();
    workAdr[4] = workStateLineEdit_->text();
    workAdr[5] = workZipLineEdit_->text();
    workAdr[6] = workCountryLineEdit_->text();
  
    contact_->setWorkAdr(workAdr);
  
    QStringList urls = contact_->URLs();
  
    while (urls.count() < 2)
      urls += "";
  
    urls[0] = homeWebsiteLineEdit_->text();
    urls[1] = workWebsiteLineEdit_->text();
  
    contact_->setURLs(urls);
  
    contact_->setNotes(notesTextEdit_->text());
  
    contact_->save();
  }
}

  void
ContactEditDialog::closeEvent(QCloseEvent * e)
{
  e->accept();
  emit(closed(this));
}

  VContactObject *
ContactEditDialog::contact()
{
  VFSModule * vfs = VObject::vfsModule();
  VContactObject * contactObject = 0;
  if(vfs)
  {
    contactObject = static_cast<VContactObject *>
      ( vfs->_fact_ref(path_, VFS_CONTACTS_MIMETYPE) );
  }
  
  return contactObject;
}
