/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 /**
   * Changed by the Eugen C. <eug@thekompany.com> for kore plug in support.
   *  Date: 27 Sep. 2001
   */

#include <qstring.h>
#include <qsize.h>
#include <qpoint.h>
#include <qfont.h>
#include <qdatastream.h>
#include <qcstring.h>
#include <qvariant.h>
#include <qdatetime.h>
#include <quuid.h>

#include <vcontactobject.h>
#include <metaobject.h>
#include <miscfunctions.h>
#include <komaetheratools.h>

/*
  #define MORE_SECURE
*/

#define IDSTRING "ContactObject: "
// #define DEBUG_CO

// implementation for the contact object

ContactObject::ContactObject()
  : _defaultEmail(-1)
{
}

ContactObject::ContactObject(const VContactObject & other)
{
  _fn = other._fn;
  _n = other._n;
  _homeAdr = other._homeAdr;
  _workAdr = other._workAdr;
  _phone = other._phone;
  _emails = other._emails;
  _defaultEmail = other._defaultEmail;
  _notes = other._notes;
  _URLs = other._URLs;
  _nick = other._nick;
  _bday = other._bday;
  _org = other._org;
  _role = other._role;
  _title = other._title;
  _classCard = other._classCard;
  _categories = other._categories;
  _category = other._category;
  _security = other._security;
  _relatedObjectList = other._relatedObjectList;
  _uid = other._uid;
  _extraData = other._extraData;
}

ContactObject::~ContactObject()
{
  // nothing here
}

/***************************************************************************/
/*  Methods for setting/getting the stuff off the object                   */
/***************************************************************************/

// VCARD3
//    param        = tel-param
//    value        = phone-number-value
//    tel-param    = "TYPE" "=" tel-type *("," tel-type)
//    tel-type     = "HOME" / "WORK" / "PREF" / "VOICE" / "FAX" / "MSG"
//                 / "CELL" / "PAGER" / "BBS" / "MODEM" / "CAR" / "ISDN"
//                 / "VIDEO" / "PCS" / iana-token / x-name
//         ; Values are case insensitive

unsigned ContactObject::convertPhoneType(QString type)
{
  unsigned result = 0;
  
  if(type.find("home",0,false)!=-1)   result|=ContactObject::Home;
  if(type.find("work",0,false)!=-1)   result|=ContactObject::Work;
  if(type.find("pref",0,false)!=-1)   result|=ContactObject::Pref;
  if(type.find("voice",0,false)!=-1)  result|=ContactObject::Voice;
  if(type.find("msg",0,false)!=-1)    result|=ContactObject::Msg;
  if(type.find("fax",0,false)!=-1)    result|=ContactObject::Fax;
  if(type.find("cell",0,false)!=-1)   result|=ContactObject::Cell;
  if(type.find("pager",0,false)!=-1)  result|=ContactObject::Pager;
  if(type.find("bbs",0,false)!=-1)    result|=ContactObject::Bbs;
  if(type.find("modem",0,false)!=-1)  result|=ContactObject::Modem;
  if(type.find("car",0,false)!=-1)    result|=ContactObject::Car;
  if(type.find("isdn",0,false)!=-1)   result|=ContactObject::Isdn;
  if(type.find("pcs",0,false)!=-1)    result|=ContactObject::Pcs;
  if(type.find("video",0,false)!=-1)  result|=ContactObject::Video;
  if( -1 != type.find("application/sdp",0,false) ||
      -1 != type.find("application/isup",0,false) )
    result |= ContactObject::SIP;
  
  return result;
}

void ContactObject::setCategory(const QString & newCategory)
{
  _category=newCategory;
}

QString ContactObject::category() const
{
  return _category;
}

void ContactObject::setFn(const QString & newFn)
{
  _fn=newFn;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::fn() const
{
  return _fn;
}
void ContactObject::setN(const QStringList & newN)
{
  _n=newN;
#ifdef MORE_SECURE
  save();
#endif
}
QStringList ContactObject::n() const
{
  return _n;
}
void ContactObject::setHomeAdr(const QStringList & newHomeAdr)
{
  _homeAdr=newHomeAdr;
#ifdef MORE_SECURE
  save();
#endif
}
QStringList ContactObject::homeAdr() const
{
  return _homeAdr;
}
void ContactObject::setPhone(const QString & key, const QString & value)
{
  _phone[key]=value;
#ifdef MORE_SECURE
  save();
#endif
}
void ContactObject::setPhone(const QMap<QString, QString> & newPhone)
{
#ifdef DEBUG_CO
  qDebug("------------------- %d phones\n", newPhone.count());
#endif
  _phone=newPhone;
#ifdef MORE_SECURE
  save();
#endif
}
QMap<QString, QString> ContactObject::phone() const
{
  return _phone;
}

QString ContactObject::phone(const QString & key) const
{
  return _phone[key];
}

void ContactObject::setWorkAdr(const QStringList & newWorkAdr)
{
  _workAdr=newWorkAdr;
#ifdef MORE_SECURE
  save();
#endif
}
QStringList ContactObject::workAdr() const
{
  return _workAdr;
}
void ContactObject::setEmails(const QStringList & newEmails)
{
  _emails=newEmails;
#ifdef MORE_SECURE
  save();
#endif
}
QStringList ContactObject::emails() const
{
  return _emails;
}
void ContactObject::setDefaultEmail(int newDefaultMail)
{
  _defaultEmail=newDefaultMail;
#ifdef MORE_SECURE
  save();
#endif
}
int ContactObject::defaultEmail() const
{
  return _defaultEmail;
}
void ContactObject::setNotes(const QString & newNotes)
{
  _notes=newNotes;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::notes() const
{
  return _notes;
}
void ContactObject::setURLs(const QStringList & newURLs)
{
  _URLs=newURLs;
#ifdef MORE_SECURE
  save();
#endif
}
QStringList ContactObject::URLs() const
{
  return _URLs;
}
void ContactObject::setNick(const QString & newNick)
{
  _nick=newNick;
#ifdef MORE_SECURE
  save();
#endif
}

QString ContactObject::nick() const
{
//  qDebug("ContactObject::nick(): (%s)", _nick.ascii());
  return _nick;
}

void ContactObject::setSecurity(int newSecurity)
{
  _security=newSecurity;
}

int ContactObject::security() const
{
  return _security;
}

void ContactObject::setBday(const QString & newBdate)
{
  _bday=newBdate;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::bday() const
{
  return _bday;
}
void ContactObject::setOrg(const QString & newOrg)
{
  _org=newOrg;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::org() const
{
  return _org;
}
void ContactObject::setTitle(const QString & newTitle)
{
  _title=newTitle;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::title() const
{
  return _title;
}
void ContactObject::setRole(const QString & newPos)
{
  _role=newPos;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::role() const
{
  return _role;
}
void ContactObject::setCategories(const QString & newPos)
{
  _categories=newPos;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::categories() const
{
  return _categories;
}
void ContactObject::setClassCard(const QString & newPos)
{
  _classCard=newPos;
#ifdef MORE_SECURE
  save();
#endif
}
QString ContactObject::classCard() const
{
  return _classCard;
}

/* This method is for importing vcards version 2.x */

void ContactObject::setVcard2(const QString & import)
{
  MetaObject mObj(import, 0);
  
  // check the vcard version
  QString ver;
  if( mObj.query("VERSION").count() )
  {
    ver = mObj["VERSION"].getContent();
  }
  if( ver.isEmpty() ||
      "2.1" != ver )
  {
    qDebug("Wrong vcard version %s or not vcard data.", ver.latin1());
    return;
  }
  
  // set UUID
  if( mObj.query("UID").count() )
  {
    _uid = mObj["UID"].getContent();
  }
  
  if( mObj.query("FN").count() )
    _fn = mObj["FN"].getContent();
  
  QStringList tempList;
  QString tempString;
  QValueList<int> matches;
  
  /**
   * getting all the emails...
   */
  matches=mObj.query("EMAIL");
  tempList.clear();
  for(uint i=0;i<matches.count();i++)
    tempList.append(mObj.getLines().at(matches[i])->getContent());
  _emails=tempList;
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _emails\n");
#endif
  matches=mObj.query("EMAIL","pref");
  _defaultEmail=0;
  if(matches.count())
  {
    tempString=mObj.getLines().at(matches[0])->getContent();
    for(uint i=0;i<_emails.count();i++)
      if(_emails[i]==tempString)
      {
        _defaultEmail=i;
        break;
      }
  }
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _defaultEmail\n");
#endif
  /**
   *  getting the name...
   */
  if(mObj.query("N").count()!=0)
    tempList=split(mObj["N"].getContent(),';');
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _n split %d\n", tempList.count());
#endif
  // NAME
  if(tempList.count()>0) _n.append(tempList[0]);
  else _n.append("");
  // FAMILY NAME
  if(tempList.count()>1) _n.append(tempList[1]);
  else _n.append("");
  // ADDITIONAL NAMES (INITIAL)
  if(tempList.count()>2) _n.append(tempList[2]);
  else _n.append("");
  // HONOR PREFIXES
  if(tempList.count()>3) _n.append(tempList[3]);
  else _n.append("");
  // HONOR SUFFIXES
  if(tempList.count()>4) _n.append(tempList[4]);
  else _n.append("");
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _n\n");
#endif
  /**
   *  getting the title...
   */
  if(mObj.query("title").count()!=0)
    _title=mObj["title"].getContent();
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _title\n");
#endif
  /**
   *  getting the URLs...
   */
  matches=mObj.query("url");
  _URLs.clear();
  {
  for(uint i=0;i<matches.count();i++)
    _URLs.append(mObj.getLines().at(matches[i])->getContent());
  }
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _URLs\n");
#endif
  /**
   *  getting the class...
   */
  tempString="";
  if(mObj.query("CLASS").count()!=0)
    tempString=mObj["class"].getContent();
  if( (tempString.find("PRIVATE",0,false)==-1) &&
      (tempString.find("PUBLIC",0,false)==-1) &&
      (tempString.find("CONFIDNTIAL",0,false)==-1))
    tempString="PRIVATE";
   _classCard=tempString;
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _class\n");
#endif
  /**
   * getting the note
   */
  _notes=QString::null;
  if(mObj.query("note").count()!=0)
    _notes=mObj["note"].getContent();
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _notes\n");
#endif
  /**
   * getting the categories
   */
  _categories=QString::null;
  if(mObj.query("categories").count()!=0)
    _categories=mObj["cetegories"].getContent();
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _categories\n");
#endif
  /**
   * getting the org
   */
  _org=QString::null;
  if(mObj.query("org").count()!=0)
    _org=mObj["org"].getContent();
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _org\n");
#endif
  /**
   * getting the role
   */
  _role=QString::null;
  if(mObj.query("ROLE").count()!=0)
    _role=mObj["role"].getContent();
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _role\n");
#endif
  /**
   * getting the nick
   */
  _nick=QString::null;
  if(mObj.query("NICKNAME").count()!=0)
    _nick=mObj["nickname"].getContent();
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _nick\n");
#endif
  /**
   * getting the bday
   */
  _bday=QString::null;
  if(mObj.query("bday").count()!=0)
    _bday=mObj["bday"].getContent();
#ifdef DEBUG_CO
  qDebug(IDSTRING"Done the _bday\n");
#endif
  
  /**
   * getting the work address
   */
  matches=mObj.query("ADR","work");
  if( matches.count() )
  {
    // doing the address...
    tempList=split(mObj.getLines().at(matches[0])->getContent(),';');
    switch(tempList.count())
    {
      case 7:
        _workAdr=tempList;
        break;
      case 1:
        _workAdr.clear();
        _workAdr.append("");
        _workAdr.append("");
        _workAdr.append(tempList[0]);
        _workAdr.append("");
        _workAdr.append("");
        _workAdr.append("");
        _workAdr.append("");
        break;
      default:
        _workAdr=tempList;
        for(uint i=0;i<7-tempList.count();i++)
          _workAdr.append("");
    }
  }
  
  /**
   * getting the home address
   */
  matches=mObj.query("ADR","home");
  if( !matches.count() )
  {
    matches=mObj.query("ADR", QString::null, "work");
  }
  if( matches.count() )
  {
    // doing the address...
    tempList=split(mObj.getLines().at(matches[0])->getContent(),';');
    switch(tempList.count())
    {
      case 7:
        _homeAdr=tempList;
        break;
      case 1:
        _homeAdr.clear();
        _homeAdr.append("");
        _homeAdr.append("");
        _homeAdr.append(tempList[0]);
        _homeAdr.append("");
        _homeAdr.append("");
        _homeAdr.append("");
        _homeAdr.append("");
        break;
      default:
        _homeAdr=tempList;
        for(uint i=0;i<7-tempList.count();i++)
          _homeAdr.append("");
    }
  }
  
  /**
   *  getting the phones
   */
  _phone.clear();
  matches=mObj.query("TEL","home");
  if( matches.count() )
  {
    setPhone("home", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","work");
  if( matches.count() )
  {
    setPhone("work", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","cell");
  if( matches.count() )
  {
    setPhone("mobile", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","fax");
  if( matches.count() )
  {
    setPhone("fax", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","pager");
  if( matches.count() )
  {
    setPhone("pager", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","application/sdp");
  if( !matches.count() )
  {
    matches=mObj.query("TEL","application/isup");
  }
  if( matches.count() )
  {
    setPhone("sip", mObj.getLines().at(matches[0])->getContent());
  }
  
#ifdef MORE_SECURE
  save();
#endif
}

QString ContactObject::vcard2() const
{
  QString result="BEGIN:VCARD\n";
  result.append("VERSION:2.1\n");
  if( !uid().isEmpty() )
   result.append(QString("UID:") + uid() + "\n");
  result.append(QString("FN:")+fn()+"\n");
  result.append(QString("N:")+join(n(),";")+"\n");
  result.append(QString("ADR:HOME;")+join(homeAdr(),";")+"\n");
  result.append(QString("ADR:WORK;")+join(workAdr(),";")+"\n");
  if(!notes().isEmpty()) result.append(QString("NOTE:")+notes()+"\n");
  if(!nick().isEmpty()) result.append(QString("NICKNAME:")+nick()+"\n");
  if(!bday().isEmpty()) result.append(QString("BDAY:")+bday()+"\n");
  if(!org().isEmpty()) result.append(QString("ORG:")+org()+"\n");
  if(!title().isEmpty()) result.append(QString("TITLE:")+title()+"\n");
  if(!role().isEmpty()) result.append(QString("ROLE:")+role()+"\n");
  if(!classCard().isEmpty()) result.append(QString("CLASS:")+classCard()+"\n");
  if(!categories().isEmpty()) result.append(QString("CATEGORIES:")+categories()+"\n");
  //default email...
  if(emails().count()!=0)
  {
    result.append(QString("EMAIL:PREF;INTERNET")+emails()[defaultEmail()]+"\n");
  }
  // the other emails
  for(uint i=0;i<emails().count();i++)
    if(i!=uint(defaultEmail())) result.append(QString("EMAIL:TYPE=PREF")+emails()[i]+"\n");

  // now the phones...
  if (!_phone["home"].isEmpty())
  {
    result.append("TEL;home:" + _phone["home"] + "\n");
  }
  if (!_phone["work"].isEmpty())
  {
    result.append("TEL;work:" + _phone["work"] + "\n");
  }
  if (!_phone["mobile"].isEmpty())
  {
    result.append("TEL;cell:" + _phone["mobile"] + "\n");
  }
  if (!_phone["fax"].isEmpty())
  {
    result.append("TEL;fax:" + _phone["fax"] + "\n");
  }
  if (!_phone["pager"].isEmpty())
  {
    result.append("TEL;pager:" + _phone["pager"] + "\n");
  }
  if (!_phone["sip"].isEmpty())
  {
    result.append("TEL;application/sdp:" + _phone["sip"] + "\n");
  }

  result.append("END:VCARD");
  return result;
}

void ContactObject::setVcard3(const QString & import)
{
  MetaObject mObj(import, 0);

  // check the vcard version
  QString ver;
  if( mObj.query("VERSION").count() )
  {
    ver = mObj["VERSION"].getContent();
  }
  if( ver.isEmpty() ||
      "3.0" != ver )
  {
    // try to import with version 2.1
    if( "2.1" == ver )
    {
      setVcard2(import);
    }
    else
    {
      qDebug("Wrong vcard version %s or not vcard data.", ver.latin1());
    }
    
    return;
  }
  
  if( mObj.query("UID").count() )
  {
    _uid = mObj["UID"].getContent();
  }
  
  if(mObj.query("FN").count()!=0)
    _fn=mObj["FN"].getContent();
  QStringList tempList;
  QString tempString;
  QValueList<int> matches;
  
  /**
   * getting all the emails...
   */
  matches = mObj.query("EMAIL");
  tempList.clear();
  for(uint i=0;i<matches.count();i++)
    tempList.append(mObj.getLines().at(matches[i])->getContent());
  _emails=tempList;
  
  // default email
  matches=mObj.query("EMAIL","TYPE=pref");
  _defaultEmail=0;
  if(matches.count())
  {
    tempString=mObj.getLines().at(matches[0])->getContent();
    for(uint i=0;i<_emails.count();i++)
      if(_emails[i]==tempString)
      {
        _defaultEmail=i;
        break;
      }
  }
  
  /**
   *  getting the name...
   */
  if(mObj.query("N").count()!=0)
    tempList=QStringList::split(';',mObj["N"].getContent());

  // NAME
  if(tempList.count()>0) _n.append(tempList[0]);
  else _n.append("");
  // FAMILY NAME
  if(tempList.count()>1) _n.append(tempList[1]);
  else _n.append("");
  // ADDITIONAL NAMES (INITIAL)
  if(tempList.count()>2) _n.append(tempList[2]);
  else _n.append("");
  // HONOR PREFIXES
  if(tempList.count()>3) _n.append(tempList[3]);
  else _n.append("");
  // HONOR SUFFIXES
  if(tempList.count()>4) _n.append(tempList[4]);
  else _n.append("");

   
  /**             1
   *  getting the title...
   */
  if(mObj.query("TITLE").count()!=0)
    _title=mObj["TITLE"].getContent();
  
  /**
   *  getting the URLs...
   */
  matches=mObj.query("url");
  _URLs.clear();
  {
  for(uint i=0;i<matches.count();i++)
    _URLs.append(mObj.getLines().at(matches[i])->getContent());
  }

  /**
   *  getting the class...
   */
  tempString="";
  if(mObj.query("CLASS").count()!=0)
    tempString=mObj["class"].getContent();
  if( (tempString.find("PRIVATE",0,false)==-1) &&
      (tempString.find("PUBLIC",0,false)==-1) &&
      (tempString.find("CONFIDENTIAL",0,false)==-1))
    tempString="PRIVATE";
   _classCard=tempString;

  /**
   * getting the note
   */
  _notes=QString::null;
  if(mObj.query("NOTE").count()!=0)
    _notes=mObj["NOTE"].getContent();

  /**
   * getting the categories
   */
  _categories=QString::null;
  if(mObj.query("CATEGORIES").count()!=0)
    _categories=mObj["categories"].getContent();

  /**
   * getting the org
   */
  _org=QString::null;
  if(mObj.query("ORG").count()!=0)
    _org=mObj["ORG"].getContent();

  /**
   * getting the role
   */
  _role=QString::null;
  if(mObj.query("ROLE").count()!=0)
    _role=mObj["ROLE"].getContent();

  /**
   * getting the nick
   */
  _nick=QString::null;
  if(mObj.query("NICKNAME").count()!=0)
    _nick=mObj["NICKNAME"].getContent();

  /**
   * getting the bday
   */
  _bday=QString::null;
  if(mObj.query("BDAY").count()!=0)
    _bday=mObj["BDAY"].getContent();

  /**
   * getting the work address
   */
  matches=mObj.query("ADR","TYPE=work");
  if(matches.count())
  {
    // doing the address...
    tempList=split(mObj.getLines().at(matches[0])->getContent(),';');
    switch(tempList.count())
    {
      case 7:
        _workAdr=tempList;
        break;
      case 1:
        _workAdr.clear();
        _workAdr.append("");
        _workAdr.append("");
        _workAdr.append(tempList[0]);
        _workAdr.append("");
        _workAdr.append("");
        _workAdr.append("");
        _workAdr.append("");
        break;
      default:
        _workAdr=tempList;
        for(uint i=0;i<7-tempList.count();i++)
          _workAdr.append("");
    }
  }
  
  /**
   * getting the home address
   */
  matches = mObj.query("ADR","TYPE=home");
  if(!matches.count())
  {
    matches = mObj.query("ADR", QString::null, "TYPE=work");
  }
  if(matches.count())
  {
    // doing the address...
    tempList=split(mObj.getLines().at(matches[0])->getContent(),';');
    switch(tempList.count())
    {
      case 7:
        _homeAdr=tempList;
        break;
      
      case 1:
        _homeAdr.clear();
        _homeAdr.append("");
        _homeAdr.append("");
        _homeAdr.append(tempList[0]);
        _homeAdr.append("");
        _homeAdr.append("");
        _homeAdr.append("");
        _homeAdr.append("");
        break;
      
      default:
        _homeAdr=tempList;
        for(uint i=0;i<7-tempList.count();i++)
          _homeAdr.append("");
    }
  }

  /**
   *  getting the phones
   */
  _phone.clear();
  matches=mObj.query("TEL","TYPE=home");
  if( matches.count() )
  {
    setPhone("home", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","TYPE=work");
  if( matches.count() )
  {
    setPhone("work", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","TYPE=cell");
  if( matches.count() )
  {
    setPhone("mobile", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","TYPE=fax");
  if( matches.count() )
  {
    setPhone("fax", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","TYPE=pager");
  if( matches.count() )
  {
    setPhone("pager", mObj.getLines().at(matches[0])->getContent());
  }
  matches=mObj.query("TEL","TYPE=application/sdp");
  if( !matches.count() )
  {
    matches=mObj.query("TEL","TYPE=application/isup");
  }
  if( matches.count() )
  {
    setPhone("sip", mObj.getLines().at(matches[0])->getContent());
  }
  
#ifdef MORE_SECURE
  save();
#endif
}

QString ContactObject::vcard3() const
{
  QString result="BEGIN:VCARD\n";
  result.append("VERSION:3.0\n");

  if( !uid().isEmpty() )
   result.append(QString("UID:") + uid() + "\n");
  
  if( !fn().isEmpty() )
   result.append(QString("FN:")+fn()+"\n");
  
  result.append(QString("N:")+join(n(),";")+"\n");

  result.append(QString("ADR;TYPE=home:")+join(homeAdr(),";")+"\n");
  result.append(QString("ADR;TYPE=work:")+join(workAdr(),";")+"\n");
  if(!notes().isEmpty()) result.append(QString("NOTE:")+notes()+"\n");
  if(!nick().isEmpty()) result.append(QString("NICKNAME:")+nick()+"\n");
  if(!bday().isEmpty()) result.append(QString("BDAY:")+bday()+"\n");
  if(!org().isEmpty()) result.append(QString("ORG:")+org()+"\n");
  if(!title().isEmpty()) result.append(QString("TITLE:")+title()+"\n");
  if(!role().isEmpty()) result.append(QString("ROLE:")+role()+"\n");
  if(!classCard().isEmpty()) result.append(QString("CLASS:")+classCard()+"\n");
  if(!categories().isEmpty()) result.append(QString("CATEGORIES:")+categories()+"\n");
  //default email...
  if(emails().count()!=0)
  {
      if( (unsigned)defaultEmail()<=emails().count() )
      result.append(QString("EMAIL;TYPE=internet,pref:")+emails()[defaultEmail()]+"\n");
  }
  // the other emails
  for(uint i=0;i<emails().count();i++)
    if(i!=uint(defaultEmail())) result.append(QString("EMAIL;TYPE=internet:")+emails()[i]+"\n");
  
  // now the phones...
  if (!_phone["home"].isEmpty())
  {
    result.append("TEL;TYPE=home:" + _phone["home"] + "\n");
  }
  if (!_phone["work"].isEmpty())
  {
    result.append("TEL;TYPE=work:" + _phone["work"] + "\n");
  }
  if (!_phone["mobile"].isEmpty())
  {
    result.append("TEL;TYPE=cell:" + _phone["mobile"] + "\n");
  }
  if (!_phone["fax"].isEmpty())
  {
    result.append("TEL;TYPE=fax:" + _phone["fax"] + "\n");
  }
  if (!_phone["pager"].isEmpty())
  {
    result.append("TEL;TYPE=pager:" + _phone["pager"] + "\n");
  }
  if (!_phone["sip"].isEmpty())
  {
    result.append("TEL;TYPE=application/sdp:" + _phone["sip"] + "\n");
  }
  
  result.append("END:VCARD\n");
  result.append("\n");

  return result;
}

QStringList ContactObject::relatedObjectList() const
{
  return _relatedObjectList;
}

QString ContactObject::uid() const
{
  return _uid;
}

QString ContactObject::getUID() const
{
  return QUuid::createUuid().toString();
}

void ContactObject::setRelatedObjectList(const QStringList &newRelatedObjectList)
{
  _relatedObjectList=newRelatedObjectList;
}

  bool
ContactObject::operator == (const VContactObject & other) const
{
  // Compare phones.

  if (_phone.count() != other._phone.count())
    return false;

  QMap<QString, QString>::ConstIterator it;

  for (it = _phone.begin(); it != _phone.end(); ++it)
  {
    if (_phone[it.key()] != other._phone[it.key()])
      return false;
  }

  // Everything else.
  return
    (
      _uid                 == other._uid
      &&
      _fn                 == other._fn
      &&
      _n                  == other._n
      &&
      _homeAdr            == other._homeAdr
      &&
      _workAdr            == other._workAdr
      &&
      _emails             == other._emails
      &&
      _defaultEmail       == other._defaultEmail
      &&
      _notes              == other._notes
      &&
      _URLs               == other._URLs
      &&
      _nick               == other._nick
      &&
      _bday               == other._bday
      &&
      _org                == other._org
      &&
      _role               == other._role
      &&
      _title              == other._title
      &&
      _classCard          == other._classCard
      &&
      _categories         == other._categories
      &&
      _category           == other._category
      &&
      _security           == other._security
      &&
      _relatedObjectList  == other._relatedObjectList
      &&
      _extraData          == other._extraData
    );
}

