/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 /**
   * Changed by the Eugen C. <eug@thekompany.com> for kore plug in support.
   *  Date: 27 Sep. 2001
   */

#ifndef __ContactObject__
#define __ContactObject__

#include <qstring.h>
#include <qfont.h>
#include <qsize.h>
#include <qpoint.h>
#include <qdatetime.h>
#include <qstringlist.h>
#include <qmap.h>

#include <defines.h>

#ifdef Q_WS_WIN
#if defined(Q_TEMPLATEDLL)
// MOC_SKIP_BEGIN
template class CONTACT_PLUGIN_API QMap<QString, QString>;
// MOC_SKIP_END
#endif
#endif

class CONTACT_PLUGIN_API VContactObject;

/**
 * the contact (business card) object class
 */
class CONTACT_PLUGIN_API ContactObject
{
  public:
    /**
     *  The types for the security
     */
    enum { Private=0, Public=1, Confidential=2 };
    
    /**
     *  The types for the phones (these are fixed)
     */
//    tel-type     = "HOME" / "WORK" / "PREF" / "VOICE" / "FAX" / "MSG"
//                 / "CELL" / "PAGER" / "BBS" / "MODEM" / "CAR" / "ISDN"
//                 / "VIDEO" / "PCS" / iana-token / x-name
    enum { Home=1, Work=1>>1, Pref=1>>2,
           Voice=1>>3, Msg=1>>4, Fax=1>>5, Cell=1>>6, Pager=1>>7,
           Bbs=1>>8, Modem=1>>9, Car=1>>10, Isdn=1>>11,
           Pcs=1>>12, Video=1>>13, SIP=1>>14 };

    ContactObject();
    ContactObject(const VContactObject &);

    virtual bool operator == (const VContactObject &) const;
    
    virtual ~ContactObject();

    /**
     *  Phone type conversion method:
     */
    static unsigned convertPhoneType(QString);
    
    /**
     * get/set methods for internal properties
     */
    virtual void setFn(const QString &);
    virtual QString fn() const;
    virtual void setN(const QStringList &);
    virtual QStringList n() const;
    virtual void setHomeAdr(const QStringList &);
    virtual QStringList homeAdr() const;
    virtual void setPhone(const QString & key, const QString & value);
    virtual void setPhone(const QMap<QString, QString> &);
    virtual QMap<QString, QString> phone() const;
    virtual QString phone(const QString & key) const;
    virtual void setWorkAdr(const QStringList &);
    virtual QStringList workAdr() const;
    virtual void setEmails(const QStringList &);
    virtual QStringList emails() const;
    virtual void setDefaultEmail(int);
    virtual int defaultEmail() const;
    virtual void setNotes(const QString &);
    virtual QString notes() const;
    virtual void setURLs(const QStringList &);
    virtual QStringList URLs() const;
    virtual void setNick(const QString &);
    virtual QString nick() const;
    virtual void setSecurity(int newSecurity=Private);
    virtual int security() const;
    virtual void setBday(const QString &);
    virtual QString bday() const;
    virtual void setOrg(const QString &);
    virtual QString org() const;
    virtual void setRole(const QString &);
    virtual QString role() const;
    virtual void setTitle(const QString &);
    virtual QString title() const;
//    virtual void setPos(const QString &);
//    virtual QString pos() const;
    virtual void setClassCard(const QString &);
    virtual QString classCard() const;
    virtual void setCategories(const QString &);
    virtual QString categories() const;
    virtual void setVcard2(const QString &);
    virtual QString vcard2() const;
    virtual void setVcard3(const QString &);
    virtual QString vcard3() const;
    virtual void setCategory(const QString &);
    virtual QString category() const;
    virtual void setRelatedObjectList(const QStringList &);
    virtual QStringList relatedObjectList() const;
    virtual QString uid() const;
    
  protected:
  
    QString getUID() const;
    
    /**
     * internal data
     */
    QString _fn;
    QStringList _n;
    QStringList _homeAdr;
    QStringList _workAdr;
    QMap<QString, QString> _phone;
    QStringList _emails;
    int _defaultEmail;
    QString _notes;
    QStringList _URLs;
    QString _nick;
    QString _bday;
    QString _org;
    QString _role;
    QString _title;
    QString _classCard;
    QString _categories;
    QString _category;
    int _security;
    QStringList _relatedObjectList;
    QString _uid;
    QByteArray _extraData;
};

#endif







