/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
  Copyright: theKompany.com
  Date     : 16 August 2003
  Author   : Ghiciuc Ilie (ilie@eco-spaces.com.ro)
  Author   : Eugen Constantinescu (eugen@eco-spaces.com.ro)
---------------------------------------------------------------------------------
*/

//own includes
#include "contactsearchservice.h"
#include "vcontactobject.h"

//Aethera includes
#include <vfolder.h>

ContactSearchService * ContactSearchService::instance_ = 0;

/*
----------------------------------------------------------------------------------------
The factory
----------------------------------------------------------------------------------------
*/

kore::Interface* contactsearchservice_factory()
{
  return ContactSearchService::instance();
}

/*
----------------------------------------------------------------------------------------
The ContactSearchService
----------------------------------------------------------------------------------------
*/

ContactSearchService * ContactSearchService::instance()
{
  if( !instance_ )
  {
    instance_ = new ContactSearchService();
  }
  
  return instance_;
}

ContactSearchService::ContactSearchService()
  : vfs(0L)
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "ContactSearchService");
  info->replaceAll("class", "ContactSearchService");
  info->replaceAll("class/fqn", "ContactSearchService");
  info->replaceAll("description", CONTACT_SEARCH_SERVICE_DESC );
  info->replaceAll("factory", "contactsearchservice_factory");
  info->insert("service",  AETHERA_SERVICE );

  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
    else
      qDebug("ContactSearchService: Can't find VFS module");
  }
  Q_CHECK_PTR(vfs);
  
  contactsRoot_ = vfs->root() + "/Contacts";
}

ContactSearchService::~ContactSearchService()
{
}

bool ContactSearchService::inAddressBook(const QString& field, const QString& info)
{
  //search for email
  if( field == CONTACT_EMAIL )
  {
    QStringList contactLst = getAllContactsList();
    for(QStringList::Iterator it=contactLst.begin(); it!=contactLst.end(); ++it)
    {
      VContactObject * contact = static_cast<VContactObject*>
        (vfs->_fact_ref((*it), VFS_CONTACTS_MIMETYPE ));
      if( contact )
      {
        QStringList emails = contact->emails();
        int index = emails.findIndex( info );
        if( index > -1 )
          return true;
      }
    }
  }  
  
  return false;
}

QStringList ContactSearchService::getAllContactsList(const QString & folder, bool recursive)
{
  QStringList resultList;
  
  buildContactsList(resultList, folder, recursive);
  
  return resultList;
}

QString ContactSearchService::contactsRoot()
{
  return contactsRoot_;
}

void ContactSearchService::buildContactsList(
    QStringList & resultList,
    const QString& path,
    bool recursive)
{
  QString folderPath = path;
  if( folderPath.isEmpty() )
  {
    folderPath = contactsRoot();
  }
  
  VFolder * folder = vfs->folderRef(folderPath);
  if(folder)
  {
    QStringList urls = vfs->children( folder->path() );
    vfs->createVInstances( urls, false );
    
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      if( folder->isSimpleObject(folder->path() + "/" + *it) )
      {
        VContactObject* contact = static_cast<VContactObject*>( vfs->_fact_ref(folder->path() + "/" + *it) );
        if( contact )
        {
          resultList.append( contact->path() );
        }
      }
    }
    
    if(recursive)
    {
      VFolderList subfolders = folder->folderList();
      VFolderListIterator it1( subfolders );
      VFolder * subfolder = 0;
      while( (subfolder=it1.current()) != 0 )
      {
        buildContactsList(resultList, subfolder->path());
        ++it1;
      }
    }
  }
}

void ContactSearchService::freeContacts(const QString & path, bool recursive)
{
  QString folderPath = path;
  if( folderPath.isEmpty() )
  {
    folderPath = contactsRoot();
  }
  
  VFolder * folder = vfs->folderRef(folderPath);
  if(folder)
  {
    folder->freeObjects();
    
    if(recursive)
    {
      VFolderList subfolders = folder->folderList();
      VFolderListIterator it1( subfolders );
      VFolder * subfolder = 0;
      while( (subfolder=it1.current()) != 0 )
      {
        if( !subfolder->path().isEmpty() )
        {
          freeContacts(subfolder->path(), recursive);
        }
        ++it1;
      }
    }
  }
}
    
