/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.

---------------------------------------------------------------------------------
    copyright             : (C) 2003 by theKompany.com
                            (C) 2004 by Atol C&D
    author                : Eugen Constantinescu <eug@thekompany.com>
                            Fabien Armand <far@atolcd.com>
    date                  : 15 Dec 2003
---------------------------------------------------------------------------------
*/

// own includes
#include "contactskolabinterface.h"
#include "vcontactobject.h"
#include "komcontactsuiservice.h"
#include <metaobject.h>

// Aethera
#include <mailobj.h>
#include <partobj.h>
#include <vfsmodule.h>
#include <clientrmi.h>
#include <vfolder.h>
#include <appstatus.h>
#include <mailagentservice.h>
// support for kolab shared folder
#include <kolabsharedfolder.h>

// qt
#include <qtextcodec.h>
#include <qtimer.h>

ContactsKolabInterface* ContactsKolabInterface::instance_ = 0L;

class ContactsKolabInterface::Private
{
  public:
  
    Private(ContactsKolabInterface*);
    ~Private();
  
    VFSModule * vfs;
    ClientRMIModule * clientRMI;
    MailAgentService * mailAgent;
    QString accountName;
    QDict<VContactObject> synchObjects;
    bool lock;
    
  private:
  
    ContactsKolabInterface * parent_;
};

ContactsKolabInterface::Private::Private(ContactsKolabInterface * parent)
  : vfs(0L),
    clientRMI(0L),
    mailAgent(0L),
    synchObjects(199),
    lock(false),
    parent_(parent)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILAGENT_SERVICE );
    if( ref.isValid() )
       mailAgent = static_cast<MailAgentService*>( ref.instance() );
  }
  
  assert(0 != vfs);
  assert(0 != clientRMI);
}

ContactsKolabInterface::Private::~Private()
{
}

// --------------

ContactsKolabInterface * ContactsKolabInterface::instance()
{
  if( !instance_ )
  {
    instance_ = new ContactsKolabInterface();
  }
  
  return instance_;
}

ContactsKolabInterface::ContactsKolabInterface()
{
  d = new Private(this);
}

ContactsKolabInterface::~ContactsKolabInterface()
{
  delete d;
  d = 0;
}

QString ContactsKolabInterface::kolabPath() const
{
  return d->vfs->root() + "/Contacts/Kolab/";
}

void ContactsKolabInterface::deleteKolabSharedFolder(const QString & path) const
{
  VFolder * contactsFolder = d->vfs->folderRef(path);
  if( contactsFolder )
  {
    // try with the current folder
    QString emailFolder = isKolabSharedFolder(path);
    if( !emailFolder.isEmpty() )
    {
      KolabSharedFolder::ref()->removeSharedFolder(d->accountName, emailFolder);
    }
    
    // check the subfolders
    VFolderList folderRefList;
    contactsFolder->tree(folderRefList, -1);
    
    for(VFolderListIterator it(folderRefList); it.current(); ++it)
    {
      emailFolder = isKolabSharedFolder(it.current()->path());
      if( !emailFolder.isEmpty() )
      {
        KolabSharedFolder::ref()->removeSharedFolder(d->accountName, emailFolder);
      }
    }
  }
}

QString ContactsKolabInterface::isKolabSharedFolder(const QString & path) const
{
  QString emailFolder;
  QString contactsKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( !path.find(contactsKolab + *itList) )
    {
      QString account = (*itList);
      QString mailbox = path.mid( QString(contactsKolab + account + "/").length() );
      
      if( KolabSharedFolder::ref()->isKolabSharedFolder(account, mailbox) )
      {
        emailFolder = d->vfs->root() + "/Mail/" + account + "/" + mailbox;
        d->accountName = account;
      }
      
      break;
    }
  }

  return emailFolder;
}

QString ContactsKolabInterface::isParentKolabSharedFolder(const QString & path) const
{
  QString emailFolder;
  QString contactsKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( !path.find(contactsKolab + *itList) )
    {
      QString account = (*itList);
      QString mailbox = path.mid( QString(contactsKolab + account + "/").length() );
      emailFolder = d->vfs->root() + "/Mail/" + account + "/" + mailbox;
      
      QStringList sharedFolders;
      KolabSharedFolder::ref()->sharedFolders(sharedFolders, account, "contacts");
      
      for(QStringList::Iterator it=sharedFolders.begin(); it!=sharedFolders.end(); ++it)
      {
        if( !(*it).find(emailFolder) )
        {
          return emailFolder;
        }
      }
    }
  }

  return QString::null;
}

QString ContactsKolabInterface::isKolabFolder(const QString & path) const
{
  QString emailFolder;
  QString contactsKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( path == contactsKolab + (*itList) )
    {
      QStringList kolabFolders;
      
      KolabSharedFolder::ref()->kolabFolders(kolabFolders, (*itList), "contacts");
      if( kolabFolders.count() )
      {
        emailFolder = kolabFolders.first();
        d->accountName = (*itList);
      }
      
      break;
    }
  }

  return emailFolder;
}
     
void ContactsKolabInterface::slotSynch(const QString & accName, const QString & mailbox, bool)
{
  if( KolabSharedFolder::ref()->isKolabSharedFolder(accName, mailbox) )
  {
    slotSynchSharedFolder(kolabPath() + accName + "/" + mailbox);
    return;
  }

  if( d->lock )
  {
    return;
  }
  d->lock = true;
  
  // check for kolab object
  QString kolab = kolabPath() + accName;
  VFolder * contactsFolder = d->vfs->folderRef(kolab);
  if( contactsFolder )
  {
    QString contactsPath = contactsFolder->path();
    QStringList urls = d->vfs->children( contactsPath );
    QStringList partsUrl;
    
    // remove all the contacts objects from the folder
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      partsUrl.append( contactsPath + "/" + (*it) );
    }
    d->vfs->remove(partsUrl);
    
    QString folderPath = d->vfs->root() + "/Mail/" + accName + "/" + mailbox;
    VFolder * folder = d->vfs->folderRef( folderPath );
    if( !folder )
    {
      qDebug("No such kolab folder.");
      d->lock = false;
      return;
    }
      
    AppStatus * status = AppStatus::instance();
    if( !status )
    {
      qDebug("Kolab calendar : no status bar");
    }
    
    QString current;
    urls = d->vfs->children( folderPath );
    partsUrl.clear();
    for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
    {
      current = (*it1) + ATTR_SEP + QString::number(0);
      partsUrl.append( current );
    }
    
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), 0, partsUrl.count());
    qApp->processEvents();
    
    d->vfs->createVInstances(urls, false);
    d->vfs->createVInstances(partsUrl, false);
    
    int i = 1;
    for(QStringList::Iterator itParts=urls.begin(); itParts!=urls.end(); ++itParts)
    {
      current = (*itParts) + ATTR_SEP + QString::number(0);
  
      PartObjectInterface *part = static_cast<PartObjectInterface*>
        (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));
        
      if( part )
      {
        if( "attachment" == part->partType() &&
            part->mimeType().contains("card") )
        {
          QByteArray data_ = part->data();
          if( !data_.size() )
          {
            qDebug("contacts data is null");
          }
          
          QCString partData( data_ );
          QString contactText;
          
          QTextCodec * codec = QTextCodec::codecForName( part->charset() );
          if(codec)
             contactText = codec->toUnicode( partData );
          else
          {
            if( "utf-8" == part->charset().lower() )
              contactText = QString::fromUtf8( partData );
            else if( "8bit" == part->encoding().lower() )
              contactText = QString::fromLocal8Bit( partData );
            else // 7 bit
              contactText = QString::fromLatin1( partData );
          }
          
          // Create a contact with text = contactText and save it
          QString contactName = d->vfs->createObject( contactsFolder->path() );
          if( !contactName.isEmpty() )
          {
            int begin = contactText.find("BEGIN:VCARD", 0, false);
            int end = contactText.find("END:VCARD", 0, false);
               
            if( begin != -1 && end != -1 && begin < end )
            {
              VContactObject * contactObject = static_cast<VContactObject*>
                ( d->vfs->_fact_ref(contactsFolder->path() + "/" + contactName, VFS_CONTACTS_MIMETYPE) );
              
              if( contactObject )
              {
                // add the object for synchronization
                d->synchObjects.insert(contactObject->path(), contactObject);
              
                contactObject->setVcard3( contactText.mid(begin) );
                contactObject->save();
              }
              else
              {
                qDebug("Couldn't create the contact object.");
              }
            }
          }
        }
      }
      else
      {
        qDebug("Warning: the contact for %s isn't available.", current.latin1());
      }
      
      if( !(i%50) )
      {
        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), i);
        qApp->processEvents(1);
      }
      i++;
    }
    
    status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, tr("Kolab contacts are loaded"));
    QTimer::singleShot(0, this, SLOT(slotClearSynch()));
    qApp->processEvents();
  }
  else
  {
    qDebug("No such kolab folder.");
  }
  
  d->lock = false;
}

void ContactsKolabInterface::slotSynchSharedFolder(const QString & contactsPath)
{
  QString folderPath = isKolabSharedFolder(contactsPath);
  if( d->lock || folderPath.isEmpty() )
  {
    return;
  }
  
  d->lock = true;

  // check for kolab object
  VFolder * contactsFolder = d->vfs->folderRef(contactsPath);
  if( contactsFolder )
  {
    QString contactsPath = contactsFolder->path();
    QStringList urls = d->vfs->children( contactsPath );
    QStringList partsUrl;

    // remove all the contacts objects from the folder
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      partsUrl.append( contactsPath + "/" + (*it) );
    }
    d->vfs->remove(partsUrl);

    VFolder * folder = d->vfs->folderRef( folderPath );
    if( !folder )
    {
      qDebug("No such kolab folder.");
      d->lock = false;
      return;
    }

    AppStatus * status = AppStatus::instance();
    if( !status )
    {
      qDebug("Kolab calendar : no status bar");
    }

    QString current;
    urls = d->vfs->children( folderPath );
    partsUrl.clear();
    for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
    {
      current = (*it1) + ATTR_SEP + QString::number(0);
      partsUrl.append( current );
    }

    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), 0, partsUrl.count());
    qApp->processEvents();
    
    d->vfs->createVInstances(urls, false);
    d->vfs->createVInstances(partsUrl, false);

    int i = 1;
    for(QStringList::Iterator itParts=urls.begin(); itParts!=urls.end(); ++itParts)
    {
      current = (*itParts) + ATTR_SEP + QString::number(0);

      PartObjectInterface *part = static_cast<PartObjectInterface*>
        (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));

      if( part )
      {
        if( "attachment" == part->partType() &&
            part->mimeType().contains("card") )
        {
          QByteArray data_ = part->data();
          if( !data_.size() )
          {
            qDebug("contacts data is null");
          }

          QCString partData( data_ );
          QString contactText;

          QTextCodec * codec = QTextCodec::codecForName( part->charset() );
          if(codec)
             contactText = codec->toUnicode( partData );
          else
          {
            if( "utf-8" == part->charset().lower() )
              contactText = QString::fromUtf8( partData );
            else if( "8bit" == part->encoding().lower() )
              contactText = QString::fromLocal8Bit( partData );
            else // 7 bit
              contactText = QString::fromLatin1( partData );
          }

          // Create a contact with text = contactText and save it
          QString contactName = d->vfs->createObject( contactsFolder->path() );
          if( !contactName.isEmpty() )
          {
            int begin = contactText.find("BEGIN:VCARD", 0, false);
            int end = contactText.find("END:VCARD", 0, false);

            if( begin != -1 && end != -1 && begin < end )
            {
              VContactObject * contactObject = static_cast<VContactObject*>
                ( d->vfs->_fact_ref(contactsFolder->path() + "/" + contactName, VFS_CONTACTS_MIMETYPE) );

              if( contactObject )
              {
                // add the object for synchronization
                d->synchObjects.insert(contactObject->path(), (VContactObject*)0);
              
                contactObject->setVcard3( contactText.mid(begin) );
                contactObject->save();
              }
              else
              {
                qDebug("Couldn't create the contact object.");
              }
            }
          }
        }
      }
      else
      {
        qDebug("Warning: the contact for %s isn't available.", current.latin1());
      }

      if( !(i%50) )
      {
        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), i);
        qApp->processEvents(1);
      }
      i++;
    }

    status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, tr("Kolab contacts are loaded"));
    QTimer::singleShot(0, this, SLOT(slotClearSynch()));
    qApp->processEvents();
  }
  else
  {
    qDebug("No such kolab folder.");
  }

  d->lock = false;
  
}

void ContactsKolabInterface::slotClearSynch()
{
  d->synchObjects.clear();
}

void ContactsKolabInterface::slotDelete(const QString & path)
{
  if( d->lock )
  {
    return;
  }
  
  VContactObject * contactObject = static_cast<VContactObject*>
    ( d->vfs->_fact_ref(path, VFS_CONTACTS_MIMETYPE) );
  
  if( contactObject )
  {
    if( d->synchObjects[contactObject->path()] )
    {
      // it's in synchronization
      return;
    }
    
    QString mailFolder = findMailFolder( contactObject->parentFolder() );
    if( !mailFolder.isEmpty() )
    {
      QString mailObject = findMailByUID(mailFolder, contactObject->uid());
      if( mailObject.isEmpty() )
      {
        qDebug("slotDelete : The mail object couldn't be found.");
      }
      else
      {
        d->vfs->remove(mailObject);
      }
    }
  }
}

void ContactsKolabInterface::slotUpdate(const QString & path)
{
  if( d->lock )
  {
    return;
  }
  
  VContactObject * contactObject = static_cast<VContactObject*>
    ( d->vfs->_fact_ref(path, VFS_CONTACTS_MIMETYPE) );
  
  if( contactObject )
  {
    if( d->synchObjects[contactObject->path()] )
    {
      // it's in synchronization
      return;
    }
    
    QString mailFolder = findMailFolder( contactObject->parentFolder() );
    if( !mailFolder.isEmpty() )
    {
      Account kolabAccount = d->clientRMI->getAccount(d->accountName);
      QString mailObject = findMailByUID(mailFolder, contactObject->uid());
      if( mailObject.isEmpty() )
      {
        qDebug("slotDelete : The mail object couldn't be found.");
      }
      else
      {
        d->vfs->remove(mailObject);
      }
      
      if( d->mailAgent )
      {
        QString email;
        if( kolabAccount.name.isEmpty() )
          email = kolabAccount.from;
        else
          email = QString("\"") + kolabAccount.name + "\"" +
                          " <"  + kolabAccount.from + ">";
          
        d->mailAgent->appendToFolder(
          mailFolder,
          kolabAccount.accname,
          email,
          email,
          QString::null,
          QString::null,
          QString("Contact: ") + contactObject->uid(),
          contactObject->vcard3(),
          "text/x-vcard");
      }
    }
  }
}

void ContactsKolabInterface::slotAdd(const QString & path)
{
  if( d->lock )
  {
    return;
  }
  
  VContactObject * contactObject = static_cast<VContactObject*>
    ( d->vfs->_fact_ref(path, VFS_CONTACTS_MIMETYPE) );
  
  if( contactObject )
  {
    if( contactObject->uid().isEmpty() )
    {
      return;
    }
  
    if( d->synchObjects[contactObject->path()] )
    {
      // it's in synchronization
      return;
    }
  
    QString mailFolder = findMailFolder( contactObject->parentFolder() );
    if( !mailFolder.isEmpty() )
    {
      Account kolabAccount = d->clientRMI->getAccount(d->accountName);
      if( d->mailAgent )
      {
        QString email;
        if( kolabAccount.name.isEmpty() )
          email = kolabAccount.from;
        else
          email = QString("\"") + kolabAccount.name + "\"" +
                          " <"  + kolabAccount.from + ">";
          
        d->mailAgent->appendToFolder(
          mailFolder,
          kolabAccount.accname,
          email,
          email,
          QString::null,
          QString::null,
          QString("Contact: ") + contactObject->uid(),
          contactObject->vcard3(),
          "text/x-vcard");
      }
    }
  }
}

QString ContactsKolabInterface::findMailByUID
  ( const QString & folderPath,
    const QString & uid ) const
{
  VFolder * folder = d->vfs->folderRef(folderPath);
  if(folder)
  {
    bool bFind = false;
    QStringList urls = d->vfs->children( folderPath );
    d->vfs->createVInstances(urls, false);
    
    MailObject * object = 0L;
    // try to find the UID in the subject line
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      object = static_cast<MailObject*> (d->vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE));
      if(object)
      {
        if( -1 != object->subject().find(uid) )
        {
          bFind = true;
          break;
        }
      }
    }
    // hmm, try to find the UID in the mail body
    // this can take a while :(
    if(!bFind)
    {
      QStringList partsUrl;
      QString current;
      for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
      {
        current = (*it1) + ATTR_SEP + QString::number(0);
        partsUrl.append( current );
      }
      
      d->vfs->createVInstances(partsUrl, false);
      
      for(QStringList::Iterator itParts=urls.begin(); itParts!=urls.end() && !bFind; ++itParts)
      {
        current = (*itParts) + ATTR_SEP + QString::number(0);
    
        PartObjectInterface *part = static_cast<PartObjectInterface*>
          (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));
          
        if( part )
        {
          if( "attachment" == part->partType() &&
              part->mimeType().contains("card") )
          {
            QByteArray data_ = part->data();
            if( !data_.size() )
            {
              qDebug("calendar data is null");
            }
            
            QCString partData( data_ );
            QString vcard;
            
            QTextCodec * codec = QTextCodec::codecForName( part->charset() );
            if(codec)
               vcard = codec->toUnicode( partData );
            else
            {
              if( "utf-8" == part->charset().lower() )
                vcard = QString::fromUtf8( partData );
              else if( "8bit" == part->encoding().lower() )
                vcard = QString::fromLocal8Bit( partData );
              else // 7 bit
                vcard = QString::fromLatin1( partData );
            }
            
            int begin = vcard.find("BEGIN:VCARD", 0, false);
            int end = vcard.find("END:VCARD", 0, false);
               
            if( begin != -1 && end != -1 && begin < end )
            {
              MetaObject mObj(vcard.mid(begin), 0L);
              if( mObj.query("UID").count() &&
                  mObj["UID"].getContent() == uid )
              {
                object = static_cast<MailObject*>
                  (d->vfs->_fact_ref((*itParts), VFS_MAILS_MIMETYPE));
                if(object)
                {
                  bFind = true;
                }
              }
            }
          }
        }
      }
    }
    
    if(bFind)
    {
      return object->path();
    }
    
    folder->freeObjects();
  }
  
  return QString::null;
}

QString ContactsKolabInterface::findMailFolder(VFolder * folder) const
{
  QString emailFolder;
  
  if( folder )
  {
    emailFolder = isKolabFolder( folder->path() );
    if( emailFolder.isEmpty() )
    {
      emailFolder = isKolabSharedFolder( folder->path() );
    }
  }
  
  return emailFolder;
}
