/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    contactview.cpp
                             -------------------
    begin                : Tue Nov 27 17:14:32 GMT 2001
    copyright            : (C) 2001 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Rik Hemsley <rik@kde.org>
                           Fabien Armand <far@atolcd.com>
 ***************************************************************************/

#include <qcursor.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qregexp.h>
#include <qfile.h>
#include <qtextcodec.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qsplitter.h>
#include <qlayout.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qsimplerichtext.h>

#include <externalbrowser.h>
#include <vdragobject.h>
#include <folderchooserdialog.h>
#include <clipboard.h>
#include <qiconloader.h>
#include <contactsmainwidget.h>
#include <contactslistview.h>
#include <contactslistviewitem.h>
#include <vfsmodule.h>
#include <vcontactobject.h>
#include <contactrichtextfactory.h>
#include <contacteditor.h>
#include <contactpreviewwidget.h>
#include <ataddressbook.h>
#include <appstatus.h>
#include <mimecodec.h>
#include <contactskolabinterface.h>
#include <charsets.h>

#include <math.h>

/*
 * m_cutList concept:
 * When we cut items, we add their paths to m_cutList.
 * When we refresh, we avoid adding anything on m_cutList.
 * When we paste, we remove any pasted items from m_cutList.
 * When we start a new cut or copy, or we are deleted, we remove any
 * items in m_cutList from VFS.
 */

ContactsMainWidget::ContactsMainWidget
(
  VFSModule   * vfsModule,
  QWidget     * parent,
  const char  * name
)
  : QWidget(parent, name),
    m_currentPath ("/Aethera/VFS/Contacts"),
    m_vfsModule   (vfsModule)
{
  QSplitter * splitter = new QSplitter(Vertical, this);

  QVBoxLayout * layout = new QVBoxLayout(this);

  layout->addWidget(splitter);

  m_contactListView = new ContactsListView(splitter);
  m_contactView = new ContactPreviewWidget(splitter);

  connect
    (
      m_contactListView,
      SIGNAL(currentChanged(QListViewItem *)),
      SLOT(slotCurrentChanged(QListViewItem *))
    );
 
  connect
    (
      m_contactListView,
      SIGNAL(doubleClicked(QListViewItem *)),
      SLOT(slotDoubleClicked(QListViewItem *))
    );

  connect
    (
      m_contactListView,
      SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)),
      SLOT(slotContextMenuRequested(QListViewItem *, const QPoint &, int))
    );

  connect
    (
      m_contactListView,
      SIGNAL(selectionChanged()),
      SLOT(slotSelectionChanged())
    );



  connect
    (
      ContactEditor::instance(),



      SIGNAL(editFinished(VContactObject *, bool)),
      SLOT(slotEditFinished(VContactObject *, bool))
    );

  // Make the preview larger than the list.
  QValueList<int> sizeList;
  sizeList << height() / 3 << height() / 3 * 2;
  splitter->setSizes(sizeList);
}

ContactsMainWidget::~ContactsMainWidget()
{
  deleteItemsInCutList();
}

  void
ContactsMainWidget::slotSetFolder(QString path)
{
//  qDebug("ContactsMainWidget::slotSetFolder(%s)", path.ascii());

  if (path.isEmpty())
  {

    // Use same path.

    path = m_currentPath;
  }

  if (0 == path.contains(QRegExp("^/Aethera/VFS/Contacts")))
  {
//    qDebug("Not our folder. Give up.");
    return;
  }

  setUpdatesEnabled(false);

  m_currentPath = path;

//  qDebug("Fixed path is now %s", path.ascii());

  m_contactListView->clear();

  assert(0 != m_vfsModule);

  VFolder * folder = m_vfsModule->folderRef(path);

  if (0 == folder)
  {
    qDebug("Couldn't get ref to folder");
    setUpdatesEnabled(true);
    return;
  }

/*  QStringList urls = m_vfsModule->children( folder->path() );
  m_vfsModule->createVInstances( urls, false );*/
  
  QPtrList<VObject> & objectList = folder->objectList();

//  qDebug("There are %d objects in this folder", objectList.count());

  AppStatus * status = AppStatus::instance();
  status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
    tr("Loading..."), 0, objectList.count());

  int i=1;  
  for (QPtrListIterator<VObject> it(objectList); it.current(); ++it)
  {
    VContactObject * contact = static_cast<VContactObject *>(it.current());

    if (0 == m_cutList.contains(contact->path()))
    {
      (void) new ContactsListViewItem(contact, m_contactListView);
    }
    
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
      tr("Loading..."), i);
    i++;
  }

  setUpdatesEnabled(true);

  slotCurrentChanged(m_contactListView->currentItem());
  
  status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, tr("Folder is loaded"));
}


  QStringList
ContactsMainWidget::getSelection()
{
  QStringList selection;
  ContactsListViewItem * item = 0L;

  for(QListViewItemIterator it(m_contactListView); it.current(); ++it)
  {
    item = static_cast<ContactsListViewItem *> ( it.current() );
    if( item->isSelected() && item->contact() )
    {
      selection.append( item->contact()->path() );
    }
  }
  
  return selection;
}

  VFolder *
ContactsMainWidget::parentFolder()
{
  QString path(m_currentPath);

  if (path.isEmpty())
  {
    path = "/Aethera/VFS/Contacts";
  }

  VFolder * folder = m_vfsModule->folderRef(path);

  Q_ASSERT(0 != folder);

  if (0 == folder)
    return 0;

  else
    return m_vfsModule->parentFolder(folder);
}

  void
ContactsMainWidget::newObject()
{
  if (m_currentPath.isEmpty())
  {
    qDebug("Hmm, I have no path");
    return;
  }

  QString contactName = m_vfsModule->createObject(m_currentPath);

  if (!contactName.isEmpty())
    m_toBeEditedWhenCreated = m_currentPath + "/" + contactName;
}

  void
ContactsMainWidget::selectAll()
{
  m_contactListView->selectAll(true);
}

  void
ContactsMainWidget::unselectAll()
{
  m_contactListView->clearSelection();
}

  bool
ContactsMainWidget::isSelectionEmpty()
{
  return true;
}

  bool
ContactsMainWidget::isSelectionSingle()
{
  switch (m_contactListView->selectionMode())
  {
    case QListView::Single:
      return true;
      break;

    default:
      return false;
      break;
  }
}

  bool
ContactsMainWidget::isSelectionMultiple()
{
  switch (m_contactListView->selectionMode())
  {
    case QListView::Multi:
    case QListView::Extended:
      return true;
      break;

    default:
      return false;
      break;

  }
}

  void
ContactsMainWidget::slotCurrentChanged(QListViewItem * item)

{
  if (0 == item)
  {
    m_contactView->setText(0);
    return;
  }
  //  qDebug("ContactsMainWidget::slotCurrentChanged(%s)", item->text(0).ascii());

  VContactObject * contact =
    static_cast<ContactsListViewItem *>(item)->contact();
    
  if (0 == contact)
  {
    qDebug("0 contact ?");
    return;
  }
                                         
  m_contactView->setContact(contact->path());

}

  void
ContactsMainWidget::slotDoubleClicked(QListViewItem * item)
{
  if (0 == item)
    return;


//  qDebug("ContactsMainWidget::slotDoubleClicked(%s)", item->text(0).ascii());

  QString path(m_currentPath);

  if (path.isEmpty())
  {
    qDebug("No current path");
  }

  VContactObject * contact =
    static_cast<ContactsListViewItem *>(item)->contact();

  QString contactPath(path + "/" + contact->name());

//  qDebug("Contact path: %s", contactPath.ascii());

  VContactObject * obj =
    static_cast<VContactObject *>( m_vfsModule->_fact_ref(contactPath, VFS_CONTACTS_MIMETYPE) );

  if (0 == obj)
  {
    qDebug("VFS couldn't find object");
    return;
  }

  // Looks like dynamic_cast won't work here.
  VContactObject * contactObject = static_cast<VContactObject *>(obj);

  if (0 == contactObject)
  {
    qDebug("contactObject is 0");
    return;
  }

  ContactEditor::instance()->edit(contactObject, this);
}

  void
ContactsMainWidget::slotContextMenuRequested(QListViewItem * i, const QPoint &, int /*col*/)
{
  enum { Create, Remove, Cut, Copy, Paste, Edit, MoveTo, CopyTo, Print };

  int action = -1;

  {
    QPopupMenu pop(m_contactListView);

    if (0 == i)
    {
      pop.insertItem("&New contact", Create);
      pop.insertItem("&Paste",    Paste);

      if (!canPaste())
        pop.setItemEnabled(Paste, false);
    }
    else
    {
      pop.insertItem("&Edit", Edit);
      pop.insertSeparator();
      pop.insertItem("&Cut",  Cut);
      pop.insertItem("&Copy", Copy);
      pop.insertSeparator();
      pop.insertItem("&Remove", Remove);
      pop.insertSeparator();
      pop.insertItem("Copy to...", CopyTo);
      pop.insertItem("&Move to...", MoveTo);
      pop.insertSeparator();
      pop.insertItem("Print", Print);
    }

    action = pop.exec(QCursor::pos());
  }

  // Get selection.

  QPtrList<ContactsListViewItem> selection;

  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
    if (it.current()->isSelected())
      selection.append(static_cast<ContactsListViewItem *>(it.current()));

  switch (action)
  {
    case Create:  newObject();            break;
    case Remove:  remove(selection);      break;
    case Edit:    slotDoubleClicked(i);   break;

    case Cut:     cut(selection);         break;
    case Copy:    copy(selection);        break;
    case Paste:   paste();                break;
    case MoveTo:  moveTo(selection);      break;
    case CopyTo:  copyTo(selection);      break;
    case Print:   printSelection(selection); break;
    default:                              break;
  }
}

  bool
ContactsMainWidget::canPaste() const
{
  QMimeSource * data = ClipboardWrapper::thisInstance()->data();

  if (0 == data)
  {
    qDebug("No data in clipboard.");
    return false;

  }

  if (!QUriDrag::canDecode(data))
  {
    qDebug("Not QUriDrag");
    return false;
  }

  QStringList pathList;

  if (!QUriDrag::decodeLocalFiles(data, pathList))
  {
    qDebug("QUriDrag lied when it said it could decode");
    return false;
  }

  if (pathList.isEmpty())
  {
    qDebug("No URIs");
    return false;
  }

  if (0 == pathList.first().contains(QRegExp("^/Aethera/VFS/Contacts/")))
  {
    qDebug("Not contacts on the clipboard");
    return false;
  }

  return true;
}

  void
ContactsMainWidget::cut(QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
  {
    emit(pastePossible(false));
    return;
  }

  deleteItemsInCutList();

  QStringList pathList = vObjectListToPathList(list);

  qDebug("ContactsMainWidget::cut: %d paths to cut", pathList.count());

  for (
    QStringList::ConstIterator it(pathList.begin());
    it != pathList.end();
    ++it
  )
  {

    m_cutList << *it;
  }

  // Don't do list.setAutoDelete(true), because we're will update before
  // the list would delete itself.

  QUriDrag * dragObject =
    new QUriDrag(m_contactListView, "Contact(s) drag object");

  dragObject->setFileNames(m_cutList);

  ClipboardWrapper::thisInstance()->cutData(dragObject);

  slotDeleteVInstances(m_cutList);

  emit(pastePossible(true));

}

  void
ContactsMainWidget::copy(const QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
  {
    emit(pastePossible(false));
    return;
  }

  deleteItemsInCutList();

  QStringList pathList;

  for (QPtrListIterator<ContactsListViewItem> it(list); it.current(); ++it)
  {
    VContactObject * contact = it.current()->contact();

    if (0 == contact)
    {
      qDebug("ContactsMainWidget::cut: Found a 0 contact");
      return;
    }

    pathList << contact->path();
  }

  QUriDrag * dragObject =
    new QUriDrag(m_contactListView, "Contact(s) drag object");

  dragObject->setFileNames(pathList);

  ClipboardWrapper::thisInstance()->copyData(dragObject);

  emit(pastePossible(true));
}

  void
ContactsMainWidget::paste()
{
  if (!canPaste())
    return;

  QMimeSource * mimeSource = ClipboardWrapper::thisInstance()->pasteData();

  if (0 == mimeSource)
  {
    qDebug("ClipboardWrapper gave me 0 to paste");
    return;
  }

  if (!QUriDrag::canDecode(mimeSource))
  {
    qDebug("Can't decode this mime source");
    return;
  }

  QStringList pathList;

  bool ok = QUriDrag::decodeLocalFiles(mimeSource, pathList);

  if (!ok)
  {
    qDebug("QUriDrag said it could decode, but it was lying.");
    return;
  }
  
  if( !pathList.count() )
    return;

  bool wasInCutList = (0 != m_cutList.remove(pathList[0]));

  if (wasInCutList)
  {
    // Cut 'n' paste.
    m_vfsModule->move(pathList, m_currentPath);
  }
  else
  {
    // Copy 'n' paste.
    m_vfsModule->copy(pathList, m_currentPath);
  }
}

  void
ContactsMainWidget::slotUpdateVInstances(const QStringList & l)
{
  if( !l.count() || 
      !l[0].contains( getCurrentFolder() ) )
  {
    return;
  }
  
  ContactsKolabInterface * kolab = ContactsKolabInterface::instance();
  bool bNormal = false;
  bool bKolab = false;

  QListViewItemIterator it(m_contactListView);
  for (; it.current(); ++it)
  {
    ContactsListViewItem * i = static_cast<ContactsListViewItem *>(it.current());

    if( i && i->contact() && l.contains(i->contact()->path()) )
    {
      AddressBook::thisInstance()->setChanged();
      i->update();
      
      if( !bNormal &&
          (bKolab ||
           !kolab->isKolabFolder(i->contact()->parentFolder()->path()).isEmpty() ||
           !kolab->isKolabSharedFolder(i->contact()->parentFolder()->path()).isEmpty()) )
      {
        bKolab = true;
        kolab->slotUpdate( i->contact()->path() );
      }
      else
      {
        bNormal = true;
      }
    }
  }

  VContactObject * currentPreviewedContact = m_contactView->contact();

  if (0 != currentPreviewedContact)
  {
    if (l.contains(currentPreviewedContact->path()))
      m_contactView->refresh();
  }
}

  void
ContactsMainWidget::slotCreateVInstances(const QStringList & l)
{
  if( !l.count() || 
      !l[0].contains( getCurrentFolder() ) )
  {
    return;
  }
  
  VContactObject * contactObject = 0;
  
  // Kolab
  ContactsKolabInterface * kolab = ContactsKolabInterface::instance();
  if( l.count() && kolab )
  {
    bool bNormal = false;
    bool bKolab = false;
    
    for(QStringList::ConstIterator itr=l.begin(); itr!=l.end(); ++itr)
    {
      contactObject = static_cast<VContactObject *>
        ( m_vfsModule->_fact_ref((*itr), VFS_CONTACTS_MIMETYPE) );
      
      if( contactObject )
      {
        if( !bNormal &&
            (bKolab ||
             !kolab->isKolabFolder(contactObject->parentFolder()->path()).isEmpty() ||
             !kolab->isKolabSharedFolder(contactObject->parentFolder()->path()).isEmpty()) )
        {
          bKolab = true;
          kolab->slotAdd( *itr );
        }
        else
        {
          bNormal = true;
        }
      }
    }
  }
  
  // TODO: Use a more efficient version.

  slotSetFolder("");

  if (m_toBeEditedWhenCreated.isEmpty())
    return;

  QString toEdit = m_toBeEditedWhenCreated;

  m_toBeEditedWhenCreated = QString::null;

  QStringList::ConstIterator it = l.begin();
  for (; it != l.end(); ++it)
  {
    if ((*it) == toEdit)
    {
      contactObject = static_cast<VContactObject *>
        ( m_vfsModule->_fact_ref(toEdit, VFS_CONTACTS_MIMETYPE) );

      if( contactObject )
      {
        ContactEditor::instance()->edit
          (contactObject, this, ContactEditDialog::RemoveIfUnedited);
        
        AddressBook::thisInstance()->setChanged();
      }
    }
  }
}

  void
ContactsMainWidget::slotDeleteVInstances(const QStringList & urls)
{
  if( !urls.count() || 
      !urls[0].contains( getCurrentFolder() ) )
  {
    return;
  }
  
  // TODO: Use a more efficient version.
  QStringList l = urls;

  slotSetFolder("");

  // Kolab
  ContactsKolabInterface * kolab = ContactsKolabInterface::instance();
  
  QPtrList<ContactsListViewItem> deathRow;
  deathRow.setAutoDelete(true);

  ContactsListViewItem * item = 0L;
  QString itemPath;
  QStringList::Iterator itFind;
  for(QListViewItemIterator it(m_contactListView); it.current(); ++it)
  {
    item = static_cast<ContactsListViewItem *>(it.current());
    itemPath = item->contact()->path();

    itFind = l.find(itemPath);
    if( itFind != l.end() )
    {
      deathRow.append(item);
      l.remove(itFind);
      
      if( !kolab->isKolabFolder(item->contact()->parentFolder()->path()).isEmpty() ||
          !kolab->isKolabSharedFolder(item->contact()->parentFolder()->path()).isEmpty() )
      {
        kolab->slotDelete( itemPath );
      }
    }
  }

  if(deathRow.count())
    AddressBook::thisInstance()->setChanged();
    
  deathRow.clear();
}

  void
ContactsMainWidget::deleteItemsInCutList()
{
  if( m_cutList.count() )
  {
    m_vfsModule->remove( m_cutList );
    
    m_cutList.clear();
  }
}

  void
ContactsMainWidget::cut()
{
  QPtrList<ContactsListViewItem> selection;

  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
    if (it.current()->isSelected())
      selection.append(static_cast<ContactsListViewItem *>(it.current()));

  cut(selection);
}

  void
ContactsMainWidget::copy()
{
  QPtrList<ContactsListViewItem> selection;

  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
    if (it.current()->isSelected())
      selection.append(static_cast<ContactsListViewItem *>(it.current()));

  copy(selection);
}

  void
ContactsMainWidget::slotSelectionChanged()
{
  QListViewItemIterator it(m_contactListView);

  for (; it.current(); ++it)
  {
    if (it.current()->isSelected())
    {
      emit(copyPossible(true));
      return;
    }

  }
  emit(copyPossible(false));
}

  void
ContactsMainWidget::moveTo(QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
    return;

  QStringList pathList = vObjectListToPathList(list);

  if( pathList.count() )
  {
    QString targetPath = FolderChooserDialog::getFolder("/Contacts", this);
  
    if (!targetPath || targetPath == m_currentPath)
      return;
  
    m_vfsModule->move(pathList, targetPath);
  }
}

  void
ContactsMainWidget::copyTo(const QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
    return;

  QStringList pathList = vObjectListToPathList(list);

  if( pathList.count() )
  {
    QString targetPath = FolderChooserDialog::getFolder("/Contacts", this);
  
    if (!targetPath || targetPath == m_currentPath)
      return;
  
    m_vfsModule->copy(pathList, targetPath);
  }
}

  QStringList
ContactsMainWidget::vObjectListToPathList
(
  const QPtrList<ContactsListViewItem> & list
) const
{
  if (list.isEmpty())
    return QStringList();

  QStringList pathList;

  for (QPtrListIterator<ContactsListViewItem> it(list); it.current(); ++it)
  {
    VContactObject * contact = it.current()->contact();

    if (0 == contact)
    {
      qDebug("ContactsMainWidget::cut: Found a 0 contact");
      continue;
    }

    pathList << contact->path();
  }


  return pathList;
}

  void
ContactsMainWidget::remove(QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
  {
    return;
  }

  QStringList pathList = vObjectListToPathList(list);
  
  if( pathList.count() )
  {
    assert(0 != m_vfsModule);
    m_vfsModule->remove( pathList );
    
    AddressBook::thisInstance()->setChanged();  
    
  }

}

  void
ContactsMainWidget::slotEditFinished(VContactObject * contact, bool del)
{
  if (del)
  {
    m_vfsModule->remove(contact->path());
  }
}

  void
ContactsMainWidget::importFile(const QString & filename)
{
  QString targetPath = FolderChooserDialog::getFolder("/Contacts", this);

  if( !targetPath.isEmpty() )
  {
    QFile file(filename);

    if( !file.open(IO_ReadOnly) )
    {
//       qDebug("ImportFile %s : %s", filename.latin1(), file.errorString().latin1());
    }
    else
    {
      QTextStream streamer(&file);
      QString information ,charset, encoding;
  
      QString line =  streamer.readLine();
      if( line.isEmpty() )
        streamer.readLine();
      
      bool hasencoding = false;
      
      // search for header info
      while( !line.isEmpty() && 
             -1 == line.find("BEGIN:VCARD", 0, false) )
      {
        // ContentType=text/x-vcard; charset
        if( -1 != line.find("charset", 0, false) )
        {
          hasencoding = true;
          int ch1 = line.find("\"");
          int ch2 = line.find("\"", ch1+1);
          charset = line.mid(ch1+1, ch2-ch1-1);
        }
        // Content-Transfer-Encoding
        else if( -1 != line.find("Content-Transfer-Encoding", 0, false) )
        {
          hasencoding = true;
          int ch1 = line.find(":");
          int ch2 = line.find("\n", ch1+1);
          encoding = line.mid(ch1+1, ch2-ch1-1);
          encoding.stripWhiteSpace();
        }
        
        line = streamer.readLine();
      }
      
      file.reset();
      QCString char_data( file.readAll() );
      
      if( !hasencoding )
      {
        importQString(QString::fromUtf8(char_data), targetPath);
      }
      else
      {
        importEncodedVcard(char_data, charset, encoding, targetPath);
      }
    }
  }
}

  void
ContactsMainWidget::importQString(const QString & cardString, const QString & path)
{
  QString contact(cardString), importContact;
  bool parse = true;
  QStringList list;

  while( !contact.isEmpty() && parse )
  {
    int begin = contact.find("BEGIN:VCARD", 0, false);
    int end = contact.find("END:VCARD", 0, false);
    
    if( begin == -1 || end == -1 || begin >= end )
    {
      parse = false;
    }
    else
    {
      importContact = contact.mid(begin, end-begin+9);
      contact.remove(begin, end-begin+9);

      QString contactName = m_vfsModule->createObject(path);
      QString target = path + "/" + contactName;

      VContactObject * imp = static_cast<VContactObject *>( m_vfsModule->_fact_ref(target,VFS_CONTACTS_MIMETYPE) );
      if( imp!=0 )
      {
        imp->setVcard3(importContact);
        imp->save();
      }
    }
  }
}

  void
ContactsMainWidget::importEncodedVcard( const QCString & data,
                                        const QString & charset,
                                        const QString & encoding,
                                        const QString & path )
{
  QCString decodedData;
  QString contact, importContact;

  decodedData = MIMECodec::decode(data, encoding);

  QTextCodec * codec = QTextCodec::codecForName(charset);
  if( codec )
  {
    contact = codec->toUnicode( decodedData );
  }
  else
  {
    contact = QString::fromUtf8( decodedData );
  }
  
  importQString(contact, path);
}

  void
ContactsMainWidget::exportVcard(const QString & filename)
{
  QString vcard;
  ContactsListViewItem * item = 0;
  VContactObject * contact = 0;

  for(QListViewItemIterator it(m_contactListView); it.current(); ++it)
  {
    if (it.current()->isSelected())
    {
       item = static_cast<ContactsListViewItem *>(it.current());
       if( item )
       {
         contact = item->contact();
         if(0 != contact)
         {
           vcard += contact->vcard3();
           vcard += QString("\n\n");
         }
       }
    }
  }
  
  exportVcardAsUTF8(vcard, filename);
//  exportVcardEncoded(vcard,filename);
}

  void
ContactsMainWidget::exportVcardEncoded(const QString & vcard, const QString & filename)
{
  QString f_name = filename;
  if( ".vcf" != f_name.right(4) )
    f_name += ".vcf";
  
  QCString encodedData;
  QString fileHeader;

  QFile file(f_name);
  if( file.open(IO_WriteOnly) )
  {
    QTextStream stream(&file);
    QTextCodec * codec = 0;
    
    if( vcard != QString::fromUtf8(vcard.utf8()) &&
        (codec = CharSets::thisInstance()->getCodec(vcard)) )
    {
      encodedData = MIMECodec::encodeQuotedPrintable( codec->fromUnicode(vcard) );
      fileHeader += QString("ContentType=text/x-vcard; charset=\"");
      fileHeader += QString( codec->mimeName() ) + "\"\n";
      fileHeader += QString("Content-Transfer-Encoding: quoted-printable\n");
    }
    else
    {
      encodedData = MIMECodec::encodeQuotedPrintable( vcard.utf8() );
      fileHeader += QString("ContentType=text/x-vcard; charset=\"");
      fileHeader += QString("UTF-8\"\n");
      fileHeader += QString("Content-Transfer-Encoding: quoted-printable\n");
    }
  
    stream<<fileHeader;
    stream<<QString("\n\n");
    stream<<encodedData;
  
    file.close();
  }
}
  
  void
ContactsMainWidget::exportVcardAsUTF8(const QString & vcard, const QString & filename)
{
  QString f_name = filename;
  if( ".vcf" != f_name.right(4) )
    f_name += ".vcf";
  
  QCString encodedData = vcard.utf8();

  QFile file(f_name);
  if( file.open(IO_WriteOnly) )
  {
    QTextStream stream(&file);
    stream << encodedData;
    file.close();
  }
}

  QString
ContactsMainWidget::exportVcardString()
{
  QString vcard;
  QPtrList<ContactsListViewItem> selection;

  QListViewItemIterator it(m_contactListView);
  for (; it.current(); ++it)
    if (it.current()->isSelected())
      selection.append(static_cast<ContactsListViewItem *>(it.current()));

  for (QPtrListIterator<ContactsListViewItem> it1(selection); it1.current(); ++it1)
  {
    VContactObject * contact = it1.current()->contact();
    if (0 != contact)
    {
      vcard += contact->vcard3();
      vcard += "\n\n";
    }
  }

  return vcard;
}

  QString
ContactsMainWidget::getCurrentFolder() const
{
  return m_currentPath;
}

// printSelection - print a contact or many contact

void ContactsMainWidget::printSelection(QPtrList<ContactsListViewItem> & list)
{
  if (list.isEmpty())
    return;
  QPrinter printer;

  if (printer.setup(this))   // configure the printer
  {

    QPainter painter(&printer);
    const int largeGap = 48 ;
    QStringList selection=getSelection();

    QString str;

    // create the richText str
            
    for( QStringList::Iterator it=selection.begin(); it!=selection.end(); ++it)
    {
      VContactObject *obj=(VContactObject *)m_vfsModule->_fact_ref((*it), VFS_MAILS_MIMETYPE );
      if(obj)
      {
        // add the generate text from the contact object to the richText str
        str += ContactRichTextFactory::generate(this,obj);
      }

    }

    int pageHeight=painter.window().height() - 2*largeGap;
        
    QSimpleRichText richText(str, QFont(), "",0,0,pageHeight);

    // count the number of pages needed
    int numPages=(int)ceil((double)richText.height() / pageHeight);

    int index;

    for (int i=0;i<(int)printer.numCopies();i++)
    {

      for (int j=0;j < numPages;++j)
      {

        if(i > 0 || j > 0)
            printer.newPage();

        if (printer.pageOrder() == QPrinter::LastPageFirst)
        {
          index = numPages -j - 1;
        }
        else
        {
          index = j;
        }

        QRect rect(0,index * pageHeight + largeGap,richText.width(),pageHeight);

        painter.saveWorldMatrix();

        painter.translate(0, -rect.y());

        richText.draw(&painter,0,largeGap, rect, colorGroup());

        painter.restoreWorldMatrix();

        painter.setFont(QFont());

        painter.drawText(painter.window(),AlignHCenter | AlignBottom,
                      QString::number(index + 1));
        
      }
    }
  }
}
