/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    contactsmainwidget.h
                             -------------------
    begin                : Tue Nov 27 17:15:26 GMT 2001
    copyright            : (C) 2001 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Rik Hemsley <rik@kde.org>
                           Fabien Armand <far@atolcd.com>
 ***************************************************************************/

#ifndef CONTACTS_MAIN_WIDGET_H
#define CONTACTS_MAIN_WIDGET_H

#include <qptrlist.h>

#include <datahandler.h>
#include <komaetheraplugin.h>

class ContactsListViewItem;
class ContactsLinkFactory;
class QListView;
class QTextBrowser;

class ContactsListView;
class ContactPreviewWidget;
class VContactObject;

class CONTACT_PLUGIN_API ContactsMainWidget : public QWidget
{
  Q_OBJECT

  public:

    ContactsMainWidget(VFSModule *, QWidget * parent, const char * name = 0);

    virtual ~ContactsMainWidget();

    virtual QStringList getSelection();
    virtual VFolder* parentFolder();

    virtual void newObject();

    virtual void selectAll();
    virtual void unselectAll();

    virtual bool isSelectionEmpty();
    virtual bool isSelectionSingle();
    virtual bool isSelectionMultiple();
    
    // TODO later
    virtual void openSelection() {};
    virtual void sendSelection() {};
    virtual void saveSelection() {};
    virtual void editProperties() {};

    virtual void printSelection(QPtrList<ContactsListViewItem> &);
    
    virtual void cut();
    virtual void copy();
    virtual void remove(QPtrList<ContactsListViewItem> &);
    virtual void cut(QPtrList<ContactsListViewItem> &);
    virtual void copy(const QPtrList<ContactsListViewItem> &);
    virtual void paste();
 
    virtual void moveTo(QPtrList<ContactsListViewItem> &);
    virtual void copyTo(const QPtrList<ContactsListViewItem> &);

    virtual void importFile(const QString & filename);
    virtual void importQString(const QString & cardString, const QString & path);
    virtual void importEncodedVcard(const QCString & data,
                                    const QString & path,
                                    const QString & charset,
                                    const QString & encoding);
    
    virtual void exportVcard(const QString & filename);
    virtual void exportVcardEncoded(const QString & vcardObj, const QString & filename);
    virtual void exportVcardAsUTF8(const QString & vcard, const QString & filename);

    virtual QString  exportVcardString();
   
    virtual QString getCurrentFolder() const;
    
  public slots:

    virtual void slotSetFolder(QString path);
    virtual void slotCurrentChanged(QListViewItem *);
    virtual void slotDoubleClicked(QListViewItem *);
    virtual void slotContextMenuRequested(QListViewItem *, const QPoint &, int);

    virtual void slotUpdateVInstances(const QStringList &);
    virtual void slotCreateVInstances(const QStringList &);
    virtual void slotDeleteVInstances(const QStringList &);

    virtual void slotEditFinished(VContactObject *, bool del);

  protected slots:

    void slotSelectionChanged();

  signals:

    void copyPossible(bool);
    void pastePossible(bool);

  protected:

    // TODO
    virtual void makeChildPopup() {};
    virtual void makeGlobalPopup() {};

    virtual bool canPaste() const;

    virtual void deleteItemsInCutList();

    virtual QStringList vObjectListToPathList
      (const QPtrList<ContactsListViewItem> &) const;
    
  private:

    QString     m_currentPath;
    QStringList m_cutList;
    QString     m_toBeEditedWhenCreated;

    ContactsListView      * m_contactListView;
    ContactPreviewWidget  * m_contactView;

    VFSModule * m_vfsModule;
};

#endif // CONTACTS_MAIN_WIDGET_H
