/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  komcontactsconfigservice.cpp
	-  the Contacts configuration service
                             -------------------
    begin                : Sun Oct  7 15:27:02 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qsettings.h>
#include <qapplication.h>
#include <qcombobox.h>
#include <qcheckbox.h>

#include <komcontactsconfigwidget.h>
#include <komcontactsconfigservice.h>

#include <settings.h>

static const char * const configGroup = "/Aethera/Contacts/";

class KomContactsConfigService::Private
{
  public:

    KomContactsConfigWidget * widget;
};

/*
static const char* komcontactsconfigservice_metainfo[][2]=
{
    { "/name", "Kom Contacts Config Service" },
    { "/class", "KomContactsConfigService" },
    { "/class/fqn", "KomContactsConfigService" },
    { "/factory", "komcontactsconfigservice_factory" },
    { 0, 0 }
};
*/

kore::Interface* komcontactsconfigservice_factory()
{
  return new KomContactsConfigService();
}
    
KomContactsConfigService::KomContactsConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom Contacts Config Service");
  info->replaceAll("class", "KomContactsConfigService");
  info->replaceAll("class/fqn", "KomContactsConfigService");
  info->replaceAll("factory", "komcontactsconfigservice_factory");
  
  d = new Private;
  d->widget = 0;
}

KomContactsConfigService::~KomContactsConfigService()
{
  delete d->widget;
  d->widget = 0;
  delete d;
}

  QWidget *
KomContactsConfigService::init(QWidget * parent)
{
  d->widget = new KomContactsConfigWidget(parent, "KomContactsConfigWidget");

  return d->widget;
}

	void
KomContactsConfigService::load()
{
	if (0 == d->widget)
		return;

	QStringList nameList;
	QStringList workList;
	QStringList homeList;
	bool showEmail = true;
	QStringList phoneList;

	QStringList entryList = Settings::instance()->entryList(configGroup);

	if (entryList.find("VisibleNameFields") != entryList.end())
	{
		nameList 	= Settings::instance()->readListEntry
			(configGroup + QString("VisibleNameFields"), ',');

		workList 	= Settings::instance()->readListEntry
			(configGroup + QString("VisibleWorkFields"), ',');

		homeList 	= Settings::instance()->readListEntry
			(configGroup + QString("VisibleHomeFields"), ',');

		showEmail = Settings::instance()->readBoolEntry
			(configGroup + QString("EmailVisible"), true);
		// ??? phoneList;
	}
	{
		// Oh shit. We are going to have to check for the old config stuff now.

		QString s = Settings::instance()->readEntry("View fields");
		QStringList sectionList = QStringList::split(';', s);

		QStringList::ConstIterator sectionIt(sectionList.begin());

		bool foundEmail = false;

		for (; sectionIt != sectionList.end(); ++sectionIt)
		{
			QString section(*sectionIt);
			QString sectionName;
			QStringList items;

			int equalsPos = section.find('=');

			if (-1 == equalsPos)
			{
				sectionName = section;
			}
			else
			{
				sectionName = section.left(equalsPos);
				items = QStringList::split(',', section.mid(equalsPos + 1));
			}

			if ("name" == section)
			{
				nameList = items;
			}
			else if ("work address" == section)
			{
				workList = items;
			}
			else if ("home address" == section)
			{
				homeList = items;
			}
			else if ("email" == section)
			{
				foundEmail = true;
			}
			else if ("phone" == section)
			{
				phoneList = items;
			}
			else
			{
				qDebug("Unrecognised section in 'View fields': %s", section.ascii());
			}
		}

		showEmail = foundEmail;
	}

	// If there's only "name" in the list, we are using the default.

	d->widget->defaultFieldsCheckBox
		->setChecked((nameList.count() == 1) && ("name" == nameList[0]));

	d->widget->prefixCheckBox
		->setChecked(nameList.find("prefix") != nameList.end());
	d->widget->otherCheckBox
		->setChecked(nameList.find("other") != nameList.end());
	d->widget->suffixCheckBox
		->setChecked(nameList.find("suffix") != nameList.end());
	d->widget->surnameCheckBox
		->setChecked(nameList.find("surname") != nameList.end());
	d->widget->nameCheckBox
		->setChecked(nameList.find("name") != nameList.end());

	d->widget->emailCheckBox->setChecked(showEmail);

	// ??? d->widget->phoneComboBox;

	d->widget->workPoBoxCheckBox
		->setChecked(workList.find("po-box") != workList.end());
	d->widget->workExtendedCheckBox
		->setChecked(workList.find("extended") != workList.end());
	d->widget->workStreetCheckBox
		->setChecked(workList.find("street") != workList.end());
	d->widget->workCityCheckBox
		->setChecked(workList.find("city") != workList.end());
	d->widget->workRegionCheckBox
		->setChecked(workList.find("region") != workList.end());
	d->widget->workCodeCheckBox
		->setChecked(workList.find("code") != workList.end());
	d->widget->workCountryCheckBox
		->setChecked(workList.find("country") != workList.end());

	d->widget->homePoBoxCheckBox
		->setChecked(workList.find("po-box") != workList.end());
	d->widget->homeExtendedCheckBox
		->setChecked(workList.find("extended") != workList.end());
	d->widget->homeStreetCheckBox
		->setChecked(workList.find("street") != workList.end());
	d->widget->homeCityCheckBox
		->setChecked(workList.find("city") != workList.end());
	d->widget->homeRegionCheckBox
		->setChecked(workList.find("region") != workList.end());
	d->widget->homeCodeCheckBox
		->setChecked(workList.find("code") != workList.end());
	d->widget->homeCountryCheckBox
		->setChecked(workList.find("country") != workList.end());
}

	void
KomContactsConfigService::save()
{
	if (0 == d->widget)
		return;

	QStringList nameList;
	QStringList workList;
	QStringList homeList;
	bool showEmail = true;
	QStringList phoneList;

	if (d->widget->defaultFieldsCheckBox->isChecked())
	{
		nameList << "name";
	}
	else
	{
		if (d->widget->prefixCheckBox->isChecked())
			nameList << "prefix";
		if (d->widget->otherCheckBox->isChecked())
			nameList << "other";
		if (d->widget->suffixCheckBox->isChecked())
			nameList << "suffix";
		if (d->widget->surnameCheckBox->isChecked())
			nameList << "surname";
		if (d->widget->nameCheckBox->isChecked())
			nameList << "name";
	}

	showEmail = d->widget->emailCheckBox->isChecked();

	// ??? d->widget->phoneComboBox;

	if (d->widget->workPoBoxCheckBox->isChecked())
		workList << "po-box";
	if (d->widget->workExtendedCheckBox->isChecked())
		workList << "extended";
	if (d->widget->workStreetCheckBox->isChecked())
		workList << "street";
	if (d->widget->workCityCheckBox->isChecked())
		workList << "city";
	if (d->widget->workRegionCheckBox->isChecked())
		workList << "region";
	if (d->widget->workCodeCheckBox->isChecked())
		workList << "code";
	if (d->widget->workCountryCheckBox->isChecked())
		workList << "country";

	if (d->widget->homePoBoxCheckBox->isChecked())
		homeList << "po-box";
	if (d->widget->homeExtendedCheckBox->isChecked())
		homeList << "extended";
	if (d->widget->homeStreetCheckBox->isChecked())
		homeList << "street";
	if (d->widget->homeCityCheckBox->isChecked())
		homeList << "city";
	if (d->widget->homeRegionCheckBox->isChecked())
		homeList << "region";
	if (d->widget->homeCodeCheckBox->isChecked())
		homeList << "code";
	if (d->widget->homeCountryCheckBox->isChecked())
		homeList << "country";

	Settings::instance()->writeEntry(configGroup + QString("VisibleNameFields"),	nameList);
	Settings::instance()->writeEntry(configGroup + QString("VisibleWorkFields"),	workList);
	Settings::instance()->writeEntry(configGroup + QString("VisibleHomeFields"),	homeList);
	Settings::instance()->writeEntry(configGroup + QString("EmailVisible"),			showEmail);
	// ??? phone
}

	void
KomContactsConfigService::defaults()
{
	if (0 == d->widget)
		return;
}

	QPixmap
KomContactsConfigService::icon()
{
	QPixmap p(32, 32);
	p.fill(Qt::blue);
	return p;
}

	QString
KomContactsConfigService::caption()
{
	return qApp->translate("Contacts configuration caption", "Contacts");
}

	bool
KomContactsConfigService::modified()
{
	return false;
}

// vim:ts=2:sw=2:tw=78:noet

