/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    komcontactsmaintreeinterface.cpp
                             -------------------
    begin                : Thu Nov  1 20:38:35 GMT 2001
    copyright            : (C) 2001 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
    author               : Fabien Armand <far@atolcd.com>
 ***************************************************************************/

#include <vfolder.h>
#include <qpopupmenu.h>

#include <komcontactsuiservice.h>
#include <komcontactsbackendservice.h>
#include <contactskolabinterface.h>

#include <komcontactsmaintreeinterface.h>
#include <komcontactsmaintreeslots.h>

#include <qiconloader.h>

// XXX: Which are the correct icons ?

KomContactsMainTreeInterface::KomContactsMainTreeInterface(QListView * parent)
  : KomMainTreeInterface  (parent),
    m_popupMenu           (0L),
    m_slots               (0L),
    m_root                (0L)
{
  KomItemType *itemType =
    new KomItemType
    (
      ItemTypeRoot,
      new QString("Contacts"),
      new QPixmap(BarIcon("Contacts.xpm")),
      true,
      true
    );
  itemTypeList().insert("root", itemType);

  itemType =
    new KomItemType
    (
      ItemTypeGroup,
      new QString("Group"),
      new QPixmap(BarIcon("Contacts.xpm")),
      true,
      true
    );
  itemTypeList().insert("contacts", itemType);

  m_popupMenu = new QPopupMenu(parentTree(), "KomContactsTreeContextPopup");

  m_slots = new KomContactsMainTreeSlots(this, "KomContactsMainTreeSlots");
}

  void
KomContactsMainTreeInterface::show()
{
  m_root = new KomTreeItem(parentTree(), *const_cast<KomItemType *>(itemTypeList().getItemOfType("root")));
  m_root->setOpen(true);
  createChildren(m_root);
  parentTree()->setCurrentItem(0);
  parentTree()->setCurrentItem(m_root);
  parentTree()->setSelected(m_root, true);
}

  void
KomContactsMainTreeInterface::show(const QString & focus)
{
  m_root = new KomTreeItem(parentTree(), *const_cast<KomItemType *>(itemTypeList().getItemOfType("root")));
  KomMainTreeInterface::show(focus);
}

  void
KomContactsMainTreeInterface::hide()
{
  delete m_root;
}

KomContactsMainTreeInterface::~KomContactsMainTreeInterface()
{
  // TODO ?
}

  KomTreeItem *
KomContactsMainTreeInterface::root()
{
  return m_root;
}

void KomContactsMainTreeInterface::createChildren(KomTreeItem * root)
{
  if (0 == root)
    return;

  QString rootPath = path(root);

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  VFolder * parentFolder = vfs->folderRef(rootPath);
  if( 0 == parentFolder ) // oops
    return;
  VFolderList & folderList = parentFolder->folderList();

  for (VFolderListIterator it(folderList); it.current(); ++it)
  {
    QString childName((it.current()->path().mid(rootPath.length() + 1)));

    // See if it exists already.

    KomTreeItem * folderItem = findChild(root, childName);

    if (0 == folderItem)
    {
      // No ? Then make it.

      folderItem = createFolderItem
      (
        root,
        it.current()->path().mid(rootPath.length() + 1),
        it.current()->viewType()
      );
    }

    // Recurse.
    createChildren(folderItem);
  }
}

  QPopupMenu *
KomContactsMainTreeInterface::contextMenu(KomTreeItem * item)
{
  assert(0 != m_popupMenu);

  m_popupMenu->clear();

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  bool kolab = false;
  bool sharedKolab = false;
  QString emailPath;
  QStringList kolabList = 
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  if( !kolabList.isEmpty() )
  {
    QString folderPath = path(item);
    QString kolabMainFolder = ContactsKolabInterface::instance()->kolabPath();
    
    emailPath = ContactsKolabInterface::instance()->isKolabFolder(folderPath);
    if( !emailPath.isEmpty() || (folderPath + "/") == kolabMainFolder )
    {
      kolab = true;
    }
    else
    {
      emailPath = ContactsKolabInterface::instance()->isParentKolabSharedFolder(folderPath);
      if( !emailPath.isEmpty() )
      {
        sharedKolab = true;
      }
    }
  }

  switch (item->getItemType())
  {
    case ItemTypeRoot:
      m_popupMenu->insertItem( QObject::tr("New contact"), m_slots, SLOT(slotNewContact()) );
      m_popupMenu->insertItem
        ( QObject::tr("New contacts group"), m_slots, SLOT(slotNewContactGroup()) );
      m_popupMenu->insertItem
        ( QObject::tr("Expunge"), m_slots, SLOT(slotExpungeContactGroup()) );
      break;

    case ItemTypeGroup:
      if(kolab)
      {
        m_popupMenu->insertItem( QObject::tr("&Kolab Synchronization"), m_slots, SLOT(slotKolabSynch()) );
        m_popupMenu->insertItem( QObject::tr("New contact"), m_slots, SLOT(slotNewContact()) );
        m_popupMenu->insertItem
          ( QObject::tr("Search for shared folders"),m_slots,SLOT(slotSearchForSharedFolders()) );
      }
      else if(sharedKolab)
      {
        m_popupMenu->insertItem( QObject::tr("&Kolab Synchronization"), m_slots, SLOT(slotKolabSynch()) );
        m_popupMenu->insertItem( QObject::tr("New contact"), m_slots, SLOT(slotNewContact()) );
        m_popupMenu->insertItem
          ( QObject::tr("Delete contacts group"), m_slots, SLOT(slotDeleteContactGroup()) );
        m_popupMenu->insertItem
          ( QObject::tr("Search for shared folders"),m_slots,SLOT(slotSearchForSharedFolders()) );
      }
      else
      {
        m_popupMenu->insertItem( QObject::tr("New contact"), m_slots, SLOT(slotNewContact()) );
        m_popupMenu->insertItem
          ( QObject::tr("New contacts group"), m_slots, SLOT(slotNewContactGroup()) );
        m_popupMenu->insertItem
          ( QObject::tr("Delete contacts group"), m_slots, SLOT(slotDeleteContactGroup()) );
        m_popupMenu->insertItem
          ( QObject::tr("Move contacts group"), m_slots, SLOT(slotMoveContactGroup()) );
        m_popupMenu->insertItem
          ( QObject::tr("Rename contacts group"), m_slots, SLOT(slotRenameContactGroup()) );
        m_popupMenu->insertItem
          ( QObject::tr("Expunge"), m_slots, SLOT(slotExpungeContactGroup()) );
      }
      break;

    default:
      qDebug("Don't know about type %d", item->getItemType());
      break;
  }

  return m_popupMenu;
}

  bool
KomContactsMainTreeInterface::hasChild(KomTreeItem * item)
{
  for (QListViewItemIterator it(parentTree()); it.current(); ++it)
  {
    if (it.current() == item)
      return true;
  }

  return false;
} 

  void
KomContactsMainTreeInterface::updateVInstances(const QStringList & _l)
{
//  VFSModule * vfs = 0;
//  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
//  if( trader )
//  {
//    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
//    if( ref.isValid() )
//      vfs = static_cast<VFSModule*>(ref.instance());
//  }
//
//  if (0 == vfs)
//  {
//    DEBUG_MSG("No VFS module ?");
//    return;
//  } 
//
//  QStringList l(vfs->reverseMap(_l));
//  l.sort();
//
//  for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
//  {
//    VReference * ref = vfs->reference(*it);
//
//    if (0 == ref || "folder" != ref->type())
//      continue;
//
//    KomTreeItem * i = folderItem(*it, false /* Do not create */);
//
//    if (0 == i)
//    {
//      qDebug("Can't update non-existant folder item");
//      continue;
//    }
//
//    VFolder * folder = vfs->folderRef(*it);
//
//    if (0 == folder)
//    {
//      qDebug("VFS does not have folder %s", (*it).ascii());
//      continue;
//    }
//
//    // The only thing that could have changed (that we care about)
//    // is the name.
//
//    i->setText(0, folder->name());
//  }
}

  void
KomContactsMainTreeInterface::createVInstances(const QStringList & _l)
{
//  VFSModule * vfs = 0;
//  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
//  if( trader )
//  {
//    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
//    if( ref.isValid() )
//      vfs = static_cast<VFSModule*>(ref.instance());
//  }
//
//  if (0 == vfs)
//  {
//    DEBUG_MSG("No VFS module ?");
//    return;
//  } 
//
//  QStringList l(vfs->reverseMap(_l));
//  l.sort();
//
//  for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
//  {
//    VReference * ref = vfs->reference(*it);
//
//    if (0 == ref || "folder" != ref->type())
//      continue;
//
//    (void) folderItem(*it, true /* Create */);
//  }
}

  void
KomContactsMainTreeInterface::deleteVInstances(const QStringList & _l)
{
//  VFSModule * vfs = 0;
//  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
//  if( trader )
//  {
//    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
//    if( ref.isValid() )
//      vfs = static_cast<VFSModule*>(ref.instance());
//  }
//
//  if (0 == vfs)
//  {
//    DEBUG_MSG("No VFS module ?");
//    return;
//  } 
//
//  QStringList l(vfs->reverseMap(_l));
//  l.sort();
//
//  for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
//  {
//    VReference * ref = vfs->reference(*it);
//
//    if (0 == ref || "folder" != ref->type())
//      continue;
//
//    KomTreeItem * i = folderItem(*it, false /* Do not create */);
//
//    if (0 == i)
//    {
//      qDebug("Hmm, delete a non-existant item (%s) ?", (*it).ascii());
//      continue;
//    }
//    else
//    {
//      delete i;
//    }
//  } 
} 

  void
KomContactsMainTreeInterface::updateHierarchy(const QString & s)
{
  qDebug("KomContactsMainTreeInterface::updateHierarchy(%s): STUB", s.ascii());
}

  QObject *
KomContactsMainTreeInterface::qObject()
{
  return m_slots;
}

  KomTreeItem *
KomContactsMainTreeInterface::folderItem(const QString & url, bool create)
{
  QString contactsRoot = "/Aethera/VFS/Contacts/";
  if ( contactsRoot != url.left( contactsRoot.length() ))
  {
    return 0;
  }

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
      return 0;
    }
  }
  VFolder *folder = vfs->folderRef(url);
  
  QStringList pathList(QStringList::split('/', url.mid( contactsRoot.length() )));

  KomTreeItem * root = m_root;
  KomTreeItem * child = 0;

  QStringList::ConstIterator it(pathList.begin());

  for (it = pathList.begin(); it != pathList.end(); ++it)
  {
    child = findChild(root, *it);

    if (0 == child)
    {
      if (create && folder)
      {
        return createFolderItem(root, *it, folder->viewType());
      }
      else
      {
        return 0;
      }
    }
    else
      root = child;
  }

  return child;
}

  KomTreeItem *
KomContactsMainTreeInterface::findChild
(
  KomTreeItem * root,
  const QString & name
)
{
  KomTreeItem * treeItem = static_cast<KomTreeItem *>(root->firstChild());

  while (0 != treeItem)
  {
    if (treeItem->text(0) == name)
      return treeItem;

    treeItem = static_cast<KomTreeItem *>(treeItem->nextSibling());
  }

  return 0;
}

  void
KomContactsMainTreeInterface::createVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & /*type*/,
  const QString & /*subType*/,
  const QString & /*serverPath*/
)
{
  folderItem(parentFolderPath+"/"+folderName, true);
}

  void
KomContactsMainTreeInterface::deleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & /*type*/
)
{
  // the VFolder should be already deleted, so only the folder item
  // has to be deleted
  KomTreeItem *deletedFolder = folderItem(parentFolderPath+"/"+folderName, false);
  if( deletedFolder )
    delete deletedFolder;
}

  KomTreeItem *
KomContactsMainTreeInterface::createFolderItem
(
  KomTreeItem * parentItem,
  const QString & folderName,
  const QString & type
)
{
  QString folderName_ = folderName;
  KomItemType komItemType = *const_cast<KomItemType *>(itemTypeList().getItemOfType(type));
  komItemType.setItemText(&folderName_);

  return static_cast<KomTreeItem *> (new KomTreeItem(parentItem, komItemType));
}

// vim:tabstop=2:shiftwidth=2:expandtab:cinoptions=(s,U1,m1
