/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  komcontactsmaintreeslots.cpp
                             -------------------
    begin                : Fri Nov  2 12:23:31 GMT 2001
    copyright            : (C) 2001 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
                           Fabien Armand <far@atolcd.com>
 ***************************************************************************/

#include <defines.h>
#include <vfolder.h>
#include <vfsmodule.h>
#include <clientrmi.h>
#include <komcontactsbackendservice.h>
#include <komcontactsmaintreeinterface.h>
#include <komcontactsmaintreeslots.h>
#include <komcontactsuiservice.h>
#include <contacteditor.h>
#include <folderchooserdialog.h>

#include <qinputdialog.h>
#include <qmessagebox.h>

// Kolab
#include <kolabsharedfolder.h>
#include <contactskolabinterface.h>

KomContactsMainTreeSlots::KomContactsMainTreeSlots
(
  KomContactsMainTreeInterface * parent,
  const char * name
)
  : QObject             (0, name),
    m_mainTreeInterface (parent)
{
}

KomContactsMainTreeSlots::~KomContactsMainTreeSlots()
{
  // Empty.
}

void KomContactsMainTreeSlots::slotNewContact()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomContactsMainTreeSlots::slotNewContact(): no current item");
    return;
  }

  qDebug("KomContactsMainTreeSlots::slotNewContact: creating object at %s", path.ascii());

  QString contactName = vfs->createObject(path);

  if (!contactName.isEmpty())
    m_toBeEditedWhenCreated = path + "/" + contactName;

}

void KomContactsMainTreeSlots::slotDeleteContact()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }


  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomContactsMainTreeSlots::slotDeleteContact(): no current item");
    return;
  }

  bool ok = vfs->remove(path);

  if (!ok)
  {
    DEBUG_MSG("KomContactsMainTreeSlots::slotDeleteContact(): couldn't delete contact");
  }
}

void KomContactsMainTreeSlots::slotEditContact()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomContactsMainTreeSlots::slotEditContact(): no current item");
    return;
  }

  VContactObject * contactRef = 
    static_cast<VContactObject *>( vfs->_fact_ref(path, VFS_CONTACTS_MIMETYPE) );

  if (0 == contactRef)
  {
    qDebug("Hmm, can't find contact to edit");
    return;
  }

  // XXX What's a contact object ?
#if 0
  VContactObject * contact = dynamic_cast<VContactObject *>(contactRef);

  if (0 == contact)
  {
    qDebug("dynamic_cast from VObject to VContactObject failed");
    return;
  }
#endif

  qDebug("STUB - need to edit contact");
}

void KomContactsMainTreeSlots::slotNewContactGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomContactsMainTreeSlots::slotNewContactGroup(): no current item");
    return;
  }
 
  qDebug("Path: `%s'", path.ascii());

  VFolder * parentRef = vfs->folderRef(path);

  if (0 == parentRef)
  {
    qDebug("Parent does not exist");
    return;
  }

  bool ok = false;

  QString name =
    QInputDialog::getText
    (
      tr(AppName),
      tr("Please enter a name for the new folder"),
      QLineEdit::Normal,
      QString::null,
      &ok,
      0 // FIXME: Parent widget ?
    );

  if (ok && !name.isEmpty())
  {
    VFolder * folderRef = vfs->folderRef(path + "/testing");

    if (0 != folderRef)
    {
      qDebug("Folder already exists");
    }
    else
    {
      qDebug("Folder does not exist. Making");

      VFolder * f = vfs->createFolder(parentRef, name, "contacts", "contacts");

      if (0 == f)
      {
        qDebug("Couldn't create folder");
        return;
      }

      qDebug("Folder made successfully");
    }

    m_mainTreeInterface->createChildren(m_mainTreeInterface->currentItem());
  }
}

void KomContactsMainTreeSlots::slotDeleteContactGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomContactsMainTreeSlots::slotDeleteContactGroup(): no current item");
    return;
  }
 
  // if it's a Kolab shared folder
  if( !path.find(ContactsKolabInterface::instance()->kolabPath()) )
  {
    ContactsKolabInterface::instance()->deleteKolabSharedFolder(path);
  }
  
  // remove the item from the list
  delete m_mainTreeInterface->currentItem();
  
  bool ok = vfs->deleteFolder(path);

  if (!ok)
  {
    qDebug("slotDeleteContactGroup: unsuccessful");
  }
}

  void
KomContactsMainTreeSlots::slotUpdateVInstances(const QStringList & l)
{
  m_mainTreeInterface->updateVInstances(l);
}

  void 
KomContactsMainTreeSlots::slotCreateVInstances(const QStringList & l)
{
  m_mainTreeInterface->createVInstances(l);

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  if (m_toBeEditedWhenCreated.isEmpty())
    return;

  QString toEdit = m_toBeEditedWhenCreated;

  m_toBeEditedWhenCreated = QString::null;

  QStringList::ConstIterator it = l.begin();

  for (; it != l.end(); ++it)
  {
    if ((*it) == toEdit)
    {
      VContactObject * contactObject =
        static_cast<VContactObject *>( vfs->_fact_ref(toEdit, VFS_CONTACTS_MIMETYPE) );

      if (0 == contactObject)
      {
        qDebug("contactObject is 0");
        return;
      }

      ContactEditor::instance()->
        edit(contactObject, 0, ContactEditDialog::RemoveIfUnedited);
    }
  }
} 

  void 
KomContactsMainTreeSlots::slotDeleteVInstances(const QStringList & l)
{
  m_mainTreeInterface->deleteVInstances(l);
}

  void 
KomContactsMainTreeSlots::slotUpdateHierarchy(const QString & s)
{
  m_mainTreeInterface->updateHierarchy(s);
}

  void
KomContactsMainTreeSlots::slotCreate(const QString & url)
{
  qDebug("KomContactsMainTreeSlots::slotCreate(%s)", url.ascii());
}

  void
KomContactsMainTreeSlots::slotCreateVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type,
  const QString & subType,
  const QString & serverPath
)
{
  m_mainTreeInterface->createVFolder
    (
      parentFolderPath,
      folderName,
      type,
      subType,
      serverPath
    );
}

  void
KomContactsMainTreeSlots::slotDeleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type
)
{
  m_mainTreeInterface->deleteVFolder(parentFolderPath, folderName, type);
}

  void
KomContactsMainTreeSlots::slotMoveContactGroup()
{
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  QString targetPath = FolderChooserDialog::getFolder("/Contacts", 0);

  if( targetPath.isEmpty() )
    return;
  
  if(targetPath != currentPath)
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }
    
    VFolder * folder = vfs->folderRef(currentPath);
    QStringList foldersTree;
    if( folder )
    {
      // check if the target folder is a subfolder
      folder->tree(foldersTree, -1);
      for(QStringList::const_iterator it=foldersTree.begin(); it!=foldersTree.end(); ++it)
      {
        if( targetPath == (*it) )
        {
          QMessageBox::warning( 0L,
                                QObject::tr("Folder Move Warning"), 
                                QObject::tr("Moving a folder in one of its subfolders isn't allowed."),
                                QMessageBox::Ok, 
                                QMessageBox::NoButton, 
                                QMessageBox::NoButton );
          return;
        }
      }
      
      vfs->moveFolder(folder->path(), targetPath + "/" + folder->name() );
    }
  }
  else
  {
    QMessageBox::warning( 0L,
                          QObject::tr("Folder Move Warning"), 
                          QObject::tr("The new folder path is the same with the old one."),
                          QMessageBox::Ok, 
                          QMessageBox::NoButton, 
                          QMessageBox::NoButton );
  }
}

  void
KomContactsMainTreeSlots::slotRenameContactGroup()
{
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  bool ok = false;
  QString targetName = QInputDialog::getText( QObject::tr("Folder Rename"),
                                              QObject::tr("Please insert the new folder name:"),
                                              QLineEdit::Normal,
                                              QString::null,
                                              &ok, 0L, "ContactsFolderRename");

  if( ok )
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }
    
    VFolder * folder = vfs->folderRef(currentPath);
    QStringList foldersTree;
    if( folder && !targetName.isEmpty() )
    {
      vfs->moveFolder(folder->path(), folder->parentFolder()->path() + "/" + targetName);
    }
    else
    {
      QMessageBox::warning( 0L, 
                            QObject::tr("Folder Rename Warning"), 
                            QObject::tr("The new name is empty or the folder is not available."),
                            QMessageBox::Ok, 
                            QMessageBox::NoButton, 
                            QMessageBox::NoButton );
    }
  }
}

  void
KomContactsMainTreeSlots::slotExpungeContactGroup()
{
}

  void
KomContactsMainTreeSlots::slotKolabSynch()
{
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  
  qDebug("KomContactsMainTreeSlots::slotKolabSynch() : %s", currentPath.latin1());
  
  VFSModule * vfs = 0;
  ClientRMIModule * clientRMI = 0;
  
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  assert(0 != vfs);
  assert(0 != clientRMI);

  // main Kolab folder
  if( (currentPath + "/") == ContactsKolabInterface::instance()->kolabPath() )
  {
    // insert a message box here
    // it should synch all the subfolders
    return;
  }
  
  VFolder * folder = vfs->folderRef(currentPath);
  if(folder)
  {
    QString emailPath = ContactsKolabInterface::instance()->isKolabFolder(currentPath);
    if( emailPath.isEmpty() )
    {
      emailPath = ContactsKolabInterface::instance()->isKolabSharedFolder(currentPath);
    }
    
    if( !emailPath.isEmpty() )
    {
      VFolder * emailfolder = vfs->folderRef(emailPath);
      if( !emailfolder )
      {
        KolabSharedFolder::ref()->checkKolabFolders();
      }
      
      clientRMI->mount(emailPath);
      clientRMI->unmount(emailPath);
    }
  }
  else
  {
    qDebug("No such Kolab folder.");
  }
}

  void
KomContactsMainTreeSlots::slotSearchForSharedFolders()
{
  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  for (QStringList::Iterator itList = kolabList.begin();itList != kolabList.end();itList++)
  {
    KolabSharedFolder::ref()->scannSharedFolders(*itList, "contacts");
  }
}
