/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          komcontactsuiservice.cpp  -  the UI service from the Contacts plug in
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
 ***************************************************************************/

#include <komcontactsuiservice.h>
#include <komcontactsbackendservice.h>
#include <komcontactsmaintreeinterface.h>
#include <contactsuipartprivate.h>
#include <contactsmainwidget.h>

// QT includes
#include <qiconset.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qiconloader.h>
#include <qlayout.h>
#include <qmessagebox.h>

/*
static const char* komcontactsuiservice_metainfo[][2]=
{
    { "/name", "KomContacts UI Service" },
    { "/class", "KomContactsUIService" },
    { "/class/fqn", "KomContactsUIService" },
    { "/description", AETHERA_UISERVICE_CONTACTS_DESC },
    { "/factory", "komcontactsuiservice_factory" },
    { "/service",  AETHERA_UISERVICE_CONTACTS },
    { 0, 0 }
};
*/

kore::Interface* komcontactsuiservice_factory()
{
  return KomContactsUIService::instance();
}

KomContactsUIService * KomContactsUIService::instance_ = 0L;
    
KomContactsUIService * KomContactsUIService::instance()
{
  if( !instance_ )
  {
    instance_ = new KomContactsUIService();
  }
  
  return instance_;
}

KomContactsUIService::KomContactsUIService()
  : m_backendService    (0),
    m_contactsMenu      (0),
    m_contactsMenuId    (-1),
    topWidget           (0),
    topBanner           (0),
    m_mainTreeInterface (0),
    m_topWidget         (0),
    m_topMainBanner     (0),
    m_topMainView       (0),
    m_topWidgetLayout   (0),
    m_actionsCreated    (false)
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom Contacts UI Service");
  info->replaceAll("class", "KomContactsUIService");
  info->replaceAll("class/fqn", "KomContactsUIService");
  info->replaceAll("description", AETHERA_UISERVICE_CONTACTS_DESC );
  info->replaceAll("factory", "komcontactsuiservice_factory");
  info->insert("service",  AETHERA_UISERVICE_CONTACTS );
  
  topWidget = NULL;
  topBanner = NULL;
  m_mainTreeInterface = NULL;

  pluginIcon.setPixmap
    (
      BarIcon(CONTACTS_ON),
      QIconSet::Large,
      QIconSet::Active,
      QIconSet::Off
    );

  pluginIcon.setPixmap
    (
      BarIcon(CONTACTS_OFF),
      QIconSet::Large,
      QIconSet::Normal,
      QIconSet::Off
    );

  _mimeType = VFS_CONTACTS_MIMETYPE;
}

KomContactsUIService::~KomContactsUIService()
{
  if( m_mainTreeInterface )
  {
    delete m_mainTreeInterface;
    m_mainTreeInterface = 0L;
  }
}


bool KomContactsUIService::hasCapability(const QString& capabilityName)
{
  if
    (
          capabilityName == TOP_WIDGET
      ||  capabilityName == BANNER_WIDGET
      ||  capabilityName == PLUGIN_ICONSET
      ||  capabilityName == TREE_INTERFACE
    )
      return true;

  return KomUserInterfaceService::hasCapability(capabilityName);
}

void KomContactsUIService::createWidget(const QString& capabilityName)
{
  if( capabilityName==TOP_WIDGET )
  {
    if ( topWidget==NULL )
    {
      if (0 != m_topWidget)
        return;

      m_topWidget       = new QWidget(0, "ContactsTopWidget");

      // Build the widgets
      VFSModule * vfs = 0;
      ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
      if( trader )
      {
        ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
        if( ref.isValid() )
          vfs = static_cast<VFSModule*>(ref.instance());
        else
        {
          DEBUG_MSG("\n NO VFS Provider !\n");
        }
      }

      Q_ASSERT(0 != vfs);

      m_topMainView = new ContactsMainWidget(vfs, m_topWidget);

      m_topWidgetLayout = new QVBoxLayout(m_topWidget);

      m_topWidgetLayout->addWidget(m_topMainView);

      // Add the main widget to the plug in widgets dict.
      widgetDict.insert(TOP_WIDGET, m_topWidget);
    }
  }
  else if( capabilityName==BANNER_WIDGET )
  {
    if( m_topMainBanner==NULL )
    {
      m_topMainBanner = new QWidget(0);
      
      QHBoxLayout *bannerLayout = new QHBoxLayout(m_topMainBanner);
      QLabel *leftBanner= new QLabel(m_topMainBanner);
      leftBanner->setPixmap( BarIcon(CONTACTS_BANNER_LEFT) );
      QLabel *rightBanner= new QLabel(m_topMainBanner);
      rightBanner->setPixmap( BarIcon(CONTACTS_BANNER_RIGHT) );
      
      bannerLayout->addWidget(leftBanner);
      bannerLayout->addStretch();
      bannerLayout->addWidget(rightBanner);
      
      // Add the banner widget to the plug in widgets dict.
      widgetDict.insert( BANNER_WIDGET, m_topMainBanner);
    }
  }
  else if( capabilityName==TREE_INTERFACE )
  {
    // TODO
  }
}

void KomContactsUIService::showToolbars(QMainWindow *window)
{
  if( toolBar==NULL )
  {
    toolBar = new QToolBar(window, "ContactsToolBar");
    createActions();


    actionDict["create"]  ->addTo(toolBar);
    actionDict["cut"]     ->addTo(toolBar);
    actionDict["copy"]    ->addTo(toolBar);
    actionDict["paste"]   ->addTo(toolBar);
    toolBar->addSeparator();
    actionDict["import"]  ->addTo(toolBar);
    actionDict["export"]  ->addTo(toolBar);

    QLabel *emptyLabel = new QLabel(toolBar);
    toolBar->setStretchableWidget(emptyLabel);
  }
  else
  {
//    toolBar->reparent(mainWindow);
    toolBar->show();
  }
}

void KomContactsUIService::hideToolbars(QMainWindow *)
{
  if( toolBar!=NULL )
    toolBar->hide();
}

void KomContactsUIService::showMenus(QMenuBar *mainMenu)
{
  if (0 == mainMenu)
    return; // Huh ?

  if (0 == m_contactsMenu)
    m_contactsMenu = new QPopupMenu(mainMenu, "ContactsMenu");

  createActions();

  m_contactsMenuId = mainMenu->insertItem("&Contact", m_contactsMenu);

  actionDict["create"]   ->addTo(m_contactsMenu);
  actionDict["cut"]      ->addTo(m_contactsMenu);
  actionDict["copy"]     ->addTo(m_contactsMenu);
  actionDict["paste"]    ->addTo(m_contactsMenu);
  m_contactsMenu->insertSeparator(4);
  actionDict["import"]    ->addTo(m_contactsMenu);
  actionDict["export"]    ->addTo(m_contactsMenu);

#if 0
  m_contactsMenu->insertSeparator();
  m_settingsAction  ->addTo(m_contactsMenu);
#endif
}

void KomContactsUIService::hideMenus(QMenuBar *mainMenu)
{
  if (0 != mainMenu && -1 != m_contactsMenuId)
  {
    mainMenu->removeItem(m_contactsMenuId);
    m_contactsMenuId = -1;
  }

  if (0 != m_contactsMenu)
    m_contactsMenu->clear();
}

  KomMainTreeInterface*
KomContactsUIService::getMainTreeInterface( QListView *parentTree )
{
  if (0 == m_mainTreeInterface)
  {
    m_mainTreeInterface = new KomContactsMainTreeInterface(parentTree);
  }

  return m_mainTreeInterface;
}

  UIPartPrivate *
KomContactsUIService::createQTHandler()
{
  return new ContactsUIPartPrivate(this);
}

  void
KomContactsUIService::setCurrentURL(QString url)
{
  if (0 == m_topMainView)
  {
    qDebug("m_topMainView is 0");
    return;
  }

  m_topMainView->slotSetFolder(url);
}

  QString
KomContactsUIService::getCurrentFolder() const
{
  if(m_topMainView)
  {
    return m_topMainView->getCurrentFolder();
  }
  
  return QString::null;
}

  void
KomContactsUIService::createActions()
{
  if (m_actionsCreated)
    return;

  m_actionsCreated = true;

  ContactsUIPartPrivate * partHandler =
    static_cast<ContactsUIPartPrivate *>(getQTHandler());

  actionDict.insert
    (
      "settings",
      new QAction
      (
        QObject::tr("Settings"),
        QIconLoader::instance()->iconSet("tools-small-full.xpm"),
        QObject::tr("&Settings..."),
        0, // No accelerator key.
        partHandler,
        QObject::tr("Settings action")
      )
    );

  actionDict.insert
    (
      "create",
      new QAction
      (
        QObject::tr("Create"),
        QIconLoader::instance()->iconSet("contact-new.png"),
        QObject::tr("&Create"),
        0, // No accelerator key.
        partHandler,
        QObject::tr("Create action")
      )
    );

  actionDict.insert
    (
      "cut",
      new QAction
      (
        QObject::tr("Cut"),
        QIconLoader::instance()->iconSet("cut.png"),
        QObject::tr("&Cut"),
        0, // No accelerator key.
        partHandler,
        QObject::tr("Cut action")
      )
    );

  actionDict.insert
    (
      "copy",
      new QAction
      (
        QObject::tr("Copy"),
        QIconLoader::instance()->iconSet("copy.png"),
        QObject::tr("&Copy"),
        0, // No accelerator key.
        partHandler,
        QObject::tr("Copy action")
      )
    );

  actionDict.insert
    (
      "paste",
      new QAction
      (
        QObject::tr("Paste"),
        QIconLoader::instance()->iconSet("paste.png"),
        QObject::tr("&Paste"),
        0, // No accelerator key.
        partHandler,
        QObject::tr("Paste action")
      )
    );

  actionDict.insert
    (
      "import",
      new QAction
      (
        QObject::tr("Import"),
        QIconLoader::instance()->iconSet("Down.png"),
        QObject::tr("&Import"),
        QKeySequence( QObject::tr("Ctrl+I", "Contact|Import") ), // No accelerator key.
        partHandler,
        QObject::tr("Import action")
      )
    );

  actionDict.insert
    (
      "export",
      new QAction
      (
        QObject::tr("Export"),
        QIconLoader::instance()->iconSet("Up.png"),
        QObject::tr("&Export"),
        QKeySequence( QObject::tr("Ctrl+E", "Contact|Export") ), // No accelerator key.
        partHandler,
        QObject::tr("Export action")
      )
    );


  actionDict["cut"]   ->setEnabled(false);
  actionDict["copy"]  ->setEnabled(false);
  actionDict["paste"] ->setEnabled(false);


  QObject::connect
    (
      actionDict["settings"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_settings())
    );

  QObject::connect
    (
      actionDict["create"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_create())
    );

  QObject::connect
    (
      actionDict["cut"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_cut())
    );

  QObject::connect
    (
      actionDict["copy"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_copy())
    );

  QObject::connect
    (
      actionDict["paste"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_paste())
    );

  QObject::connect
    (
      actionDict["import"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_import())
    );

  QObject::connect
    (
      actionDict["export"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_export())
    );


  QObject::connect
    (
      m_topMainView,
      SIGNAL(copyPossible(bool)),
      actionDict["copy"],
      SLOT(setEnabled(bool))
    );

  QObject::connect
    (
      m_topMainView,
      SIGNAL(copyPossible(bool)),
      actionDict["cut"],
      SLOT(setEnabled(bool))
    );

  QObject::connect
    (
      m_topMainView,
      SIGNAL(pastePossible(bool)),
      actionDict["paste"],
      SLOT(setEnabled(bool))
    );

}
  void
KomContactsUIService::create()
{
  if (0 != m_topMainView)
    m_topMainView->newObject();
}

  void
KomContactsUIService::cut()
{
  if (0 != m_topMainView)
    m_topMainView->cut();
}

  void
KomContactsUIService::copy()
{
  if (0 != m_topMainView)
    m_topMainView->copy();
}

  void
KomContactsUIService::paste()
{
  if (0 != m_topMainView)
    m_topMainView->paste();
}

void
KomContactsUIService::import(QString & filename)
{
  if (0 != m_topMainView)
    m_topMainView->importFile(filename);
}

void
KomContactsUIService::exportVcard(QString & filename)
{
  if (0 != m_topMainView)
    m_topMainView->exportVcard(filename);
}


  void
KomContactsUIService::buildTree()
{
  if (0 == m_mainTreeInterface)
  {
    qDebug("KomContactsUIService::buildTree: No m_mainTreeInterface");
    return;
  }

  ContactsUIPartPrivate* partHandler = (ContactsUIPartPrivate*)getQTHandler();

  QObject::connect
    (
      partHandler,
      SIGNAL(create(const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreate(const QString &))
    );

      QObject::connect
        (
          partHandler,
          SIGNAL(updateVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotUpdateVInstances(const QStringList &))
        );

      QObject::connect
        (
          partHandler,
          SIGNAL(createVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotCreateVInstances(const QStringList &))
        );

      QObject::connect
        (
          partHandler,
          SIGNAL(deleteVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotDeleteVInstances(const QStringList &))
        );
        
//  QObject::connect
//    (
//      partHandler,
//      SIGNAL(updateVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotUpdateVInstances(const QStringList &))
//    );
//
//  QObject::connect
//    (
//      partHandler,
//      SIGNAL(createVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotCreateVInstances(const QStringList &))
//    );
//
//
//  QObject::connect
//    (
//      partHandler,
//      SIGNAL(deleteVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotDeleteVInstances(const QStringList &))
//    );
    
  QObject::connect
    (
      partHandler,
      SIGNAL(createVFolder(const QString &, const QString &, const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreateVFolder(const QString &, const QString &, const QString &, const QString &, const QString &))
    );

  QObject::connect
    (
      partHandler,
      SIGNAL(deleteVFolder(const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotDeleteVFolder(const QString &, const QString &, const QString &))
    );

  m_mainTreeInterface->show( m_topMainView->getCurrentFolder() );
}

  void
KomContactsUIService::removeTree()
{
  if (0 == m_mainTreeInterface)
  {
    qDebug("KomContactsUIService::buildTree: No m_mainTreeInterface");
    return;
  }

  ContactsUIPartPrivate* partHandler = (ContactsUIPartPrivate*)getQTHandler();

  QObject::disconnect
    (
      partHandler,
      SIGNAL(create(const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreate(const QString &))
    );

      QObject::disconnect
        (
          partHandler,
          SIGNAL(updateVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotUpdateVInstances(const QStringList &))
        );

      QObject::disconnect
        (
          partHandler,
          SIGNAL(createVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotCreateVInstances(const QStringList &))
        );

      QObject::disconnect
        (
          partHandler,
          SIGNAL(deleteVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotDeleteVInstances(const QStringList &))
        );
        
//  QObject::disconnect
//    (
//      partHandler,
//      SIGNAL(updateVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotUpdateVInstances(const QStringList &))
//    );
//
//  QObject::disconnect
//    (
//      partHandler,
//      SIGNAL(createVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotCreateVInstances(const QStringList &))
//    );
//
//  QObject::disconnect
//    (
//      partHandler,
//      SIGNAL(deleteVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotDeleteVInstances(const QStringList &))
//    );
    
  QObject::disconnect
    (
      partHandler,
      SIGNAL(createVFolder(const QString &, const QString &, const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreateVFolder(const QString &, const QString &, const QString &, const QString &, const QString &))
    );

  QObject::disconnect
    (
      partHandler,
      SIGNAL(deleteVFolder(const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotDeleteVFolder(const QString &, const QString &, const QString &))
    );
}

  QString
KomContactsUIService::prettyName() const
{
  return "AddressBook";
}

  void
KomContactsUIService::execAboutDialog(QWidget * parent) const
{
  QMessageBox::about
    (
      parent,
      QString("About plugin: %1").arg(prettyName()),
      QString("This plugin handles addressbook data")
    );
}

