/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
      komcontactsuiservice.h  -  the UI service from the Notes plug in
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
 ***************************************************************************/

#ifndef KOMCONTACTSUISERVICE_H
#define KOMCONTACTSUISERVICE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlabel.h>
#include <qasciidict.h>

#include <komaetheraplugin.h>
#include <komaetheratools.h>

#define CONTACTS_ON             "AddressBook2_active.png"
#define CONTACTS_OFF            "AddressBook2_inactive.png"
#define CONTACTS_BANNER         "aethera-banner-contacts.png"
#define CONTACTS_BANNER_LEFT    "AddressBook_Type.png"
#define CONTACTS_BANNER_RIGHT   "AddressBook_Banner2.png"

#define AETHERA_UISERVICE_CONTACTS          AETHERA_UISERVICE"/Contacts"
#define AETHERA_UISERVICE_CONTACTS_DESC     "The Aethera UI service for Contacts."

extern "C"
{
    CONTACT_PLUGIN_API kore::Interface* komcontactsuiservice_factory();
};

class KomContactsMainTreeInterface;
class KomContactsBackendService;
class ContactsUIPartPrivate;

class QLabel;
class QVBoxLayout;
class ContactsMainWidget;

class CONTACT_PLUGIN_API KomContactsUIService : public KomUserInterfaceService
{
  private:
  
    KomContactsUIService();
    
    static KomContactsUIService * instance_;

  public:
  
    static KomContactsUIService * instance();
    virtual ~KomContactsUIService();
    
    /** Get the tree interface.*/
    virtual KomMainTreeInterface* getMainTreeInterface( QListView *parentTree );

    // From UIPart
    /** Query for capabilities.*/
    virtual bool hasCapability(const QString&);
    /** Create a new widget corresponding to the string.*/
    virtual void createWidget(const QString&);
    /** Show the part toolbars.*/
    virtual void showToolbars(QMainWindow *);
    /** Hide the part toolbars.*/
    virtual void hideToolbars(QMainWindow *);
    /** Show the part manus.*/
    virtual void showMenus(QMenuBar *);
    /** Hide the part menu.*/
    virtual void hideMenus(QMenuBar *);

    virtual void setCurrentURL(QString);
    virtual QString getCurrentFolder() const;

    virtual void create();
    virtual void cut();
    virtual void copy();
    virtual void paste();
    virtual void import(QString & filename);
    virtual void exportVcard(QString & filename);

    virtual void buildTree();
    virtual void removeTree();

    virtual QString prettyName() const;

    virtual void execAboutDialog(QWidget * parent) const;

  protected:

    virtual UIPartPrivate * createQTHandler();
    virtual void createActions();

    KomContactsBackendService     * m_backendService;

    QPopupMenu                * m_contactsMenu;
    int                         m_contactsMenuId;

    KomContainerWidget *topWidget;
    QLabel *topBanner;

    KomContactsMainTreeInterface  * m_mainTreeInterface;

    QWidget             * m_topWidget;
    QWidget             * m_topMainBanner;
    ContactsMainWidget  * m_topMainView;
    QVBoxLayout         * m_topWidgetLayout;

    bool                        m_actionsCreated;

};


#endif


