/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
 
#include <stdio.h>
#include <string.h>
#include <qstring.h>
#include <qvaluelist.h>
#include <metaquality.h>
#include <metaline.h>
#include <metaobject.h>
#include <miscfunctions.h>

// #define DEBUG_META_OBJECT

#ifdef IDSTRING
#undef IDSTRING
#endif

#define IDSTRING "Metaobject:"

#define fold      MIMEfold
#define unfold    MIMEunfold


QString &MetaObject::getPrivateLine(QString &source)
{
  int i;
  i=source.find('\n');
  if(i==0)
    i=source.find('\n', 1);
  if(i==-1)
    i=source.length();
  static QString qsLine;
  qsLine = source.left(i);
  source.remove(0,i+1);
#ifdef DEBUG_META_OBJECT
  qDebug(IDSTRING" getPrivateLine - %s", qsLine.latin1());
#endif
  return qsLine;
}


void MetaObject::parse(QString &qs)
{
#ifdef DEBUG_META_OBJECT
  qDebug("MetaObject::parse : %s", qs.latin1());
#endif
  
  children.setAutoDelete(TRUE);
  lines.setAutoDelete(TRUE);
  
  replaceCRLF(qs);
  
  QString mainTitle;
  QString title;
  QString curline = getPrivateLine(qs);
  
  MetaLine * ml = new MetaLine(curline);
  
#ifdef DEBUG_META_OBJECT
  qDebug("MetaLine = %s", ml->getName().latin1() );
#endif
  
  if( -1 != (ml->getName()).find("BEGIN", 0, false) )
  {
    mainTitle = "END:" + ml->getContent();
    objectName = ml->getContent();
    
    while( !curline.isEmpty() &&
           -1 == curline.find(mainTitle, 0, false) )
    {
      curline = getPrivateLine(qs);
#ifdef DEBUG_META_OBJECT
        qDebug(IDSTRING"parse: line1: {%s}\n", curline.latin1());
#endif
      delete ml;
      ml = new MetaLine(curline);
      if( !ml )
      {
        break;
      }
      
#ifdef DEBUG_META_OBJECT
        qDebug(IDSTRING"parse: line1: [%s] [%s]\n", (const char *)ml->getName(),
          (const char *)ml->getContent());
#endif

      // test if i have another block
      if( -1 != (ml->getName()).find("BEGIN", 0, false) )
      {
        title = "END:" + ml->getContent();
#ifdef DEBUG_META_OBJECT
          qDebug(IDSTRING"parse:     2: [%s]\n", (const char *)title);
#endif

        QString block = curline;
        while( !curline.isEmpty() && -1 == curline.find(title, 0, false) )
        {
          curline = getPrivateLine(qs);
#ifdef DEBUG_META_OBJECT
            qDebug(IDSTRING"parse: line2: %s\n", (const char *)curline);
#endif
          
          delete ml;
          ml = new MetaLine(curline);
          
          block += "\n" + curline;
        }
        
        block += "\n";
        
#ifdef DEBUG_META_OBJECT
        qDebug(IDSTRING"parse: new block:\n%s\n---\n", (const char *)block);
        qDebug(IDSTRING"parse: new block append\n");
        qDebug(IDSTRING"parse: new block parse done\n");
#endif
        children.append( new MetaObject(block, this) );
#ifdef DEBUG_META_OBJECT
        qDebug(IDSTRING"parse: new block append done\n");
#endif
      }
      else
      {
#ifdef DEBUG_META_OBJECT
        qDebug(IDSTRING"parse: new line append\n");
#endif
        if( !curline.isEmpty() )
          lines.append( new MetaLine(curline) );
#ifdef DEBUG_META_OBJECT
        qDebug(IDSTRING"parse: new line append done\n");
#endif
      }
    }
  }
  else
  {
#ifdef DEBUG_META_OBJECT
    qDebug(IDSTRING"parse: ERROR: Invalid structure\n");
#endif
    lines.clear();
    text = "";
    children.clear();
    objectName = "UnknownObject";
  }
  
  delete ml;
}

void MetaObject::appendLine(QString line)
{
  // if the line is empty, nothing happens
  if(line.stripWhiteSpace().isEmpty())
    return;
  lines.insert(0,new MetaLine(line));
}


MetaObject::MetaObject()
{
  objectName="";
}

MetaObject::MetaObject(QString qs, MetaObject *dp)
{
  directParent = dp;
  unfold(qs);
  parse(qs);
}


MetaObject::MetaObject(MetaObject &mo)
{
  lines=mo.getLines();
  children=mo.getChildren();
  objectName=mo.getObjectName();
  directParent=mo.getDirectParent();
}

MetaObject &MetaObject::operator=(MetaObject &mo)
{
  lines=mo.getLines();
  children=mo.getChildren();
  objectName=mo.getObjectName();
  return *this;
}

MetaObject::~MetaObject()
{
  // deleting the childrens...
  unsigned it;
  #ifdef DEBUG_META_OBJECT
    qDebug("Deleting children: [");
  #endif
  for(it=0; it<children.count();it++)
  {
    #ifdef DEBUG_META_OBJECT
      qDebug(".");
    #endif
    children.remove((unsigned)0);
  }
  #ifdef DEBUG_META_OBJECT
    qDebug("]\n");
  #endif
  // deleting the lines ...
  #ifdef DEBUG_META_OBJECT
    qDebug("Deleting lines: [");
  #endif
  for(it=0; it<lines.count();it++)
  {
    #ifdef DEBUG_META_OBJECT
      qDebug(".");
    #endif
    lines.remove((unsigned)0);
  }
  #ifdef DEBUG_META_OBJECT
    qDebug("]\n");
  #endif
}

MetaLine & MetaObject::operator[](QString qs)
{
  unsigned it;
  for(it=0;it<lines.count();it++)
  {
    if(lines.at(it)->getName().find(qs,0,false)==0)
      return *(lines.at(it));
  }
  
  qDebug("MetaObject::operator = null");
  MetaLine * emptyLine = 0;
  return *emptyLine;
}

MetaLine & MetaObject::getLine(int idx)
{
  if((unsigned)idx>lines.count() || idx<0)
  {
    qDebug("MetaObject::getLine = null");
    MetaLine * emptyLine = 0;
    return *emptyLine;
  }
  
  return *(lines.at(idx));
}

MetaObject *MetaObject::operator[](int index)
{
  return (children.at(index));
}

QValueList<int> MetaObject::query(QString name, 
        QString includedQuality, QString excludedQuality)
{
  QValueList<int> result;
  bool test;
  unsigned it;
  for(it=0;it<lines.count();it++)
  {
    if(0 == lines.at(it)->getName().find(name,0,false))
    {
      test=lines.at(it)->checkQualities(includedQuality, excludedQuality);
      if(test) result.append(it);
    }
  }
  return result;
}

int MetaObject::extendedQuery(QString name, 
        QString includedQuality, QString excludedQuality, 
        QString includedContent, QString excludedContent,
        intList *results)
{
  bool test;
  if(results==0)
    return -1;
  results->setAutoDelete(true);
  unsigned it;
  unsigned counter=0;
  for(it=0;it<lines.count();it++)
  {
    if(lines.at(it)->getName().find(name,0,false)==0)
    {
      test=lines.at(it)->checkQualities(includedQuality, excludedQuality);
      if(test && !includedContent.isEmpty() &&
         (lines.at(it)->getContent().find(includedContent,0,false)==-1)) 
        test=FALSE;
      if( test && !excludedContent.isEmpty() &&
         (lines.at(it)->getContent().find(excludedContent,0,false)!=-1)) 
        test=FALSE;
      if(test){
        int *match;
        match=new int;
        *match=it;
        results->append(match);
        counter++;
      }
    }
  }
  return counter;
}
void MetaObject::appendLineQuality(QString name, QString quality)
{
  unsigned it;
  for(it=0;it<lines.count();it++)
  {
    if(lines.at(it)->getName().find(name,0,false)==0)
    {
      lines.at(it)->addQualities(quality);
      return;
    }
  }
  return;
}
void MetaObject::insertLineContent(QString name, QString content, int pos)
{
  unsigned it;
  for(it=0;it<lines.count();it++)
  {
    if(lines.at(it)->getName().find(name,0,false)==0)
    {
      if(pos==-1)
        lines.at(it)->setContent(lines.at(it)->getContent()+content);
      else
        lines.at(it)->setContent(
            lines.at(it)->getContent().insert(pos,content));
      return;
    }
  }
  return;
}
void MetaObject::replaceLineQuality(QString name, QString quality)
{
  unsigned it;
  for(it=0;it<lines.count();it++)
  {
    if(lines.at(it)->getName().find(name,0,false)==0)
    {
      lines.at(it)->replaceQualities(quality);
      return;
    }
  }
  return;
}

void MetaObject::replaceLine(int lineno, QString lineContent)
{
  if(lineno!=-1 && (unsigned)lineno<lines.count())
    lines.remove(lineno);
  else
    lineno=0;
  lines.insert(lineno,new MetaLine(lineContent));
}


void MetaObject::replaceLineContent(QString name, QString content)
{
  unsigned it;
  for(it=0;it<lines.count();it++)
  {
    if(lines.at(it)->getName().find(name,0,false)==0)
    {
      lines.at(it)->setContent(content);
      return;
    }
  }
  return;
}
bool MetaObject::eraseLine(QString name)
{
  unsigned it;
  for(it=0;it<lines.count();it++)
  {
    if(lines.at(it)->getName().find(name,0,false)==0)
    {
      lines.remove(it);
      return TRUE;
    }
  }
  return FALSE;
}
QString MetaObject::recompose()
{
  text="BEGIN:"+objectName+"\n";
  // now adding the lines...
  for(unsigned i=0;i<lines.count()-1;i++)
    text+=lines.at(i)->recompose()+"\n";
  // and adding the nested objects
  {
  for(unsigned i=0;i<children.count();i++)
    text+=children.at(i)->recompose()+"\n";
  }
  text+="END:"+objectName;
  fold(text);
  return text;
}

void MetaObject::deleteNestedObject(int idx)
{
  if (idx>int(children.count()) || idx < 0)
    return;
  children.remove(idx);
}

void MetaObject::replaceCRLF(QString &source)
{
  if(source==QString::null) return;
  for(unsigned i=0;i<source.length();)
  {
    i=source.find("\r\n",i);
    if(i<source.length())
      source.replace(i,2,"\n");
  }
}




