/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 /**
   * Changed by the Eugen C. <eug@thekompany.com> for kore plug in support.
   *  Date: 27 Sep. 2001
   */

#include <qstring.h>
#include <qsize.h>
#include <qpoint.h>
#include <qfont.h>
#include <qdatastream.h>
#include <qcstring.h>
#include <qvariant.h>

#include <vcontactobject.h>
#include <vobject.h>
#include <vobjectfactory.h>
#include <queryprocessor.h>
#include <metaobject.h>
#include <miscfunctions.h>
#include <vfsmodule.h>
#include <komaetheratools.h>
#include <defines.h>

/*
  #define MORE_SECURE
*/

#define IDSTRING "VContactObject: "

// #define DEBUG_CO

// implementation for the contact object

VContactObject::VContactObject(const QString & _path)
  : ContactObjectInterface(_path)
{
  // restore the object properties from the server
  sync();
}

VContactObject::VContactObject(const QString & _path, const QByteArray & _data)
  : ContactObjectInterface(_path)
{
  // restore the object properties from the server
  if( !_data || !_data.size() )
  {
    sync();
  }
  else
    _narrow(_data);
}

VContactObject::~VContactObject()
{
  // nothing here
}

QByteArray VContactObject::_stream()
{
  if( _uid.isEmpty() )
  {
    _uid = getUID();
  }
  
  QByteArray tdata;
  QDataStream tstream(tdata, IO_WriteOnly);

  /*
   * stream the object properties
   */
  tstream << QString("##VER##") << QString("%1").arg( VCONTACT_VERSION )
          << _uid << _fn << _n
          << _homeAdr << _workAdr << _phone
          << _emails << _defaultEmail
          << _notes << _URLs << _nick << _security
          << _bday << _org << _title << _role
          << _classCard << _categories << _category
          << _relatedObjectList << _extraData;

  return tdata;
}

void VContactObject::_narrow(const QByteArray &tdata)
{
  // check if the stream actually contains any data
  if(tdata.size())
  {
    QDataStream tstream(tdata, IO_ReadOnly);
    QString version;
    
    tstream >> version;
    if( "##VER##" == version )
    {
      tstream >> version;
      int iVersion = version.toInt();
      
      if( iVersion == VCONTACT_VERSION )
      {
        // unstream the object properties
        tstream >> _uid >> _fn >> _n
                >> _homeAdr >> _workAdr >> _phone
                >> _emails >> _defaultEmail
                >> _notes >> _URLs >> _nick >>_security
                >> _bday >> _org >>_title >> _role
                >> _classCard >> _categories >> _category
                >> _relatedObjectList >> _extraData;
      }
      else
      {
        qDebug("VContact streaming is bad.");
        _uid = QString::null;
        _fn = QString::null;
        _n=QStringList();
        _homeAdr=QStringList();
        _workAdr=QStringList();
        _phone.clear();
        _emails=QStringList();
        _defaultEmail=-1;
        _security=Private;
        _notes = QString::null;
        _URLs=QStringList();
        _nick = QString::null;
        _bday = _org = _title = _role = 
        _classCard = _categories = _category = QString::null;
      }
    }
    else
    {
      _fn = version;
      
      // unstream the object properties
      tstream >> _n
              >> _homeAdr >> _workAdr >> _phone
              >> _emails >> _defaultEmail
              >> _notes >> _URLs >> _nick >>_security
              >> _bday >> _org >>_title >> _role
              >> _classCard >> _categories >> _category
              >> _relatedObjectList >> _extraData;
      
      // the old contacts didn't have UID
      // so, just generate a new one
      if( _uid.isEmpty() )
      {
        _uid = getUID();
      }
    }
  }
  else
  {
    _uid = QString::null;
    _fn = QString::null;
    _n=QStringList();
    _homeAdr=QStringList();
    _workAdr=QStringList();
    _phone.clear();
    _emails=QStringList();
    _defaultEmail=-1;
    _security=Private;
    _notes = QString::null;
    _URLs=QStringList();
    _nick = QString::null;
    _bday = _org = _title = _role = 
    _classCard = _categories = _category = QString::null;
  }
}

bool VContactObject::getProperty(const QString &query, Any &property)
{
  qDebug("VContactObject::getProperty(%s)", query.ascii());
  if(query=="fn")
  {
    property=Any(_fn);
    return true;
  }
  if(query=="n")
  {
    property=Any(_n);
    return true;
  }
  if(query=="homeAddr")
  {
    property=Any(_homeAdr);
    return true;
  }
  if(query=="phone")
  {

#ifndef _QTWIN_
#warning TODO
#endif

    return false;
#if 0
    property=Any(_phone);
    return true;
#endif
  }
  if(query=="workAddr")
  {
    property=Any(_workAdr);
    return true;
  }
  if(query=="emails")
  {
    property=Any(_emails);
    return true;
  }
  if(query=="defaultEmail")
  {
    property=Any(_defaultEmail);
    return true;
  }
  if(query=="notes")
  {
    property=Any(_notes);
    return true;
  }
  if(query=="url")
  {
    property=Any(_URLs);
    return true;
  }
  if(query=="nick")
  {
    property=Any(_nick);
    return true;
  }
  if(query=="security")
  {
    property=Any(_security);
    return true;
  }
  if(query=="bdate")
  {
    property=Any(_bday);
    return true;
  }
  if(query=="org")
  {
    property=Any(_org);
    return true;
  }
  if(query=="title")
  {
    property=Any(_title);
    return true;
  }
  if(query=="role")
  {
    property=Any(_role);
    return true;
  }
  if(query=="class")
  {
    property=Any(_classCard);
    return true;
  }
  if(query=="categories")
  {
    property=Any(_categories);
    return true;
  }
  if(query=="category")
  {
    property=Any(_category);
    return true;
  }
  if(query=="relatedObjectList")
  {
    property=Any(_relatedObjectList);
    return true;
  }
  return false;
}

bool VContactObject::setProperty(const QString &query, Any &property)
{
  qDebug("VContactObject::setProperty(%s)", query.ascii());
  if(query=="fn")
  {
    _fn=property.toString();
    return true;
  }
  if(query=="n")
  {
    _n=property.toStringList();
    return true;
  }
  if(query=="homeAddr")
  {
    _homeAdr=property.toString();
    return true;
  }
  if(query=="phone")
  {
    
#ifndef _QTWIN_
#warning TODO
#endif

    return false;
#if 0
    _phone=property.toStringList();
    return true;
#endif
  }
  if(query=="workAddr")
  {
    _workAdr=property.toString();
    return true;
  }
  if(query=="emails")
  {
    _emails=property.toStringList();
    return true;
  }
  if(query=="defaultEmail")
  {
    _defaultEmail=property.toInt();
    return true;
  }
  if(query=="notes")
  {
    _notes=property.toString();
    return true;
  }
  if(query=="url")
  {
    _URLs=property.toStringList();
    return true;
  }
  if(query=="nick")
  {
    _nick=property.toString();
    return true;
  }
  if(query=="security")
  {
    _security=property.toInt();
    return true;
  }
  if(query=="bdate")
  {
    _bday=property.toString();
    return true;
  }
  if(query=="org")
  {
    _org=property.toString();
    return true;
  }
  if(query=="title")
  {
    _title=property.toString();
    return true;
  }
  if(query=="class")
  {
    _classCard=property.toString();
    return true;
  }
  if(query=="role")
  {
    _role=property.toString();
    return true;
  }
  if(query=="categories")
  {
    _categories=property.toString();
    return true;
  }
  if(query=="category")
  {
    _category=property.toString();
    return true;
  } 
  if(query=="relatedObjectList")
  {
    _relatedObjectList=property.toStringList();
    return true;
  }
  return false;
}

QueryProcessor::QueryType VContactObject::queryType(const QString &query)
{
  qDebug("VContactObject::queryType(%s)", query.ascii());
  if(query=="fn" || query=="n"
    || query=="homeAdr" || query=="phone" || query=="phoneType"
    || query=="workAdr"
    || query=="emails" || query=="defaultEmail"
    || query=="notes"
    || query=="URLs"
    || query=="nick"
    || query=="security"
    || query=="bdate"
    || query=="org" || query=="title" || query=="role" || query=="class"
    || query=="categories" || query=="category"
    || query=="relatedObjectList")
    return QueryProcessor::ReadWriteQuery;
  
  
  return QueryProcessor::InvalidQuery;
}

// implementation of the contact object factory

#ifdef _QTWIN_
VObject *VContactObjectFactory::_new(const QString & _path)
#else
VContactObject *VContactObjectFactory::_new(const QString & _path)
#endif
{
  // check if it is possible to create a new contact
  if(vfs()->canDefine(_path) && !vfs()->reference(_path))
    return new VContactObject(_path);
  else
    return 0;
}

#ifdef _QTWIN_
VObject *VContactObjectFactory::_new
#else
VContactObject *VContactObjectFactory::_new
#endif
(
  const QString     & _path,
  const QByteArray  & _data
)
{
  return new VContactObject(_path, _data);
}

