/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  accountmanagerwidget.cpp
  -  Account manager widget - derived from AccountManagerWidgetBase to handle
     the complexity.
                             -------------------
    begin                : Thu Oct  4 23:42:06 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qtl.h>

#include <qlistview.h>
#include <qheader.h>

#include <accounts.h> // Unfortunate naming. It actually contains class Account.
#include <accountmanagerwidget.h>
#include <accountmanagerlistitem.h>
#include <accountwizard.h>
#include <accounteditdialog.h>

#include <vfsmodule.h>
#include <vfolder.h>

AccountManagerWidget::AccountManagerWidget(QWidget * parent, const char * name)
  : AccountManagerWidgetBase(parent, name)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
    else
    {
      qDebug("AccountManagerWidget: Can't find VFS module");
    }
  }
  
  assert(0 != vfs);
  assert(0 != clientRMI);
  
  // Button clicked connections are already made in designer.
  accountList->header()->hide();
}

AccountManagerWidget::~AccountManagerWidget()
{
}

  void
AccountManagerWidget::load()
{
  qDebug("Getting account info from manager");

  accountList->clear();

  // Make sure the manager is up-to-date.

  clientRMI->loadAccounts();

  QStringList accountNameList;

  accountNameList += clientRMI->getAccountNames(Account::MAILBOX);
  accountNameList += clientRMI->getAccountNames(Account::POP3);
  accountNameList += clientRMI->getAccountNames(Account::IMAP4);

  QString defaultAccount = clientRMI->getDefaultAccount();

  QStringList::ConstIterator it;

  for (it = accountNameList.begin(); it != accountNameList.end(); ++it)
  {
    AccountManagerListItem * item = new AccountManagerListItem
      (
        accountList,
        clientRMI->getAccount(*it)
      );

    item->setDefault((*it) == defaultAccount);
  }
}

  void
AccountManagerWidget::save()
{
  qDebug("\n Save the accounts settings.");
  
  // TODO : The account manager should save all the accounts
  // Just use Account::save() for each account
  clientRMI->saveAccounts();
}

  void
AccountManagerWidget::defaults()
{
  qDebug("AccountManagerWidget::defaults: STUB - logical ?");
}

  void
AccountManagerWidget::slotAddPOP3()
{
  AccountWizard wizard(Account::POP3, this, "Account wizard (POP3)");

  if (QDialog::Accepted != wizard.exec())
    return;

  Account account = wizard.account();

  qDebug("Adding account with type: %d", account.type);

  if( clientRMI->addAccount(account) )
  {
    AccountManagerListItem * i =
      new AccountManagerListItem(accountList, account);
  
    i->setEnabled(true);
  }
}

  void
AccountManagerWidget::slotAddIMAP4()
{
  AccountWizard wizard(Account::IMAP4, this, "Account wizard (IMAP4)");

  if (QDialog::Accepted != wizard.exec())
    return;

  Account account = wizard.account();

  qDebug("Adding account with type: %d", account.type);

  if( clientRMI->addAccount(account) )
  {
    AccountManagerListItem * i =
      new AccountManagerListItem(accountList, account);
  
    i->setEnabled(true);
    
    VFolder * folderRef = vfs->folderRef( vfs->root() + "/Mail");
    if( folderRef )
    {
      folderRef = vfs->_new(folderRef->path() + "/" + account.accname,
                            "imap", "genmail", "", "");
      if( folderRef )
      {
        folderRef->setFlags("c");
        folderRef = vfs->_new(folderRef->path() + "/" + "Inbox",
                              "imap", "mail", "", "thisInstance",
                              QString("imap_") + account.accname);
        if(folderRef)
        {
          folderRef->setFlags("c");
        }
      }
      
      if( !folderRef )
      {
        qDebug("Cannot create the IMAP folders.");
      }
    }
  }
}

  void
AccountManagerWidget::slotAddCitadel()
{
  // it's important to have 'kolab' in widget name
  AccountWizard wizard(Account::IMAP4, this, "Account wizard (Citadel)");

  if (QDialog::Accepted != wizard.exec())
    return;

  Account account = wizard.account();

  qDebug("Adding a Citadel account.");

  if( clientRMI->addAccount(account) )
  {
    AccountManagerListItem * i =
      new AccountManagerListItem(accountList, account);
  
    i->setEnabled(true);
    
    // build the new folders from VFS
    VFolder * folderRef = vfs->folderRef( vfs->root() + "/Mail");
    if( folderRef )
    {
      folderRef = vfs->_new(folderRef->path() + "/" + account.accname,
                            "imap", "genmail", "", "");
      if( folderRef )
      {
        folderRef->setFlags("c");
        folderRef = vfs->_new(folderRef->path() + "/" + "Inbox",
                              "imap", "mail", "", "thisInstance",
                              QString("imap_") + account.accname);
        if(folderRef)
        {
          folderRef->setFlags("c");
        }
      }
      
      if( !folderRef )
      {
        qDebug("Cannot create the IMAP folders.");
      }
    }
    
    // build Kolab folder for DateBook
    folderRef = vfs->folderRef( vfs->root() + "/Datebook/Kolab");
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Datebook");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, "Kolab", "datebook", "datebook");
        if( !folderRef )
        {
          qDebug("Kolab folder for Datebook cannot be created.");
        }
        else
        {
          // build the new calendar object for Kolab
        }
      }
    }
    
    // build Kolab folder for Notes
    folderRef = vfs->folderRef( vfs->root() + "/Notes/Kolab");
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Notes");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, "Kolab", "notes", "notes");
        if( !folderRef )
        {
          qDebug("Kolab folder for Notes cannot be created.");
        }
      }
    }
    folderRef = vfs->folderRef( vfs->root() + "/Notes/Kolab/" + account.accname);
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Notes/Kolab");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, account.accname, "notes", "notes");
        if( !folderRef )
        {
          qDebug("Kolab folder for Notes cannot be created.");
        }
      }
    }
    
    // build Kolab folder for Contacts
    folderRef = vfs->folderRef( vfs->root() + "/Contacts/Kolab");
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Contacts");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, "Kolab", "contacts", "contacts");
        if( !folderRef )
        {
          qDebug("Kolab folder for Contacts cannot be created.");
        }
      }
    }
    folderRef = vfs->folderRef( vfs->root() + "/Contacts/Kolab/" + account.accname);
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Contacts/Kolab");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, account.accname, "contacts", "contacts");
        if( !folderRef )
        {
          qDebug("Kolab folder for Contacts cannot be created.");
        }
      }
    }
  }
}

  void
AccountManagerWidget::slotAddKolab()
{
  // it's important to have 'kolab' in widget name
  AccountWizard wizard(Account::IMAP4, this, "Account wizard (Kolab)");

  if (QDialog::Accepted != wizard.exec())
    return;

  Account account = wizard.account();

  qDebug("Adding a kolab account.");

  if( clientRMI->addAccount(account) )
  {
    AccountManagerListItem * i =
      new AccountManagerListItem(accountList, account);
  
    i->setEnabled(true);
    
    // build the new folders from VFS
    VFolder * folderRef = vfs->folderRef( vfs->root() + "/Mail");
    if( folderRef )
    {
      folderRef = vfs->_new(folderRef->path() + "/" + account.accname,
                            "imap", "genmail", "", "");
      if( folderRef )
      {
        folderRef->setFlags("c");
        folderRef = vfs->_new(folderRef->path() + "/" + "Inbox",
                              "imap", "mail", "", "thisInstance",
                              QString("imap_") + account.accname);
        if(folderRef)
        {
          folderRef->setFlags("c");
        }
      }
      
      if( !folderRef )
      {
        qDebug("Cannot create the IMAP folders.");
      }
    }
    
    // build Kolab folder for DateBook
    folderRef = vfs->folderRef( vfs->root() + "/Datebook/Kolab");
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Datebook");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, "Kolab", "datebook", "datebook");
        if( !folderRef )
        {
          qDebug("Kolab folder for Datebook cannot be created.");
        }
        else
        {
          // build the new calendar object for Kolab
        }
      }
    }
    
    // build Kolab folder for Notes
    folderRef = vfs->folderRef( vfs->root() + "/Notes/Kolab");
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Notes");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, "Kolab", "notes", "notes");
        if( !folderRef )
        {
          qDebug("Kolab folder for Notes cannot be created.");
        }
      }
    }
    folderRef = vfs->folderRef( vfs->root() + "/Notes/Kolab/" + account.accname);
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Notes/Kolab");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, account.accname, "notes", "notes");
        if( !folderRef )
        {
          qDebug("Kolab folder for Notes cannot be created.");
        }
      }
    }
    
    // build Kolab folder for Contacts
    folderRef = vfs->folderRef( vfs->root() + "/Contacts/Kolab");
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Contacts");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, "Kolab", "contacts", "contacts");
        if( !folderRef )
        {
          qDebug("Kolab folder for Contacts cannot be created.");
        }
      }
    }
    folderRef = vfs->folderRef( vfs->root() + "/Contacts/Kolab/" + account.accname);
    if( !folderRef )
    {
      folderRef = vfs->folderRef( vfs->root() + "/Contacts/Kolab");
      if(folderRef)
      {
        folderRef = vfs->createFolder(folderRef, account.accname, "contacts", "contacts");
        if( !folderRef )
        {
          qDebug("Kolab folder for Contacts cannot be created.");
        }
      }
    }
  }
}

  void
AccountManagerWidget::slotEdit()
{
  QListViewItem * i = accountList->currentItem();

  if (0 == i)
    return;

  AccountManagerListItem * item = static_cast<AccountManagerListItem *>(i);

  QString accountName = item->account().accname;

  QStringList accountNameList;

  accountNameList += clientRMI->getAccountNames(Account::MAILBOX);
  accountNameList += clientRMI->getAccountNames(Account::POP3);
  accountNameList += clientRMI->getAccountNames(Account::IMAP4);

  if (!accountNameList.contains(accountName))
  {
    qDebug("BUG: Can't find account name in server's list");
    return;
  }

  Account acc = clientRMI->getAccount(accountName);
  QString defaultAccountName = clientRMI->getDefaultAccount();
  bool bDefault = false;
  if( !defaultAccountName.isEmpty() && 
      defaultAccountName == acc.accname )
    bDefault = true;

  AccountEditDialog dialog(acc, this, "Account edit dialog");

  if (QDialog::Accepted != dialog.exec())
    return;

  Account account = dialog.account();

  // TODO: sometime is enough to change the account attributes
  // Also IMAP needs something special
  // 
  // remove the edited account
  clientRMI->deleteAccount(accountName);
  // add the new one
  clientRMI->addAccount(account);
  
  if( bDefault )
  {
    clientRMI->setDefaultAccount(account.accname);
  }
  
// TODO: enable it as soon as IMAP account name can be changed too
//  if( account.kolab )
//  {
//    // If it's a Kolab account, rename(move) the associated folders
//    VFolder * folder = vfs->folderRef(vfs->root() + "/Contacts/Kolab/" + accountName);
//    if( folder )
//    {
//      vfs->moveFolder(folder->path(), folder->parentFolder()->path() + "/" + account.accname);
//    }
//    
//    folder = vfs->folderRef(vfs->root() + "/Notes/Kolab/" + accountName);
//    if( folder )
//    {
//      vfs->moveFolder(folder->path(), folder->parentFolder()->path() + "/" + account.accname);
//    }
//  }

  item->setAccount(account);
}

  void
AccountManagerWidget::slotRemove()
{
  QListViewItem * currentItem = accountList->currentItem();
  Account currentAccount = static_cast<AccountManagerListItem *>(currentItem)->account();

  if (0 == currentItem)
  {
    qDebug("AccountManagerWidget::slotRemove() : current item is null");
    return;
  }

  bool bDeleted = clientRMI->deleteAccount(currentAccount.accname);
  
  if( bDeleted )
  {
    delete currentItem;
    currentItem = 0;
    
    // If it's a Kolab account, remove the associated folders
    vfs->deleteFolder(vfs->root() + "/Contacts/Kolab/" + currentAccount.accname);
    vfs->deleteFolder(vfs->root() + "/Notes/Kolab/" + currentAccount.accname);
    vfs->deleteFolder(vfs->root() + "/Datebook/Kolab/" + currentAccount.accname);
  }
  else
  {
    qDebug("AccountManagerWidget::slotRemove - false");
  }
}

  void
AccountManagerWidget::slotSetDefault()
{
  QListViewItem * currentItem = accountList->currentItem();

  if (0 == currentItem)
  {
    qDebug("Set 0 to default ? I don't think so.");
    return;
  }

  for (QListViewItemIterator it(accountList); it.current(); ++it)
  {
    AccountManagerListItem * item =
      static_cast<AccountManagerListItem *>(it.current());

    item->setDefault(false);
  }

  AccountManagerListItem * item =
    (static_cast<AccountManagerListItem *>(currentItem));

  if( item )
  {
    clientRMI->setDefaultAccount(item->account().accname);
  
    item->setDefault(true);
  }
}

  void
AccountManagerWidget::slotToggleEnable()
{
  QListViewItem * currentItem = accountList->currentItem();

  if (0 == currentItem)
  {
    qDebug("Toggle 0 QListViewItem en/disabled ? I don't think so.");
    return;
  }

  AccountManagerListItem * item =
    (dynamic_cast<AccountManagerListItem *>(currentItem));

  if( item )
  {
    clientRMI->toggleAccountActive(item->account().accname);
  
    item->toggleAccountEnabled();
  }
  else
    qDebug("AccountManagerWidget::slotToggleEnable() : item null");
}

