/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  accountwizard.cpp
  -  Ask user for info about account.
                             -------------------
    begin                : Mon Oct  8 22:25:29 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
 ***************************************************************************/

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qbuttongroup.h>

#include <pop3configwidget.h>
#include <imap4configwidget.h>
#include <accountwizard.h>
#include <accountinfoconfigwidget.h>
#include <smtpconfigwidget.h>
#include <kolabfoldersbase.h>
#include <certificatesmanager.h>

class AccountWizard::Private
{
  public:

    Private();
    ~Private();
    
    Account accountInfo;

    AccountInfoConfigWidget * page1;
    QWidget                 * page2;
    SMTPConfigWidget        * page3;
    KolabFoldersBase        * page4;
};

AccountWizard::Private::Private()
  : page1(0),
    page2(0),
    page3(0),
    page4(0)
{
}

AccountWizard::Private::~Private()
{
  // the QWidgets are deleted by QT.
}

AccountWizard::AccountWizard
(
  Account::ServerType   type,
  QWidget             * parent,
  const char          * name
)
  : QWizard(parent, name, true)
{
  QString temp(name);
  d = new Private;
  d->accountInfo.type = type;
  
  if( temp.contains("kolab", false) )
  {
    d->accountInfo.groupware = true;
    d->accountInfo.groupType = Account::Kolab1;
  }
  else if( temp.contains("citadel", false) )
  {
    d->accountInfo.groupware = true;
    d->accountInfo.groupType = Account::Citadel;
  }
  else
  {
    d->accountInfo.groupware = false;
  }

  d->page1 = new AccountInfoConfigWidget(this);

  connect
    (
      d->page1->accountNameLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotUpdate())
    );

  connect
    (
      d->page1->yourNameLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotUpdate())
    );

  connect
    (
      d->page1->emailLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotUpdate())
    );

  addPage(d->page1, tr("Create account"));

  switch (type)
  {
    case Account::POP3:

      d->page2 = new POP3ConfigWidget(this);

      connect
        (
          static_cast<POP3ConfigWidget *>(d->page2)->usernameLineEdit,
          SIGNAL(textChanged(const QString &)),
          SLOT(slotUpdate())
        );

      break;

    case Account::IMAP4:

      d->page2 = new IMAP4ConfigWidget(this);

      connect
        (
          static_cast<IMAP4ConfigWidget *>(d->page2)->usernameLineEdit,
          SIGNAL(textChanged(const QString &)),
          SLOT(slotUpdate())
        );

      break;

    default:
      qDebug("No page 2 ?");
      return;
      break;
  }

  addPage(d->page2, tr("Receiving"));

  d->page3 = new SMTPConfigWidget(this);

  connect
    (
      d->page3->addressLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotUpdate())
    );
//  connect
//    (
//      static_cast<SMTPConfigWidget *>(d->page3)->usernameLineEdit,
//      SIGNAL(textChanged(const QString &)),
//      SLOT(slotUpdate())
//    );

  addPage(d->page3, tr("Sending"));
  
  if( d->accountInfo.groupware )
  {
    d->page4 = new KolabFoldersBase(this, "Kolab Folders Base");
    addPage(d->page4, tr("Kolab folders"));
  }

  slotUpdate();
}

AccountWizard::~AccountWizard()
{
  delete d;
}

  Account
AccountWizard::account() const
{
  d->accountInfo.accname  = d->page1->accountNameLineEdit->text();
  d->accountInfo.name     = d->page1->yourNameLineEdit->text();
  d->accountInfo.from     = d->page1->emailLineEdit->text();
  d->accountInfo.reply    = d->page1->replyToLineEdit->text();

  d->accountInfo.enabled  = true;

  switch (d->accountInfo.type)
  {
    case Account::POP3:
      {
        POP3ConfigWidget * w = static_cast<POP3ConfigWidget *>(d->page2);
        if(w)
        {
          d->accountInfo.type     = Account::POP3;
  
          d->accountInfo.server   = w->serverLineEdit->text();
          d->accountInfo.port     = w->portSpinBox->value();
          d->accountInfo.user     = w->usernameLineEdit->text();
          d->accountInfo.password = w->passwordLineEdit->text();
  
          d->accountInfo.haspassword = w->rememberPasswordCheckBox->isChecked();
          d->accountInfo.leave    = w->leaveOnServerCheckBox->isChecked();
          d->accountInfo.sync     = w->syncOnDeleteCheckBox->isChecked();
          d->accountInfo.sorted   = w->sortCheckBox->isChecked();
          d->accountInfo.ssl      = w->secureCheckBox->isChecked();
          d->accountInfo.tls      = w->enableTLS->isChecked();
          d->accountInfo.useCertificates = w->checkBoxCertificate->isChecked();
          
          if( d->accountInfo.useCertificates && !w->certImportFile.isEmpty() )
          {
            QString certFileName = CertificatesManager::instance()->
              setCertificate(d->accountInfo.accname, "POP3", w->certImportFile);
            if( certFileName.isEmpty() )
            {
              qDebug("POP3 : Cannot set the certificate.");
            }
          }
        }
      }
      break;

    case Account::IMAP4:
      {
        IMAP4ConfigWidget * w = static_cast<IMAP4ConfigWidget *>(d->page2);
        if(w)
        {
          d->accountInfo.type     = Account::IMAP4;
  
          d->accountInfo.server   = w->serverLineEdit->text();
          d->accountInfo.port     = w->portSpinBox->value();
          d->accountInfo.user     = w->usernameLineEdit->text();
          d->accountInfo.password = w->passwordLineEdit->text();
  
          d->accountInfo.haspassword  = w->rememberPasswordCheckBox->isChecked();
          d->accountInfo.ssl          = w->secureCheckBox->isChecked();
          d->accountInfo.tls          = w->enableTLS->isChecked();
          d->accountInfo.useCertificates = w->checkBoxCertificate->isChecked();
          
          if( d->accountInfo.useCertificates && !w->certImportFile.isEmpty() )
          {
            QString certFileName = CertificatesManager::instance()->
              setCertificate(d->accountInfo.accname, "IMAP4", w->certImportFile);
            if( certFileName.isEmpty() )
            {
              qDebug("IMAP4 : Cannot set the certificate.");
            }
          }
          
          if( d->page4 )
          {
            d->accountInfo.calendarFolder = d->page4->calendarLineEdit->text();
            d->accountInfo.contactsFolder = d->page4->contactsLineEdit->text();
            d->accountInfo.notesFolder    = d->page4->notesLineEdit->text();
            d->accountInfo.todoFolder     = d->page4->todoLineEdit->text();
          }
        }
      }
      break;

    default:
      break;
  }

  SMTPConfigWidget * w = static_cast<SMTPConfigWidget *>(d->page3);
  if(w)
  {
    d->accountInfo.smtp   = w->addressLineEdit->text();
    d->accountInfo.s_port = w->portSpinBox->value();
    d->accountInfo.s_type = Account::Sendmail;
    d->accountInfo.s_ssl  = w->secureCheckBox->isChecked();
    d->accountInfo.s_tls  = w->enableTLS->isChecked();
    d->accountInfo.s_useCertificates  = w->checkBoxCertificate->isChecked();
    
    if( d->accountInfo.s_useCertificates && !w->certImportFile.isEmpty() )
    {
      QString certFileName = CertificatesManager::instance()->
        setCertificate(d->accountInfo.accname, "SMTP", w->certImportFile);
      if( certFileName.isEmpty() )
      {
        qDebug("SMTP : Cannot set the certificate.");
      }
    }
    
    if( w->authCheckBox->isChecked() )
    {
      d->accountInfo.s_user     = w->usernameLineEdit->text();
      d->accountInfo.s_password = w->passwordLineEdit->text();
      d->accountInfo.s_haspassword = w->passwordCheckBox->isChecked();
      
      int i = w->authTypeButtonGroup->id( w->authTypeButtonGroup->selected() );
      switch(i)
      {
        case 0:
          d->accountInfo.s_auth = Account::LOGIN;
          break;
        case 1:
          d->accountInfo.s_auth = Account::PLAIN;
          break;
        case 2:
          d->accountInfo.s_auth = Account::DIGEST_MD5;
          break;
        case 3:
          d->accountInfo.s_auth = Account::CRAM_MD5;
          break;
        default:
          d->accountInfo.s_auth = Account::NONE;
          break;
      }
    }
    else
    {
      d->accountInfo.s_auth = Account::NONE;
    }
  }

  return d->accountInfo;
}

  void
AccountWizard::slotUpdate()
{
  setNextEnabled
    (d->page1,
      (
           !d->page1->accountNameLineEdit->text().isEmpty()
        && !d->page1->yourNameLineEdit->text().isEmpty()
        && !d->page1->emailLineEdit->text().isEmpty()
      )
    );

  switch (d->accountInfo.type)
  {
    case Account::POP3:
      {
        POP3ConfigWidget * w = static_cast<POP3ConfigWidget *>(d->page2);
        setNextEnabled(d->page2, !w->usernameLineEdit->text().isEmpty());
      }
      break;

    case Account::IMAP4:
      {
        IMAP4ConfigWidget * w = static_cast<IMAP4ConfigWidget *>(d->page2);
        setNextEnabled(d->page2, !w->usernameLineEdit->text().isEmpty());
      }
      break;

    default:
      break;
  }

  if( d->accountInfo.groupware )
  {
    setNextEnabled(d->page3, !d->page3->addressLineEdit->text().isEmpty());
    setFinishEnabled(d->page4, true);
  }
  else
  {
    setFinishEnabled(d->page3, !d->page3->addressLineEdit->text().isEmpty());
  }
}

