/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
#include "folderchooserdialog.h"
#include <qregexp.h>
#include "vfolder.h"
#include "vcontactobject.h"

#include <addaddressesdialog.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qcheckbox.h>

#include <addressclass.h>
#include <addresslistclass.h>
//#include <ataddressbook.h>
#include <miscfunctions.h>

#include <komaetheratools.h>
#include <stdio.h>

#define IDSTRING "AddAddressesDialog: "

WidgetsFactory* AddAddressesDialog::addressBookViewFactory = 0;

//#define DEBUG_AAD

// it's a Modal dialog!
AddAddressesDialog::AddAddressesDialog(QWidget *parent, const char *name)
  : AddAddressesDialogBase(parent, name, true),
    _to(0),
    _cc(0),
    _bcc(0)
{
#ifdef DEBUG_AAD
  printf(IDSTRING"Constructor start\n");
#endif
  toList->header()->hide();
  toList->setAllColumnsShowFocus(true);
  ccList->header()->hide();
  ccList->setAllColumnsShowFocus(true);
  bccList->header()->hide();
  bccList->setAllColumnsShowFocus(true);
#ifdef DEBUG_AAD
  printf(IDSTRING"Constructor set-up columns\n");
#endif
  if( !addressBookViewFactory )
  {
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_ADDRESSBOOK_VIEW_FACTORY );
      if( ref.isValid() )
        addressBookViewFactory = dynamic_cast<WidgetsFactory*>(ref.instance());
    }
  }
  
  if( addressBookViewFactory )
  {
    abViewInterface = (AddressBookViewInterface*)addressBookViewFactory->newWidget(this);
    topLayout->insertWidget(0, abViewInterface);
    
    abViewInterface->initMode(  AddressBookViewInterface::Contacts | 
                                AddressBookViewInterface::ShowHeader | 
                                AddressBookViewInterface::ListOnly );
    
    connect(deleteToButton, SIGNAL(clicked()), this, SLOT(deleteToClicked()));
    connect(deleteCcButton, SIGNAL(clicked()), this, SLOT(deleteCcClicked()));
    connect(deleteBccButton, SIGNAL(clicked()), this, SLOT(deleteBccClicked()));
    connect(groupToButton, SIGNAL(clicked()), this, SLOT(groupToClicked()));
    connect(groupCcButton, SIGNAL(clicked()), this, SLOT(groupCcClicked()));
    connect(groupBccButton, SIGNAL(clicked()), this, SLOT(groupBccClicked()));
    connect(toButton, SIGNAL(clicked()), this, SLOT(toClicked()));
    connect(ccButton, SIGNAL(clicked()), this, SLOT(ccClicked()));
    connect(bccButton, SIGNAL(clicked()), this, SLOT(bccClicked()));
    connect(okButton, SIGNAL(clicked()), this, SLOT(okClicked()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelClicked()));
  }
  
#ifdef DEBUG_AAD
  printf(IDSTRING"Constructor end\n");
#endif
}

AddAddressesDialog::~AddAddressesDialog()
{
}

void AddAddressesDialog::setTo(AddressListClass *item)
{
  _to=item;
  if(!_to)
  {
    toFrame->setEnabled(false);
    return;
  }

  // get all the addresses from the list and put them in the listview
  for(int i=0;i<_to->count();i++)
  {
    if( !(*item)[i].email.isEmpty() )
    {
    _toList[0].append( (*item)[i].getTranslated_name() );
    _toList[1].append( (*item)[i].email );
    }
    populateList(toList, _toList[0], _toList[1]);
  }
}

void AddAddressesDialog::setCc(AddressListClass *item)
{
  _cc=item;
  if(!_cc)
  {
    ccFrame->setEnabled(false);
    return;
  }
  // get all the addresses from the list and put them in the listview
  for(int i=0;i<_cc->count();i++)
  {
    if( !(*item)[i].email.isEmpty() )
    {
    _ccList[0].append( (*item)[i].getTranslated_name() );
    _ccList[1].append( (*item)[i].email );
    }
    populateList(ccList, _ccList[0], _ccList[1]);
  }
}

void AddAddressesDialog::setBcc(AddressListClass *item)
{
  _bcc=item;
  if(!_bcc)
  {
    bccFrame->setEnabled(false);
    return;
  }
  // get all the addresses from the list and put them in the listview
  for(int i=0;i<_bcc->count();i++)
  {
    if( !(*item)[i].email.isEmpty() )
    {
    _bccList[0].append( (*item)[i].getTranslated_name() );
    _bccList[1].append( (*item)[i].email);
    }
    populateList(bccList, _bccList[0], _bccList[1]);
  }
}

void AddAddressesDialog::newDialog(AddressListClass *to, 
  AddressListClass *cc, 
  AddressListClass *bcc, 
  QWidget *parent)
{
  AddAddressesDialog *ad = new AddAddressesDialog(parent);
  ad->setTo(to);
  ad->setCc(cc);
  ad->setBcc(bcc);
  ad->exec();
}

void AddAddressesDialog::bccClicked()
{
  QStringList selected;
  abViewInterface->getSelectedEmailList(selected);
  for(unsigned int i=0;i<selected.count();i++)
//  for( QStringList::Iterator it = selected.begin() ; it!= selected.end() ; ++it)
  {
    AddressClass *ac=new AddressClass( selected[i] );
    if(_bccList[1].findIndex((const char *)ac->email)==-1)
    {
      //add the new mail to the qstringlist
      _bccList[0].append( ac->getTranslated_name() );
      _bccList[1].append( QString( ac->email ) );
    }
    delete ac;
  }
  populateList(bccList, _bccList[0], _bccList[1]);
  if(dupesCheck->isChecked())
    checkDuplicates(_bccList[0], _bccList[1], _toList[0], _toList[1], _ccList[0], _ccList[1]);
}

void AddAddressesDialog::ccClicked()
{
  QStringList selected;
  abViewInterface->getSelectedEmailList(selected);
  for(unsigned int i=0;i<selected.count();i++)
  {
    AddressClass *ac=new AddressClass( selected[i] );
    if(_ccList[1].findIndex( QString(ac->email) )==-1)
    {
      //add the new mail to the qstringlist
      _ccList[0].append( ac->getTranslated_name() );
      _ccList[1].append( QString(ac->email) );
    }
    delete ac;
  }
  populateList(ccList, _ccList[0], _ccList[1]);
  if(dupesCheck->isChecked())
    checkDuplicates(_ccList[0], _ccList[1], _toList[0], _toList[1], _bccList[0], _bccList[1]);
}

void AddAddressesDialog::toClicked()
{
  QStringList selected;
  abViewInterface->getSelectedEmailList(selected);
  for(unsigned int i=0;i<selected.count();i++)
  {
   AddressClass *ac=new AddressClass( selected[i] );
    if(_toList[1].findIndex( QString(ac->email) )==-1)
    {
      _toList[0].append( ac->getTranslated_name() );
      _toList[1].append( QString(ac->email) );
    }
    delete ac;
  }
  populateList(toList, _toList[0], _toList[1]);
  if(dupesCheck->isChecked())
    checkDuplicates(_toList[0], _toList[1], _ccList[0], _ccList[1], _bccList[0], _bccList[1]);
}

void AddAddressesDialog::groupClicked(int iField)
{
  QString pathRoot = FolderChooserDialog::getFolder("/Contacts", this);

  if( !pathRoot.isEmpty() )
  {  
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());  
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        qDebug("\n NO VFS Provider !\n");
        return;
      }
    }
    
    VFolder * folder = vfs->folderRef(pathRoot);
    if(folder)
    {
      setUpdatesEnabled(false);
      
      VContactObject * contact = 0;
      VFolderList tree;
      tree.append(folder);
      folder->tree(tree, -1);
      
      for(VFolderListIterator it(tree); it.current(); ++it)
      {
        QPtrList<VObject> & objectList = it.current()->objectList();
        for(QPtrListIterator<VObject> it1(objectList); it1.current(); ++it1)
        {
          contact = static_cast<VContactObject *>(it1.current());
          
          if( contact->emails().count() && !contact->emails()[0].isEmpty() )
          {
            AddressClass ac( contact->fn() + " " + contact->n().join(" ") + " <" + contact->emails()[0] + ">" );
            switch( iField )
            {
              case 0:
              default:
                if(-1 == _toList[1].findIndex( QString(ac.email) ))
                {
                  _toList[0].append( ac.getTranslated_name() );
                  _toList[1].append( QString(ac.email) );
                }
                break;
                
              case 1:
                if(-1 == _ccList[1].findIndex( QString(ac.email) ))
                {
                  _ccList[0].append( ac.getTranslated_name() );
                  _ccList[1].append( QString(ac.email) );
                }
                break;
                
              case 2:
                if(-1 == _bccList[1].findIndex( QString(ac.email) ))
                {
                  _bccList[0].append( ac.getTranslated_name() );
                  _bccList[1].append( QString(ac.email) );
                }
                break;
            }
          }
        }
      }
    
      switch( iField )
      {
        case 0:
        default:
          populateList(toList, _toList[0], _toList[1]);
          if(dupesCheck->isChecked())
            checkDuplicates(_toList[0], _toList[1], _ccList[0], _ccList[1], _bccList[0], _bccList[1]);
          break;
          
        case 1:
          populateList(ccList, _ccList[0], _ccList[1]);
          if(dupesCheck->isChecked())
            checkDuplicates(_ccList[0], _ccList[1], _toList[0], _toList[1], _bccList[0], _bccList[1]);
          break;
          
        case 2:
          populateList(bccList, _bccList[0], _bccList[1]);
          if(dupesCheck->isChecked())
            checkDuplicates(_bccList[0], _bccList[1], _toList[0], _toList[1], _ccList[0], _ccList[1]);
          break;
      }
      
      setUpdatesEnabled(true);
    }
  }
}

void AddAddressesDialog::groupToClicked()
{
  groupClicked(0);
}

void AddAddressesDialog::groupCcClicked()
{
  groupClicked(1);
}

void AddAddressesDialog::groupBccClicked()
{
  groupClicked(2);
}

void AddAddressesDialog::deleteToClicked()
{
  QListViewItem *it= toList->firstChild();
  while(it)
  {
    if(it->isSelected())
      deleteItem(it->text(1),_toList[0], _toList[1]);
    it=it->nextSibling();
  }
  populateList(toList, _toList[0], _toList[1]);
}

void AddAddressesDialog::deleteCcClicked()
{
  QListViewItem *it= ccList->firstChild();
  while(it)
  {
    if(it->isSelected())
      deleteItem(it->text(1),_ccList[0], _ccList[1]);
    it=it->nextSibling();
  }
  populateList(ccList, _ccList[0], _ccList[1]);
}

void AddAddressesDialog::deleteBccClicked()
{
  QListViewItem *it= bccList->firstChild();
  while(it)
  {
    if(it->isSelected())
      deleteItem(it->text(1),_bccList[0], _bccList[1]);
    it=it->nextSibling();
  }
  populateList(bccList, _bccList[0], _bccList[1]);
}

void AddAddressesDialog::deleteItem(QString ocr, QStringList &name, QStringList &mail)
{
  int i=mail.findIndex(ocr);
  if(i==-1 || name.count()!=mail.count()) // then something is wrong... !
    return;
  mail.remove(mail.at(i));
  name.remove(name.at(i));
}

void AddAddressesDialog::deleteItem(int idx, QStringList &name, QStringList &mail)
{
  if(mail.count()<=unsigned(idx) || name.count()!=mail.count()) // then something is wrong... !
    return;
  mail.remove(mail.at(idx));
  name.remove(name.at(idx));
}

void AddAddressesDialog::populateList(QListView *list, QStringList name, QStringList mail)
{
  if(!list || name.count()!=mail.count())
    return;
  list->clear();
  for(unsigned int i=0;i<name.count();i++)
  {
    QListViewItem *it= new QListViewItem(list);
    it->setText(0,name[i]+"  ");
    it->setText(1,mail[i]);
  }
}

void AddAddressesDialog::checkDuplicates(QStringList &list10, QStringList &list11,
      QStringList &list20, QStringList &list21,
      QStringList &list30, QStringList &list31)
{
  bool altered=false;
  for(unsigned int i=0;i<list11.count();i++)
  {
    // check against list2
    int idx=list21.findIndex(list11[i]);
    if(idx!=-1)
    {
      altered=true;
      deleteItem(idx, list20, list21);
    }
    idx=list31.findIndex(list11[i]);
    if(idx!=-1)
    {
      altered=true;
      deleteItem(idx, list30, list31);
    }
  }
  if(altered)
  {
    populateList(toList, _toList[0], _toList[1]);
    populateList(ccList, _ccList[0], _ccList[1]);
    populateList(bccList, _bccList[0], _bccList[1]);
  }
}

QString AddAddressesDialog::makelist(QStringList names, QStringList emails)
{
  QStringList result;
  result.clear();
  for(unsigned int i=0;i<emails.count();i++)
  {
    if( !names[i].isEmpty() )
    result.append( QString("\"") + names[i] + QString("\" ") + emails[i]  );
    else
    result.append( emails[i] );
//    printf(IDSTRING"Address: [%s][%s]\n", (const char *)names[i], (const char *)emails[i]);
//    fflush(stdout);
  }
  return result.join(", ");
}

void AddAddressesDialog::cancelClicked()
{
  close();
}

void AddAddressesDialog::okClicked()
{
  if(_to)
    (*_to)=AddressListClass(makelist(_toList[0], _toList[1]), true);
  if(_cc)
    (*_cc)=AddressListClass(makelist(_ccList[0], _ccList[1]), true);
  if(_bcc)
    (*_bcc)=AddressListClass(makelist(_bccList[0], _bccList[1]), true);
  close();
}

