/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <qfiledialog.h>
#include <qpopupmenu.h>

#include <attachmentpane.h>
#include <attachmentpanewidget.h>
#include <mailobj.h>

#include <vmailattribute.h>
#include <vpartattribute.h>
#include <vfsmimetype.h>
#include <komaetheratools.h>

AttachmentPane::AttachmentPane(VFSModule* vfsModule, QWidget *parent, const char *name, WFlags f)
  : SortingWidget(vfsModule, parent, name, f)
{
//  viewport()->setBackgroundColor(QColor("white"));
  setSelectionMode(QIconView::Extended);
  setAllwaysShowPopup(false);

  makeChildPopup();
  
  connect(this, SIGNAL(clicked(QIconViewItem*)),
          this, SLOT(slotStatusChanged(QIconViewItem*)));
  connect(this, SIGNAL(currentChanged(QIconViewItem*)),
          this, SLOT(slotStatusChanged(QIconViewItem*)));
}

AttachmentPane::~AttachmentPane()
{
}

void AttachmentPane::makeChildPopup()
{
  childPopup = new QPopupMenu();

  childPopup->insertItem("Open", this, SLOT(slotOpenSelection()));
  childPopup->insertItem("Save", this, SLOT(slotSaveSelection()));
  
//  childPopup->insertSeparator();
//  childPopup->insertItem("Copy");
//  childPopup->insertItem("Send to...");
//  childPopup->insertSeparator();
//  childPopup->insertItem("Properties", this, SLOT(slotEditProperties()));
}

QString AttachmentPane::address() const
{
  return _address;
}

void AttachmentPane::setAddress(const QString &address)
{
  clear();

  _address = address;

  VMailAttribute *attr=(VMailAttribute *)vfs->_fact_ref(_address+ATTR_SEP+"attr", VFS_MAILATTRIBUTE_MIMETYPE );

  if(attr && attr->sync()) // huh
  {
    // set the widget VFS folder !
    setParentFolder( attr->parentFolder() );
    
    for(int i=0;i<attr->partCount;i++)
    {
      PartObject *part=(PartObject *)vfs->_fact_ref(_address+ATTR_SEP+QString::number(i), VFS_PART_MIMETYPE );
      if(part)
      {
        part->sync();
        if(part->partType()=="attachment")
          new AttachmentPaneWidget(part, this);
      }
      else
      {
        qDebug("Warning: [AttachmentPane] could not get part object for address %s, part %d, skipping...", (const char *)_address, i);
      }
    }
    
    // Hmm, to delete or not delete it... This is the question.
    // We should leave Aethera engine to manage it.
    // delete attr;
  }
  else
  {
    printf("Warning: [AttachmentPane] could not get message attribute for address %s, discarding...\n", (const char *)_address);
  }
}

void AttachmentPane::saveSelection()
{
  if( currentItem() )
    ( static_cast<AttachmentPaneWidget *> (currentItem()) )->save();
}

void AttachmentPane::slotSaveSelection()
{
  saveSelection();
}

void AttachmentPane::slotOpenSelection()
{
  if( currentItem() )
    ( static_cast<AttachmentPaneWidget *> (currentItem()) )->open();
}

void AttachmentPane::slotStatusChanged(QIconViewItem * item_)
{
  SortingWidgetItem *item = dynamic_cast<SortingWidgetItem*>(item_);
  if( !item )
  {
    item = dynamic_cast<SortingWidgetItem*>(currentItem());
  }
  
  if( item )
  {
    MailObject *obj =      (MailObject *)
                           (vfs->_fact_ref(_address, VFS_MAILS_MIMETYPE));
    VPartAttribute *part = (VPartAttribute *)
                           ( vfs->_fact_ref(item->path()+ATTR_SEP+"attr", VFS_PARTATTRIBUTE_MIMETYPE) );
    
    if(obj && obj->sync() && part && part->sync())
    {
      emit statusChanged( part->name, part->mimetype,
                          part->mtype, part->encoding,
                          part->charset, part->partLength, obj->size() );
    }
  }
  else
  {
    MailObject *obj = (MailObject *)
                      (vfs->_fact_ref(_address, VFS_MAILS_MIMETYPE));
    if(obj && obj->sync())
    {
      emit statusChanged( QString::null, QString::null,
                          QString::null, QString::null,
                          QString::null, 0, obj->size() );
    }
  }
}

