/*!
  ______________________________________________________________________________
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.

               COPYRIGHT   : (C) 2003 by theKompany.com
  ______________________________________________________________________________

  \date 5 July 2003
  \author Iulian Ghiciuc <ilie@eco-spaces.com.ro>
  \file Implementation of the AutoGetMail class

*/

//own includes
#include "autogetmail.h"

//tkcBase includes
#include <settings.h>
#include <qsettingsextra.h>

//defines
#define group "/Misc Mail"


AutoGetMail::AutoGetMail( QObject* parent, const char* name )
  : QObject( parent, name ),
    interval( 60000 ),
    isOn( true ),
    readAll( true ),
    timer( 0L ),
    clientRMI( 0L ),
    status( 0L )
{
  readSettings();

  timer = new QTimer( this, "AutoGetMailTimer" );
  assert( 0!=timer );
  connect( timer, SIGNAL( timeout() ),
           this, SLOT( fetchMail() )
         );
 
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  assert(0 != clientRMI);

  clientRMI->loadAccounts();

  accounts += clientRMI->getAccountNames(Account::MAILBOX);
  accounts += clientRMI->getAccountNames(Account::POP3);
  accounts += clientRMI->getAccountNames(Account::IMAP4);

  status = AppStatus::instance();
}

AutoGetMail* AutoGetMail::instance()
{
  static AutoGetMail* _instance = 0L;
  if( 0L == _instance )
    _instance = new AutoGetMail();

  return _instance;
}

AutoGetMail::~AutoGetMail()
{
  if( clientRMI )
  {
    delete clientRMI;
    clientRMI = 0L;
  }
}


void AutoGetMail::start()
{
  // 'interval * 1000 ' because we save the interval in secound
  timer->start( interval * 1000 );
}


void AutoGetMail::readSettings()
{
  isOn = Settings::instance()->readBoolEntry(Settings::instance()->name() + group + "/AutomaticCheck", isOn );
  interval = QSettingsExtra::readUnsignedNumEntry
               ( Settings::instance(),
                 Settings::instance()->name() + group + "/Interval",
                 interval
               );
  QString tmp =
    Settings::instance()->readEntry(Settings::instance()->name() + group + "/AutoCheckAccounts", QString("all") );

  if( tmp == "all" )
    readAll = true;
  else if( tmp=="default" )
    readAll = false;
  
}

void AutoGetMail::fetchMail(  )
{
  unsigned uTemp = interval;
  readSettings();
  
  if( uTemp != interval )
    timer->changeInterval( interval * 1000 );
    
  if( isOn )
  {
    QString account;

    if( readAll )
    {
      QStringList::ConstIterator it = accounts.begin();
      for( ; it!=accounts.end(); ++it )
      {
        account = *it;
        clientRMI->startReceive( account );
      }
    }
    else
    {
      account = clientRMI->getDefaultAccount();
      clientRMI->startReceive( account );
    }
  }
}

