/*
_____________________________________________________________________________

    Begin date: 20 April 2003
    Copyright: (C)2003 by theKompany.com
    Author: Vlad Bodi
    Author: Ghiciuc Ilie
    Description: composer config service
_____________________________________________________________________________
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

//own includes
#include <composerconfigservice.h>
#include <composerconfigwidget.h>

//Aethera includes
#include <qfontchooser.h>
#include <fontcolorbutton.h>
#include <qradiobutton.h>

//QT includes
#include <qsettings.h>
#include <qapplication.h>
#include <qlineedit.h>

//tkcBase includes
#include <settings.h>
#include <qsettingsextra.h>
#include <qiconloader.h>

static const char * const configGroup = "/Aethera/Composer/";
QString defSignatureFile="";
QString defEmailFormat="plain";

class ComposerConfigService::Private
{
  public:

    ComposerConfigWidget * widget;
};

/*
static const char* komsendreceiveconfigservice_metainfo[][2]=
{
    { "/name", "Kom SendReceive Config Service" },
    { "/class", "KomSendReceiveConfigService" },
    { "/class/fqn", "KomSendReceiveConfigService" },
    { "/factory", "komsendreceiveconfigservice_factory" },
    { 0, 0 }
};
*/


kore::Interface* composerconfigservice_factory()
{
  return new ComposerConfigService();
}

ComposerConfigService::ComposerConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();

  info->replaceAll("name", "Composer Config Service");
  info->replaceAll("class", "ComposerConfigService");
  info->replaceAll("class/fqn", "ComposerConfigService");
  info->replaceAll("factory", "composerconfigservice_factory");
  info->replaceAll("description", "Composer preferences");
  

  d = new Private;
  d->widget = 0;
}

ComposerConfigService::~ComposerConfigService()
{
  delete d->widget;
  d->widget = 0;
  delete d;
}

  QWidget *
ComposerConfigService::init(QWidget * parent)
{
  	d->widget =
      new ComposerConfigWidget( parent,
                                "ComposerConfigService top widget"
                              );

  return d->widget;
}

	void
ComposerConfigService::load()
{
  if (0 == d->widget)
    return;

  QString signatureFile =
    Settings::instance()->readEntry
      (
         configGroup + QString("SignaturePath"),
         defSignatureFile
      );

  d->widget->filePathLine->setText(signatureFile);

  //font settings
  QFont defaultFont("courier",12);
  QFont composerFont;
  composerFont = QSettingsExtra::readFontEntry( Settings::instance(), "/Aethera/Composer/Font", &defaultFont );
  d->widget->fontChooserButton->setSelectedFont( composerFont );

  //color settings
  QColor composerColor;
  QColor defaultColor( "black" );
  composerColor = QSettingsExtra::readColorEntry( Settings::instance(), "/Aethera/Composer/Color", &defaultColor );
  d->widget->colorChooserButton->setColor( composerColor );

  //email format
  QString emailFormat =
    Settings::instance()->readEntry
      (
         configGroup + QString("Format"),
         defEmailFormat
      );
  if( emailFormat == "HTML" )
  {
    d->widget->htmlFormatButton->setChecked( true );
  }
  else if( emailFormat == "plain" )
  {
    d->widget->plainFormatButton->setChecked( true );
  }
}

  void
ComposerConfigService::save()
{
  if (0 == d->widget)
    return;

  //signature file
  Settings::instance()->writeEntry
    (
      configGroup + QString("SignaturePath"),
      d->widget->filePathLine->text()
    );

  //font
  QSettingsExtra::writeEntry
    (Settings::instance(), "/Aethera/Composer/Font", d->widget->fontChooserButton->selectedFont());

  //color
  QSettingsExtra::writeEntry
    (Settings::instance(), "/Aethera/Composer/Color", d->widget->colorChooserButton->color());

  //email format
  if( d->widget->htmlFormatButton->isChecked() )
  {
    Settings::instance()->writeEntry
      (
        configGroup + QString("Format"),
        "HTML"
      );
  }
  else if( d->widget->plainFormatButton->isChecked() )
  {
    Settings::instance()->writeEntry
      (
        configGroup + QString("Format"),
        "plain"
      );
  }
  
}

  void
ComposerConfigService::defaults()
{
  if (0 == d->widget)
    return;

  //signature file
  d->widget->filePathLine->setText( "" );

  //default font
  QFont defaultFont("courier",12);
  d->widget->fontChooserButton->setSelectedFont( defaultFont );

  //default color
  QColor defaultColor("black");
  d->widget->colorChooserButton->setColor( defaultColor );

  //email format
  d->widget->plainFormatButton->setChecked( true );  
  
}

QPixmap ComposerConfigService::icon()
{
  return QIconLoader::instance()->loadIcon( "MailCenter_Composer.png" );
}

QString ComposerConfigService::caption()
{
  return QObject::tr("Mail/Composer");
}

bool ComposerConfigService::modified()
{
  return false;
}


