/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          composerfactory.cpp  -  description
                             -------------------
    begin                : Wed Sep 25 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#include "composerfactory.h"
#include "composerwindow.h"
#include <vfsmodule.h>

#include <qptrlist.h>

ComposerFactory * ComposerFactory::instance_ = 0L;

class ComposerFactory::Private
{
  public:
    Private();
    ~Private();

    QPtrList<QMainWindow> windows;
    ClientRMIModule * clientRMI;
    VFSModule * vfs;
};

ComposerFactory::Private::Private()
  : clientRMI(0L),
    vfs(0L)
{
  ComponentTrader * trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
    ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  assert(0L != clientRMI);
  assert(0L != vfs);

  windows.setAutoDelete(true);
}

ComposerFactory::Private::~Private()
{
  windows.clear();
}

ComposerFactory::ComposerFactory()
{
  d = new Private;
}

ComposerFactory::~ComposerFactory()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

ComposerFactory * ComposerFactory::instance()
{
  if(0 == instance_)
    instance_ = new ComposerFactory;

  return instance_;
}

void ComposerFactory::openComposer()
{
  ComposerWindow * cWin = new ComposerWindow(d->vfs, d->clientRMI, d->windows.count());
  if( cWin )
  {
    d->windows.append(cWin);
    cWin->show();
  }
}

void ComposerFactory::openComposer(const QString & vfsUrl, ComposerFactory::Mode mode)
{
  ComposerWindow * cWin = new ComposerWindow(d->vfs, d->clientRMI, d->windows.count(), vfsUrl, mode);
  if( cWin )
  {
    d->windows.append(cWin);
    cWin->show();
  }
}

void ComposerFactory::openComposer(const QStringList & vfsUrlList, ComposerFactory::Mode mode)
{
  for(QStringList::ConstIterator it=vfsUrlList.begin(); it!=vfsUrlList.end(); ++it)
  {
    openComposer((*it), mode);
  }
}

void ComposerFactory::slotCloseWindow(QMainWindow * window)
{
  d->windows.remove(window);
}

const QPtrList<QMainWindow> & ComposerFactory::windows()
{
  return d->windows;
}

void ComposerFactory::clear()
{
  d->windows.clear();
}

