/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * $Id: composerwidget.cpp,v 1.20 2004/10/28 16:25:04 eug Exp $
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "composerwidget.h"

#include <qlineedit.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qsettings.h>
#include <qtextedit.h>
#include <qmessagebox.h>

#include <mail.h>
#include <mailobj.h>
#include <vnoteobject.h>
#include <vmailattribute.h>
#include <partobj.h>
#include <vpartattribute.h>
#include <addresslistclass.h>
#include <elineedit.h>
#include <qsplitter.h>
#include <composerattpane.h>
#include <composerwindow.h>

#include <settings.h>
#include <texthtml.h>
#include <textprocessor.h>


ComposerWidget::ComposerWidget( VFSModule * vfsModule, ClientRMIModule * clientRmiModule,
                                ComposerWindow * parent, const char * name)
  : QWidget(parent, name),
    vfs( vfsModule ),
    clientRMI( clientRmiModule )
{
  layout=new QVBoxLayout(this, 0, -1, "ComposerWidget::layout");
  
  // edit fields
  toField = new AddressFieldEdit(this, "toField");
  ccField = new AddressFieldEdit(this, "ccField");
  bccField = new AddressFieldEdit(this, "bccField");
  subjectField = new EditField(this, "subjectField");

  inReplyToAddresses=new QString(0);
  referencesAddresses=new QStringList(0);
        
  toField->setLabel("To:");
  ccField->setLabel("Cc:");
  bccField->setLabel("Bcc:");
  subjectField->label()->setText("Subject:");
  subjectField->label()->setFixedSize(81, 25);
  
  // main area
  attachmentSplitter = new QSplitter(QSplitter::Vertical, this, "attachmentSplitter");
  editor_ = new QTextEdit(attachmentSplitter, "editor");
  
  attachmentPane = new ComposerAttachmentPane(vfsModule, parent, attachmentSplitter, "attachmentPane");
  
  QValueList<int> sizeList;
  sizeList << height() / 5 * 4 << height() / 5 ;
  attachmentSplitter->setSizes(sizeList);
  
  // add widgets in the layout
  layout->addWidget(toField);
  layout->addWidget(ccField);
  layout->addWidget(bccField);
  layout->addWidget(subjectField);
  layout->addWidget(attachmentSplitter);
  
  toField->show();
  ccField->show();
  
  // check if the bcc field is enabled
  if( Settings::instance()->readBoolEntry("/Aethera/Composer/Bcc", false))
    bccField->show();
  else
    bccField->hide();
  
  subjectField->show();
  editor_->show();
  attachmentPane->hide();
  attachmentSplitter->show();
  
  
//  // connect the edit fields to the signature update slot
//  connect(toField, SIGNAL(addressChanged()), this, SLOT(updateSignature()));
//  connect(ccField, SIGNAL(addressChanged()), this, SLOT(updateSignature()));
//  connect(bccField, SIGNAL(addressChanged()), this, SLOT(updateSignature()));
//  connect(subjectField, SIGNAL(textChanged(const QString &)), this, SLOT(updateSignature()));
}

ComposerWidget::~ComposerWidget()
{
}

AddressListClass *ComposerWidget::toAddresses()
{
  return toField->addressList();
}

AddressListClass *ComposerWidget::ccAddresses()
{
  return ccField->addressList();
}

AddressListClass *ComposerWidget::bccAddresses()
{
  return bccField->addressList();
}

QString *ComposerWidget::getInReplyToAddresses()
{
  return inReplyToAddresses;
}

QStringList *ComposerWidget::getReferencesAddresses()
{
  return referencesAddresses;
}

void ComposerWidget::setToAddresses(AddressListClass addr)
{
  toField->setAddressList(addr);
}

void ComposerWidget::setCcAddresses(AddressListClass addr)
{
  ccField->setAddressList(addr);
}

void ComposerWidget::setBccAddresses(AddressListClass addr)
{
  bccField->setAddressList(addr);
}

void ComposerWidget::setInReplyToAddresses(const QString& addr)
{
  inReplyToAddresses=new QString(addr);
}

void ComposerWidget::setReferencesAddresses(const QStringList& addr)
{
  referencesAddresses=new QStringList(addr);
}

bool ComposerWidget::isToFieldEmpty()
{
  return toField->isEmpty();
}

void ComposerWidget::loadMessage(MailObject * obj)
{
  if(obj && obj->sync())
  {
    QString text;
    
    if(obj->hasText())
    {
      text = obj->text();
    }
    else if(obj->hasHTML())
    {
      text = obj->html();
//  textHtmlConverter->html2text(text);
    }
   
    setText(text);

    setToAddresses(obj->to());
    setCcAddresses(obj->cc());
    setBccAddresses(obj->bcc());
    setInReplyToAddresses(obj->inReplyTo());
    setReferencesAddresses(obj->references());
     
    setSubject(obj->subject());

    setURL( obj->path() );
    
    if(obj->attachments())
      attachmentPane->show();
  }
}

void ComposerWidget::loadReply( MailObject * obj,
                                const QCString & myEmail, const QString & myReplyTo,
                                bool replyAll)
{
  if(obj && obj->sync())
  {
    QString format = Settings::instance()->readEntry( "/Aethera/Composer/Format", "plain" );
    QString data;

    if( format=="plain" )
    {
      if( obj->hasHTML() )
      {
        QString text = obj->html();

        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data = text;

        setText( data );
        
        int lines=0;
        for( int i=0; i<editor_->paragraphs(); i++)
        {
          for( int j=0; j<editor_->linesOfParagraph(i); j++, lines++ )
            editor_->insertAt( quoteSeq, lines, false );
        }
        
        editor_->setTextFormat( PlainText );
      }
      else
      {
        QString text = obj->text();
     
        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data=TextProcessor::quoteText(text, quoteSeq);

        setText( data );
      }
    }
    else if( format=="HTML" )
    {
      if( obj->hasHTML() )
      {
        QString text = obj->html();

        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data = text;

        setText( data );
        
        int lines=0;
        for( int i=0; i<editor_->paragraphs(); i++)
        {
          for( int j=0; j<editor_->linesOfParagraph(i); j++, lines++ )
            editor_->insertAt( quoteSeq, lines, false );
        }
      }
      else
      {
        QString text = obj->text();

        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data=TextProcessor::quoteText(text, quoteSeq);

        setText( data );
      }
    }

    // set subject
    QString replyPrefix =
      Settings::instance()->readEntry("/Aethera/Composer/Reply prefix");
      
    if( !replyPrefix.contains("%1") )
      replyPrefix="Re: %1";
    
    QString re = obj->subject().left(3);
    if( "Re:" == re ||
        tr("Re:") == re ||
        replyPrefix.left(3) == re )
      setSubject( obj->subject() );
    else
      setSubject( replyPrefix.arg(obj->subject()) );
    
    if(replyAll)
    {
      // get address pool
      AddressListClass origTo(obj->to());
      AddressClass origFrom(obj->from());;
      AddressListClass origCc(obj->cc());
      
      // map from + to => to, cc => cc except for our address
      QString destTo = origFrom.fullAddress() + ", ";

      for(int i = 0; i<origTo.count() ; i++)
       {
        QString toAdr= origTo[i].fullAddress() ;
        if(toAdr!=QString(myEmail) && toAdr!=myReplyTo)
        {
          destTo.append(toAdr);
          destTo.append(", ");
        }
      }
      
      // remove trailing ", "
      if( static_cast<unsigned int>(destTo.findRev(", ")) == (destTo.length()-2) )
        destTo.truncate(destTo.length()-2);
      
      QString destCc;
      {
        for (int i = 0; i<origCc.count() ; i++)
        {
          QString ccAdr= origCc[i].fullAddress() ;
          if (ccAdr!=QString(myEmail) && ccAdr!=myReplyTo)
          {
            destCc.append(ccAdr);
            destCc.append(", ");
          }
        }
      }
      
      // remove trailing ", "
      if (static_cast<unsigned int>(destCc.findRev(", ")) == (destCc.length()-2))
        destCc.truncate(destCc.length()-2);
      
      // load them in the To: and Cc: composer fields
      setToAddresses(AddressListClass(destTo, true));
      setCcAddresses(AddressListClass(destCc, true));
    }
    else
    {
      setToAddresses( AddressListClass(obj->from().fullAddress(), true) );
    }

    // load in the field In reply to: message identifiers (rfc822)
  
    setInReplyToAddresses(obj->messageID());

    // load in the field references: other correspondances (rfc822)

    if (obj->getReferences().isEmpty())
    {
      QStringList _references = QStringList(obj->inReplyTo());
      setReferencesAddresses(_references);
    }
    else
    {
      QStringList _references= QStringList(obj->getReferences());
      // QStringList::iterator it=_references.append(obj->inReplyTo());
      _references.append(obj->inReplyTo());
      // if( it != _references.end() )
      setReferencesAddresses(_references);
    }
  }
}

void ComposerWidget::loadNote( NoteObjectInterface * obj,
                               const QCString & myEmail,
                               const QString & myReplyTo )
{
  if (obj)
  {

    // set text
    QString text;
    text.append((obj->time()).toString());
    text.append("\n");

    if (!(0 == obj->category()))
    {
      text.append("Category : ");
      text.append(obj->category());
      text.append("\n");
    }
  
    text.append(obj->text()); 
    setText(text);

    //set Subject
    setSubject(obj->text());
  }
  else
  {
    qDebug("the VNoteObject does not exist");
  }
}


void ComposerWidget::setURL(const QString & vfsUrl)
{
  attachmentPane->setAddress( vfsUrl );
}

void ComposerWidget::usingRecipient(const QString & urlSource)
{
  if( !urlSource.isEmpty() )
  {
    MailObject *obj = (MailObject *)vfs->_fact_ref(urlSource, VFS_MAILS_MIMETYPE );
    if(obj)
    {
      // TODO
      toField->setAddressList( AddressListClass(obj->to().returnString()) );
    }
    else
    {
      qDebug("usingRecipient : wrong source object");
    }
  }
}

QString ComposerWidget::text()
{
  return editor_->text();
}

void ComposerWidget::setText(const QString & txt)
{
  editor_->setText(txt);
}

QString ComposerWidget::subject()
{
  return subjectField->lineEdit()->text();
}

void ComposerWidget::setSubject(const QString & txt)
{
  subjectField->lineEdit()->setText(txt);
}

void ComposerWidget::slotAttachFile()
{
  attachmentPane->newObject();
}

QTextEdit * ComposerWidget::editor()
{
  return editor_;
}

EditField * ComposerWidget::subjectEdit()
{
  return subjectField;
}

void ComposerWidget::slotUseRichText()
{
  editor_->setTextFormat( Qt::RichText );
}

ComposerAttachmentPane * ComposerWidget::attachmentsWidget()
{
  return attachmentPane;
}
