/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    copyright            : (C) 2002 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Eugen Constantinescu
                           Fabien Armand
    email                : eug@thekompany.com
                           far@atolcd.com
 ***************************************************************************/

#include "composerwindow.h"

#include <qapplication.h>
#include <qaction.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qregexp.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qtextedit.h>
#include <qdict.h>
#include <qlabel.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qtextcodec.h>
#include <qvaluelist.h>

#include <qiconloader.h>
#include <qfontchooser.h>
#include <qcolorbutton.h>
#include <fontcolorbutton.h>
#include <settings.h>
#include <qsettingsextra.h>
#include <texthtml.h>
#include <textprocessor.h>
#include <mimecodec.h>
#include <charsets.h>

#include <addresslineedit.h>
#include <addresslistclass.h>
#include <addressfieldedit.h>
#include <composerattpane.h>
#include <composereditor.h>
#include <composerwidget.h>
#include <mailobj.h>
#include <noteobject.h>
#include <vmailattribute.h>
#include <partobj.h>
#include <vpartattribute.h>
#include <popupaction.h>
#include <composerstatus.h>

#include <qmessagebox.h>

class ComposerWindow::Private
{
  public:
  
    Private( ComposerWindow * parent,
             VFSModule * vfsModule, 
             ClientRMIModule * clientRmiModule,
             unsigned id );
    virtual ~Private();
  
    // load message from server using d->url
    void loadMessage();

    void loadForward(const QString &);
    void loadForward(const QStringList &);
    void loadForwardInline(const QString &);
    void loadForwardQuoted(const QString &);
    void loadForwardAsAttachment(const QString &);
    void loadReply(const QString &, bool replyAll=false);
    void loadNote(const QString &);
    
    void loadObjectAsAttachment(const QString &);

    void initBasicMessage(const QString &, const QString &);
    void attachObject(const QString &, const QString &, const QString &, const QString &);
    void copyAttachmentsFrom(const QString &);

    // Save the message

    void save();
    
    // move message into Outbox (if needed)
    void saveToOutbox();

    // send message
    void sendMessage();

    bool canSend();
    
    void buildAccountList();

    // UI
    ComposerWidget *composerWidget;
    ComposerStatus *statusBar;
    
    QPopupMenu * sendAccounts;
    QPopupMenu * sendLaterAccounts;
    QFontChooser * fontChooser;
    FontColorButton * colorChooser;
    
    QDict<QAction> actions;

    // Accounts
    QStringList accounts;
    QString sourceAccount;
    QString account;

    //hold the id's returned by insertItem from QPopupMenu from matching when action activated
    QValueList<int> accountMenuId;

    bool dirty;
    bool forcedClose;

    Mail::Priority priority;
    QStringList extendedStatus;

    QTimer * autoSaveTimer;

    /** The message VFS URL.*/
    QString url;
    
    /** VFS.*/
    VFSModule * vfs;
    /** RMI.*/
    ClientRMIModule * clientRMI;
    /** The parent object.*/
    ComposerWindow * parent_;
    /** The window ID.*/
    unsigned windowID_;
};

ComposerWindow::Private::Private( ComposerWindow * parent,
                                  VFSModule * vfsModule, 
                                  ClientRMIModule * clientRmiModule,
                                  unsigned id )
  : composerWidget(0L),
    statusBar(0L),
    sendAccounts(0L),
    sendLaterAccounts(0L),
    fontChooser(0L),
    colorChooser(0L),
    dirty(false),
    forcedClose(false),
    priority( Mail::Normal ),
    vfs( vfsModule ),
    clientRMI( clientRmiModule ),
    parent_( parent ),
    windowID_( id )
{
  statusBar = new ComposerStatus(parent, "ComposerStatusBar");
  
  composerWidget = new ComposerWidget(vfs, clientRmiModule, parent_);
  
  if( composerWidget && composerWidget->attachmentsWidget() )
  {
    connect( composerWidget->attachmentsWidget(),
             SIGNAL( statusChanged(const QString&, const QString&, const QString&, const QString&, const QString&, int, int) ),
             statusBar,
             SLOT  ( slotChangeInfo(const QString&, const QString&, const QString&, const QString&, const QString&, int, int) ) );
  }
  else
  {
    qDebug("ComposerWindow: composer widget is null");
  }

  statusBar->hide();
  // start the autosave timer
  int saveTimeout = Settings::instance()->readNumEntry("/Aethera/Composer/Save timeout");
  if( !saveTimeout )
    saveTimeout = 5;

  autoSaveTimer = new QTimer(parent_);
  QObject::connect(autoSaveTimer, SIGNAL(timeout()), parent_, SLOT(slotAutoSaveTimer()));
  autoSaveTimer->start(saveTimeout * 60 * 1000);
}

ComposerWindow::Private::~Private()
{
  if( sendAccounts )
  {
    sendAccounts->clear();
//    delete sendAccounts;
//    sendAccounts = 0L;
  }
  
  if( sendLaterAccounts )
  {
    sendLaterAccounts->clear();
//    delete sendLaterAccounts;
//    sendLaterAccounts = 0L;
  }
}

bool ComposerWindow::Private::canSend()
{
  if( composerWidget->isToFieldEmpty() )
  {
    QMessageBox::warning(parent_, QString(AppName) + tr(": No address"),
      tr("Please specify a recipient in the To: field"), QMessageBox::Ok, 0, 0);
    return false;
  }
  
  return true;
}

void ComposerWindow::Private::save()
{
  if( 1/*dirty*/ )
  {
    // create object in Drafts if no address was set; set address
    if(url.isEmpty() || !vfs->exists(url))
    {
      url=vfs->createObject(vfs->root()+"/Mail/Drafts");
      composerWidget->setURL(url);
    }
  
    MailObject *obj=(MailObject *)(vfs->_fact_ref(url, VFS_MAILS_MIMETYPE));
  
    if(obj)
    {
      // set text part, subject, recipients, priority, in reply to, references and scheduling information
      obj->setSubject(composerWidget->subject());
      obj->setTo(*composerWidget->toAddresses());
      obj->setCc(*composerWidget->ccAddresses());
      obj->setBcc(*composerWidget->bccAddresses());
      obj->setInReplyTo(*composerWidget->getInReplyToAddresses());
      obj->setReferences(*composerWidget->getReferencesAddresses());
      obj->setStatus("Not sent");
      obj->setPriority( Mail::toString(priority) );
//      obj->setScheduling(scheduling);
      obj->save();
  
      int textPart = obj->messageText();
      QString partID;
  
      if( textPart == -1 )
      {
        // no text part exists, create one
        partID = vfs->createObject(url);
      }
      else
      {
        partID = url + ATTR_SEP + QString::number(textPart);
      }
  
      VPartAttribute *attr=(VPartAttribute *)vfs->_fact_ref(partID+ATTR_SEP+"attr", VFS_PARTATTRIBUTE_MIMETYPE );
      if(attr)
      {
        attr->sync();
        attr->mtype = "text";
     
        // Set the sending type
        QString sendingType = Settings::instance()->readEntry("/Aethera/Composer/SendingType", "AUTO");
        if( "HTML" == sendingType )
        {
          // todo
        }
        else if( "ALTERNATIVE" == sendingType )
        {
          // todo
        }
        else if( "AUTO" == sendingType )
        {
          // todo
        }
        else // only TEXT
        {
          // todo
        }
        
        if( Qt::PlainText != composerWidget->editor()->textFormat() )
        {
          attr->mimetype = "text/html";
        }
        else
        {
          attr->mimetype = "text/plain";
        }

        QCString _text;
        QString composerText = composerWidget->text();
        
        if( composerText.isEmpty() )
        {
          attr->charset = "us-ascii";
          attr->encoding = "7bit";
          _text = " ";
        }
        else
        {
          QTextCodec * codec = 0L;
          if( composerText != QString::fromUtf8( composerText.utf8() ) && 
              (codec = CharSets::thisInstance()->getCodec(composerText)) )
          {
            attr->charset = codec->mimeName();
            attr->encoding = "base64";
            _text = codec->fromUnicode( composerText );
            _text = MIMECodec::encodeBase64(_text);
          }
          else
          {
            // hmm
            if( "System" != CharSets::thisInstance()->getDefaultCharset() &&
                composerText == QString::fromLocal8Bit( composerText.local8Bit() ) )
            {
              attr->charset = CharSets::thisInstance()->getDefaultCharset();
              attr->encoding = "8bit";
              _text = composerText.local8Bit();
            }
            else
            {
              attr->charset = "utf-8";
              attr->encoding = "7bit";
              _text = composerText.utf8();
            }
          }
        }
          
        attr->save();
        
        vfs->setObjectData(partID, _text);
  
        int saveTimeout=Settings::instance()->
          readNumEntry("/Aethera/Composer/Save timeout");
        if( !saveTimeout )
          saveTimeout = 3;
        
        autoSaveTimer->changeInterval(saveTimeout * 60 * 1000);
      }
      else
      {
        QMessageBox::warning(
          parent_,
          QString("%1: Could not retrieve text part").arg(AppName),
          "Could not retrieve a text part in this message.\n"
          "The composer cannot edit this message.",
          QMessageBox::Ok, 0, 0);
        forcedClose=true;
        // huh
        parent_->close();
      }
    }
    else
    {
      QMessageBox::warning(
        parent_,
        QString("%1: Could not save message").arg(AppName),
        "Could not save message. The server reply was:\n" + vfs->regetLastError(),
        QMessageBox::Ok, 0, 0);
    }
  
    dirty=false;
  }
}

void ComposerWindow::Private::saveToOutbox()
{
  // save message
  save();
  
  // move message to Outbox, if needed
  if( url.find( vfs->root()+"/Mail/Outbox") )
    url = vfs->move(url, vfs->root()+"/Mail/Outbox");
}

void ComposerWindow::Private::sendMessage()
{
  // save to Outbox
  saveToOutbox();
  
  // ask the broker to send this message
  if( !vfs->send(url) )
  {
    QMessageBox::warning( parent_, 
                          QString("%1: Could not send message").arg(AppName),
                          "Could not send message to the outgoing spooler.\nReason given by server:\n"+
                          vfs->regetLastError(), QMessageBox::Ok, 0, 0);
  }
}

void ComposerWindow::Private::buildAccountList()
{
  // set up the default account
  sourceAccount = clientRMI->getDefaultAccount();
  
  // get the account list from server using ClientRMI
  QStringList allAccountList = clientRMI->getAccountNames();
  
  accounts.clear();
  for(QStringList::ConstIterator it=allAccountList.begin(); it!=allAccountList.end(); ++it)
    if( clientRMI->isAccountActive(*it) )
      accounts.push_back(*it);
}

void ComposerWindow::Private::loadMessage()
{
  // load text, subject, recipients, priority, extended status and scheduling from server
  MailObject *obj=(MailObject *)vfs->_fact_ref(url, VFS_MAILS_MIMETYPE );

  if(obj)
  {
    composerWidget->loadMessage(obj);
    
    dirty = true;

    priority = Mail::toPriority(obj->priority());
    
//  scheduling=obj->scheduling();

    extendedStatus = obj->extendedStatus();

    // @todo: load the extended status in the status view   
  }
  else
  {
    QMessageBox::warning(parent_, QString("%1: Message does not exist").arg(AppName), "The composer attempted to load a non-existent message.\n"
      "Probably the message you have referenced was dispatched and deleted by the outgoing mail spooler.\n", QMessageBox::Ok, 0, 0);
      
    url="";
  }
}

void ComposerWindow::Private::loadForward(const QString &sourcePath)
{
  QString fwType=
    Settings::instance()->readEntry("/Aethera/Read Options/Forward type");

  if(fwType == "Inline")
    loadForwardInline(sourcePath);
  else if(fwType == "As attachment")
    loadForwardAsAttachment(sourcePath);
  else
    loadForwardQuoted(sourcePath);
}

void ComposerWindow::Private::initBasicMessage(const QString &subject, const QString &text)
{
  // create new message object
  url = vfs->createObject(vfs->root()+"/Mail/Drafts");
  
  if( !url.isEmpty() )
  {

    MailObject *obj=(MailObject *)vfs->_fact_ref(url, VFS_MAILS_MIMETYPE );
    
    if(obj)
    {
      // set subject
      obj->setSubject(subject);
      
      // set text
      QString partID;
      
      partID=vfs->createObject(url);
      
      VPartAttribute *attr=(VPartAttribute *)vfs->_fact_ref(partID+ATTR_SEP+"attr", VFS_PARTATTRIBUTE_MIMETYPE );

      if(!attr)
      {
        QMessageBox::warning(parent_, QString("%1: Could not retrieve text part").arg(AppName), "Could not retrieve a text part in this message.\n"
          "This is an unexpected error and the composer cannot allow to further edit this message.", QMessageBox::Ok, 0, 0);

          
        forcedClose=true;
        // huh
        parent_->close();
      }
      else
      {
        attr->sync();
        attr->mtype = "text";
        
        // Set the sending type
        QString sendingType = Settings::instance()->readEntry("/Aethera/Composer/SendingType", "AUTO");
        if( "HTML" == sendingType )
        {
          // todo
        }
        else if( "ALTERNATIVE" == sendingType )
        {
          // todo
        }
        else if( "AUTO" == sendingType )
        {
          // todo
        }
        else // only TEXT
        {
          // todo
        }
        
        if( Qt::PlainText != composerWidget->editor()->textFormat() )
        {
          attr->mimetype = "text/html";
        }
        else
        {
          attr->mimetype = "text/plain";
//          composerWidget->editor()->setTextFormat(Qt::PlainText);
        }
        
        attr->encoding = "8bit";
        attr->charset = "utf-8";
        attr->save();
        
        delete attr;
        
        QCString _text;
        
        if( text.isEmpty() )
        {
          _text = "";
        }
        else
        {
          _text = text.utf8();
        }
        
        vfs->setObjectData(partID, _text);
      }
    }
  }
  else
  {
    QMessageBox::warning(parent_, QString("%1: Could not create text part").arg(AppName), "Could not create a text part in this message.\n"
      "This is an unexpected error and the composer cannot allow to further edit this message.", QMessageBox::Ok, 0, 0);
    forcedClose = true;
    // huh
    parent_->close();
  }
}

void ComposerWindow::Private::loadObjectAsAttachment(const QString & urlSource)
{
  if( !urlSource.isEmpty() )
  {
    MailObject *obj = (MailObject *)vfs->_fact_ref(urlSource, VFS_MAILS_MIMETYPE );
    if(obj)
    {
      // TODO
    }
    else
    {
      qDebug("loadObjectAsAttachment : wrong source object");
    }
  }
}

void ComposerWindow::Private::attachObject
  ( const QString &path, const QString &name,
    const QString &mimeType, const QString &encoding )
{
  QString partID = vfs->createObject(url);

  qDebug("ici est le problme");
  PartObject *part=(PartObject *)vfs->_fact_ref(partID, VFS_PART_MIMETYPE );
  
  if(part)
  {
    part->setFilename(name);
    part->setPartType("attachment");
    part->setMimeType(mimeType);
    part->setEncoding(encoding);
    
    part->save();
     
    QByteArray messageData;
    vfs->getObjectData(path, messageData);
    vfs->setObjectData(partID, messageData);
     
//    delete part;
  }
  else
  {
    QMessageBox::warning(parent_, QString("%1: Could not create attachment").arg(AppName), "Could not create an attachment in this message.\n"
      "This is an unexpected error and the composer cannot allow to further edit this message.", QMessageBox::Ok, 0, 0);
    forcedClose=true;
    // huh
    parent_->close();
  }
}


void ComposerWindow::Private::copyAttachmentsFrom(const QString &sourcePath)
{
  MailObject *srcObj=(MailObject *)vfs->_fact_ref(sourcePath, VFS_MAILS_MIMETYPE );
  
  if(srcObj)
  {
    srcObj->sync();
    
    if(srcObj->attachments())
    {
      VMailAttribute *attr=(VMailAttribute *)vfs->_fact_ref(sourcePath+ATTR_SEP+"fattr", VFS_MAILATTRIBUTE_MIMETYPE );
      
      if(attr)
      {
        attr->sync();
        
        for(int i=0;i<attr->partCount;i++)
        {

          QString partID=sourcePath+ATTR_SEP+QString::number(i);
          
          PartObject *part=(PartObject *)vfs->_fact_ref(partID, VFS_PART_MIMETYPE );
          
          if(part)
          {

            part->sync();
            
            if( part->partType() == "attachment" )
            {
              attachObject(partID, part->filename(), part->mimeType(), part->encoding());
            } 
            
// DON'T delete the part here since it will be deleted 
// by the attachment panel from the original message
// Ask Eug
//        delete part;
          }
        }
        
        delete attr;
      }
    }
  }
}

void ComposerWindow::Private::loadForward(const QStringList &sourcePathList)
{
  initBasicMessage("Fwd: ", "");
        
  // add attachments
  for(unsigned int i=0;i<sourcePathList.count();i++)
  {
    MailObject *srcObj=(MailObject *)vfs->_fact_ref(sourcePathList[i], VFS_MAILS_MIMETYPE );
    
    if(srcObj)

    {


      srcObj->sync();
      attachObject(sourcePathList[i]+"%rfc822message", srcObj->subject()+".msg", "message/rfc822", "8bit");
    }
  }

  // load message
  loadMessage();
}

void ComposerWindow::Private::loadForwardInline(const QString &sourcePath)
{
  MailObject *srcObj=(MailObject *)vfs->_fact_ref(sourcePath, VFS_MAILS_MIMETYPE );

  if(srcObj)
  {
    srcObj->sync();

    QString format = Settings::instance()->readEntry( "/Aethera/Composer/Format", "plain" );

    QString data = QString::null;

    if( format=="plain" )
    {
      if( srcObj->hasHTML() )
      {
        QString text = srcObj->html();

        data = text;

        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);
        copyAttachmentsFrom(sourcePath);
        loadMessage();
      }
      else
      {
        QString text = srcObj->text();

        data = text;

        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);
        copyAttachmentsFrom(sourcePath);
        loadMessage();
      }
    }
    else if( format=="HTML" )
    {
      if( srcObj->hasHTML() )
      {
        QString text = srcObj->html();

        data = text;
        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);

        copyAttachmentsFrom(sourcePath);
        loadMessage();

      }
      else
      {
        QString text = srcObj->text();

        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data = text;

        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);
        copyAttachmentsFrom(sourcePath);
        loadMessage();
      }
    }
  }
}

void ComposerWindow::Private::loadForwardQuoted(const QString &sourcePath)
{
  MailObject *srcObj=(MailObject *)vfs->_fact_ref(sourcePath, VFS_MAILS_MIMETYPE );

  if(srcObj)
  {
    srcObj->sync();

    QString format = Settings::instance()->readEntry( "/Aethera/Composer/Format", "plain" );

    QString data = QString::null;

    if( format=="plain" )
    {
      if( srcObj->hasHTML() )
      {
        QString text = srcObj->html();
        
        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data=TextProcessor::quoteText(text, quoteSeq);

        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);
        copyAttachmentsFrom(sourcePath);
        loadMessage();
      }
      else
      {
        QString text = srcObj->text();

        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data=TextProcessor::quoteText(text, quoteSeq);

        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);
        copyAttachmentsFrom(sourcePath);
        loadMessage();
      }
    }
    else if( format=="HTML" )
    {
      if( srcObj->hasHTML() )
      {
        QString text = srcObj->html();

        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data = text;
        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);

        copyAttachmentsFrom(sourcePath);
        loadMessage();

        int lines=0;
        for( int i=0; i<composerWidget->editor()->paragraphs(); i++)
        {
          for( int j=0; j<composerWidget->editor()->linesOfParagraph(i); j++, lines++ )
            composerWidget->editor()->insertAt( quoteSeq, lines, false );
        }
      }
      else
      {
        QString text = srcObj->text();

        QString quoteSeq=Settings::instance()->readEntry("/Aethera/Composer/Quote char");
          if(quoteSeq.isEmpty()) quoteSeq="> ";

        data=TextProcessor::quoteText(text, quoteSeq);

        initBasicMessage(QString("Fwd: ")+srcObj->subject(), data);
        copyAttachmentsFrom(sourcePath);
        loadMessage();
      }
    }
  }
}

void ComposerWindow::Private::loadForwardAsAttachment(const QString &sourcePath)
{
  MailObject *srcObj=(MailObject *)vfs->_fact_ref(sourcePath, VFS_MAILS_MIMETYPE );
  
  if(srcObj)
  {
    srcObj->sync();
    
    initBasicMessage(QString("Fwd: ")+srcObj->subject(), "");

    // add attachment
    attachObject(sourcePath+"%rfc822message", srcObj->subject()+".msg", "message/rfc822", "8bit");
  }


  // load message
  loadMessage();
}

void ComposerWindow::Private::loadReply(const QString &sourcePath, bool replyAll)
{
  QCString myEmail = (const char *)clientRMI->getAccountFromAddress( sourceAccount );
  QString myReplyTo = clientRMI->getAccountReplyToAddress( sourceAccount );
  
  // load text, subject, recipients, priority, extended status and scheduling from server
  MailObject *obj = static_cast<MailObject *>
    (vfs->_fact_ref(sourcePath, VFS_MAILS_MIMETYPE ));
  if(obj)
  {
    composerWidget->loadReply(obj, myEmail, myReplyTo, replyAll);
    
    dirty=true;
    
    // @todo: load the extended status in the status view
    // @todo: trigger a status update on the source object
  }
  else
  {
    QMessageBox::warning(parent_, QString("%1: Message does not exist").arg(AppName), "The composer attempted to reply to a non-existent message.\n"
      "Probably the message you have referenced was deleted meanwhile.\n", QMessageBox::Ok, 0, 0);

  }
}

void ComposerWindow::Private::loadNote(const QString &sourcePath)
{

  NoteObjectInterface * obj = static_cast<NoteObjectInterface*>
    (vfs->_fact_ref(sourcePath, VFS_NOTES_MIMETYPE));

  if(obj)
  {
    QCString myEmail = (const char *)clientRMI->getAccountFromAddress( sourceAccount );
    QString myReplyTo = clientRMI->getAccountReplyToAddress( sourceAccount);
    composerWidget->loadNote(obj, myEmail, myReplyTo);
  }
  else
  {
    qDebug("the VNoteObject does not exist");
  }  
}

ComposerWindow::ComposerWindow( VFSModule * vfsModule, 
                                ClientRMIModule * clientRmiModule,
                                unsigned id )
{
  d = new Private(this, vfsModule, clientRmiModule, id);

  // main widget
  setCentralWidget( d->composerWidget );
  d->composerWidget->show();

  // get size
  QRect defaultGeometry(50, 50, 600, 400);
  setGeometry( QSettingsExtra::readRectEntry( Settings::instance(),
                 "/Aethera/Composer/Geometry",
                 &defaultGeometry ) );
    
  connect(d->composerWidget->editor(), SIGNAL(currentFontChanged(const QFont&)),
          d->composerWidget, SLOT(slotUseRichText()));
          
  connect(d->composerWidget->editor(), SIGNAL(currentColorChanged(const QColor&)),
          d->composerWidget, SLOT(slotUseRichText()));
  
  setIcon( BarIcon("lo22-aethera") );
  
  // initial caption
  slotShowCaption("no subject");

  // build actions
  buildActions();
  
  // build the menu and toolbar
  buildMenuAndToolbar();
  
  // TODO: build the statusbar
  // What to show in statusbar ?
  
  // connect stuff
  buildConnections();

  QString format = Settings::instance()->readEntry( "/Aethera/Composer/Format", "plain" );
  if( format=="plain" )
  {
    slotChangeFormat( false );
  }
  else if( format=="HTML" )
  {
    slotChangeFormat( true );
    d->composerWidget->slotUseRichText();
    
    QFont defaultFont;
    // huh :(
    d->composerWidget->editor()->setFont(
        QSettingsExtra::readFontEntry( Settings::instance(), "/Aethera/Composer/Font", &defaultFont) );

    QColor defaultColor;
    // huh :(
    d->composerWidget->editor()->setColor(
        QSettingsExtra::readColorEntry( Settings::instance(), "/Aethera/Composer/Color", &defaultColor) );
  }
}

ComposerWindow::ComposerWindow( VFSModule * vfsModule, ClientRMIModule * clientRmiModule, unsigned id,
                                const QString & vfsUrl, ComposerFactory::Mode mode)
{
  d = new Private(this, vfsModule, clientRmiModule, id);
  
  // main widget
  setCentralWidget( d->composerWidget );
  d->composerWidget->show();

  // get size
  QRect defaultGeometry(50, 50, 600, 400);
  setGeometry( QSettingsExtra::readRectEntry( Settings::instance(),
                 "/Aethera/Composer/Geometry",
                 &defaultGeometry ) );

  connect(d->composerWidget->editor(), SIGNAL(currentFontChanged(const QFont&)),
          d->composerWidget, SLOT(slotUseRichText()));
          
  connect(d->composerWidget->editor(), SIGNAL(currentColorChanged(const QColor&)),
          d->composerWidget, SLOT(slotUseRichText()));
  
  setIcon( BarIcon("lo22-aethera") );
  
  // initial caption
  slotShowCaption("no subject");

  // build actions

  buildActions();
  
  // build the menu and toolbar
  buildMenuAndToolbar();
  
  // TODO: build the statusbar
  
  // connect stuff
  buildConnections();

  switch(mode)
  {
    case ComposerFactory::EditMessage:
      d->url = vfsUrl;
      d->loadMessage();
      break;
    case ComposerFactory::Forward:
      d->loadForward(vfsUrl);
      break;
    case ComposerFactory::ForwardInline:
      d->loadForwardInline(vfsUrl);
      break;

    case ComposerFactory::ForwardQuoted:
      d->loadForwardQuoted(vfsUrl);
      break;
    case ComposerFactory::ForwardAsAttachment:
      d->loadForwardAsAttachment(vfsUrl);
      break;
    case ComposerFactory::Reply:
      d->loadReply(vfsUrl);
      break;
    case ComposerFactory::ReplyAll:
      d->loadReply(vfsUrl, true);
      break;
    case ComposerFactory::SendObject:
      d->loadObjectAsAttachment(vfsUrl);
      break;
    case ComposerFactory::UsingRecipient:
      d->composerWidget->usingRecipient(vfsUrl);
      break;
    case ComposerFactory::WriteToContact:
      d->composerWidget->setToAddresses(AddressListClass(vfsUrl));
      break;      
    case ComposerFactory::SendNote:
      d->loadNote(vfsUrl);
      break;
    default:
      break;
  }

  QString format = Settings::instance()->readEntry( "/Aethera/Composer/Format", "plain" );
  if( format=="plain" )
  {
    slotChangeFormat( false );
  }
  else if( format=="HTML" )
  {
    slotChangeFormat( true );
    d->composerWidget->slotUseRichText();
    
    QFont defaultFont;
    // huh :(
    d->composerWidget->editor()->setFont(
        QSettingsExtra::readFontEntry( Settings::instance(), "/Aethera/Composer/Font", &defaultFont) );

    QColor defaultColor;
    // huh :(
    d->composerWidget->editor()->setColor(
        QSettingsExtra::readColorEntry( Settings::instance(), "/Aethera/Composer/Color", &defaultColor) );
  }
}

ComposerWindow::~ComposerWindow()
{
  if(d)
  {
    delete d;
    d = 0L;
  }
  
  QSettingsExtra::writeEntry
    (Settings::instance(), "/Aethera/Composer/Geometry", geometry());
}


bool ComposerWindow::buildActions()
{
  // Create all the QAction-s and insert it in d->actions
  QAction * action = 0L;

  action = new QAction( tr("Create a new Composer window."),
                        QIconLoader::instance()->iconSet( COMPOSER_NEW_PICS ),
                        tr("&New"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_NEW, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Create a new Composer window with the same content as the current one."),
                        QIconLoader::instance()->iconSet( COMPOSER_DUPLICATE_PICS ),
                        tr("&Duplicate"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_DUPLICATE, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Send the mail right now using the default account."),
                        QIconLoader::instance()->iconSet( "getmail-normal" ),
                        tr("Send &now"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");


  d->actions.insert(COMPOSER_SEND, action);
  
  action = new QAction( tr("Close the window."),
                        tr("&Close"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_CLOSE, action);
  
  action = new QAction( tr("Save the message."),
                        QIconLoader::instance()->iconSet( COMPOSER_SAVE_PICS ),
                        tr("&Save"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_SAVE, action);
  
  action = new QAction( tr("Save the message in the Draft folder."),
                        QIconLoader::instance()->iconSet( COMPOSER_SAVEAS_DRAFT_PICS ),
                        tr("Save As &Draft"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_SAVEAS_DRAFT, action);
  
  action = new QAction( tr("Save the message as a file."),
                        QIconLoader::instance()->iconSet( COMPOSER_SAVEAS_FILE_PICS ),
                        tr("Save As &File"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_SAVEAS_FILE, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Print the message."),
                        QIconLoader::instance()->iconSet( COMPOSER_PRINT_PICS ),
                        tr("&Print"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_PRINT, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Cut the selection and copy it in clipboard."),
                        QIconLoader::instance()->iconSet( COMPOSER_CUT_PICS ),
                        tr("C&ut"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_CUT, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Copy the selection in clipboard."),
                        QIconLoader::instance()->iconSet( COMPOSER_COPY_PICS ),
                        tr("&Copy"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_COPY, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Paste the clipboard data."),
                        QIconLoader::instance()->iconSet( COMPOSER_PASTE_PICS ),
                        tr("&Paste"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_PASTE, action);
  
  action = new QAction( tr("Delete the selection."),
                        QIconLoader::instance()->iconSet( COMPOSER_DELETE_PICS ),
                        tr("&Delete"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_DELETE, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Undo the changes."),
                        QIconLoader::instance()->iconSet( COMPOSER_UNDO_PICS ),
                        tr("&Undo"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_UNDO, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Redo the changes."),
                        QIconLoader::instance()->iconSet( COMPOSER_REDO_PICS ),
                        tr("&Redo"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_REDO, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Select all the text."),
                        tr("&Select All"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_SELECTALL, action);
  
  action = new QAction( tr("Attach a file to the message."),
                        QIconLoader::instance()->iconSet( COMPOSER_ATTACH_FILE_PICS ),
                        tr("Attach &File"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_ATTACH_FILE, action);
  
  action = new QAction( tr("Attach your signature."),
                        QIconLoader::instance()->iconSet( COMPOSER_ATTACH_SIGNATURE_PICS ),
                        tr("Attach &Signature"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_ATTACH_SIGNATURE, action);
  
  action = new QAction( tr("Set the message priority to low level."),
                        QIconLoader::instance()->iconSet( Mail::instance()->toPixmap(Mail::Low) ),
                        tr("&Low"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_PRIORITY_LOW, action);
  
  action = new QAction( tr("Set the message priority to lowest level."),
                        QIconLoader::instance()->iconSet( Mail::instance()->toPixmap(Mail::Lowest) ),
                        tr("L&owest"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_PRIORITY_LOWEST, action);
  
  action = new QAction( tr("Set the message priority to normal level."),
                        QIconLoader::instance()->iconSet( Mail::instance()->toPixmap(Mail::Normal) ),
                        tr("&Normal"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_PRIORITY_NORMAL, action);
  
  action = new QAction( tr("Set the message priority to high level."),
                        QIconLoader::instance()->iconSet( Mail::instance()->toPixmap(Mail::High) ),
                        tr("&High"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_PRIORITY_HIGH, action);
  
  action = new QAction( tr("Set the message priority to highest level."),

                        QIconLoader::instance()->iconSet( Mail::instance()->toPixmap(Mail::Highest) ),
                        tr("H&ighest"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_PRIORITY_HIGHEST, action);
  
  action = new QAction( tr("Help index."),
                        QIconLoader::instance()->iconSet( COMPOSER_HELP_INDEX_PICS ),
                        tr("&Help Index"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_HELP_INDEX, action);
  action->setEnabled(false);
  
  action = new QAction( tr("Help About"),
                        tr("Help &About"),
                        QKeySequence(),
                        this );
  action->setStatusTip("");
  d->actions.insert(COMPOSER_HELP_ABOUT, action);
  action->setEnabled(false);

  action = new QAction( tr("HTML"),
                        tr("Format HTML"),
                        QKeySequence(),
                        this,
                        COMPOSER_HTML_FORMAT,
                        true );
  action->setStatusTip("");
  action->setOn( true );
  d->actions.insert(COMPOSER_HTML_FORMAT, action);

  
  return true;
}

bool ComposerWindow::buildMenuAndToolbar()
{
  QAction * action = 0L;
  QMenuBar * menuBar = this->menuBar();
  QToolBar * toolBar = new QToolBar(this, "ComposerToolBar");
  
  if( !menuBar || !toolBar )
    return false;
  
  d->sendAccounts = new QPopupMenu();
  d->sendLaterAccounts = new QPopupMenu();

  // add the Send button in the toolbar as the first button
  action = d->actions.find( COMPOSER_SEND );
  if (0 != action)
    action->addTo( toolBar );
  
  toolBar->addSeparator();  
  
  // build Mail menu
  QPopupMenu * mailMenu = new QPopupMenu(menuBar, "MailMenu");
  
  menuBar->insertItem(tr("&Mail"), mailMenu);
  
  action = d->actions.find( COMPOSER_NEW );
  if (0 != action)
  {
    action->addTo( mailMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_DUPLICATE );
  if (0 != action)
  {
    action->addTo( mailMenu );
//    action->addTo( toolBar );
  }
  
  mailMenu->insertSeparator();
    
  action = d->actions.find( COMPOSER_SEND );
  if (0 != action)
    action->addTo( mailMenu );
  
  mailMenu->insertItem(tr("Send now using"), d->sendAccounts);
  mailMenu->insertItem(tr("Send &later"), this, SLOT(slotSendLaterDefault()));
  int itemId = mailMenu->insertItem(tr("Send later using"), d->sendLaterAccounts);
  mailMenu->setItemEnabled( itemId, false );
  
  action = d->actions.find( COMPOSER_SAVE );
  if (0 != action)
  {
    action->addTo( mailMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_SAVEAS_DRAFT );
  if (0 != action)
    action->addTo( mailMenu );
  
  action = d->actions.find( COMPOSER_SAVEAS_FILE );
  if (0 != action)
    action->addTo( mailMenu );
  
  mailMenu->insertSeparator();
    
  action = d->actions.find( COMPOSER_PRINT );
  if (0 != action)
  {
    action->addTo( mailMenu );
    action->addTo( toolBar );
  }
  
  mailMenu->insertSeparator();
    
  action = d->actions.find( COMPOSER_CLOSE );
  if (0 != action)
    action->addTo( mailMenu );
  
  toolBar->addSeparator();  
  
  // build Edit menu
  QPopupMenu * editMenu = new QPopupMenu(menuBar, "EditMenu");
  menuBar->insertItem(tr("&Edit"), editMenu);
  
  action = d->actions.find( COMPOSER_UNDO );
  if (0 != action)
  {
    action->addTo( editMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_REDO );
  if (0 != action)
  {
    action->addTo( editMenu );
    action->addTo( toolBar );
  }
  
  editMenu->insertSeparator();
  toolBar->addSeparator();
  
  action = d->actions.find( COMPOSER_CUT );
  if (0 != action)
  {
    action->addTo( editMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_COPY );
  if (0 != action)
  {
    action->addTo( editMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_PASTE );
  if (0 != action)
  {
    action->addTo( editMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_DELETE );
  if (0 != action)
  {
    action->addTo( editMenu );
    action->addTo( toolBar );
  }
  
  editMenu->insertSeparator();
    
  action = d->actions.find( COMPOSER_SELECTALL );
  if (0 != action)
    action->addTo( editMenu );
  
  toolBar->addSeparator();
  
  // build Attach menu
  QPopupMenu * attachMenu = new QPopupMenu(menuBar, "AttachMenu");

  menuBar->insertItem(tr("&Attach"), attachMenu);
  
  action = d->actions.find( COMPOSER_ATTACH_FILE );
  if (0 != action)
  {
    action->addTo( attachMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_ATTACH_SIGNATURE );
  if (0 != action)
  {
    action->addTo( attachMenu );
    action->addTo( toolBar );
  }
  
  toolBar->addSeparator();
  
  // build Options menu
  QPopupMenu * optionsMenu = new QPopupMenu(menuBar, "OptionsMenu");
  QPopupMenu * priorityPopup = new QPopupMenu(menuBar, "PriorityMenu");
  menuBar->insertItem(tr("&Options"), optionsMenu);
  optionsMenu->insertItem(tr("&Priority"), priorityPopup);

  action = d->actions.find( COMPOSER_HTML_FORMAT );
  if (0 != action)
  {
    QString form_htm = Settings::instance()->readEntry("/Aethera/Composer/Format","plain");
    if(form_htm=="HTML")
    {
      action->setOn(true);
      action->addTo( optionsMenu );
    }
    else
    {
      action->setOn(false);
      action->addTo( optionsMenu );
    }

  }
  
  PopupAction * priorityAction = new PopupAction(this);
  priorityAction->setPopup( priorityPopup );
  priorityAction->setIconSet( QIconLoader::instance()->iconSet( Mail::instance()->toPixmap(Mail::Normal) ) );
  priorityAction->addTo(toolBar);
  
  action = d->actions.find( COMPOSER_PRIORITY_LOWEST );
  if (0 != action)
  {
    action->addTo( priorityPopup );
  }
  
  action = d->actions.find( COMPOSER_PRIORITY_LOW );
  if (0 != action)
  {
    action->addTo( priorityPopup );
  }
  
  action = d->actions.find( COMPOSER_PRIORITY_NORMAL );
  if (0 != action)
  {
    action->addTo( priorityPopup );
  }
  
  action = d->actions.find( COMPOSER_PRIORITY_HIGH );
  if (0 != action)
  {
    action->addTo( priorityPopup );
  }
  
  action = d->actions.find( COMPOSER_PRIORITY_HIGHEST );
  if (0 != action)
  {
    action->addTo( priorityPopup );
  }
  
  toolBar->addSeparator();
  
  d->fontChooser = new QFontChooser(toolBar, "FontChooser");
  d->fontChooser->setSelectedFont(d->composerWidget->editor()->font());
  
  toolBar->addSeparator();
  
  d->colorChooser = new FontColorButton(toolBar, "ColorChooser");
  
  // build Help menu
  QPopupMenu * helpMenu = new QPopupMenu(menuBar, "HelpMenu");
  menuBar->insertItem(tr("&Help"), helpMenu, -1);
  
  action = d->actions.find( COMPOSER_HELP_INDEX );
  if (0 != action)
  {
    action->addTo( helpMenu );
    action->addTo( toolBar );
  }
  
  action = d->actions.find( COMPOSER_HELP_ABOUT );
  if (0 != action)
    action->addTo( helpMenu );
  
  toolBar->addSeparator();
  
  QLabel * emptyLabel = new QLabel(toolBar);
  toolBar->setStretchableWidget( emptyLabel );

  return true;
}

bool ComposerWindow::buildConnections()
{
  QAction * action = 0L;
  
  // build the popup
  connect( d->sendAccounts, SIGNAL(aboutToShow()),
           this, SLOT(slotSendToShow()) );
  connect( d->sendAccounts, SIGNAL(activated(int)),
           this, SLOT(slotSendUsing(int)) );
  action = d->actions.find( COMPOSER_SEND );
  if (0 != action)
    connect( action, SIGNAL(activated()),
             this, SLOT(slotSendDefault()) );
  
  connect( d->sendLaterAccounts, SIGNAL(aboutToShow()),
           this, SLOT(slotSendLaterToShow()) );
  connect( d->sendLaterAccounts, SIGNAL(activated(int)),
           this, SLOT(slotSendLaterUsing(int)) );
  
  // mail changes
  connect(d->composerWidget->subjectEdit()->lineEdit(), SIGNAL(textChanged(const QString&)),
          this, SLOT(slotSubjectChanged(const QString&)));
  
//  connect(d->composerWidget->toField, SIGNAL(addressChanged()),
//          this, SLOT(ackAddressChanged()));
//  connect(d->composerWidget->ccField, SIGNAL(addressChanged()),
//          this, SLOT(ackAddressChanged()));
//  connect(d->composerWidget->bccField, SIGNAL(addressChanged()),
//          this, SLOT(ackAddressChanged()));

  // editor actions
  action = d->actions.find( COMPOSER_UNDO );
  if (0 != action)
  {
    connect(action, SIGNAL(activated()), d->composerWidget->editor(), SLOT(undo()));
    connect(d->composerWidget->editor(), SIGNAL(undoAvailable(bool)), action, SLOT(setEnabled(bool)));
  }
  
  action = d->actions.find( COMPOSER_REDO );
  if (0 != action)
  {
    connect(action, SIGNAL(activated()), d->composerWidget->editor(), SLOT(redo()));
    connect(d->composerWidget->editor(), SIGNAL(redoAvailable(bool)), action, SLOT(setEnabled(bool)));
  }
  
  action = d->actions.find( COMPOSER_CUT );
  if (0 != action)
  {
    connect(action, SIGNAL(activated()), d->composerWidget->editor(), SLOT(cut()));
    connect(d->composerWidget->editor(), SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
  }
  
  action = d->actions.find( COMPOSER_COPY );
  if (0 != action)
  {
    connect(action, SIGNAL(activated()), d->composerWidget->editor(), SLOT(copy()));
    connect(d->composerWidget->editor(), SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
  }
  
  action = d->actions.find( COMPOSER_PASTE );
  if (0 != action)
    connect(action, SIGNAL(activated()), d->composerWidget->editor(), SLOT(paste()));
  
  action = d->actions.find( COMPOSER_DELETE );
  if (0 != action)
  {
    connect(action, SIGNAL(activated()), d->composerWidget->editor(), SLOT(del()));
    connect(d->composerWidget->editor(), SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
  }
  
  action = d->actions.find( COMPOSER_SELECTALL );
  if (0 != action)

    connect(action, SIGNAL(activated()), d->composerWidget->editor(), SLOT(selectAll()));
  
  action = d->actions.find( COMPOSER_PRIORITY_LOWEST );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(setPriorityLowest()));
    
  action = d->actions.find( COMPOSER_PRIORITY_LOW );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(setPriorityLow()));
  
  action = d->actions.find( COMPOSER_PRIORITY_NORMAL );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(setPriorityNormal()));
  
  action = d->actions.find( COMPOSER_PRIORITY_HIGH );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(setPriorityHigh()));
  
  action = d->actions.find( COMPOSER_PRIORITY_HIGHEST );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(setPriorityHighest()));
  
  action = d->actions.find( COMPOSER_ATTACH_FILE );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(slotAttachFile()));
  

  action = d->actions.find( COMPOSER_ATTACH_SIGNATURE );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(slotAttachSignature()));
  
  action = d->actions.find( COMPOSER_SAVEAS_DRAFT );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(slotSaveAsDraft()));
  
  action = d->actions.find( COMPOSER_SAVEAS_FILE );
  if (0 != action)
    connect(action, SIGNAL(activated()), this, SLOT(slotSaveAsFile()));

  action = d->actions.find( COMPOSER_HTML_FORMAT );
  if (0 != action)
    connect(action, SIGNAL(toggled(bool)), this, SLOT(slotChangeFormat(bool)));
    
  
  connect(d->fontChooser, SIGNAL(fontChanged(const QFont &)),
    d->composerWidget->editor(), SLOT(setCurrentFont(const QFont &)));
  
  connect(d->colorChooser, SIGNAL(colorChanged(const QColor&)),
    d->composerWidget->editor(), SLOT(setColor(const QColor&)));
  
  return true;
}

bool ComposerWindow::deleteConnections()
{
  // nothing to do right now
  return true;
}

void ComposerWindow::slotSendDefault()
{
  if( !d->canSend() )
    return;

  d->buildAccountList();
  
  // find index in the account list
  int index = -1;
  int i=0;
  for(QStringList::ConstIterator it=d->accounts.begin(); it!=d->accounts.end(); ++it)
  {
    if( (*it) == d->sourceAccount )
    {
      index = i;
      break;
    }
    
    i++;
  }

  if( index != -1 )
    slotSendUsing(index);
  else
    QMessageBox::warning(this, QString(AppName) + tr(": No accounts defined"),
      tr("Cannot send the message, there is not a default account"), QMessageBox::Ok, 0, 0);
}

void ComposerWindow::slotSendUsing(int accountNumber)
{
//  qDebug("------------------------------------------------");
//  qDebug("Account number: %d", accountNumber);
  if( !d->canSend() )
    return;

  d->buildAccountList();

  QValueList<int>::const_iterator itAccountMenuId = d->accountMenuId.begin();
  QStringList::ConstIterator itAccounts=d->accounts.begin();
  for(; itAccounts!=d->accounts.end() && itAccountMenuId!=d->accountMenuId.end() ; ++itAccounts, ++itAccountMenuId)
  {
    if( (*itAccountMenuId) == accountNumber )
    {
      d->sourceAccount = (*itAccounts);
      break;
    }
  }
//  qDebug(" *-*- %d *-*- %s", *itAccountMenuId, (*itAccounts).latin1() );

  // save to Outbox
  d->saveToOutbox();
 
  MailObject *obj=(MailObject *)d->vfs->_fact_ref(d->url, VFS_MAILS_MIMETYPE );
  if(obj)
  {
    obj->setScheduling("");
    obj->setAccount(d->sourceAccount);
    obj->save();
  }

  // try to send message
  d->sendMessage();

  d->forcedClose = true;
  close();
//  qDebug("------------------------------------------------");
}

void ComposerWindow::slotSendLaterDefault()
{
  if( !d->canSend() )
    return;

  d->buildAccountList();
  
  // find index in the account list
  int index = -1;
  int i=0;
  for(QStringList::ConstIterator it=d->accounts.begin(); it!=d->accounts.end(); ++it)
  {
    if( (*it) == d->sourceAccount )
    {
      index = i;
      break;
    }
    
    i++;
  }

  if( index != -1 )
    slotSendLaterUsing(index);
  else
    QMessageBox::warning(this, QString(AppName) + tr(": No accounts defined"),
      tr("Cannot send the message, there is not a default account"), QMessageBox::Ok, 0, 0);
}

void ComposerWindow::slotSendLaterUsing(int accountNumber)
{
  if( !d->canSend() )
    return;

  int i=0;

  for(QStringList::ConstIterator it=d->accounts.begin(); it!=d->accounts.end(); ++it)
  {
    if( i == accountNumber )
    {
      d->sourceAccount = (*it);
      break;

    }
    
    i++;
  }
  
  // save to Outbox; the message will be sent at the next Send&Receive activation
  d->saveToOutbox();
  
  // set scheduling data and outgoing account
  MailObject *obj=(MailObject *)d->vfs->_fact_ref(d->url, VFS_MAILS_MIMETYPE );
  if(obj)
  {
    obj->setScheduling("");
    obj->setAccount(d->sourceAccount);
    obj->save();
  }
    
  // forcibly close window

  d->forcedClose=true;
  close();
}

void ComposerWindow::closeEvent(QCloseEvent *ev)
{
  // todo step 1
  bool shouldSave=false, shouldSend=false, shouldSendLater=false, shouldDelete=false;
  
  // if forced, just close window
  if(d->forcedClose)
  {
    ComposerFactory::instance()->slotCloseWindow( (QMainWindow*)this );
    
//    ev->accept();
    return;
  }

  // do we have a copy in Drafts or Outbox?
  bool inDrafts, inOutbox=false;


  inDrafts = d->vfs->map(d->url).contains("sysmail_drafts");
  if(!inDrafts)
    inOutbox = d->vfs->map(d->url).contains("sysmail_outbox");

  if(!d->url.isEmpty() && !d->vfs->exists(d->url))
  {
    // message was deleted by the user from the "Drafts" folder
    inDrafts=false;
    inOutbox=false;
    d->dirty=true;
  }

  // we have a copy in Drafts. does the user want to cancel close, keep it, or delete it?

  if(!inDrafts && !inOutbox)
  {
    switch(QMessageBox::information(this, QString("%1: Save message?").arg(AppName),
          "You can save this message in the Drafts folder\n"
          "and resume composition later. Save message?",
          "&Save", "&Discard", "&Cancel", 0, 2))
    {
      case 0:

        shouldSave=true;
        break;
      case 1:
        shouldDelete=true;
        break;
      case 2:
        ev->ignore();
        return;
    }
  }
  
  if(inDrafts)
  {

    // we have a copy in Drafts, does the user want to keep it?
    switch(QMessageBox::information(this, QString("%1: Keep message?").arg(AppName),
          "This message was saved in the Drafts folder.\n"

          "Do you want to keep this message and resume composition later?",
          "&Keep", "&Discard", "&Cancel", 0, 2))
    {
      case 0:
        shouldSave=true;

        break;
      case 1:
        shouldDelete=true;
        break;
      case 2:
        ev->ignore();
        return;
    }
  }
  
  if(inOutbox)
  {
    // we have a copy in Outbox. does the user want to cancel close, send it now, later or delete it?
    switch(QMessageBox::information(this, QString("%1: Send message?").arg(AppName),
          "This message was saved in the Outbox folder.\n"
          "You can send this message or discard it. Send message?",
          "&Send", "&Discard", "&Cancel", 0, 2))

    {
      case 0:
        shouldSend=true;
        shouldSendLater=QMessageBox::information(this, QString("%1: Send now?").arg(AppName),
            "Send message now or later?", "&Now", "&Later", QString::null, 0, 1);
        break;
      case 1:
        shouldDelete=true;
        break;
      case 2:
        ev->ignore();
        return;
    }
  }
    
  // delete message, if needed
  if(shouldDelete && !d->url.isEmpty())
    d->vfs->remove(d->url);

  // send message, if needed
  if(shouldSend)
  {
    if(shouldSendLater)
      slotSendLaterDefault();
    else
      slotSendDefault();
  }
  
  // save message, if needed
  if(shouldSave)
  {


    slotSave();
  }
  

  // close window
  ComposerFactory::instance()->slotCloseWindow( (QMainWindow*)this );
  ev->accept();
}

bool ComposerWindow::isOffline()
{
  // get offline mode
  return Settings::instance()->readBoolEntry

    ("/Aethera/Send Options/Work offline");
}

void ComposerWindow::slotSaveAsDraft()
{
  // save message
  slotSave();
  
  // if not in Drafts move to Drafts
  if(d->url.find(d->vfs->root()+"/Mail/Drafts")!=0)
    d->url = d->vfs->move(d->url, d->vfs->root()+"/Mail/Drafts");
}

void ComposerWindow::slotSave()
{
  d->save();
  
  slotShowCaption( d->composerWidget->subject() );
}

void ComposerWindow::slotSubjectChanged(const QString & subject)
{
  d->dirty = true;

  slotShowCaption( subject );
}

void ComposerWindow::slotAttachFile()
{
  d->dirty = true;
  d->save();
  
  d->composerWidget->slotAttachFile();
}

void ComposerWindow::slotAttachSignature()
{
  QString signaturePath = Settings::instance()->readEntry("/Aethera/Composer/SignaturePath");
  
  if(signaturePath.isEmpty())
  {
    signaturePath = QFileDialog::getOpenFileName(QString::null, QString::null, this);
    if( signaturePath.isEmpty() )
      return;
  }
  
  QFile signatureFile(signaturePath);

  if( !signatureFile.open(IO_ReadOnly) )
  {
    QMessageBox::warning(this, tr("Signature error"), tr("The signature file isn't accessible."));
    Settings::instance()->writeEntry("/Aethera/Composer/SignaturePath", "");
    return;
  }
  else
    Settings::instance()->writeEntry("/Aethera/Composer/SignaturePath", signaturePath);
  
  // insert the signature
  QString signaturePlace = Settings::instance()->readEntry("/Aethera/Composer/SignaturePlace", "BOTTOM");
  if( "BOTTOM" == signaturePlace )
  {
    QTextStream textStream(&signatureFile);
    QString signature = textStream.read();
    d->composerWidget->editor()->append(signature);
  }
  else
  {
    QTextStream textStream(&signatureFile);
    QString signature = textStream.read();
    d->composerWidget->editor()->insert(signature);
  }
  
  signatureFile.close();
}

void ComposerWindow::slotSaveAsFile()
{
  // TODO
}


void ComposerWindow::slotPriority(Mail::Priority newPriority)
{
  d->priority = newPriority;
  
  d->dirty = true;
}


void ComposerWindow::slotShowCaption(const QString & text)
{
  
  QString caption = QString("Composer ") + QString::number(d->windowID_) +
                    " - " + text;
  setCaption(caption);
}

void ComposerWindow::slotAutoSaveTimer()
{
  d->save();
  
  int saveTimeout = Settings::instance()->readNumEntry("/Aethera/Compose/Save timeout");
  if( !saveTimeout )
    saveTimeout = 3;
  
  d->autoSaveTimer->changeInterval(saveTimeout * 60 * 1000);
}

void ComposerWindow::slotSendToShow()
{
  d->buildAccountList();
  
  d->sendAccounts->clear();

  //be very carefull to always keep the accounts and the accountsMenuId in sync
  d->accountMenuId.clear();

  int menuId;
  for(QStringList::ConstIterator it=d->accounts.begin(); it!=d->accounts.end(); ++it)
  {
    menuId = d->sendAccounts->insertItem(*it);
    d->accountMenuId.push_back( menuId );
  }
}

void ComposerWindow::slotSendLaterToShow()

{
  d->buildAccountList();
  
  d->sendLaterAccounts->clear();

  for(QStringList::ConstIterator it=d->accounts.begin(); it!=d->accounts.end(); ++it)
  {
    d->sendLaterAccounts->insertItem(*it);
  }
}

void ComposerWindow::setPriorityLowest()
{
  slotPriority( Mail::Lowest );
}

void ComposerWindow::setPriorityLow()
{
  slotPriority( Mail::Low );
}

void ComposerWindow::setPriorityNormal()
{
  slotPriority( Mail::Normal );

}

void ComposerWindow::setPriorityHigh()
{
  slotPriority( Mail::High );
}

void ComposerWindow::setPriorityHighest()
{
  slotPriority( Mail::Highest );
}

void ComposerWindow::slotChangeFormat(bool isHtml)
{
  if( isHtml )
  {
    Q_CHECK_PTR( d->fontChooser );
    d->fontChooser->setEnabled( true );
    
    Q_CHECK_PTR( d->colorChooser );
    d->colorChooser->setEnabled( true );
    
    Q_CHECK_PTR( d->composerWidget->editor() );
    d->composerWidget->editor()->setTextFormat( Qt::RichText );
    
    d->composerWidget->slotUseRichText();
  }
  else
  {
    Q_CHECK_PTR( d->fontChooser );
    d->fontChooser->setEnabled( false );

    Q_CHECK_PTR( d->colorChooser );
    d->colorChooser->setEnabled( false );

    Q_CHECK_PTR( d->composerWidget->editor() );
    d->composerWidget->editor()->setTextFormat( Qt::PlainText );
  }
}

