/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#ifndef __ComposerWindow__
#define __ComposerWindow__

#include <komaetheraplugin.h>
#include <clientrmi.h>
#include <composerfactory.h>
#include <mail.h>

#include <qmainwindow.h>
#include <qstring.h>
#include <qpushbutton.h>

// Pics names
#define COMPOSER_NEW_PICS               "NewMail"
#define COMPOSER_DUPLICATE_PICS         "NewMail++"
#define COMPOSER_SAVE_PICS              "Save"
#define COMPOSER_SAVEAS_DRAFT_PICS      "SaveDraft"
#define COMPOSER_SAVEAS_FILE_PICS       "SaveAs"
#define COMPOSER_PRINT_PICS             "Print"
#define COMPOSER_CUT_PICS               "Cut"
#define COMPOSER_COPY_PICS              "Copy"
#define COMPOSER_PASTE_PICS             "Paste"
#define COMPOSER_DELETE_PICS            "Delete"
#define COMPOSER_UNDO_PICS              "Left"
#define COMPOSER_REDO_PICS              "Right"
#define COMPOSER_ATTACH_FILE_PICS       "Attachment"
#define COMPOSER_ATTACH_SIGNATURE_PICS  "signature"
#define COMPOSER_HELP_INDEX_PICS        "Help"

// Actions string id.
#define COMPOSER_NEW              "COMPOSER_NEW"
#define COMPOSER_DUPLICATE        "COMPOSER_DUPLICATE"
#define COMPOSER_CLOSE            "COMPOSER_CLOSE"
#define COMPOSER_SAVE             "COMPOSER_SAVE"
#define COMPOSER_SAVEAS_DRAFT     "COMPOSER_SAVEAS_DRAFT"
#define COMPOSER_SAVEAS_FILE      "COMPOSER_SAVEAS_FILE"
#define COMPOSER_PRINT            "COMPOSER_PRINT"
#define COMPOSER_CUT              "COMPOSER_CUT"
#define COMPOSER_COPY             "COMPOSER_COPY"
#define COMPOSER_PASTE            "COMPOSER_PASTE"
#define COMPOSER_DELETE           "COMPOSER_DELETE"
#define COMPOSER_UNDO             "COMPOSER_UNDO"
#define COMPOSER_REDO             "COMPOSER_REDO"
#define COMPOSER_SELECTALL        "COMPOSER_SELECTALL"
#define COMPOSER_ATTACH_FILE      "COMPOSER_ATTACH_FILE"
#define COMPOSER_ATTACH_SIGNATURE "COMPOSER_ATTACH_SIGNATURE"
#define COMPOSER_PRIORITY_LOW     "COMPOSER_LOWER"
#define COMPOSER_PRIORITY_LOWEST  "COMPOSER_LOWEST"
#define COMPOSER_PRIORITY_NORMAL  "COMPOSER_NORMAL"
#define COMPOSER_PRIORITY_HIGH    "COMPOSER_HIGHER"
#define COMPOSER_PRIORITY_HIGHEST "COMPOSER_HIGHEST"
#define COMPOSER_HELP_INDEX       "COMPOSER_HELP_INDEX"
#define COMPOSER_HELP_ABOUT       "COMPOSER_HELP_ABOUT"
#define COMPOSER_SEND             "COMPOSER_SEND_POPUP"
#define COMPOSER_HTML_FORMAT      "COMPOSER_HTML_FORMAT"


class MAIL_PLUGIN_API ComposerWindow : public QMainWindow
{
  Q_OBJECT
  
  public:
  
    /** Constructor using VFS, RMI and the window ID.*/
    ComposerWindow( VFSModule*, ClientRMIModule*, unsigned );
    /** Constructor using VFS, RMI, the window ID, 
      * the message vfsURL and the action MODE.
      */
    ComposerWindow( VFSModule*, ClientRMIModule*, unsigned,
                    const QString&, ComposerFactory::Mode );
    virtual ~ComposerWindow();

  public slots:
  
    /** Send now the message using the default account.*/
    virtual void slotSendDefault();
    /** Send now the message using an account from the list.*/
    virtual void slotSendUsing(int);
    /** Send later the message using the default account.*/
    virtual void slotSendLaterDefault();
    /** Send later the message using an account from the list.*/
    virtual void slotSendLaterUsing(int);

    /** Add a file as attachment.*/
    virtual void slotAttachFile();
    /** Add a signature as attachment.*/
    virtual void slotAttachSignature();

    /** Set the message priority.*/
    virtual void slotPriority(Mail::Priority priority=Mail::Normal);
    virtual void setPriorityLowest();
    virtual void setPriorityLow();
    virtual void setPriorityNormal();
    virtual void setPriorityHigh();
    virtual void setPriorityHighest();
    
    /** Save the message.*/
    virtual void slotSave();
    /** Save the message in Draft folder.*/
    virtual void slotSaveAsDraft();
    /** Save the message as file.*/
    virtual void slotSaveAsFile();

    void slotChangeFormat(bool);

  private slots:
  
    /** Set the window caption using a string.*/
    virtual void slotShowCaption(const QString &);

    /** Internal slot used to acknowledge that the subject was changed.*/
    void slotSubjectChanged(const QString&);

    /** Internal slot for autosave.*/
    void slotAutoSaveTimer();
    
    /** Build the send popup menu.*/
    void slotSendToShow();
    
    /** Build the send later popup menu.*/
    void slotSendLaterToShow();
    
  protected:
  
    /** Build the QAction-s.*/
    virtual bool buildActions();
    /** Build the window's menu and main toolbar.*/
    virtual bool buildMenuAndToolbar();
    /** Connect.*/
    virtual bool buildConnections();
    /** Disconnect.*/
    virtual bool deleteConnections();
    
    /** Check the config file for offline field.*/
    bool isOffline();
    
  private:
  
    /**Do special things when the window is closed.*/
    void closeEvent(QCloseEvent *);
    
    class Private;
    Private * d;
};

#endif
