/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/***************************************************************************
                             indexmailitem.cpp 
                             -------------------

  copyright : (C) by theKompany.com     <www.thekompany.com>
              (C) by Atol C&D           <www.atolcd.com>

  authors :   Eugen C.      <eug@thekompany.com>
              Fabien Armand <f.armand@atolcd.com>

 ***************************************************************************/
 
#include <indexmailitem.h>

#include <qapplication.h>
#include <qfile.h>
#include <qfont.h>
#include <qtextstream.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qradiobutton.h>
#include <qregexp.h>
#include <qsettings.h>
#include <qtextcodec.h>

// TINO include
#include <tino/uipartmanager.h>

#include <komaetheraplugin.h>
#include <qsettingsextra.h>
#include <magellan.h>

#include <maillistview.h>
#include <mailobj.h>
#include <saveformatdialog.h>
#include <texthtml.h>
#include <textfitter.h>

#include <assert.h>

QFont * IndexMailItem::readFont = 0L;
QFont * IndexMailItem::unreadFont = 0L;

void IndexMailItem::setFonts()
{
  // TODO: set and use a configurable font size
  
  if( !readFont )
  {
    readFont = new QFont();
    readFont->setBold(false);
    readFont->setPixelSize(12);
  }
  if( !unreadFont )
  {
    unreadFont = new QFont();
    unreadFont->setBold(true);
    unreadFont->setPixelSize(12);
  }
}

IndexMailItem::IndexMailItem(MailListView *parent, VFSModule * vfsModule, MailObject * object)
  : EListViewItem(parent),
    _size(0),
    _flag(false),
    _attachments( Mail::NoAttachment ),
    vfs( vfsModule )
{
  if( object )
  {
    _path = object->path();

    setDragEnabled(true);
    
    setFonts();

    update(object);
  }
}

IndexMailItem::IndexMailItem(MailListView *parent, QListViewItem *after, VFSModule * vfsModule, MailObject * object)
  : EListViewItem(parent, after),
    _size(0),
    _flag(false),
    _attachments( Mail::NoAttachment ),
    vfs( vfsModule )
{
  if( object )
  {
    _path = object->path();

    setDragEnabled(true);

    setFonts();

    update(object);
  }
}

IndexMailItem::IndexMailItem(MailListView *parent, VFSModule * vfsModule, const QString& path)
  : EListViewItem(parent),
    _size(0),
    _flag(false),
    _attachments( Mail::NoAttachment ),
    vfs( vfsModule ),
    _path( path )
{
  setDragEnabled(true);

  setFonts();
  
  update();
}

IndexMailItem::IndexMailItem( MailListView *parent, QListViewItem *after,
                              VFSModule * vfsModule, const QString& path)
  : EListViewItem(parent, after),
    _size(0),
    _flag(false),
    _attachments( Mail::NoAttachment ),
    vfs( vfsModule ),
    _path( path )
{
  setDragEnabled(true);
  
  setFonts();
  
  update();
}

IndexMailItem::~IndexMailItem()
{
}

QString IndexMailItem::path() const
{
  return _path;
}

void IndexMailItem::update()
{
  MailObject *item = vObject();

  if( item )
    update(item);
}

void IndexMailItem::update(MailObject * item)
{
  int type=((MailListView *)listView())->getType();
  bool rdate=((MailListView *)listView())->byRecDate();
  QString t;
  QDateTime epoch(QDate(1980, 1, 1));

  _new = ("New" == item->status());

  // common columns
  setPixmap(0, Mail::instance()->toPixmap( Mail::toPriority( item->priority() ) ) );

  _attachments = item->attachments() ? Mail::HasAttachment : Mail::NoAttachment;
  setPixmap(1, Mail::instance()->toPixmap( _attachments ) );

//    QString status;
//
//    if( item->isReplied() )
//      status = "replyed";
//    if( item->isForwarded() )
//      status += "forwarded";
//    if( status.isEmpty() )
//      status = "default";


  switch(type)
  {
    default:
    case Magellan::mail :
      if( item->from().getTranslated_name().isEmpty() )
        setText(2, QString(item->from().email) );
      else
        setText(2, item->from().getTranslated_name() );
      setText(3, item->subject() );

      _date=rdate?item->receivedDate():item->sentDate();
      // transform the date from the rfc822 english in local date
      setText(4, _date.toLocalDateQCString());
      _dateKey.sprintf("%08d", epoch.secsTo(_date.toQDateTime()));

      setText(5, item->status());

      _flag=item->flag();
      if(_flag)
        setPixmap(6, BarIcon("flag"));
      else
        setPixmap(6, QPixmap());

      _size = MAX(1, (item->size())/1024);
      t.setNum(_size);
      t+=" kb";
      setText(7, t);
      break;
    case Magellan::sent:
      setText(2, item->to().returnString() );
      setText(3, item->subject());

      _date=item->sentDate();
      // transform the date from the rfc822 english in local date
      setText(4, _date.toLocalDateQCString());
      _dateKey.sprintf("%08d", epoch.secsTo(_date.toQDateTime()));

      setText(5, item->status());
      _size=MAX(1, (item->size())/1024);
      t.setNum(_size);
      t+=" kb";
      setText(6, t);
      break;
    case Magellan::drafts:
      setText(2, item->to().returnString() );
      setText(3, item->subject());
      _size=MAX(1, (item->size())/1024);
      t.setNum(_size);
      t+=" kb";
      setText(4, t);
      break;
    case Magellan::outgoing:
      setText(2, item->to().returnString() );
      setText(3, item->subject());
      setText(4, item->scheduling());
      _size=MAX(1, (item->size())/1024);
      t.setNum(_size);
      t+=" kb";
      setText(5, t);
      break;
  }
}

void IndexMailItem::paintCell(QPainter * p, const QColorGroup & cg, int column, int width, int align)
{
  if( _new )
    p->setFont(*unreadFont);
  else
    p->setFont(*readFont);

  EListViewItem::paintCell(p,cg,column,width,align);
}


void IndexMailItem::paintFocus(QPainter *, const QColorGroup &, const QRect &)
{
  // we use this to disable the Qt focus
}

QString IndexMailItem::key(int column, bool /*ascending*/) const
{
  QString tmpString;
  int type=((MailListView *)listView())->getType();

  if((type==Magellan::mail || type==Magellan::sent) && column == 4)
  {
    tmpString = _dateKey;
  }
  else if((type==Magellan::mail && column== 7) ||
          (type==Magellan::sent && column== 6) ||
          (type==Magellan::outgoing && column== 5) ||
          (type==Magellan::drafts && column== 4))
  {
    tmpString.sprintf("%08d", _size);
  }
  else if(type==Magellan::mail && column==6)
  {
    tmpString=_flag?"a":"b";
  }
  else if(column==1)
  {
    tmpString=_attachments?"a":"b";
  }
  else
  {
    tmpString=text(column);
  }

  return tmpString;
  
return QString::null;
}

void IndexMailItem::save()
{
  UIPartManager *manager = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_MANAGER );
    if( ref.isValid() )
      manager = dynamic_cast<UIPartManager*>(ref.instance());
  }

  if( manager )
  {
    SaveFormatDialog *formatDialog=new SaveFormatDialog((QWidget*)(manager->mainWindow()), 0, true);
    MailObject *obj = vObject();
    TextHtmlConvertor *textHtmlConverter = TextHtmlConvertor::ref();
    
    if( obj && textHtmlConverter )
    {
      if(!obj->hasHTML())
      {
        formatDialog->originalHTMLRadio->setEnabled(false);
        formatDialog->formattedHTMLRadio->setEnabled(false);
      }
      if(!obj->hasText())
      {
        formatDialog->originalTextRadio->setEnabled(false);
        formatDialog->formattedTextRadio->setEnabled(false);
      }
//      qDebug("\tIndexMailItem2 subject\n %s",obj->subject().latin1() );
      QString subject=obj->subject();
      subject.replace(QRegExp(QString::fromLatin1("[:/#]")), QString::fromLatin1(" "));
      subject.stripWhiteSpace();

      QString filter;

      if(formatDialog->exec()==QDialog::Accepted)
      {
        if(formatDialog->originalHTMLRadio->isOn()
            || formatDialog->formattedTextRadio->isOn()
            || formatDialog->formattedHTMLRadio->isOn())
        {
          filter="*.html";
          subject+=".html";
        }
        else if(formatDialog->entireMessageRadio->isOn())
        {
          filter="*.msg";
          subject+=".msg";
        }
        else
        {
          filter="*.txt";
          subject+=".txt";
        }

        QString filename=QFileDialog::getSaveFileName(subject, filter, (QWidget*)manager->mainWindow());

        if(!filename.isEmpty())
        {
          QString text;
          if(formatDialog->originalHTMLRadio->isOn())
          {
            text=obj->html();
          }
          else if(formatDialog->originalTextRadio->isOn())
          {
            text=obj->text();
          }
          else if(formatDialog->formattedTextRadio->isOn())
          {
            text=obj->text();
            textHtmlConverter->text2html(text);
          }
          else if(formatDialog->formattedHTMLRadio->isOn())
          {
            text=obj->html();
            textHtmlConverter->html2html(text);
          }
          else
          {
            text=obj->message();
          }

          QFile saveFile(filename);
          if(saveFile.open(IO_WriteOnly))
          {
            QTextStream txt(&saveFile);
            txt<<text;
            saveFile.close();
          }
          else
          {
            QMessageBox::warning((QWidget*)manager->mainWindow(), QString("%1: Could not save message").arg(AppName),
                "Could not open file for saving.\nPlease check that you have write permissions and try again",
                QMessageBox::Ok, QMessageBox::NoButton);
          }
        }
      }
    }
  }
}

MailObject *IndexMailItem::vObject() const
{
  if( _path.isEmpty() )
  {
    return 0;
  }
  
  return static_cast<MailObject*> (vfs->_fact_ref(_path, VFS_MAILS_MIMETYPE));
}

int IndexMailItem::rtti() const
{
  return INDEX_MAIL_ITEM_ID;
}

//  int
//IndexMailItem::width
//(
//  const QFontMetrics & fm,
//  const QListView * lv,
//  int col
//) const
//{
//  switch (col)
//  {
//    case 0:
//      return 22;
//      break;
//    case 1:
//      return 22;
//      break;
//    case 2:
//      return 22;
//      break;
//  }
//  return QListViewItem::width(fm, lv, col);
//}
