/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  kommailmaintreeinterface.cpp
                             -------------------
    begin                : Thu Nov  1 20:51:18 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qpopupmenu.h>

#include <vfolder.h>
#include <kolabsharedfolder.h>
#include <kommailuiservice.h>
#include <kommailbackendservice.h>

#include <kommailtreeitem.h>
#include <kommailmaintreeinterface.h>
#include <kommailmaintreeslots.h>

#include <qiconloader.h>

KomMailMainTreeInterface::KomMailMainTreeInterface(QListView * parent)
  : KomMainTreeInterface  (parent),
    m_popupMenu           (0L),
    m_slots               (0L),
    m_currentItem         (0L),
    m_root                (0L)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
    else
    {
      qDebug("AccountManagerWidget: Can't find VFS module");
    }
  }
  
  assert(0 != vfs);
  assert(0 != clientRMI);
  
  KomItemType *itemType =
    new KomItemType
    (
      ItemTypeRoot,
      new QString("Mail"),
      new QPixmap(BarIcon("MailContainer.xpm")),
      true,
      true
    );
  itemTypeList().insert("root", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeInbox,
      new QString("Inbox"),
      new QPixmap(BarIcon("Inbox.xpm")),
      true,
      true
    );
  itemTypeList().insert("inbox", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeOutbox,
      new QString("Outbox"),
      new QPixmap(BarIcon("Outbox.xpm")),
      true,
      true
    );
  itemTypeList().insert("outbox", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeSaved,
      new QString("Saved messages"),
      new QPixmap(BarIcon("SavedMessages.xpm")),
      true,
      true
    );
  itemTypeList().insert("saved", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeSent,
      new QString("Sent Mail"),
      new QPixmap(BarIcon("SentMail.xpm")),
      true,
      true
    );
  itemTypeList().insert("sent", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeDrafts,
      new QString("Drafts"),
      new QPixmap(BarIcon("Drafts.xpm")),
      true,
      true
    );
  itemTypeList().insert("drafts", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeDeleted,
      new QString("Deleted Messages"),
      new QPixmap(BarIcon("Trash.xpm")),
      true,
      true
    );
  itemTypeList().insert("deleted", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeGroup,
      new QString("Group"),
      new QPixmap(BarIcon("MailContainer.xpm")),
      true,
      true
    );
  itemTypeList().insert("genmail", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeLeaf,
      new QString("Mail"),
      new QPixmap(BarIcon("MailFolder.xpm")),
      true,
      true
    );
  itemTypeList().insert("mail", itemType);

  m_popupMenu = new QPopupMenu(parentTree(), "KomMailTreeContextPopup");

  m_slots = new KomMailMainTreeSlots(this, "KomMailMainTreeSlots");
}

KomMailMainTreeInterface::~KomMailMainTreeInterface()
{
  delete m_slots;
}

  void
KomMailMainTreeInterface::show()
{
  m_root = new KomMailTreeItem(parentTree(), *const_cast<KomItemType *>(itemTypeList().getItemOfType("root")));
  m_root->setOpen(true);
  createChildren(m_root);
  parentTree()->setCurrentItem(m_root);
  parentTree()->setSelected(m_root, true);
}

  void
KomMailMainTreeInterface::show(const QString & focus)
{
  m_root = new KomMailTreeItem(parentTree(), *const_cast<KomItemType *>(itemTypeList().getItemOfType("root")));
  KomMainTreeInterface::show(focus);
}

  void
KomMailMainTreeInterface::hide()
{
  delete m_root;
}

QString KomMailMainTreeInterface::path(KomTreeItem *item)
{
  QString vfsPath = static_cast<KomMailTreeItem*>(item)->path();
  
  if( vfsPath.isEmpty() )
    return KomMainTreeInterface::path(item);
  else
    return vfsPath;
}
    
  KomTreeItem *
KomMailMainTreeInterface::root()
{
      return m_root;
}

  void
KomMailMainTreeInterface::createChildren(KomTreeItem * root)
{
  if (0 == root)
    return;

  QString rootPath = path(root);

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qWarning("No VFS module ?");
    return;
  }

  VFolder * parentFolder = vfs->folderRef(rootPath);
  if( 0 == parentFolder ) // oops
    return;
  VFolderList & folderList = parentFolder->folderList();  

  for (VFolderListIterator it(folderList); it.current(); ++it)
  {
    QString childName((it.current()->path().mid(rootPath.length() + 1)));

    // See if it exists already.

    KomTreeItem * folderItem = findChild(root, childName);

    if (0 == folderItem)
    {
      // No ? Then make it.
      folderItem = createFolderItem
                    (
                      root,
                      it.current()->path().mid(rootPath.length() + 1),
                      it.current()->viewType()
                    );
    }

    // Recurse.
    createChildren(folderItem);
  }
}

  QPopupMenu *
KomMailMainTreeInterface::contextMenu(KomTreeItem * item)
{
  assert(0 != m_popupMenu);

  m_popupMenu->clear();

  QString folderPath = path(item);
  
  bool bIMAP = false;
  if( isIMAP(folderPath) )
  {
    bIMAP = true;
  }
  
  bool bKolab = false;
  if( KolabSharedFolder::ref()->isKolabParentFolder(folderPath) )
  {
    bKolab = true;
  }
  
  switch (item->getItemType())
  {
    case ItemTypeRoot:

      m_popupMenu->insertItem
        ( QObject::tr("New mail group"), m_slots, SLOT(slotNewMailGroup()) );

      break;

    case ItemTypeGroup:

      m_popupMenu->insertItem
        ( QObject::tr("New mail group"), m_slots, SLOT(slotNewMailGroup()) );

      if( !bKolab && !bIMAP )
      {
        m_popupMenu->insertItem
          ( QObject::tr("Delete mail group"), m_slots, SLOT(slotDeleteMailGroup()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Move mail group"), m_slots, SLOT(slotMoveMailGroup()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Rename mail group"), m_slots, SLOT(slotRenameMailGroup()) );
      }

      break;

    default:
    case ItemTypeLeaf:
    
      if( !bKolab && !bIMAP )
      {
        m_popupMenu->insertItem
          ( QObject::tr("Delete mail group"), m_slots, SLOT(slotDeleteMailGroup()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Move mail group"), m_slots, SLOT(slotMoveMailGroup()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Rename mail group"), m_slots, SLOT(slotRenameMailGroup()) );
      }

    case ItemTypeInbox:
    case ItemTypeOutbox:
    case ItemTypeSent:
    case ItemTypeSaved:
    case ItemTypeDeleted:
    case ItemTypeDrafts:

//      m_popupMenu->setItemEnabled
//        (
//          m_popupMenu->insertItem(QObject::tr("[No actions available]")),
//          false
//        );

      m_popupMenu->insertItem
        ( QObject::tr("New mail group"), m_slots, SLOT(slotNewMailGroup()) );

      m_popupMenu->insertItem
        ( QObject::tr("Expunge"), m_slots, SLOT(slotExpungeMailGroup()) );

      break;
  }

  return m_popupMenu;
}

  bool
KomMailMainTreeInterface::hasChild(KomTreeItem * item)
{
  for (QListViewItemIterator it(parentTree()); it.current(); ++it)
  {
    if (it.current() == item)
      return true;
  }

  return false;
}

  void 
KomMailMainTreeInterface::updateVInstances(const QStringList & _l)
{

//  qDebug("KomMailMainTreeInterface::updateVInstances");

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qWarning("No VFS module ?");
    return;
  }

  QStringList l(_l);
  l.sort();
  QString parentPath;

  for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
  {
    // update only for folders
    VFolder * folder = vfs->folderRef(*it);
    // try the parent folder
    if (0 == folder)
    {
      folder = vfs->folderRef((*it).left( (*it).findRev("/") ));
    }
    // wrong url, skip it
    if (0 == folder)
      continue;

    QString folderPath = folder->path();
      
    // avoid to repeat the update
    if( parentPath != folderPath )
    {
      KomMailTreeItem * parentItem = static_cast<KomMailTreeItem *>
      ( folderItem(folderPath, false /* Do not create */) );
        
      if( parentItem )
      {
        parentItem->setText(0, folder->name());
        // force unread mails scanning
        parentItem->forceUpdate();
        parentItem->repaint();
      }
      parentPath = folderPath;
    }
  }
}

  void
KomMailMainTreeInterface::createVInstances(const QStringList &_l)
{
// only objects are created with this signal
// so, we need it just for showing the number of unread mails

  // update the folders
  updateVInstances(_l);
}

  void 
KomMailMainTreeInterface::deleteVInstances(const QStringList &_l)
{
  // update the folders
  updateVInstances(_l);
}

  void 
KomMailMainTreeInterface::updateHierarchy(const QString & s)
{
  // You should rebuild the tree with the root 's'
  qDebug("KomMailMainTreeInterface::updateHierarchy(%s): STUB", s.ascii());
}

  QObject *
KomMailMainTreeInterface::qObject()
{
  return m_slots;
}

  KomTreeItem *
KomMailMainTreeInterface::folderItem(const QString & url, bool create)
{
  QString mailRoot = "/Aethera/VFS/Mail/";
  if ( mailRoot != url.left( mailRoot.length() ))
  {
//    qDebug("Hmm,mail wrong path `%s'", url.ascii());
    return 0;
  }

//  qDebug("folderItem %s", url.latin1());
  
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      qWarning("\n NO VFS Provider !\n");
      return 0;
    }
  }
  VFolder *folder = vfs->folderRef(url);
  
  QStringList pathList(QStringList::split('/', url.mid( mailRoot.length() )));

  KomTreeItem * root = m_root;
  KomTreeItem * child = 0;

  QStringList::ConstIterator it(pathList.begin());

  for (it = pathList.begin(); it != pathList.end(); ++it)
  {
    child = findChild(root, *it);

    if (0 == child)
    {
      if (create && folder)
      {
        return createFolderItem(root, *it, folder->viewType());
      }
      else
      {
        return 0;
      }
    }
    else
      root = child;
  }

  return child;
}

  KomTreeItem *
KomMailMainTreeInterface::findChild(KomTreeItem * root, const QString & name)
{
  KomTreeItem * treeItem = static_cast<KomTreeItem *>(root->firstChild());

  while (0 != treeItem)
  {
    if (treeItem->text(0) == name)
      return treeItem;

    treeItem = static_cast<KomTreeItem *>(treeItem->nextSibling());
  }

  return 0;
}

  void
KomMailMainTreeInterface::create(const QString & url)
{
  if ("folder" == url.lower())
  {
    folderItem(url, true);
  }
  else
  {
    qDebug("Don't yet know how to create %s", url.ascii());
  }
}

  void
KomMailMainTreeInterface::createVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & /*type*/,
  const QString & /*subType*/,
  const QString & /*serverPath*/
)
{
  folderItem(parentFolderPath+"/"+folderName, true);
}

  void
KomMailMainTreeInterface::deleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & /*type*/
)
{
  // the VFolder should be already deleted, so only the folder item
  // has to be deleted
  KomTreeItem *deletedFolder = folderItem(parentFolderPath+"/"+folderName, false);
  if( deletedFolder )
    delete deletedFolder;
}

  KomTreeItem *
KomMailMainTreeInterface::createFolderItem
(
  KomTreeItem * parentItem,
  const QString & folderName,
  const QString & type
)
{
  QString itemText(folderName);
  KomItemType komItemType = *const_cast<KomItemType *>(itemTypeList().getItemOfType(type));
  komItemType.setItemText(&itemText);

  return static_cast<KomTreeItem *> (new KomMailTreeItem(parentItem, komItemType));
}

bool KomMailMainTreeInterface::isIMAP(const QString & path) const
{
  QStringList accounts = clientRMI->getAccountNames(Account::IMAP4);
  
  for(QStringList::Iterator it=accounts.begin(); it!=accounts.end(); ++it)
  {
    if( path == vfs->root() + "/Mail/" + (*it) )
    {
      return true;
    }
  }
  
  return false;
}

// vim:tabstop=2:shiftwidth=2:expandtab:cinoptions=(s,U1,m1
