/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  kommailmaintreeslots.cpp
                             -------------------
    begin                : Fri Nov  2 12:42:27 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
 ***************************************************************************/

#include <defines.h>
#include <clientvfs.h>
#include <clientrmi.h>
#include <vfolder.h>
#include <mailobj.h>
#include <kommailbackendservice.h>
#include <kommailmaintreeinterface.h>
#include <kommailmaintreeslots.h>
#include <kommailuiservice.h>
#include <foldernamerequestdialog.h>
#include <folderchooserdialog.h>

#include <qinputdialog.h>
#include <qmessagebox.h>

KomMailMainTreeSlots::KomMailMainTreeSlots
(
  KomMailMainTreeInterface * parent,
  const char * name
)
  : QObject             (0, name),
    m_mainTreeInterface (parent)
{
}

KomMailMainTreeSlots::~KomMailMainTreeSlots()
{
  // Empty.
}

void KomMailMainTreeSlots::slotNewMail()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomMailMainTreeSlots::slotNewMail(): no current item");
    return;
  }

  qDebug("KomMailMainTreeSlots::slotNewMail: creating object at %s", path.ascii());

  // Fire and forget...
  vfs->createObject(path);
}

void KomMailMainTreeSlots::slotDeleteMail()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomMailMainTreeSlots::slotDeleteMail(): no current item");
    return;
  }

  bool ok = vfs->remove(path);

  if (!ok)
  {
    DEBUG_MSG("KomMailMainTreeSlots::slotDeleteMail(): couldn't delete mail");
  }
}

void KomMailMainTreeSlots::slotEditMail()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomMailMainTreeSlots::slotEditMail(): no current item");
    return;
  }

  VObject * mailRef = vfs->_fact_ref(path, VFS_MAILS_MIMETYPE);

  if (0 == mailRef)
  {
    qDebug("Hmm, can't find mail to edit");
    return;
  }

  MailObject * mail = dynamic_cast<MailObject *>(mailRef);

  if (0 == mail)
  {
    qDebug("dynamic_cast from VObject to MailObject failed");
    return;
  }

  qDebug("STUB - need to edit mail");
}

void KomMailMainTreeSlots::slotNewMailGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomMailMainTreeSlots::slotNewMailGroup(): no current item");
    return;
  }
 
  qDebug("Path: `%s'", path.ascii());

  VFolder * parentRef = vfs->folderRef(path);

  if (0 == parentRef)
  {
    qDebug("Parent does not exist");
    return;
  }

  struct FolderInfo folderInfo = FolderNameRequestDialog::getFolderInfo();

  if (!folderInfo.name.isEmpty())
  {
    VFolder * folderRef = vfs->folderRef(path + "/testing");

    if (0 != folderRef)
    {
      qDebug("Folder already exists");
    }
    else
    {
      qDebug("Folder does not exist. Making");

      VFolder *f=0;
      // Only the container folders can have other folders inside.
//      if( parentRef->viewType()=="genmail" )
      {
        if( folderInfo.containerOnly )
        {
          if( parentRef->type()=="imap" )
            f = vfs->createFolder(parentRef, folderInfo.name, parentRef->type(), "genmail");
          else
            f = vfs->createFolder(parentRef, folderInfo.name, "genmail", "genmail");
        }
        else
        {
          if( parentRef->type()=="imap" )
            f = vfs->createFolder(parentRef, folderInfo.name, parentRef->type(), "mail");
          else
            f = vfs->createFolder(parentRef, folderInfo.name, "mail", "mail");
        }
      }

      if (0 == f)
      {
        qDebug("Couldn't create folder");
        return;
      }

      qDebug("Folder made successfully");
    }

    m_mainTreeInterface->createChildren(m_mainTreeInterface->currentItem());
  }
}

void KomMailMainTreeSlots::slotDeleteMailGroup()
{
  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomMailMainTreeSlots::slotDeleteMailGroup(): no current item");
    return;
  }
 
  // remove the item from the list
  delete m_mainTreeInterface->currentItem();
  
  emit deleteMailGroup(path);
}

  void
KomMailMainTreeSlots::slotUpdateVInstances(const QStringList & l)
{
  m_mainTreeInterface->updateVInstances(l);
}

  void 
KomMailMainTreeSlots::slotCreateVInstances(const QStringList & l)
{
  m_mainTreeInterface->createVInstances(l);
} 

  void 
KomMailMainTreeSlots::slotDeleteVInstances(const QStringList & l)
{
  m_mainTreeInterface->deleteVInstances(l);
}

  void 
KomMailMainTreeSlots::slotUpdateHierarchy(const QString & s)
{
  m_mainTreeInterface->updateHierarchy(s);
}

  void
KomMailMainTreeSlots::slotCreate(const QString & url)
{
  qDebug("KomMailMainTreeSlots::slotCreate(%s)", url.ascii());
  m_mainTreeInterface->create(url);
}

  void
KomMailMainTreeSlots::slotCreateVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type,
  const QString & subType,
  const QString & serverPath
)
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  QString vfsFolder;
  if( type=="imap" && 
      vfs->root() != parentFolderPath.left(vfs->root().length()) )
    vfsFolder = vfs->root() + "/Mail/" + parentFolderPath;
  else
    vfsFolder = parentFolderPath;
    
  m_mainTreeInterface->createVFolder
    (
      vfsFolder,
      folderName,
      type,
      subType,
      serverPath
    );
}

  void
KomMailMainTreeSlots::slotDeleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type
)
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  QString vfsFolder;
  if( type=="imap" && 
      vfs->root() != parentFolderPath.left(vfs->root().length()) )
    vfsFolder = vfs->root() + "/Mail/" + parentFolderPath;
  else
    vfsFolder = parentFolderPath;
  
  m_mainTreeInterface->deleteVFolder(vfsFolder, folderName, type);
}

  void
KomMailMainTreeSlots::slotMoveMailGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  QString targetPath = FolderChooserDialog::getFolder("/Mail", 0);

  if( targetPath.isEmpty() )
    return;
  
  if(targetPath != currentPath)
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }
    
    VFolder * folder = vfs->folderRef(currentPath);
    VFolder * targetFolder = vfs->folderRef(targetPath);
    
    QStringList foldersTree;
    if( folder && targetFolder )
    {
      if( "imap" == folder->type() || 
          "imap" == targetFolder->type() )
      {
          QMessageBox::warning( 0L,
                                QObject::tr("Folder Move Warning"), 
                                QObject::tr("Moving folders isn't working for IMAP folders yet."),
                                QMessageBox::Ok, 
                                QMessageBox::NoButton, 
                                QMessageBox::NoButton );
          return;
      }
      
      // check if the target folder is a subfolder
      folder->tree(foldersTree, -1);
      for(QStringList::const_iterator it=foldersTree.begin(); it!=foldersTree.end(); ++it)
      {
        if( targetPath == (*it) )
        {
          QMessageBox::warning( 0L,
                                QObject::tr("Folder Move Warning"), 
                                QObject::tr("Moving a folder in one of its subfolders isn't allowed."),
                                QMessageBox::Ok, 
                                QMessageBox::NoButton, 
                                QMessageBox::NoButton );
          return;
        }
      }
      
      vfs->moveFolder(folder->path(), targetPath + "/" + folder->name() );
    }
  }
  else
  {
    QMessageBox::warning( 0L,
                          QObject::tr("Folder Move Warning"), 
                          QObject::tr("The new folder path is the same with the old one."),
                          QMessageBox::Ok, 
                          QMessageBox::NoButton, 
                          QMessageBox::NoButton );
  }
}

  void
KomMailMainTreeSlots::slotRenameMailGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  bool ok = false;
  QString targetName = QInputDialog::getText( QObject::tr("Folder Rename"),
                                              QObject::tr("Please insert the new folder name:"),
                                              QLineEdit::Normal,
                                              QString::null,
                                              &ok, 0L, "NotesFolderRename");
  if( ok )
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }
    
    VFolder * folder = vfs->folderRef(currentPath);
    QStringList foldersTree;
    if( folder && !targetName.isEmpty() )
    {
      if( "imap" == folder->type() )
      {
        if( "genmail" == folder->viewType() )
        {
          QMessageBox::warning( 0L, 
                                QObject::tr("Folder Rename Warning"), 
                                QObject::tr("Renaming this folder is not available yet."),
                                QMessageBox::Ok, 
                                QMessageBox::NoButton, 
                                QMessageBox::NoButton );
        }
        else
        {
          // send the rename command to the server
          ClientRMIModule * clientRMI = 0L;
          
          ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
          if( trader )
          {
            ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
            if( ref.isValid() )
              clientRMI = static_cast<ClientRMIModule*>(ref.instance());
          }
          assert(0 != clientRMI);
          
          clientRMI->rename(folder->path(), targetName);
        }
      }
      else
      {
        vfs->moveFolder(folder->path(), folder->parentFolder()->path() + "/" + targetName);
      }
    }
    else
    {
      QMessageBox::warning( 0L, 
                            QObject::tr("Folder Rename Warning"), 
                            QObject::tr("The new name is empty or the folder is not available."),
                            QMessageBox::Ok, 
                            QMessageBox::NoButton, 
                            QMessageBox::NoButton );
    }
  }
}

  void
KomMailMainTreeSlots::slotExpungeMailGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    DEBUG_MSG("No VFS module ?");
    return;
  }
  
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  VFolder * folder = vfs->folderRef(currentPath);
  
  if( folder )
  {
    if( "imap" == folder->type() )
    {
      // send the expunge command to the server
      ClientRMIModule * clientRMI = 0L;
        
      ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
      if( trader )
      {
        ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
        if( ref.isValid() )
          clientRMI = static_cast<ClientRMIModule*>(ref.instance());
      }
      assert(0 != clientRMI);
        
      clientRMI->expunge(folder->path());
    }
  }
}

// vim:tabstop=2:shiftwidth=2:expandtab:cinoptions=(s,U1,m1
