/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
  Date     : 10 August 2003
  Copyright: 2003 theKompany.com
  Author   : Eugen Constantinescu <eug@thekompany.com>
  Author   : Ghiciuc Ilie <ilie@eco-spaces.com.ro>
---------------------------------------------------------------------------------
*/

//own includes
#include "mailagentservice.h"
#include "mailobj.h"
#include "vpartattribute.h"
#include "partobj.h"

//libs includes
#include <vfolder.h>
#include <charsets.h>
#include <mimecodec.h>

// QT
#include <qtextcodec.h>

kore::Interface* mailagentservice_factory()
{
  return new MailAgentService();
}

MailAgentService::MailAgentService()
  : vfs(0L),
    clientRMI(0L)
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Mail Agent Service");
  info->replaceAll("class", "MailAgentService");
  info->replaceAll("class/fqn", "MailAgentService");
  info->replaceAll("factory", "mailagentservice_factory");
  
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  Q_CHECK_PTR( vfs );
  Q_CHECK_PTR( clientRMI );
}


MailAgentService::~MailAgentService()
{
}

QString MailAgentService::appendToFolder
  (
    const QString & folder,
    const QString & account,
    const QString & from,
    const QString & toList,
    const QString & ccList,
    const QString & bccList,
    const QString & subject,
    const QString & body,
    const QString & bodyMIME,
    const QString & headerExtra,
    const QString & encAttach,
    const QString & attachMIME
  )
{
  QString url =
    formEmail(account, from, toList, ccList, bccList, subject, body, bodyMIME, headerExtra, encAttach, attachMIME);
  
  if( !url.isEmpty() )
  {
    url = vfs->move(url, folder);
  }
  
  return url;
}

void MailAgentService::send
  (
    const QString & account,
    const QString & from,
    const QString & toList,
    const QString & ccList,
    const QString & bccList,
    const QString & subject,
    const QString & body,
    const QString & bodyMIME,
    const QString & headerExtra,
    const QString & encAttach,
    const QString & attachMIME
  )
{
  QString url =
    formEmail(account, from, toList, ccList, bccList, subject, body, bodyMIME, headerExtra, encAttach, attachMIME);
  
  if( !url.isEmpty() )
  {
    //move the message to outbox then send it then delete the local mail
    url = vfs->move( url, vfs->root() + "/Mail/Outbox" );
  }
  
  vfs->send( url );
}


QString MailAgentService::sendLater
  (
    const QString & account,
    const QString & from,
    const QString & toList,
    const QString & ccList,
    const QString & bccList,
    const QString & subject,
    const QString & body,
    const QString & bodyMIME,
    const QString & headerExtra,
    const QString & encAttach,
    const QString & attachMIME
  )
{
  QString url =
    formEmail(account, from, toList, ccList, bccList, subject, body, bodyMIME, headerExtra, encAttach, attachMIME);
  
  if( !url.isEmpty() )
  {
    // move the message to outbox
    url = vfs->move( url, vfs->root() + "/Mail/Outbox" );
  }
  
  return url;
}

QString MailAgentService::formEmail
  (
    const QString & account,
    const QString & from,
    const QString & toList,
    const QString & ccList,
    const QString & bccList,
    const QString & subject,
    const QString & body,
    const QString & bodyMIME,
    const QString & headerExtra,
    const QString & encAttach,
    const QString & attachMIME
  )
{
  QString url;
  QString accountName;
  
  if( account.isEmpty() )
  {
    clientRMI->loadAccounts();
    accountName = clientRMI->getDefaultAccount();
  }
  else
  {
    accountName = account;
  }
  
  //create the mail in the draft folder first
  url = vfs->createObject(vfs->root()+"/Mail/Drafts");
  MailObject *obj = static_cast<MailObject*>
    ( vfs->_fact_ref(url, VFS_MAILS_MIMETYPE) );
  
  if(obj)
  {
    AddressClass fromAdd(from);
    AddressListClass toAdd(toList, true);
    
    obj->setAccount(accountName);
    obj->setFrom(fromAdd);
    obj->setTo(toAdd);
    
    if( !ccList.isEmpty() )
    {
      AddressListClass ccAdd(ccList, true);
      obj->setCc(ccAdd);
    }
    
    if( !bccList.isEmpty() )
    {
      AddressListClass bccAdd(bccList, true);
      obj->setCc(bccAdd);
    }
    
    if( !headerExtra.isEmpty() )
    {
      obj->setExtraInfo( QString("HEADER_EXTRA ") + headerExtra );
    }
    
    obj->setSubject( subject );
    obj->setStatus("Read");
    obj->save();
    
    int textPart = obj->messageText();
    QString partID;
  
    if( textPart == -1 )
    {
      // no text part exists, create one
      partID = vfs->createObject(url);
    }
    else
    {
      partID = url + ATTR_SEP + QString::number(textPart);
    }
  
    VPartAttribute *attr=(VPartAttribute *)vfs->_fact_ref(partID+ATTR_SEP+"attr", VFS_PARTATTRIBUTE_MIMETYPE );
    if(attr)
    {
      attr->sync();
     
      // Set the sending type
      QString sendingType = Settings::instance()->readEntry("/Aethera/Composer/SendingType", "AUTO");
      if( "HTML" == sendingType )
      {
        // todo
      }
      else if( "ALTERNATIVE" == sendingType )
      {
        // todo
      }
      else if( "AUTO" == sendingType )
      {
        // todo
      }
      else // only TEXT
      {
        // todo
      }
      
      if( bodyMIME.isEmpty() )
        attr->mimetype = "text/plain";
      else
        attr->mimetype = bodyMIME;
      
      if( "text/plain" == attr->mimetype.lower() ||
          "text/html" == attr->mimetype.lower() )
      {
        attr->mtype = "text";
      }
      else
      {
        attr->mtype = "attachment";
      }
      
      QCString _text;
      if( body.isEmpty() )
      {
        attr->charset = "us-ascii";
        attr->encoding = "7bit";
        _text = " ";
      }
      else
      {
        QTextCodec * codec = 0L;
        if( body != QString::fromUtf8(body.utf8()) &&
            (codec = CharSets::thisInstance()->getCodec(body)) )
        {
          attr->charset = codec->mimeName();
          attr->encoding = "base64";
          _text = codec->fromUnicode( body );
          _text = MIMECodec::encodeBase64(_text);
        }
        else
        {
          if( "System" != CharSets::thisInstance()->getDefaultCharset() &&
              body == QString::fromLocal8Bit( body.local8Bit() ) )
          {
            attr->charset = CharSets::thisInstance()->getDefaultCharset();
            attr->encoding = "8bit";
            _text = body.local8Bit();
          }
          else
          {
            attr->charset = "utf-8";
            attr->encoding = "7bit";
            _text = body.utf8();
          }
        }
      }
      
      attr->save();
      
      vfs->setObjectData(partID, _text);
    }

//  set the attachment of the message
//  if( attachment != QString::null )
//  {
//    QString partID = vfs->createObject(url);
//  
//    PartObject *part=(PartObject *)vfs->_fact_ref(partID, VFS_PART_MIMETYPE );
//   
//    if(part)
//    {
//      part->setFilename( "cal.ics" );
//      part->setPartType("attachment");
//      part->setMimeType( "text/calendar" );
//      part->setEncoding( "7bit" );
//      
//      part->save();
//       
//      //set the data of the attachment 
//      QCString data( attachment );
//      vfs->setObjectData(partID, data);
//       
//      delete part;
//    }
//  }
    
    return obj->path();
  }
  
  return QString::null;
}


void MailAgentService::deleteMail( const QString& url )
{
  vfs->remove( url );
}


void MailAgentService::openComposer( )
{
  ComposerFactory::instance()->openComposer();
}


void MailAgentService::openComposer(const QString & vfsUrl, ComposerFactory::Mode mode)
{
  ComposerFactory::instance()->openComposer( vfsUrl, mode  );
}


void MailAgentService::openComposer(const QStringList & vfsUrlList, ComposerFactory::Mode mode)
{
  ComposerFactory::instance()->openComposer( vfsUrlList, mode  );  
}

