/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
  Date     : 10 August 2003
  Author   : Ghiciuc Ilie (ilie@eco-spaces.com.ro)
---------------------------------------------------------------------------------
*/

#ifndef MAIL_AGENT_SERVICE
#define MAIL_AGENT_SERVICE

#define MAILAGENT_SERVICE "/Mail/Services/MailAgent"

//own includes
#include "mailobj.h"
#include "composerfactory.h"

//Aethera includes
#include <komaetheraplugin.h>
#include <clientrmi.h>

extern "C"
{
    MAIL_PLUGIN_API kore::Interface* mailagentservice_factory();
};

class MailAgentService : public KomAetheraService
{
  public:
    MailAgentService();
    virtual ~MailAgentService();
    
    /*
      This method is used for sending KOPlugin sync mails ... so it is the only thing that
      it supports.
    */
    virtual void send(
      const QString & account,
      const QString & from,
      const QString & toList,
      const QString & ccList,
      const QString & bccList,
      const QString & subject,
      const QString & body,
      const QString & bodyMIME = QString::null,
      const QString & headerExtra = QString::null,
      const QString & encAttach = QString::null,
      const QString & attachMIME = QString::null );
    
    /*
      create an email with from, to, subject... provided in the folder indicated.
      Return the path of the email.
    */
    virtual QString sendLater(
      const QString & account,
      const QString & from,
      const QString & toList,
      const QString & ccList,
      const QString & bccList,
      const QString & subject,
      const QString & body,
      const QString & bodyMIME = QString::null,
      const QString & headerExtra = QString::null,
      const QString & encAttach = QString::null,
      const QString & attachMIME = QString::null );
    
    /** Append a new email to an IMAP or LOCAL folder.*/
    virtual QString appendToFolder(
      const QString & folder,
      const QString & account,
      const QString & from,
      const QString & toList,
      const QString & ccList,
      const QString & bccList,
      const QString & subject,
      const QString & body,
      const QString & bodyMIME = QString::null,
      const QString & headerExtra = QString::null,
      const QString & encAttach = QString::null,
      const QString & attachMIME = QString::null );
        
    //send the full url path of the mail to this method
    virtual void deleteMail( const QString& url );
    
    //open a composer window
    virtual void openComposer( );
    virtual void openComposer(const QString & vfsUrl, ComposerFactory::Mode);
    virtual void openComposer(const QStringList & vfsUrlList, ComposerFactory::Mode);
              
  private:
    
    // TODO: extend it for more attachments
    
    /** It builds a new email in the Draft folder.*/
    QString formEmail(
      const QString & account,
      const QString & from,
      const QString & toList,
      const QString & ccList,
      const QString & bccList,
      const QString & subject,
      const QString & body,
      const QString & bodyMIME = QString::null,
      const QString & headerExtra = QString::null,
      const QString & encAttach = QString::null,
      const QString & attachMIME = QString::null );
      
    VFSModule * vfs;
    ClientRMIModule * clientRMI;
};

#endif

