/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/** 
  * Changed by Eugen C. for using KURLLabel and all associated code.
  * Changed by Fabien Armand : links with the composer window
  */

#include <qiconloader.h>

#include <qcstring.h>
#include <qstring.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qcursor.h>
#include <qaction.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <maillabel.h>
#include <conversions.h>
#include <mailobj.h>
#include <mailview.h>
#include <vfsmimetype.h>

// contacts
#include <contactsfactory.h>
#include <composerfactory.h>

MailLabel::MailLabel(VFSModule *vfsModule, MailView *parent, const char *name)
  : QWidgetStack(parent, name)
{
  mailLabel = new MailLabelWidget(vfsModule, this, "MailLabelWidget");
  nothingLabel = new QLabel(this, "NothingLabel");
  
  if( mailLabel && nothingLabel )
  {
    // hide the attachment panel from mail view widget
    connect(mailLabel, SIGNAL(hideAttachmentPane()),
            parent,    SLOT(hideAttachmentPane()));
    connect(mailLabel->attButton, SIGNAL(clicked()),
            parent,               SLOT(togglePane()));

    nothingLabel->setText("No item selected");
    nothingLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    
    addWidget(mailLabel);
    addWidget(nothingLabel);
    
    raiseWidget(nothingLabel);
  }
  else
  {
    qDebug("MailLabel error: cannot build the widgets.");
  }
}

MailLabel::~MailLabel()
{
}

QString MailLabel::getAddress() const
{
  return mailLabel->getAddress();
}

void MailLabel::setAddress(const QString & newAddress)
{
  if( mailLabel->setAddress(newAddress) )
  {
    raiseWidget(mailLabel);
  }
  else
  {
    raiseWidget(nothingLabel);
  }
}

void MailLabel::showDefault()
{
  raiseWidget(nothingLabel);
  mailLabel->setAddress(QString::null);
}

// ---- MailLabelWidget

MailLabelWidget::MailLabelWidget(VFSModule *vfsModule, QWidget *parent, const char *name)
  : MailLabelBase(parent, name),
    vfs( vfsModule )
{
  attButton->setIconSet( QIconLoader::instance()->iconSet("attach-normal.xpm") );
  
  // Popup menu
  fromMenu = new QPopupMenu(this, "FromMenu");
  subjectMenu = new QPopupMenu(this, "SubjectMenu");
  toMenu = new QPopupMenu(this, "ToMenu");
  ccMenu = new QPopupMenu(this, "CCMenu");
  
  // Actions for popup menu
  QAction *action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotFromCopy()));
  action->addTo( fromMenu );
  
  action = new QAction( QObject::tr("Add to addressbook."),
                        QObject::tr("&Add to addressbook."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotFromAdd()));
  action->addTo( fromMenu );
  
  action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotToCopy()));
  action->addTo( toMenu );
  
  action = new QAction( QObject::tr("Add to addressbook."),
                        QObject::tr("&Add to addressbook."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotToAdd()));
  action->addTo( toMenu );
  
  action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotCcCopy()));
  action->addTo( ccMenu );
  
  action = new QAction( QObject::tr("Add to addressbook."),
                        QObject::tr("&Add to addressbook."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotCcAdd()));
  action->addTo( ccMenu );
  
  action = new QAction( QObject::tr("Copy to clipboard."),
                        QObject::tr("&Copy to clipboard."),
                        QKeySequence(),
                        this );
  connect( action, SIGNAL(activated()), this, SLOT(slotSubjectCopy()));
  action->addTo( subjectMenu );
  
  connect(FromLabel, SIGNAL(leftClickedURL()), this, SLOT(slotFromLeftClicked()));
  connect(FromLabel, SIGNAL(rightClickedURL()), this, SLOT(slotFromRightClicked()));
  connect(CcLabel, SIGNAL(leftClickedURL()), this, SLOT(slotCcLeftClicked()));
  connect(CcLabel, SIGNAL(rightClickedURL()), this, SLOT(slotCcRightClicked()));
  connect(ToLabel, SIGNAL(leftClickedURL()), this, SLOT(slotToLeftClicked()));
  connect(ToLabel, SIGNAL(rightClickedURL()), this, SLOT(slotToRightClicked()));
  connect(SubjectLabel, SIGNAL(leftClickedURL()), this, SLOT(slotSubjectLeftClicked()));
  connect(SubjectLabel, SIGNAL(rightClickedURL()), this, SLOT(slotSubjectRightClicked()));
}

bool MailLabelWidget::setAddress(const QString & addr)
{
  address=addr;
  if(addr.isEmpty())
  {
    return false;
  }

  int attachments = 0;
  MailObject *item = (MailObject *)vfs->_fact_ref(address, VFS_MAILS_MIMETYPE);

  if(item)
  {
    FromLabel->setText(item->from().fullAddress() );
    FromLabel->setTipText(item->from().fullAddress() );
    FromLabel->setUseTips(true);

    ToLabel->setText( item->to().returnString() );
    ToLabel->setTipText( item->to().returnString() );
    ToLabel->setUseTips(true);
    SubjectLabel->setText(item->subject());
    SubjectLabel->setTipText(item->subject());
    SubjectLabel->setUseTips(true);
    
    if( item->cc().returnString().isEmpty() )
    {
      CcText->hide();
      CcLabel->hide();
    }
    else
    {
      CcLabel->setText( item->cc().returnString() );
      CcLabel->setTipText( item->cc().returnString() );
      CcLabel->setUseTips(true);
      CcText->show();
      CcLabel->show();
    }
    
    attachments=item->attachments();
    if(attachments)
      attButton->show();
    else
      attButton->hide();
    emit hideAttachmentPanel();
  }
  else
  {
    attButton->hide();
    emit hideAttachmentPanel();
    return false;
  }

  return true;
}

QString MailLabelWidget::getAddress() const
{
  return address;
}

void MailLabelWidget::slotFromLeftClicked()
{
  ComposerFactory::instance()->openComposer
  ( FromLabel->text(), ComposerFactory::WriteToContact);
}

void MailLabelWidget::slotFromRightClicked()
{
  if(fromMenu)
    fromMenu->popup(QCursor::pos());
}

void MailLabelWidget::slotCcLeftClicked()
{
  ComposerFactory::instance()->openComposer
  ( CcLabel->text(), ComposerFactory::WriteToContact);
}

void MailLabelWidget::slotCcRightClicked()
{
  if(ccMenu)
    ccMenu->popup(QCursor::pos());
}

void MailLabelWidget::slotToLeftClicked()
{
  ComposerFactory::instance()->openComposer
  ( ToLabel->text(), ComposerFactory::WriteToContact);
}

void MailLabelWidget::slotToRightClicked()
{
  if(toMenu)
    toMenu->popup(QCursor::pos());
}

void MailLabelWidget::slotSubjectLeftClicked()
{
}

void MailLabelWidget::slotSubjectRightClicked()
{
  if(subjectMenu)
    subjectMenu->popup(QCursor::pos());

}

void MailLabelWidget::slotFromCopy()
{
  
//  qDebug("slotFromCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(FromLabel->text());
  cb->setSelectionMode(false);
  cb->setText(FromLabel->text());
}

void MailLabelWidget::slotCcCopy()
{
  
//  qDebug("slotCcCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(CcLabel->text());
  cb->setSelectionMode(false);
  cb->setText(CcLabel->text());
}

void MailLabelWidget::slotToCopy()
{
  
//  qDebug("slotToCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(ToLabel->text());
  cb->setSelectionMode(false);
  cb->setText(ToLabel->text());
}

void MailLabelWidget::slotSubjectCopy()
{
  
//  qDebug("slotSubjectCopy");
  
  QClipboard *cb = QApplication::clipboard();
  
  cb->setSelectionMode(true);
  cb->setText(SubjectLabel->text());
  cb->setSelectionMode(false);
  cb->setText(SubjectLabel->text());
}

void MailLabelWidget::slotFromAdd()
{
  addToAddressBook( FromLabel->text() );
}

void MailLabelWidget::slotCcAdd()
{
  addToAddressBook( CcLabel->text() );
}

void MailLabelWidget::slotToAdd()
{
  addToAddressBook( ToLabel->text() );
}

void MailLabelWidget::addToAddressBook(const QString & line)
{
  QStringList addresses = QStringList::split(",", line);
  
  ContactsFactory * contactsFactory = 0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_CONTACTS_FACTORY );
    if( ref.isValid() )
      contactsFactory = static_cast<ContactsFactory*>( ref.instance() );
  }  
  
  if( !contactsFactory )
  {
    qDebug("MailLabelWidget: Can't find the Contacts Factory");
    return;
  }
  
  VContactObject * contact = 0L;
  for(QStringList::ConstIterator it(addresses.begin()); it!=addresses.end(); ++it)
  {
    contact = contactsFactory->createContact(*it);
    if( !contact )
    {
      qDebug("MailLabelWidget: Couldn't create the contact object.");
      return;
    }
  }
}
