/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "mailobj.h"

#include <cassert>

#include <conversions.h>
#include <vcacheobject.h>
#include <vfolder.h>
#include <vmailattribute.h>
#include <mimecodec.h>

#include <qcstring.h>
#include <qmap.h>
#include <qregexp.h>

MailObject::MailObject(const QString & _path)
  : VObject(_path),
    _flag(0),
    _requestReceipt(0),
    _persistence(0),
    _attachments(0),
    _size(0)
{
  sync();
}

MailObject::MailObject(const QString & _path, const QByteArray & _data)
  : VObject(_path),
    _flag(0),
    _requestReceipt(0),
    _persistence(0),
    _attachments(0),
    _size(0)
{
  if( !_data || !_data.size() )
    sync();
  else
    _narrow( _data);
}

MailObject::~MailObject()
{
}

bool MailObject::sync()
{
	VCacheObject *cacheObj=parentFolder()->entryCacheBlock();
	
	if(cacheObj)
	{
		QByteArray cacheData=cacheObj->cacheData();
		QDataStream cstream(cacheData, IO_ReadOnly);
		QMap<QString, QByteArray> cmap;
		cstream>>cmap;
		QByteArray objCache=cmap[name()];
		QDataStream dstream(objCache, IO_ReadOnly);
		
		Q_UINT8 red, green, blue;
		QString strSentDate, strReceivedDate, strFrom;
		QStringList strTo;
		dstream>>_status>>_extendedStatus>>_flag>>_attachments>>red>>green>>blue;
		dstream>>strFrom>>strTo>>_subject>>strSentDate>>strReceivedDate>>_scheduling>>_priority>>_parentIndexID>>_size>>_contentType;

		_color=QColor(red, green, blue);
		_sentDate=DateClass((const char *)strSentDate);
		_receivedDate=DateClass((const char *)strReceivedDate);
    _from = AddressClass(strFrom);
		_to=AddressListClass( strTo.join(", ") );
    QString sub=_subject;
		__cached=true;
		
		return true;
	}
	else
	{
		__cached=false;
		
		return VObject::sync();
	}
}

void MailObject::_narrow(const QByteArray &descriptor)
{
	__cached=false;
		
	if(descriptor.size())
	{
		QDataStream stream(descriptor, IO_ReadOnly);
		QString strFrom, strSentDate, strReceivedDate, strReplyTo;
		QStringList strCc, strBcc, strTo;
		Q_UINT8 red, green, blue;

		stream>>strFrom;
		stream>>strTo;
		stream>>_subject;
		stream>>_status;
		stream>>_priority;
		stream>>_contentType;
		stream>>strSentDate;
		stream>>strReceivedDate;
		stream>>_scheduling;
		stream>>_flags;
		stream>>red>>green>>blue;
		stream>>_flag;
		stream>>_inReplyTo;
		stream>>_references;
		stream>>_indexID;
		stream>>_parentIndexID;
		stream>>_relatedIndexIDList;
		stream>>strReplyTo;
		stream>>strCc;
		stream>>strBcc;
		stream>>_messageID;
		stream>>_organization;
		stream>>_uid;
		stream>>_extendedStatus;
		stream>>_contentEncoding;
		stream>>_account;
		stream>>_route;
		stream>>_requestReceipt;
		stream>>_extraInfo;
		stream>>_category;
		stream>>_persistence;
		stream>>_extraData;

		stream>>_attachments;
		stream>>_size;

		// build client-specific data structures
    AddressClass addr(strFrom);
		_from = addr ;
		_to=AddressListClass( strTo.join(", "), true );
		_sentDate=DateClass((const char *)strSentDate);
		_receivedDate=DateClass((const char *)strReceivedDate);
		_color=QColor(red, green, blue);

		_replyTo=AddressClass( strReplyTo );
		_cc=AddressListClass( strCc.join(", "), true );
		_bcc=AddressListClass( strBcc.join(", "), true );
    
    }
	else
	{
		_subject="no data";
	}
}


QByteArray MailObject::_stream()
{
	QByteArray descriptor;
	QDataStream stream(descriptor, IO_WriteOnly);
  QStringList greper;

	stream<<_from.fullAddress();
	stream<< _to.returnNameStringList();
	stream<<_subject;
	stream<<_status;
	stream<<_priority;
	stream<<_contentType;
	stream<<QString(_sentDate.toQCString());
	stream<<QString(_receivedDate.toQCString());
	stream<<_scheduling;
	stream<<_flags;
	stream<<(Q_UINT8)_color.red()<<(Q_UINT8)_color.green()<<(Q_UINT8)_color.blue();
	stream<<_flag;
	stream<<_inReplyTo;
	stream<<_references;
	stream<<_indexID;
	stream<<_parentIndexID;
	stream<<_relatedIndexIDList;

	stream<<_replyTo.fullAddress();
	stream<< _cc.returnNameStringList();
	stream<< _bcc.returnNameStringList();
	stream<<_messageID;
	stream<<_organization;
	stream<<_uid;
	stream<<_extendedStatus;
	stream<<_contentEncoding;
	stream<<_account;
	stream<<_route;
	stream<<_requestReceipt;
	stream<<_extraInfo;
	stream<<_category;
  stream<<_persistence;
	stream<<_extraData;

	stream<<_attachments;
	stream<<_size;
	
	return descriptor;
}

AddressClass MailObject::from() const
{
	return _from;
}

void MailObject::setFrom(const AddressClass &newFrom)
{
	_from=newFrom;
}

AddressListClass MailObject::to() const
{
	return _to;
}

void MailObject::setTo(const AddressListClass &newTo)
{
  _to = newTo;
}

QString MailObject::subject() const
{
  return _subject;
}

void MailObject::setSubject(const QString & newSubject)
{
  _subject = newSubject;
}

QString MailObject::status() const
{
  if( !_status || _status.isEmpty() )
    return QString("Read");
  
  return _status;
}

void MailObject::setStatus(const QString &newStatus)
{
	_status=newStatus;
}

QString MailObject::priority() const
{
	return _priority;
}

void MailObject::setPriority(const QString &newPriority)
{
	_priority=newPriority;
}

DateClass MailObject::sentDate() const
{
	return _sentDate;
}

void MailObject::setSentDate(const DateClass &newSentDate)
{
	_sentDate=newSentDate;
}

DateClass MailObject::receivedDate() const
{
	return _receivedDate;
}

void MailObject::setReceivedDate(const DateClass &newReceivedDate)
{
	_receivedDate=newReceivedDate;
}

QString MailObject::scheduling() const
{
	return _scheduling;
}

void MailObject::setScheduling(const QString &newScheduling)
{
	_scheduling=newScheduling;
}

QStringList MailObject::flags() const
{
	return _flags;
}

void MailObject::setFlags(const QStringList &newFlags)
{
	_flags=newFlags;
}

QColor MailObject::color() const
{
	return _color;
}

void MailObject::setColor(const QColor &newColor)
{
	_color=newColor;
}

bool MailObject::flag() const
{
	return _flag;
}

void MailObject::setFlag(bool newFlag)
{
	_flag=newFlag;
}

QString MailObject::inReplyTo() const
{
  return _inReplyTo;
}

void MailObject::setInReplyTo(const QString &newInReplyTo)
{
  _inReplyTo=newInReplyTo;
}

QStringList MailObject::getReferences() const
{
	return _references;
}

void MailObject::setReferences(const QStringList &newReferences)
{
	_references=newReferences;
}

QString MailObject::indexID() const
{
	return _indexID;
}

void MailObject::setIndexID(const QString &newIndexID)
{
	_indexID=newIndexID;
}

QString MailObject::parentIndexID() const
{
	return _parentIndexID;
}

void MailObject::setParentIndexID(const QString &newParentIndexID)
{
	_parentIndexID=newParentIndexID;
}

QStringList MailObject::relatedIndexIDList() const
{
	return _relatedIndexIDList;
}

void MailObject::setRelatedIndexIDList(const QString &newRelatedIndexIDList)
{
	_relatedIndexIDList=newRelatedIndexIDList;
}

AddressClass MailObject::replyTo() const
{
	return _replyTo;
}

void MailObject::setReplyTo(const AddressClass &newReplyTo)
{
	_replyTo=newReplyTo;
}

AddressListClass MailObject::cc() const
{
	return _cc;
}

void MailObject::setCc(const AddressListClass &newCc)
{
	_cc=newCc;
}

AddressListClass MailObject::bcc() const
{
	return _bcc;
}

void MailObject::setBcc(AddressListClass &newBcc)
{
	_bcc=newBcc;
}

QString MailObject::messageID() const
{
	return _messageID;
}

void MailObject::setMessageID(const QString &newMessageID)
{
  _messageID=newMessageID;
}

QString MailObject::organization() const
{
	return _organization;
}

void MailObject::setOrganization(const QString &newOrganization)
{
	_organization=newOrganization;
}

QString MailObject::uid() const
{
	return _uid;
}

void MailObject::setUid(const QString &newUid)
{
	_uid=newUid;
}

QStringList MailObject::extendedStatus() const
{
	return _extendedStatus;
}

void MailObject::setExtendedStatus(const QStringList &newExtendedStatus)
{
	_extendedStatus=newExtendedStatus;
}

QString MailObject::contentType() const
{
	return _contentType;
}

void MailObject::setContentType(const QString &newContentType)
{
	_contentType=newContentType;
}

QString MailObject::contentEncoding() const
{
	return _contentEncoding;
}

void MailObject::setContentEncoding(const QString &newContentEncoding)
{
	_contentEncoding=newContentEncoding;
}

QString MailObject::account() const
{
	return _account;
}

void MailObject::setAccount(const QString &newAccount)
{
	_account=newAccount;
}

QStringList MailObject::route() const
{
	return _route;
}

void MailObject::setRoute(const QStringList &newRoute)
{
	_route=newRoute;
}

bool MailObject::requestReceipt() const
{
	return _requestReceipt;
}

void MailObject::setRequestReceipt(bool newRequestReceipt)
{
	_requestReceipt=newRequestReceipt;
}

QString MailObject::extraInfo() const
{
  return _extraInfo;
}

void MailObject::setExtraInfo(const QString & newInfo)
{
  _extraInfo = newInfo;
}

QString MailObject::category() const
{
	return _category;
}

void MailObject::setCategory(const QString &newCategory)
{
	_category=newCategory;
}

int MailObject::persistence() const
{
	return _persistence;
}

void MailObject::setPersistence(int newPersistence)
{
	_persistence=newPersistence;
}

// non-resident data

unsigned long MailObject::size() const
{
  return _size;
}

bool MailObject::attachments() const
{
  return _attachments;
}

QCString MailObject::message() const
{
  return QCString( vfsModule()->getSimpleData(path() + "%rfc822message") );
}

QCString MailObject::header() const
{
  return QCString( vfsModule()->getSimpleData(path() + "%rfc822header") );
}

bool MailObject::hasHTML() const
{
  return _narrowBool( vfsModule()->getSimpleData(path() + "%hashtml") );
}

bool MailObject::hasText() const
{
  return _narrowBool( vfsModule()->getSimpleData(path() + "%hastext") );
}

QString MailObject::text() const
{
  QByteArray data;
  
  if( vfsModule()->getObjectData(path() + "%text", data) )
    return _narrowQString( data );
  else
    return "";
}

QString MailObject::html() const
{
  QByteArray data;
  
  if( vfsModule()->getObjectData(path() + "%html", data) )
    return _narrowQString( data );
  else
    return "";
}

bool MailObject::textPart() const
{
  return _narrowBool( vfsModule()->getSimpleData(path() + "%textpart") );
}

int MailObject::messageText() const
{
  return _narrowInt( vfsModule()->getSimpleData(path() + "%messagetext") );
}

bool MailObject::htmlPart() const
{
  return _narrowBool( vfsModule()->getSimpleData(path() + "%htmlpart") );
}

bool MailObject::isReplied() const
{
	return _extendedStatus.grep(QRegExp("^replied:")).count();
}

bool MailObject::isForwarded() const
{
	return _extendedStatus.grep(QRegExp("^forwarded:")).count();
}

bool MailObject::setProperty(const QString&, Any&)
{
	return false;
}

bool MailObject::getProperty(const QString&, Any&)
{
	return false;
}

QueryProcessor::QueryType MailObject::queryType(const QString &)
{
	return QueryProcessor::InvalidQuery;
}

#ifdef _QTWIN_
VObject *MailObjectFactory::_new(const QString & _path)
#else
MailObject *MailObjectFactory::_new(const QString & _path)
#endif
{
  if( vfs()->canDefine(_path) )
    return new MailObject(_path);
  else
    return 0;
}

#ifdef _QTWIN_
VObject *MailObjectFactory::_new
#else
MailObject *MailObjectFactory::_new
#endif
(
  const QString     & _path,
  const QByteArray  & _data
)
{
  return new MailObject(_path, _data);
}


