/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#ifndef __MailObject__
#define __MailObject__

#include <vobject.h>
#include <vobjectfactory.h>
#include <dateclass.h>
#include <addressclass.h>
#include <addresslistclass.h>

#include <qstring.h>
#include <qcolor.h>

class MAIL_PLUGIN_API MailObject : public VObject
{
  friend class MailObjectFactory;
  
  protected:
  
    MailObject(const QString & _path, const QByteArray & _data);
    MailObject(const QString & _path);
    
  public:
    
    virtual ~MailObject();
    
    /**
     * properties
     */
    virtual AddressClass from() const;
    virtual void setFrom(const AddressClass &newFrom);

    virtual AddressListClass to() const;
    virtual void setTo(const AddressListClass &newTo);

    virtual QString subject() const;
    virtual void setSubject(const QString &newSubject);

    virtual QString status() const;
    virtual void setStatus(const QString &newStatus);

    virtual QString priority() const;
    virtual void setPriority(const QString &newPriority);

    virtual DateClass sentDate() const;
    virtual void setSentDate(const DateClass &newSentDate);

    virtual DateClass receivedDate() const;
    virtual void setReceivedDate(const DateClass &newReceivedDate);

    virtual QString scheduling() const;
    virtual void setScheduling(const QString &newScheduling);

    virtual QStringList flags() const;
    virtual void setFlags(const QStringList &newFlags);

    virtual QColor color() const;
    virtual void setColor(const QColor &newColor);

    virtual bool flag() const;
    virtual void setFlag(bool newFlag);

    virtual QString inReplyTo() const;
    virtual void setInReplyTo(const QString &newInReplyTo);

    virtual QStringList getReferences() const;
    virtual void setReferences(const QStringList &newReferences);
    
    virtual QString indexID() const;
    virtual void setIndexID(const QString &newIndexID);

    virtual QString parentIndexID() const;
    virtual void setParentIndexID(const QString &newParentIndexID);

    virtual QStringList relatedIndexIDList() const;
    virtual void setRelatedIndexIDList(const QString &newRelatedIndexIDList);

    virtual AddressClass replyTo() const;
    virtual void setReplyTo(const AddressClass &newReplyTo);

    virtual AddressListClass cc() const;
    virtual void setCc(const AddressListClass &newCc);

    virtual AddressListClass bcc() const;
    virtual void setBcc(AddressListClass &newBcc);

    virtual QString messageID() const;
    virtual void setMessageID(const QString &newMessageID);

    virtual QString organization() const;
    virtual void setOrganization(const QString &newOrganization);

    virtual QString uid() const;
    virtual void setUid(const QString &newUid);

    virtual QStringList extendedStatus() const;
    virtual void setExtendedStatus(const QStringList &newExtendedStatus);

    virtual QString contentType() const;
    virtual void setContentType(const QString &newContentType);

    virtual QString contentEncoding() const;
    virtual void setContentEncoding(const QString &newContentEncoding);

    virtual QString account() const;
    virtual void setAccount(const QString &newAccount);

    virtual QStringList route() const;
    virtual void setRoute(const QStringList &newRoute);

    virtual bool requestReceipt() const;
    virtual void setRequestReceipt(bool newRequestReceipt);

    virtual QString extraInfo() const;
    virtual void setExtraInfo(const QString&);

    virtual QString category() const;
    virtual void setCategory(const QString &newCategory);

    virtual int persistence() const;
    virtual void setPersistence(int newPersistence);

    // volatile data
    virtual bool attachments() const;
    virtual unsigned long size() const;

    // server-query data
    virtual QCString message() const;
    virtual QCString header() const;
    virtual bool hasHTML() const;
    virtual bool hasText() const;
    
    virtual QString text() const;
    virtual QString html() const;
    
    virtual bool textPart() const;
    virtual int messageText() const;
    virtual bool htmlPart() const;

    // reply and forward features
    virtual bool isReplied() const;
    virtual bool isForwarded() const;

    // this is a special kind of object, and we overload sync() from VObject
    virtual bool sync();

    // save/load VObject methods
    virtual QByteArray _stream();
    virtual void _narrow(const QByteArray &);

    /**
     * query handling
     */
    virtual bool getProperty(const QString &query, Any &property);
    virtual bool setProperty(const QString &query, Any &property);
    virtual QueryProcessor::QueryType queryType(const QString &query);
  
  private:
    // core
    AddressClass _from;
    AddressListClass _to;
    QString _subject;
    QString _status;
    QString _priority;
    QString _contentType;
    DateClass _sentDate;
    DateClass _receivedDate;
    QString _scheduling;
    QStringList _flags;
    QColor _color;
    Q_UINT8 _flag;
    QString _indexID;
    QString _parentIndexID;
    QStringList _relatedIndexIDList;
    QString _inReplyTo;
    QStringList _references;

    
    // other
    AddressClass _replyTo;
    AddressListClass _cc;
    AddressListClass _bcc;
    QString _messageID;
    QString _organization;
    QString _uid;
    QStringList _extendedStatus;
    QString _contentEncoding;
    QString _account;
    QStringList _route;
    Q_UINT8 _requestReceipt;
    QString _extraInfo;
    QString _category;
    Q_UINT32 _persistence;

    /** If it has attachments. */
    Q_UINT8 _attachments;
    /** The message size.*/
    Q_UINT32 _size;

    QByteArray _extraData;

    bool __cached;
};

/** The MailObject Interface.*/
class MAIL_PLUGIN_API MailObjectInterface : public MailObject
{
  protected:
    MailObjectInterface(const QString&, const QByteArray&);
    MailObjectInterface(const QString&);
    
  public:
    virtual int objectInterface() const = 0;
};

/** The mail object factory. */
class MAIL_PLUGIN_API MailObjectFactory : public VObjectFactory
{
  public:
    /** Build a new mail object.*/
#ifdef _QTWIN_
    VObject *_new(const QString & _path, const QByteArray & _data);
    VObject *_new(const QString & _path);
#else
    MailObject *_new(const QString & _path, const QByteArray & _data);
    MailObject *_new(const QString & _path);
#endif
};

#endif







