/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
        mailuipartprivate.cpp  -  the QT handler from the Mail UI service
                             -------------------
    begin                : Thu Oct 31 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/

#include <mailuipartprivate.h>
#include <kommailuiservice.h>
#include <komaetheratools.h>
#include <composerfactory.h>
#include <clientrmi.h>
#include <filter.h>
#include <mailfilter_plugin.h>
#include <mailimportservice.h>
#include <folderchooserdialog.h>

#include <qpopupmenu.h>
#include <qptrlist.h>
#include <qmainwindow.h>

MailUIPartPrivate::MailUIPartPrivate(UIPart* part)
  : KomUIPartPrivate(part),
    currentOperation( NOTHING )
{
  connect(&timer, SIGNAL(timeout()), this, SLOT(delayedSendReceive()));
}

MailUIPartPrivate::~MailUIPartPrivate()
{
}

void MailUIPartPrivate::startTimer()
{
  timer.start(100, true);
}

void MailUIPartPrivate::slotNewMail()
{
  ComposerFactory::instance()->openComposer();
}

void MailUIPartPrivate::slot_settings()
{
  DEBUG_MSG("slot_settings");
  emit(showSettingsDialog());
}

  void
MailUIPartPrivate::slotCurrentURLChanged(const QString & url)
{
  static_cast<KomMailUIService *>(owner)->setCurrentURL(url);
}

void MailUIPartPrivate::slotSendReceive()
{
  currentAccount = QString::null;
  currentOperation = SEND_RECEIVE;
  startTimer();
}

void MailUIPartPrivate::delayedSendReceive()
{
  ClientRMIModule *clientRMI = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  assert(0 != clientRMI);
  
  if( !accounts.count() )
  {
    accounts = clientRMI->getAccountNames();
  }
  
  switch( currentOperation )
  {
    case SEND_RECEIVE :
      {
        if( currentAccount.isEmpty() )
        {
          for (unsigned int i = 0; i < accounts.count(); i++)
            if(clientRMI->isAccountActive(accounts[i]))
              clientRMI->startReceive(accounts[i]);
          
          clientRMI->startSend( clientRMI->getDefaultAccount() );
        }
        else
        {
          clientRMI->startSendReceive(currentAccount);
        }
      }
      break;
    
    case RECEIVE :
      {
        if( currentAccount.isEmpty() )
        {
          for (unsigned int i = 0; i < accounts.count(); i++)
            if(clientRMI->isAccountActive(accounts[i]))
              clientRMI->startReceive(accounts[i]);
        }
        else
        {
          clientRMI->startReceive(currentAccount);
        }
      }
      break;
    
    case SEND :
      {
        if( currentAccount.isEmpty() )
        {
          clientRMI->startSend( clientRMI->getDefaultAccount() );
        }
        else
        {
          clientRMI->startSend(currentAccount);
        }
      }
      break;
    
    case NOTHING :
    default:
      qDebug("delayedSendReceive: NOTHING");
      break;
  }
  
  currentOperation = NOTHING;
}

void MailUIPartPrivate::genSendReceiveMenus()
{
  KomMailUIService *ui = static_cast<KomMailUIService*> (owner);
  ClientRMIModule *clientRMI = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  assert(0 != clientRMI);
  
  accounts=clientRMI->getAccountNames();
  bool hasActiveAccounts=false;
  
  ui->toolsSendReceiveMenu->clear();
  ui->toolsSendMenu->clear();
  ui->toolsReceiveMenu->clear();

  for (unsigned int i = 0; i < accounts.count(); i++)
  {
    if (clientRMI->isAccountActive(accounts[i]))
    {
      hasActiveAccounts=true;
      ui->toolsSendReceiveMenu->insertItem(accounts[i], this, SLOT(startSendReceive(int)), 0, i+3);
      ui->toolsSendMenu->insertItem(accounts[i], this, SLOT(startSend(int)), 0, i+3);
      ui->toolsReceiveMenu->insertItem(accounts[i], this, SLOT(startReceive(int)), 0, i+3);
    }
  }
}

void MailUIPartPrivate::startSendReceive(int id)
{
  if(id<=2)
  {
    currentAccount = QString::null;
  }
  else
  {
    currentAccount = accounts[id-3];
  }
  
  currentOperation = SEND_RECEIVE;
  startTimer();
}

void MailUIPartPrivate::startSend(int id)
{
  if(id<=2)
  {
    currentAccount = QString::null;
  }
  else
  {
    currentAccount = accounts[id-3];
  }
  currentOperation = SEND;
  startTimer();
}

void MailUIPartPrivate::startReceive(int id)
{
  if(id<=2)
  {
    currentAccount = QString::null;
  }
  else
  {
    currentAccount = accounts[id-3];
  }
  currentOperation = RECEIVE;
  startTimer();
}

void MailUIPartPrivate::slotImport()
{
  MailImportService *mis = 0;
  VFSModule *vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILIMPORTSERVICE_SERVICE );
    if( ref.isValid() )
      mis = static_cast<MailImportService*>(ref.instance());
    ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  
  assert(0 != vfs);
  
  // Configure the import parameters
  if( mis && mis->setup() )
  {
    // choose a folder where to import the emails
    QString targetPath = FolderChooserDialog::getFolder("/Mail", 0);
    if( targetPath.isEmpty() )
      return;

    ClientRMIModule *clientRMI = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
      if( ref.isValid() )
        clientRMI = static_cast<ClientRMIModule*>(ref.instance());
    }
    assert(0 != clientRMI);
    
    /*
      vfs->map(targetPath) --> do the path translation in the client side with the vfs... it is not
      sane to call vfs on the folder side
    */
    clientRMI->importMessages( vfs->map(targetPath) );
  }
    
}

void MailUIPartPrivate::slotFilters()
{
  MailFilterV *mailFilter = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_MAILFILTER_SERVICE );
    if( ref.isValid() )
      mailFilter = static_cast<MailFilterV*>(ref.instance());
  }
  
  if( !mailFilter )
    return;
  
  mailFilter->configure(0L,0L);
}

void MailUIPartPrivate::genWindowShowMenus()
{
  int i = 0;
  KomMailUIService *ui = static_cast<KomMailUIService*> (owner);
  QPtrList<QMainWindow> windows( ComposerFactory::instance()->windows() );

  ui->windowShowMenu->clear();
    
  for(QPtrListIterator<QMainWindow> it(windows); it.current(); ++it)
  {
    ui->windowShowMenu->insertItem((*it)->caption(), this, SLOT(slotWindowShow(int)), 0, i);
    i++;
  }
}

void MailUIPartPrivate::genWindowHideMenus()
{
  int i = 0;
  KomMailUIService *ui = static_cast<KomMailUIService*> (owner);
  QPtrList<QMainWindow> windows( ComposerFactory::instance()->windows() );
  
  ui->windowHideMenu->clear();
    
  for(QPtrListIterator<QMainWindow> it(windows); it.current(); ++it)
  {
    if( (*it)->isVisible() )
    {
      ui->windowHideMenu->insertItem((*it)->caption(), this, SLOT(slotWindowHide(int)), 0, i);
    }
    
    i++;
  }
}

void MailUIPartPrivate::genWindowCloseMenus()
{
  int i = 0;
  KomMailUIService *ui = static_cast<KomMailUIService*> (owner);
  QPtrList<QMainWindow> windows( ComposerFactory::instance()->windows() );
  
  ui->windowCloseMenu->clear();
    
  for(QPtrListIterator<QMainWindow> it(windows); it.current(); ++it)
  {
    ui->windowCloseMenu->insertItem((*it)->caption(), this, SLOT(slotWindowClose(int)), 0, i);
    i++;
  }
}

void MailUIPartPrivate::slotWindowShow(int id)
{
  int i = 0;
  QPtrList<QMainWindow> windows( ComposerFactory::instance()->windows() );
  
  for(QPtrListIterator<QMainWindow> it(windows); it.current(); ++it)
  {
    if( i == id )
    {
      (*it)->show();
      break;
    }
    i++;
  }
}

void MailUIPartPrivate::slotWindowHide(int id)
{
  int i = 0;
  QPtrList<QMainWindow> windows( ComposerFactory::instance()->windows() );
  
  for(QPtrListIterator<QMainWindow> it(windows); it.current(); ++it)
  {
    if( i == id )
    {
      (*it)->hide();
      break;
    }
    i++;
  }
}

void MailUIPartPrivate::slotWindowClose(int id)
{
  int i = 0;
  QPtrList<QMainWindow> windows( ComposerFactory::instance()->windows() );
  
  for(QPtrListIterator<QMainWindow> it(windows); it.current(); ++it)
  {
    if( i == id )
    {
      (*it)->close();
      break;
    }
    i++;
  }
}

void MailUIPartPrivate::slotWindowCloseAll()
{
  ComposerFactory::instance()->clear();
}
