/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mailview.h>
#include <maillabel.h>
#include <mailobj.h>
#include <magellan.h>
#include <indexmailitem.h>
#include <miscfunctions.h>
#include <texthtml.h>
#include <conversions.h>
#include <qiconloader.h>

#include <qapplication.h>
#include <qclipboard.h>
#include <qsettings.h>
#include <qmessagebox.h>

#include <defines.h>
#include <externalbrowser.h>
#include <atextbrowser.h>
#include <settings.h>
#include <vfsmimetype.h>
#include <appstatus.h>

MailView::MailView(VFSModule *vfsModule, QWidget *parent, bool vert, const char *name)
  : QWidget(parent, name),
    view(0),
    ShowFlag(0),
    attvert(vert),
    label(0),
    showatt(false),
    forceUpdate(false),
    vfs(vfsModule)
{
  textHtmlConverter = TextHtmlConvertor::ref();

  QVBoxLayout * l = new QVBoxLayout(this);
  
  split = new QSplitter(this);
  if(!attvert)
    split->setOrientation(QSplitter::Vertical);
  split->setSizePolicy( QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding) );

  view = new ATextBrowser(split, "Mail View");
  view->setSizePolicy( QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding) );
  view->setTextFormat(Qt::AutoText);
  connect(view, SIGNAL(linkClicked(const QString&)), this, SLOT(slotLinkClicked(const QString&)));

  attpane = new AttachmentPane(vfs, split);
  if( attpane )
  {
    connect( attpane,
             SIGNAL( statusChanged(const QString&, const QString&, const QString&, const QString&, const QString&, int, int) ),
             this,
             SLOT( slotAttachmentInfo(const QString&, const QString&, const QString&, const QString&, const QString&, int, int) ) );
    attpane->hide();
  }
  
  label = new MailLabel(vfs, this);

  l->addWidget(label);
  l->addWidget(split);

  view->show();
  label->show();
  
  showDefaultPage();
}

MailView::~MailView()
{
  if( attpane )
    attpane->hide();
}

void MailView::setAddress(QString addr)
{
  if( addr.isEmpty() )
  {
    if( attpane )
    {
      delete attpane;
      attpane = 0L;
    }
    showDefaultPage();
    address=QString::null;
    return;
  }

  // avoid repeated repaints, they can be fairly slow
  if(addr==address && forceUpdate==false)
    return;
  else
  {
    address=addr;
    forceUpdate=false;
    // remove the old attachement widget
    if( attpane )
    {
      delete attpane;
      attpane = 0L;
    }
    
    // get the object
    MailObject *item=(MailObject *)vfs->_fact_ref(address, VFS_MAILS_MIMETYPE);
    if( !item )
    {
      address=QString::null;
      showDefaultPage();
      return;
    }

    // build the mail label
    label->setAddress(address);
    // build the attachment widget
    if( item->attachments() )
    {
      attpane = new AttachmentPane(vfs, split);
      if( attpane )
      {
        connect( attpane,
                 SIGNAL( statusChanged(const QString&, const QString&, const QString&, const QString&, const QString&, int, int) ),
                 this,
                 SLOT( slotAttachmentInfo(const QString&, const QString&, const QString&, const QString&, const QString&, int, int) ) );
        attpane->setAddress(address);
        
        QValueList<int> sizeList;
        sizeList << width() / 3 * 2 << width() / 3;
        split->setSizes(sizeList);
      }
      else
      {
        qDebug("No attachment pane.");
        return;
      }
    }

    // build the mail view
    // find out if user prefers plain text or html	
    bool preferHTML = true;
//      Settings::instance()->readBoolEntry("/Aethera/Read Options/Prefer HTML");

    QString text;
    bool htmlSource;

    if(preferHTML && item->hasHTML())
    {
      text=item->html();
      htmlSource=true;
      view->setTextFormat(Qt::RichText);
    }
    else
    {
      text=item->text();
      htmlSource=false;
      view->setTextFormat(Qt::PlainText);
    }

    // process text
//    if(preferHTML && htmlSource)
//    {
//      // we got HTML text
//      textHtmlConverter->html2html(text);
//    }
//    else
//    {
//      // we got plain text
//      textHtmlConverter->text2html(text);
//    }

    // change the view
    view->setPaper(Qt::white);
    view->setText(text);
  }
}

void MailView::togglePane()
{
  if(attpane)
  {
    showatt=!showatt;
    if(showatt)
      attpane->show();
    else
      attpane->hide();
  }
}

void MailView::hideAttachmentPane()
{
  showatt=false;
  if( attpane )
    attpane->hide();
}

void MailView::objectChanged(QString address)
{
  forceUpdate=true;
  setAddress(address);
}

void MailView::showDefaultPage()
{
  label->showDefault();
  QString defaultText=" \n     Nothing to show ";

  view->setText(defaultText);
}

void MailView::copyText()
{
  view->copy();
}

void MailView::slotLinkClicked(const QString & link)
{
  if ("mailto:" == link.left(7))
  {
    QMessageBox::warning
      (
        this,
        "Warning - " + QString(AppName),
        "Need to tell mail plugin to compose.\n"
        "Not yet implemented !"
      );
  }
  else if (!ExternalBrowser::launchUrl(link, this))
  {
    QMessageBox::warning
      (
        this,
        "Warning - " + QString(AppName),
        "Can't launch a web browser.\n"
        "Please contact aethera@thekompany.com for help."
      );
  }
}

void MailView::slotAttachmentInfo( const QString&, const QString& mime,
                                   const QString& type, const QString& encoding,
                                   const QString& charset, int partSize, int )
{
  AppStatus * status = AppStatus::instance();
  
  if( status )
  {
    QString text = QString("Info: ") + mime + " | " + type + " | " + encoding + " | " + charset +
                   QString(" | %1 kb").arg((int)(partSize/1024));
    
    status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, text);
  }
}

