/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include <qapplication.h>
#include <qtimer.h>
#include <qdict.h>
#include <qfile.h>
#include <qsettings.h>
#include <qpushbutton.h>
#include <mailwindow.h>
#include <mailobj.h>
#include <vfolder.h>
#include <vfolderattribute.h>
#include <maillistview.h>
#include <mailview.h>
#include <indexmailitem.h>
#include <komaetheratools.h>
#include <appstatus.h>
#include <qlineedit.h>
#include <settings.h>

// The folder status class
FolderStatus::FolderStatus(const QString &name, const QString &type, const QString &viewType, const unsigned state)
{
  _name=name;
  _type=type;
  _viewType=viewType;
  _state=state;
}

void FolderStatus::changeStatus(const QString &name, const QString &type, const QString &viewType, const unsigned state)
{
  _name=name;
  _type=type;
  _viewType=viewType;
  _state=state;
}

void FolderStatus::reset()
{
  _name="/Aethera/VFS/Mail";
  _type=QString::null;
  _viewType=QString::null;
  _state=FolderStatus::NO_STATE;
}


// The contents widget class
MailWindow::MailWindow(VFSModule *vfsModule, QWidget *parent)
  : QWidget(parent),
    mailListView(0)
{
  vfs = vfsModule;
  ClientRMIModule *clientRMI = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  assert(0 != clientRMI);

  lastID=0;
  lastMailID=0;

  // init the views
  // mail splitter

  mailSplitter=new QSplitter(QSplitter::Vertical, this);
  mailSplitter->show();

  // make the main layout
  mainLayout=new QVBoxLayout(this);

  // add the widget stack
  mainLayout->addWidget( mailSplitter );
  mainLayout->activate();

  // set current folder
  currentFolder = new FolderStatus("/Aethera/VFS/Mail/", QString::null, QString::null, FolderStatus::NO_STATE);
  parentFolder = vfs->folderRef( currentFolder->_name );

  // add find widget

  mailFind = new MailFindWidget(mailSplitter);
  mailFind->hide();  
  mailFind->mailFindContainsComboBox->insertItem("Subject");
  mailFind->mailFindContainsComboBox->insertItem("From");
  mailFind->mailFindContainsComboBox->insertItem("Body");
    
  // connect find widget
  
  connect(mailFind->mailFindDeleteButton,
  SIGNAL(clicked()),
   this,
   SLOT(slotCancelFindMail()));
  
  connect(mailFind->mailFindSearchButton,
  SIGNAL(clicked()),
   this,
   SLOT(slotFindMail()));

  connect(mailFind->mailFindLine,
  SIGNAL(returnPressed()),
   this,
   SLOT(slotFindMail()));    

  // mailListContainer : emails list
                  
  mailListContainer = new KomContainerWidget(mailSplitter, "MailListContainer");

  // set current displayed message
  // mail view
    
  mailView=new MailView(vfs, mailSplitter);
  mailView->show();

  currentMessage=QString::null;
}

MailWindow::~MailWindow()
{
  delete currentFolder;
}

QString MailWindow::getCurrentFolder()
{
  return currentFolder->_name;
}

void MailWindow::deleteCurrentFolder()
{
  currentFolder->reset();
}

void MailWindow::slotFindMail()
{
  slotSetFolder(getCurrentFolder(), true);
}

void MailWindow::slotCancelFindMail()
{
  slotSetFolder(getCurrentFolder(), false);
}

void MailWindow::slotShowMessage(QString address)
{
//  qDebug("slotShowMessage %s", address.latin1());
  
  currentMessage=address;
  
  mailView->setAddress(address);

  // mark message as read
  int timeout =
    Settings::instance()->readNumEntry("/Aethera/Read Options/Read timeout");

  if(timeout<0) timeout=3;

  if(timeout==0)
  {
    if( mailListView )
      mailListView->markRead(address);
  }
  else
  {
    lastReadMessage=address;
    QTimer::singleShot(timeout*1000, this, SLOT(markAsRead()));
  }
}

void MailWindow::slotShowMessage(IndexMailItem *item)
{
  QString address = QString::null;

  if( item )
  {
    MailObject * mail = item->vObject();
    if( mail )
    {
      address = mail->path();
    }
    else
      return;
  }

  slotShowMessage(address);
}

void MailWindow::slotShowMessage(QListViewItem *item)
{
  slotShowMessage((IndexMailItem *)item);
}

void MailWindow::unmountFolder()
{
  // empty mail view, delete the attachment and it's vobjects
  if( mailView )
    mailView->setAddress(QString::null);
 
  // unload all the VObjects from the last used folder
  VFolder *folder = vfs->folderRef(currentFolder->_name);
  if( folder && 
      (folder->type()==VFS_MAILS_MIMETYPE || 
      folder->type()==VFS_IMAP_MIMETYPE) )
    folder->freeObjects();
  
  if( currentFolder->_state == FolderStatus::MOUNT )
  {
//    qDebug("-------------- UNMOUNT -------------------");
    fflush(stdout);
    currentFolder->_state=FolderStatus::UMOUNT;
    clientRMI->unmount(currentFolder->_name);
  }
  
  currentFolder->reset();
}

bool MailWindow::mountFolder(QString folderPath)
{
  VFolder *folder = vfs->folderRef(folderPath);
  // why we need to check again for ClientRMI ?
  // it was a crash
  clientRMI = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  assert(0 != clientRMI);
  
  if( !folder )
    return false;
  
  if( folder->type()=="imap" && 
      folder->viewType()=="mail" )
  {
//    qDebug("-------------- MOUNT -------------------");
    currentFolder->changeStatus(folderPath, folder->type(), folder->viewType(), FolderStatus::MOUNT);
    clientRMI->mount(folderPath);
  }
  else
    currentFolder->changeStatus(folderPath, "default", "default", FolderStatus::NO_STATE);

  parentFolder = folder;
  
  return true;
}

void MailWindow::slotSetFolder(QString folderPath)
{
  // nothing to change if it is the same like before
//  if( currentFolder->_name == folderPath )
//    return;

  // Mount and unmount the remote folders.
  // Check if the last selected folder was an IMAP one.
  // If so, we have to unmount the folder.
  // Check if selected folder is an IMAP one.
  // If so, we have to mount the folder.
  
  unmountFolder();
  if( !mountFolder(folderPath) )
    return;

  // debug
  qDebug("MailWindow: showing %s", (const char *)folderPath);

  if( mailListView )
  {
    disconnect(mailListView, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(slotShowMessage(QListViewItem *)));
  }
  
  // add a new MailListView to the stack
  mailListView = mailsViews.find(folderPath);
  if( !mailListView )
  {
    // create a new mail list
    mailListView = new MailListView(vfs, parentFolder, mailListContainer, 0);

    if( mailListView )
    {
      // add it to the stack
      mailsViews.insert( folderPath, mailListView);
     
      mailListContainer->addChildWidget( mailListView );
      mailListContainer->showChild( mailListView );
  
      mailFind->show();
                  
      // connect
      connect(mailListView, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(slotShowMessage(QListViewItem *)));
      
      // load the mail objects
      mailListView->loadMails();
      
      showNext("first",true);
    }
  }
  else
  {
    // connect
    connect(mailListView, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(slotShowMessage(QListViewItem *)));
    
    mailListContainer->showChild( mailListView );
    showNext("last",true);
  }
}

void MailWindow::slotSetFolder(QString folderPath, bool bSearch)
{
  // nothing to change if it is the same like before
//  if( currentFolder->_name == folderPath )
//    return;

  // Mount and unmount the remote folders.
  // Check if the last selected folder was an IMAP one.
  // If so, we have to unmount the folder.
  // Check if selected folder is an IMAP one.
  // If so, we have to mount the folder.
  
  unmountFolder();
  if( !mountFolder(folderPath) )
    return;

  // debug
  qDebug("MailWindow: showing %s", (const char *)folderPath);

  if( mailListView )
  {
    disconnect(mailListView, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(slotShowMessage(QListViewItem *)));
  }
  
  // add a new MailListView to the stack
  mailListView = mailsViews.find(folderPath);
  if( !mailListView )
  {
    // create a new mail list
    mailListView = new MailListView(vfs, parentFolder, mailListContainer, 0);

    // add it to the stack
    mailsViews.insert( folderPath, mailListView);
   
    mailListContainer->addChildWidget( mailListView );
    mailListContainer->showChild( mailListView );

    mailFind->show();
                
    // connect
    connect(mailListView, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(slotShowMessage(QListViewItem *)));
    
    // load the mail objects
    if(bSearch)
    {
      mailListView->loadMails( mailFind->mailFindContainsComboBox->currentText(),
                               mailFind->mailFindLine->text() );
    }
    else
    {
      mailListView->loadMails();
    }
    
    showNext("first",true);
  }
  else
  {
    // connect
    connect(mailListView, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(slotShowMessage(QListViewItem *)));
    
    if(bSearch)
    {
      mailListView->clear();
      mailListContainer->showChild( mailListView );
      mailListView->loadMails( mailFind->mailFindContainsComboBox->currentText(),
                               mailFind->mailFindLine->text() );
      showNext("first",true);
    }
    else
    {
      mailListView->clear();
      mailListContainer->showChild( mailListView );
      mailListView->loadMails();
      showNext("first",true);
    }
  }
}

void MailWindow::slotDeleteFolder(const QString & folderPath)
{
//  qDebug("MailWindow::slotDeleteFolder");
  
  MailListView *view = 0L;
  view = mailsViews.find( folderPath );
  if( view )
  {
    mailsViews.remove( folderPath );
    if( view == mailListView )
      mailListView = 0L;
      
    delete view;
    view = 0;
  }
  
  bool ok = vfs->deleteFolder(folderPath);

  if (!ok)
  {
    qDebug("MailWindow::slotDeleteFolder: unsuccessful");
  }
}

void MailWindow::updateCurrentView()
{
  showNext("last",true);
}

void MailWindow::slotCreateVInstances(const QStringList &urls)
{
  if( urls.size() )
  {
    AppStatus * status = AppStatus::instance();
    
    // if it isn't a folder, huh
    if( !vfs->folderRef(urls[0]) )
    {
      MailListView *view = 0;
      QString folderPath;
      
      folderPath = urls[0].left( urls[0].findRev('/') );
      VFolder * folder = vfs->folderRef( folderPath );
      view = mailsViews.find( folderPath );

      if( view && folder )
      {
        view->setSorting( -1 );
        
/*        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
          tr("Loading..."), 0, urls.count());*/
  
        view->setUpdatesEnabled( false );
        
/*        int i=1;*/
        for(QStringList::ConstIterator it=urls.begin(); it!=urls.end(); ++it)
        {
          MailObject *item = static_cast<MailObject*> (vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE));
        
          if(item)
          {
            new IndexMailItem(view, vfs, item);
          }
          
/*          if( !(i%300) )
          {
            status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
              tr("Loading..."), i);
          }
          
          i++;*/
        }
        
        view->setSortingColumn();
        view->sort();
        
        view->setUpdatesEnabled( true );
        updateCurrentView();
        
        folder->freeObjects();
        
        status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, tr("The folder is loaded."));
      }
    }
  }
}

void MailWindow::slotCreateVInstance(QString path)
{
  MailListView *view = mailsViews.find( vfs->parentFolder(path) );
  if( view )
  {
    MailObject *item = static_cast<MailObject*> (vfs->_fact_ref(path, VFS_MAILS_MIMETYPE));

    if(item)
    {
      new IndexMailItem(view, vfs, path);
      
      delete item;
    }
    
    if( view == mailListView )
      updateCurrentView();
  }
}

void MailWindow::slotUpdateVInstances(const QStringList &paths)
{
  if( paths.size() )
  {
//    qDebug("MailWindow::slotUpdateVInstances(const QStringList &) %s", paths[0].latin1());
    
    MailListView *view = 0;
    
    // if it is a folder
    if( vfs->folderRef(paths[0]) )
    {
      // if it is the window parent folder
      view = mailsViews.find( paths[0] );
      if( view )
        view->update();
      else
        return;
    }
    else
    {
      // if it is an object from the window parent folder
      view = mailsViews.find( vfs->parentFolder(paths[0]) );
      if( view )
      {
        view->slotUpdateVInstances(paths);
        IndexMailItem *item = dynamic_cast<IndexMailItem*>(view->currentItem());
        if( item )
          mailView->objectChanged( item->path() );
      }
      else
        return;
    }
//    updateMailListView();
  }
}

void MailWindow::slotUpdateVInstance(const QString &path)
{
  if( path.isNull() || path.isEmpty() )
    return;
    
  MailListView *view = 0;
    
//  qDebug("MailWindow::slotUpdateVInstances(const QString &) %s", path.latin1());
  // if it is a folder
  if( vfs->folderRef(path) )
  {
    // if it is the window parent folder
    view = mailsViews.find( path );
    if( view )
      view->update();
    else
      return;
  }
  else
  {
    // if it is an object from the window parent folder
    view = mailsViews.find( vfs->parentFolder(path) );
    if( view )
      view->slotUpdateVInstances(path);
    else
      return;
  }
//  updateMailListView();
}

void MailWindow::slotDeleteVInstances(const QStringList &paths)
{
  MailListView *view = 0;
  
  if( paths.size() && (view = mailsViews.find(vfs->parentFolder(paths[0])) ) )
  {
  
//    DEBUG_MSG("MailWindow::slotDeleteVInstances(const QStringList &paths) ----");
  
    view->slotDeleteVInstances(paths);
    
    if( view == mailListView )
      showNext("last", true);
  }
}

void MailWindow::updateMailListView()
{
  if(mailListView && mailListView->isSelectionMultiple())
  {
    // view is in multi selection mode

    IndexMailItem *item= static_cast<IndexMailItem *>(mailListView->currentItem());
    if(!item)
      item = mailListView->firstSelected();
    slotShowMessage(item);
  }
  else
  {
    // view has at most one selected item

    QString mode = Settings::instance()->readEntry("/Aethera/Misc Mail/Goto");

    if(mode.isEmpty()) mode="last";
    showNext(mode, true);
  }
}

void MailWindow::showNext(const QString &mode, bool direction)
{
  if( !mailListView )
    return;
  
  IndexMailItem *item=0;

  if(mode=="last")
  {
    // keep the last current item

    // get current item
    item=(IndexMailItem *)mailListView->currentItem();
  }
  else if(mode=="top")
  {
    // set the current item to be the first item

    item=(IndexMailItem *)mailListView->firstChild();
  }
  else if(mode=="first")
  {
    // set the current item to be the first item

    item=(IndexMailItem *)mailListView->firstChild();
  }
  else if(mode=="bottom")
  {
    // set the current item to be the last item

    item=(IndexMailItem *)mailListView->firstChild();
    if(item)
    {
      while(item->nextSibling()) item=(IndexMailItem *)item->itemBelow();
    }
  }
  else if(mode=="next")
  {
    // jump one item above or below

    item=(IndexMailItem *)mailListView->currentItem();
    if(item) item=direction ? (IndexMailItem *)item->itemBelow() :
                              (IndexMailItem *)item->itemAbove();
  }
  else if(mode=="priority")
  {
    // jump to the next item with "High" or "Highest" priority, above or below

    IndexMailItem *temp=(IndexMailItem *)mailListView->currentItem();
    if(!temp) temp=(IndexMailItem *)mailListView->firstChild();

    while(!item && temp)
    {
      temp=direction?(IndexMailItem *)temp->itemBelow():(IndexMailItem *)temp->itemAbove();

      if(temp)

      {
        MailObject *obj = temp->vObject();
        if(obj->priority().left(4)=="High") item=temp;
      }
    }
  }
  else if(mode=="unread")
  {
    // jump to the next item with "New" status, above or below

    IndexMailItem *temp=(IndexMailItem *)mailListView->currentItem();
    if(!temp) temp=(IndexMailItem *)mailListView->firstChild();

    while(!item && temp)
    {
      temp=direction?(IndexMailItem *)temp->itemBelow():(IndexMailItem *)temp->itemAbove();

      if(temp)
      {
        MailObject *obj = temp->vObject();
        if(obj->status()=="New") item=temp;
      }
    }
  }
  else if(mode=="flag")
  {
    // jump to the next item that has a little flag, above or below

    IndexMailItem *temp=(IndexMailItem *)mailListView->currentItem();
    if(!temp) temp=(IndexMailItem *)mailListView->firstChild();

    while(!item && temp)
    {
      temp=direction?(IndexMailItem *)temp->itemBelow():(IndexMailItem *)temp->itemAbove();

      if(temp)
      {
        MailObject *obj = temp->vObject();
        if(obj->flag()) item=temp;
      }
    }
  }

  // we should display something even if the previous selection methods didn't find anything useful
  if(!item) item=(IndexMailItem *)mailListView->currentItem();
  if(!item) item=(IndexMailItem *)mailListView->firstChild();

  if(item)
  {
    mailListView->clearSelection();
    mailListView->setCurrentItem(item);
    mailListView->setSelected(item, true);
    mailListView->ensureItemVisible(item);
//    mailListView->update();

    slotShowMessage(item);
  }
  else
  {
    slotShowMessage(QString::null);
  }
}

// This method is designed for recursive building of the mail unread messages
// table for presentation page
QString MailWindow::buildUnreadMessagesTable(QString /*parentFolder*/, QString /*parent*/)
{
  QString result;
  QString tmp;
  VFolder *folder;
  QDict<VFolder> dict = vfs->folderDict();
  QDictIterator<VFolder> it(dict);
  int newMessages=0;
  while((folder=it.current()))
  {
    if(folder->type()=="mail" || folder->type()=="imap")
    {
      VFolderAttribute *attr;
      attr=(VFolderAttribute *)vfs->_fact_ref(folder->path()+"/default"+ATTR_SEP+"fattr", "vfolderattribute");
      if(attr) newMessages+=attr->recent;
    }
    ++it;
  }
  //rescan to display folders only if necessary
  if(newMessages==0)
  {
    result="<br><br>&nbsp;&nbsp;&nbsp;&nbsp;There are no new messages.";
  }
  else
  {

    result="<br><br>&nbsp;&nbsp;&nbsp;&nbsp;You have unread messages:<br><br>";
    result += "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">";
    QDictIterator<VFolder> it2(dict);
    while((folder=it2.current()))
    {
      if(folder->type()=="mail" || folder->type()=="imap")
      {
        VFolderAttribute *attr;
        attr=(VFolderAttribute *)vfs->_fact_ref(folder->path()+"/default"+ATTR_SEP+"fattr", "vfolderattribute");
        if(attr && attr->recent)
        {
          result += "<tr><td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>";
          result += "<td align=\"center\"><font color=\"blue\">";
          tmp.sprintf("%d&nbsp;", attr->recent); result += tmp + "</font></td>";
          result += "<td>&nbsp;<b>unread</b> messages in <a href=\"" + folder->path() + "\"><b>";
          if(folder->path().find("/Magellan/Mail/")!=-1)
            result += folder->name();
          else
            result += folder->path().mid(10, folder->path().length()-10);
          result += "</b></a></td></tr>";
        }
      }
      ++it2;
    }
    result+="</table>";
  }
  return result;
}

QString MailWindow::getDate()
{
  QString result;
  QDate today;
  result="<font face=\"Arial, Helvetica, sans-serif\" size=\"-1\"> \
    <b><font color=\"#660000\">";
  switch(today.currentDate().dayOfWeek())
  {
    case 1:
      result.append("Monday");
      break;
    case 2:
      result.append("Tuesday");
      break;
    case 3:
      result.append("Wednesday");
      break;
    case 4:
      result.append("Thursday");
      break;
    case 5:
      result.append("Friday");
      break;
    case 6:
      result.append("Saturday");
      break;
    case 7:
      result.append("Sunday");
      break;
    default:
      // This should *NEVER* occur!
      result.append("<Incorrect date set>");
  } // switch
  result.append("</font>,&nbsp;");
  QString number;
  result.append(number.setNum(today.currentDate().day()));
  result.append(" ");
  switch(today.currentDate().month())
  {
    case 1:
      result.append("January");
      break;
    case 2:
      result.append("February");
      break;
    case 3:
      result.append("March");
      break;
    case 4:
      result.append("April");
      break;
    case 5:
      result.append("May");
      break;
    case 6:
      result.append("June");
      break;
    case 7:
      result.append("July");
      break;
    case 8:
      result.append("August");
      break;
    case 9:
      result.append("September");
      break;
    case 10:
      result.append("October");
      break;
    case 11:
      result.append("November");
      break;
    case 12:
      result.append("December");
      break;
  }
  result.append(" ");
  result.append(number.setNum(today.currentDate().year()));
  result.append(" &nbsp;</b></font>");
  return result;
}

void MailWindow::slotReply()
{
  if( mailListView )
    mailListView->reply();
}

void MailWindow::slotForward()
{
  if( mailListView )
    mailListView->forward();
}

void MailWindow::slotForwardInline()
{
  if( mailListView )
    mailListView->forwardInline();
}

void MailWindow::slotForwardQuoted()
{
  if( mailListView )
    mailListView->forwardQuoted();
}

void MailWindow::slotForwardAsAttachment()
{
  if( mailListView )
    mailListView->forwardAsAttachment();
}

void MailWindow::slotCut()
{
  if( mailListView )
    mailListView->cutSelection();
}

void MailWindow::slotCopy()
{
  if( mailListView )
    mailListView->copySelection();
}

void MailWindow::slotPaste()
{
  if( mailListView )
    mailListView->paste();
}


void MailWindow::slotDelete()
{
  if( mailListView )
    mailListView->slotDeleteSelection();
}

void MailWindow::slotReplyAll()
{
  if( mailListView )
    mailListView->replyAll();
}

void MailWindow::slotSelectAll()
{
  if( mailListView )
    mailListView->selectAll();
}

void MailWindow::slotUnSelectAll()
{
  if( mailListView )
    mailListView->unselectAll();
}

void MailWindow::slotPrint()
{
  if( mailListView )
    mailListView->printSelection();
}
void MailWindow::slotRead()
{
  if( mailListView )
    mailListView->markRead();
}

void MailWindow::slotUnRead()
{
  if( mailListView )
    mailListView->markUnread();
}

void MailWindow::slotAllRead()
{
  if( mailListView )
    mailListView->markAllRead();
}

void MailWindow::slotAllUnRead()
{
  if( mailListView )
    mailListView->markAllUnread();
}

void MailWindow::slotToggleFlag()
{
  if( mailListView )
    mailListView->toggleFlag();
}

void MailWindow::slotProperties()
{
  if( mailListView )
    mailListView->slotEditProperties();
}

void MailWindow::slotNextUnread()
{
  showNext("unread", true);
}

void MailWindow::slotPreviousUnread()
{
  showNext("unread", false);
}

void MailWindow::slotNextFlag()
{
  showNext("flag", true);
}

void MailWindow::slotPreviousFlag()
{
  showNext("flag", false);
}

void MailWindow::slotNextImportant()
{
  showNext("priority", true);
}

void MailWindow::slotPreviousImportant()
{
  showNext("priority", false);
}
