/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef _MAILWINDOW_H
#define _MAILWINDOW_H

#include <qwidget.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <maillistview.h>
#include <mailfindwidget.h>
#include <qsplitter.h>
#include <mailview.h>
#include <clientrmi.h>
#include <vfsmodule.h>
#include <komcontainerwidget.h>

class VFolder;

class MAIL_PLUGIN_API FolderStatus
{
  public:
    enum  FOLDER_STATE { NO_STATE=0, MOUNT, UMOUNT};

    FolderStatus(const QString &name, const QString &type, const QString &viewType, const unsigned state);
    /** Change the current folder status.*/
    void changeStatus(const QString &name, const QString &type, const QString &viewType, const unsigned state);
    /** Reset the current folder status.*/
    void reset();

    /** The current folder name.*/
    QString _name;
    /** The current folder name.*/
    QString _type;
    /** The current folder name.*/
    QString _viewType;
    /** The current folder name.*/
    unsigned  _state;
};

#ifdef Q_WS_WIN
#if defined(Q_TEMPLATEDLL)
// MOC_SKIP_BEGIN
template class MAIL_PLUGIN_API QDict<MailListView>;
template class MAIL_PLUGIN_API QDictIterator<MailListView>;
// MOC_SKIP_END
#endif
#endif

typedef QDict<MailListView> MLVDict;
typedef QDictIterator<MailListView> MLVDictIterator;

class MAIL_PLUGIN_API MailWindow : public QWidget
{
  Q_OBJECT
  
  public:
  
    MailWindow(VFSModule *vfsModule, QWidget *parent);
    
    virtual ~MailWindow();

    // message navigation; "mode" can be: "last", "top", "bottom", "priority", "unread", "next", "flag"
    // "direction" makes sense only for "priority", "next", "flag" or "unread" modes; "true" is down
    void showNext(const QString &mode, bool direction);

    /** current folder.*/
    QString getCurrentFolder();
    /** delete the current folder. */
    void deleteCurrentFolder();

    /** update current view.*/
    void updateCurrentView();
    
    /** Mount an folder.*/
    bool mountFolder(QString folderPath);
    
    /** Unmount the current folder.*/
    void unmountFolder();

  public slots:
    void slotCreateVInstance(QString);
    void slotCreateVInstances(const QStringList &);
    void slotUpdateVInstances(const QStringList &);
    void slotUpdateVInstance(const QString &);
    void slotDeleteVInstances(const QStringList &);
    void slotSetFolder(QString, bool);
    void slotSetFolder(QString);
    void slotDeleteFolder(const QString&);
    void slotFindMail();
    void slotCancelFindMail();
    void slotShowMessage(QListViewItem *);
    void slotShowMessage(IndexMailItem *);
    void slotShowMessage(QString);
    void slotReply();
    void slotForward();
    void slotForwardInline();
    void slotForwardQuoted();
    void slotForwardAsAttachment();
    void slotCut();
    void slotCopy();
    void slotPaste();
    void slotDelete();
    void slotReplyAll();
    void slotSelectAll();
    void slotUnSelectAll();
    void slotPrint();
    void slotRead();
    void slotUnRead();
    void slotAllRead();
    void slotAllUnRead();
    void slotToggleFlag();
    void slotProperties();
    void slotNextUnread();
    void slotPreviousUnread();
    void slotNextFlag();
    void slotPreviousFlag();
    void slotNextImportant();
    void slotPreviousImportant();

  signals:
    void viewChanged();
    
  private:
    VFSModule *vfs;
    ClientRMIModule *clientRMI;
    
    int lastID, lastMailID;

    // the main layout
    QVBoxLayout *mainLayout;

    // the mails view splitter
    QSplitter *mailSplitter;
           
    // the mail view
    MailView *mailView;
    
    // the mails list view
    KomContainerWidget *mailListContainer;
    MLVDict mailsViews;
    // the visible mail list view
    MailListView *mailListView;

    MailFindWidget * mailFind;
    // current folder status
    FolderStatus *currentFolder;
    VFolder *parentFolder;

    // current message displayed in the mail view; QString::null means "Nothing to display"
    QString currentMessage;
    QString lastReadMessage;

    void updateMailListView();
    
    QString buildUnreadMessagesTable(QString, QString);
    QString getDate();
};

#endif



