/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <cassert>
#include <partobj.h>
#include <vpartattribute.h>
#include <vfsmimetype.h>
#include <vfsmodule.h>

PartObject::PartObject(const QString & _path)
  :VObject(_path)
{
  sync();
}

PartObject::PartObject(const QString & _path, const QByteArray & _data)
  :VObject(_path)
{
  if( !_data || !_data.size() )
    sync();
  else
    _narrow(_data);
}

PartObject::~PartObject()
{
}

bool PartObject::sync()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  assert(0 != vfs);

  VPartAttribute *attr=(VPartAttribute *)vfs->_fact_ref(path()+ATTR_SEP+"fattr", VFS_PARTATTRIBUTE_MIMETYPE );
  
  if(attr)
  {
    attr->sync();
    _filename=attr->name;
    _mimeType=attr->mimetype;
    _contentID=attr->cid;
    _partType=attr->mtype;
    _encoding=attr->encoding;
    _charset=attr->charset;
    delete attr;
    return true;
  }
  else
  {
    qDebug("Warning: [part sync] could not create part attribute object, bailing out");
    return false;
  }
}

bool PartObject::save()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  assert(0 != vfs);

  VPartAttribute *attr=(VPartAttribute *)vfs->_fact_ref(path()+ATTR_SEP+"fattr", VFS_PARTATTRIBUTE_MIMETYPE );
  
  if(attr)
  {
    attr->name=_filename;
    attr->mimetype=_mimeType;
    attr->cid=_contentID;
    attr->mtype=_partType;
    attr->encoding=_encoding;
    attr->charset=_charset;
    
    // debug
    // qDebug("partobject: [save] saving attribute...");
    
    bool res=attr->save();
    delete attr;
    
    // debug
    // qDebug("partobject: [save] save result was %s", res?"true":"false");
    
    return res;
  }
  else
  {
    // qDebug("Warning: [part save] could not create part attribute object, bailing out");
    return false;
  }
}

QString PartObject::filename() const
{
  return _filename;
}

void PartObject::setFilename(const QString &newName)
{
  _filename=newName;
}

QString PartObject::mimeType() const
{
  return _mimeType;
}

void PartObject::setMimeType(const QString &newMimeType)
{
  if( !newMimeType.isEmpty() )
    _mimeType = newMimeType;
}

QString PartObject::contentID() const
{
  return _contentID;
}

void PartObject::setContentID(const QString &newContentID)
{
  _contentID=newContentID;
}

QString PartObject::partType() const
{
  return _partType;
}

void PartObject::setPartType(const QString &newPartType)
{
  _partType=newPartType;
}

QString PartObject::encoding() const
{
  return _encoding;
}

void PartObject::setEncoding(const QString &newEncoding)
{
  _encoding=newEncoding;
}

QString PartObject::charset() const
{
  return _charset;
}

void PartObject::setCharset(const QString &newCharset)
{
  _charset=newCharset;
}

bool PartObject::getProperty(const QString&, Any&)
{
  return false;
}

bool PartObject::setProperty(const QString&, Any&)
{
  return false;
}

QueryProcessor::QueryType PartObject::queryType(const QString&)
{
  return QueryProcessor::InvalidQuery;
}

QByteArray PartObject::_stream()
{
  return QByteArray();
}

void PartObject::_narrow(const QByteArray &)
{
}

#ifdef _QTWIN_
VObject *PartObjectFactory::_new(const QString & _path)
#else
PartObject *PartObjectFactory::_new(const QString & _path)
#endif
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  assert(0 != vfs);

  // check if it is possible to create a new contact
  if(vfs->canDefine(_path) && !vfs->reference(_path))
    return new PartObject(_path);
  else
    return 0;
}

#ifdef _QTWIN_
VObject *PartObjectFactory::_new
#else
PartObject *PartObjectFactory::_new
#endif
(
  const QString     & _path,
  const QByteArray  & _data
)
{
  return new PartObject(_path, _data);
}





