/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  komnotesconfigservice.cpp  -  the Notes configuration service based on VFS
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <notesconfigwidget.h>
#include <komnotesconfigservice.h>
#include <qsettingsextra.h>
#include <fontcolorbutton.h>
#include <qfontchooser.h>
#include <notewindow.h>
#include <notepreview.h>
#include <noteobject.h>

#include <qfont.h>
#include <qsettings.h>
#include <qspinbox.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qlayout.h>

static const char * const configPrefix = "/Aethera/Notes/";

static const int defaultWidth 	= 100;
static const int defaultHeight 	= 100;

//tkcBase includes
#include <qiconloader.h>
#include <settings.h>

class KomNotesConfigService::Private
{
  public:

    NotesConfigWidget 				* widget;
};

/*
static const char* komnotesconfigservice_metainfo[][2]=
{
    { "/name", "Kom Notes Config Service" },
    { "/class", "KomNotesConfigService" },
    { "/class/fqn", "KomNotesConfigService" },
    { "/factory", "komnotesconfigservice_factory" },
    { 0, 0 }
};
*/

kore::Interface* komnotesconfigservice_factory()
{
  return new KomNotesConfigService();
}
    
KomNotesConfigService::KomNotesConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom Notes Config Service");
  info->replaceAll("class", "KomNotesConfigService");
  info->replaceAll("class/fqn", "KomNotesConfigService");
  info->replaceAll("factory", "komnotesconfigservice_factory");
  
  d = new Private;
  d->widget 		= 0;
}

KomNotesConfigService::~KomNotesConfigService()
{
  delete d->widget;
  d->widget = 0;
  delete d;
}

  QWidget *
KomNotesConfigService::init(QWidget * parent)
{
	if (0 == d->widget)
	{
		d->widget = new NotesConfigWidget(parent, "KomNotesConfigService widget");
	}

  return d->widget;
}

	void
KomNotesConfigService::load()
{
	if (0 == d->widget)
		return;

	QFont widgetFont(d->widget->font());

	QFont savedFont =
		QSettingsExtra::readFontEntry
		(Settings::instance(), configPrefix + QString("Default font"), &widgetFont);

	d->widget->fontChooserButton->setSelectedFont( savedFont );

	QColor savedColor;

	QStringList entries = Settings::instance()->entryList(configPrefix);

	if (entries.find("Default color") != entries.end())
	{
		savedColor =
			QSettingsExtra::readColorEntry
			(
			 Settings::instance(),
			 configPrefix + QString("Default color"),
       &Qt::yellow
      );
    d->widget->colorButton->setColor(savedColor);
  }
	if (entries.find("Default text color") != entries.end())
	{
		savedColor =
			QSettingsExtra::readColorEntry
			(
			 Settings::instance(),
			 configPrefix + QString("Default text color"),
       &Qt::black
      );
    d->widget->textColorButton->setColor(savedColor);
  }

	QSize defaultSize(defaultWidth, defaultHeight);

	QSize savedSize =
		QSettingsExtra::readSizeEntry
		(Settings::instance(), configPrefix + QString("Default size"), &defaultSize);

	d->widget->widthSpinBox		->setValue(savedSize.width());
	d->widget->heightSpinBox	->setValue(savedSize.height());
}

	void
KomNotesConfigService::save()
{
	if (0 == d->widget)
	{
		qDebug("No widget !");
		return;
	}

	QSettingsExtra::writeEntry
		(
		 Settings::instance(),
		 configPrefix + QString("Default font"),
		 d->widget->fontChooserButton->selectedFont()
		);

	QSettingsExtra::writeEntry
		(
		 Settings::instance(),
		 configPrefix + QString("Default color"),
		 d->widget->colorButton->color()
		);


	QSettingsExtra::writeEntry
		(
		 Settings::instance(),
		 configPrefix + QString("Default size"),
		 QSize(d->widget->widthSpinBox->value(), d->widget->heightSpinBox->value())
		);
 	QSettingsExtra::writeEntry
		(
		 Settings::instance(),
		 configPrefix + QString("Default text color"),
		 QColor(d->widget->textColorButton->color() )
		);

}

	void
KomNotesConfigService::defaults()
{
	if (0 == d->widget)
		return;

  QFont defaultFont;
  QFont font
    = QSettingsExtra::readFontEntry( Settings::instance(), "/Aethera/General/Default font", &defaultFont );
	d->widget->fontChooserButton->setSelectedFont( font );

	d->widget->colorButton->setColor(Qt::yellow);

	d->widget->widthSpinBox		->setValue(defaultWidth);
	d->widget->heightSpinBox	->setValue(defaultHeight);

  d->widget->textColorButton->setColor( Qt::black );
  d->widget->colorButton->setColor( Qt::yellow );
}

QPixmap KomNotesConfigService::icon()
{
  return QIconLoader::instance()->loadIcon( "NoteBook_Settings.png" );
}

	QString
KomNotesConfigService::caption()
{
	return QObject::tr("Notes");
}

	bool
KomNotesConfigService::modified()
{
	return false;
}

