/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
  komnotesmaintreeslots.cpp
                             -------------------
    begin                : Thu Nov  1 20:54:51 GMT 2001
    copyright            : (C) 2001 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
    author               : Fabien Armand <far@atolcd.com>
 ***************************************************************************/

#include <defines.h>
#include <vfsmodule.h>
#include <clientrmi.h>
#include <vfolder.h>
#include <vnoteobject.h>
#include <notewindow.h>
#include <komnotesbackendservice.h>
#include <komnotesmaintreeinterface.h>
#include <komnotesmaintreeslots.h>
#include <komnotesuiservice.h>
#include <folderchooserdialog.h>

#include <qinputdialog.h>
#include <qmessagebox.h>

#include <kolabsharedfolder.h>
#include <noteskolabinterface.h>

KomNotesMainTreeSlots::KomNotesMainTreeSlots
(
  KomNotesMainTreeInterface * parent,
  const char * name
)
  : QObject             (0, name),
    m_mainTreeInterface (parent)
{
}

KomNotesMainTreeSlots::~KomNotesMainTreeSlots()
{
  // Empty.
}

void KomNotesMainTreeSlots::slotNewNote()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
      return;
    }
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomNotesMainTreeSlots::slotNewNote(): no current item");
    return;
  }

  QString noteName = vfs->createObject(path);
}

void KomNotesMainTreeSlots::slotDeleteNote()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
      return;
    }
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomNotesMainTreeSlots::slotDeleteNote(): no current item");
    return;
  }

  bool ok = vfs->remove(path);

  if (!ok)
  {
    DEBUG_MSG("KomNotesMainTreeSlots::slotDeleteNote(): couldn't delete note");
  }
}

void KomNotesMainTreeSlots::slotEditNote()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
      return;
    }
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if( path.isEmpty() )
  {
    DEBUG_MSG("KomNotesMainTreeSlots::slotEditNote(): no current item");
    return;
  }

  VObject * noteRef = vfs->objectRef(path);

  if (0 == noteRef)
  {
    qDebug("Hmm, can't find note to edit");
    return;
  }

  VNoteObject * note = dynamic_cast<VNoteObject *>(noteRef);

  if (0 == note)
  {
    qDebug("dynamic_cast from VObject to VNoteObject failed");
    return;
  }

  qDebug("STUB - need to edit note");
}

void KomNotesMainTreeSlots::slotNewNoteGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
      return;
    }
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomNotesMainTreeSlots::slotNewNoteGroup(): no current item");
    return;
  }
 
//  qDebug("Path: `%s'", path.ascii());

  VFolder * parentRef = vfs->folderRef(path);

  if (0 == parentRef)
  {
    qDebug("Parent does not exist");
    return;
  }

  bool ok = false;

  QString name =
    QInputDialog::getText
    (
      tr(AppName),
      tr("Please enter a name for the new folder"),
      QLineEdit::Normal,
      QString::null,
      &ok,
      0 // FIXME: Parent widget ?
    );

  if (ok && !name.isEmpty())
  {
    VFolder * folderRef = vfs->folderRef(path + "/testing");

    if (0 != folderRef)
    {
      qDebug("Folder already exists");
    }
    else
    {
      qDebug("Folder does not exist. Making");

      VFolder * f = vfs->createFolder(parentRef, name, "notes", "notes");

      if (0 == f)
      {
        qDebug("Couldn't create folder");
        return;
      }

      qDebug("Folder made successfully");
    }

    m_mainTreeInterface->createChildren(m_mainTreeInterface->currentItem());
  }
}

void KomNotesMainTreeSlots::slotDeleteNoteGroup()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
      return;
    }
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if (!path)
  {
    DEBUG_MSG("KomNotesMainTreeSlots::slotDeleteNoteGroup(): no current item");
    return;
  }
 
  // if it's a Kolab shared folder
  if( !path.find(NotesKolabInterface::instance()->kolabPath()) )
  {
    NotesKolabInterface::instance()->deleteKolabSharedFolder(path);
  }
  
  // remove the item from the list
  delete m_mainTreeInterface->currentItem();
  
  bool ok = vfs->deleteFolder(path);

  if (!ok)
  {
    qDebug("slotDeleteNoteGroup: unsuccessful");
  }
}

  void
KomNotesMainTreeSlots::slotUpdateVInstances(const QStringList & l)
{
//  m_mainTreeInterface->updateVInstances(l);
}

  void 
KomNotesMainTreeSlots::slotCreateVInstances(const QStringList & l)
{
//  m_mainTreeInterface->createVInstances(l);
} 

  void 
KomNotesMainTreeSlots::slotDeleteVInstances(const QStringList & l)
{
//  m_mainTreeInterface->deleteVInstances(l);
}

  void 
KomNotesMainTreeSlots::slotUpdateHierarchy(const QString & s)
{
  m_mainTreeInterface->updateHierarchy(s);
}

  void
KomNotesMainTreeSlots::slotCreate(const QString & url)
{
  m_mainTreeInterface->create(url);
}

  void
KomNotesMainTreeSlots::slotCreateVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type,
  const QString & subType,
  const QString & serverPath
)
{
  m_mainTreeInterface->createVFolder
    (
      parentFolderPath,
      folderName,
      type,
      subType,
      serverPath
    );
}

  void
KomNotesMainTreeSlots::slotDeleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type
)
{
  m_mainTreeInterface->deleteVFolder(parentFolderPath, folderName, type);
}

  void
KomNotesMainTreeSlots::slotMoveNoteGroup()
{
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  QString targetPath = FolderChooserDialog::getFolder("/Notes", 0);

  if( targetPath.isEmpty() )
    return;
  
  if(targetPath != currentPath)
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }
    
    VFolder * folder = vfs->folderRef(currentPath);
    QStringList foldersTree;
    if( folder )
    {
      // check if the target folder is a subfolder
      folder->tree(foldersTree, -1);
      for(QStringList::const_iterator it=foldersTree.begin(); it!=foldersTree.end(); ++it)
      {
        if( targetPath == (*it) )
        {
          QMessageBox::warning( 0L,
                                QObject::tr("Folder Move Warning"), 
                                QObject::tr("Moving a folder in one of its subfolders isn't allowed."),
                                QMessageBox::Ok, 
                                QMessageBox::NoButton, 
                                QMessageBox::NoButton );
          return;
        }
      }
      
      vfs->moveFolder(folder->path(), targetPath + "/" + folder->name() );
    }
  }
  else
  {
    QMessageBox::warning( 0L,
                          QObject::tr("Folder Move Warning"), 
                          QObject::tr("The new folder path is the same with the old one."),
                          QMessageBox::Ok, 
                          QMessageBox::NoButton, 
                          QMessageBox::NoButton );
  }
}

  void
KomNotesMainTreeSlots::slotRenameNoteGroup()
{
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  bool ok = false;
  QString targetName = QInputDialog::getText( QObject::tr("Folder Rename"),
                                              QObject::tr("Please insert the new folder name:"),
                                              QLineEdit::Normal,
                                              QString::null,
                                              &ok, 0L, "NotesFolderRename");
  if( ok )
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }
    
    VFolder * folder = vfs->folderRef(currentPath);
    QStringList foldersTree;
    if( folder && !targetName.isEmpty() )
    {
      vfs->moveFolder(folder->path(), folder->parentFolder()->path() + "/" + targetName);
    }
    else
    {
      QMessageBox::warning( 0L, 
                            QObject::tr("Folder Rename Warning"), 
                            QObject::tr("The new name is empty or the folder is not available."),
                            QMessageBox::Ok, 
                            QMessageBox::NoButton, 
                            QMessageBox::NoButton );
    }
  }
}

  void
KomNotesMainTreeSlots::slotExpungeNoteGroup()
{
}

  void
KomNotesMainTreeSlots::slotKolabSynch()
{
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  
  VFSModule * vfs = 0;
  ClientRMIModule * clientRMI = 0L;
  
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  assert(0 != vfs);
  assert(0 != clientRMI);

  if( (currentPath + "/") == NotesKolabInterface::instance()->kolabPath() )
  {
    // insert a message box here
    // it should synch all the subfolders
    return;
  }
  
  VFolder * folder = vfs->folderRef(currentPath);
  if(folder)
  {
    QString emailPath = NotesKolabInterface::instance()->isKolabFolder(currentPath);
    if( emailPath.isEmpty() )
    {
      emailPath = NotesKolabInterface::instance()->isKolabSharedFolder(currentPath);
    }
    
    if( !emailPath.isEmpty() )
    {
      VFolder * emailfolder = vfs->folderRef(emailPath);
      if( !emailfolder )
      {
        KolabSharedFolder::ref()->checkKolabFolders();
      }
      
      clientRMI->mount(emailPath);
      clientRMI->unmount(emailPath);
    }
  }
  else
  {
    qDebug("No such Kolab folder.");
  }
}

  void
KomNotesMainTreeSlots::slotSearchForSharedFolders()
{
  // TODO: scann all Kolab accounts or just one ?
  
  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  for (QStringList::Iterator itList = kolabList.begin();itList != kolabList.end();itList++)
  {
    KolabSharedFolder::ref()->scannSharedFolders(*itList, "notes");
  }
}
