/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          komnotesuiservice.cpp  -  the UI service from the Notes plug in
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <komnotesuiservice.h>
#include <komnotesbackendservice.h>
#include <komnotesmaintreeinterface.h>
#include <notesuipartprivate.h>
#include <notesmainwidget.h>

// QT includes
#include <qiconset.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>

#include <qiconloader.h>

/*
static const char* komnotesuiservice_metainfo[][2]=
{
    { "/name", "KomNotes UI Service" },
    { "/class", "KomNotesUIService" },
    { "/class/fqn", "KomNotesUIService" },
    { "/factory", "komnotesuiservice_factory" },
    { 0, 0 }
};
*/

kore::Interface* komnotesuiservice_factory()
{
  return new KomNotesUIService();
}
    
KomNotesUIService::KomNotesUIService()
  : m_notesMenu         (0),
    m_notesMenuId       (-1),
    m_topWidget         (0),
    m_topWidgetLayout   (0),
    m_topBannerWidget   (0),
    m_topViewWidget     (0),
    m_topMainBanner     (0),
    m_topMainView       (0),
    m_mainTreeInterface (0),
    m_actionsCreated    (false)
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Kom Notes UI Service");
  info->replaceAll("class", "KomNotesUIService");
  info->replaceAll("class/fqn", "KomNotesUIService");
  info->replaceAll("factory", "komnotesuiservice_factory");
  
  // The plug in symbols: mimeType and QIconSet.

  pluginIcon.setPixmap
    (
      BarIcon(NOTES_ON),
      QIconSet::Large,
      QIconSet::Active,
      QIconSet::Off
    );

  pluginIcon.setPixmap
    (
      BarIcon(NOTES_OFF),
      QIconSet::Large,
      QIconSet::Normal,
      QIconSet::Off
    );
  
  _mimeType = VFS_NOTES_MIMETYPE;
}

KomNotesUIService::~KomNotesUIService()
{
  if( m_mainTreeInterface )
  {
    delete m_mainTreeInterface;
    m_mainTreeInterface = 0L;
  }
}

  UIPartPrivate *
KomNotesUIService::createQTHandler()
{
  return new NotesUIPartPrivate(this);
}

  bool
KomNotesUIService::hasCapability(const QString & capabilityName)
{
  if
    (
          capabilityName == TOP_WIDGET
      ||  capabilityName == BANNER_WIDGET
      ||  capabilityName == PLUGIN_ICONSET
      ||  capabilityName == TREE_INTERFACE
    )
      return true;

  return KomUserInterfaceService::hasCapability(capabilityName);
}

  void
KomNotesUIService::createWidget(const QString & capabilityName)
{
  qDebug("KomNotesUIService::createWidget(%s)", capabilityName.ascii());

  if (capabilityName == TOP_WIDGET)
  {
    if (0 != m_topWidget)
      return;

    m_topWidget       = new QWidget(0, "NotesTopWidget");

    // Build the widgets
    VFSModule *vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
      }
    }

    Q_ASSERT(0 != vfs);

    m_topMainView = new NotesMainWidget(vfs, m_topWidget);

    m_topWidgetLayout = new QVBoxLayout(m_topWidget);

    m_topWidgetLayout->addWidget(m_topMainView);

    // Add the main widget to the plug in widgets dict.
    widgetDict.insert(TOP_WIDGET, m_topWidget);
  }
  else if( capabilityName==BANNER_WIDGET )
  {
    if( m_topMainBanner==NULL )
    {
      m_topMainBanner = new QWidget(0);
      
      QHBoxLayout *bannerLayout = new QHBoxLayout(m_topMainBanner);
      QLabel *leftBanner= new QLabel(m_topMainBanner);
      leftBanner->setPixmap( BarIcon(NOTES_BANNER_LEFT) );
      QLabel *rightBanner= new QLabel(m_topMainBanner);
      rightBanner->setPixmap( BarIcon(NOTES_BANNER_RIGHT) );
      
      bannerLayout->addWidget(leftBanner);
      bannerLayout->addStretch();
      bannerLayout->addWidget(rightBanner);
      
      // Add the banner widget to the plug in widgets dict.
      widgetDict.insert( BANNER_WIDGET, m_topMainBanner);
    }
  }
  else if (capabilityName == TREE_INTERFACE)
  {
    // XXX This seems to be unnecessary now. Why ?
  }
}

  void
KomNotesUIService::showToolbars(QMainWindow * mainWindow)
{
  if (0 == toolBar)
  {
    toolBar = new QToolBar(mainWindow, "NotesToolBar");
    createActions();

    actionDict["create"]  ->addTo(toolBar);
    actionDict["cut"]     ->addTo(toolBar);
    actionDict["copy"]    ->addTo(toolBar);
    actionDict["paste"]   ->addTo(toolBar);

    QLabel *emptyLabel = new QLabel(toolBar);
    toolBar->setStretchableWidget(emptyLabel);
  }
  else
  {
    toolBar->show();
  }
}

  void
KomNotesUIService::hideToolbars(QMainWindow *)
{
  if (0 != toolBar)
    toolBar->hide();
}

  void
KomNotesUIService::showMenus(QMenuBar * mainMenu)
{
  if (0 == mainMenu)
    return; // Huh ?

  if (0 == m_notesMenu)
    m_notesMenu = new QPopupMenu(mainMenu, "NotesMenu");

  createActions();

  m_notesMenuId = mainMenu->insertItem("&Note", m_notesMenu);

  actionDict["create"]  ->addTo(m_notesMenu);
  actionDict["cut"]     ->addTo(m_notesMenu);
  actionDict["copy"]    ->addTo(m_notesMenu);
  actionDict["paste"]   ->addTo(m_notesMenu);

#if 0
  m_notesMenu->insertSeparator();
  m_settingsAction  ->addTo(m_notesMenu);
#endif
}

  void
KomNotesUIService::hideMenus(QMenuBar * mainMenu)
{
  if (0 != mainMenu && -1 != m_notesMenuId)
  {
    mainMenu->removeItem(m_notesMenuId);
    m_notesMenuId = -1;
  }

  if (0 != m_notesMenu)
    m_notesMenu->clear();
}


  KomMainTreeInterface *
KomNotesUIService::getMainTreeInterface(QListView * parentTree)
{
  if (0 == m_mainTreeInterface)
  {
    m_mainTreeInterface = new KomNotesMainTreeInterface(parentTree);
  }

  return m_mainTreeInterface;
}

  void
KomNotesUIService::setCurrentURL(QString url)
{
  if (0 == m_topMainView)
  {
    qDebug("m_topMainView is 0");
    return;
  }

  m_topMainView->slotSetFolder(url);
}

  void
KomNotesUIService::create()
{
  if (0 != m_topMainView)
    m_topMainView->newObject();
}

  void
KomNotesUIService::cut()
{
  if (0 != m_topMainView)
    m_topMainView->cut();
}

  void
KomNotesUIService::copy()
{
  if (0 != m_topMainView)
    m_topMainView->copy();
}

  void
KomNotesUIService::paste()
{
  if (0 != m_topMainView)

    m_topMainView->paste();
}

  void
KomNotesUIService::createActions()
{
  if (m_actionsCreated)
    return;

  m_actionsCreated = true;

  NotesUIPartPrivate * partHandler =
    static_cast<NotesUIPartPrivate *>(getQTHandler());

  actionDict.insert
    (
      "settings",
      new QAction
      (
        "Settings",
        QIconLoader::instance()->iconSet("tools-small-full.xpm"),
        "&Settings...",
        0, // No accelerator key.
        partHandler,
        "Settings action"
      )
    );

  actionDict.insert
    (
      "create",
      new QAction
      (
        "Create",
        QIconLoader::instance()->iconSet("note-new.png"),
        "&Create",
        0, // No accelerator key.
        partHandler,
        "Create action"
      )
    );

  actionDict.insert
    (
      "cut",
      new QAction
      (
        "Cut",
        QIconLoader::instance()->iconSet("cut.png"),
        "&Cut",
        0, // No accelerator key.
        partHandler,

        "Cut action"
      )
    );

  actionDict.insert
    (
      "copy",
      new QAction
      (
        "Copy",
        QIconLoader::instance()->iconSet("copy.png"),
        "&Copy",
        0, // No accelerator key.
        partHandler,
        "Copy action"
      )
    );

  actionDict.insert
    (
      "paste",
      new QAction
      (
        "Paste",
        QIconLoader::instance()->iconSet("paste.png"),
        "&Paste",
        0, // No accelerator key.
        partHandler,
        "Paste action"
      )
    );

  actionDict["cut"]   ->setEnabled(false);
  actionDict["copy"]  ->setEnabled(false);
  actionDict["paste"] ->setEnabled(false);

  QObject::connect
    (
      actionDict["settings"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_settings())
    );

  QObject::connect
    (
      actionDict["create"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_create())
    );

  QObject::connect
    (
      actionDict["cut"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_cut())
    );

  QObject::connect
    (
      actionDict["copy"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_copy())
    );

  QObject::connect
    (
      actionDict["paste"],
      SIGNAL(activated()),
      partHandler,
      SLOT(slot_paste())
    );

  QObject::connect
    (
      m_topMainView,
      SIGNAL(copyPossible(bool)),
      actionDict["copy"],
      SLOT(setEnabled(bool))
    );

  QObject::connect
    (
      m_topMainView,
      SIGNAL(copyPossible(bool)),
      actionDict["cut"],
      SLOT(setEnabled(bool))
    );

  QObject::connect
    (
      m_topMainView,
      SIGNAL(pastePossible(bool)),
      actionDict["paste"],
      SLOT(setEnabled(bool))
    );
}

  void
KomNotesUIService::buildTree()
{
  if (0 == m_mainTreeInterface)
  {
    qDebug("KomContactsUIService::buildTree: No m_mainTreeInterface");
    return;
  }

  NotesUIPartPrivate * partHandler =
    static_cast<NotesUIPartPrivate *>(getQTHandler());

  QObject::connect
    (
      partHandler,
      SIGNAL(create(const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreate(const QString &))
    );

      QObject::connect
        (
          partHandler,
          SIGNAL(updateVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotUpdateVInstances(const QStringList &))
        );

      QObject::connect
        (
          partHandler,
          SIGNAL(createVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotCreateVInstances(const QStringList &))
        );

      QObject::connect
        (
          partHandler,
          SIGNAL(deleteVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotDeleteVInstances(const QStringList &))
        );
        
//  QObject::connect
//    (
//      partHandler,
//      SIGNAL(updateVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotUpdateVInstances(const QStringList &))
//    );
//
//  QObject::connect
//    (
//      partHandler,
//      SIGNAL(createVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotCreateVInstances(const QStringList &))
//    );
//
//  QObject::connect
//    (
//      partHandler,
//      SIGNAL(deleteVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotDeleteVInstances(const QStringList &))
//    );

  QObject::connect
    (
      partHandler,
      SIGNAL(createVFolder(const QString &, const QString &, const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreateVFolder(const QString &, const QString &, const QString &, const QString &, const QString &))
    );

  QObject::connect
    (
      partHandler,
      SIGNAL(deleteVFolder(const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotDeleteVFolder(const QString &, const QString &, const QString &))
    );
  
  m_mainTreeInterface->show( m_topMainView->getCurrentFolder() );
}

  void
KomNotesUIService::removeTree()
{
  if (0 == m_mainTreeInterface)
  {
    qDebug("KomContactsUIService::buildTree: No m_mainTreeInterface");
    return;
  }

  NotesUIPartPrivate * partHandler =
    static_cast<NotesUIPartPrivate *>(getQTHandler());

  QObject::disconnect
    (
      partHandler,
      SIGNAL(create(const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreate(const QString &))
    );

      QObject::disconnect
        (
          partHandler,
          SIGNAL(updateVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotUpdateVInstances(const QStringList &))
        );

      QObject::disconnect
        (
          partHandler,
          SIGNAL(createVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotCreateVInstances(const QStringList &))
        );

      QObject::disconnect
        (
          partHandler,
          SIGNAL(deleteVInstances(const QStringList &)),
          m_topMainView,
          SLOT(slotDeleteVInstances(const QStringList &))
        );
    
//  QObject::disconnect
//    (
//      partHandler,
//      SIGNAL(updateVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotUpdateVInstances(const QStringList &))
//    );
//
//  QObject::disconnect
//    (
//      partHandler,
//      SIGNAL(createVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotCreateVInstances(const QStringList &))
//    );
//
//  QObject::disconnect
//    (
//      partHandler,
//      SIGNAL(deleteVInstances(const QStringList &)),
//      m_mainTreeInterface->qObject(),
//      SLOT(slotDeleteVInstances(const QStringList &))
//    );
    
  QObject::disconnect
    (
      partHandler,
      SIGNAL(createVFolder(const QString &, const QString &, const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotCreateVFolder(const QString &, const QString &, const QString &, const QString &, const QString &))
    );

  QObject::disconnect
    (
      partHandler,
      SIGNAL(deleteVFolder(const QString &, const QString &, const QString &)),
      m_mainTreeInterface->qObject(),
      SLOT(slotDeleteVFolder(const QString &, const QString &, const QString &))
    );
}

  QString
KomNotesUIService::prettyName() const
{
  return "Notes";
}

  void
KomNotesUIService::execAboutDialog(QWidget * parent) const
{
  QMessageBox::about
    (
      parent,
      QString("About plugin: %1").arg(prettyName()),
      QString("This plugin handles sticky notes")
    );
}

