/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
        komnotesuiservice.h  -  the UI service from the Notes plug in
                             -------------------
    begin                : Thu Sep 27 15:55:16 EEST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    author               : Iustin A. <aiustin@thinvest.ro>
 ***************************************************************************/

#ifndef KOMNOTESUISERVICE_H
#define KOMNOTESUISERVICE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <komaetheraplugin.h>
#include <komaetheratools.h>

#include <qlabel.h>
#include <qasciidict.h>

// rikkus: #defines ?
#define NOTES_ON              "NoteBook2_active.png"
#define NOTES_OFF             "NoteBook2_inactive.png"
#define NOTES_BANNER          "aethera-banner-notes.png"
#define NOTES_BANNER_LEFT     "NoteBook_Type.png"
#define NOTES_BANNER_RIGHT    "NoteBook_Banner2.png"

class KomNotesMainTreeInterface;
class KomNotesBackendService;
class NotesMainWidget;
class QPopupMenu;

extern "C"
{
    NOTE_PLUGIN_API kore::Interface* komnotesuiservice_factory();
};

class NOTE_PLUGIN_API KomNotesUIService : public KomUserInterfaceService
{
  public:

    KomNotesUIService();
    virtual ~KomNotesUIService();

    virtual KomMainTreeInterface * getMainTreeInterface(QListView * parentTree);
    
    // From UIPart.

    /** Query for capabilities.*/
    virtual bool hasCapability(const QString&);

    /** Create a new widget corresponding to the string.*/
    virtual void createWidget(const QString&);

    /** Show the part toolbars.*/
    virtual void showToolbars(QMainWindow *);

    /** Hide the part toolbars.*/
    virtual void hideToolbars(QMainWindow *);

    /** Show the part manus.*/
    virtual void showMenus(QMenuBar *);

    /** Hide the part menu.*/
    virtual void hideMenus(QMenuBar *);

    virtual void setCurrentURL(QString);

    virtual void create();
    virtual void cut();
    virtual void copy();
    virtual void paste();

    virtual void buildTree();
    virtual void removeTree();

    virtual QString prettyName() const;

    virtual void execAboutDialog(QWidget * parent) const;

  protected:

    /** Create the QT signal/slot handler.*/

    virtual UIPartPrivate *createQTHandler(void);
    virtual void createActions();
    
    QPopupMenu                * m_notesMenu;
    int                         m_notesMenuId;
    QWidget                   * m_topWidget;
    QVBoxLayout               * m_topWidgetLayout;
    KomContainerWidget        * m_topBannerWidget;
    KomContainerWidget        * m_topViewWidget;
    QWidget                   * m_topMainBanner;
    NotesMainWidget           * m_topMainView;
    KomNotesMainTreeInterface * m_mainTreeInterface;
    bool                        m_actionsCreated;
};
#endif



