/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.

---------------------------------------------------------------------------------
    copyright             : (C) 2003 by theKompany.com
                            (C) 2004 by Atol C&D
    author                : Eugen Constantinescu <eug@thekompany.com>
                            Fabien Armand <far@atolcd.com>
    date                  : 15 Dec 2003
---------------------------------------------------------------------------------
*/

// own includes
#include "noteskolabinterface.h"
#include "komnotesrmiservice.h"
#include "vnoteobject.h"
#include "notewindow.h"

// Aethera
#include <mailobj.h>
#include <partobj.h>
#include <vfsmodule.h>
#include <clientrmi.h>
#include <vfolder.h>
#include <appstatus.h>
#include <mailagentservice.h>

// qt
#include <qtextcodec.h>
#include <qtimer.h>

// support for kolab shared folder
#include <kolabsharedfolder.h>

NotesKolabInterface* NotesKolabInterface::instance_ = 0L;

class NotesKolabInterface::Private
{
  public:
  
    Private(NotesKolabInterface*);
    ~Private();
  
    VFSModule * vfs;
    ClientRMIModule * clientRMI;
    MailAgentService * mailAgent;
    QString accountName;
    QDict<VNoteObject> synchObjects;
    bool lock;
    
  private:
  
    NotesKolabInterface * parent_;
};

NotesKolabInterface::Private::Private(NotesKolabInterface * parent)
  : vfs(0L),
    clientRMI(0L),
    mailAgent(0L),
    synchObjects(199),
    lock(false),
    parent_(parent)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILAGENT_SERVICE );
    if( ref.isValid() )
       mailAgent = static_cast<MailAgentService*>( ref.instance() );
  }
  
  assert(0 != vfs);
  assert(0 != clientRMI);
}

NotesKolabInterface::Private::~Private()
{
}

// --------------

NotesKolabInterface * NotesKolabInterface::instance()
{
  if( !instance_ )
  {
    instance_ = new NotesKolabInterface();
  }
  
  return instance_;
}

NotesKolabInterface::NotesKolabInterface()
{
  d = new Private(this);
}

NotesKolabInterface::~NotesKolabInterface()
{
  delete d;
  d = 0;
}

QString NotesKolabInterface::kolabPath() const
{
  return d->vfs->root() + "/Notes/Kolab/";
}

void NotesKolabInterface::deleteKolabSharedFolder(const QString & path) const
{
  VFolder * notesFolder = d->vfs->folderRef(path);
  if( notesFolder )
  {
    // try with the current folder
    QString emailFolder = isKolabSharedFolder(path);
    if( !emailFolder.isEmpty() )
    {
      KolabSharedFolder::ref()->removeSharedFolder(d->accountName, emailFolder);
    }
    
    // check the subfolders
    VFolderList folderRefList;
    notesFolder->tree(folderRefList, -1);
    
    for(VFolderListIterator it(folderRefList); it.current(); ++it)
    {
      emailFolder = isKolabSharedFolder(it.current()->path());
      if( !emailFolder.isEmpty() )
      {
        KolabSharedFolder::ref()->removeSharedFolder(d->accountName, emailFolder);
      }
    }
  }
}

QString NotesKolabInterface::isKolabSharedFolder(const QString & path) const
{
  QString emailFolder;
  QString notesKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( !path.find(notesKolab + *itList) )
    {
      QString account = (*itList);
      QString mailbox = path.mid( QString(notesKolab + account + "/").length() );
      
      if( KolabSharedFolder::ref()->isKolabSharedFolder(account, mailbox) )
      {
        emailFolder = d->vfs->root() + "/Mail/" + account + "/" + mailbox;
        d->accountName = account;
      }
      
      break;
    }
  }

  return emailFolder;
}

QString NotesKolabInterface::isParentKolabSharedFolder(const QString & path) const
{
  QString emailFolder;
  QString notesKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( !path.find(notesKolab + *itList) )
    {
      QString account = (*itList);
      QString mailbox = path.mid( QString(notesKolab + account + "/").length() );
      emailFolder = d->vfs->root() + "/Mail/" + account + "/" + mailbox;
      
      QStringList sharedFolders;
      KolabSharedFolder::ref()->sharedFolders(sharedFolders, account, "notes");
      
      for(QStringList::Iterator it=sharedFolders.begin(); it!=sharedFolders.end(); ++it)
      {
        if( !(*it).find(emailFolder) )
        {
          return emailFolder;
        }
      }
    }
  }

  return QString::null;
}

QString NotesKolabInterface::isKolabFolder(const QString & path) const
{
  QString emailFolder;
  QString notesKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( path == notesKolab + (*itList) )
    {
      QStringList kolabFolders;
      
      KolabSharedFolder::ref()->kolabFolders(kolabFolders, (*itList), "notes");
      if( kolabFolders.count() )
      {
        emailFolder = kolabFolders.first();
        d->accountName = (*itList);
      }
      
      break;
    }
  }

  return emailFolder;
}

void NotesKolabInterface::slotSynch(const QString & accName, const QString & mailbox, bool)
{
  if( KolabSharedFolder::ref()->isKolabSharedFolder(accName, mailbox) )
  {
    slotSynchSharedFolder(kolabPath() + accName + "/" + mailbox);
    return;
  }

  if( d->lock )
    return;
  
  d->lock = true;
  
  // check for kolab object
  QString kolab = kolabPath() + accName;
  VFolder * notesFolder = d->vfs->folderRef(kolab);
  if( notesFolder )
  {
    QString notesPath = notesFolder->path();
    QStringList urls = d->vfs->children( notesPath );
    QStringList partsUrl;
    
    // close all the notes windows !!!
    
    // remove all the notes objects from the folder
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      partsUrl.append( notesPath + "/" + (*it) );
    }
    d->vfs->remove(partsUrl);
    
    QString folderPath = d->vfs->root() + "/Mail/" + accName + "/" + mailbox;
    VFolder * folder = d->vfs->folderRef( folderPath );
    if( !folder )
    {
      d->lock = false;
      qDebug("No such kolab folder.");
      return;
    }
      
    AppStatus * status = AppStatus::instance();
    if( !status )
    {
      qDebug("Kolab calendar : no status bar");
    }
    
    QString current;
    urls = d->vfs->children( folderPath );
    partsUrl.clear();
    for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
    {
      current = (*it1) + ATTR_SEP + QString::number(0);
      partsUrl.append( current );
    }
    
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), 0, partsUrl.count());
    qApp->processEvents();
    
    d->vfs->createVInstances(urls, false);
    d->vfs->createVInstances(partsUrl, false);
    
    PartObjectInterface * part = 0L;
    MailObject * object = 0L;
    int i = 1;
    for(QStringList::Iterator it2=urls.begin(); it2!=urls.end(); ++it2)
    {
      current = (*it2) + ATTR_SEP + QString::number(0);
  
      object = static_cast<MailObject*>
        (d->vfs->_fact_ref((*it2), VFS_MAILS_MIMETYPE));
      part = static_cast<PartObjectInterface*>
        (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));
        
      if( part && object )
      {
        if( "text" == part->partType() &&
            "text/plain" == part->mimeType() )
        {
          QByteArray data_ = part->data();
          if( !data_.size() )
          {
            qDebug("notes data is null");
          }
          
          QCString partData( data_ );
          QString noteText;
          
          QTextCodec * codec = QTextCodec::codecForName( part->charset() );
          if(codec)
             noteText = codec->toUnicode( partData );
          else
          {
            if( "utf-8" == part->charset().lower() )
              noteText = QString::fromUtf8( partData );
            else if( "8bit" == part->encoding().lower() )
              noteText = QString::fromLocal8Bit( partData );
            else // 7 bit
              noteText = QString::fromLatin1( partData );
          }
          
          // Create a note with text = noteText and save it
          QString noteName = d->vfs->createObject( notesFolder->path() );
          if( !noteName.isEmpty() )
          {
            VNoteObject * noteObject = static_cast<VNoteObject*>
              ( d->vfs->_fact_ref(notesFolder->path() + "/" + noteName, VFS_NOTES_MIMETYPE) );
            
            if( noteObject )
            {
              // add the object for synchronization
              d->synchObjects.insert(noteObject->path(), (VNoteObject*)0);
              
              QString uid;
              QString subject = object->subject();
              
              // find the UID in the Subject line or in the Note-Id header line
              if( "Note-Id: " == subject.left(9) )
              {
                uid = subject.mid(9);
//                qDebug("UID found in Subject = %s", uid.latin1());
              }
              else if( !subject.isEmpty() && 38 == subject.length() )
              {
                // Citadel Ex: {145a7833-4b73-4989-9d82-e87ddcf40c37}
                uid = subject;
              }
              
              // try the header
              if( uid.isEmpty() )
              {
                QCString header = object->header();
                int iFind = header.find("Note-Id: ");
                int iEnd = header.find("\n", iFind + 9);
                if( iFind != -1 && iEnd != -1 )
                {
                  uid = header.mid(iFind + 9, iEnd - iFind - 9);
//                  qDebug("UID found in Header = %s", uid.latin1());
                }
              }
              if( uid.isEmpty() )
              {
                qDebug("UID isn't found.");
              }
              
              noteObject->setUID( uid );
              noteObject->setText( noteText );
              noteObject->save();
            }
            else
            {
              qDebug("Couldn't create the note object.");
            }
          }
        }
      }
      else
      {
        qDebug("Warning: the note for %s isn't available.", current.latin1());
      }
      
      if( !(i%10) )
      {
        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), i);
        qApp->processEvents(1);
      }
      i++;
    }
    
    status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, tr("Kolab notes are loaded"));
    QTimer::singleShot(0, this, SLOT(slotClearSynch()));
    qApp->processEvents();
  }
  else
  {
    d->lock = false;
    qDebug("No such kolab folder.");
    return;
  }
  
  d->lock = false;
}

void NotesKolabInterface::slotClearSynch()
{
  d->synchObjects.clear();
}

void NotesKolabInterface::slotSynchSharedFolder(const QString & notesPath)
{
  QString folderPath = isKolabSharedFolder(notesPath);
  if( d->lock || folderPath.isEmpty() )
  {
    return;
  }
  
  d->lock = true;

  // check for kolab object

  VFolder * notesFolder = d->vfs->folderRef(notesPath);
  if( notesFolder )
  {
    QString notesPath = notesFolder->path();
    QStringList urls = d->vfs->children( notesPath );
    QStringList partsUrl;

    // close all the notes windows !!!

    // remove all the notes objects from the folder
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      partsUrl.append( notesPath + "/" + (*it) );
    }
    d->vfs->remove(partsUrl);

    VFolder * folder = d->vfs->folderRef( folderPath );
    if( !folder )
    {
      d->lock = false;
      qDebug("No such kolab folder.");
      return;
    }

    AppStatus * status = AppStatus::instance();
    if( !status )
    {
      qDebug("Kolab calendar : no status bar");
    }

    QString current;
    urls = d->vfs->children( folderPath );
    partsUrl.clear();
    for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
    {
      current = (*it1) + ATTR_SEP + QString::number(0);
      partsUrl.append( current );
    }

    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), 0, partsUrl.count());
    qApp->processEvents();

    d->vfs->createVInstances(urls, false);
    d->vfs->createVInstances(partsUrl, false);

    PartObjectInterface * part = 0L;
    MailObject * object = 0L;
    int i = 1;
    for(QStringList::Iterator it2=urls.begin(); it2!=urls.end(); ++it2)
    {
      current = (*it2) + ATTR_SEP + QString::number(0);

      object = static_cast<MailObject*>
        (d->vfs->_fact_ref((*it2), VFS_MAILS_MIMETYPE));
      part = static_cast<PartObjectInterface*>
        (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));

      if( part && object )
      {
        if( "text" == part->partType() &&
            "text/plain" == part->mimeType() )
        {
          QByteArray data_ = part->data();
          if( !data_.size() )
          {
            qDebug("notes data is null");
          }

          QCString partData( data_ );
          QString noteText;

          QTextCodec * codec = QTextCodec::codecForName( part->charset() );
          if(codec)
             noteText = codec->toUnicode( partData );
          else
          {
            if( "utf-8" == part->charset().lower() )
              noteText = QString::fromUtf8( partData );
            else if( "8bit" == part->encoding().lower() )
              noteText = QString::fromLocal8Bit( partData );
            else // 7 bit
              noteText = QString::fromLatin1( partData );
          }

          // Create a note with text = noteText and save it
          QString noteName = d->vfs->createObject( notesFolder->path() );
          if( !noteName.isEmpty() )
          {
            VNoteObject * noteObject = static_cast<VNoteObject*>
              ( d->vfs->_fact_ref(notesFolder->path() + "/" + noteName, VFS_NOTES_MIMETYPE) );

            if( noteObject )
            {
              // add the object for synchronization
              d->synchObjects.insert(noteObject->path(), (VNoteObject*)0);
              
              QString uid;
              QString subject = object->subject();

              // find the UID in the Subject line or in the Note-Id header line
              if( "Note-Id: " == subject.left(9) )
              {
                uid = subject.mid(9);
//                qDebug("UID found in Subject = %s", uid.latin1());
              }
              else if( !subject.isEmpty() && 38 == subject.length() )
              {
                // Citadel Ex: {145a7833-4b73-4989-9d82-e87ddcf40c37}
                uid = subject;
              }
              
              // try the header
              if( uid.isEmpty() )
              {
                QCString header = object->header();
                int iFind = header.find("Note-Id: ");
                int iEnd = header.find("\n", iFind + 9);
                if( iFind != -1 && iEnd != -1 )
                {
                  uid = header.mid(iFind + 9, iEnd - iFind - 9);
//                  qDebug("UID found in Header = %s", uid.latin1());
                }
              }
              if( uid.isEmpty() )
              {
                qDebug("UID isn't found.");
              }

              noteObject->setUID( uid );
              noteObject->setText( noteText );
              noteObject->save();
            }
            else
            {
              qDebug("Couldn't create the note object.");
            }
          }
        }
      }
      else
      {
        qDebug("Warning: the note for %s isn't available.", current.latin1());
      }

      if( !(i%10) )
      {
        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, tr("Kolab data is loading..."), i);
        qApp->processEvents();
      }
      i++;
    }

    status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, tr("Kolab notes are loaded"));
    QTimer::singleShot(0, this, SLOT(slotClearSynch()));
    qApp->processEvents();
  }
  else
  {
    d->lock = false;
    qDebug("No such kolab folder.");
    return;
  }

  d->lock = false;
  
}
void NotesKolabInterface::slotDelete(const QString & path)
{
  if( d->lock )
  {
    return;
  }
  
  VNoteObject * noteObject = static_cast<VNoteObject*>
    ( d->vfs->_fact_ref(path, VFS_NOTES_MIMETYPE) );
  
  if( noteObject )
  {
    if( d->synchObjects[noteObject->path()] )
    {
      // it's in synchronization
      return;
    }
    
    QString mailFolder = findMailFolder( noteObject->parentFolder() );
    if( !mailFolder.isEmpty() )
    {
      QString mailObject = findMailByUID(mailFolder, noteObject->uid());
      if( mailObject.isEmpty() )
      {
        qDebug("slotDelete : The mail object couldn't be found.");
      }
      else
      {
        d->vfs->remove(mailObject);
      }
    }
  }
}

void NotesKolabInterface::slotUpdate(const QString & path)
{
  if( d->lock )
  {
    return;
  }
  
  VNoteObject * noteObject = static_cast<VNoteObject*>
    ( d->vfs->_fact_ref(path, VFS_NOTES_MIMETYPE) );
  
  if( noteObject )
  {
    if( d->synchObjects[noteObject->path()] )
    {
      // it's in synchronization
      return;
    }
    
    QString mailFolder = findMailFolder( noteObject->parentFolder() );
    if( !mailFolder.isEmpty() )
    {
      Account kolabAccount = d->clientRMI->getAccount(d->accountName);
      QString mailObject = findMailByUID(mailFolder, noteObject->uid());
      if( mailObject.isEmpty() )
      {
        qDebug("slotDelete : The mail object couldn't be found.");
      }
      else
      {
        d->vfs->remove(mailObject);
      }
      
      if( d->mailAgent )
      {
        QString email;
        if( kolabAccount.name.isEmpty() )
          email = kolabAccount.from;
        else
          email = QString("\"") + kolabAccount.name + "\"" +
                          " <"  + kolabAccount.from + ">";
          
        d->mailAgent->appendToFolder(
          mailFolder,
          kolabAccount.accname,
          email,
          email,
          QString::null,
          QString::null,
          QString("Note-Id: ") + noteObject->uid(),
          noteObject->text(),
          "text/plain",
          QString("X-KOrg-Note-Id: ") + noteObject->uid() + "\n");
      }
    }
  }
}

void NotesKolabInterface::slotAdd(const QString & path)
{
  if( d->lock )
  {
    return;
  }
  
  VNoteObject * noteObject = static_cast<VNoteObject*>
    ( d->vfs->_fact_ref(path, VFS_NOTES_MIMETYPE) );
  
  if( noteObject )
  {
    // it's in synchronization
    if( d->synchObjects[noteObject->path()] )
    {
      return;
    }
    
    // it's empty
    if( noteObject->uid().isEmpty() || 
        noteObject->text().isEmpty() )
    {
      NoteWindowManager * manager = NoteWindowManager::instance();
      if(manager)
      {
        manager->slotOpenNoteWindow(noteObject->path(), NoteWindow::RemoveIfUnedited, 0);
      }
      
      return;
    }
    
    // Add it as email too
    QString mailFolder = findMailFolder( noteObject->parentFolder() );
    if( !mailFolder.isEmpty() )
    {
      Account kolabAccount = d->clientRMI->getAccount(d->accountName);
      if( d->mailAgent )
      {
        QString email;
        if( kolabAccount.name.isEmpty() )
          email = kolabAccount.from;
        else
          email = QString("\"") + kolabAccount.name + "\"" +
                          " <"  + kolabAccount.from + ">";
          
        d->mailAgent->appendToFolder(
          mailFolder,
          kolabAccount.accname,
          email,
          email,
          QString::null,
          QString::null,
          QString("Note-Id: ") + noteObject->uid(),
          noteObject->text(),
          "text/plain",
          QString("X-KOrg-Note-Id: ") + noteObject->uid() + "\n");
      }
    }
  }
}

QString NotesKolabInterface::findMailByUID
  ( const QString & folderPath,
    const QString & uid ) const
{
  VFolder * folder = d->vfs->folderRef(folderPath);
  if(folder)
  {
    bool bFind = false;
    QStringList urls = d->vfs->children( folderPath );
    d->vfs->createVInstances(urls, false);
    
    MailObject * object = 0L;
    // try to find the UID in the subject line
    for(QStringList::Iterator it=urls.begin(); !bFind && it!=urls.end(); ++it)
    {
      object = static_cast<MailObject*> (d->vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE));
      if(object)
      {
        if( -1 != object->subject().find(uid) )
        {
          bFind = true;
        }
      }
    }
    
    // hmm, try to find the UID in the header
    // this can take a while :(
    if(!bFind)
    {
      QCString header;
      QCString noteID = QCString("Note-Id: ") + uid.utf8();
      
      for(QStringList::Iterator it=urls.begin(); !bFind && it!=urls.end(); ++it)
      {
        object = static_cast<MailObject*>
          (d->vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE));
        if(object)
        {
          header = object->header();
          if( -1 != header.find(noteID) )
          {
            bFind = true;
          }
        }
      }
    }
    
    if(bFind)
    {
      return object->path();
    }
    
    folder->freeObjects();
  }
  
  return QString::null;
}

QString NotesKolabInterface::findMailFolder(VFolder * folder) const
{
  QString emailFolder;
  
  if( folder )
  {
    emailFolder = isKolabFolder( folder->path() );
    if( emailFolder.isEmpty() )
    {
      emailFolder = isKolabSharedFolder( folder->path() );
    }
  }
  
  return emailFolder;
}

