/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
    notesmainwidget.cpp
                             -------------------
    begin                : Sun Nov 25 16:09:14 GMT 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen Constantinescu <eug@thekompany.com>
 ***************************************************************************/

#include "notesmainwidget.h"
#include <vnoteobject.h>
#include <notewindow.h>
#include <notesiconviewitem.h>
#include <noteskolabinterface.h>

#include <folderchooserdialog.h>
#include <clipboard.h>
#include <appstatus.h>
#include <vdragobject.h>
#include <mailagentservice.h>

#include <qcursor.h>
#include <qpopupmenu.h>

/*
 * m_cutList concept:
 * When we cut items, we add their paths to m_cutList.
 * When we refresh, we avoid adding anything on m_cutList.
 * When we paste, we remove any pasted items from m_cutList.
 * When we start a new cut or copy, or we are deleted, we remove any
 * items in m_cutList from VFS.
 */

NotesMainWidget::NotesMainWidget
(
  VFSModule   * vfsModule,
  QWidget     * parent,
  const char  * name
)
  : SortingWidget(vfsModule, parent, name),
    m_currentPath("/Aethera/VFS/Notes"),
    bEdit(false)
{
  setSelectionMode(QIconView::Extended);

  connect
    (
      this,
      SIGNAL(doubleClicked(QIconViewItem *)),
      SLOT(slotClicked(QIconViewItem *))
    );

  connect
    (
      this,
      SIGNAL(contextMenuRequested(QIconViewItem *, const QPoint &)),
      SLOT(slotContextMenuRequested(QIconViewItem *, const QPoint &))
    );

  connect
    (
      this,
      SIGNAL(selectionChanged()),
      SLOT(slotSelectionChanged())
    );
}

NotesMainWidget::~NotesMainWidget()
{
  deleteItemsInCutList();
}

  void
NotesMainWidget::slotSetFolder(QString path)
{
//  qDebug("NotesMainWidget::slotSetFolder(%s)", path.ascii());

  if (path.isEmpty())
  {
    // Use same path.

    path = m_currentPath;
  }

  if (path.left(18) != "/Aethera/VFS/Notes")
  {
//    qDebug("Not our folder. Give up.");
    return;
  }

  m_currentPath = path;

//  qDebug("Fixed path is now %s", path.ascii());

  setUpdatesEnabled(false);

  clear();

  assert(0 != vfs);

  VFolder * folder = vfs->folderRef(path);

  if (0 == folder)
  {
    qDebug("Couldn't get ref to folder");
    setUpdatesEnabled(true);
    return;
  }

  QList<VObject> & objectList = folder->objectList();

//  qDebug("There are %d objects in this folder", objectList.count());

//  qDebug("There are %d objects in the cut list", m_cutList.count());

  AppStatus * status = AppStatus::instance();
  status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
    tr("Loading..."), 0, objectList.count());
  
  int i=1;
  for (QPtrListIterator<VObject> it(objectList); it.current(); ++it)
  {
    VNoteObject * note = static_cast<VNoteObject *>(it.current());

    if (0 == m_cutList.contains(note->path()))
    {
      (void) new NotesIconViewItem(note, this);
    }
    
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting,
      tr("Loading..."), i);
    i++;
  }

  status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, tr("The folder is loaded."));
  
  setUpdatesEnabled(true);
}


  QStringList
NotesMainWidget::getSelection()
{
  QStringList ret;
  VNoteObject * note = 0;

  for (QIconViewItem * item = firstItem(); item != 0; item = item->nextItem())
  {
    if (item->isSelected())
    {
      note = (static_cast<NotesIconViewItem *>(item))->note();

      if(!note)
      {
        qDebug("0 note in NotesIconViewItem");
      }
      else
      {
        ret.append(note->path());
      }
    }
  }

  return ret;
}

  VFolder *
NotesMainWidget::parentFolder()
{
  QString path(m_currentPath);

  if (path.isEmpty())
  {
    path = "/Aethera/VFS/Notes";
  }

  VFolder * folder = vfs->folderRef(path);

  Q_ASSERT(0 != folder);

  if (0 == folder)
    return 0;

  else
    return vfs->parentFolder(folder);
}

  void
NotesMainWidget::newObject()
{
  if (m_currentPath.isEmpty())
  {
    qDebug("Hmm, I have no path");
    return;
  }

  bEdit = true;
  vfs->createObject(m_currentPath);
  bEdit = false;
}

  void
NotesMainWidget::selectAll()
{
  // Which wanker hid QIconView::selectAll(bool) ?
  QIconView::selectAll(true);
}

  void
NotesMainWidget::unselectAll()
{
  clearSelection();
}

  bool
NotesMainWidget::isSelectionEmpty()
{
  return true;
}

  bool
NotesMainWidget::isSelectionSingle()
{
  switch (selectionMode())
  {
    case Single:
      return true;
      break;

    default:
      return false;
      break;
  }
}

  bool
NotesMainWidget::isSelectionMultiple()
{
  switch (selectionMode())
  {
    case Multi:
    case Extended:
      return true;
      break;

    default:
      return false;
      break;
  }
}

  void
NotesMainWidget::slotClicked(QIconViewItem * item)
{
  VNoteObject * note = static_cast<NotesIconViewItem *>(item)->note();
  if(note)
  {
    NoteWindowManager * manager = NoteWindowManager::instance();
    if(manager)
    {
      manager->slotOpenNoteWindow(note->path(), NoteWindow::SaveIfUnedited, 0);
    }
  }
  else
  {
    qDebug("Note object not found");
  }
}

  void
NotesMainWidget::slotContextMenuRequested(QIconViewItem * i, const QPoint &)
{
  enum { Create, Remove, Cut, Copy, Paste, Edit, MoveTo, CopyTo, SendByMail};

  int action = -1;

  {
    QPopupMenu pop(this);

    if (0 == i)
    {
      pop.insertItem("&New note", Create);
      pop.insertItem("&Paste",    Paste);

      if (!canPaste())
        pop.setItemEnabled(Paste, false);
    }
    else
    {
      pop.insertItem("&Edit", Edit);
      pop.insertSeparator();
      pop.insertItem("&Cut",  Cut);
      pop.insertItem("Copy", Copy);
      pop.insertSeparator();
      pop.insertItem("&Remove", Remove);
      pop.insertSeparator();
      pop.insertItem("Copy to...", CopyTo);
      pop.insertItem("&Move to...", MoveTo);
      pop.insertSeparator();
      pop.insertItem("Send by mail", SendByMail);
    }

    action = pop.exec(QCursor::pos());
  }

  // Get selection.

  QPtrList<NotesIconViewItem> selection;

  for (QIconViewItem * item = firstItem(); item != 0; item = item->nextItem())
    if (item->isSelected())
      selection.append(static_cast<NotesIconViewItem *>(item));

  switch (action)
  {
    case Create:  newObject();        break;
    case Remove:  remove(selection);  break;
    case Edit:    slotClicked(i);     break;
    case Cut:     cut(selection);     break;
    case Copy:    copy(selection);    break;
    case Paste:   paste();            break;
    case MoveTo:  moveTo(selection);  break;
    case CopyTo:  copyTo(selection);  break;
    case SendByMail:  sendByMail(selection);   break;
    default:                          break;
  }
}

  bool
NotesMainWidget::canPaste() const
{
  QMimeSource * data = ClipboardWrapper::thisInstance()->data();

  if (0 == data)
  {
    qDebug("No data in clipboard.");
    return false;
  }

  if (!QUriDrag::canDecode(data))
  {
    qDebug("Not QUriDrag");
    return false;
  }

  QStringList pathList;

  if (!QUriDrag::decodeLocalFiles(data, pathList))
  {
    qDebug("QUriDrag lied when it said it could decode");
    return false;
  }

  if (pathList.isEmpty())
  {
    qDebug("No URIs");
    return false;
  }

  if ("/Aethera/VFS/Notes/" != pathList.first().left(19))
  {
    qDebug("Not notes on the clipboard");
    return false;
  }

  return true;
}

  void
NotesMainWidget::cut(QPtrList<NotesIconViewItem> & list)
{
  qDebug("NotesMainWidget::cut: %d items to cut", list.count());

  if (list.isEmpty())
  {
    emit(pastePossible(false));
    return;
  }

  deleteItemsInCutList();

  QStringList pathList = vObjectListToPathList(list);

  qDebug("NotesMainWidget::cut: %d paths to cut", pathList.count());

  for (
    QStringList::ConstIterator it(pathList.begin());
    it != pathList.end();
    ++it
  )
  {
    m_cutList << *it;
  }


  // Don't do list.setAutoDelete(true), because we're will update before
  // the list would delete itself.

  QUriDrag * dragObject = new QUriDrag(this, "Note(s) drag object");

  dragObject->setFileNames(m_cutList);

  ClipboardWrapper::thisInstance()->cutData(dragObject);

  slotDeleteVInstances(m_cutList);

  emit(pastePossible(true));
}

  void
NotesMainWidget::copy(const QPtrList<NotesIconViewItem> & list)
{
  if (list.isEmpty())
    return;

  deleteItemsInCutList();

  QStringList pathList = vObjectListToPathList(list);

  QUriDrag * dragObject = new QUriDrag(this, "Note(s) drag object");

  dragObject->setFileNames(pathList);

  ClipboardWrapper::thisInstance()->copyData(dragObject);

  emit(pastePossible(true));
}

  void
NotesMainWidget::paste()
{
  if (!canPaste())
  {
    emit(pastePossible(false));
    return;
  }

  QMimeSource * mimeSource = ClipboardWrapper::thisInstance()->pasteData();

  if (0 == mimeSource)
  {
    qDebug("ClipboardWrapper gave me 0 to paste");
    return;
  }

  if (!QUriDrag::canDecode(mimeSource))
  {
    qDebug("Can't decode this mime source");
    return;
  }

  QStringList pathList;

  bool ok = QUriDrag::decodeLocalFiles(mimeSource, pathList);

  if (!ok)
  {
    qDebug("QUriDrag said it could decode, but it was lying.");
    return;
  }

  if( !pathList.count() )
    return;
  
  bool wasInCutList = (0 != m_cutList.remove(pathList[0]));

  if (wasInCutList)
  {
    // Cut 'n' paste.
    vfs->move(pathList, m_currentPath);
  }
  else
  {
    // Copy 'n' paste.
    vfs->copy(pathList, m_currentPath);
  }
}

  void
NotesMainWidget::moveTo(QPtrList<NotesIconViewItem> & list)
{
  if (list.isEmpty())
    return;

  QStringList pathList = vObjectListToPathList(list);

  QString targetPath = FolderChooserDialog::getFolder("/Notes", this);

  if (!targetPath || targetPath == m_currentPath)
    return;

  vfs->move(pathList, targetPath);
}

  void
NotesMainWidget::copyTo(const QPtrList<NotesIconViewItem> & list)
{
  if (list.isEmpty())
    return;

  QStringList pathList = vObjectListToPathList(list);

  QString targetPath = FolderChooserDialog::getFolder("/Notes", this);

  if (!targetPath || targetPath == m_currentPath)
    return;

  vfs->copy(pathList, targetPath);
}

  void
NotesMainWidget::slotUpdateVInstances(const QStringList & l)
{
  if( !l.count() || 
      !l[0].contains( getCurrentFolder() ) )
  {
    return;
  }
  
  NotesKolabInterface * kolab = NotesKolabInterface::instance();

  for (QIconViewItem * item = firstItem(); item != 0; item = item->nextItem())
  {
    NotesIconViewItem * i = static_cast<NotesIconViewItem *>(item);

    if( i && i->note() )
    {
      if (l.contains(i->note()->path()))
      {
        i->update();
        
        if( !kolab->isKolabFolder(i->note()->parentFolder()->path()).isEmpty() ||
            !kolab->isKolabSharedFolder(i->note()->parentFolder()->path()).isEmpty() )
        {
          kolab->slotUpdate( i->note()->path() );
        }
      }
    }
  }
}

  void
NotesMainWidget::slotCreateVInstances(const QStringList & l)
{
  if( !l.count() || 
      !l[0].contains( getCurrentFolder() ) )
  {
    return;
  }
  
  // TODO: Make this efficient.
  slotSetFolder("");
  
  VNoteObject * noteObject = 0;
  
  // Kolab
  NotesKolabInterface * kolab = NotesKolabInterface::instance();
  if( l.count() && kolab )
  {
    bool bNormal = false;
    bool bKolab = false;
    
    for(QStringList::ConstIterator itr=l.begin(); itr!=l.end(); ++itr)
    {
      noteObject = static_cast<VNoteObject*>
        ( vfs->_fact_ref((*itr), VFS_NOTES_MIMETYPE) );
      
      if( noteObject )
      {
        if( !bNormal &&
            (bKolab || 
             !kolab->isKolabFolder(noteObject->parentFolder()->path()).isEmpty() ||
             !kolab->isKolabSharedFolder(noteObject->parentFolder()->path()).isEmpty()) )
        {
          bKolab = true;
          kolab->slotAdd( *itr );
        }
        else
        {
          bNormal = true;
          
          if( noteObject->uid().isEmpty() )
          {
            NoteWindowManager * manager = NoteWindowManager::instance();
            if(manager)
            {
              manager->slotOpenNoteWindow((*itr), NoteWindow::RemoveIfUnedited, 0);
            }
              
            break;
          }
        }
      }
    }
  }
}

  void
NotesMainWidget::slotDeleteVInstances(const QStringList & urls)
{
  if( !urls.count() || 
      !urls[0].contains( getCurrentFolder() ) )
  {
    return;
  }
  
  // TODO: Use a more efficient version.
  QStringList l = urls;

  slotSetFolder("");

  // Kolab
  NotesKolabInterface * kolab = NotesKolabInterface::instance();
  
  QPtrList<NotesIconViewItem> deathRow;
  deathRow.setAutoDelete(true);

  NoteWindowManager * manager = NoteWindowManager::instance();
  NotesIconViewItem * item = 0;
  QString itemPath;
  QStringList::Iterator itFind;
  
  for(QIconViewItem * it=firstItem(); it!=0; it=it->nextItem())
  {
    item = static_cast<NotesIconViewItem *>(it);
    if( item->note() )
    {
      itemPath = item->note()->path();
  
      itFind = l.find(itemPath);
      if( itFind != l.end() )
      {
        deathRow.append(item);
        manager->slotCloseNoteWindow(itemPath, false);
        l.remove(itFind);
        
        if( !kolab->isKolabFolder(item->note()->parentFolder()->path()).isEmpty() ||
            !kolab->isKolabSharedFolder(item->note()->parentFolder()->path()).isEmpty() )
        {
          kolab->slotDelete( itemPath );
        }
      }
    }
  }

  deathRow.clear();
}

  void
NotesMainWidget::deleteItemsInCutList()
{
  if( m_cutList.count() )
  {
    qDebug("NotesMainWidget::deleteItemsInCutList: %d items to delete",
      m_cutList.count());
  
    if( vfs->remove(m_cutList) )
    {
      qDebug("NotesMainWidget::deleteItemsInCutList succeded");
    }
    else
    {
      qDebug("NotesMainWidget::deleteItemsInCutList failed");
    }
    
    m_cutList.clear();
  }
}

  void
NotesMainWidget::cut()
{
  QPtrList<NotesIconViewItem> selection;

  for (QIconViewItem * item = firstItem(); item != 0; item = item->nextItem())
    if (item->isSelected())
      selection.append(static_cast<NotesIconViewItem *>(item));

  cut(selection);
}

  void
NotesMainWidget::copy()
{
  QPtrList<NotesIconViewItem> selection;

  for (QIconViewItem * item = firstItem(); item != 0; item = item->nextItem())
    if (item->isSelected())
      selection.append(static_cast<NotesIconViewItem *>(item));

  copy(selection);
}

  void
NotesMainWidget::slotSelectionChanged()
{
  for (QIconViewItem * item = firstItem(); item != 0; item = item->nextItem())
  {
    if (item->isSelected())
    {
      emit(copyPossible(true));
      return;
    }
  }

  emit(copyPossible(false));
}

  QStringList
NotesMainWidget::vObjectListToPathList
(
  const QPtrList<NotesIconViewItem> & list
) const
{
  if (list.isEmpty())
    return QStringList();

  QStringList pathList;

  for (QPtrListIterator<NotesIconViewItem> it(list); it.current(); ++it)
  {
    VNoteObject * note = it.current()->note();

    if (0 == note)
    {
      qDebug("NotesMainWidget::vObjectListToPathList: Found a 0 note");
      continue;
    }

    pathList << note->path();
  }

  return pathList;
}

  QDragObject *
NotesMainWidget::dragObject()
{
  if (0 == currentItem())
    return 0;

  QPoint orig = viewportToContents(viewport()->mapFromGlobal(QCursor::pos()));

  VDragObject * drag = new VDragObject(viewport());

  drag->setPixmap
    (
      *currentItem()->pixmap(),
      QPoint
      (
        currentItem()->pixmapRect().width() / 2,
        currentItem()->pixmapRect().height() / 2)
    );

  for (QIconViewItem * item = firstItem(); item != 0; item = item->nextItem())
  {
    if (item->isSelected())
    {
      drag->append
        (
          static_cast<NotesIconViewItem *>(item)->note()->path(),
          QRect
          (
            item->pixmapRect(false).x() - orig.x(),
            item->pixmapRect(false).y() - orig.y(),
            item->pixmapRect().width(), item->pixmapRect().height()
          ),
          QRect
          (
            item->textRect(false).x() - orig.x(),
            item->textRect(false).y() - orig.y(),
            item->textRect().width(), item->textRect().height()
          )
        );
    }
  }

  return drag;
}

  void
NotesMainWidget::remove(QPtrList<NotesIconViewItem> & list)
{
  if (list.isEmpty())
  {
    return;
  }

  QStringList pathList = vObjectListToPathList(list);

  assert(0 != vfs);

  if( pathList.count() )
    vfs->remove( pathList );
}

  QString
NotesMainWidget::getCurrentFolder() const
{
  return m_currentPath;
}

  void
NotesMainWidget::sendByMail(const QPtrList<NotesIconViewItem> & list)
{
  if (list.isEmpty())
    return;

  QStringList pathList = vObjectListToPathList(list);
     
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILAGENT_SERVICE );
    if( ref.isValid() )
    {
      MailAgentService* mailAgent = static_cast<MailAgentService*>( ref.instance() );
      if(mailAgent)
      {
        mailAgent->openComposer(pathList, ComposerFactory::SendNote);
      }
    }
  }
}
