/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#include <qmultilinedit.h>
#include <qstring.h>
#include <qevent.h>
#include <qdatetime.h>
#include <qlabel.h>
#include <qframe.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qfont.h>
#include <qcolor.h>
#include <qframe.h>
#include <textfitter.h>
#include <qcursor.h>
#include <qtextstream.h>
#include <qframe.h>
#include <qsizegrip.h>

#include <qiconloader.h>
#include <settings.h>
#include <qsettingsextra.h>
#include <ebutton.h>
#include <notewindow.h>
#include <textfitter.h>
#include <noteedit.h>
#include <vnoteobject.h>
#include <vfsmodule.h>

NoteWindowManager * NoteWindowManager::instance_ = 0L;

NoteWindowManager * NoteWindowManager::instance()
{
  if( !instance_ )
  {
    instance_ = new NoteWindowManager();
  }
  
  return instance_;
}

NoteWindowManager::NoteWindowManager()
{
  windows.setAutoDelete(false);
}

NoteWindowManager::~NoteWindowManager()
{
  // try to save it
  slotCloseAll(true);
}

bool NoteWindowManager::isEmpty()
{
  return windows.isEmpty();
}

NoteWindow * NoteWindowManager::findWindow(const QString & path)
{
  return windows[path];
}

void NoteWindowManager::slotOpenNoteWindow
      (const QString & path, NoteWindow::IfUnedited ifUnedited, QWidget * parent)
{
  NoteWindow * newWindow = windows[path];
  if(!newWindow)
  {
    newWindow = new NoteWindow(path, ifUnedited, parent);
    if(newWindow)
    {
      windows.insert(path, newWindow);
      newWindow->show();
    }
  }
  else
  {
    newWindow->show();
  }
}

void NoteWindowManager::slotCloseNoteWindow(const QString & path, bool bSave)
{
  NoteWindow * note = windows[path];
  if( note )
  {
    note->slotClose(bSave);
    windows.remove(path);
  }
}

void NoteWindowManager::slotCloseAll(bool bSave)
{
  for(QDictIterator<NoteWindow> it(windows); it.current(); ++it)
  {
    it.current()->slotClose(bSave);
  }
  
  windows.clear();
}


//   ******** Note Window class *******

NoteWindow::NoteWindow( const QString & path,
                        IfUnedited ifUnedited,
                        QWidget * parent )
  : QFrame(parent, "NoteWindow",
      WDestructiveClose | WStyle_Customize | WStyle_NoBorder | WType_TopLevel ),
    notePath(path),
    ifUnedited_(ifUnedited),
    vfs(0L)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  Q_ASSERT(0 != vfs);

  if( note() )
    note()->setOnDisplay(true);
  setFrameStyle(QFrame::Box | QFrame::Plain);

  QColor defaultColor( Qt::yellow );
  QColor color
    = QSettingsExtra::readColorEntry( Settings::instance(),
                                      "/Aethera/Notes/Default color",
                                      &defaultColor
                                    );

  QColorGroup light( QColor("black"), color, QColor("black"), QColor("black"),
                     QColor("black"), QColor("black"), color
                   );
  QColorGroup dark( QColor(255, 255, 140), QColor(128, 0, 0), QColor("black"), QColor("black"),
                    QColor("black"), QColor(255, 255, 140), QColor(128, 0, 0)
                  );

  setBackgroundColor(color);

  label=new QLabel(this);
  label->setText("");
  label->setAlignment( AlignHCenter);
  label->installEventFilter(this);
  label->setPalette(QPalette(dark, dark, dark));
  label->setGeometry(18, 2, width()-36, 14);
  label->setFont(QFont("helvetica", 11, QFont::Bold));

  cl_button=new EButton(BarIcon("x.xpm"), this);
  cl_button->setNormalFrameStyle(QFrame::NoFrame);
  cl_button->setFocusFrameStyle(QFrame::Box|QFrame::Plain);
  cl_button->setClickedFrameStyle(QFrame::Box|QFrame::Plain);
  cl_button->setContentMargin(0);
  cl_button->setGeometry(width()-16, 2, 14, 14);
  cl_button->installEventFilter(this);

  edit_box = new NoteEdit(this);
  edit_box->setGeometry(2, 18, width()-4 , height()-39);
  edit_box->installEventFilter(this);
  edit_box->setFocus();
  edit_box->setFrameStyle(QFrame::NoFrame); 
  edit_box->setPalette(QPalette(light, light, light));
  edit_box->setWordWrap(QMultiLineEdit::WidgetWidth);
  QColor defaultTextColor( Qt::black );
  QColor textColor
    = QSettingsExtra::readColorEntry( Settings::instance(),
                                      "/Aethera/Notes/Default text color",
                                      &defaultTextColor
                                    );
  edit_box->setColor( textColor );
  if( note() )
    edit_box->setText(note()->text());

  QFont widgetFont( edit_box->font() );
  QFont savedFont = QSettingsExtra::readFontEntry
    (Settings::instance(), "/Aethera/Notes/Default font", &widgetFont);
  edit_box->setFont( savedFont );

//  QColor widgetColor( edit_box->color() );
//  QColor savedColor = QSettingsExtra::readColorEntry
//    (Settings::instance(), "/Aethera/Notes/Default color", &widgetColor);
//  edit_box->setColor( savedColor );

  hline=new QFrame(this);
  hline->setFrameStyle(QFrame::HLine|QFrame::Plain);
  hline->setLineWidth(1);
  hline->setMidLineWidth(0);
  hline->setGeometry(2, height()-20, width()-2, 1);
  hline->installEventFilter(this);

  date_line=new QLabel(this);
  QString s;

  QDateTime d;
  if( note() )
    d=note()->time();
  s.sprintf("%02d %s %04d  %02d:%02d", d.date().day(), (const char *)d.date().monthName(d.date().month()),
    d.date().year(), d.time().hour(), d.time().minute());
  date_line->setText(s);
  date_line->setGeometry(2, height()-16, width()-20, 14);
  date_line->setPalette(QPalette(light, light, light));
  date_line->installEventFilter(this);

  resize_pix=new QSizeGrip(this);
  resize_pix->setPalette(QPalette(light, light, light));
  resize_pix->setBackgroundMode(PaletteBackground);
  resize_pix->setGeometry(width()-16, height()-16, 14, 14);

  icon=new QLabel(this);
  icon->setPixmap(BarIcon("notes.xpm"));
  icon->setGeometry(2, 2, 14, 14);
  icon->setPalette(QPalette(light, light, light));
  icon->setBackgroundMode(PaletteBackground);
  icon->installEventFilter(this);

  setMinimumSize(100, 100);

//  resize(note()->size());
  QSize defaultSize( 100, 100 );
  QSize noteSize
    = QSettingsExtra::readSizeEntry( Settings::instance(),
                                    "/Aethera/Notes/Default size",
                                    &defaultSize
                                   );
  resize( noteSize );

  connect(cl_button, SIGNAL(clicked()), this, SLOT(close()));
  connect(edit_box, SIGNAL(textChanged()), this, SLOT(setTitle()));

  popupMenu=new QPopupMenu();
  colorPopup=new QPopupMenu();
  popupMenu->insertItem(BarIcon("Cut.png"),    "Cut",   this, SLOT(cut()),  0, 1);
  popupMenu->insertItem(BarIcon("Copy.png"),   "Copy",  this, SLOT(copy()), 0, 2);
  popupMenu->insertItem(BarIcon("Paste.png"),  "Paste", this, SLOT(paste()),0, 3);
  popupMenu->insertItem(BarIcon("Delete.png"), "Delete");
  popupMenu->insertItem("Select all", this, SLOT(selectAllText()), 0, 4);
  popupMenu->insertSeparator();
  colorPopup->insertItem("Yellow", this, SLOT(setYellow()));
  colorPopup->insertItem("Red",    this, SLOT(setRed()));
  colorPopup->insertItem("Blue",   this, SLOT(setBlue()));
  colorPopup->insertItem("Green",  this, SLOT(setGreen()));
  popupMenu->insertItem("Change color to..", colorPopup);
  popupMenu->insertSeparator();
  popupMenu->insertItem("Print", this, SLOT(printNote()), 0, 6);
  popupMenu->insertSeparator();
  popupMenu->insertItem("Send to ...");
  popupMenu->insertSeparator();
  popupMenu->insertItem("Change font ...");
  popupMenu->setItemEnabled(1, false);
  popupMenu->setItemEnabled(2, false);
  popupMenu->setItemEnabled(3, false);
  setTitle();
}

NoteWindow::~NoteWindow()
{
  VNoteObject * object = note();
  
  if( object )
  {
    object->setOnDisplay(false);
    if( ifUnedited_ == NoteWindow::RemoveIfUnedited && 
        object->text().isEmpty() )
    {
      vfs->remove( object->path() );
    }
  }
}

QString NoteWindow::path() const
{
  return notePath;
}

VNoteObject * NoteWindow::note()
{
  return static_cast<VNoteObject*> ( vfs->_fact_ref(notePath, VFS_NOTES_MIMETYPE) );
}

void NoteWindow::resizeEvent(QResizeEvent *e)
{
  label->setGeometry(18, 2, width()-36, 14);
  cl_button->setGeometry(width()-16, 2, 14, 14);
  edit_box->setGeometry(2, 18, width()-4 , height()-39);  
  hline->setGeometry(2, height()-20, width()-2, 1);
  date_line->setGeometry(2, height()-16, width()-20, 14);
  resize_pix->setGeometry(width()-16, height()-16, 14, 14);
  icon->setGeometry(2, 2, 14, 14);
  setTitle();

  if( note() )
    note()->setSize(e->size());
}

bool NoteWindow::eventFilter(QObject *o, QEvent *ev)
{
  static QPoint tmp_point;
  QMouseEvent *e = static_cast<QMouseEvent *>(ev);
  
  if(!o && !e)
  {
    qDebug("NoteWindow::eventFilter - null parameters.");
    return false;
  }

  if(o==edit_box && e->type()==QEvent::MouseButtonPress && e->button()==RightButton)
  {
    showPopup();
    return true;
  }

  if(o==label)
  {
    if(ev->type()==QEvent::MouseButtonRelease)
    {
      if(e->button()==LeftButton)
      {
        dragging=false;
        label->releaseMouse();
        raise();
      }
      if(e->button()==MidButton)
        lower();
      return true;
    }
    if(ev->type()==QEvent::MouseButtonPress && e->button() == LeftButton)
    {
      pointerOffset=e->pos();
      pointerOffset.setX(pointerOffset.x()+18);
      pointerOffset.setY(pointerOffset.y()+2);
      label->grabMouse(sizeAllCursor);
      return true;
    }
    if(ev->type()==QEvent::MouseMove && label==mouseGrabber())
    {
      if(dragging) 
      {
        note()->setPosition(QCursor::pos()-pointerOffset);
        move(QCursor::pos()-pointerOffset);
      }
      else 
      {
        dragging=(
          (e->pos().x()-pointerOffset.x())
          *
          (e->pos().x()-pointerOffset.x())
          +
          (e->pos().y()-pointerOffset.y())
          *
          (e->pos().y()-pointerOffset.y())
          >= 9);
      }
      return true;
    }
    if(ev->type()==QEvent::MouseMove)
      return true;
  }

  tmp_point = QCursor::pos();

  return QWidget::eventFilter(o, ev);
}

void NoteWindow::setTitle()
{
  QString t, src;
  unsigned int i=0;

  if (!edit_box->text().isEmpty())
    t=edit_box->text();

  if (t.length() > 0)
  {
    while (i<t.length()) 
      if (t[(int)i]==0x0a || t[(int)i]==0x0d || t[(int)i]==0x20)
        i++;
      else
        break;
    while (i<t.length()) 
      if (t[(int)i]!=0x0a && t[(int)i]!=0x0d)
        src+=t[(int)i++];
      else
        break;
  }
  TextFitter tf;
  label->setText(tf.fitText(src,label->width()-10,label->font()));
}

void NoteWindow::cut()
{
  if(edit_box->hasMarkedText())
  {
    edit_box->cut();
  }
}

void NoteWindow::copy()
{
  if(edit_box->hasMarkedText())
  {
    edit_box->copy();
  }
}

void NoteWindow::paste()
{
}

void NoteWindow::setYellow()
{
  setNoteColor("yellow");
}

void NoteWindow::setBlue()
{
  setNoteColor("blue");
}

void NoteWindow::setRed()
{
  setNoteColor("red");
}

void NoteWindow::setGreen()
{
  setNoteColor("green");
}

void NoteWindow::printNote()
{
}

void NoteWindow::selectAllText()
{
  edit_box->selectAll();
}

void NoteWindow::showPopup()
{
  if(edit_box->hasMarkedText()==TRUE)
  {
    popupMenu->setItemEnabled(1, TRUE);
    popupMenu->setItemEnabled(2, TRUE);
  }
  else
  {
    popupMenu->setItemEnabled(1, FALSE);
    popupMenu->setItemEnabled(2, FALSE);
  }
  popupMenu->updateItem(1);
  popupMenu->updateItem(2);
  popupMenu->updateItem(2);
  popupMenu->popup(QCursor::pos());
}

void NoteWindow::setNoteColor(const QColor & color)
{
  if( note() )
    if (color == note()->color())
      return;

  QColorGroup light(QColor("black"), color, QColor("black"), QColor("black"),
    QColor("black"), QColor("black"), color);

  setBackgroundColor(color);

  edit_box->setPalette(QPalette(light, light, light));
  date_line->setPalette(QPalette(light, light, light));
  icon->setPalette(QPalette(light, light, light));
  icon->setBackgroundMode(PaletteBackground);
  resize_pix->setPalette(QPalette(light, light, light));
  resize_pix->setBackgroundMode(PaletteBackground); 

  repaint();

  if( note() )
  {
    note()->setColor(color);
    note()->save();
  }
}

void NoteWindow::updateText()
{
  VNoteObject * object = note();
  if( object && edit_box->isModified() )
  {
    object->setText(edit_box->text());
    object->save();
  }
}

void NoteWindow::update()
{
  VNoteObject * object = note();
  if( object )
  {
    QColor color = object->color();

    QColorGroup light(QColor("black"), color, QColor("black"), QColor("black"),
      QColor("black"), QColor("black"), color);

    setBackgroundColor(color);

    edit_box->setPalette(QPalette(light, light, light));
    edit_box->setText(object->text());

    QString s;
    QDateTime d=object->time();
    s.sprintf("%02d %s %04d  %02d:%02d", d.date().day(), (const char *)d.date().monthName(d.date().month()),
      d.date().year(), d.time().hour(), d.time().minute());
    date_line->setText(s);

    resize_pix->setPalette(QPalette(light, light, light));

    icon->setPalette(QPalette(light, light, light));

    resize(object->size());

    setTitle();
  }
}

  void
NoteWindow::close()
{
  NoteWindowManager * manager = NoteWindowManager::instance();
  if(manager)
  {
    manager->slotCloseNoteWindow(notePath, true);
  }
  else
  {
    slotClose(true);
  }
}

  void
NoteWindow::slotClose(bool bSave)
{
  if(bSave)
  {
    updateText();
  }
  
  QFrame::close();
}

