/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
//own includes
#include <vfsnoteiconview.h>

//komnotesplugin includes
#include <vnoteobject.h>

//Aethera includes
#include<vfsmodule.h> //vfs service - required t build contacts list
#include<vfolder.h>

//QT includes
#include <qregexp.h>
#include <qiconview.h>

//tkcBase includes
#include <qiconloader.h>

VFSNoteIconView::VFSNoteIconView(QWidget* parent, const char* name)
  :VFSObjectIconView(parent,name)
{
  setSelectionMode( QIconView::Single );
  setArrangement( QIconView::LeftToRight);
  setAutoArrange( true );

  connect(this,
          SIGNAL( selectionChanged(QIconViewItem*) ),
          SLOT( slotObjectSelected(QIconViewItem*) )
         );
}


VFSNoteIconView::~VFSNoteIconView()
{

}


void VFSNoteIconView::slotObjectSelected(QIconViewItem* )
{
  QString tmp = getSelectedObject();

  emit objectSelected(tmp);
}


void VFSNoteIconView::slotSetFolder(const QString& fld)
{
  noteFilesMap.clear();//clean the contacts file map
  clear(); //clear the contacts list first

  //fill the list with contact entries
  VFSModule *vfs=0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }

  //scanning for contacts
  if(vfs)
  {
    VFolder *folder=0L;

    folder = vfs->folderRef( fld );
    if(!folder)
    {
      qDebug("No such folder %s",(vfs->root()+fld).latin1());
      return;
    }
    if( "notes" == folder->type() )
    {
      QPtrList<VObject> & entries = folder->objectList();
      for(QPtrListIterator<VObject> it1(entries); it1.current(); ++it1)
        if( folder->isSimpleObject( it1.current()->name() ) )
        {
          VNoteObject * note = dynamic_cast<VNoteObject*>(it1.current());
          if(note)
            addListObject(note,folder->path()+"/"+note->name());
        }
    }
  }
}


QString VFSNoteIconView::getSelectedObject()
{
  return noteFilesMap[ currentItem() ];
}


void VFSNoteIconView::addListObject(VNoteObject* c, const QString& path)
{

  QIconViewItem *itm = new QIconViewItem(this);
  noteFilesMap[itm]=path;

  QString noteText=c->text();

  if (noteText.isEmpty())
    noteText = "[Empty]";

  if (noteText.length() > 10)
    noteText = noteText.left(10) + "...";

  itm->setText( noteText );
  itm->setPixmap( BarIcon("notes-large-yellow.png") );
}
