/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <any.h>
#include <vfsmodule.h>
#include <vnoteobject.h>

#include <qsettings.h>
#include <qsettingsextra.h>
#include <qdatastream.h>
#include <qcstring.h>

#include <komaetheratools.h>

// implementation for the note object
#define IDSTRING "VNoteObject: "

#include <settings.h>

VNoteObject::VNoteObject(const QString & _path)
  : VObject(_path),
    NoteObject()
{
  // debug
//  qDebug(IDSTRING"VNoteObject constructor: syncing object...");
  
  // restore the object properties from the server
  sync();
  
  // debug
//  qDebug(IDSTRING"NoteWidget contructor: done");
}

VNoteObject::VNoteObject(const QString & _path, const QByteArray &_data)
  : VObject(_path),
    NoteObject()
{
  // debug
//  qDebug(IDSTRING"VNoteObject constructor: syncing object...");
  
  // restore the object properties from the server
  if( !_data || !_data.size() )
    sync();
  else
    _narrow(_data);
  
  // debug
//  qDebug(IDSTRING"NoteWidget contructor: done");
}

VNoteObject::~VNoteObject()
{
  // nothing here
}

QByteArray VNoteObject::_stream()
{
  if( _uid.isEmpty() )
  {
    _uid = getUID();
  }
  
  QByteArray tdata;
  QDataStream tstream(tdata, IO_WriteOnly);

  tstream << _size << _position 
          << QString("##VER##") << QString("%1").arg( VNOTE_VERSION ) << _uid  
          << _text << _font << _color
          << _time << _category
          << _onDisplay << _extraData;
  
  return tdata;
}

void VNoteObject::_narrow(const QByteArray &tdata)
{
  // check if the stream actually contains any data
  if(tdata.size())
  {
    QDataStream tstream(tdata, IO_ReadOnly);
    
    // huh, I couldn't replace QSize with QString :(
    // this change needs a complet format update
    tstream >> _size >> _position;
    
    QString version;
    tstream >> version;
    if( "##VER##" == version )
    {
      tstream >> version;
      int iVersion = version.toInt();
      
      if( iVersion == 1 )
      {
        // unstream the object properties
        tstream >> _uid 
                >> _text >> _font >> _color
                >> _time >> _category
                >> _onDisplay >> _extraData;
      }
      else
      {
        resetToDefault();
      }
    }
    else
    {
      _text = version;
      
      // unstream the object properties
      tstream >> _font >> _color
              >> _time >> _category
              >> _onDisplay >> _extraData;
              
      // old notes didn't had UID
      // so, just generate one
      if( _uid.isEmpty() )
      {
        _uid = getUID();
      }
    }
  }
  else
  {
    resetToDefault();
  }
}

void VNoteObject::resetToDefault()
{
  QString prefix("/Aethera/Notes/");

  // provide default values if needed
  _font =
    QSettingsExtra::readFontEntry
    (Settings::instance(), prefix + "Default font", &_font);

  _size =
    QSettingsExtra::readSizeEntry
    (Settings::instance(), prefix + "Default size", &_size);

  QStringList entries = Settings::instance()->readEntry(prefix);

  if (entries.find("Default color (new_aethera)") != entries.end())
  {
    _color =
      QSettingsExtra::readColorEntry
      (Settings::instance(), prefix + "Default color (new_aethera)", &Qt::yellow);
  }
  else
  {
    // Only old config key to read :(

    QString savedColorString =
      Settings::instance()->readEntry(prefix + "Default color", "yellow").lower();

    if ("red" == savedColorString)
      _color = Qt::red;
    else if ("blue" == savedColorString)
      _color = Qt::blue;
    else if ("green" == savedColorString)
      _color = Qt::green;
    else
      _color = Qt::yellow;
  }

  _time=QDateTime::currentDateTime();
  _text="";
  _position=QPoint(0, 0);
  _category="";
  _onDisplay=true;
}

bool VNoteObject::getProperty(const QString &query, Any &property)
{
  if(query=="size")
  {
    property=Any(_size);
    return true;
  }
  if(query=="position")
  {
    property=Any(_position);
    return true;
  }
  if(query=="text")
  {
    property=Any(_text);
    return true;
  }
  if(query=="font")
  {
    property=Any(_font);
    return true;
  }
  if(query=="color")
  {
    property=Any(_color);
    return true;
  }
  if(query=="time")
  {
    property=Any(_time);
    return true;
  }
  if(query=="onDisplay")
  {
    property=Any(_onDisplay);
    return true;
  }
  if(query=="category")
  {
    property=Any(_category);
    return true;
  }
  return false;
}

bool VNoteObject::setProperty(const QString &query, Any &property)
{
  if(query=="size")
  {
    _size=property.toSize();
    return true;
  }
  if(query=="position")
  {
    _position=property.toPoint();
    return true;
  }
  if(query=="text")
  {
    _text=property.toString();
    return true;
  }
  if(query=="font")
  {
    _font=property.toFont();
    return true;
  }
  if(query=="color")
  {
    _color=property.toColor();
    return true;
  }
  if(query=="onDisplay")
  {
    _onDisplay=property.toInt();
    return true;
  }
  if(query=="category")
  {
    _category=property.toString();
    return true;
  }
  return false;
}

QueryProcessor::QueryType VNoteObject::queryType(const QString &query)
{
  if(query=="size" || query=="position" || query=="text" || query=="font" || query=="color" || query=="category")
    return QueryProcessor::ReadWriteQuery;
  
  if(query=="time")
    return QueryProcessor::ReadOnlyQuery;
  
  return QueryProcessor::InvalidQuery;
}

// implementation of the note object factory

#ifdef _QTWIN_
VObject *VNoteObjectFactory::_new(const QString & _path)
#else
VNoteObject *VNoteObjectFactory::_new(const QString & _path)
#endif
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      DEBUG_MSG("\n NO VFS Provider !\n");
      return 0;
    }
  }

  // check if it is possible to create a new contact
  if(vfs->canDefine(_path) && !vfs->reference(_path))
    return new VNoteObject(_path);
  else
    return 0;
}

#ifdef _QTWIN_
VObject *VNoteObjectFactory::_new(const QString & _path, const QByteArray &_data)
#else
VNoteObject *VNoteObjectFactory::_new(const QString & _path, const QByteArray &_data)
#endif
{
  return new VNoteObject(_path, _data);
}

