/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef FILTER_H
#define FILTER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define ACTIONS_COUNT 5

#include <qstringlist.h>

class QSettings;
class IndexClass;
class MessageDevice;
class MessageClass;
class VFSModule;

class FilterRule
{
friend class Filter;
public:
  FilterRule(QString &field, QString &function, QString &context );
  ~FilterRule();

  bool match( MessageDevice* );

  bool match( const MessageClass*, const QCString& data );

protected:
  bool matchList( QStringList );

  QString m_field;
  QString m_function;
  QString m_context;
};


class FilterAction
{
friend class Filter;
public:
  FilterAction( QString &action, QString &param );
  ~FilterAction();

  bool apply( IndexClass*, MessageDevice*, bool useForwardActions = true );

  QString messageFolder();
  QString folderToCopy();

private:
  QString m_action;
  QString m_param;

  // "execute" action stuff

};



class Filter
{
public:
  Filter( QString group, int id );
  ~Filter();

  bool match( MessageDevice*  );

  /** apply filter actions to message,
   *  dosn't check if message much to filter, use @match for this.
   */
  void apply( IndexClass*, MessageDevice*, bool useForwardActions );

  /** return folder for message ( if "forward" action present )
   *  or QString::NULL
   */
  QString messageFolder( const MessageClass*, const QCString& );
  
  /** return a list of folders where this message should get copyed
  */
  QStringList folderToCopy( const MessageClass*, const QCString& );

  void readConfig( QString group, int id );

protected:

  bool match( const MessageClass*, const QCString& );

  FilterRule *rule1, *rule2;
  QString ruleLogic; // "ignore", "and", "or" , "unless"

  FilterAction *actions[ ACTIONS_COUNT ];
  VFSModule* vfs;

};


#endif
