/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          SAMport_oebox.cpp  -  description
                             -------------------
    begin                : Wed Apr 18 2001
    copyright            : (C) 2001 by Sam "Criswell" Hart
    email                : criswell@geekcomix.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "SAMport.h"
#include "SAMport_oebox.h"
#include "liboe.h"

#include <defines.h>

#ifndef Q_WS_WIN
#include <unistd.h>
#endif

//using namespace SAMport;

char fn[1024];
FILE *box = NULL;

void writeit(const char *s, int i) {
  // i is a dummy, we don't need it, but liboe requires it,
  // so let's just pretend to use it to keep the compiler happy
  i = 0;
  if (box==NULL) {
    box=fopen(fn,"w");
    if(box==NULL)
    {
//      throw SAMport_FileOpenFailed("The temp MS Outlook conversion file failed to open!");
      return;
    }
  }
  
  fprintf(box,"%s",s);
}


// Basic import - call this first before you can actually import
void SAMport_oebox::load(const char *Filename)
{
  std::string line;
//  ios::sync_with_stdio(true); //We need this for liboe work

  if(!fin.is_open()) {
    if(box != NULL) {
       // we need to have this thing closed!
       fclose(box);
       box = NULL;
    }

#ifdef Q_WS_WIN
    sprintf(fn,"oe4convert_temp");
#else
    sprintf(fn,"/tmp/oe4convert_temp.%d", getpid());
#endif

    results = oe_readbox(Filename, writeit);
    if(box != NULL) {
      fclose(box);
      box = NULL;
    }
    if(results->success) {
      fin.open(fn, std::ios::in);
      if(fin.is_open()) {
         //Obtain the first From line (signifying the start of the first record)
         do{
          //fin >> line;
          std::getline(fin, line, '\n');
          //x = line.find("From");
         } while( line.npos == line.find("From") && !fin.eof());

         last_from_line = line;
         recnum = 0;
         fname = Filename;
      }
      else
      {
//        throw SAMport_FileOpenFailed("File did not open, check permissions");
      }
    }
    else
    {
//      throw SAMport_SourceBoxProblem("No Messages Converted!");
    }
  }
  else
  {
//    throw SAMport_FileAlreadyOpen("File was already open");
  }
}

void SAMport_oebox::save(char *Filename)
{
  //stubbly
}

void SAMport_oebox::close()
{
  fin.close();
  fout.close();
}

// Next method
SAMport_Abstract SAMport_oebox::next()
{
  SAMport_Abstract myReturn;
  myReturn.Num = 0;
  std::string line, pre, post;
  int blank_lines = 0;
  bool in_header = true;
  bool in_msg_block = true;
  int x;

  myReturn.FullHeader += last_from_line + "\n";
  if(fin.eof()) {
    fin.close();
    return myReturn;
  }

  if(fin.is_open()) {
    recnum++;
    myReturn.Num = recnum;
    do {
      std::getline(fin, line, '\n');
        if(in_header) {
          if(line.length() > 1) {
            // We are still in the header
            myReturn.FullHeader += line + "\n";
            x = line.find(":");
            if(x > -1) {
              pre = line.substr(0, x);
              post = line.substr((x+1), (line.length() - x));
              if((int)pre.find("Content-Type") > -1) {
                myReturn.Content_Type = post;
              } else if((int)pre.find("From") > -1) {
                myReturn.From = post;
              } else if((int)pre.find("To") > -1) {
                myReturn.To = post;
              } else if((int)pre.find("Subject") > -1) {
                myReturn.Subject = post;
              } else if((int)pre.find("X-Mailer") > -1) {
                myReturn.X_Mailer = post;
              } else if((int)pre.find("Sender") > -1) {
                myReturn.Sender = post;
              } else if((int)pre.find("Reply-To") > -1) {
                myReturn.Reply_To = post;
              } else if((int)pre.find("Date") > -1) {
                myReturn.Date = post;
              } else if((int)pre.find("Message-Id") > -1) {
                myReturn.Message_Id = post;
              } else if((int)pre.find("In-Reply-To") > -1) {
                myReturn.In_Reply_To = post;
              } else if((int)pre.find("MIME-Version") > -1) {
                myReturn.MIME_Version = post;
              }
            }
          } else {
            // We are in the blank line betwixt header and message
            in_header = false;
          }
        } else {
          // We are in the message itself
          if(line.length() > 0) {
            // First, let's check to see if we have a new message block
            if(((int)line.find("From") > -1) & (blank_lines > 0)) {
              // We have a "From" in the line
              x = line.find(" ");
              if(x > -1) {
                post = line.substr((x+1), (line.length() - x));
                x = post.find(" ");
                pre = post.substr(0, x);
                // Check for an e-mail address in specific place
                if((int)pre.find("@") > -1) {
                  // Okay, I guess we have a new message block (this isn't perfect, I know)
                  last_from_line = line;
                  in_msg_block = false;
                }
              }
            } else {
              // Add this to the existing msg
              myReturn.FullMsg += line + "\n";
              blank_lines = 0;
            }
          } else {
            // We have a blank line
            myReturn.FullMsg += line + "\n";
            blank_lines++;
          }
        }
      } while(in_msg_block && !fin.eof());
  }
  else
  {
//      throw SAMport_FileNotOpen("The file has not been loaded yet");
  }
  
  return myReturn;
}

// Prev method
SAMport_Abstract SAMport_oebox::prev()
{
  SAMport_Abstract myReturn;
  myReturn.Num = 0;
  if(fin.is_open()) {
    if(recnum > 0) {
      fin.close();
      load(fname);
      do {
        myReturn = next();
      } while (myReturn.Num < (recnum-1));
    }
    if(myReturn.Num == (recnum-1))
      myReturn = next();
  }
  else
  {
//    throw SAMport_FileNotOpen("The file has not been loaded yet");
  }
  
  return myReturn;
}

// First method
SAMport_Abstract SAMport_oebox::first()
{
  if(fin.is_open()) {
    if(recnum > 0) {
      fin.close();
      load(fname);
    }
  }
  else
  {
//    throw SAMport_FileNotOpen("The file has not been loaded yet");
  }
  
  return next();
}

// Last method
SAMport_Abstract SAMport_oebox::last()
{
  SAMport_Abstract myReturn;
  if(fin.is_open()) {
    if(recnum > 0) {
      fin.close();
      load(fname);
    }
    do{
      myReturn = next();
    } while(!fin.eof());
  }
  else
  {
//    throw SAMport_FileNotOpen("This file has not been loaded yet");
  }
  
  return myReturn;
}

// Jump method
SAMport_Abstract SAMport_oebox::jump(int num)
{
  SAMport_Abstract myReturn;
  if(num > 0) {
    if(fin.is_open()) {
      if(recnum > 0) {
        fin.close();
        load(fname);
      }
      do{
        myReturn = next();
      } while(!fin.eof() && (myReturn.Num < num));
      
      if(myReturn.Num < num)
      {
//        throw SAMport_OutOfRange("The file end was reached before the requested record num");
      }
    }
    else
    {
//      throw SAMport_FileNotOpen("This file has not been loaded yet");
    }
  }
  else
  {
//    throw SAMport_OutOfRange("Requested record number was out of range");
  }
  
  return myReturn;
}

// Default constructor
SAMport_oebox::SAMport_oebox()
{
  last_from_line = "";
  box = NULL;
  recnum = 0;
  if(fin.is_open())
    fin.close();
  if(fout.is_open())
    fout.close();
}

// Default constructor
SAMport_oebox::~SAMport_oebox()
{
  if (box) fclose(box);
  box = NULL;
  if(fin.is_open())
    fin.close();
  if(fout.is_open())
    fout.close();
}
