/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                commandsprocessor.cpp  -  a smart command list
                             -------------------
    begin                : Thu Jul  5 17:35:17 EEST 2001
    copyright            : (C) 2001 by Eugen C.
    email                : eug@thekompany.com
 ***************************************************************************/

#include "commandsprocessor.h"

#include <accounts.h>

// ------------ Command class -------------
Command::Command(IMAP4Client::Commands cmmType,
                 const QString cmmAccount,
                 const QString cmmMailbox,
                 const QString cmmParameters,
                 const QString cmmOther)
  : type( cmmType ),
    account( cmmAccount ),
    mailbox( cmmMailbox ),
    parameters( cmmParameters ),
    other( cmmOther )
{
}

Command::~Command()
{
}

// ------------ CommandsPrivate class -------------
class CommandsPrivate
{
  public:
    CommandsPrivate(IMAP4Client* clientParent);
    virtual ~CommandsPrivate();
    
    /** The list of commands.*/
    CommandsList commands;

    Command* currentCommand;
    
    IMAP4Client* client;
};

CommandsPrivate::CommandsPrivate(IMAP4Client* clientParent)
  : currentCommand(0L)
{
  if( !clientParent )
  {
    qDebug("CommandsPrivate: Hmm, parent client is null.");
  }
  
  client = clientParent;
}

CommandsPrivate::~CommandsPrivate()
{
  if( currentCommand )
  {
    delete currentCommand;
    currentCommand = 0L;
  }
  
  commands.setAutoDelete(true);
  commands.clear();
}

// -------------- CommandsProcessor ----------------

CommandsProcessor::CommandsProcessor(IMAP4Client* clientParent)
{
  d = new CommandsPrivate(clientParent);
}

CommandsProcessor::~CommandsProcessor()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

bool CommandsProcessor::insert( const IMAP4Client::Commands cmmType,
                                const QString cmmAccount, 
                                const QString cmmMailbox,
                                const QString cmmParameters,
                                const QString cmmOther )
{
  if( cmmAccount != d->client->account()->accname )
    return false;
  
  Command * pCommand = new Command(cmmType, cmmAccount, cmmMailbox, cmmParameters, cmmOther);
  
  d->commands.append( pCommand );
  
  return true;
}

Command* CommandsProcessor::pop()
{
  if( d->currentCommand )
  {
    delete d->currentCommand;
    d->currentCommand = 0L;
  }
    
  if( !d->commands.isEmpty() )
  {
    Command* pCommand = d->commands.getFirst();
    
    // commands for NON_AUTHENTICATED state
    switch( pCommand->type )
    {
      case IMAP4Client::NO_COMMAND :
        qDebug("CommandsProcessor: no command");
        d->commands.removeFirst();
        delete pCommand;
        return pop();
        break;
        
      case IMAP4Client::LOGOUT :
        d->commands.removeFirst();
        if( !d->commands.isEmpty() )
        {
          if( d->client->state() == IMAP4Client::SELECTED )
          {
            d->currentCommand = pCommand;
            d->currentCommand->type = IMAP4Client::CLOSE;
            return d->currentCommand;
          }
          else
          {
            delete pCommand;
            return pop();
          }
        }
        else
        {
          d->currentCommand = pCommand;
          return d->currentCommand;
        }
        break;
        
      case IMAP4Client::CAPABILITY :
      case IMAP4Client::NOOP :
        d->commands.removeFirst();
        d->currentCommand = pCommand;
        return d->currentCommand;
        break;
        
      default:
        // nothing
        break;
    }
    
    // TODO:
    // try LOGIN or AUTHENTICATE
    // Capability -> detecting the login ways (plain text or encrypted)
    // Login -> authetication
    // List "" "" -> detecting the delimiter
      
    // if it isn't logged, return the LOGIN command
    if( d->client->state() == IMAP4Client::NOT_AUTHENTICATED )
    {
      Command * login = new Command(IMAP4Client::LOGIN, pCommand->account);
      d->currentCommand = login;
      return d->currentCommand;
    }
    
    if( d->client->delimiter().isEmpty() )
    {
      Command * delimiter = new Command(IMAP4Client::DELIMITER, pCommand->account);
      d->currentCommand = delimiter;
      return d->currentCommand;
    }
    
    // commands for AUTHENTICATED state
    switch( pCommand->type )
    {
      case IMAP4Client::NO_COMMAND :
      case IMAP4Client::NOOP :
      case IMAP4Client::CAPABILITY :
      case IMAP4Client::LOGOUT :
        d->commands.removeFirst();
        delete pCommand;
        qDebug("CommandsProcessor: error");
        return 0L;
        break;
        
      case IMAP4Client::LOGIN :
      case IMAP4Client::AUTHENTICATE :
        d->commands.removeFirst();
        delete pCommand;
        return pop();
        break;
        
      case IMAP4Client::EXAMINE :
      case IMAP4Client::CREATE :
      case IMAP4Client::_DELETE_ :
      case IMAP4Client::RENAME :
      case IMAP4Client::SUBSCRIBE :
      case IMAP4Client::UNSUBSCRIBE :
      case IMAP4Client::LIST :
      case IMAP4Client::LSUB :
      case IMAP4Client::STATUS :
      case IMAP4Client::APPEND :
      case IMAP4Client::ADD_MESSAGE_JOB :
      case IMAP4Client::CREATE_FOLDER_JOB :
      case IMAP4Client::DELETE_FOLDER_JOB :
      case IMAP4Client::RENAME_FOLDER_JOB :
      case IMAP4Client::SUBFOLDERS_JOB :
      case IMAP4Client::KOLAB_JOB :
        d->commands.removeFirst();
        d->currentCommand = pCommand;
        return d->currentCommand;
        break;
        
      default:
        // nothing
        break;
    }
    
    // if it isn't selected, return the SELECT command
    if( d->client->state() != IMAP4Client::SELECTED )
    {
      Command * select = new Command(IMAP4Client::SELECT, 
                                     pCommand->account, 
                                     pCommand->mailbox);
      d->currentCommand = select;
      return d->currentCommand;
    }
    else if( d->client->mailbox() != pCommand->mailbox )
    {
      Command * close = new Command(IMAP4Client::CLOSE, 
                                    pCommand->account, 
                                    pCommand->mailbox);
      d->currentCommand = close;
      return d->currentCommand;
    }
    
    // commands for SELECTED state
    switch( pCommand->type )
    {
      case IMAP4Client::LOGIN :
      case IMAP4Client::AUTHENTICATE :
      case IMAP4Client::EXAMINE :
      case IMAP4Client::CREATE :
      case IMAP4Client::_DELETE_ :
      case IMAP4Client::RENAME :
      case IMAP4Client::SUBSCRIBE :
      case IMAP4Client::UNSUBSCRIBE :
      case IMAP4Client::LIST :
      case IMAP4Client::LSUB :
      case IMAP4Client::STATUS :
      case IMAP4Client::ADD_MESSAGE_JOB :
      case IMAP4Client::CREATE_FOLDER_JOB :
      case IMAP4Client::DELETE_FOLDER_JOB :
      case IMAP4Client::RENAME_FOLDER_JOB :
      case IMAP4Client::SUBFOLDERS_JOB :
      case IMAP4Client::KOLAB_JOB :
        d->commands.removeFirst();
        delete pCommand;
        qDebug("CommandsProcessor: error -");
        return 0L;
        break;
      case IMAP4Client::SELECT :
        d->commands.removeFirst();
        delete pCommand;
        return pop();
        break;
      
      case IMAP4Client::SYNC_JOB :
      case IMAP4Client::SYNC_FULL_JOB :
        d->commands.removeFirst();
        if( isRedundant(pCommand) )
        {
          delete pCommand;
          return pop();
        }
        else
        {
          d->currentCommand = pCommand;
          return d->currentCommand;
        }
        break;
        
      
      case IMAP4Client::CHECK :
      case IMAP4Client::EXPUNGE :
      case IMAP4Client::SEARCH :
      case IMAP4Client::FETCH :
      case IMAP4Client::STORE :
      case IMAP4Client::COPY :
      case IMAP4Client::UID :
      case IMAP4Client::CLOSE :
      case IMAP4Client::DELETE_MESSAGE_JOB :
      case IMAP4Client::UPDATE_MESSAGE_JOB :
        d->commands.removeFirst();
        d->currentCommand = pCommand;
        return d->currentCommand;
        break;
        
      default:
        qDebug("CommandsProcessor: unknown command");
        d->commands.removeFirst();
        delete pCommand;
        return 0L;
        break;
    }
  }
  
  return 0L;
}

Command* CommandsProcessor::currentCommand()
{
  return d->currentCommand;
}

Command* CommandsProcessor::find(IMAP4Client::Commands type)
{
  CommandsListIterator it(d->commands);
  
  for(; it.current(); ++it)
    if( it.current()->type == type )
      break;

  return it.current();
}

bool CommandsProcessor::isRedundant(Command * pCommand)
{
  Command * current;
  
  for(CommandsListIterator it(d->commands); it.current(); ++it)
  {
    current = it.current();
    
    if( current->type == pCommand->type &&
        current->account == pCommand->account &&
        current->mailbox == pCommand->mailbox &&
        current->parameters == pCommand->parameters &&
        current->other == pCommand->other )
      return true;
  }
  
  return false;
}

bool CommandsProcessor::isEmpty()
{
  return d->commands.isEmpty();
}

void CommandsProcessor::clear()
{
  d->commands.setAutoDelete(true);
  d->commands.clear();
  d->commands.setAutoDelete(false);
}


