/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include "imap4addmessagejob.h"

#include <qsocket.h>

#include <orb.h>
#include <indexclass.h>
#include <commandsprocessor.h>
#include <imap4handler.h>
#include <messagedevice.h>

#include <appstatus.h>

IMAP4AddMessageJob::IMAP4AddMessageJob
(
  QSocket           * sock,
  IMAP4Client       * parent
)
  : IMAP4Job(sock, parent),
    state_  (START)
{
  // Empty.
}

IMAP4AddMessageJob::~IMAP4AddMessageJob()
{
  // Empty.
}

  void
IMAP4AddMessageJob::run()
{
  status->message(AppStatus::Server, AppStatus::Text, AppStatus::Waiting,
    tr("Appending a message..."));
  
  state_ = WaitingForAppend;
  
  IndexClass * index = ObjectRequestBroker::thisInstance()->indexReference( command()->parameters );
  client()->sendAppend( index, command()->mailbox );
}

  bool
IMAP4AddMessageJob::processLine(const QString & line)
{
  QString firstWord;
  
  client()->parseNextWord(firstWord, line);
  if( firstWord == client()->getTag() )
  {
    if( client()->replyOK(line) )
    {
      switch( state_ )
      {
        case WaitingForAppend:
          state_ = DONE;
          IMAP4Handler::ref()->slotAddMessage( client() );
          emit finished(true);
          break;
        default:
          qDebug("IMAP4AddMessageJob::processLine() : wrong state");
          emit finished(false);
          return false;
          break;
      }
      
      return true;
    }
    else
    {
      qDebug("IMAP4AddMessageJob::processLine() : bad answer : %s", line.latin1());
      
      emit finished(false);
      
      return false;
    }
  }
  else
    return client()->parseLine(line);
    
  return true;
}

  bool
IMAP4AddMessageJob::processMultiLine(const QByteArray &)
{
  qDebug("IMAP4AddMessageJob::processMultiLine(): not available");
  
  return false;
}

