/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

/***************************************************************************
               imap4client.cpp - communication with the imap server
                             -------------------
                             
  copyright : (C) by theKompany.com     <www.thekompany.com>
              (C) by Atol C&D           <www.atolcd.com>

  authors :   Eugen C.      <eug@thekompany.com>
              Fabien Armand <f.armand@atolcd.com>

 ***************************************************************************/


#include "imap4client.h"

#include <sslsocket.h>
#include <ssldevice.h>

#include <defines.h>

#include <accounts.h>
#include <accountmanager.h>
#include <appstatus.h>
#include <indexclass.h>
#include <mailinfo.h>
#include <imap4handler.h>
#include <commandsprocessor.h>
#include <orb.h>
#include <messagedevice.h>
#include <imaputf7convertor.h>
#include <certificatesmanager.h>

// jobs
#include <imap4job.h>
#include <imap4addmessagejob.h>
#include <imap4deletemessagejob.h>
#include <imap4updatemessagejob.h>
#include <imap4createfolderjob.h>
#include <imap4deletefolderjob.h>
#include <imap4renamefolderjob.h>
#include <imap4syncjob.h>
#include <imap4subfoldersjob.h>
#include <kolabjob.h>

#include <qregexp.h>
#include <qapplication.h>

// include for password
#include <settings.h>

using namespace tkcSSL;

Mbox::Mbox( const QString & mboxName,
            const QString & mboxDelimiter,
            unsigned mboxState)
  : name(mboxName),
    delimiter(mboxDelimiter),
    state(mboxState)
{
}

Mbox::~Mbox()
{
}

FetchInfo::FetchInfo()
  : rcvtime(0),
    flags(0),
    size(0L)
{
}

FetchInfo::~FetchInfo()
{
}

// private class
class IMAP4Client;

class IMAP4Private
{
  public:
  
    IMAP4Private(IMAP4Client*, Account*);
    ~IMAP4Private();
    
    IMAP4Client * parent_;
    Account * account_;

    SSLSocket * socket_;

    QDateTime lastRun_;

    IMAP4Client::Connection connection_;
    /** The server state.*/
    IMAP4Client::State state_;
    /** The mailbox name.*/
    QString mailbox_;
    
    /** The tag name.*/
    QString tagName_;
    /** The tag number.*/
    unsigned int tagNo_;
    
    /** The current command.*/
    IMAP4Client::Commands command_;
    /**
      * The current sub command.
      * See: UID FETCH and UID SEARCH
      */
    IMAP4Client::Commands subCommand_;
    /** Commands list.*/
    CommandsProcessor * commands_;
    /** The current job.*/
    IMAP4Job * job_;
    
    /** Authenticate protocol.*/
    QString auth_;
  
    // Flags
    /** Permanent flags.*/
    unsigned permanentFlags_;
    /** Flags.*/
    unsigned flags_;
    /** Access flag.*/
    IMAP4Client::Access access_;
  
    /** The last error text.*/
    QString error_;
    
    //Info
    unsigned long uidValidity_;
    unsigned long messagesNumber_;
    unsigned long unseen_;
    unsigned long recent_;
    unsigned long uidNext_;

    /** The UID list result of the UID SEARCH command.*/
    UIDMailList searchUID;
    /** The unsigned long vector result of the SEARCH command.*/
    RetrMailList searchList;
    /** It contains the fetched message.*/
    FetchInfo fetchMessage;

    QString        fetchBuffer_;
    QCString       fetchBufferData_;
    unsigned long  fetchSize_;
    unsigned long  fetchBufferSize_;
    
    /** The mailboxes list as a result of the LIST/LSUB command.*/
    MboxList mboxList;
    QString delimiter_;
    
    AppStatus * status;
    bool endOfMessage_;
    bool singleLine_;
    bool starttls_;
    bool stop_;
    
    QTimer checkTimer_;
    QTimer idleTimer_;
    
};


IMAP4Private::IMAP4Private(IMAP4Client * parent, Account * acc)
  : parent_         (parent),
    lastRun_        (QDateTime(QDate(0, 0, 0), QTime(0, 0, 0, 0))),
    connection_     (IMAP4Client::NOT_CONNECTED),
    state_          (IMAP4Client::NOT_AUTHENTICATED),
    tagName_        ("AETHERA"),
    tagNo_          (0),
    command_        (IMAP4Client::NO_COMMAND),
    subCommand_     (IMAP4Client::NO_COMMAND),
    commands_       (0L),
    job_            (0),
    auth_           ("LOGIN"),
    permanentFlags_ (0),
    flags_          (0),
    access_         (IMAP4Client::READ_WRITE),
    error_          ("NO ERROR"),
    uidValidity_    (0L),
    messagesNumber_ (0L),
    unseen_         (0L),
    recent_         (0L),
    uidNext_        (0L),
    status          (0L),
    endOfMessage_   (false),
    singleLine_     (true),
    starttls_       (false),
    stop_           (false)
{
  // copy the account, it should be safest than using the original one
  account_ = new Account(*acc);
  
  socket_ = new SSLSocket();
  
  if( account_->ssl )
  {
    SSLDevice * sslDevice = 0;
    if( account_->tls )
    {
      qDebug("******** TLS connection **********");
      starttls_ = true;
      sslDevice = new SSLDevice(QSocketDevice::Stream, SSLDevice::TLS);
    }
    else
    {
      qDebug("******** SSL connection **********");
      sslDevice = new SSLDevice(QSocketDevice::Stream, SSLDevice::SSL23);
    }
    
    if(sslDevice)
    {
      if( account_->useCertificates )
      {
        QObject::connect(
          sslDevice->qtHandler(),
          SIGNAL(doYouAcceptCertificate(SSLDevice *, const QString&)),
          CertificatesManager::instance(),
          SLOT(slotAcceptCertificateDialog(SSLDevice *, const QString& )));
        QObject::connect(
          sslDevice->qtHandler(),
          SIGNAL(certificateVerifyFailed(const QString&)),
          CertificatesManager::instance(),
          SLOT(slotCertificateFailed(const QString&)));
        
        sslDevice->setVerifyType( SSLDevice::ASKING );
        sslDevice->setCertificatesFolder
          ( CertificatesManager::instance()->certificatesFolder() );
        sslDevice->setCertificatesFile
          ( CertificatesManager::instance()->certificateFile(account_->accname, "IMAP4") );
      }
      
      socket_->setSocketDevice(sslDevice, true);
      if( !sslDevice->initSSL() )
      {
        qDebug("IMAP4Client : Error initSSL()");
      }
    }
    else
    {
      qDebug("IMAP4Client : cannot create a new ssl device.");
    }
  }
  
  commands_ = new CommandsProcessor(parent);
  
  searchUID.setAutoDelete(true);
  mboxList.setAutoDelete(true);

  status    =   AppStatus::instance();
}

IMAP4Private::~IMAP4Private()
{
  if( commands_ )
  {
    delete commands_;
    commands_ = 0L;
  }
  
  searchList.clear();
  searchUID.clear();
  mboxList.clear();
  
  if(account_)
  {
    delete account_;
    account_ = 0;
  }
}


// ------------ IMAP4Client ---------------

IMAP4Client::IMAP4Client(Account * acc, QObject * parent, const char * name)
  : QObject       (parent, name)
{
  d = new IMAP4Private(this, acc);

  connect
    (
      d->socket_,
      SIGNAL(hostFound()),
      this,
      SLOT(slotHostFound())
    );

  connect
    (
      d->socket_,
      SIGNAL(connected()),
      this,
      SLOT(slotConnected())
    );

  connect
    (
      d->socket_,
      SIGNAL(connectionClosed()),
      this,
      SLOT(slotConnectionClosed())
    );

  connect
    (
      d->socket_,
      SIGNAL(readyRead()),
      this,
      SLOT(slotReadyRead())
    );

  connect
    (
      d->socket_,
      SIGNAL(bytesWritten(int)),
      this,
      SLOT(slotBytesWritten(int))
    );

  connect
    (
      d->socket_,
      SIGNAL(error(int)),
      this,
      SLOT(slotError(int))
    );

  connect
    (
      &d->idleTimer_,
      SIGNAL(timeout()),
      this,
      SLOT(slotIdle())
    );
    
  connect
    (
      this,
      SIGNAL( signalNewCommand() ),
      SLOT( slotRunQueue() )
    );
    
  connect
    (
      this,
      SIGNAL( signalCommandCompleted() ),
      SLOT( slotRunQueue() )
    );
}

IMAP4Client::~IMAP4Client()
{
  delete d;
  d = 0;
}

  void
IMAP4Client::connectToHost()
{
  if( NOT_CONNECTED != d->connection_ ||
      QSocket::Connected == d->socket_->state() )
  {
    qDebug("IMAP4Client::Already connected");
    d->connection_ = IMAP4Client::IDLE;
    return;
  }

  setConnection( IMAP4Client::WAITING_FOR_CONNECTION );
  qDebug("IMAP4Client::connecting to host ...");

  d->socket_->connectToHost(d->account_->server, d->account_->port);

  resetIdleTimer();
}

  void
IMAP4Client::stop()
{
#ifdef IMAP4_DEBUG
  qDebug("--------------- IMAP4Client::stop() -----------------");
#endif

  if( !d->stop_ )
  {
    d->stop_ = true;
    
    d->commands_->clear();
    d->command_    = IMAP4Client::NO_COMMAND;
    d->subCommand_ = IMAP4Client::NO_COMMAND;
    
    d->connection_ = IMAP4Client::NOT_CONNECTED;
    d->state_ =      IMAP4Client::NOT_AUTHENTICATED;
  
    d->socket_->close();
    
    d->idleTimer_.stop();
  
    d->searchList.clear();
    d->searchUID.clear();
    
    d->mboxList.clear();
    
    d->mailbox_ = QString::null;
      
    // it's better to be made in events loop
    // see SSLDevice
    QTimer::singleShot(100, this, SLOT(slotEmitFinished()));
  }
}

  void
IMAP4Client::slotEmitFinished()
{
  IMAP4Handler::ref()->slotClientFinished(this);
}

  bool
IMAP4Client::insertCommand( Commands type,
                    const QString & account, 
                    const QString & mailbox, 
                    const QString & parameters,
                    const QString & other )
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::insertCommand |%s|%s|%s|%s|",
      account.latin1(),
      mailbox.latin1(),
      parameters.latin1(),
      other.latin1() );
#endif

  if( !d->stop_ && d->commands_->insert(type, account, mailbox, parameters, other) )
  {
    QTimer::singleShot(0, this, SLOT(slotEmitNewCommand()));
    return true;
  }
  
  return false;
}

void IMAP4Client::slotEmitNewCommand()
{
  emit signalNewCommand();
}

Command * IMAP4Client::currentCommand()
{
  return d->commands_->currentCommand();
}
    
Account * IMAP4Client::account()
{
  return d->account_;
}

FetchInfo& IMAP4Client::incomingMail()
{
  return d->fetchMessage;
}

UIDMailList& IMAP4Client::searchUID()
{
  return d->searchUID;
}
    
RetrMailList& IMAP4Client::searchList()
{
  return d->searchList;
}

MboxList& IMAP4Client::subfolders()
{
  return d->mboxList;
}

QString IMAP4Client::delimiter()
{
  return d->delimiter_;
}

void IMAP4Client::commandCompleted()
{
  setConnection( IMAP4Client::IDLE );
  
  emit signalCommandCompleted();
}
    
bool IMAP4Client::isJob()
{
  switch( d->command_ )
  {
    case IMAP4Client::SYNC_JOB :
    case IMAP4Client::SYNC_FULL_JOB :
    case IMAP4Client::ADD_MESSAGE_JOB :
    case IMAP4Client::DELETE_MESSAGE_JOB :
    case IMAP4Client::UPDATE_MESSAGE_JOB :
    case IMAP4Client::CREATE_FOLDER_JOB :
    case IMAP4Client::DELETE_FOLDER_JOB :
    case IMAP4Client::RENAME_FOLDER_JOB :
    case IMAP4Client::SUBFOLDERS_JOB :
    case IMAP4Client::KOLAB_JOB :
      break;
    
    default :
      return false;
      break;
  }
  
  return true;
}
    
bool IMAP4Client::isSingleLine()
{
  return d->singleLine_;
}

void IMAP4Client::setSingleLine(bool singleLine)
{
  d->singleLine_ = singleLine;
}
    
  void
IMAP4Client::resetIdleTimer()
{
  d->idleTimer_.start(60000 /* 1 minute */, true);
}

  void
IMAP4Client::slotJobFinished(bool ok)
{
#ifdef IMAP4_DEBUG
  qDebug("Job finished %d", ok);
#endif
  
  d->command_ = IMAP4Client::NO_COMMAND;
  d->subCommand_ = IMAP4Client::NO_COMMAND;
  if(d->job_)
  {
    d->job_->deleteLater();
    d->job_ = 0;
  }
  
  setConnection( IMAP4Client::IDLE );
  setSingleLine( true );
  if( ok )
    d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Good, tr("IMAP: job finished"));
  else
  {
    d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Error, tr("IMAP: job not finished"));
    d->error_ = "couldn't finished the current operation";
  }
  
  if( ok )
    slotRunQueue();
}

  void
IMAP4Client::slotHostFound()
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotHostFound");
#endif
  
  d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Waiting, tr("Host found..."));
  
  resetIdleTimer();
}

  void
IMAP4Client::slotConnected()
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotConnected");
#endif
  
  d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Waiting, tr("Connected..."));
  
  resetIdleTimer();
  
  setConnection( IMAP4Client::IDLE );
  
  slotRunQueue();
}

  QDateTime
IMAP4Client::lastRun() const
{
  return d->lastRun_;
}

  void
IMAP4Client::slotConnectionClosed()
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotConnectionClosed");
#endif

  d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Warning, tr("Connection closed..."));

  stop();
}

  void 
IMAP4Client::slotRunQueue()
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotRunQueue()");
#endif

  Command * pCommand=0L;
  if( d->stop_ || d->commands_->isEmpty() )
  {
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotRunQueue : empty queue");
#endif
    if( IMAP4Client::IDLE == d->connection_ ||
        IMAP4Client::NOT_CONNECTED == d->connection_ )
    {
      d->command_ = IMAP4Client::NO_COMMAND;
      d->subCommand_ = IMAP4Client::NO_COMMAND;
    }
    return;
  }
  
  switch( d->connection_ )
  {
    case IMAP4Client::NOT_CONNECTED :
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::NOT_CONNECTED");
#endif
      connectToHost();
      break;
      
    case IMAP4Client::IDLE :
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::IDLE");
#endif
      d->fetchBufferData_.resize(0);
      d->fetchBufferSize_=0;
      d->fetchBuffer_=QString::null;
      d->fetchMessage.message.resize(0);
      d->fetchMessage.flags = 0;
      d->fetchMessage.size = 0;
      d->fetchMessage.uid = QString::null;
      d->fetchMessage.rcvtime = 0;
      d->searchList.clear();
      d->searchUID.clear();
      d->mboxList.clear();
      pCommand = d->commands_->pop();
      
      if( !pCommand )
      {
        d->command_ = IMAP4Client::NO_COMMAND;
        d->subCommand_ = IMAP4Client::NO_COMMAND;
        return;
      }
      
      d->connection_ = IMAP4Client::BUSY;
      d->command_    = pCommand->type;
      d->subCommand_ = IMAP4Client::NO_COMMAND;
      
      if( isJob() )
      {
        switch( d->command_ )
        {
          case IMAP4Client::SYNC_FULL_JOB :
            {
              IMAP4SyncJob * job = new IMAP4SyncJob(d->socket_, this);
              if(job)
              {
                job->setFullMessage(true);
              }
              
              d->job_ = job;
            }
            break;
          case IMAP4Client::SYNC_JOB :
            d->job_ = new IMAP4SyncJob(d->socket_, this);
            break;
          case IMAP4Client::ADD_MESSAGE_JOB :
            d->job_ = new IMAP4AddMessageJob(d->socket_, this);
            break;
          case IMAP4Client::DELETE_MESSAGE_JOB :
            d->job_ = new IMAP4DeleteMessageJob(d->socket_, this);
            break;
          case IMAP4Client::UPDATE_MESSAGE_JOB :
            d->job_ = new IMAP4UpdateMessageJob(d->socket_, this);
            break;
          case IMAP4Client::CREATE_FOLDER_JOB :
            d->job_ = new IMAP4CreateFolderJob(d->socket_, this);
            break;
          case IMAP4Client::DELETE_FOLDER_JOB :
            d->job_ = new IMAP4DeleteFolderJob(d->socket_, this);
            break;
          case IMAP4Client::SUBFOLDERS_JOB :
            d->job_ = new IMAP4SubfoldersJob(d->socket_, this);
            break;
          case IMAP4Client::KOLAB_JOB :
            d->job_ = new KolabJob(d->socket_, this);
            break;
          case IMAP4Client::RENAME_FOLDER_JOB :
            d->job_ = new IMAP4RenameFolderJob(d->socket_, this);
            break;
          default :
            qDebug("IMAP4Client: wrong way");
            return;
            break;
        }
        
        if( d->job_ )
          d->job_->run();
      }
      else
      {
        switch( d->command_ )
        {
          case IMAP4Client::NO_COMMAND :
            slotRunQueue();
            break;
          case IMAP4Client::STARTTLS :
            sendSTARTTLS();
            break;
          case IMAP4Client::LOGOUT :
            sendLogout();
            break;
          case IMAP4Client::AUTHENTICATE :
          case IMAP4Client::LOGIN :
            sendLogin();
            break;
          case IMAP4Client::SELECT :
            sendSelect( pCommand->mailbox );
            break;
          case IMAP4Client::CLOSE :
            sendClose();
            break;
          case IMAP4Client::UID :
            sendUid( pCommand->parameters );
            break;
          case IMAP4Client::FETCH :
            sendFetch( pCommand->parameters );
            break;
          case IMAP4Client::CAPABILITY :
            sendCapability();
            break;
          case IMAP4Client::NOOP :
            sendNoop();
            break;
          case IMAP4Client::CREATE :
            sendCreate( pCommand->mailbox );
            break;
          case IMAP4Client::_DELETE_ :
            sendDelete( pCommand->mailbox );
            break;
          case IMAP4Client::RENAME :
            sendRename(pCommand->mailbox, pCommand->parameters);
            break;
          case IMAP4Client::SUBSCRIBE :
            sendSubscribe( pCommand->mailbox );
            break;
          case IMAP4Client::UNSUBSCRIBE :
            sendUnsubscribe( pCommand->mailbox );
            break;
          case IMAP4Client::DELIMITER :
            sendList("\"\"", "\"\"");
            break;
          case IMAP4Client::LIST :
            sendList( pCommand->mailbox );
            break;
          case IMAP4Client::LSUB :
            sendLsub( pCommand->mailbox );
            break;
          case IMAP4Client::STATUS :
            sendStatus();
            break;
          case IMAP4Client::APPEND :
          {
            IndexClass * index = ObjectRequestBroker::thisInstance()->indexReference( pCommand->parameters );
            sendAppend(index, pCommand->mailbox);
          }
            break;
          case IMAP4Client::EXPUNGE :
            sendExpunge();
            break;
          case IMAP4Client::SEARCH :
//            sendSearch( pCommand->parameters );
//            break;
          case IMAP4Client::STORE :
//            sendStore( pCommand->parameters );
//            break;
          case IMAP4Client::COPY :
//            sendCopy( pCommand->parameters );
//            break;
          case IMAP4Client::EXAMINE :
//            sendExamine();
//            break;
          case IMAP4Client::CHECK :
//            sendCheck();
//            break;
          default:
            qDebug("Unknown Command");
            slotRunQueue();
            break;
        }
      }
    case IMAP4Client::BUSY :
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::BUSY");
#endif
    case IMAP4Client::WAITING_FOR_CONNECTION :
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::WAITING_FOR_CONNECTION");
#endif
    default :
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::DEFAULT");
#endif
      break;
  }
}

  void
IMAP4Client::writeLine(const QString & s)
{
#ifdef IMAP4_DEBUG
  qDebug("Sending: `%s'", s.latin1());
#endif

  QTextStream str(d->socket_);

  str << s << "\r\n";
}

  void
IMAP4Client::slotReadyRead()
{
  bool bRet=true;
    
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotReadyRead");
#endif

  resetIdleTimer();

  if( isSingleLine() )
  {
    QString line;
    
    while (d->socket_->canReadLine())
    {
      line = d->socket_->readLine();
  
#ifdef IMAP4_DEBUG
      // QString replaces in-place. Must copy first.
      QString stripped = line;
      stripped.replace(QRegExp("[\r\n]*$"), "");
  
      qDebug("Read: `%s'", stripped.latin1());
#endif
  
      bRet = processLine(line);
      line = QString::null;
      
      if (!bRet || !isSingleLine())
        break;
    }
  }
  
  if( bRet && !isSingleLine() )
  {
    QByteArray block;
    
    long longSize = 0L;
    while( (longSize = d->socket_->bytesAvailable()) )
    {
      block.resize(longSize);
      longSize = d->socket_->readBlock(block.data(), block.size());
      
      if( -1 == longSize )
      {
        bRet = false;
        break;
      }
      
      bRet = processMultiLine(block);
      block.resize(0);
      longSize = 0L;
      
      if (!bRet)
        break;
    }
  }
  
  if( !bRet )
  {
    if (d->command_==IMAP4Client::LOGIN &&  d->state_==IMAP4Client::NOT_AUTHENTICATED)
    {
      d->error_ = " bad authentication";
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Error, tr("IMAP error:") + error());
#ifdef IMAP4_DEBUG
      qDebug("Bad authentication");
#endif
      
      d->command_    = IMAP4Client::NO_COMMAND;
      d->subCommand_ = IMAP4Client::NO_COMMAND;

      d->connection_ = IMAP4Client::IDLE;
      d->state_ = IMAP4Client::NOT_AUTHENTICATED;
      
      Account * pAccount = 0L;
      
      pAccount = AccountManager::ref()->getAccount( d->account_->accname );
      
      pAccount->password="";
      
      d->account_->password="";
      
      IMAP4Handler::ref()->login(d->account_->accname);
    }
    else
    {
#ifdef IMAP4_DEBUG
      qDebug("Error: couldn't finish the current operation ");
#endif
      
      d->error_ = " couldn't finish the current operation";
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Error, tr("IMAP error:") + error());
      stop();
    }
  }
}

  void
IMAP4Client::slotError(int err)
{
  qDebug("IMAP4Client::slotError(%d)", err);

  stop();
  
  switch (err)
  {
    case QSocket::ErrConnectionRefused:
      qDebug("Connection refused");
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Error, tr("Connection refused"));
      break;
    
    case QSocket::ErrHostNotFound:
      qDebug("Host not found");
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Error, tr("Host not found"));
      break;
    
    case QSocket::ErrSocketRead:
      qDebug("Read error");
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Error, tr("Read error"));
      break;
      
    default :
      qDebug("Protocol error");
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Error, tr("Protocol error"));
      break;
  }
}

  void
IMAP4Client::slotIdle()
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotIdle");
#endif

  if (IDLE == d->connection_)
  {
    if( d->state_ == IMAP4Client::SELECTED )
    {
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Warning, tr("IMAP Idle"));
  
      // insert NOOP to keep the connection online
      sendNoop();
    }
    else
    {
      // LOGOUT
      insertCommand(IMAP4Client::LOGOUT, account()->accname,
                    QString::null, QString::null, QString::null);
    }
  }
}

  bool
IMAP4Client::processLine(const QString & line)
{
  if( line.isEmpty() )
    return false;
    
#ifdef IMAP4_DEBUG
  qDebug("processLine = %s", line.latin1());
#endif
  
  bool bRet=true;
  
  if( isJob() && d->job_ )
    bRet = d->job_->processLine(line);
  else
  {
    switch( d->command_ )
    {
      case IMAP4Client::NO_COMMAND :
        qDebug("processLine: NO COMMAND");
        break;
      case IMAP4Client::STARTTLS :
        bRet = handleSTARTTLS(line);
        break;
      case IMAP4Client::LOGOUT :
        bRet = handleLogout(line);
        break;
      case IMAP4Client::AUTHENTICATE :
      case IMAP4Client::LOGIN :
        bRet = handleLogin(line);
        break;
      case IMAP4Client::SELECT :
        bRet = handleSelect(line);
        break;
      case IMAP4Client::CLOSE :
        bRet = handleClose(line);
        break;
      case IMAP4Client::UID :
        bRet = handleUid(line);
        break;
      case IMAP4Client::FETCH :
        bRet = handleFetch(line);
        break;
      case IMAP4Client::SUBSCRIBE :
        bRet = handleSubscribe(line);
        break;
      case IMAP4Client::UNSUBSCRIBE :
        bRet = handleUnsubscribe(line);
        break;
      case IMAP4Client::DELIMITER :
        bRet = handleList(line);
        break;
      case IMAP4Client::LIST :
        bRet = handleList(line);
        break;
      case IMAP4Client::LSUB :
        bRet = handleLsub(line);
        break;
      case IMAP4Client::APPEND :
        bRet = handleAppend(line);
        break;
      case IMAP4Client::CREATE :
        bRet = handleCreate(line);
        break;
      case IMAP4Client::_DELETE_ :
        bRet = handleDelete(line);
        break;
      case IMAP4Client::RENAME :
        bRet = handleRename(line);
        break;
      case IMAP4Client::STATUS :
        bRet = handleStatus(line);
        break;
      case IMAP4Client::EXPUNGE :
        bRet = handleExpunge(line);
        break;
      case IMAP4Client::SEARCH :
      case IMAP4Client::CAPABILITY :
      case IMAP4Client::NOOP :
      case IMAP4Client::EXAMINE :
      case IMAP4Client::CHECK :
      case IMAP4Client::STORE :
      case IMAP4Client::COPY :
        bRet = handleCommand(line);
        break;
      default:
        // nothing
        break;
    }
  }

  return bRet;
}

  bool
IMAP4Client::processMultiLine(const QByteArray & block)
{
  bool bRet = true;
  
  if( isJob() && d->job_ )
    bRet = d->job_->processMultiLine(block);
  else
  {
    switch( d->command_ )
    {
      case IMAP4Client::FETCH :
        bRet = handleFetchData(block);
        break;
      case IMAP4Client::UID :
        if( IMAP4Client::FETCH == d->subCommand_ )
          bRet = handleFetchData(block);
        break;
      default:
        qDebug("processMultiLine: error");
        break;
    }
  }

  return bRet;
}

  bool
IMAP4Client::handleCommand(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleCommand : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

#ifdef IMAP4_DEBUG
  void
IMAP4Client::slotBytesWritten(int byteCount)
#else
  void
IMAP4Client::slotBytesWritten(int)
#endif
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotBytesWritten(%d)", byteCount);
#endif

  resetIdleTimer();
}

QString IMAP4Client::mailbox() const
{
  return d->mailbox_;
}

IMAP4Client::Connection IMAP4Client::connectionState() const
{
  return d->connection_;
}

void IMAP4Client::setConnection(IMAP4Client::Connection state)
{
  d->connection_ = state;
}

void IMAP4Client::setState(IMAP4Client::State state)
{
  d->state_ = state;
}

IMAP4Client::State IMAP4Client::state() const
{
  return d->state_;
}

QString IMAP4Client::error() const
{
  return d->error_;
}
  
QString IMAP4Client::getTag() const
{
  QString tag = d->tagName_ + QString::number(d->tagNo_);
  
  return tag;
}

QString IMAP4Client::commandTag()
{
  d->tagNo_++;
  
  return getTag();
}

// ------------------------
// Parsers
// ------------------------
bool IMAP4Client::replyOK(const QString & reply)
{
  unsigned uPos=0;
  QString word;
  
  uPos = parseNextWord(word, reply);
  if( word != getTag() )
  {
    return false;
  }
  
  uPos = parseNextWord(word, reply, uPos);
  parseLineResult( reply.right(reply.length() - uPos) );
  
  if( "OK" != word.upper() )
  {
    return false;
  }

  return true;
}

bool IMAP4Client::parseLineResult(const QString &line)
{
  unsigned uPos=0;
  QString word;
  char first = 0;

  
  uPos = skipSpaces(line);
  
#ifdef _QTWIN_  
  first = line[(int)uPos].latin1();
#else
  first = line[uPos].latin1();
#endif
  
  if( '[' == first )
  {
    uPos++;
    uPos = parseNextWord(word, line, uPos);
    
    word = word.upper();
    first = word[0].latin1();
    
    switch( first )
    {
      case 'A' :
        if( "ALERT" == word )
          qDebug("Alert - %s", line.latin1());
          // TODO: send a warning message to the client
        break;
  
      case 'N' :
        if( "NEWNAME" == word )
          qDebug("NewName - %s", line.latin1());
        break;
  
      case 'P' :
        if( "PARSE" == word )
          qDebug("Parse - %s", line.latin1());
        else if( "PERMANENTFLAGS" == word )
        {
          setPermanentFlags( line.right(line.length() - uPos) );
        }
        break;
  
      case 'R' :
        if( "READ-ONLY" == word )
        {
          setAccess(IMAP4Client::READ_ONLY);
        }
        else if( "READ-WRITE" == word )
        {
          setAccess(IMAP4Client::READ_WRITE);
        }
        break;
  
      case 'T' :
        if( "TRYCREATE" == word )
          qDebug("TryCreate - %s", line.latin1());
        break;
  
      case 'U' :
        if( "UIDVALIDITY" == word )
        {
          unsigned long uid=0;
          if( parseUnsignedLong(line.right(line.length() - uPos), uid) )
          {
            setUidValidity( uid );
          }
        }
        else if( "UNSEEN" == word )
        {

          unsigned long unseen=0;
          if( parseUnsignedLong(line.right(line.length() - uPos), unseen) )
            setUnseen( unseen );
        }
        else if( "UIDNEXT" == word )
        {
          unsigned long uid=0;
          if( parseUnsignedLong(line.right(line.length() - uPos), uid) )
          {
            setUidNext( uid );
          }
        }
        break;
    }
  }
  
  return true;
}
    
unsigned IMAP4Client::skipSpaces(const QString & line, unsigned uPos)
{

  unsigned i;
  unsigned length = line.length();
  
  register const QChar *pChar;
  pChar = line.unicode() + uPos;
  
  for(i=uPos; i<length; i++)
  {
    char currentChar = pChar->latin1();
    
    switch(currentChar)
    {
      case ' ':
      case '\t':
      case '\r':
      case '\n':
        break;
      default :
        return i;
        break;
    }
    
    pChar++;
  }
  
  return length;
}

unsigned IMAP4Client::parseNextWord(QString & strWord, const QString & line, unsigned uPos)
{
  unsigned i;
  unsigned length = line.length();

  strWord = QString::null;
  
  uPos = skipSpaces(line, uPos);
  
  register const QChar *pChar;
  pChar = line.unicode() + uPos;
  
  char currentChar = 0;
  for(i=uPos; i<length; i++)
  {
    currentChar = pChar->latin1();
    
    switch(currentChar)
    {
      case ' ':
      case '\t':
      case '\r':
      case '\n':
      case '{':
      case '}':
      case '(':
      case ')':
      case '[':
      case ']':
        strWord = line.mid(uPos, i-uPos);
        return i;
        break;
      default :
        break;
    }
    
    pChar++;
  }
  
  strWord=line.mid(uPos, length-uPos);

  return length;
}

bool IMAP4Client::parseUnsignedLong(const QString & line, unsigned long & number)
{
  QString word;
  bool ok=true;
  
  parseNextWord(word, line);
  
  number = word.toULong(&ok);
  
  return ok;
}

bool IMAP4Client::parseLine(const QString &line)
{
  // current word
  QString word;
  // current position in line
  unsigned uPos=0;

  d->error_ = QString::null;

  uPos = parseNextWord(word, line);      // passing over "*"
  if( "*" != word )
  {
    if( "+" == word )
      return true;
    else
      return false;
  }
  
  uPos = parseNextWord(word, line, uPos); // get the first word
  
  word = word.upper();
  char first = word[0].latin1();

  switch( first )
  {
    case 'B' :
      if( "BAD" == word )
      {
        qDebug("parseLine: BAD %s", line.latin1());
        return parseLineResult(line.right(line.length() - uPos));
      }
      else if( "BYE" == word )
      {
        qDebug("parseLine: BYE %s", line.latin1());
        return true;
      }
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    case 'C' :
      if( "CAPABILITY" == word )
        return parseCapability(line.right(line.length() - uPos));
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    case 'F' :
      if( "FLAGS" == word )
      {
        unsigned flags;
        
        if( parseFlags(line.right(line.length() - uPos), flags) )
        {
          d->flags_ = flags;
          return true;
        }
        else
        {
          qDebug("parseLine: FLAGS %s", line.latin1());
          return false;
        }
      }
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    case 'L' :
      if( "LIST" == word )
        return parseList( line.right(line.length() - uPos) );
      else if( "LSUB" == word )
        return parseList( line.right(line.length() - uPos) );
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    case 'N' :
      if( "NO" == word )
        return parseLineResult(line.right(line.length() - uPos));
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    case 'O' :
      if( "OK" == word )
        return parseLineResult(line.right(line.length() - uPos));
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    case 'P' :
      if( "PREAUTH" == word )
        return parseLineResult(line.right(line.length() - uPos));
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    case 'S' :
      if( "SEARCH" == word )
        return parseSearch(line.right(line.length() - uPos));
      else if( "STATUS" == word )
        parseStatus(line.right(line.length() - uPos));
      else
      {
        d->error_=QString("parseLine: unknown: ")+line;
        return false;
      }
      break;

    // It has to begin with a number
    default:
      {
        unsigned long ulNumber=0;
        bool ok=true;
        
        ulNumber = word.toULong(&ok);
        
        if( !ok )
        {
          d->error_=QString("parseLine: unknown: ")+line;
          return false;
        }

        uPos = parseNextWord(word, line, uPos);
        word = word.upper();
        first = word[0].latin1();
        
        switch( first )
        {
          case 'E' :
            if( "EXISTS" == word )
            {
              setMessagesNumber(ulNumber);
              return true;
            }
            else if( "EXPUNGE" == word )
            {
              qDebug("Expunge the deleted mails!");
              return true;
            }
            else
            {
              d->error_=QString("parseLine: unknown: ")+line;
              return false;
            }
            break;

          case 'F' :
            if( "FETCH" == word )
            {
              d->fetchBufferData_.resize(0);
              d->fetchBufferSize_=0;
              d->fetchMessage.message.resize(0);

              d->fetchMessage.flags = 0;
              d->fetchMessage.size = 0;
              d->fetchMessage.uid = QString::null;
              d->fetchMessage.rcvtime = 0;
              // what to do with ulNumber ?
              return parseFetch( line.right(line.length() - uPos) );
            }
            else
            {
              d->error_=QString("parseLine: unknown: ")+line;
              return false;
            }
            break;

          case 'R' :
            if( "RECENT" == word )
            {
              setRecent(ulNumber);
              return true;
            }
            else
            {
              d->error_=QString("parseLine: unknown: ")+line;
              return false;
            }
            break;

          case 'S' :
            if( "STORE" == word )
            {
              qDebug("STORE: %s", line.latin1());
              return true;
            }
            else
            {
              d->error_=QString("parseLine: unknown: ")+line;
              return false;
            }
            break;

          default:
            d->error_=QString("parseLine: unknown: ")+line;
            return false;
            break;
        }
      }
      break;
  }

  return true;
}

bool IMAP4Client::parseCapability(const QString &line)
{
  QStringList words(QStringList::split(" ", line));
  
  // TODO : check a lot more info 
  // Ex: starttls
  
  QStringList auth = words.grep("AUTH=", false);


  if( auth.size() )
    setAuth( auth.front().upper() );
  
  return true;
}

bool IMAP4Client::parseFetch(const QString &line)
{
  // current word
  QString word;
  // current position in line
  unsigned uPos=0;
  unsigned length = line.length();
  char first=0;

  d->subCommand_ = IMAP4Client::FETCH;
  
  d->error_ = QString::null;

  uPos = skipSpaces(line);
  
#ifdef _QTWIN_  
  first = line[(int)uPos].latin1();
#else
  first = line[uPos].latin1();
#endif

  // it has to begin with '(' and to end with ')'
  if( '(' == first )
  {
    uPos++;
    uPos = skipSpaces(line, uPos);

#ifdef _QTWIN_  
    while( uPos<length && ')'!=(first = line[(int)uPos].latin1()) )
#else
    while( uPos<length && ')'!=(first = line[uPos].latin1()) )
#endif
    {
      if( '(' != first )
      {
        uPos = parseNextWord(word, line, uPos);
        word = word.upper();
        first = word[0].latin1();
        
        switch( first )
        {
          case 'B' :
            if( "BODY" == word )
            {
              // BODY[HEADER] {1234} \r\n
              // Return-Path ....
              // Remove [HEADER]
              uPos = skipSpaces(line, uPos);
              
              // pass over [...]
#ifdef _QTWIN_  
              first = line[(int)uPos].latin1();
#else
              first = line[uPos].latin1();
#endif
              if( '[' == first )
              {
                int iFind = line.find(']', uPos);
                if( iFind != -1 )
                  uPos = iFind+1;
                else
                {
                  d->error_=QString("parseFetch: wrong body: ")+line;
                  return false;
                }
              }

              // Set the size of the message. {size}
              uPos = skipSpaces(line, uPos);
#ifdef _QTWIN_  
              first = line[(int)uPos].latin1();
#else
              first = line[uPos].latin1();
#endif
              if( '{' == first )
              {
                uPos++;
                int iFind = line.find('}', uPos);
                if( iFind == -1 )
                {
                  d->error_=QString("parseFetch: wrong size: ")+line;
                  return false;
                }
                
                QString size = line.mid(uPos, iFind-uPos);
                
                if( !parseUnsignedLong(size, d->fetchMessage.size) )
                {
                  d->error_=QString("parseFetch: wrong size: ")+line;
                  return false;
                }
                
                uPos = iFind+1;
              }
              else
              {
                d->error_=QString("parseFetch: wrong BODY: ")+line;
                return false;
              }
            }
            else if( "BODYSTRUCTURE" == word )
            {
              qDebug("parseFetch - BODYSTRUCTURE - not available yet");
              return false;
            }
            break;

          case 'E' :
            if( "ENVELOPE" == word )
            {
              qDebug("parseFetch - ENVELOPE - not available yet");
              return false;
            }
            break;

          case 'F' :
            if( "FLAGS" == word )
            {
              QString flags;
              int flagsBegin = line.find('(', uPos);
              int flagsEnd = line.find(')', flagsBegin);
              
              if( flagsBegin == -1 ||
                  flagsEnd   == -1 )
              {
                d->error_=QString("parseFetch: wrong FLAGS: ")+line;
                return false;
              }
              
              flags = line.mid(flagsBegin, flagsEnd-flagsBegin+1);
              
              if( !parseFlags(flags, d->fetchMessage.flags) )
              {
                d->error_=QString("parseFetch: wrong FLAGS: ")+line;
                return false;
              }
              
              uPos = flagsEnd+1;
            }
            break;

          case 'I' :
            if( "INTERNALDATE" == word )
            {
              uPos = skipSpaces(line, uPos);
#ifdef _QTWIN_  
              first = line[(int)uPos].latin1();
#else
              first = line[uPos].latin1();
#endif
              if( '"' == first )
              {
                uPos++;
                
                int iFind = line.find('"', uPos);
                if( iFind == -1)
                {
                  d->error_=QString("parseFetch: wrong INTERNALDATE: ")+line;
                  return false;
                }
                
                // TODO: read and use the DATE field
                
                uPos = iFind+1;
              }
              else
              {
                d->error_=QString("parseFetch: wrong INTERNALDATE: ")+line;
                return false;
              }
            }
            break;

          case 'R' :
            if( "RFC822.SIZE" == word )
            {
              qDebug("parseFetch - RFC822.SIZE - not available yet");
              return false;
            }
            else if( "RFC822" == word )
            {
              qDebug("parseFetch - RFC822 - not available yet");
              return false;
            }
            break;

          case 'U' :
            if( "UID" == word )
            {
              uPos = parseNextWord(word, line, uPos);
              if( !word.isEmpty() )
                d->fetchMessage.uid = word;
              else
              {
                d->error_=QString("parseFetch: wrong UID: ")+line;
                return false;
              }
            }
            break;

          default:
              qDebug("parseFetch - default - %s", word.latin1());
              return false;
            break;
        }
      }
      else
      {
        qDebug("Not available yet! :(");
        return false;
      }
      
      uPos = skipSpaces(line, uPos);
    }
    
    // if there isn't ) in the end, try to download the message
    if( ')' != first )
      setSingleLine( false );
    else
      setSingleLine( true );
  }
  else
  {
    d->error_=QString("parseFetch: wrong answer")+line;
    return false;
  }
  
  return true;
}

unsigned IMAP4Client::toMail(const QByteArray & bufferData)
{
//  char lastChar = d->fetchMessage.message.length() ? d->fetchMessage.message[ (int)d->fetchMessage.message.length() - 1 ] 
//                                          : 0;
  char currentChar = 0;
  unsigned uPrepend = 0;
  QCString theEnd(")\r\n");
  
  QByteArray data;
  QCString output;
  
  if( d->fetchBufferData_.isEmpty() ||
      !d->fetchBufferData_.size() ||
      !d->fetchBufferData_.length() )
  {
    data.resize(bufferData.size());
    memcpy(data.data(), bufferData.data(), bufferData.size());
  }
  else
  {
    uPrepend = d->fetchBufferData_.size() - 1;
    
    data.resize(uPrepend + bufferData.size());
    memcpy(data.data(), d->fetchBufferData_.data(), uPrepend);
    memcpy(data.data() + uPrepend, bufferData.data(), bufferData.size());
  }
    
  unsigned dataLength = data.size();
  output.resize( data.size()+10 );
  
  d->fetchBufferData_.resize(4);
  d->fetchBufferData_.data()[0] = 0;
    
  unsigned j=0;
  for(unsigned i=0; i<dataLength; ++i)
  {
    currentChar = data.data()[i];
    
    switch( currentChar )
    {
      case '\0':
#ifdef IMAP4_DEBUG
        qDebug("toMail() - Null char detected!");
#endif
        break;
        
      // just nothing to do
      case '\r':
        break;
        
      case ')':
        // read the next 3 chars (if possible)
        if( dataLength - i >= 3 )
        {
          memcpy(d->fetchBufferData_.data(), &data.data()[i], 3);
          if( theEnd == d->fetchBufferData_ && 
              d->fetchBufferSize_ + i + 3 >= d->fetchMessage.size )
          {
#ifdef IMAP4_DEBUG
            qDebug("End of message detected!");
#endif
            d->endOfMessage_ = true;
            
            output.data()[j] = 0;
             
            d->fetchMessage.message += output;
            
            return (i+3-uPrepend);
          }
          else
            d->fetchBufferData_.data()[0] = 0;
        }
        else
        {
          uPrepend = dataLength - i;
          
          d->fetchBufferData_.resize(uPrepend + 1);
          memcpy(d->fetchBufferData_.data(), data.data()+i, uPrepend);
          d->fetchBufferData_.data()[uPrepend] = 0;
          
          output.data()[j] = 0;
          d->fetchMessage.message += output;
            
          return bufferData.size();
        }
        break;
        
      default :
//        lastChar = currentChar;
        output.data()[j] = currentChar;
        j++;
        break;
    }
  }
  
  output.data()[j] = 0;
  d->fetchMessage.message += output;
  
  return bufferData.size();
}

bool IMAP4Client::handleFetchData(const QByteArray & block)
{
#ifdef IMAP4_DEBUG
  qDebug("handleFetchData(const QByteArray & block) size = %u || %u", block.size(), d->fetchMessage.size);
#endif
  
  // check if the fetch line is read
  if( !d->fetchMessage.size )
  {
    int iFind = block.find('\n');
    if( -1 != iFind )
    {
      d->fetchBuffer_ += QString::fromLatin1(block.data(), iFind+1);
      
      if( !processLine( d->fetchBuffer_ ) )
        return false;
      
      if( isSingleLine() )
        d->fetchBuffer_ = QString::null;
      
      if( block.size()-iFind > 1 )
      {
        QByteArray nextMessage;
          
        nextMessage.resize( block.size()-iFind-1 );
        memcpy( nextMessage.data(), &block.data()[iFind+1], block.size()-iFind-1);
          
        return handleFetchData( nextMessage );
      }
    }
    else
    {
      d->fetchBuffer_ += QString::fromLatin1(block.data(), block.size());
    }
    
    return true;
  }
  
  if( !d->fetchMessage.message.size() )
  {
    d->fetchMessage.message.resize( d->fetchMessage.size + 10 );
    d->fetchMessage.message[0] = 0;
  }
    
  unsigned int uSize = 0;
  uSize = toMail(block);
  d->fetchBufferSize_ += uSize;
  
  // end of message is detected
  if( d->endOfMessage_  )
  {
    d->endOfMessage_ = false;
    
    if( IMAP4Client::SYNC_JOB == d->command_ ||
        IMAP4Client::SYNC_FULL_JOB == d->command_ )
      IMAP4Handler::ref()->slotDownload( this );
    if( IMAP4Client::UPDATE_MESSAGE_JOB == d->command_ )
      IMAP4Handler::ref()->slotUpdateMessage( this );
      
    // initialize buffer variable
    d->fetchBufferData_.resize(0);
    d->fetchBufferSize_=0;
    d->fetchBuffer_=QString::null;
    d->fetchMessage.message = QString::null;
    d->fetchMessage.flags = 0;
    d->fetchMessage.size = 0;
    d->fetchMessage.uid = QString::null;
    d->fetchMessage.rcvtime = 0;
      
    int iFind = block.find( '\n', uSize);
    if( -1 != iFind )
    {
      d->fetchBuffer_ = QString::fromLatin1(&block.data()[uSize], iFind-uSize+1);
      
      if( !processLine( d->fetchBuffer_ ) )
        return false;
        
      if( isSingleLine() )
        d->fetchBuffer_ = QString::null;
      
      if( block.size()-iFind > 1 )
      {
        QByteArray nextMessage;
          
        nextMessage.resize( block.size()-iFind-1 );
        memcpy( nextMessage.data(), &block.data()[iFind+1], block.size()-iFind-1);
          
        return handleFetchData( nextMessage );
      }
    }
    else
    {
      d->fetchBuffer_ = QString::fromLatin1(&block.data()[uSize], block.size()-uSize);
    }
  }
  else if( d->fetchBufferSize_ > d->fetchMessage.size + 5 )
  {
#ifdef IMAP4_DEBUG
    qDebug("The last %u bytes !!%*.*s!!", uSize, uSize, uSize, block.data());
    qDebug("Buffer %u bytes !!%*.*s!!", block.size(), block.size(), block.size(), block.data());
#endif
      
    d->fetchBufferData_.resize(0);
    d->fetchBufferSize_=0;
    d->fetchBuffer_=QString::null;
    d->fetchMessage.message.resize(0);
    d->fetchMessage.flags = 0;
    d->fetchMessage.size = 0;
    d->fetchMessage.uid = QString::null;
    d->fetchMessage.rcvtime = 0;
  
    qDebug("handleFetchData: That's bad.");
      
    setSingleLine(true);
      
    return false;
  }
  
  return true;
}

bool IMAP4Client::parseSearch(const QString &line)
{
  QString word;
  unsigned uPos=0;
  unsigned length = line.length();

//  qDebug("parseSearch: %s", line.latin1());
    
  d->searchList.clear();
  d->searchUID.setAutoDelete(true);
  d->searchUID.clear();
  d->searchUID.setAutoDelete(false);
  
  if( 1 /*d->command_ == IMAP4Client::UID*/ )
  {
    while( uPos<length )
    {
      uPos = parseNextWord(word, line, uPos);
      if( word.isEmpty() )
        break;
      else
      {
        MailInfo * mailInfo = new MailInfo(word);
        d->searchUID.insert(word, mailInfo);
      }
    }
    
#ifdef IMAP4_DEBUG
    qDebug("Search size: %u", d->searchUID.count());
#endif
  }
  else if( d->command_ == IMAP4Client::SEARCH )
  {
    // TODO: 
    qDebug(" parseSearch: not implemented yet :(");
  }
  
  return true;
}

bool IMAP4Client::parseStatus(const QString &line)
{
// INBOX (MESSAGES 1234 RECENT 0 UNSEEN 12 UIDNEXT 12345 UIDVALIDITY 123456)

  QString word;
  unsigned uPos=0;
  unsigned length = line.length();
  unsigned long number=0;
  char first = 0;
  
  // parse the mailbox name
  uPos = parseNextWord(word, line, uPos);

  // it has to begin with '(' and to end with ')'
  uPos = skipSpaces(line, uPos);
#ifdef _QTWIN_  
  first = line[(int)uPos].latin1();
#else
  first = line[uPos].latin1();
#endif
  if( '(' == first )
  {
    uPos++;
    uPos = skipSpaces(line, uPos);
#ifdef _QTWIN_  
    first = line[(int)uPos].latin1();
#else
    first = line[uPos].latin1();
#endif

    while( uPos<length && ')'!=first )
    {
      if( '(' != first )
      {
        uPos = parseNextWord(word, line, uPos);
        if( word.isEmpty() )
          break;
        
        if( !parseUnsignedLong(word, number) )
        {
          qDebug("parseStatus: wrong number");
          return false;
        }
                        
        if( "MESSAGES" == word )
          setMessagesNumber( number );
        else if( "RECENT" == word )
          setRecent( number );
        else if( "UIDVALIDITY" == word )
          setUidValidity( number );
        else if( "UNSEEN" == word )
          setUnseen( number );
        else if( "UIDNEXT" == word )
          setUidNext( number );
        else
        {
          qDebug("parseStatus: unknown field");
          return false;
        }
      }
      else
      {
        qDebug("Not available yet! :(");
        return false;
      }
      
      uPos = skipSpaces(line, uPos);
#ifdef _QTWIN_  
      first = line[(int)uPos].latin1();
#else
      first = line[uPos].latin1();
#endif
    }
  }
  else
  {
    qDebug("parseStatus: wrong answer");
    return false;
  }
  
  return true;
}

bool IMAP4Client::parseList(const QString &line)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::parseList %s", line.latin1());
#endif
  
  unsigned uPos=0;
  unsigned length = line.length();
  QString word;
  char first = 0;
  
  QString name;
  unsigned state=IMAP4Client::NoState;
  
  
  // it has to begin with '('
  uPos = skipSpaces(line, uPos);
#ifdef _QTWIN_  
  first = line[(int)uPos].latin1();
#else
  first = line[uPos].latin1();
#endif
  if( '(' == first )
  {
    uPos++;
    uPos = skipSpaces(line, uPos);

    // parse the mailbox state state
    while( uPos<length )
    {
      uPos = parseNextWord(word, line, uPos);
      
      if( !word.isEmpty() )
      {
        word = word.upper();
  

        if( "\\NOSELECT" == word )
          state |= IMAP4Client::NoSelect;
        else if( "\\NOINFERIORS" == word )
          state |= IMAP4Client::NoInferiors;
        else if( "\\MARKED" == word )
          state |= IMAP4Client::Marked;
        else if( "\\UNMARKED" == word )
          state |= IMAP4Client::UnMarked;
        // TODO:
        else if( "\\HASNOCHILDREN" == word )
          state |= IMAP4Client::HasNoChildren;
        // TODO:
        else if( "\\HASCHILDREN" == word )
          state |= IMAP4Client::HasChildren;
        else
        {
          qDebug(" parseList: unknown mailbox state");
          return false;
        }
      }
      else
        break;
    }
    
    uPos = skipSpaces(line, uPos);
#ifdef _QTWIN_  
      first = line[(int)uPos].latin1();
#else
      first = line[uPos].latin1();
#endif
    if( ')' != first )
    {
      qDebug("parseList: ) is missing");
      return false;
    }
    uPos++;

    // parse the delimiter
    uPos = parseNextWord(word, line, uPos);
    
    if( word.isEmpty() )
    {
      qDebug("parseList: delimiter is missing");
      return false;
    }
    
    if( "NIL" != word.upper() )
    {
      d->delimiter_ = word;
      // remove the quotes
      d->delimiter_.replace( QRegExp("\""), "");
    }
        
    // parse the mailbox name
    
    if(line[(int)uPos+1]=='"')    // if mailbox is quoted
    {
      word=line.mid(uPos+1,(line.mid(uPos+2)).find('"')+2);
    }
    else
    {
      uPos = parseNextWord(word, line, uPos);
    }
    
    if( word.isEmpty() )
    {
      qDebug("parseList: mailbox name is missing");
      return false;
    }

#ifdef IMAP4_DEBUG
    if( "\"\"" == word )
    {
      qDebug("Delimiter found : %s", d->delimiter_.latin1());
    }
#endif
    
    // replace the delimiter with "/"
    if( !d->delimiter_.isEmpty() &&
        "/" != d->delimiter_ )
    {
      word.replace(d->delimiter_, "/");
    }
        
    // remove the quotes
    name = word.replace( QRegExp("\""), "");
        
    // use only 'Inbox' not 'INBOX', not 'inbox' etc.
    int iInbox = name.find("inbox", 0, false);
    if( -1 != iInbox )
    {
      name = name.left(iInbox) + "Inbox" + name.mid(iInbox + 5);
    }
            
    // insert a new element in the mailboxes list
    Mbox * mbox = d->mboxList[ImapUTF7Convertor::imapUTF7ToUnicode(name)];
    if( mbox )
    {
      mbox->state = state;
    }
    else
    {
      mbox = new Mbox(ImapUTF7Convertor::imapUTF7ToUnicode(name), d->delimiter_, state);
      d->mboxList.insert(name, mbox);
    }
  }
  else
  {
    qDebug("parseList: ( is missing");
    return false;
  }
  
  return true;
}

bool IMAP4Client::parseFlags(const QString & line, unsigned & uFlags)
{
  QString word;
  unsigned uPos=0;
  unsigned length = line.length();
  char first = 0;
  uFlags=0;
  
  uPos = skipSpaces(line);
#ifdef _QTWIN_  
  first = line[(int)uPos].latin1();
#else
  first = line[uPos].latin1();
#endif
  if('(' != first )
  {
    qDebug("parseFlags: ( is missing");
    return false;
  }
  
  uPos++;
#ifdef _QTWIN_  
  first = line[(int)uPos].latin1();
#else
  first = line[uPos].latin1();
#endif

#ifdef _QTWIN_  
  while( uPos<length && ')'!=(first = line[(int)uPos].latin1()) )
#else
  while( uPos<length && ')'!=(first = line[uPos].latin1()) )
#endif
  {
    uPos = parseNextWord(word, line, uPos);
    
    if( word.isEmpty() )
      break;
    
    word = word.upper();
    
    first = word[1].latin1();
    
    switch( first )
    {
      case 'A':
        if( "\\ANSWERED" == word )
          uFlags |= IMAP4Client::Answered;
        break;
      case 'D':
        if( "\\DELETED" == word )
          uFlags |= IMAP4Client::Deleted;
        else if( "\\DRAFT" == word )
          uFlags |= IMAP4Client::Draft;
        break;
      case 'F':
        if( "\\FLAGGED" == word )
          uFlags |= IMAP4Client::Flagged;
        break;
      case 'R':
        if( "\\RECENT" == word )
          uFlags |= IMAP4Client::Recent;
        break;
      case 'S':
        if( "\\SEEN" == word )
          uFlags |= IMAP4Client::Seen;
        break;
      case '*':
        if( "\\*" == word )
          uFlags |= IMAP4Client::Other;
        break;
      default:
        qDebug("parseFlag: unknown field");
        return true;
        break;
    }
    
    uPos = skipSpaces(line, uPos);
  }

  return true;
}
  

void IMAP4Client::setAuth(const QString & auth)
{
  d->auth_ = auth;
}

QString IMAP4Client::auth() const
{
  return d->auth_;
}
  
void IMAP4Client::setPermanentFlags(const QString& line)
{
  unsigned number=0;
  
  if( parseFlags(line, number) )
  {
    d->permanentFlags_ = number;
  }
}

unsigned IMAP4Client::permanentFlags() const
{
  return d->permanentFlags_;
}

void IMAP4Client::setFlags(const QString & line)
{
  unsigned number=0;
  
  if( parseFlags(line, number) )
  {
    d->flags_ = number;
  }
}

unsigned IMAP4Client::flags() const
{
  return d->flags_;
}

void IMAP4Client::setAccess(Access access)
{
  d->access_ = access;
}

bool IMAP4Client::access() const
{
  return d->access_;
}

void IMAP4Client::showStatus()
{
  qDebug("MAILBOX = %s", d->mailbox_.latin1());
  qDebug("Uid Validity = %lu", d->uidValidity_);
  qDebug("Messages number = %lu", d->messagesNumber_);
  qDebug("Unseen = %lu", d->unseen_);
  qDebug("Recent = %lu", d->recent_);
  qDebug("Uid Next = %lu", d->uidNext_);
}
  
void IMAP4Client::setUidValidity(unsigned long uidValidity)
{
  d->uidValidity_ = uidValidity;
}

unsigned long IMAP4Client::uidValidity() const
{
  return d->uidValidity_;
}

void IMAP4Client::setMessagesNumber(unsigned long messagesNumber)
{
  d->messagesNumber_ = messagesNumber;
}

unsigned long IMAP4Client::messagesNumber() const
{
  return d->messagesNumber_;
}

void IMAP4Client::setUnseen(unsigned long unseen)
{
  d->unseen_ = unseen;
}

unsigned long IMAP4Client::unseen() const
{
  return d->unseen_;
}

void IMAP4Client::setRecent(unsigned long recent)
{
  d->recent_ = recent;
}

unsigned long IMAP4Client::recent() const
{
  return d->recent_;

}

void IMAP4Client::setUidNext(unsigned long uidNext)
{
  d->uidNext_ = uidNext;
}

unsigned long IMAP4Client::uidNext() const
{
  return d->uidNext_;
}
  
void IMAP4Client::sendCapability()
{
  writeLine( commandTag() + " CAPABILITY" );
}

bool IMAP4Client::handleCapability(const QString &)
{
  qDebug("Not implemented.");
  
  return false;
}

void IMAP4Client::sendNoop()
{
  writeLine( commandTag() + " NOOP" );
}

bool IMAP4Client::handleNoop(const QString &)
{
  // just nothing here
  return true;
}

bool IMAP4Client::sendSTARTTLS()
{
  if( d->starttls_ )
  {
    qDebug("Sending STARTTLS...");
    
    writeLine( commandTag() + " STARTTLS" );
  }
  
  return true;
}

bool IMAP4Client::handleSTARTTLS(const QString & line)
{
  qDebug("handleSTARTTLS - begin");
  
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Waiting, tr("Starting TLS negotiation..."));
  
      qDebug("Starting TLS connection...");
      if( d->socket_->startTLS() )
      {
        // this flag should be set ONLY here
        d->starttls_ = false;
        
        // back to login
        sendLogin();
        return true;
      }
    }
    
    qDebug("handleSTARTTLS : cannot start TLS : %s", line.latin1());
    return false;
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendLogin()
{
  if( d->state_ != IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendLogin() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Good, tr("Login ..."));
    
    // for TLS connection you should call STARTTLS here
    if( d->starttls_ )
    {
      if( !sendSTARTTLS() )
        qDebug("IMAP4Client : error at STARTTLS");
    }
    else
    {
      writeLine( commandTag() + " LOGIN " + 
                 "\"" + d->account_->user + "\" \"" +
                 d->account_->password + "\"");
    }
  }
}

bool IMAP4Client::handleLogin(const QString & line)
{
//  qDebug("handleLogin - ");
  if(d->starttls_)
    return handleSTARTTLS(line);

  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Waiting, tr("Authenticated..."));
  
      setState( AUTHENTICATED );
      if(d->account_->haspassword)
      {
        Settings::instance()->writeEntry("/Aethera/Accounts/" + d->account_->accname + "/Password",QString(d->account_->encodePassword(d->account_->password)));
      }
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleCommand : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}
    
void IMAP4Client::sendLogout()
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED &&
      d->connection_ == IMAP4Client::NOT_CONNECTED )
  {
    qDebug("sendLogout() called in an wrong server state");
    commandCompleted();
  }
  else
    writeLine( commandTag() + " LOGOUT" );
}

bool IMAP4Client::handleLogout(const QString & line)
{
  qDebug("IMAP4Client::handleLogout");
  
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      IMAP4Handler::ref()->slotLogout( this );
      
      d->status->message(AppStatus::Server, AppStatus::Text, AppStatus::Warning, tr("IMAP is stopped."));
      stop();
      
      return true;
    }
    else
    {
      qDebug("handleCommand : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}
    
void IMAP4Client::sendClose()
{
  if( d->state_ != IMAP4Client::SELECTED )
  {
    qDebug("sendClose() called in an wrong server state");
    commandCompleted();
  }
  else
    writeLine( commandTag() + " CLOSE" );
}

bool IMAP4Client::handleClose(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      setState( AUTHENTICATED );
      d->mailbox_ = QString::null;
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleClose : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}
    
void IMAP4Client::sendExpunge()
{
  if( d->state_ != IMAP4Client::SELECTED )
  {
    qDebug("sendExpunge() called in an wrong server state");
    commandCompleted();
  }
  else
    writeLine( commandTag() + " EXPUNGE" );
}

bool IMAP4Client::handleExpunge(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      // TODO - remove the local messages too
      
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleExpunge : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendStatus(const QString & mailbox_, const QString & fields)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendStatus() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString mailbox(mailbox_);
    // set the right delimiter
    changeDelimiter(mailbox);

    // convert to utf7
    mailbox=ImapUTF7Convertor::unicodeToIMAPUTF7(mailbox);

    // quote the name with space
    quoteName(mailbox);
    
    if( fields.isEmpty() )
      writeLine( commandTag() + " STATUS " +
                 mailbox + " (MESSAGES RECENT UNSEEN UIDNEXT UIDVALIDITY)" );
    else
      writeLine( commandTag() + " STATUS " +
                 mailbox + " (" + fields + ")");
  }
}

bool IMAP4Client::handleStatus(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      
      // TODO: send current status to decide if the mailbox needs a sync()
      
      return true;
    }
    else
    {
      qDebug("handleStatus : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendSelect(const QString & mailbox_)
{
  if( d->state_ != IMAP4Client::AUTHENTICATED )
  {
    qDebug("sendSelect() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString mailbox(mailbox_);
    // set the right delimiter
    changeDelimiter(mailbox);

    // convert to utf7
    mailbox=ImapUTF7Convertor::unicodeToIMAPUTF7(mailbox);

    // quote the name with space
    quoteName(mailbox);
           
    writeLine( commandTag() + " SELECT " + mailbox );
  }
}

bool IMAP4Client::handleSelect(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      setState( IMAP4Client::SELECTED );
      if( d->commands_->currentCommand() )
        d->mailbox_ = d->commands_->currentCommand()->mailbox;
      
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleSelect : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendUid(const QString & subCommand)
{
  if( d->state_ != IMAP4Client::SELECTED )
  {
    qDebug("sendUid() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    if( "SEARCH" == subCommand.left(6).upper() )
      d->subCommand_ = IMAP4Client::SEARCH;
    else if( "FETCH" == subCommand.left(5).upper() )
      d->subCommand_ = IMAP4Client::FETCH;
    else if( "STORE" == subCommand.left(5).upper() )
      d->subCommand_ = IMAP4Client::STORE;
    else
    {
      // COPY is unsupported yet
      d->subCommand_ = IMAP4Client::NO_COMMAND;
    }
    
#ifdef IMAP4_DEBUG
    qDebug("Send: UID %s", subCommand.latin1());
#endif

    writeLine( commandTag() + " UID " + subCommand );
  }
}

bool IMAP4Client::handleUid(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleUid : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendFetch(const QString & subCommand)
{
  if( d->state_ != IMAP4Client::SELECTED )
  {
    qDebug("sendFetch() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    writeLine( commandTag() + " FETCH " + subCommand );
  }
}

bool IMAP4Client::handleFetch(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      
      // TODO - send end of download ?!
      
      return true;
    }
    else
    {
      qDebug("handleFetch : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendAppend(IndexClass *index, const QString & mailbox_)
{
  if( index && 
      d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendAppend() called in an wrong server state");
    commandCompleted();
    return;
  }
  
  QFile mailFile( index->getDataFilename() );
  if( !mailFile.open(IO_ReadOnly) )
  {
    qDebug("sendAppend() : error when reading the mail");
    commandCompleted();
    return;
  }
  
  QString mailbox(mailbox_);
  // set the right delimiter
  changeDelimiter(mailbox);

  // convert to utf7
  mailbox=ImapUTF7Convertor::unicodeToIMAPUTF7(mailbox);

  // quote the name 
  quoteName(mailbox);
  
  unsigned long dataLength = index->getUniblockLength();
  mailFile.at( index->getUniblockOffset() );
  
  char buffer[65536];
  unsigned long readBytes = 0L;
  unsigned long totalBytes = 0L;
  QDataStream fileStream( &mailFile );
  QCString line(4096);
  QCString outMessage(dataLength);
  char lastChar = 0;
  char currentChar = 0;
  
  while( (readBytes = dataLength - totalBytes) )
  {
    if( readBytes > 65536 )
      readBytes = 65536;
      
    fileStream.readRawBytes(buffer, readBytes);
    
    // change it for sending
    for(unsigned i=0; i<readBytes; ++i)
    {
      currentChar = buffer[i];
      
      switch( currentChar )
      {
        case '\n':
          lastChar = currentChar;
          line += "\r\n";
          if(line.length() > 4000)
          {
            outMessage += line;
            line = 0;
            line.resize(4096);
          }
          break;
//        case '.':
//          if( '\n' == lastChar )
//          {
//            lastChar = currentChar;
//            line += "..";
//            break;
//          }
        default :
          lastChar = currentChar;
          line += currentChar;
          break;
      }
    }
    
    totalBytes+=readBytes;
  }
  
  // add the last chars
  outMessage += line;
  
  mailFile.close();
  
  dataLength = outMessage.length();
 
  if( index->getUnreadMark() )
    writeLine( commandTag() + " APPEND " + mailbox
               + " (\\RECENT) {" + QString::number(dataLength) + "}");
  else
    writeLine( commandTag() + " APPEND " + mailbox 
               + " (\\SEEN) {" + QString::number(dataLength) + "}");
  
  QDataStream socketStream( d->socket_ );
  socketStream.writeRawBytes(outMessage,  dataLength);
    
  writeLine("");
}

bool IMAP4Client::handleAppend(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
//      IMAP4Handle::ref()->slotAddMessage( this );
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleAppend : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendCreate(const QString & mboxName)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendCreate() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString mailbox(mboxName);
    // set the right delimiter
    changeDelimiter(mailbox);

    // convert to utf7
    mailbox=ImapUTF7Convertor::unicodeToIMAPUTF7(mailbox);

    // quote the name with space
    quoteName(mailbox);
    
    writeLine( commandTag() + " CREATE " + mailbox );
  }
}

bool IMAP4Client::handleCreate(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleCreate : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendDelete(const QString & mboxName)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendDelete() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString mailbox(mboxName);
    // set the right delimiter
    changeDelimiter(mailbox);

    // convert to utf7
    mailbox=ImapUTF7Convertor::unicodeToIMAPUTF7(mailbox);

    // quote the name with space
    quoteName(mailbox);
      
    writeLine( commandTag() + " DELETE " + mailbox );
  }
}

bool IMAP4Client::handleDelete(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleDelete : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendRename(const QString & oldName_, const QString & newName_)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendRename() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString oldName(oldName_);
    QString newName(newName_);
    
    // set the right delimiter
    changeDelimiter(newName);
    // set the right delimiter
    changeDelimiter(oldName);

    // convert to utf7
    newName=ImapUTF7Convertor::unicodeToIMAPUTF7(newName);
    oldName=ImapUTF7Convertor::unicodeToIMAPUTF7(oldName);

    // quote the name with space
    quoteName(newName);
    quoteName(oldName);
      
    writeLine( commandTag() + " RENAME " + oldName + " " + newName );
  }
}

bool IMAP4Client::handleRename(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleRename : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendSubscribe(const QString & mailbox_)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendSubscribe() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString mailbox(mailbox_);
    // set the right delimiter
    changeDelimiter(mailbox);

    // convert to utf7
    mailbox=ImapUTF7Convertor::unicodeToIMAPUTF7(mailbox);

    // quote the name with space
    quoteName(mailbox);
    
    writeLine( commandTag() + " SUBSCRIBE " + mailbox);
  }
}

bool IMAP4Client::handleSubscribe(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();
      return true;
    }
    else
    {
      qDebug("handleSubscribe : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendUnsubscribe(const QString & mailbox_)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendUnsubscribe() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString mailbox(mailbox_);
    // set the right delimiter
    changeDelimiter(mailbox);

    // convert to utf7
    mailbox=ImapUTF7Convertor::unicodeToIMAPUTF7(mailbox);

    // quote the name with space
    quoteName(mailbox);
    
    writeLine( commandTag() + " UNSUBSCRIBE " + mailbox);
  }
}

bool IMAP4Client::handleUnsubscribe(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      commandCompleted();

      return true;
    }
    else
    {
      qDebug("handleUnsubscribe : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendList(const QString & path_, const QString & folders_)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendList() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString path(path_);
    QString folders(folders_);

    if(path=="\"\"")
    {
      path=QString("""");
    }

    if(folders=="\"\"")
    {
      folders=QString("""");
    }
    
    // convert to imap utf7
    path=ImapUTF7Convertor::unicodeToIMAPUTF7(path);
    folders=ImapUTF7Convertor::unicodeToIMAPUTF7(folders);

    // quote the name with space
    quoteName(path);
    quoteName(folders);
        
    writeLine( commandTag() + " LIST " + path + " " + folders );
  }
}

bool IMAP4Client::handleList(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      // send the list of folders
//      IMAP4Handler::ref()->slotSubfolders( this );
      
      // remove the list after it was used
//      d->mboxList.clear();
      
      commandCompleted();
      
      return true;
    }
    else
    {
      qDebug("handleList : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

void IMAP4Client::sendLsub(const QString & path_, const QString & folders_)
{
  if( d->state_ == IMAP4Client::NOT_AUTHENTICATED )
  {
    qDebug("sendLsub() called in an wrong server state");
    commandCompleted();
  }
  else
  {
    QString folders(folders_);
    QString path(path_);

    if(path=="\"\"")
    {
      path=QString("""");
    }

    if(folders=="\"\"")
    {
      folders=QString("""");
    }
   
    // convert to imap utf7
    path=ImapUTF7Convertor::unicodeToIMAPUTF7(path);
    folders=ImapUTF7Convertor::unicodeToIMAPUTF7(folders);
    // quote the name with space
    quoteName(folders);
    quoteName(path);
       
    writeLine( commandTag() + " LSUB " + path + " " + folders );
  }
}

bool IMAP4Client::handleLsub(const QString & line)
{
  QString firstWord;
  
  parseNextWord(firstWord, line);
  if( firstWord == getTag() )
  {
    if( replyOK(line) )
    {
      // send the list of folders
      IMAP4Handler::ref()->slotSubfolders( this );
      
      commandCompleted();
      
      return true;
    }
    else
    {
      qDebug("handleLsub : bad answer : %s", line.latin1());
      return false;
    }
  }
  else
    return parseLine(line);
}

QString& IMAP4Client::changeDelimiter(QString& mailbox)
{
  if( !d->delimiter_.isEmpty() &&
      "/" != d->delimiter_ )
  {
    mailbox.replace("/", d->delimiter_);
  }
  
  return mailbox;
}

QString& IMAP4Client::quoteName(QString& mailbox)
{

  if(!(mailbox.left(1)=='"' && mailbox.right(1)=='"'))
    {
      mailbox= '"' + mailbox + '"';
    }
    
  return mailbox;
}
