/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

/***************************************************************************
              imap4client.h - communication with the imap server
                             -------------------

  copyright : (C) by theKompany.com     <www.thekompany.com>
              (C) by Atol C&D           <www.atolcd.com>

  authors :   Eugen C.      <eug@thekompany.com>
              Fabien Armand <f.armand@atolcd.com>

 ***************************************************************************/

#ifndef IMAP4CLIENT_H
#define IMAP4CLIENT_H

#include <qobject.h>
#include <qtimer.h>
#include <qptrqueue.h>
#include <qdatetime.h>
#include <qptrlist.h>

#include <mailinfo.h>

// #define IMAP4_DEBUG

class IMAP4Private;
class Account;
class Command;
class Mbox;
class FetchInfo;
class IndexClass;

typedef QDict<Mbox> MboxList;
typedef QDictIterator<Mbox> MboxListIterator;

class IMAP4Client : public QObject
{
  Q_OBJECT
  
  public:
  
    /** The server answer about accepting the command.*/
    enum Answer {OK=0, NO, BAD, IMAP_ERROR};
    /** The connection state.*/
    enum Connection {NOT_CONNECTED=0, WAITING_FOR_CONNECTION, BUSY, IDLE};
    /** The server state.*/
    enum State {NOT_AUTHENTICATED=0, AUTHENTICATED, SELECTED};
    /** The protocol commands.*/
    enum Commands { NO_COMMAND=-1, NOOP=0,
                CAPABILITY, STARTTLS, LOGOUT, LOGIN, AUTHENTICATE, SELECT,
                EXAMINE, CREATE, _DELETE_, RENAME, SUBSCRIBE, UNSUBSCRIBE,
                LIST, LSUB, STATUS, APPEND, CHECK, CLOSE, EXPUNGE, SEARCH,
                FETCH, STORE, COPY, UID, DELIMITER,
                SYNC_JOB, SYNC_FULL_JOB, UPDATE_MESSAGE_JOB,
                CREATE_FOLDER_JOB, DELETE_FOLDER_JOB, RENAME_FOLDER_JOB,
                ADD_MESSAGE_JOB, DELETE_MESSAGE_JOB, SUBFOLDERS_JOB,
                KOLAB_JOB, CITADEL_JOB, SHARED_FOLDERS_JOB };
    /** The protocol flags.*/
    enum Flags {NO_FLAGS=0, Seen=1, Answered=2, Flagged=4, Draft=8, Recent=16, Deleted=32, Other=64};
  
    /** The protocol access types.*/
    enum Access { READ_WRITE=0, READ_ONLY};
  
    /** The folders and mailboxes list state.*/
    enum MboxState { NoState=0, NoInferiors=1, NoSelect=2, Marked=4,
                     UnMarked=8, HasChildren=16, HasNoChildren=32 };
  
  
    IMAP4Client(Account *, QObject * parent = 0, const char * name = 0);
    virtual ~IMAP4Client();

    bool insertCommand( Commands type,
                        const QString & account, 
                        const QString & mailbox, 
                        const QString & parameters,
                        const QString & other );
    
    void stop();
    
    Command * currentCommand();
    
    QDateTime lastRun() const;
    
    Account * account();
    
    FetchInfo& incomingMail();
    
    MboxList& subfolders();
    QString delimiter();
    
    UIDMailList& searchUID();
    
    RetrMailList& searchList();
    
    /** Get the server state.*/
    State state() const;

    void commandCompleted();
    
    bool isJob();
    
    bool isSingleLine();
    void setSingleLine(bool);
    
    void resetIdleTimer();
    
    virtual bool processLine(const QString &);
    virtual bool processMultiLine(const QByteArray &);
    
    virtual unsigned toMail(const QByteArray & bufferData);
    
    virtual void writeLine(const QString &);

    QString mailbox() const;
    
    Connection connectionState() const;
    void setConnection(Connection);
    /** Set the server state.*/
    void setState(State);
    /** Get the last error.*/
    QString error() const;
  
    /** Get the command tag.*/
    QString getTag() const;
    /** Get the command tag and increment the tag number.*/
    QString commandTag();
  
    //Capability
    /** Set the authenticate attribute.*/
    void setAuth(const QString & auth);
    /** Get the authenticate attribute.*/
    QString auth() const;
  
    // Flags methods.
    /** Set permanent flags.*/
    void setPermanentFlags(const QString & flags);
    /** Get the permanent flags.*/
    unsigned permanentFlags() const;
    /** Set the flags.*/
    void setFlags(const QString & flags);
    /** Get the flags.*/
    unsigned flags() const;
    /** Set the access flag.*/
    void setAccess(Access = READ_WRITE);
    /** Get the access flag.*/
    bool access() const;
  
    // Info attributes methods
    /** Set the UIDVALIDITY.*/
    void setUidValidity(unsigned long uidValidity);
    /** Get the UIDVALIDITY.*/
    unsigned long uidValidity() const;
    /** Set the messages number.*/
    void setMessagesNumber(unsigned long messagesNumber);
    /** Get the messages number.*/
    unsigned long messagesNumber() const;
    /** Set the unseen message number.*/
    void setUnseen(unsigned long unseen);
    /** Get the unseen message number.*/
    unsigned long unseen() const;
    /** Set the recent messages number.*/
    void setRecent(unsigned long recent);
    /** Get the recent messages number.*/
    unsigned long recent() const;
    /** Set the next message uid.*/
    void setUidNext(unsigned long uidNext);
    /** Get the next message uid.*/
    unsigned long uidNext() const;
  
    void connectToHost();

    // ------------ IMAPv4 Commands ----------------
    /**
      * Capability.
      * Ask the server what authenticate does it support.
      */
    void sendCapability();
    bool handleCapability(const QString &);
  
    /**
      * Noop.
      * Just a command for keeping alive the connection,
      * but the server can send the last changes like an
      * answer.
      */
    void sendNoop();
    bool handleNoop(const QString &);
    
    
    /** Send STARTTLS command for a secure TLS connection.*/
    bool sendSTARTTLS();
    bool handleSTARTTLS(const QString & line);
    
    /**
      * Login.
      * The server will be in authenticated state.
      */
    void sendLogin();
    bool handleLogin(const QString & line);
    
    /**
      * Logout.
      * It close the connection and the server will be in
      * the logout state.
      */
    void sendLogout();
    bool handleLogout(const QString & line);
    
    /**
      * Close.
      * It close the connection and the server will be in
      * the logout state.
      */
    void sendClose();
    bool handleClose(const QString & line);
    
    /**
      * Expunge.
      * Delete all marked messages.
      */
    void sendExpunge();
    bool handleExpunge(const QString & line);
    
    /**
      * Status.
      * Get the actual status of the IMAP folder (mailbox).
      */
    void sendStatus(const QString & mailbox="INBOX", const QString & fields=QString::null);
    bool handleSatus(const QString & line);
    
    /**
      * Select.
      * It has one parameter:
      *   - mailbox name
      * It select a mailbox and the server will send some
      * information about it.
      * The server will be in the selected state.
      */
    void sendSelect(const QString & mboxName="INBOX");
    bool handleSelect(const QString & line);

    /**
      * Uid.
      * UID SEARCH ALL and UID SEARCH NEW
      */
    void sendUid(const QString & subCommand);
    bool handleUid(const QString & line);

    /**
      * FETCH.
      */
    void sendFetch(const QString & subCommand);
    bool handleFetch(const QString & line);

    /**
      * APPEND.
      * Add a new mail to the mailbox.
      */
    void sendAppend(IndexClass * index, const QString & mboxName="INBOX");
    bool handleAppend(const QString & line);

    /**
      * CREATE.
      * Create a new mailbox.
      */
    void sendCreate(const QString & mboxName);
    bool handleCreate(const QString & line);

    /**
      * DELETE.
      * Delete a mailbox.
      * It doesn't make unsubscribe!
      */
    void sendDelete(const QString & mboxName);
    bool handleDelete(const QString & line);

    /**
      * RENAME.
      * Rename the mailbox.
      * It doesn't change the subscribed name.
      * The INBOX rename will move all the mails into the new mailbox and
      * INBOX is still available.
      */
    void sendRename(const QString & newName, const QString & oldName);
    bool handleRename(const QString & line);


    /**
      * SUBSCRIBE.
      * Subscribe a mailbox.
      */
    void sendSubscribe(const QString & mailbox);
    bool handleSubscribe(const QString & line);

    /**
      * UNSUBSCRIBE.
      * Unsubscribe a mailbox.
      */
    void sendUnsubscribe(const QString & mailbox);
    bool handleUnsubscribe(const QString & line);

    /**
      * LIST.
      * Return a list of folders and mailboxes.
      * A path = "" means 'use the selected mailbox by default'
      * The folders parameter can have wildchars.
      */
    void sendList(const QString & path="\"\"", const QString & folders="*");
    bool handleList(const QString & line);

    /**
      * LSUB.
      * Return a list of subscribed mailboxes.
      * A path = "" means 'use the selected mailbox by default'
      * The folders parameter can have wildchars.
      */
    void sendLsub(const QString & path="\"\"", const QString & folders="*");
    bool handleLsub(const QString & line);
    
    bool handleStatus(const QString & line);
    bool handleCommand(const QString & line);


    // --------- IMAPv4 parser methods -------------
    /**
      * Parse the server answer for the command status.
      */
    virtual bool replyOK(const QString & reply);
    
    /**
      * Parse the line result.
      */
    bool parseLineResult(const QString &line);

    /** Return the first position with a valid char.*/
    unsigned skipSpaces(const QString & line, unsigned uPos=0);
    
    /**
      * Find the next word and return the first position after it.
      */
    virtual unsigned parseNextWord(QString & strWord, const QString & line, unsigned uPos=0);

    /** Get the first word from line like an unsigned long.*/
    bool parseUnsignedLong(const QString & line, unsigned long & number);

    /**
      * Parse the untaged lines.
      */
    bool parseLine(const QString &line);

    /**
      * Parse the capability answer.
      */
    bool parseCapability(const QString &line);

    /**
      * Parse the FETCH answer.
      */
    bool parseFetch(const QString &line);
    /** Manage the message received bytes.*/
    bool handleFetchData(const QByteArray & block);

    /**
      * Parse the SEARCH answer.
      */
    bool parseSearch(const QString &line);

    /**
      * Parse the STATUS answer.
      */
    bool parseStatus(const QString &line);

    /**
      * Parse the LIST/LSUB answer.
      */
    bool parseList(const QString &line);

    /**
      * Parse the flags (permanent or not).
      */
    bool parseFlags(const QString & flags, unsigned & uFlags);

    /** Debug method.*/
    void showStatus();
    
  public slots:
    void slotJobFinished(bool);
  
  protected slots:

    void slotHostFound();
    void slotConnected();
    void slotConnectionClosed();
    void slotReadyRead();
    void slotBytesWritten(int);
    void slotError(int);
    void slotIdle();
    void slotRunQueue();
    void slotEmitFinished();
    void slotEmitNewCommand();

  signals:
    void signalNewCommand();
    void signalCommandCompleted();
    void finished(IMAP4Client*);
    
  protected:
  
    QString& changeDelimiter(QString& mailbox);
    QString& quoteName(QString& mailbox);

  private:
  
    IMAP4Private * d;

};

// other util classes

class Mbox
{
  public:
    Mbox( const QString & mboxName = QString::null,
          const QString & mboxDelimiter = QString::null,
          unsigned mboxState = IMAP4Client::NoState);
    ~Mbox();
  
    QString   name;
    QString   delimiter;
    unsigned  state;
};

class FetchInfo
{
  public:
    FetchInfo();
    virtual ~FetchInfo();
    
    QCString message;
    QString uid;
    unsigned int rcvtime;
    unsigned flags;
    unsigned long size;
};

#endif
