/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          imap4handler.cpp  -  the handler for the IMAPv4 protocol
                             -------------------
    begin                : Wed Feb  8 14:02:00 EET 2001
    copyright            : (C) 2001-2005 by theKompany <www.thekompany.com>
                           (C) 2004 by Atol C&D   <www.atolcd.com>
    author               : Eugen Constantinescu
                           Fabien Armand
    email                : eug@thekompany.com
                           f.armand@atolcd.com
 ***************************************************************************/

#include "imap4handler.h"

#include <accountmanager.h>
#include <accounts.h>
#include <iosynchandler.h>
#include <servernotifier.h>
#include <remotemailfolder.h>
#include <orb.h>
#include <messagedescriptor.h>
#include <messagedevice.h>
#include <commandsprocessor.h>
#include <settings.h>
#include <kolabsharedfolder.h>

#include <qdialog.h>
#include <passwddialog.h>

#include <qregexp.h>

/** The singleton IMAP handler instance.*/
IMAP4Handler *IMAP4Handler::thisInstance = 0;

IMAP4Handler::IMAP4Handler()
  : QObject()
{
}

IMAP4Handler::~IMAP4Handler()
{
}

IMAP4Handler *IMAP4Handler::ref()
{
  return thisInstance ? thisInstance : (thisInstance=new IMAP4Handler());
}


void IMAP4Handler::setImapFolder( IMAP4Client * client )
{
  if( client )
  {
    Command * pCommand = client->currentCommand();
    if( pCommand )
    {
      if( -1 == pCommand->other.find("/") )
      {
        imapFolder_ = (RemoteMailFolder*)ObjectRequestBroker::thisInstance()->mailfolderReference( pCommand->other );
      }
      else
      {
        IndexClass * index = ObjectRequestBroker::thisInstance()->indexReference( pCommand->other );
        
        if( index )
          imapFolder_ = dynamic_cast<RemoteMailFolder*> (index->getParentFolder());
        else
          imapFolder_ = 0;
      }
      
//      if( imapFolder_ )
//        qDebug("IMAP folder is %s", imapFolder_->name().latin1());
    }
  }
  else
    imapFolder_ = 0;
}

RemoteMailFolder* IMAP4Handler::imapFolder()
{
  return imapFolder_;
}

bool IMAP4Handler::insertCommand( IMAP4Client::Commands type,
                                  const QString & account, 
                                  const QString & mailbox, 
                                  const QString & parameters,
                                  const QString & other )
{

  removeAllStopped();
  
  Account * pAccount = 0;
  
  if( account.isEmpty() )
  {
#ifdef IMAP4_DEBUG
    qDebug("IMAP4Handler::insertCommand : default account");
#endif

    pAccount = AccountManager::ref()->getDefaultAccount();

  }
  else
  {
    pAccount = AccountManager::ref()->getAccount( account );
  }

  if (0 == pAccount)
  {
    qDebug("IMAPHandler::insertCommand : no account");
    return false;
  }
  
  if (Account::IMAP4 != pAccount->type)
  {
    qWarning("IMAP4Handler::insertCommand() : wrong account type");
    return false;
  }

  IMAP4Client * client = clientMap_[pAccount->accname];

  if (0 == client)
  {
#ifdef IMAP4_DEBUG
    qDebug("No existing IMAP4Client for account `%s'", pAccount->accname.latin1());
#endif

    client =
      new IMAP4Client
      (
        pAccount,
        this,
        ("IMAP4Client for " + pAccount->accname).latin1()
      );

    clientMap_.insert(pAccount->accname, client);
    
    connect(client, SIGNAL(finished(IMAP4Client*)), this, SLOT(slotClientFinished(IMAP4Client*)));
  }
#ifdef IMAP4_DEBUG
  else
  {
    qDebug("Already have IMAP4Client for account `%s'", pAccount->accname.latin1());
  }
#endif
  
  if(pAccount->password.isEmpty())
  {
    PasswdDialog passwdDialog(pAccount->accname + " (imap)");
    if( QDialog::Accepted==passwdDialog.exec() )
    {
      pAccount->password=passwdDialog.getPassword();
      client->account()->password=passwdDialog.getPassword();
    }
    else
    {
      return false;
    }
  }

  return client->insertCommand(type, account, mailbox, parameters, other);

}
  
bool IMAP4Handler::mount(const QString & url, bool bSync)
{
  if( url.isEmpty() )
  {
    qDebug("IMAP4Handler::mount() : empty url");
    return false;
  }
    
#ifdef IMAP4_DEBUG
  qDebug("Mount folder=%s", url.latin1());
#endif

  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(ObjectRequestBroker::thisInstance()->mailfolderReference(url));
  if( !folder )
  {
    qDebug("imap4handler: mount: wrong folder url");
    return false;
  }

  bool bRet = true;
  
  // Set the command buffer.
  bRet = insertCommand( IMAP4Client::LOGIN,
                        folder->account(), 
                        folder->mailbox(), 
                        QString::null, 
                        url);
  if( !bRet )
    return false;
  
  bRet = insertCommand( IMAP4Client::SELECT,
                        folder->account(), 
                        folder->mailbox(), 
                        QString::null,
                        url);
  if( !bRet )
    return false;
  
  if( bSync )
  {
    if( !isKolab(folder->account(), folder->mailbox()) &&
        !isKolabSharedFolder(folder->account(), folder->mailbox()) )
    {
      bRet = insertCommand( IMAP4Client::SYNC_JOB,
                            folder->account(), 
                            folder->mailbox(),
                            QString::null,
                            url);
    }
    else
    {
      if( KolabSharedFolder::ref()->isTempSharedFolder
          (folder->account(), folder->mailbox()) )
      {
        bRet = insertCommand( IMAP4Client::SYNC_FULL_JOB,
                              folder->account(), 
                              folder->mailbox(),
                              "refresh",
                              url);
      }
      else
      {
        bRet = insertCommand( IMAP4Client::SYNC_FULL_JOB,
                              folder->account(), 
                              folder->mailbox(),
                              QString::null,
                              url);
      }
    }
  }
  
  if( !bRet )
    return false;
  
  return true;
}

bool IMAP4Handler::login(const QString & account)
{
#ifdef IMAP4_DEBUG
  qDebug("Login account %s", account.latin1());
#endif
 
  // Set the command buffer.
  if( !insertCommand( IMAP4Client::LOGIN, account) )
    return false;
  
  return true;

}

bool IMAP4Handler::sync(const QString & url)
{
#ifdef IMAP4_DEBUG
  qDebug("Sync %s", url.latin1());
#endif

  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(ObjectRequestBroker::thisInstance()->mailfolderReference(url));
  if( !folder )
  {
    qDebug("imap4handler: sync: wrong folder url");
    return false;
  }

  bool bRet = true;
  if( !isKolab(folder->account(), folder->mailbox()) &&
      !isKolabSharedFolder(folder->account(), folder->mailbox()) )
  {
    bRet = insertCommand( IMAP4Client::SYNC_JOB,
                          folder->account(), 
                          folder->mailbox(),
                          QString::null,
                          url);
  }
  else
  {
      if( KolabSharedFolder::ref()->isTempSharedFolder
          (folder->account(), folder->mailbox()) )
      {
        bRet = insertCommand( IMAP4Client::SYNC_FULL_JOB,
                              folder->account(), 
                              folder->mailbox(),
                              "refresh",
                              url);
      }
      else
      {
        bRet = insertCommand( IMAP4Client::SYNC_FULL_JOB,
                              folder->account(), 
                              folder->mailbox(),
                              QString::null,
                              url);
      }
  }

  return bRet;
}

bool IMAP4Handler::refresh(const QString & url)
{
#ifdef IMAP4_DEBUG
  qDebug("Refresh %s", url.latin1());
#endif

  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(ObjectRequestBroker::thisInstance()->mailfolderReference(url));
  if( !folder )
  {
    qDebug("imap4handler: refresh: wrong folder url");
    return false;
  }

  bool bRet = true;
  if( !isKolab(folder->account(), folder->mailbox()) &&
      !isKolabSharedFolder(folder->account(), folder->mailbox()) )
  {
    bRet = insertCommand( IMAP4Client::SYNC_JOB,
                          folder->account(), 
                          folder->mailbox(),
                          "refresh",
                          url);
  }
  else
  {
    bRet = insertCommand( IMAP4Client::SYNC_FULL_JOB,
                          folder->account(), 
                          folder->mailbox(),
                          "refresh",
                          url);
  }

  return bRet;
}

bool IMAP4Handler::unmount(const QString & url)
{
#ifdef IMAP4_DEBUG
  qDebug("UnMount folder=%s", url.latin1());
#endif

  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(ObjectRequestBroker::thisInstance()->mailfolderReference(url));
  if( !folder )
  {
    qDebug("imap4handler: mount: wrong folder url");
    return false;
  }

  // Set the command buffer.
  if( !insertCommand( IMAP4Client::CLOSE,
                      folder->account(),
                      folder->mailbox(),
                      QString::null,
                      url ) )
    return false;

  return true;
}

bool IMAP4Handler::logout(const QString & account)
{
#ifdef IMAP4_DEBUG
  qDebug("Logout %s", account.latin1());
#endif

  if( !insertCommand(IMAP4Client::LOGOUT, account) )
    return false;

  return true;
}

bool IMAP4Handler::stopDelivery(Account * account)
{
  removeAllStopped();
  
  IMAP4Client * client = clientMap_[account->accname];

  if (0 == client)
  {
    qDebug
      (
        "stopDelivery for `%s': no delivery in progress",
        account->accname.latin1()
      );
  }
  else
  {
    client->stop();
  }

  removeAllStopped();
  
  return true;
}

bool IMAP4Handler::stopDelivery()
{
  removeAllStopped();
  
  for(QDictIterator<IMAP4Client> it(clientMap_); it.current(); ++it)
  {
    it.current()->stop();
  }
  
  removeAllStopped();
  
  return true;
}

bool IMAP4Handler::updateMessage(IndexClass &index)
{
  // get the parent folder
  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(index.getParentFolder());
  if(!folder)
  {
    qDebug("IMAP4Handler::updateMessage : no parent folder!");
    return false;
  }
  
  // Eugen C. - lock the mail till the update is done
  index.lock();

  // load descriptor
  MessageDevice *dev=new MessageDevice(&index);
  dev->loadDescriptor();
  MessageDescriptor &descriptor=dev->getDescriptor();

  QString uid = descriptor.uid;
  uid = uid.left( (unsigned)uid.find("@") );

  // deallocate the device
  delete dev;

  // call the update command
  if( !insertCommand( IMAP4Client::UPDATE_MESSAGE_JOB,
                      folder->account(),
                      folder->mailbox(),
                      uid,
                      index.url() ) )
    return false;

  return true;
}

bool IMAP4Handler::deleteMessage(IndexClass &index)
{
  // get the parent folder
  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(index.getParentFolder());

  // load descriptor
  MessageDevice *dev=new MessageDevice(&index);
  dev->loadDescriptor();
  MessageDescriptor &descriptor=dev->getDescriptor();
  
  QString uid = descriptor.uid;
  uid = uid.left( (unsigned)uid.find("@") );
  
  // deallocate the device
  delete dev;
  
  if( !insertCommand( IMAP4Client::DELETE_MESSAGE_JOB,
                      folder->account(),
                      folder->mailbox(),
                      uid,
                      index.url() ) )
    return false;

  return true;
}

bool IMAP4Handler::getMail(QString & accountName, QString & mailbox)
{
  //qDebug("IMAP4Handler::getMail() is not available yet!");
  //qDebug("Please select the IMAP folder if you want to get the mails");
     
  // TODO: check the INBOX folder
  
  qDebug(accountName);
  qDebug(mailbox);
  QString url="imap_"+accountName;
 
  bool bRet = true;
    
  bRet = insertCommand( IMAP4Client::SYNC_JOB,
                          accountName,
                          mailbox,
                          QString::null,
                          url);
                 
  return bRet;   
}

bool IMAP4Handler::addMessage(const QString & url, IndexClass *index)
{
  // get the parent folder
  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(ObjectRequestBroker::thisInstance()->mailfolderReference(url));
  if( !folder )
  {
    qDebug("imap4handler: addMessage: wrong folder url");
    return false;
  }

  if( !insertCommand( IMAP4Client::ADD_MESSAGE_JOB,
                      folder->account(),
                      folder->mailbox(),
                      index->url(),
                      url ) )
    return false;

  bool bRet = true;
  if( !isKolab(folder->account(), folder->mailbox()) && 
      !isKolabSharedFolder(folder->account(), folder->mailbox()) )
  {
    bRet = insertCommand( IMAP4Client::SYNC_JOB,
                          folder->account(), 
                          folder->mailbox(),
                          QString::null,
                          url);
  }
  else
  {
    bRet = insertCommand( IMAP4Client::SYNC_FULL_JOB,
                          folder->account(), 
                          folder->mailbox(),
                          QString::null,
                          url);
  }

  return bRet;
}

bool IMAP4Handler::createFolder(const QString & account, const QString & mailbox, const QString & url)
{
  if( !insertCommand( IMAP4Client::CREATE_FOLDER_JOB,
                      account, 
                      mailbox,
                      QString::null,
                      url ) )
    return false;

  return true;
}

bool IMAP4Handler::deleteFolder(const QString & account, const QString & mailbox, const QString & url)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Handler::deleteFolder %s", url.latin1());
#endif

  fflush(stdout);
  
  if( !insertCommand( IMAP4Client::DELETE_FOLDER_JOB,
                      account, 
                      mailbox,
                      QString::null,
                      url ) )
    return false;

  return true;
}

bool IMAP4Handler::subfolders(const QString & account, const QString & mailbox, const QString & other)
{
  if( !insertCommand( IMAP4Client::SUBFOLDERS_JOB,
                      account, 
                      mailbox,
                      QString::null,
                      other ) )
    return false;

  return true;
}

bool IMAP4Handler::runKolabJob(const QString & account, const QString & mailbox, const QString & state)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Handler::runKolabJob : %s", account.latin1());
#endif

  if( !insertCommand( IMAP4Client::KOLAB_JOB,
                      account, 
                      mailbox,
                      "KOLAB",
                      state ) )
    return false;

  return true;
}

bool IMAP4Handler::runKolabSharedJob(const QString & account, const QString & mailbox, const QString & state)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Handler::runKolabSharedJob : %s", account.latin1());
#endif

  if( !insertCommand( IMAP4Client::KOLAB_JOB,
                      account, 
                      mailbox,
                      "SHARED",
                      state ) )
    return false;

  return true;
}

void IMAP4Handler::slotClientFinished(IMAP4Client * client)
{
  if(!client)
    return;
  
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Client::slotClientFinished(%s)", client->account()->accname.latin1());
#endif

  stoped_.append(client);
}

void IMAP4Handler::slotUpdateMessage(IMAP4Client * client)
{
  if(!client)
    return;
  
  setImapFolder( client );
  
  IOSyncHandler::ref()->updateMessage( client->incomingMail() , client->currentCommand()->other);
}

void IMAP4Handler::slotLogout(IMAP4Client * client)
{
  if( !client ||
      client->mailbox().isEmpty() )
    return;
  
  setImapFolder( client );
  // force sync
  IOSyncHandler::ref()->imap4Sync();
  // save the folder status
  imapFolder()->saveStatus();
  // save the folder messages UID
  imapFolder()->uidJar->saveLists();
}

void IMAP4Handler::slotDownload(IMAP4Client * client)
{
  setImapFolder( client );
  // get message and pass it to the IOSyncHandler
  Command * pCommand = client->currentCommand();
  if( !pCommand )
  {
    qDebug("IMAP4Handler::slotDownload : fatal error");
    return;
  }
  
  if( pCommand->type == IMAP4Client::SYNC_JOB )
  {
    IOSyncHandler::ref()->dispatchMessage(client->incomingMail(), false, true);
  }
  else if( pCommand->type == IMAP4Client::SYNC_FULL_JOB )
  {
    IOSyncHandler::ref()->dispatchMessage(client->incomingMail(), true, true);
  }
  else
  {
    IOSyncHandler::ref()->dispatchMessage(client->incomingMail(), false, false);
  }
}

void IMAP4Handler::slotProgress(IMAP4Client * client)
{
  setImapFolder( client );
  Command * pCommand = client->currentCommand();

  IOSyncHandler::ref()->imap4Sync();
  
  if( pCommand->type == IMAP4Client::SYNC_FULL_JOB )
  {
    IOSyncHandler::ref()->kolabSync();
  }
  
//
//  ServerNotifier::thisInstance()->ioProgress(QString(progressData.c_str()));
//
}

void IMAP4Handler::slotSubfolders(IMAP4Client * client)
{
  Command * pCommand = client->currentCommand();
  
  if( !pCommand )
  {
    qDebug("IMAP4Handler::slotSubfolders() : null command");
    return;
  }
  
  QString type="imap";
  QString viewType="mail";
  QString name;
  MboxList & subfoldersList = client->subfolders();
  
  QStringList foldersList;
  KolabSharedFolder::ref()->subfolders(foldersList, pCommand->account, pCommand->mailbox);
  
  QStringList imapList;
  for(MboxListIterator it(subfoldersList); it.current(); ++it)
  {
    imapList.append(it.current()->name);
  }
  imapList.sort();
  
  Mbox * currentMbox = 0;
  for(QStringList::Iterator itList=imapList.begin(); itList!=imapList.end(); ++itList)
  {
    name = (*itList);
    currentMbox = subfoldersList[name];
    
    if( currentMbox )
    {
      if( (currentMbox->state & IMAP4Client::NoSelect) == IMAP4Client::NoSelect )
        viewType="genmail";
      else
        viewType="mail";
      
#ifdef IMAP4_DEBUG
      qDebug("Try: %s", name.latin1());
#endif
      if( -1 == foldersList.findIndex(name) )
      {
#ifdef IMAP4_DEBUG
        qDebug("Create: %s", name.latin1());
#endif
        
        IOSyncHandler::ref()->createFolder(pCommand->account, name, type, viewType);
      }
      else
      {
        foldersList.remove(name);
      }
    }
    else
    {
#ifdef IMAP4_DEBUG
      qDebug("######### BAD ********");
#endif
      return;
    }
  }
  
  // remove the folders which doesn't exists on the server
  for(QStringList::Iterator it1=foldersList.fromLast(); it1!=foldersList.end(); --it1)
  {
    if( "inbox" != (*it1).lower() )
    {
#ifdef IMAP4_DEBUG
      qDebug("Delete: %s", (*it1).latin1());
#endif
      if( viewType == "genmail" )
        IOSyncHandler::ref()->deleteFolder(pCommand->account, *it1, type, "default");
      else
        IOSyncHandler::ref()->deleteFolder(pCommand->account, *it1, type, QString::null);
    }
  }
}

void IMAP4Handler::slotCreateFolder(IMAP4Client * client)
{
#ifdef IMAP4_DEBUG
  qDebug("imap4handler: create folder ...");
#endif

  Command * pCommand = client->currentCommand();
  
  if( !pCommand )
  {
    qDebug("IMAP4Handler::slotCreateFolder()");
    return;
  }
  
  // create the Aethera folder
  IOSyncHandler::ref()->createFolder(pCommand->account, pCommand->mailbox, "imap", pCommand->other);
}

void IMAP4Handler::slotDeleteFolder(IMAP4Client * client)
{
#ifdef IMAP4_DEBUG
  qDebug("imap4handler: delete folder ...");
#endif

  Command * pCommand = client->currentCommand();
  
  if( !pCommand )
  {
    qDebug("IMAP4Handler::slotCreateFolder()");
    return;
  }
  
  qDebug("***** Delete: %s", pCommand->other.latin1());
  
  IOSyncHandler::ref()->deleteFolder(pCommand->account, pCommand->mailbox, "imap", pCommand->other);
}

void IMAP4Handler::slotRenameFolder(IMAP4Client * client)
{
#ifdef IMAP4_DEBUG
  qDebug("imap4handler: rename folder ...");
#endif
  
  Command * pCommand = client->currentCommand();
  
  if( !pCommand )
  {
    qDebug("IMAP4Handler::slotRenameFolder()");
    return;
  }
  
  IOSyncHandler::ref()->renameFolder(pCommand->account, pCommand->mailbox, pCommand->parameters, pCommand->other);
}

void IMAP4Handler::slotAddMessage(IMAP4Client * client)
{
#ifdef IMAP4_DEBUG
  qDebug("imap4handler: add message ...");
#endif

  // the source object can be deleted now, if it should
  setImapFolder( client );
  IOSyncHandler::ref()->checkToDo(client->currentCommand()->parameters);
}

void IMAP4Handler::slotDeleteMessage(IMAP4Client * client)
{
#ifdef IMAP4_DEBUG
  qDebug("imap4handler: delete message ...");
#endif
  
  setImapFolder( client );
  IOSyncHandler::ref()->deleteMessage(client->currentCommand()->other);
}

void IMAP4Handler::slotDeleteTrashMessages(IMAP4Client * client)
{
#ifdef IMAP4_DEBUG
  qDebug("imap4handler: delete local trash messages ...");
#endif
  
  setImapFolder( client );
  
  IOSyncHandler::ref()->deleteTrashMessages();
}

void IMAP4Handler::remove(IMAP4Client * client)
{
  clientMap_.remove(client->account()->accname);
  delete client;
  client = 0L;
}

void IMAP4Handler::removeAllStopped()
{
  for(QPtrListIterator<IMAP4Client> it(stoped_); it.current(); ++it)
  {
      remove(it.current());
  }
  
  stoped_.clear();
}

bool IMAP4Handler::expunge(const QString & url)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4Handler::expunge() %s", url.latin1());
#endif

  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(ObjectRequestBroker::thisInstance()->mailfolderReference(url));
  if( !folder )
  {
    qDebug("imap4handler: expunge: wrong folder url");
    return false;
  }

  // Set the command buffer.
  if( !insertCommand( IMAP4Client::EXPUNGE,
                      folder->account(),
                      folder->mailbox(),
                      QString::null,
                      url ) )
    return false;

  return true;
}

bool IMAP4Handler::rename(const QString & url, const QString & newName)
{
#ifdef IMAP4_DEBUG
  qDebug("Rename %s with %s", url.latin1(), newName.latin1());
#endif

  RemoteMailFolder *folder =
    dynamic_cast<RemoteMailFolder*>(ObjectRequestBroker::thisInstance()->mailfolderReference(url));
  if( !folder )
  {
    qDebug("imap4handler: rename: wrong folder url");
    return false;
  }

  // Set the command buffer.
  if( !insertCommand( IMAP4Client::RENAME_FOLDER_JOB,
                      folder->account(),
                      folder->mailbox(),
                      newName,
                      url ) )
    return false;

  return true;
}

bool IMAP4Handler::isKolab(const QString& account, const QString& mailbox) const
{
  return KolabSharedFolder::ref()->isKolabFolder(account, mailbox);
}

bool IMAP4Handler::isKolabSharedFolder(const QString& account, const QString& mailbox) const
{
  return KolabSharedFolder::ref()->isKolabSharedFolder(account, mailbox);
}
