/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
          imap4handler.h  -  the handler for the IMAPv4 protocol
                             -------------------
    begin                : Wed Feb  8 14:02:00 EET 2001
    copyright            : (C) 2001 by theKompany <www.thekompany.com>
                           (C) 2004 by Atol C&D   <www.atolcd.com>
    author               : Eugen Constantinescu
                           Fabien Armand
    email                : eug@thekompany.com
                           far@atolcd.com
 ***************************************************************************/

#ifndef IMAP4HANDLER_H
#define IMAP4HANDLER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qobject.h>
#include <qptrlist.h>
#include <qdict.h>

#include <remotemailfolder.h>
#include <imap4client.h>
#include <indexclass.h>
#include <accounts.h>

class IMAP4Handler : public QObject
{
  Q_OBJECT
  
  public:
    /** Return a reference to the IMAP handler class.*/
    static IMAP4Handler *ref();
    /** The remote folder reference.*/
    RemoteMailFolder* imapFolder();
    
    virtual ~IMAP4Handler();
    
    // public methods
    /** Stop the messages download for account.*/
    virtual bool stopDelivery(Account *);
    
    /** Stop the messages download for all the accounts.*/
    virtual bool stopDelivery();
    
    /**
      * Mount the IMAP folder.
      * IMAP folders needs a mount action for starting the connection
      * and an unmount action for closing the connection.
      * In this way the connection will be open only when you are seeing
      * the IMAP folder.
      * A mount acction will read the folder content too.
      */
    bool mount(const QString & url, bool bSync=true);
    /** Unmount an IMAP folder, see mount.*/
    bool unmount(const QString & url);
    /** It searches for the subfolders.*/
    bool subfolders(const QString & account, const QString & mailbox, const QString & other);
    /** Expunge.*/
    bool expunge(const QString & url);
    /** Rename.*/
    bool rename(const QString & url, const QString & newName);
    /** Sync.*/
    bool sync(const QString & url);
    /** Refresh.*/
    bool refresh(const QString & url);
    /** Login. It doesn't perform select.*/
    bool login(const QString & account);
    /** Logout.*/
    bool logout(const QString & account);
    /** Update the message contents.*/
    bool updateMessage(IndexClass &index);
    /** Add a message to the mailbox.*/
    bool addMessage(const QString & url, IndexClass *index);
    /** Check for the new mails.*/
    bool getMail(QString & accountName, QString & mailbox);
    /** Delete a message from the mailbox.*/
    bool deleteMessage(IndexClass &index);
    /** Create a new mailbox on the IMAP server.*/
    bool createFolder(const QString & account, const QString & mailbox, const QString & url);
    /** Delete a mailbox from the IMAP server.*/
    bool deleteFolder(const QString & account, const QString & mailbox, const QString & url);
    
    /** Run a new Kolab job for account with state.*/
    bool runKolabJob(const QString & account, const QString & mailbox, const QString & state);
    bool runKolabSharedJob(const QString & account, const QString & mailbox, const QString & state);
    
    /** Detect Kolab folders.*/
    bool isKolab(const QString& account, const QString& mailbox) const;
    bool isKolabSharedFolder(const QString& account, const QString& mailbox) const;

  public slots:
    void slotSubfolders(IMAP4Client *);
    void slotCreateFolder(IMAP4Client *);
    void slotDeleteFolder(IMAP4Client *);
    void slotRenameFolder(IMAP4Client *);
    void slotDownload(IMAP4Client *);
    void slotProgress(IMAP4Client *);
    void slotAddMessage(IMAP4Client *);
    void slotDeleteMessage(IMAP4Client *);
    void slotDeleteTrashMessages(IMAP4Client *);
    void slotUpdateMessage(IMAP4Client *);
    void slotLogout(IMAP4Client *);
    void slotClientFinished(IMAP4Client *);

  protected:
    void setImapFolder( IMAP4Client *);
    bool insertCommand( IMAP4Client::Commands type = IMAP4Client::NO_COMMAND,
                        const QString & account = QString::null, 
                        const QString & mailbox = QString::null, 
                        const QString & parameters = QString::null,
                        const QString & other = QString::null);
  
  private:
    IMAP4Handler();
    
    void remove(IMAP4Client*);
    void removeAllStopped();
    
    // Attributes
    static IMAP4Handler * thisInstance;
    
    RemoteMailFolder * imapFolder_;
    QDict<IMAP4Client> clientMap_;
    QPtrList<IMAP4Client> stoped_;
};

#endif
















