/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          imap4syncjob.cpp  -  description
                             -------------------
    begin                : Tue Jun 18 2002
    copyright            : (C) 2002 by theKompany.com
                           (C) 2004 by Atol C&D 
    authors              : Eugen C.      <eug@thekompany.com>
                           Fabien Armand <f.armand@atolcd.com>
 ***************************************************************************/

#include "imap4syncjob.h"

#include <imap4client.h>
#include <imap4handler.h>
#include <commandsprocessor.h>
#include <appstatus.h>
#include <mailinfo.h>
#include <messagedevice.h>
#include <remotemailfolder.h>
#include <orb.h>

#include <qapplication.h>
#include <qsettings.h>

#include <notificationpopup.h>
#include <qlabel.h>

#include <settings.h>

#include <accounts.h>

IMAP4SyncJob::IMAP4SyncJob
(
  QSocket           * sock,
  IMAP4Client       * parent
)
  : IMAP4Job(sock, parent),
    state_( START ),
    start_(true),
    fullMessage_(false),
    validityCheck_(true),
    refresh_(false),
    uMessages(0),
    folder (0)
{
  // Empty.
}

IMAP4SyncJob::~IMAP4SyncJob()
{
  // Empty.
}

  void
IMAP4SyncJob::setFullMessage(bool fullMessage)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4SyncJob::setFullMessage( %s )", fullMessage ? "true" : "false");
#endif

  if(state_ == START)
  {
    fullMessage_ = fullMessage;
  }
  else
  {
    qDebug("IMAP4SyncJob::setFullMessage : wrong state");
  }
}

  void
IMAP4SyncJob::setRefresh(bool refresh)
{
  if(state_ == START)
  {
    refresh_ = refresh;
  }
  else
  {
    qDebug("IMAP4SyncJob::setRefresh : wrong state");
  }
}

  void
IMAP4SyncJob::run()
{
  folder = (RemoteMailFolder*)ObjectRequestBroker::thisInstance()->mailfolderReference( command()->other );

  if( !folder )
  {
    emit finished(false);
    return;
  }
  
  if( "refresh" == command()->parameters )
  {
    setRefresh(true);
  }

  client()->setSingleLine( true );
  
#ifdef IMAP4_DEBUG
  client()->showStatus();
  folder->showStatus();
#endif

  if( refresh_ )
  {
    // remove all the messages from the local folder
    for(UIDMap::iterator it=folder->uidJar->rcvList.begin();
        it!=folder->uidJar->rcvList.end(); ++it)
    {
      folder->uidJar->trashList.insert((*it));
    }
    IMAP4Handler::ref()->slotDeleteTrashMessages( client() );
    
    // load again all the messages
    state_ = WaitingForAllMails;
    client()->sendUid( QString("SEARCH NOT DELETED") );
  }
  else
  {
    if( client()->uidValidity() == folder->uidValidity() &&
        client()->messagesNumber() == folder->messages() &&
        client()->messagesNumber() == folder->getMessageCount() &&
        client()->uidNext() == folder->uidNext() )
    {
      IMAP4Handler::ref()->slotProgress( client() );
      emit finished(true);
      return;
    }
  
    start_ = true;
    
    if( 0 == folder->uidNext() &&
        0 == folder->uidValidity() &&
        0 == folder->messages() )
    {
      /*
        We need all the messages because it is first time
        when this folder is accessed.
      */
      state_ = WaitingForAllMails;
      client()->sendUid( QString("SEARCH NOT DELETED") );
    }
    else
    {
      if( 0 == client()->uidValidity() )
      {
        validityCheck_ = false;
      }
      else
      {
        validityCheck_ = true;
      }
      
      // Check for the new messages.
      // UID SEARCH NEW
      // TODO: this has to be changed
      state_ = CheckingAllMails;
  //    client()->sendFetch("1:* (UID)"); // better for Citadel
      client()->sendUid( QString("SEARCH NOT DELETED") );
    }
  }
}

  bool
IMAP4SyncJob::processLine(const QString & line)
{
#ifdef IMAP4_DEBUG
  qDebug("IMAP4SyncJob::processLine : %s", line.latin1());
#endif

  QString firstWord;
  
  client()->parseNextWord(firstWord, line);
  if( firstWord == client()->getTag() )
  {
    if( client()->replyOK(line) )
    {
      UIDMailList &uidDict = client()->searchUID();
      
      if( start_ )
      {
        uMessages = uidDict.count();
        status->message(AppStatus::Server, AppStatus::Progress, AppStatus::Waiting,
          tr("Downloading..."), 0, uMessages);
      }
        
//#ifdef IMAP4_DEBUG
//      QStringList temp;
//      for(UIDIterator it(uidDict); it.current(); ++it)
//      {
//        temp += it.currentKey();
//      }
//      qDebug("UID list: %s", temp.join(",").latin1());
//#endif
      
      switch( state_ )
      {
        case CheckingForDeletedMails :
          // remove from local all the mails which aren't on the server
          if( start_ )
          {
            // check what should be deleted from local
            for(UIDMap::iterator it=folder->uidJar->rcvList.begin();
                it!=folder->uidJar->rcvList.end(); ++it)
            {
              QString uidNumber( (*it).first.c_str() );
              uidNumber = uidNumber.left( (unsigned)uidNumber.find("@") );
                
              if( !uidDict[uidNumber] )
                folder->uidJar->trashList.insert((*it));
              else
                uidDict.remove(uidNumber);
            }
              
            IMAP4Handler::ref()->slotDeleteTrashMessages( client() );
            
            start_ = false;
          }
          
          //Set the status
          folder->setStatus(client()->uidValidity(), client()->uidNext(),
                            client()->messagesNumber(), client()->unseen(),
                            client()->recent());
          state_ = DONE;
          emit finished(true);
          return true;
          break;
          
        case WaitingForAllMails :
#ifdef IMAP4_DEBUG
          qDebug("WaitingForAllMails : %u", uidDict.count());
#endif
          
          if( start_ )
          {
            start_ = false;
          }
          
          if( uidDict.isEmpty() )
          {
            //Set the status
            folder->setStatus(client()->uidValidity(), client()->uidNext(),
                              client()->messagesNumber(), client()->unseen(),
                              client()->recent());
            state_ = DONE;
            IMAP4Handler::ref()->slotProgress( client() );
            emit finished(true);
            return true;
          }
          else
          {
            status->message(AppStatus::Server, AppStatus::Progress, AppStatus::Waiting,
              tr("Downloading..."), uMessages - uidDict.count(), uMessages);
              
            int i=0;
            QStringList uids;
            
            {
              for(UIDIterator it(uidDict); it.current(); ++it)
              {
                if( i>=25 )
                  break;
                else
                {
                  uids += it.currentKey();
                }
                
                i++;
              }
            }
          
            uids.sort();
            {
              for(QStringList::Iterator it=uids.begin(); it!=uids.end(); ++it)
              {
                uidDict.remove( *it );
              }
            }
          
            // UID FETCH it (FLAGS INTERNALDATE UID BODY[HEADER])
            client()->setSingleLine( true );
            if( fullMessage_ )
            {
              client()->sendUid( QString("FETCH ") + uids.join(",") +
                                 " (FLAGS INTERNALDATE UID BODY[])" );
            }
            else
            {
              client()->sendUid( QString("FETCH ") + uids.join(",") +
                                 " (FLAGS INTERNALDATE UID BODY[HEADER])" );
            }
          }
          break;
          
        case CheckingAllMails :
#ifdef IMAP4_DEBUG
          qDebug("CheckingAllMails : %u", uidDict.count());
#endif
          
          if( start_ )
          {
            if( validityCheck_ && client()->uidValidity() != folder->uidValidity() )
            {
              // remove everything since uidvalidity is changed
              for(UIDMap::iterator it=folder->uidJar->rcvList.begin();
                  it!=folder->uidJar->rcvList.end(); ++it)
              {
                folder->uidJar->trashList.insert((*it));
              }
            }
            else
            {
              // check what should be deleted from local
              for(UIDMap::iterator it=folder->uidJar->rcvList.begin();
                  it!=folder->uidJar->rcvList.end(); ++it)
              {
                QString uidNumber( (*it).first.c_str() );
                uidNumber = uidNumber.left( (unsigned)uidNumber.find("@") );
                
                if( !uidDict[uidNumber] )
                  folder->uidJar->trashList.insert((*it));
                else
                  uidDict.remove(uidNumber);
              }
            }
            
            IMAP4Handler::ref()->slotDeleteTrashMessages( client() );
            
            start_ = false;
          }
          
          // download the new mails (unseen or seen)
          if( uidDict.isEmpty() )
          {
            //Set the status
            folder->setStatus(client()->uidValidity(), client()->uidNext(),
                              client()->messagesNumber(), client()->unseen(),
                              client()->recent());
            state_ = DONE;
            IMAP4Handler::ref()->slotProgress( client() );
            emit finished(true);
            return true;
          }
          else
          {
            status->message(AppStatus::Server, AppStatus::Progress, AppStatus::Waiting,
              tr("Downloading..."), uMessages - uidDict.count(), uMessages);


            bool notificationPopup =
              Settings::instance()->readBoolEntry
              (Settings::instance()->name() + "/Misc Mail" + QString("/NotificationPopup"));

            bool notificationSound =
              Settings::instance()->readBoolEntry
              (Settings::instance()->name() + "/Misc Mail" + QString("/NotificationSound"));
              

            // if the mailbox is the client default mailbox, for example : Inbox
              
            if(client()->mailbox()==client()->account()->mbox)
            {

              // system sound notifies new mails

              if(notificationSound)
                QApplication::beep();

              if(notificationPopup)
              {
                // popup for notification of new mails

                NotificationPopup * notification = new NotificationPopup();

                // set popup text

                QCString t;

                if (1 == uidDict.count())
                {
                  t.sprintf("You have %d new message",uidDict.count());
                }
                else
                {
                  t.sprintf("You have %d new messages",uidDict.count());
                }

                notification->NotificationPopupText->setText(t);

                // show the popup

                notification->show();
              }
            }    
                                                
            int i=0;
            QStringList uids;
            {
              for(UIDIterator it(uidDict); it.current(); ++it)
              {
                if( i>=25 )
                  break;
                else
                {
                  uids += it.currentKey();
                }
                
                i++;
              }
            }
            
            uids.sort();
            {
              for(QStringList::Iterator it=uids.begin(); it!=uids.end(); ++it)
              {
                uidDict.remove( *it );
              }
            }
          
            // UID FETCH it (FLAGS INTERNALDATE UID BODY[HEADER])
            client()->setSingleLine( true );
            if( fullMessage_ )
            {
              client()->sendUid( QString("FETCH ") + uids.join(",") +
                                 " (FLAGS INTERNALDATE UID BODY[])" );
            }
            else
            {
              client()->sendUid( QString("FETCH ") + uids.join(",") +
                                 " (FLAGS INTERNALDATE UID BODY[HEADER])" );
            }
          }
          break;
          
        default:
          state_ = DONE;
          qDebug("IMAP4SyncJob::processLine() : wrong state");
          emit finished(false);
          return false;
          break;
      }
      
      return true;
    }
    else
    {
      qDebug("IMAP4SyncJob::processLine() : bad answer : %s", line.latin1());
      
      emit finished(false);
      
      return false;
    }
  }
  else
    return client()->parseLine(line);
    
  emit finished(false);
  return false;
}

  bool
IMAP4SyncJob::processMultiLine(const QByteArray & block)
{
  return client()->handleFetchData(block);
}

