/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

/***************************************************************************
                    iosynchandler.cpp  -  description
                             -------------------
    copyright            : (C) 2002-2005 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Eugen Constantinescu <eug@thekompany.com>
                           Fabien Armand <far@atolcd.com>
 ***************************************************************************/

#include <pop3handler.h>
#include <imap4handler.h>
#include <iosynchandler.h>
#include <mailfolder.h>
#include <indexclass.h>
#include <orb.h>
#include <string>
#include <qsettings.h>
#include <qtextcodec.h>
#include <messagedescriptor.h>
#include <messagedevice.h>
#include <dateclass.h>
#include <mimecodec.h>
#include <vfolder.h>
#include <accounts.h>

//tkcBase includes
#include <settings.h>

//Aethera includes
#include <komrmiservice.h>

// Isn't it a bit crazy to link with a plugin ?
// It isn't a plugin right now, but it should be.
#include <mailfilter.h>

//MailImportService includes
#include <mailimportservice.h>

// support for kolab shared folders
#include <kolabsharedfolder.h>

IOSyncHandler * IOSyncHandler::thisInstance = 0;

WaitingOperations::WaitingOperations(const QString & folderName, ToDo action, QStringList param)
  : folderURL(folderName),
    whatToDo(action),
    parameters(param)
{
}

WaitingOperations::~WaitingOperations()
{
}


IOSyncHandler::IOSyncHandler()
  : vfs(0),
    mailFilter(0)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = dynamic_cast<VFSModule*>(ref.instance());
    else
    {
      qDebug("\n NO VFS Provider !\n");
    }
    
    ref = trader->registeredComponent( AETHERA_MAILFILTER_SERVICE );
    if( ref.isValid() )
      mailFilter = dynamic_cast<MailFilterV*>(ref.instance());
    else
    {
      qDebug("\n NO MAILFILTER Provider !\n");
    }
  }
}
  
IOSyncHandler::~IOSyncHandler()
{
  toDo.setAutoDelete(true);
}

IOSyncHandler *IOSyncHandler::ref()
{
  if( !thisInstance )
  {
    thisInstance = new IOSyncHandler();
  }
  
  return thisInstance;
}

void IOSyncHandler::dispatchToInbox
(
  const QCString  & messageData,
  const QCString  & uidl,
  const QString   & accountName
)
{
  MessageClass message(messageData);

  Q_ASSERT(0 != mailFilter);

  // Find folder to create the message.
  QString mFolder = mailFilter->messageFolder(&message, messageData );

  MailFolder * folder = 0;
  if (!mFolder.isEmpty())
  {
    folder = ObjectRequestBroker::thisInstance()->mailfolderReference( mFolder );

    if (!foldersToSync.contains(mFolder))
      foldersToSync.append(mFolder);
  }

  if (0 == folder)
  {
    folder = ObjectRequestBroker::thisInstance()->mailfolderReference("sysmail_inbox");
  }

  if (0 == folder)
  {
    // TODO: Simply return false here, and don't send DELE to the server.
    qFatal
      (
        "A fatal error has occured. Inbox interface not found."
        " Message not saved (left on server)."
      );
  }
  else
  {
    QString messageId =
      folder->createMessage
      (
        messageData,
        uidl,
        QDateTime::currentDateTime(),
        accountName,
        &message,
        true
      );

    if( messageId.isEmpty() )
    {
      qWarning("IOSyncHandler::dispatchToInbox: null id");
      return;
    }
    
    //Check if the message has iCal data
    checkICal( messageData, uidl, accountName, message );
  }  
  
  // "copy" the message to other folders then do a folder sync()
  filterMessages( messageData, uidl, accountName, message );
}



void IOSyncHandler::dispatchToFolder
(
  MailFolder* folder,
  const QCString  & messageData,
  const QCString  & uidl,
  const QString   & accountName
)
{
  MessageClass message(messageData);


  if (0 == folder)
  {
    qFatal
      (
        "A fatal error has occured. Inbox interface not found."
        " Message not saved (left on server)."
      );
  }
  else
  {
    // We use sync signal - last param is true!
//    qDebug("IOSyncHandler::dispatchToInbox: uidl == `%s'", uidl.data());

    QString messageId =
      folder->createMessage
      (
        messageData,
        uidl,
        QDateTime::currentDateTime(),
        accountName,
        &message,
        true
      );

    if( messageId.isEmpty() )
    {
      qWarning("IOSyncHandler::dispatchToFolder: null id");
      return;
    }
  }
}


bool IOSyncHandler::importMessages( const QString& toFolder )
{
//  qDebug( "\nIOSyncHandler::importMessages( const QString& toFolder )\n" );
  
  MailImportService *mis = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILIMPORTSERVICE_SERVICE );
    if( ref.isValid() )
      mis = static_cast<MailImportService*>(ref.instance());
  }

  MailFolder * folder = 0;

  if ( !toFolder.isEmpty())
  {
    folder = ObjectRequestBroker::thisInstance()->mailfolderReference( toFolder );
  }
  else
  {
    return false;
  }

  if(!folder)
  {
    qDebug
      (
        "IOSyncHandler::dispatchToFolder: Folder for this message"
        " (as defined by filter) seems to not exist."
        " Trying to write to inbox instead."
      );

    folder =  ObjectRequestBroker::thisInstance()->mailfolderReference("sysmail_inbox");
  }
  if(!folder)
  {
    return false;
  }
    
  // Browse the records from 'first' while we have 'next'
  SAMport_Abstract record;
  record=mis->first();
  while( record.Num )
  {
    QByteArray data;
    QCString rfc822=record.FullHeader.c_str();
    rfc822+="\r\n";
    rfc822+=record.FullMsg.c_str();
    
    dispatchToFolder( folder, rfc822, QCString(record.Message_Id.c_str()), "default" );
               
    // go to the next record
    record=mis->next();
  }
  mis->close();
  
  folder->sync();
  
  return true;
}

void IOSyncHandler::dispatchMessage(const FetchInfo &msg, bool fullMessage, bool bSync)
{
  RemoteMailFolder *imapFolder = IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message discarded.");
  }
  else
  {
    QDateTime rcvTime;
    rcvTime.setTime_t(msg.rcvtime);
    
    if(fullMessage)
    {
      imapFolder->createFullMessage(msg.message, msg.uid.latin1(), rcvTime,
                                    imapFolder->account(), 0, bSync, msg.flags);
    }
    else
    {
      imapFolder->createMessage(msg.message, msg.uid.latin1(), rcvTime,
                                imapFolder->account(), 0, bSync, msg.flags);
    }
  }
}

void IOSyncHandler::updateMessage(const FetchInfo &msg, const QString & indexURL)
{
  RemoteMailFolder *imapFolder=IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message discarded.");
  }
  else
  {
    imapFolder->updateMessage(msg.message, indexURL);
    checkToDo(indexURL);
  }
}

void IOSyncHandler::deleteMessage(const QString & indexURL)
{
  RemoteMailFolder *imapFolder=IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message not deleted.");
  }
  else
  {
    imapFolder->deleteMessage(indexURL);
  }
}

void IOSyncHandler::deleteTrashMessages()
{
  RemoteMailFolder *imapFolder=IMAP4Handler::ref()->imapFolder();
  if(!imapFolder)
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message not deleted.");
  }
  else
  {
    imapFolder->deleteTrashMessages();
  }
}

void IOSyncHandler::pop3Sync()
{
  MailFolder *inbox=ObjectRequestBroker::thisInstance()->mailfolderReference("sysmail_inbox");
  
  if(!inbox)
  {
    printf("A fatal error has occured, inbox interface not found. Message discarded.");
  }
  else
    inbox->sync();
  
  // there is a problem here
  // ClientVFS should not be called into the server side
  for ( QStringList::Iterator it = foldersToSync.begin(); it != foldersToSync.end(); ++it ) {
    MailFolder *folder=ObjectRequestBroker::thisInstance()->mailfolderReference( *it );
    if ( folder ) {
      folder->sync();
    }
  }
  foldersToSync.clear();
}

void IOSyncHandler::imap4Sync()
{
  if(!IMAP4Handler::ref()->imapFolder())
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found. Message discarded.");
  }
  else
    IMAP4Handler::ref()->imapFolder()->sync();
}

void IOSyncHandler::kolabSync()
{
  if(!IMAP4Handler::ref()->imapFolder())
  {
    qDebug("A fatal error has occured, IMAP mailbox interface not found.");
  }
  else
  {
    QString accName = IMAP4Handler::ref()->imapFolder()->account();
    QString mailbox = IMAP4Handler::ref()->imapFolder()->mailbox();
    
    if( KolabSharedFolder::ref()->isKolabFolder(accName, mailbox) ||
        KolabSharedFolder::ref()->isKolabSharedFolder(accName, mailbox) )
    {
      int iToDo = 0;
      QString plugin = KolabSharedFolder::ref()->type(accName, mailbox);
      if( !plugin.isEmpty() )
      {
        if("calendar" == plugin)
        {
          plugin = "Datebook";
        }
        else if("todo" == plugin)
        {
          plugin = "Datebook";
          iToDo = 1;
        }
        else if("contacts" == plugin)
        {
          plugin = "Contacts";
        }
        else if("notes" == plugin)
        {
          plugin = "Notes";
        }
        else if("jabber" == plugin)
        {
          plugin = "Jabber";
        }
        
        KomRMIService* rmiService = 0;
        ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
        if( trader )
        {
          ComponentRef ref = 
            trader->registeredComponent( QString(AETHERA_RMI_SERVICE) + "/" + plugin );
          
          if( ref.isValid() )
            rmiService = static_cast<KomRMIService*>(ref.instance());
          else
          {
            qDebug("\n NO %s RMI service!", plugin.latin1());
            return;
          }
        }
        
//        qDebug("Kolab - call RMI service");
        
        QDomDocument callDoc;
        QDomElement call = callDoc.createElement( "call" );
        callDoc.appendChild( call );
          
        QDomElement name = callDoc.createElement("name");
        call.appendChild( name );
        QDomText methodName = callDoc.createTextNode( "kolab" );
        name.appendChild( methodName );
          
        QDomElement parameters = callDoc.createElement( "parameters" );
        parameters.setAttribute( "param1", accName );
        parameters.setAttribute( "param2", mailbox );
        parameters.setAttribute( "param3", iToDo );
        parameters.setAttribute( "count", 3 );
        parameters.setAttribute( "signature", "QString,QString,int" );
        call.appendChild( parameters );
          
        QDomElement returnType = callDoc.createElement("return");
        call.appendChild( returnType );
        QDomText returnTypeName = callDoc.createTextNode( "void" );
        returnType.appendChild( returnTypeName );
          
        rmiService->callMethod( callDoc );
      }
      else
      {
        qDebug("kolab synch without plugin");
      }
    }
  }
}

void IOSyncHandler::createFolder( const QString &parent, const QString &folderPath,
                                  const QString& type,   const QString &viewType )
{
  ObjectRequestBroker::thisInstance()->createFolder(parent, folderPath, type, viewType);
}

void IOSyncHandler::deleteFolder( const QString &parent, const QString &folderPath,
                                  const QString& type,   const QString &serverPath )
{
  ObjectRequestBroker::thisInstance()->deleteFolder(parent, folderPath, type, serverPath);
}

void IOSyncHandler::renameFolder( const QString &account, const QString &mailbox,
                                  const QString &newName, const QString &url )
{
  ObjectRequestBroker::thisInstance()->renameFolder(account, mailbox, newName, url);
}

void IOSyncHandler::slotMessageDelivered(IndexClass * idx)
{
 qDebug("IOSyncHandler::slotMessageDelivered");

 bool saveSent =
   Settings::instance()->readBoolEntry
   ("/Aethera/Send Options/Save sent");

 if (saveSent)
 {
   QCString currentDateAsString =
     DateClass(QDateTime::currentDateTime()).toQCString();

   // Replace info from the extended status.

   MessageDevice dev(idx);

   dev.loadDescriptor();

   MessageDescriptor & descriptor = dev.getDescriptor();

   descriptor.extendedStatus.clear();

   descriptor.extendedStatus.append("sent:" + currentDateAsString);
      descriptor.status   = "Sent";
   descriptor.sentDate = currentDateAsString;
      dev.saveDescriptor();

   // Move message to "Sent Mail", if needed.

   ObjectRequestBroker::thisInstance()
     ->move("sysmail_outbox/" + idx->getID(), "sysmail_sent");
 }
 else
 {
   // Delete message.
   ObjectRequestBroker::thisInstance()
     ->remove("sysmail_outbox/" + idx->getID());
 }
 
} 

void IOSyncHandler::insertToDo(QString indexURL, QString folderName, WaitingOperations::ToDo action, QStringList param)
{
  if( indexURL.isEmpty() || 
      folderName.isEmpty() ||
      WaitingOperations::Nothing == action )
    return;
  
  WaitingOperations * waitingToDo = toDo[indexURL];
  if( !waitingToDo )
  {
    waitingToDo = new WaitingOperations(folderName, action, param);
    
    toDo.insert(indexURL, waitingToDo);
  }
}

void IOSyncHandler::checkToDo(const QString & indexURL)
{
#ifdef IMAP4_DEBUG
  qDebug("IOSyncHandler::checkToDo(%s)", indexURL.latin1());
#endif

  if( indexURL.isEmpty() )
    return;
  
  WaitingOperations * waitingToDo = toDo[indexURL];
  if( waitingToDo )
  {
    // remove it here, but don't delete the reference too
    // setAutoDelete(false)
    toDo.remove(indexURL);
  
    IndexClass * index = ObjectRequestBroker::thisInstance()->indexReference(indexURL);
    
    if(index)
    {
      MailFolder * mailFolder =
        ObjectRequestBroker::thisInstance()->mailfolderReference(waitingToDo->folderURL);
      
      if(mailFolder)
      {
        switch( waitingToDo->whatToDo )
        {
          case WaitingOperations::CopyObject :
            mailFolder->copyMessage(index);
            break;
          case WaitingOperations::MoveObject :
            mailFolder->moveMessage(index);
            break;
          case WaitingOperations::DeleteObject :
            mailFolder->deleteMessage(index);
            break;
          case WaitingOperations::Nothing :
          default:
            break;
        }
      }
      else
      {
        qDebug("checkToDo : bad folder");
      }
      
    }
    else
    {
      qDebug("checkToDo : bad index");
    }
    
    delete waitingToDo;
    waitingToDo = 0;
  }
}


void IOSyncHandler::checkICal
  (
    const QCString  & messageData,
    const QCString  & uidl,
    const QString   & accountName,
    const MessageClass & message
  )
{
  QPtrList<MimePart> mimePartList = message.partList;
  MimePart* part = mimePartList.at( 0 );
  if( (part->type=="text" || part->type=="attachment") && 
      (part->mimetype=="text/plain" ||
       part->mimetype=="text/calendar") 
    )
  {
    // Extract the decoded text
    QCString partData = messageData.mid( part->offset, part->length );
    partData = MIMECodec::decode( partData, part->encoding );
    QString iCalData;
    
    QTextCodec * codec = QTextCodec::codecForName( part->charset );
    if(codec)
      iCalData += codec->toUnicode( partData );
    else
    {
      if( "utf-8" == part->charset.lower() )
        iCalData += QString::fromUtf8( partData.data() );
      else if( "8bit" == part->encoding.lower() )
        iCalData += QString::fromLocal8Bit( partData.data() );
      else // 7 bit
        iCalData += QString::fromLatin1( partData.data() );
    }
    
    if( !iCalData.isEmpty() )
    {
      //detect if the mail contains data for the KOPlugin
      int data_begin_pos = iCalData.find( "BEGIN:VCALENDAR" );
      int data_end_pos = iCalData.find( "END:VCALENDAR" );
        
      //if we found that this is a KOPlugin email
      if( data_begin_pos >-1 && data_end_pos>-1 )
      {
        //copy the KOPlugin sync message to the sysko_incoming
        MailFolder* koInFolder = 0L;
        koInFolder = ObjectRequestBroker::thisInstance()->mailfolderReference("sysko_incoming");
        if(!koInFolder)
        {
          qDebug("IOSyncHandler::checkICal : sysko_incoming not available");
          return;
        }
          
        QString newUUID =
          koInFolder->createMessage
          (
            messageData,
            uidl,
            QDateTime::currentDateTime(),
            accountName,
            &message,
            true
          );
            
        if( newUUID.isEmpty() )
        {
          qDebug( "Unable to create a new email in koplugin sync folder." );
          return;
        }
          
        //make the KOPlugin take into account the new messages that we received  
        KomRMIService* rmiService = 0L;
        ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
        if( trader )
        {
          ComponentRef ref = trader->registeredComponent( AETHERA_RMI_SERVICE"/Datebook" );
          if( ref.isValid() )
            rmiService = static_cast<KomRMIService*>(ref.instance());
          else
          {
            qDebug("\n NO datebook RMI service!");
            return;
          }
        }
          
        QDomDocument callDoc;
        QDomElement call = callDoc.createElement( "call" );
        callDoc.appendChild( call );
          
        QDomElement name = callDoc.createElement("name");
        call.appendChild( name );
        QDomText methodName = callDoc.createTextNode( "retrieve" );
        name.appendChild( methodName );
          
        QDomElement parameters = callDoc.createElement( "parameters" );
        call.appendChild( parameters );
        parameters.setAttribute( "count", 0 );
          
        QDomElement returnType = callDoc.createElement("return");
        call.appendChild( returnType );
        QDomText returnTypeName = callDoc.createTextNode( "void" );
        returnType.appendChild( returnTypeName );
          
        rmiService->callMethod( callDoc );
      }
    }
  }
}

void IOSyncHandler::filterMessages
  (
    const QCString  & messageData,
    const QCString  & uidl,
    const QString   & accountName,
    const MessageClass & message
  )
{
  //copy the message to other folders
  QStringList copyFolders = mailFilter->foldersToCopy( &message, messageData );
  for( QStringList::Iterator it = copyFolders.begin(); it!=copyFolders.end(); ++it )
  {
    MailFolder* folderIt = 0L;
    folderIt = ObjectRequestBroker::thisInstance()->mailfolderReference( *it );

    if ( !foldersToSync.contains(*it) )
      foldersToSync.append( *it );

    if (!folderIt)
      folderIt =  ObjectRequestBroker::thisInstance()->mailfolderReference("sysmail_inbox");
  
    if (!folderIt)
    {
      // TODO: Simply return false here, and don't send DELE to the server.
      qFatal
        (
          "A fatal error has occured. Inbox interface not found."
          " Message not saved (left on server)."
        );
    }
    else
    {
      QString newUUID =
        folderIt->createMessage
        (
          messageData,
          uidl,
          QDateTime::currentDateTime(),
          accountName,
          &message,
          true
        );
  
      if( newUUID.isEmpty() )
      {
        qWarning("IOSyncHandler::filterMessages: null id");
      }
    }           
  }                                              
}





