/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                        kolabjob.cpp  -  description
                             -------------------
    begin                : Wed Dec 8 14:21:39 2004
    copyright            : (C) 2004 by theKompany.com
    email                : Eugen Constantinescu <eug@thekompany.com>
 ***************************************************************************/

#include "kolabjob.h"

#include <imap4client.h>
#include <imap4handler.h>
#include <commandsprocessor.h>
#include <mailinfo.h>
#include <messagedevice.h>

#include <appstatus.h>
#include <kolabsharedfolder.h>

KolabJob::KolabJob
(
  QSocket           * sock,
  IMAP4Client       * parent
)
  : IMAP4Job(sock, parent)
{
  // Empty.
}

KolabJob::~KolabJob()
{
  // Empty.
}

  void
KolabJob::run()
{
  QString state = command()->other;
  QString jobType = command()->parameters;

#ifdef IMAP4_DEBUG
  qDebug("KolabJob::run : %s, %s", state.latin1(), jobType.latin1());
#endif
  
  if( !state.isEmpty() && 
      !jobType.isEmpty() && 
      ("KOLAB" == jobType || "SHARED" == jobType) )
  {
    if( "KOLAB" == jobType )
    {
      if( "subscribe" == state )
      {
        KolabSharedFolder::ref()->subscribeFolders( command()->account );
        IMAP4Handler::ref()->runKolabJob(command()->account, command()->mailbox, "imap_folders");
      }
      else if( "imap_folders" == state )
      {
        KolabSharedFolder::ref()->checkIMAPFolders( command()->account );
        IMAP4Handler::ref()->runKolabJob(command()->account, command()->mailbox, "vfs_folders");
      }
      else if( "vfs_folders" == state )
      {
        KolabSharedFolder::ref()->checkVFSFolders( command()->account );
      }
      else
      {
        qDebug("KolabJob: Invalid State");
        emit finished(false);
        return;
      }
    }
    else
    {
      if( "subscribe" == state )
      {
        KolabSharedFolder::ref()->subscribeFolders( command()->account );
        IMAP4Handler::ref()->runKolabSharedJob(command()->account, command()->mailbox, "synch");
      }
      else if( "synch" == state )
      {
        KolabSharedFolder::ref()->synchSharedFolders( command()->account );
        IMAP4Handler::ref()->runKolabSharedJob(command()->account, command()->mailbox, "scann");
      }
      else if( "scann" == state )
      {
        KolabSharedFolder::ref()->createSharedFolders( command()->account );
      }
      else
      {
        qDebug("KolabJob: Invalid State");
        emit finished(false);
        return;
      }
    }
  }
  else
  {
    qDebug("KolabJob: Invalid State");
    emit finished(false);
    return;
  }
  
  emit finished(true);
}

  bool
KolabJob::processLine(const QString & line)
{
  qDebug("KolabJob::processLine() : not available");
  return false;
}

  bool
KolabJob::processMultiLine(const QByteArray &)
{
  qDebug("KolabJob::processMultiLine() : not available");
  return false;
}

