/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include "accountmanager.h"
#include "pop3handler.h"
#include "pop3client.h"

#include <qdialog.h>

#include <passwddialog.h>


POP3Handler * POP3Handler::instance_ = 0L;

  POP3Handler *
POP3Handler::ref()
{
  if (0 == instance_)
    instance_ = new POP3Handler;

  return instance_;
}

POP3Handler::POP3Handler(QObject * parent, const char * name)
  : QObject(parent, name)
{
}

POP3Handler::~POP3Handler()
{
  // Don't delete the clients - our QObject parent took ownership on
  // instantiation.
}

  void 
POP3Handler::getMail(Account * account)
{
  removeAllStopped();
  
  if (0 == account)
  {
    qDebug("POP3Handler::getMail(default account)");

    account = AccountManager::ref()->getDefaultAccount();

    if (0 == account)
    {
      qDebug("POP3Handler::getMail: No default account");
      return;
    }
  }
  else
  {
    qDebug("POP3Handler::getMail(%s)", account->accname.latin1());
  }

  if (Account::POP3 != account->type)
  {
    qWarning("POP3Handler::getMail: This is not a POP3 account");
    return;
  }

  POP3Client * client = clientMap_[account->accname];
  
  if (0 == client)
  {
    qDebug("No existing POP3Client for account `%s'", account->accname.latin1());

    if( account->password.isEmpty() )
    {
      PasswdDialog passwdDialog(account->accname + " (pop3)");
      if( QDialog::Accepted == passwdDialog.exec() )
      {
        account->password = passwdDialog.getPassword();
      }
      else
      {
        qDebug("No password, cancel was pressed.");
        return;
      }
    }
  
    client =
      new POP3Client
      (
        account,
        this,
        ("POP3Client for " + account->accname).latin1()
      );

    clientMap_.insert(account->accname, client);
    
    connect(client, SIGNAL(finished(POP3Client*)), this, SLOT(slotClientFinished(POP3Client*)));
  }
  else
  {
    qDebug("Already have POP3Client for account `%s'", account->accname.latin1());
  }

  client->getMail();
}

  bool
POP3Handler::stopDelivery(Account * account)
{
  removeAllStopped();
  
  POP3Client * client = clientMap_[account->accname];

  if (0 == client)
  {
    qDebug
      (
        "stopDelivery for `%s': no delivery in progress",
        account->accname.latin1()
      );
  }
  else
  {
    client->stop();
  }

  return true;
}

  bool
POP3Handler::stopAllDeliveries()
{
  removeAllStopped();
  
  for(QDictIterator<POP3Client> it(clientMap_); it.current(); ++it)
  {
    it.current()->stop();
  }
  
  return true;
}

  void
POP3Handler::slotClientFinished(POP3Client * client)
{
  if(!client)
    return;
  
  qDebug("POP3Handler::slotClientFinished(%s)", client->account()->accname.latin1());

  stoped_.append(client);
}

void POP3Handler::remove(POP3Client * client)
{
  clientMap_.remove(client->account()->accname);
  delete client;
  client = 0;
}

void POP3Handler::removeAllStopped()
{
  for(QPtrListIterator<POP3Client> it(stoped_); it.current(); ++it)
  {
      remove(it.current());
  }
  
  stoped_.clear();
}
