/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-1999 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-1999 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <blockdev.h>
#include <qfile.h>
#include <qdatastream.h>
#include <transactionserver.h>

//#define DEBUG_BLOCKDEV

BlockDevice::BlockDevice(QString filename, QString prefix)
  : DataCollection(prefix)
{
#ifdef DEBUG_BLOCKDEV
	printf("BlockDevice: entering the constructor...\n");
#endif
  storageDevice=filename;
  type=FileStorage;
  buildEntryList();
}

BlockDevice::~BlockDevice()
{
}

void BlockDevice::setData(ObjectReference * object, const QByteArray & info)
{
//  qDebug("BlockDevice::setData()");
  
  QFile dfile(storageDevice);
    
  if( !object->isNewObject )
  {
    QByteArray first;
    QByteArray end;
    
    if(dfile.open(IO_ReadOnly))
    {
      if( object->offset )
      {
        first.resize(object->offset);
        dfile.readBlock(first.data(), object->offset);
      }
      
      unsigned long uEnd = object->offset + object->size;
      if( uEnd < dfile.size() )
      {
        dfile.at(uEnd);
        
        end.resize(dfile.size() - uEnd);
        dfile.readBlock(end.data(), dfile.size() - uEnd);
      }
      
      dfile.close();
    }
    else
    {
      qDebug("BlockDevice %s: cannot read data", storageDevice.latin1());
      return;
    }
      
    if(dfile.open(IO_WriteOnly))
    {
      if( first.size() )
        dfile.writeBlock(first.data(), first.size());
        
      if( end.size() )
        dfile.writeBlock(end.data(), end.size());
      
      dfile.close();
    }
    else
    {
      qDebug("BlockDevice %s: cannot write data", storageDevice.latin1());
      return;
    }
    
    // update the objects offset
    for(QPtrListIterator<ObjectReference> it(objectsList); it.current(); ++it)
    {
      if( it.current()->offset > object->offset )
        it.current()->offset -= object->size;
    }
  }
  
  if(dfile.open(IO_ReadWrite | IO_Append))
  {
    QDataStream stream(&dfile);
    
    object->offset = dfile.at();
    
    // write the object id
    stream << object->id;
      
    // write the object data
    stream << info;
    
    // set the object size
    object->size = dfile.at() - object->offset;
      
    dfile.close();
    
    object->isNewObject = false;
  }
  else
  {
    qDebug("BlockDevice %s: cannot write data", storageDevice.latin1());
    return;
  }
}

void BlockDevice::data(ObjectReference * object, QByteArray & info)
{
//  qDebug("BlockDevice::data()");
  
  if( object->size )
  {
    QFile dfile(storageDevice);
    
    if(dfile.open(IO_ReadOnly))
    {
      QDataStream stream(&dfile);
      QString id;
      
      // go to offset
      dfile.at(object->offset);
      
      // read the object id
      stream >> id;
      
      // test the id
      
      // read the object data
      stream >> info;
      
      dfile.close();
    }
    else
      qDebug("BlockDevice %s: cannot read data", storageDevice.latin1());
  }
  else
  {
    qDebug("Object empty or new!");
    info = QByteArray();
  }
}

void BlockDevice::buildEntryList()
{
#ifdef DEBUG_BLOCKDEV
	printf("BlockDevice: Building entry list...\n");
#endif
  QByteArray buffer;
  QFile dfile(storageDevice);
  
  if(dfile.open(IO_ReadOnly))
  {
    QDataStream stream(&dfile);
    while(!stream.atEnd())
    {
      // create a new object
      int offset = dfile.at();
      ObjectReference *tobj = new ObjectReference(this);
      QString id;
      
      // read the object id
      stream>>id;
      
//      qDebug("Contact ID = %s", id.latin1());
      
      // test the id
      if(!isValidID(id) || objectDict()[id])
        id=nextID(); // not valid, reassign id
        
      // read the object data
      stream >> buffer;
      tobj->size = dfile.at() - offset;
      tobj->offset = offset;
      tobj->id = id;
      tobj->isNewObject = false;
      
      addEntry(tobj);
      // set lastID accordingly
      unsigned long idNumber=id.mid(idPrefix.length()).toULong();
      if(idNumber>lastIDNum)
       lastIDNum=idNumber;
    }
  }
}

bool BlockDevice::removeEntry(QString id)
{
  ObjectReference * object = ref(id);
  if( object &&
      objectsDict.remove(object->id) &&
      objectsList.removeRef(object) )
  {
    QFile dfile(storageDevice);
      
    if( !object->isNewObject && object->size )
    {
      QByteArray first;
      QByteArray end;
      
      if(dfile.open(IO_ReadOnly))
      {
        if( object->offset )
        {
          first.resize(object->offset);
          dfile.readBlock(first.data(), object->offset);
        }
        
        unsigned long uEnd = object->offset + object->size;
        if( uEnd < dfile.size() )
        {
          dfile.at(uEnd);
          
          end.resize(dfile.size() - uEnd);
          dfile.readBlock(end.data(), dfile.size() - uEnd);
        }
        
        dfile.close();
      }
      else
        qDebug("BlockDevice %s: cannot read data", storageDevice.latin1());
        
      if(dfile.open(IO_WriteOnly))
      {
        if( first.size() )
          dfile.writeBlock(first.data(), first.size());
          
        if( end.size() )
          dfile.writeBlock(end.data(), end.size());
        
        dfile.close();
      }
      else
        qDebug("BlockDevice %s: cannot write data", storageDevice.latin1());
        
      // update the objects offset
      for(QPtrListIterator<ObjectReference> it(objectsList); it.current(); ++it)
      {
        if( it.current()->offset > object->offset )
          it.current()->offset -= object->size;
      }
    }
    
    // this just do nothing
    // I have added it in case we will really use Transaction
    TransactionServer::thisInstance()->remove(object);
    
    delete object;
    object = 0L;
    
    return true;
  }

  return false; // object does not exist
}

