/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */


 /** Changes: 2001-2002 Eugen C. <eug@thekompany.com>*/
 
#include <collectionmanager.h>
#include <qsettings.h>
#include <filefolder.h>
#include <blockdev.h>
#include <qdir.h>
#include <orb.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <transactionserver.h>
#include <remotemailfolder.h>
#include <imap4handler.h>

#include <settings.h>

class CollectionManager::Private
{
  public:
    Private(CollectionManager * parent);
    virtual ~Private();
    
    QString createFolderStructure(QString baseName);
    QString createBlockStructure(QString baseName);
    
    /** Remove a local mail folder.*/
    bool removeLocalMailFolder(const QString &name);
    /** Remove a remote mail folder and its local mirror.*/
    bool removeRemoteMailFolder(const QString &name);
    bool removeFileFolder(const QString &name);
    bool removeBlockDevice(const QString &name);
    
    // init methods
    void addSysMailFolders();
    void addSysBlockDevice(const QString&);
    void addSysFileFolder(const QString&);
    void addSystemFolders(QStringList, CollectionManager::BackendType);
    
    CollectionManager::BackendType mime2backend(const QString &);
//    void addSysGroups();
    
    /** Add the local and remote mail folders.*/
    void addMailFolders();
    /** Add the local mail folders.*/
    void addLocalMailFolders();
    /** Add the remote mail folders.*/
    void addRemoteMailFolders();
    void addBlockFolders();
    void addFileFolders();
    
    //void addJournals();
    //void addScheduler();
    //void addTasks();
    //void addToDos();
    //void addScripts();

    QDict<DataCollection> collectionDict;
    QDict<MailFolder> mailDict;
    QDict<RemoteMailFolder> remoteMailDict;
    QDict<int> plugins;
    
    QString mainDirectory;
    
  private:
    CollectionManager * parent_;
};

CollectionManager::Private::Private(CollectionManager * parent)
  : parent_(parent)
{
  mailDict.setAutoDelete(true);
  collectionDict.setAutoDelete(true);
  
  mainDirectory = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  
//  // system folders
//  addSysContacts();
//  addSysGroups();
//  addSysNotes();
//  addSysMailFolders();

  // init the collections
  addMailFolders();
  addBlockFolders();
  addFileFolders();
  
//  addJournals();
//  addScheduler();
//  addTasks();
//  addToDos();
//  addScripts();
}

CollectionManager::Private::~Private()
{
}

void CollectionManager::Private::addMailFolders()
{
  addLocalMailFolders();
  addRemoteMailFolders();
}

void CollectionManager::Private::addLocalMailFolders()
{
  qDebug("Scanning for the local mail folders...");

  // scan for additional mail directories
  QDir mdir(mainDirectory);
  mdir.setFilter(QDir::Dirs | QDir::Readable | QDir::Writable);
  mdir.setNameFilter("mailfolder*");
  QStringList mfolders=mdir.entryList();
  for(unsigned i=0;i<mfolders.count();i++)
  {
    // debug
    qDebug("Adding the local mail folder %s", (const char *)(mdir.path()+"/"+mfolders[i]));
    
    mailDict.insert(mfolders[i], new LocalMailFolder(mdir.path()+"/"+mfolders[i]));
  }
  
  qDebug("Scan complete");
}

void CollectionManager::Private::addRemoteMailFolders()
{
  qDebug("Scanning for the remote mail folders...");

  // scan for additional mail directories
  QDir mdir(mainDirectory);
  mdir.setFilter(QDir::Dirs | QDir::Readable | QDir::Writable);
  mdir.setNameFilter("imap_*");
  QStringList mfolders=mdir.entryList();
  for(unsigned i=0;i<mfolders.count();i++)
  {
    // debug
    qDebug("Adding the remote mail folder %s", (const char *)(mdir.path()+"/"+mfolders[i]));
    RemoteMailFolder *mailFolder=new RemoteMailFolder(mdir.path()+"/"+mfolders[i]);
    mailDict.insert(mfolders[i], mailFolder);
    remoteMailDict.insert(mailFolder->account()+"/"+mailFolder->mailbox(), mailFolder);
  }
  
  qDebug("Scan complete");
}

void CollectionManager::Private::addSysMailFolders()
{
  QString mdir=mainDirectory;
  
  // sanity checks
  if(!QFile::exists(mdir+"/sysmail_inbox"))
  {
    qDebug("Warning: sysmail_inbox does not exists.");
    createFolderStructure("sysmail_inbox");
  }
  if(!QFile::exists(mdir+"/sysmail_outbox"))
  {
    qDebug("Fatal: sysmail_outbox does not exists.");
    createFolderStructure("sysmail_outbox");
  }
  if(!QFile::exists(mdir+"/sysmail_drafts"))
  {
    qDebug("Fatal: sysmail_drafts does not exists.");
    createFolderStructure("sysmail_drafts");
  }
  if(!QFile::exists(mdir+"/sysmail_sent"))
  {
    qDebug("Fatal: sysmail_sent does not exists.");
    createFolderStructure("sysmail_sent");
  }
  if(!QFile::exists(mdir+"/sysmail_saved"))
  {
    qDebug("Fatal: sysmail_saved does not exists.");
    createFolderStructure("sysmail_saved");
  }
  if(!QFile::exists(mdir+"/sysmail_deleted"))
  {
    qDebug("Fatal: sysmail_deleted does not exists.");
    createFolderStructure("sysmail_deleted");
  }

  //TODO maybe the ko sync folders should be somewhere else
  if(!QFile::exists(mdir+"/sysko_incoming"))
  {
    qDebug("Fatal: sysmail_deleted does not exists.");
    createFolderStructure("sysko_incoming");
  }
  if(!QFile::exists(mdir+"/sysko_outgoing"))
  {
    qDebug("Fatal: sysmail_deleted does not exists.");
    createFolderStructure("sysko_outgoing");
  }

  
  // init system inbox
  mailDict.insert("sysmail_inbox", new LocalMailFolder(mdir+"/sysmail_inbox"));
  // debug
  qDebug("sysmail_inbox added");
  
  // init system outbox
  mailDict.insert("sysmail_outbox", new LocalMailFolder(mdir+"/sysmail_outbox"));
  // debug
  qDebug("sysmail_outbox added");
  
  // init system drafts
  mailDict.insert("sysmail_drafts", new LocalMailFolder(mdir+"/sysmail_drafts"));
  // debug
  qDebug("sysmail_drafts added");
  
  // init system sent mail
  mailDict.insert("sysmail_sent", new LocalMailFolder(mdir+"/sysmail_sent"));
  // debug
  qDebug("sysmail_sent added");
  
  // init system saved messages
  mailDict.insert("sysmail_saved", new LocalMailFolder(mdir+"/sysmail_saved"));
  // debug
  qDebug("sysmail_saved added");
  
  // init system deleted messages
  mailDict.insert("sysmail_deleted", new LocalMailFolder(mdir+"/sysmail_deleted"));
  // debug
  qDebug("sysmail_deleted added");
  
  mailDict.insert("sysko_incoming", new LocalMailFolder(mdir+"/sysko_incoming"));
  // debug
  qDebug("sysko_incoming added");
  
  mailDict.insert("sysko_outgoing", new LocalMailFolder(mdir+"/sysko_outgoing"));
  // debug
  qDebug("sysko_outgoing added");

}

void CollectionManager::Private::addSysBlockDevice(const QString & name)
{
  // insert the system contacts folder
  QString sysName =
    mainDirectory+"/"+name;
 
  if(!QFile::exists(sysName))
  {
    createBlockStructure( name );
  }
  collectionDict.insert(name, new BlockDevice(sysName, "i"));

  // debug
  qDebug("Adding system BLOCK %s...", name.latin1());
}

void CollectionManager::Private::addBlockFolders()
{
  // scan the data dir
  QString mdir=mainDirectory;
  
  QDir d(mdir);
  d.setNameFilter("blockdev*");
  d.setFilter(QDir::Files | QDir::Readable | QDir::Writable);
  QStringList files=d.entryList();
  
  for(unsigned i=0;i<files.count();i++)
  {
    // add a new folder
    collectionDict.insert(files[i], new BlockDevice(d.path()+"/"+files[i], "b"));
    
    // debug
    qDebug("Adding block device %s...", (const char *)(d.path()+"/"+files[i]));
  }
}

//void CollectionManager::Private::addSysGroups()
//{
//  // insert the system groups folder
//  QString sysgroups=
//    mainDirectory+"/sysgroups";
//
//  if(!QFile::exists(sysgroups))
//  {
//    qDebug("Fatal: system folder [%s] not found, improper installation.", (const char *)sysgroups);
//    createBlockStructure("sysgroups");
//  }
//  collectionDict.insert("sysgroups", new BlockDevice(sysgroups, "b"));
//  
//  // debug
//  qDebug("Adding system groups folder %s...", (const char *)sysgroups);
//}
//

void CollectionManager::Private::addFileFolders()
{
  // scan the data dir
  QString mdir=mainDirectory;
  
  QDir d(mdir);
  d.setNameFilter("filefolder*");
  d.setFilter(QDir::Dirs | QDir::Readable | QDir::Writable);
  QStringList files=d.entryList();
  
  for(unsigned i=0;i<files.count();i++)
  {
    // add a new folder
    collectionDict.insert(files[i], new FileFolder(d.path()+"/"+files[i], "f"));
    
    // debug
    qDebug("Adding file folder %s...", (const char *)(d.path()+"/"+files[i]));
  }
}

void CollectionManager::Private::addSysFileFolder(const QString & name)
{
  // insert the system notes folder
  QString sysName =
    mainDirectory+"/"+name;
 
  if(!QFile::exists(sysName))
  {
    createFolderStructure(name);
  }
  collectionDict.insert(name, new FileFolder(sysName, "f"));

  qDebug("Adding system file folder %s...", sysName.latin1());
}

// void CollectionManager::Private::addJournals()
// {
//   Settings::instance()->setGroup("Directory");
//   QString path=Settings::instance()->readEntry("AetheraDir");
//   path+="/Journals/";
//   QDir d(path);
//   d.setFilter(QDir::Files | QDir::Readable | QDir::Writable | QDir::NoSymLinks);
//   d.setNameFilter("*.jf");
//   for(int i=0;i<d.count();i++)
//   {
//     QString jname=d[i];
//     jname.truncate(jname.length()-3);
//     collectionDict.insert("journals://Journals/"+jname, new iFolder(path+d[i], "JEV"));
//
//     qDebug("Adding journals://Journals/%s ...", (const char *)jname);
//   }
// }

// void CollectionManager::Private::addScheduler()
// {
//   Settings::instance()->setGroup("Directory");
//   QString path=Settings::instance()->readEntry("AetheraDir");
//   path+="/Calendar/schedule";
//   collectionDict.insert("scheduler://Scheduler", new iFolder(path, "SEV"));
//
//   qDebug("Adding scheduler://Scheduler ...");
// }
//
// void CollectionManager::Private::addTasks()
// {
//   Settings::instance()->setGroup("Directory");
//   QString path=Settings::instance()->readEntry("AetheraDir");
//   path+="/Calendar/tasks";
//   collectionDict.insert("tasks://Tasks", new iFolder(path, "TEV"));
//
//   qDebug("Adding tasks://Tasks ...");
// }
//
// void CollectionManager::Private::addToDos()
// {
//   Settings::instance()->setGroup("Directory");
//   QString path=Settings::instance()->readEntry("AetheraDir");
//   path+="/Calendar/ToDos";
//   QDir d(path);
//   d.setFilter(QDir::Files | QDir::Readable | QDir::Writable | QDir::NoSymLinks);
//   d.setNameFilter("*.tf");
//   for(int i=0;i<d.count();i++)
//   {
//     QString tname=d[i];
//     tname.truncate(tname.length()-3);
//     collectionDict.insert("todos://ToDos/"+tname, new iFolder(path+d[i], "MTD"));
//
//     qDebug("Adding todos://ToDos/%s ...", (const char *)tname);
//   }
// }
//
// void CollectionManager::Private::addScripts()
// {
// #warning TODO: CollectionManager::Private::addScripts()
// }

// Maximum number of folders with the same type
#define MAX_FOLDERS_NR     65536

QString CollectionManager::Private::createFolderStructure(QString baseName)
{
  QString folderName;
  QDir aetheraDir = mainDirectory;
  int baseIndex=0;

  for(unsigned i=0; i<MAX_FOLDERS_NR; i++)
  {
    if(baseName.left(3)!="sys")
      folderName.sprintf(baseName+"%05d", ++baseIndex);
    else
      folderName=baseName;

    if(!aetheraDir.exists(folderName))
    {
      // debug
      printf("collectionmanager: creating folder %s...\n", (const char *)folderName);

      if(aetheraDir.mkdir(folderName))
      {
        return folderName;
      }
      else
      {
        printf("\ncollectionmanager: couldn't create a new folder!\n");
        return QString::null;
      }
    }
  }
  
  return QString::null;
}

QString CollectionManager::Private::createBlockStructure(QString baseName)
{
  QDir aetheraDir(mainDirectory);
  int baseIndex=0;

  while(1)
  {
    QString blockName;

    if(baseName.left(3)!="sys")
      blockName.sprintf(baseName+"%05d", ++baseIndex);
    else
      blockName=baseName;

    if(!aetheraDir.exists(blockName))
    {
      // debug
      printf("collectionmanager: creating block device %s...\n", (const char *)blockName);

      QFile blockFile(aetheraDir.path()+"/"+blockName);
      if(blockFile.open(IO_WriteOnly))
      {
        blockFile.close();
        return blockName;
      }
      else
      {
        return QString::null;
      }
    }
  }
  
  return QString::null;
}

bool CollectionManager::Private::removeLocalMailFolder(const QString &name)
{
  QDir aetheraDir(mainDirectory);
  QDir mailFolder(mainDirectory+"/"+name);

  mailFolder.remove("descriptor");
  mailFolder.remove("index");
  mailFolder.remove("messages");

  return aetheraDir.rmdir(mailFolder.dirName());
}

bool CollectionManager::Private::removeRemoteMailFolder(const QString &name)
{
  QDir aetheraDir(mainDirectory);
  QDir mailFolder(mainDirectory+"/"+name);

  mailFolder.remove("deluids");
  mailFolder.remove("descriptor");
  mailFolder.remove("index");
  mailFolder.remove("headers");
  mailFolder.remove("rcvuids");
  mailFolder.remove("config");

  return aetheraDir.rmdir(mailFolder.dirName());
}

bool CollectionManager::Private::removeBlockDevice(const QString &name)
{
  QDir aetheraDir(mainDirectory);
  
  return aetheraDir.remove(name);
}

bool CollectionManager::Private::removeFileFolder(const QString &name)
{
  QDir aetheraDir(mainDirectory);
  QDir fileFolder(mainDirectory+"/"+name);

  QStringList fileList=fileFolder.entryList();
  for(unsigned i=0;i<fileList.count();i++)
    fileFolder.remove(fileList[i]);
  
  return aetheraDir.rmdir(fileFolder.dirName());
}


// ---------------------------------------------------------------------

CollectionManager::CollectionManager()
{
  d = new Private(this);
}

CollectionManager::~CollectionManager()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

bool CollectionManager::deleteObject(const QStringList & urls)
{
  QString folder = ObjectRequestBroker::thisInstance()->folderpath(urls[0]);
  DataCollection * sourceFolder = d->collectionDict[folder];
  if(!sourceFolder)
    return false;
  
  QStringList deleteList;
  QString objectId;
  
  for(QStringList::ConstIterator it=urls.begin(); it!=urls.end(); ++it)
  {
    objectId = ObjectRequestBroker::thisInstance()->object(*it);
    if(objectId.isEmpty())
      break;
    
    // remove the source object
    sourceFolder->removeEntry(objectId);
    
    deleteList.append( *it );
  }
  
  if( deleteList.count() )
    ServerNotifier::thisInstance()->objectDeleted( deleteList );
  
  return true;
}

bool CollectionManager::copyObject(const QStringList & urls, const QString & dest_folder)
{
  // get the target collection
  DataCollection * targetFolder = d->collectionDict[dest_folder];
  if(!targetFolder)
    return false;
  
  // get the source collection
  QString source_folder = ObjectRequestBroker::thisInstance()->folderpath(urls[0]);
  DataCollection * sourceFolder = d->collectionDict[source_folder];
  if(!sourceFolder)
    return false;
  
  // it has to have the same type
  if(sourceFolder->type != targetFolder->type)
    return false;
  
  QStringList createList;
  
  QString objectId;
  ObjectReference * targetObject = 0L;
  ObjectReference * sourceObject = 0L;
  QByteArray buffer;
  
  for(QStringList::ConstIterator it=urls.begin(); it!=urls.end(); ++it)
  {
    objectId = ObjectRequestBroker::thisInstance()->object(*it);
    if(objectId.isEmpty())
      break;
    
    // create the target object
    targetObject = targetFolder->addNewEntry();
    sourceObject = sourceFolder->ref(objectId);
    
    if(!targetObject || !sourceObject)
      break;
    
    sourceObject->data(buffer);
    targetObject->setData(buffer);
    
    createList.append( dest_folder + "/" + targetObject->id );
  }
  
  if( createList.count() )
    ServerNotifier::thisInstance()->objectCreated( createList );
  
  return true;
}

bool CollectionManager::moveObject(const QStringList & urls, const QString & dest_folder)
{
  // get the target collection
  DataCollection * targetFolder = d->collectionDict[dest_folder];
  if(!targetFolder)
    return false;
  
  // get the source collection
  QString source_folder = ObjectRequestBroker::thisInstance()->folderpath(urls[0]);
  DataCollection * sourceFolder = d->collectionDict[source_folder];
  if(!sourceFolder)
    return false;
  
  // it has to have the same type
  if(sourceFolder->type != targetFolder->type)
    return false;
  
  QStringList deleteList;
  QStringList createList;
  
  QString objectId;
  ObjectReference * targetObject = 0L;
  ObjectReference * sourceObject = 0L;
  QByteArray buffer;
  
  for(QStringList::ConstIterator it=urls.begin(); it!=urls.end(); ++it)
  {
    objectId = ObjectRequestBroker::thisInstance()->object(*it);
    if(objectId.isEmpty())
      break;
    
    // create the target object
    targetObject = targetFolder->addNewEntry();
    sourceObject = sourceFolder->ref(objectId);
    
    if(!targetObject || !sourceObject)

      break;
    
    sourceObject->data(buffer);
    targetObject->setData(buffer);
    
    // remove the source object
    sourceFolder->removeEntry(objectId);
    
    deleteList.append( *it );
    createList.append( dest_folder + "/" + targetObject->id );
  }
  
  if( deleteList.count() )
    ServerNotifier::thisInstance()->objectDeleted( deleteList );
  if( createList.count() )
    ServerNotifier::thisInstance()->objectCreated( createList );
  
  return true;
}

QString CollectionManager::moveObject(const QString &url, const QString &dest_folder)
{
  DataCollection *t = d->collectionDict[dest_folder];
  if(!t)
  {
    qDebug("Warning: the collection dictionary returned null when a valid pointer was expected, bailing out");
    return QString::null;
  }
  QString tobj=ObjectRequestBroker::thisInstance()->object(url);
  if(tobj.isEmpty())
  {
    qDebug("Warning: ORB::object() returned an empty string when a valid object id was expected, bailing out");
    return QString::null;
  }
  QString sfolder=ObjectRequestBroker::thisInstance()->folderpath(url);
  DataCollection *s = d->collectionDict[sfolder];
  if(!s)
  {
    qDebug("Warning: the collection dictionary returned null when a valid pointer was expected, bailing out");
    return QString::null;
  }
  if(s->type!=t->type)
  {
    qDebug("Warning: reparenting can be done only between storage devices of the same type, bailing out");
    return QString::null;
  }
  QString tid=t->addEntry();
  ObjectReference *tref=t->ref(tid), *sref=s->ref(tobj);
  if(!tref || !sref)
  {
    qDebug("Warning: DataCollection::ref() returned null when a valid pointer was expected, bailing out");
    return QString::null;
  }
  
  QByteArray buffer;
  sref->data(buffer);
  tref->setData(buffer);
//  tref->filename=sref->filename; // hmm... here we should check if the filename does not already exist in our destination
  s->removeEntry(tobj);

  TransactionServer::thisInstance()->save(tref);

  return dest_folder+"/"+tid;
}

QString CollectionManager::moveCollection(const QString &src_folder, const QString &dest_folder)
{
  DataCollection *t = d->collectionDict[dest_folder];
  DataCollection *s = d->collectionDict[src_folder];
  
  if(!t || !s)
  {
    qDebug("Warning: the collection dictionary returned null when a valid pointer was expected, bailing out");
    return QString::null;
  }
  if(t->type!=s->type)
  {
    qDebug("Warning: reparenting can be done only between devices of the same type, bailing out");
    return QString::null;
  }
  QStringList ids=s->getEntryList();
  for(unsigned i=0;i<ids.count();i++)
  {
    ObjectReference *sref=s->ref(ids[i]), *tref=t->ref(t->addEntry());
    if(!sref||!tref)
    {
      qDebug("Warning: at least one object reference was null while reparenting, aborting");
//      if(s->type==DataCollection::FileStorage) // save what we can, then abort
//      {
//        TransactionServer::thisInstance()->save(s);
//        TransactionServer::thisInstance()->save(t);
//      }
      return dest_folder;
    }
    tref->filename=sref->filename;
    
    QByteArray buffer;
    sref->data(buffer);
    tref->setData(buffer);
    
//    if(s->type==DataCollection::FolderStorage)
      TransactionServer::thisInstance()->save(tref);

    s->removeEntry(ids[i]);
  }

//  if(s->type==DataCollection::FileStorage)
//  {
//    TransactionServer::thisInstance()->save(s);
//    TransactionServer::thisInstance()->save(t);
//  }

  return dest_folder;
}

void CollectionManager::createFolder(QString parent, QString folderPath, QString type, QString viewType)
{
  QString folderName;
  QString aetheraDir(d->mainDirectory);
  
  if(type=="mail")
  {
    folderName = d->createFolderStructure("mailfolder");
    if(!folderName.isEmpty())
      d->mailDict.insert(folderName, new LocalMailFolder(aetheraDir+"/"+folderName));
  }
  else if(type=="imap")
  {
    if(viewType=="mail")
    {
      QString key = parent + "/" + folderPath;
      RemoteMailFolder *folder = d->remoteMailDict[key];
      if( !folder )
      {
        folderName = d->createFolderStructure("imap_");
        if(!folderName.isEmpty())
        {
          // create the folder in Aethera server
          RemoteMailFolder * mailFolder =
            new RemoteMailFolder(aetheraDir+"/"+folderName, parent, folderPath);
            
          if( mailFolder )
          {
            d->mailDict.insert(folderName, mailFolder);
            d->remoteMailDict.insert(key, mailFolder);
            mailFolder->removeStatus();
          }
          else
          {
            qDebug("Cannot create the RemoteMailFolder object.");
          }
        }
        else
          return;
      }
      else
        folderName=folder->name();
    }
  }
  else if(type=="contacts" || type=="groups")
  {
    folderName = d->createBlockStructure("blockdev");
    if(!folderName.isEmpty())
      d->collectionDict.insert(folderName, new BlockDevice(aetheraDir+"/"+folderName, "b"));
  }
  else if(type=="notes")
  {
    folderName = d->createFolderStructure("filefolder");
    if(!folderName.isEmpty())
      d->collectionDict.insert(folderName, new FileFolder(aetheraDir+"/"+folderName, "f"));
  }
  
  // rikkus: these should be called:
  // parentFolderPath, folderName, type, viewType, serverFolderPath

  // Send the create file notification
  QStringList parameters;
  parameters.append(parent);
  parameters.append(folderPath);
  parameters.append(type);
  parameters.append(viewType);
  parameters.append(folderName);
  ServerNotifier::thisInstance()->folderCreated(parameters);
}

QString CollectionManager::createFolder(QStringList paramList)
{
  QString account, mailbox, type, viewType;
  unsigned i=0;

  for(QStringList::Iterator it=paramList.begin(); it!=paramList.end(); ++it)
  {
    switch(i)
    {
      case 0:
        type=(*it);
        break;
      case 1:
        viewType=(*it);
        break;
      case 2:
        account=(*it);
        break;
      case 3:
        mailbox=(*it);
        break;
    }
    i++;
  }

  qDebug("collectionmanager: trying to create folder, dev type %s", (const char *)type);

  QString aetheraDir(d->mainDirectory);

  CollectionManager::BackendType backend = d->mime2backend(type);
  QString folderName;
  
  switch( backend )
  {
    case Mail:
      if(type=="mail")
      {
        folderName = d->createFolderStructure("mailfolder");
        
        if(!folderName.isEmpty())
          d->mailDict.insert(folderName, new LocalMailFolder(aetheraDir+"/"+folderName));
      }
      else if(type=="imap")
      {
        if(viewType=="mail")
          IMAP4Handler::ref()->createFolder(account, mailbox, viewType);
        else
          IMAP4Handler::ref()->createFolder(account, mailbox+"/", viewType);
      }
      break;
      
    case Block:
      folderName = d->createBlockStructure("blockdev");
      if(!folderName.isEmpty())
        d->collectionDict.insert(folderName, new BlockDevice(aetheraDir+"/"+folderName, "b"));
      break;
      
    case SQL:
      break;
      
    case Folder:
    default:
      folderName = d->createFolderStructure("filefolder");
      if(!folderName.isEmpty())
        d->collectionDict.insert(folderName, new FileFolder(aetheraDir+"/"+folderName, "f"));
      break;
  }
  
  return folderName;
}

bool CollectionManager::removeFolder(const QString &name)
{
  if(name.find("imap_")==0) 
  {
    // TODO: that's bad... remoteMailDict is using "account + / + mailbox" as key
    
    // Remove it from the IMAP server
    qDebug("CollectionManager::removeRemoteMailFolder: Async IMAP...");
    RemoteMailFolder * folder = dynamic_cast<RemoteMailFolder*> (d->remoteMailDict[name]);
    if( folder )
      return IMAP4Handler::ref()->deleteFolder(folder->account(), folder->mailbox(), name);
    else
      return false;
  }

  d->mailDict.remove(name) || d->collectionDict.remove(name);

  if(name.find("mailfolder")==0)
    return d->removeLocalMailFolder(name);
  else if(name.find("filefolder")==0)
    return d->removeFileFolder(name);
  else if(name.find("blockdev")==0)
    return d->removeBlockDevice(name);

  return false;
}

void CollectionManager::deleteFolder( const QString & parent, 
                                      const QString & folderPath, 
                                      const QString & type, 
                                      const QString & serverPath)
{
  if( type=="imap" )
  {
    if( serverPath!="default" )
    {
      QString backendFolder = serverPath;
      QString key = parent + "/" + folderPath;
      RemoteMailFolder * emailFolder = d->remoteMailDict[key];
      
      if( emailFolder )
      {
        emailFolder->removeStatus();
        if( backendFolder.isEmpty() )
        {
          backendFolder = emailFolder->name();
        }
      }
      if( !backendFolder.isEmpty() )
      {
        d->mailDict.remove(backendFolder);
        d->remoteMailDict.remove(parent + "/" + folderPath);
        d->removeRemoteMailFolder(backendFolder);
      }
    }

    QStringList parameters;
    parameters.append(parent);
    parameters.append(folderPath);
    parameters.append(type);
    ServerNotifier::thisInstance()->folderDeleted(parameters);
  }
}

void CollectionManager::renameFolder(const QString &account, const QString &mailbox, const QString &newName, const QString &url)
{
//  qDebug("acc=%s, mailbox=%s, newname=%s, url=%s",
//    account.latin1(),
//    mailbox.latin1(),
//    newName.latin1(),
//    url.latin1());
  QString newMailbox = mailbox.left( mailbox.findRev("/") + 1 ) + newName;
  unsigned uLength = mailbox.length();
  
  // Change the mailbox from config files of RemoteMailFolders
  RemoteMailFolder * folder = 0L;
  for(QDictIterator<RemoteMailFolder> it(d->remoteMailDict); it.current(); ++it)
  {
    folder = it.current();
    
    if( folder &&
        folder->account() == account &&
        folder->mailbox().left(uLength) == mailbox )
    {
      QString temp = newMailbox + folder->mailbox().mid(uLength);
      folder->makeConfig(folder->account(), temp);
      // TODO: change remoteMailDict too
    }
  }
  
  QStringList parameters;
  parameters.append(url);
  parameters.append(newName);

  ServerNotifier::thisInstance()->renameFolder(parameters);
}

void CollectionManager::addPlugin(const QString & mimeType,
                                  QStringList systemFolders,
                                  const QString & backendType)
{
  if( !backendType.isEmpty() )
  {
    int * type;
  
    BackendType backend = CollectionManager::Unknown;
    char c = backendType[0].lower().latin1();
    
    switch(c)
    {
      case 'm':
        backend = CollectionManager::Mail;

        break;
      case 'b':
        backend = CollectionManager::Block;
        break;
      case 'f':
      default:
        backend = CollectionManager::Folder;
        break;
    }
    
    type = new int( backend );
    
    d->plugins.insert(mimeType, type);
    
    if( "mail" == mimeType )
    {
      d->addSysMailFolders();
    }
    else
    {
      if( systemFolders.size() )
        d->addSystemFolders(systemFolders, backend);
    }
  }
}

void CollectionManager::Private::addSystemFolders(QStringList folders, CollectionManager::BackendType type)
{
  for(QStringList::Iterator it=folders.begin(); it!=folders.end(); ++it)
  {
    switch(type)
    {
      case Mail:
        // nothing because the mail folders are already created
        break;
      case Block:
        addSysBlockDevice( (*it) );
        break;
      case Folder:
      default:
        addSysFileFolder( (*it) );
        break;
    }
  }
}

CollectionManager::BackendType CollectionManager::Private::mime2backend(const QString & type)
{
  QString mime;
  
  if( "mail" == type || "imap" == type)
    mime = "mail";
  else
    mime = type;
  
  return (CollectionManager::BackendType)(*(plugins[mime]));
}

DataCollection *CollectionManager::folderReference(const QString url) const
{
  return d->collectionDict[url];
}

MailFolder *CollectionManager::mailfolderReference(const QString url) const
{
  return d->mailDict[url];
}

