/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

 /**
 * Changed by Eugen C. for speeding reasons.
 */

#include <datacollection.h>
#include <transactionserver.h>
#include <stdio.h>
#include <assert.h>
#include <qlist.h>

//#define DEBUG_DATACOLLECTION

ObjectReference::ObjectReference(DataCollection * parent)
  : dirty(true),
    isNewObject(true),
    size(0),
    offset(0),
    collection(parent)
{
}

ObjectReference::~ObjectReference()
{
}

void ObjectReference::setData(const QByteArray & info)
{
  collection->setData(this, info);
}

void ObjectReference::data(QByteArray & info)
{
  collection->data(this, info);
}
    
DataCollection::DataCollection(QString prefix)
{
  if(!prefix)
    idPrefix="OBJ";
  else
    idPrefix=prefix;
  objectsDict.setAutoDelete(false);
  objectsList.setAutoDelete(false);
  lastIDNum=0;
}

DataCollection::~DataCollection()
{
//  objectsList.setAutoDelete(true);
  objectsList.clear();
  objectsDict.setAutoDelete(true);
  objectsDict.clear();
}

QPtrList<ObjectReference>& DataCollection::objectList()
{
  return objectsList;
}

QDict<ObjectReference>& DataCollection::objectDict()
{
  return objectsDict;
}

QStringList DataCollection::getEntryList(const QString url) const
{
//  qDebug("DataCollection::getEntryList: %s", url.latin1());
  
  QStringList idsList;

  // TODO: this has to be changed
  if( 1 /*url.isEmpty()*/ )
    for(QPtrListIterator<ObjectReference> it(objectsList); it.current(); ++it)
      idsList.append(it.current()->id);
  else
    for(QPtrListIterator<ObjectReference> it(objectsList); it.current(); ++it)
      idsList.append(url + "/" + it.current()->id);

  return idsList;
}

QString DataCollection::addEntry()
{
  ObjectReference * object = addNewEntry();
  
  if(object)
    return object->id;
  
  return QString::null;
}

ObjectReference * DataCollection::addNewEntry()
{
#ifdef DEBUG_DATACOLLECTION
  printf("DataCollection: Adding a new entry...(void)\n");
#endif

  ObjectReference * t = new ObjectReference(this);
  
  t->id=nextID();
  t->isNewObject=true;
  if(type==FolderStorage)
    t->filename=t->id;
  t->dirty=false;
  
  objectsDict.insert(t->id, t);
  objectsList.append(t);
  
  return t;
}

QString DataCollection::addEntry(ObjectReference *obj)
{
#ifdef DEBUG_DATACOLLECTION
  printf("DataCollection: Adding a new entry...(ObjectReference)\n");
#endif
  if( !objectsDict.find(obj->id) )
  {
  
// Hmm, why to set a parent in this way ?!
//    obj->collection=this;

    obj->dirty=false;
    objectsDict.insert(obj->id, obj);
    objectsList.append(obj);
    return obj->id;
  }
  else
    return QString::null;
}

ObjectReference* DataCollection::ref(QString id)
{
  return objectsDict.find(id);
}

QString DataCollection::nextID()
{
  QString t;
  QDict<ObjectReference>& objects=objectDict();
  bool bCheck=false;
  unsigned long nextId;

  // check if there are free ids
  for(nextId=1; nextId<=lastIDNum+1; nextId++)
  {
    t.sprintf("%s%09ld", (const char *)idPrefix, nextId);
    // check the dict if there is the "next" entry
    if( !objects[t] )
    {
      bCheck=true;
      break;
    }
  }

  // that's a bug
  assert(bCheck);

  // there wasn't any free id, so just increase the ids number
  if( nextId==lastIDNum+1 )
    lastIDNum++;
  return t;
}

QString DataCollection::lastID()
{
  QString t;
  t.sprintf("%s%09ld", (const char *)idPrefix, lastIDNum);
  return t;
}

bool DataCollection::isValidID(QString id)
{
  if(id.left(idPrefix.length())!=idPrefix)
    return false;
  for(uint i=idPrefix.length();i<id.length();i++)
    if(!id[(int)i].isNumber())
      return false;
  
  return true;
}



