/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */

#include <orb.h>
#include <magellan.h>
#include <indexclass.h>
#include <transactionserver.h>
#include <attributemarshall.h>
#include <conversions.h>
#include <messagedevice.h>

#include <qfileinfo.h>

//#define DEBUG_ORB

ObjectRequestBroker * ObjectRequestBroker::inst = 0;

ObjectRequestBroker::ObjectRequestBroker()
  : CollectionManager()
{
  // nothing here
}

ObjectRequestBroker::~ObjectRequestBroker()
{
  // nothing here
}

ObjectRequestBroker *ObjectRequestBroker::thisInstance()
{
  if(!inst)
    inst = new ObjectRequestBroker();
  
  return inst;
}

QString ObjectRequestBroker::lastError()
{
  return err;
}

bool ObjectRequestBroker::registerNotify(QString url)
{
  // the loopback server notifies the client by default

  return exists(url);
}

void ObjectRequestBroker::clearStatus()
{
  err=QString::null;
}

bool ObjectRequestBroker::isFolder(const QString url) const
{
  if(url.isEmpty())
    return false;
    
  if( mailfolderReference(url) ) // check the mail folder dictionary
    return true;
  
  if( folderReference(url) ) // check the collection dictionary
    return true;

  return false;
}

bool ObjectRequestBroker::exists(const QString url) const
{
  if(url.isEmpty())
    return false;
    
  // remove the query, if any
//  if(url.contains('%'))
//    url=url.left(url.findRev('%'));
    
  // check if the given url is a valid folder
  if( isFolder(url) )
    return true;

  QString fpath  = folderpath(url);
  QString objectPath = object(url);
  
  // check for subobject queries
  if(objectPath.contains(ATTR_SEP))
  {
    // if it is a folder attribute
    if( objectPath == "default"+QString(ATTR_SEP)+"attr" || 
        objectPath == "default"+QString(ATTR_SEP)+"fattr" || 
        objectPath == "dir"+QString(ATTR_SEP)+"cache" )
      return isFolder(fpath);
      
    // if it is an object attribute
    if( objectPath.right(5)==QString(ATTR_SEP)+"attr" || 
        objectPath.right(6)==QString(ATTR_SEP)+"fattr" )
    {

      objectPath = objectPath.left( objectPath.findRev(ATTR_SEP) );
      
    } 
//      if(objectPath.contains(ATTR_SEP)>1)
//        return false;
      
    if(objectPath.contains(ATTR_SEP))
    {
      // this should be a mail part query
      
      // check mail folders
      MailFolder *mailFolder = mailfolderReference(fpath);
      if( mailFolder )
      {
        IndexClass *idx = mailFolder->indexIDict()[objectPath.left( objectPath.findRev(ATTR_SEP) )];
        if(idx)
          return true;
      }
      
      return false;
    }
  }

  // check if the given url is a valid object
  MailFolder * mailFolder = mailfolderReference(fpath);
  if( mailFolder )
  {
    IndexClass * idx = mailFolder->indexIDict()[objectPath];
    if(idx)
      return true;
  }
  
  DataCollection * dataFolder = folderReference(fpath);
  if( dataFolder )
  {
    ObjectReference * obj = dataFolder->objectDict()[objectPath];
    if(obj)
      return true;
  }

  return false;
}

QString ObjectRequestBroker::parent(const QString url) const
{
  // folders don't have logical parents
  if(isFolder(url))
    return QString::null;

  // check for folder attributes
  if(url.mid(url.find('/')+1)== "default"+QString(ATTR_SEP)+"attr" || url.mid(url.find('/')+1)== "default"+QString(ATTR_SEP)+"fattr")
    return folderpath(url);
    
  // check for subobjects - they always have parents
  int k=url.findRev(ATTR_SEP);
  if(k>-1)
    return url.left(k);
      
  if(subtype(url)==Magellan::Mail)
  {
    // have a look at the index relationship
    QString pidx=indexReference(url)->getParentID();
    return pidx.isEmpty()?folderpath(url):(folderpath(url)+'/'+pidx);
  }
  
  return folderpath(url);
}

QString ObjectRequestBroker::folder(const QString url) const
{
  QString folderPath = folderpath(url);
  
  if( isFolder(folderPath) )
    return folderPath;
  
  return QString::null;
}

bool ObjectRequestBroker::children(const QString url, QStringList& urls) const
{
  // debug
//  qDebug("ORB::children(): url %s\n", url.latin1());

  MailFolder * mailFolder = mailfolderReference(url);
  if( mailFolder )
  {
    urls = mailfolderReference(url)->getEntryList(url);
    
    return true;
  }
  
  DataCollection * dataFolder = folderReference(url);
  if( dataFolder )
  {
      urls = folderReference(url)->getEntryList(url);
      
      return true;
  }
  
  return false;
}

QStringList ObjectRequestBroker::related(const QString url) const
{
  QStringList rList;
  
  if(exists(url) && subtype(url)==Magellan::Mail)
  {
    // this is a mail object, query the related index id list
//#warning implement related()
  }
  
  return rList;
}

int ObjectRequestBroker::type(const QString url) const
{
  MailFolder * mailFolder = mailfolderReference( folderpath(url) );
  if( mailFolder )
    return Magellan::Message;
    
  DataCollection * collection = folderReference( folderpath(url) );
  if( collection )
  {
    if( DataCollection::FileStorage == collection->type )
      return Magellan::BlockObject;
    
    if( DataCollection::FolderStorage == collection->type )
      return Magellan::FileObject;
  }
  
  return Magellan::Unknown;
}

int ObjectRequestBroker::subtype(const QString url) const
{
  // remove all queries
//  if(url.contains('%')) url=url.left(url.findRev('%'));

  if(url.right(6)==QString(ATTR_SEP)+"cache")
  {
    return Magellan::FolderCache;
  }

  if(url.right(5)==QString(ATTR_SEP)+"attr" || url.right(6)==QString(ATTR_SEP)+"fattr")
  {
    // an attribute type
    if(url.contains("default"+QString(ATTR_SEP)+"attr") || url.contains("default"+QString(ATTR_SEP)+"fattr"))
    {
      return Magellan::FolderAttribute;
    }
    
    // TODO: stupid since a folder name can have '.' too
    //  debug with ATTR_SEP to replace it
    if(url.contains(ATTR_SEP)>1)
    {
      // this should be a mail part attribute
      if(subtype(url.left(url.findRev(ATTR_SEP)))==Magellan::Part)
      {
        return Magellan::PartAttribute;
      }
      else
      {
        return Magellan::Unknown;
      }
    }
    
    switch(type(url.left(url.find(ATTR_SEP))))
    {
      case Magellan::Message:
        return Magellan::MailAttribute;
      case Magellan::BlockObject:
        return Magellan::BlockObjectAttribute;
      default:
        return Magellan::Attribute;
    }
  }
  
  if(url.find(ATTR_SEP)>-1)
  {
    // this should be a mail part
    return Magellan::Part;
  }

  // return the storage device type 
  return type(url);
}

bool ObjectRequestBroker::setRelated(QString url, QStringList /* rList */)
{
  if(subtype(url)!=Magellan::Mail)
    return false;
  
  IndexClass *idx=indexReference(url);
  if(idx)
  {
//#warning implement setRelated()
  }
  else
  {
    // this shouldn't occur
#ifdef DEBUG_URLMANAGER
    printf("Warning: indexReference() returned null when a valid pointer was expected, bailing out\n");
#endif
    return false;
  }
  
  return true;
}

bool ObjectRequestBroker::setParent(QString url, QString parent)
{
  if(subtype(url)!=Magellan::Mail)
    return false;
  
  IndexClass *idx=indexReference(url);
  IndexClass *pdx=indexReference(folderpath(url)+"/"+parent);
  
  if(idx && pdx)
  {
//#warning implement setParent()
  }
  else
  {
    return false;
  }
  
  return true;
}

ObjectReference *ObjectRequestBroker::objectReference(const QString url) const
{
  DataCollection *t = folderReference( folderpath(url) );
  if(t)
  {
    return t->objectDict()[object(url)];
  }
  
  return 0;
}

IndexClass *ObjectRequestBroker::indexReference(const QString url) const
{
  MailFolder *t = mailfolderReference( folderpath(url) );

  if(t)
  {
    IndexClass *idx=t->indexIDict()[object(url)];
    return idx;
  }

  return 0;
}

QString ObjectRequestBroker::path(const QString url) const
{
  int k=url.find("://");
  if(k!=-1)
    return url.mid(k+3);

  return QString(url);
}

QString ObjectRequestBroker::object(const QString url) const
{
//  QString t(url);
//  if(t[t.length()-1]=='/')
//    t.truncate(t.length()-1);
//
//  int k=t.findRev('/');
//  if(k!=-1)
//    return t.mid(k+1);
//
//  return QString::null;

  return url.mid( url.findRev('/')+1 );
}

QString ObjectRequestBroker::folderpath(const QString url) const
{
  return url.left( url.findRev('/') );
}




