/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#ifndef __MagellanObjectRequestBroker__
#define __MagellanObjectRequestBroker__

#include <qstring.h>
#include <qstringlist.h>
#include <qcstring.h>
#include <collectionmanager.h>
#include <servernotifier.h>
#include <vfsmodule.h>

class ObjectRequestBroker : public CollectionManager
{
  public:
  
    ~ObjectRequestBroker();
    static ObjectRequestBroker *thisInstance();
    
    /** Returns true if the url is an existent folder.*/
    bool isFolder(const QString url) const;
    /** Returns true if the url is an existent folder or object.*/
    bool exists(const QString url) const;
    /** Returns the logical parent of a folder or object.*/
    QString parent(const QString url) const;
    /** Returns the folder parent of a folder or object.*/
    QString folder(const QString url) const;
    /** Returns the related objects.*/
    QStringList related(const QString url) const;
    /** Returns true if the url has children.
      * And 'urls' will be filled with the children's urls.
      */
    bool children(const QString url, QStringList& urls) const;
    
    // set methods
    /** Set related objects.*/
    bool setRelated(QString url, QStringList rList);
    /** Set the logical parent.*/
    bool setParent(QString url, QString parentUrl);
    
    // data handling methods
    /** Returns a reference to the required object (except for mail objects).*/
    ObjectReference *objectReference(const QString url) const;
    /** Returns a reference to the required mail.*/
    IndexClass *indexReference(const QString url) const;

    // string URL handling methods
    /** Returns the object major type, based on its url.*/
    int type(const QString url) const;
    /** Returns the object fine type.*/
    int subtype(const QString url) const;
    /** Returns the path string.*/
    QString path(const QString url) const;
    /** Returns the object name.*/
    QString object(const QString url) const;
    /** Returns the folder path.*/
    QString folderpath(const QString url) const;
    
    // object handling methods
    /** Get the object data.*/
    bool getObjectData(const QString &url, QByteArray &data);
    /** Get the objects data.*/
    bool getObjectData(const QStringList &urls, QByteArray &data);
    /** Get the recent mails number for a mails folder.*/
    int getFolderRecentData(const QString & folderURL);
    /** Get the data for all children from the folder.*/
    bool getChildrenData(const QString &url, QByteArray &data);
    /** Set the object data.*/
    bool setObjectData(const QString &url, const QByteArray &data);
    /** Set the objects data.*/
    bool setObjectsData(const QStringList &url, const QByteArray &data);
    /** Set the data for all children from the folder.*/
    bool setChildrenData(const QString &url, const QByteArray &data);
    QString createObject(QString folder_url);
    /** Create a new object (mail) using the data.*/
    QString createObject(QString folder_url, QByteArray &data);
    
    QString copy(QString url, QString dest_folder_url);
    QString move(QString url, QString dest_folder_url);
    /** Remove an object.*/
    bool remove(QString url);
    
    // multiple objects operations
    bool copy(const QStringList & urls, QString dest_folder_url);
    bool move(const QStringList & urls, QString dest_folder_url);
    bool remove(const QStringList & urls);
    
    bool send(QString url);
    /** Remove a folder.*/
    bool removeFolder(QStringList paramList);
    
    // notifiers
    bool registerNotify(QString url);

    // error handling methods
    QString lastError();
    void clearStatus();
    
  protected:
  
  private:
  
    /** It's a singleton class.*/
    ObjectRequestBroker();
    
    bool getMailData(const QString &url, QByteArray &data);
    bool getMailPropertyData(const QString &url, const QString &property, QByteArray &data);
    bool getPartData(const QString &url, QByteArray &data);
    bool getFolderCacheData(const QString &url, QByteArray &data);
    bool getFolderAttributeData(const QString &url, QByteArray &data);
    bool getMailAttributeData(const QString &url, QByteArray &data);
    bool getPartAttributeData(const QString &url, QByteArray &data);
    bool getBlockObjectAttributeData(const QString &url, QByteArray &data);
    bool getBlockObjectData(const QString &url, QByteArray &data);
    bool getAttributeData(const QString &url, QByteArray &data);
    bool getCollectionObjectData(const QString &url, QByteArray &data);
    bool getFileObjectData(const QString &url, QByteArray &data);
        
    bool setCollectionObjectData(const QString &url, const QByteArray &data);
    bool setPartData(const QString &url, int part, const QByteArray &data);
    bool setMailData(const QString &url, const QByteArray &data);
    bool setMailPropertyData(const QString &url, const QString &property, const QByteArray &data);
    bool setPartAttributeData(const QString &url, const QByteArray &data);
  
    // instance pointer
    static ObjectRequestBroker *inst;

    QString err;
};

#endif

