/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <orb.h>
#include <magellan.h>
#include <indexclass.h>
#include <mailfolder.h>
#include <transactionserver.h>
#include <attributemarshall.h>
#include <conversions.h>
#include <qfileinfo.h>
#include <messagedevice.h>
#include <messagedescriptor.h>
#include <messagefactory.h>
#include <imap4handler.h>

bool ObjectRequestBroker::getObjectData(const QString &url, QByteArray &data)
{
  if( !url || url.isEmpty() )
    return false;
  
  int t     = subtype(url);
  bool bRet = false;
  
  switch(t)
  {
    case Magellan::FolderCache :
      bRet = getFolderCacheData(url, data);
      break;
    case Magellan::FolderAttribute :
      bRet = getFolderAttributeData(url, data);
      break;
    case Magellan::MailAttribute :
      bRet = getMailAttributeData(url, data);
      break;
    case Magellan::PartAttribute :
      bRet = getPartAttributeData(url, data);
      break;
    case Magellan::BlockObjectAttribute :
      bRet = getBlockObjectAttributeData(url, data);
      break;
    case Magellan::Attribute :
      bRet = getAttributeData(url, data);
      break;
    case Magellan::Message :
      bRet = getMailData(url, data);
      break;
    case Magellan::Part :
      bRet = getPartData(url, data);
      break;
    case Magellan::BlockObject :
      bRet = getBlockObjectData(url,data);
      break;
    case Magellan::FileObject :
      bRet = getFileObjectData(url,data);
      break;
    default:
      qDebug("getObjectData() : Unknown type");
      err="Unknown type";
      break;
  }
  
  if( !bRet )
    data.resize(0);

  return bRet;
}

bool ObjectRequestBroker::getObjectData(const QStringList &urls, QByteArray &data)
{
  QDataStream tempStream(data, IO_WriteOnly);
  QByteArray  tempData;
  bool bRet=true;

  // Resize the buffer for better performance
  data.resize( 4096 * urls.count() );
  
  // First insert the number of objects
  Q_UINT32 urlsNumber = urls.count();
  tempStream << urlsNumber;
  
  if( !urls.count() )
  {
    qDebug("Nothing to sent!");
    return true;
  }
  
  tempStream << urls;
  
  for(QStringList::ConstIterator urlIt=urls.begin(); (urlIt != urls.end()) && bRet; ++urlIt)
  {
  // debug
//   qDebug("orb: entering getObjectsData(%s)", (const char *)(*urlIt));

    tempData.resize(0);
    
    int t = subtype((*urlIt));
    
    switch(t)
    {
      case Magellan::FolderCache :
        bRet=getFolderCacheData((*urlIt), tempData);
        break;
      case Magellan::FolderAttribute :
        bRet=getFolderAttributeData((*urlIt), tempData);
        break;
      case Magellan::MailAttribute :
        bRet=getMailAttributeData((*urlIt), tempData);
        break;
      case Magellan::PartAttribute :
        bRet=getPartAttributeData((*urlIt), tempData);
        break;
      case Magellan::BlockObjectAttribute :
        bRet=getBlockObjectAttributeData((*urlIt), tempData);
        break;
      case Magellan::Attribute :
        bRet=getAttributeData((*urlIt), tempData);
        break;
      case Magellan::Message :
        bRet=getMailData((*urlIt), tempData);
        break;
      case Magellan::Part :
        bRet=getPartData((*urlIt), tempData);
        break;
      case Magellan::BlockObject :
        bRet=getBlockObjectData((*urlIt), tempData);
        break;
      case Magellan::FileObject :
        bRet=getFileObjectData((*urlIt), tempData);
        break;
      default:
        err="Unknown error";
        bRet=false;
        break;
    }
    
    tempStream<<tempData;
  }

  return bRet;
}

bool ObjectRequestBroker::getChildrenData(const QString &url, QByteArray &data)
{
  QDataStream tempStream(data, IO_WriteOnly);

  // debug
//  qDebug("orb: entering getChildrenData(%s)", (const char *)url);

  QStringList urls;
  
  if( !url.isEmpty() && children(url, urls) )
  {
    if( !urls.count() )
    {
      tempStream << 0;
      return(true);
    }
    else
    {
      for(QStringList::Iterator it=urls.begin(); it!=urls.end(); it++)
        (*it).insert(0,url+"/");
      return getObjectData(urls, data);
    }
  }
  else
  {
    qDebug("Oops! getChildrenData() called with not such a folder name");
    err="url null OR not such a folder url";
    tempStream << 0;
    return false;
  }
}

bool ObjectRequestBroker::getFolderCacheData(const QString &url, QByteArray &data)
{
  MailFolder *mailFolder=mailfolderReference(folder(url));
  if(!mailFolder)
  {
    err="Cannot retrieve cache for non-mail folders";
    qDebug("orb: Warning, folder %s is not a mail folder, cannot retrieve cache", (const char *)folder(url));
    return false;
  }
  else
  {
    data=_stream(mailFolder->entryCacheBlock());
    return true;
  }
}

bool ObjectRequestBroker::getFolderAttributeData(const QString &url, QByteArray &data)
{
  // debug
//  qDebug("orb:%s", url.latin1());
//  qDebug("orb: %s-statting folder %s", url.contains("fattr")?"fast ":"", folderpath(url).latin1());

  QString folder = folderpath(url);
  _FolderAttribute fattr;
  
  if( type(url) == Magellan::Message)
  {
    MailFolder * mfolder = mailfolderReference(folder);
    
    if( mfolder )
    {
      if(url.contains(QString(ATTR_SEP)+"attr"))
      {
        QFileInfo finfo(mfolder->getStorageDevice());
  
        fattr.owner=finfo.owner();
        fattr.device=mfolder->getStorageDevice();
        fattr.rtype="indexed,extended:(data descriptors,multipart storage,uniblock storage,incremental data change,incremental index change,data relocation)";
        fattr.size=finfo.size();
        fattr.lmodified=finfo.lastModified(); 
        fattr.folders=0; // no recursive folders
      }
          
      fattr.items=mfolder->getMessageCount();
      fattr.recent=mfolder->getUnread();
    }

    data=_stream(fattr);
    
    return true;
  }
  else
  {
    // debug
    // qDebug("orb: this is a generic folder, proceeding");

    DataCollection *dfolder=folderReference(folder);
    if( dfolder )
    {
      if(url.contains(QString(ATTR_SEP)+"attr"))
      {
        QFileInfo finfo(dfolder->storageDevice);
  
        fattr.owner=finfo.owner();
        fattr.device=dfolder->storageDevice;
        fattr.device=dfolder->type==DataCollection::FileStorage?"file":"folder";
        fattr.rtype="sequential";
        fattr.size=finfo.size();
        fattr.lmodified=finfo.lastModified();
        fattr.folders=0; // no recursive folders
      }

      fattr.items = dfolder->getEntryList().count();
      fattr.recent = 0;
    }

    data = _stream(fattr);
    
    return true;
  }
}

bool ObjectRequestBroker::getMailAttributeData(const QString &url, QByteArray &data)
{
  QString objectUrl=url.left(url.find(ATTR_SEP));
  _MailAttribute attr;
  IndexClass *idx=indexReference(objectUrl);

  if(!idx)
  {
    err="Error getting the mail attribute data: object does not exist";
    
    qDebug("Warning: %s", (const char *)err);
    
    return false;
  }
  
  if(url.contains(QString(ATTR_SEP)+"attr"))
  {
    QFileInfo finfo(idx->getParentFolder()->getStorageDevice()+"/messages");

    attr.owner=finfo.owner();
    attr.device=idx->getParentFolder()->getStorageDevice()+"/messages";
    attr.device="file";
    attr.rtype="indexed,extended:(data descriptors,multipart storage,uniblock storage,incremental data change,incremental index change,data relocation)";
    attr.size=finfo.size();
    attr.lmodified=finfo.lastModified();
  }

  attr.uniblockOffset=idx->getUniblockOffset();
  attr.uniblockLength=idx->getUniblockLength();
  attr.descriptorOffset=idx->getDescriptorOffset();
  attr.descriptorLength=idx->getDescriptorLength();
  attr.partCount=idx->getPartCount();
  attr.multipartOnly=idx->isMultipartOnly();

  data=_stream(attr);
  return true;
}

bool ObjectRequestBroker::getPartAttributeData(const QString &url, QByteArray &data)
{
  QString objectUrl=url.left(url.find(ATTR_SEP));
  _PartAttribute attr;
  IndexClass *idx=indexReference(objectUrl);

  if(!idx)
  {
    err="Error getting the part attribute data: object does not exist";
    
    qDebug("Warning: %s", (const char *)err);
    
    return false;
  }
  
  if(url.contains(QString(ATTR_SEP)+"attr"))
  {
    QFileInfo finfo(idx->getParentFolder()->getStorageDevice()+"/messages");

    attr.owner=finfo.owner();
    attr.device=idx->getParentFolder()->getStorageDevice()+"/messages";
    attr.device="file";
    attr.rtype="indexed,extended:(data descriptors,multipart storage,uniblock storage,incremental data change,incremental index change,data relocation)";
    attr.size=finfo.size();
    attr.lmodified=finfo.lastModified();
  }

  // .fattr
  int pBegin=url.find(ATTR_SEP)+1, pEnd=url.findRev(ATTR_SEP);
  MimePart *part=idx->getPartAt(url.mid(pBegin, pEnd-pBegin).toInt());

  attr.mimetype = part->mimetype;
  attr.name = QString::fromUtf8(part->name);
  attr.cid = part->cid;
  attr.encoding = part->encoding;
  attr.charset = part->charset;
  attr.mtype = part->type;
  attr.partOffset = part->offset;
  attr.partLength = part->length;
  attr.embeddedPath = part->embeddedPath;

  data=_stream(attr);
  return true;
}

bool ObjectRequestBroker::getBlockObjectAttributeData(const QString &url, QByteArray &data)
{
  QString objectUrl=url.left(url.find(ATTR_SEP));
  _BlockObjectAttribute attr;
  ObjectReference *ref=objectReference(objectUrl);
  DataCollection *dfolder=folderReference(folderpath(url));

  if(!ref || !dfolder)
  {
    err="Error getting the block object attribute data: object does not exist";
    
    qDebug("Warning: %s", (const char *)err);
    
    return false;
  }
    
  if(url.contains(QString(ATTR_SEP)+"attr"))
  {
    QFileInfo finfo(dfolder->storageDevice);

    attr.owner=finfo.owner();
    attr.device=dfolder->storageDevice;
    attr.device="file";
    attr.rtype="sequential";
    attr.size = ref->size;
    attr.lmodified=ref->lastModified;
  }

  data=_stream(attr);
  return true;
}

bool ObjectRequestBroker::getBlockObjectData(const QString &url, QByteArray &data)
{
  QString objectUrl = url.left(url.find(ATTR_SEP));
  ObjectReference * ref = objectReference(objectUrl);
  
  if(!ref)
    return false;
    
  ref->data( data );
  
  return true;
}

bool ObjectRequestBroker::getFileObjectData(const QString &url, QByteArray &data)
{
  QString objectUrl=url.left(url.find(ATTR_SEP));
  ObjectReference *ref=objectReference(objectUrl);
  
  if(!ref)
    return false;
    
  ref->data( data );
  
  return true;
}

bool ObjectRequestBroker::getAttributeData(const QString &url, QByteArray &data)
{
  // debug
  // qDebug("orb: attribute query in progress");

  QString objectUrl=url.left(url.find(ATTR_SEP));
  _Attribute attr;

  // debug
  // qDebug("orb: %s is a file attribute, stat follows", (const char *)url);

  ObjectReference *ref=objectReference(objectUrl);

  // debug
  // qDebug("orb: object reference: %p", ref);
  // qDebug("orb: filename: %s", (const char *)(folderReference(folderpath(objectUrl))->storageDevice+"/"+ref->filename));

  if(!ref)
  {
    err="Error getting the object attribute data: object does not exist";
    
    qDebug("Warning: %s", (const char *)err);
    
    return false;
  }
  
  if(url.contains(QString(ATTR_SEP)+"attr"))
  {
    QFileInfo finfo(folderReference(folderpath(objectUrl))->storageDevice+"/"+ref->filename);

    attr.owner=finfo.owner();
    attr.device=folderReference(folderpath(objectUrl))->storageDevice+"/"+ref->filename;
    attr.dtype="file, extended:(standalone data)";
    attr.rtype="random";
    attr.size=finfo.size();
    attr.lmodified=finfo.lastModified();
  }

  // qDebug("orb: got object attribute; owner:%s", (const char *)attr.owner);

  data=_stream(attr);
  return true;
}

bool ObjectRequestBroker::getMailData(const QString &url, QByteArray &data)
{
  // extract query
  QString property, objectUrl=object(url);
  if(url.contains('%'))
  {
    property=url.mid(url.find('%')+1).lower();
    objectUrl=url.left(url.findRev('%'));
    return getMailPropertyData(objectUrl, property, data);
  }

  IndexClass *idx=indexReference(url);
  if(!idx)
  {
    qDebug("Warning: indexReference() returned null when a vadid pointer was expected, bailing out");
    err="Internal error in URLManager::indexReference()";
    return false;
  }

// qDebug("orb: getting message descriptor for object %s, index %s(%p)...", (const char *)url, (const char *)idx->getID(), idx);
  
  QDataStream stream(data, IO_WriteOnly);
  
// qDebug("orb: index parent name is %s", (const char *)idx->getParentFolder()->name());
// qDebug("orb: index message file is %s", (const char *)idx->getDataFilename());
  
//  MessageDevice *dev=MessageFactory::thisInstance()->getMessage(idx);
  MessageDevice * dev = new MessageDevice(idx);
  if(dev)
  {
    dev->loadDescriptor(false);
    stream<<dev->getDescriptor();
    delete dev;
    dev = 0L;
  }
  else
  {
    qDebug("Warning: MessageFactory::getMessage() returned null when a valid pointer was expected, bailing out");
    err="Internal error in MessageFactory::getMessage()";
    return false;
  }
  
  return true;
}

bool ObjectRequestBroker::getMailPropertyData(const QString &path, const QString &property, QByteArray &data)
{
  IndexClass *idx=indexReference(path);
  if(!idx)
  {
    qDebug("Warning: indexReference() returned null when a vadid pointer was expected, bailing out");
    err="Internal error in URLManager::indexReference()";
    return false;
  }

//  qDebug("Property=%s", property.latin1());

  // check if it is an IMAP folder
//  if( property!="size" && property!="hasattachments")
  if( property=="text" || property=="html" ||
      property=="alltext" || property=="allhtml" )
    if( MailFolder::IMAP4==idx->getParentFolder()->getFolderProtocol() && 
        !idx->isFullMessage() &&
        !idx->isLocked() )
      IMAP4Handler::ref()->updateMessage(*idx);

  MessageDevice dev(idx);

  if(property=="text")
  {
    data = _stream( dev.text() );
  }
  else if(property=="alltext")
  {
    data = _stream( dev.text(true) );
  }
  else if(property=="html")
  {
    data = _stream( dev.html() );
  }
  else if(property=="allhtml")
  {
    data = _stream( dev.html(true) );
  }
  else if(property=="rfc822header")
  {
    data=dev.rfc822Header();
  }
  else if(property=="rfc822message")
  {
    data=dev.rfc822Message();
  }
  else if(property=="textpart")
  {
    data=_stream(dev.textPart());
  }
  else if(property=="messagetext")
  {
    data=_stream(dev.messageText());
  }
  else if(property=="htmlpart")
  {
    data=_stream(dev.htmlPart());
  }
  else if(property=="partcount")
  {
    data=_stream(idx->getPartCount());
  }
  else if(property=="hashtml")
  {
    data=_stream(dev.hasHtml());
  }
  else if(property=="hastext")
  {
    data=_stream(dev.hasText());
  }
  else if(property=="hasattachments")
  {
    data=_stream(dev.hasAttachments());
  }
  else if(property=="size")
  {
    data=_stream(idx->getSize());
  }
  else
  {
    // debug
    qDebug("orb: warning: unknown message query (%s) on object %s", (const char *)property, (const char *)path);
    
    err="Unknown message query: "+property;
    return false;
  }
  
  return true;
}

bool ObjectRequestBroker::getPartData(const QString &url, QByteArray &data)
{
  QString objectUrl=url.left(url.findRev(ATTR_SEP));
  IndexClass *idx=indexReference(objectUrl);

  if(!idx)
  {
    qDebug("Warning: indexReference() returned null when a vadid pointer was expected, bailing out");
    err="Internal error in URLManager::indexReference()";
    return false;
  }

  int k=url.findRev(ATTR_SEP);
  int p=url.mid(k+1).toInt();
  if(p<0 || p>idx->getPartCount())
  {
    err="Non-existent part requested";
    return false;
  }

  MessageDevice dev(idx);
  dev.getPartData(p, data);
  
  return true;
}

bool ObjectRequestBroker::getCollectionObjectData(const QString &url, QByteArray &data)
{
  ObjectReference * tobj = objectReference(url);
  
  if(tobj)
  {
    // debug
    // qDebug("orb: narrowed object reference (%p)", tobj);
    // qDebug("orb: object data size %d", tobj->rawData.size());

    tobj->data( data );
    
    return true;
  }

  qDebug("Warning: objectReference() returned null when a valid pointer was expected, bailing out");
  err="Internal error in URLManager::objectReference()";
  return false;
}


int ObjectRequestBroker::getFolderRecentData(const QString & folderURL)
{
  MailFolder * folder = mailfolderReference(folderURL);
  if( folder )
  {
    return folder->getUnread();
  }
  
  return 0;
}

