/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <qfileinfo.h>
#include <qdatetime.h>

#include <orb.h>
#include <magellan.h>
#include <indexclass.h>
#include <transactionserver.h>
#include <attributemarshall.h>
#include <conversions.h>
#include <messagedescriptor.h>
#include <messagedevice.h>
#include <messagefactory.h>
#include <servernotifier.h>
#include <accounts.h>
#include <accountmanager.h>
#include <mimecodec.h>

bool ObjectRequestBroker::setObjectData(const QString &url, const QByteArray &data)
{
  // debug
  // qDebug("orb: setObjectData called on %s, data size %d", (const char *)url, data.size());
  
  if(!exists(url))
  {
    // debug
    qDebug("orb: warning: object %s does not exist", (const char *)url);
    
    err="Non-existent object or unknown protocol";
    return false;
  }
  else if(isFolder(url))
  {
    // debug
    qDebug("orb: warning: setObjectData can't be applied on folders (%s)", (const char *)url);
    
    err="Can't apply setObjectData() on a folder";
    return false;
  }
  else if(url.contains(QString(ATTR_SEP)+"attr") || url.contains(QString(ATTR_SEP)+"fattr"))
  {
    // debug
    qDebug("orb: this is an attribute object");
      
    if(subtype(url)==Magellan::PartAttribute)
      return setPartAttributeData(url, data);
    
    err="Cannot modify a read-only attribute object";
    return false;
  }
  else if(type(url)==Magellan::Message)
  {
    if(subtype(url)==Magellan::Part)
    {
      QString object=url.left(url.find(ATTR_SEP));
      int part=url.mid(url.find(ATTR_SEP)+1).toInt();
      return setPartData(object, part, data);
    }
    else
    {
      return setMailData(url, data);
    }
  }
  else
  {
    return setCollectionObjectData(url, data);
  }
}

bool ObjectRequestBroker::setCollectionObjectData(const QString &url, const QByteArray &data)
{
//  qDebug("ObjectRequestBroker::setCollectionObjectData(%s)", url.ascii());
  ObjectReference *objectRef=objectReference(url);
  if(!objectRef)
  {
    qDebug("Warning: objectReference() returned null when a valid pointer was expected, bailing out");
    err="Internal error in URLManager::objectReference()";
    return false;
  }

  // debug
  // qDebug("orb: setting data on collection object %s(%p), data size %d...", (const char *)url, objectRef, data.size());
  
  objectRef->setData( data );
  objectRef->dirty=true;
  TransactionServer::thisInstance()->save(objectRef);
//  qDebug("Calling ServerNotifier::objectChanged(%s)", url.ascii());
  ServerNotifier::thisInstance()->objectChanged(url);
  
  return true;
}

bool ObjectRequestBroker::setMailData(const QString &url, const QByteArray &data)
{

  IndexClass *index=indexReference(url);
  if(!index)
  {
    qDebug("setMailData %s", (const char *)url);
    qDebug("Warning: indexReference() returned null when a valid pointer was expected, bailing out");
    err="Internal error in URLManager::indexReference()";
    return false;
  }
  
  if(index->isLocked())
  {
    qDebug("Warning: message is locked, cannot write");
    err="Message was locked by the mail spooler and cannot be modified";
    return false;
  }
    
//  MessageDevice *dev=MessageFactory::thisInstance()->getMessage(index);
  MessageDevice * dev = new MessageDevice(index);
  
  if(!dev)
  {
    qDebug("Warning: MessageFactory::getMessage() returned null when a valid pointer was expected, bailing out");
    err="Internal error in MessageFactory::getMessage()";
    return false;
  }
  
  // debug
  // qDebug("orb: setting data on mail object %s(index:%p, device:%p), data size %d...", (const char *)url, index, dev, data.size());
  
  QDataStream stream(data, IO_ReadOnly);
  MessageDescriptor &descriptor=dev->getDescriptor();
  stream>>descriptor;
  
  bool triggerHierarchyUpdate=false;
  
  // set index unread mark
  bool oldUnreadMark=index->getUnreadMark();
  index->setUnreadMark(descriptor.status=="New");
  
  // set index parent id
  // ... not sure about implications
  // update descriptor relatedIndexIDList;
  // ... not sure about implications
  
  // set default account if needed
  if(url.find("sysmail_outbox")==0 || url.find("sysmail_drafts")==0)
  {
    Account *acc;
    if(descriptor.account.isEmpty())
    {
      acc=AccountManager::ref()->getDefaultAccount();
      if(acc) descriptor.account=acc->accname;
    }
    else
    {
      acc=AccountManager::ref()->getAccount(descriptor.account);
    }

    // set from address, reply-to and messageID based on the account
    if(acc)
    {
      descriptor.from="\""+acc->name+"\" <"+acc->from+">";
      if(!acc->reply.isEmpty()) descriptor.replyTo="\""+acc->name+"\" <"+acc->reply+">";
      descriptor.messageID= MIMECodec::encodeMessageId(descriptor.indexID.right(5), acc->name+QString("%1").arg(QDateTime::currentDateTime().toTime_t()) )
                            +"@"+acc->smtp;
    }
  }

  // set index message id
  index->setMessageID(descriptor.messageID);
  
  if(oldUnreadMark!=index->getUnreadMark())
  {
    if(index->getUnreadMark()) index->getParentFolder()->incrementUnread();
    else index->getParentFolder()->decrementUnread();
  }
  
  dev->saveDescriptor();
  delete dev;
  dev = 0L;
  
  // Update the view
  ServerNotifier::thisInstance()->objectChanged(index->getParentFolder()->name()+"/"+index->getID());
  
  if(triggerHierarchyUpdate)
    ServerNotifier::thisInstance()->hierarchyChanged(index->getParentFolder()->name()+"/"+index->getID());
  
  return true;
}

bool ObjectRequestBroker::setMailPropertyData(const QString & /* url */, const QString & /* property */, const QByteArray & /* data */)
{
  return false;
}

bool ObjectRequestBroker::setPartData(const QString &url, int part, const QByteArray &data)
{
  IndexClass *idx=indexReference(url);

  if(!idx)
  {
    // debug
    qDebug("orb: could not find index %s", (const char *)url);
    
    err="Internal error in URLManager::indexReference()";
    return false;
  }
  
  // debug
  // qDebug("orb: setting part data (%s.%d)", (const char *)url, part);
  
  err="Could not set part data: unknown error";
  MessageDevice *dev=new MessageDevice(idx);
  bool res=dev->setPartData(part, data);
  delete dev;
  
  return res;
}

bool ObjectRequestBroker::setPartAttributeData(const QString &url, const QByteArray &data)
{
  QString objectUrl=url.left(url.find(ATTR_SEP));
  _PartAttribute attr=_narrowPartAttribute(data);
  IndexClass *idx=indexReference(objectUrl);
  
  if(!idx)
  {
    // debug
    qDebug("orb: warning, setPartAttributeData could not find index %s", (const char *)objectUrl);
    
    err="Internal error in URLManager::indexReference()";
    return false;
  }

  int pBegin=url.find(ATTR_SEP)+1, pEnd=url.findRev(ATTR_SEP);
  MimePart *part=idx->getPartAt(url.mid(pBegin, pEnd-pBegin).toInt());
  
  // debug
   qDebug("orb: setting properties for part %d, index %s", url.mid(pBegin, pEnd-pBegin).toInt(), (const char *)idx->getID());

  part->mimetype=attr.mimetype;
  part->name=attr.name;
  part->cid=attr.cid;
  part->encoding=attr.encoding;
  part->charset=attr.charset;
  part->type=attr.mtype;
  
  idx->getParentFolder()->saveIndex(idx);
  
  return true;
}
