/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                remotemailfolder.cpp  -  the remote mail folder class
                             -------------------
    begin                : Mon Mar  5 16:16:00 EET 2001
    copyright            : (C) 2001 by theKompany (www.thekompany.com>
    author               : Eugen Constantinescu
    email                : eug@thekompany.com
 ***************************************************************************/

#ifndef REMOTEMAILFOLDER_H
#define REMOTEMAILFOLDER_H

#include <qstring.h>
#include <qcstring.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qdict.h>
#include <qlist.h>
#include <qmap.h>

#include <messageclass.h>
#include <mailfolder.h>
#include <imap4uidjar.h>

typedef struct IMAPStatus
{
  unsigned long messages;
  unsigned long recent;
  unsigned long unseen;
  unsigned long uidNext;
  unsigned long uidValidity;
} IMAP_STATUS;

class RemoteMailFolder : public MailFolder
{
  public:
    RemoteMailFolder(const QString &_storageDevice);
    RemoteMailFolder(const QString &_storageDevice, const QString account, const QString mailbox);
    virtual ~RemoteMailFolder();
   
    /**The headers file name.*/
    QString getMessagesFileName() const;
    /** Make the config file.*/
    void makeConfig(QString account, QString mailbox);
    /** Get the account name.*/
    QString account() const;
    /** Get the mailbox name.*/
    QString mailbox() const;
    
    /** Create a new message based on the header information.*/
    QString createFullMessage(const QCString &text, const QCString &uid, const QDateTime &rcvtime,
                              const QString &account=QString::null,
                              const MessageClass *parsedMessage = 0,
                              bool bSync=false, const unsigned flags=0);
    /** Create a new message based on the header information.*/
    QString createMessage(const QCString &text, const QCString &uid, const QDateTime &rcvtime,
                          const QString &account=QString::null,
                          const MessageClass *parsedMessage = 0,
                          bool bSync=false, const unsigned flags=0);
    
    bool updateMessage(const QString & text, const QString & indexURL);
    /** sync the messages from the list.*/
    void sync();
    
    /** Delete a message from the folder.*/
    virtual bool deleteMessage(IndexClass *);
    /** Delete a message from trash.*/
    virtual bool deleteTrashMessages();
    /** Delete the message from local folder.*/
    virtual bool deleteMessage(const QString&);
    /** Copy a message in this folder.*/
    virtual QString copyMessage(IndexClass *idx);
    /** Move a message in this folder.*/
    virtual QString moveMessage(IndexClass *idx);
    /** Expunge.*/
    virtual bool expunge(bool force=false);
    
    virtual bool moveMessage(const QStringList & urls);
    virtual bool copyMessage(const QStringList & urls);
    virtual bool deleteMessage(const QStringList & urls);
    
    // Status
    /** Messages number.*/
    unsigned long messages() const;
    /** The recent messages.*/
    unsigned long recent() const;
    /** The unseen messages.*/
    unsigned long unseen() const;
    /** The next uid message.*/
    unsigned long uidNext() const;
    /** The uid validity for the mailbox.*/
    unsigned long uidValidity() const;
    
    /** Set the folder status.*/
    void setStatus(unsigned long uidValidity,
                   unsigned long uidNext,
                   unsigned long messages,
                   unsigned long unseen,
                   unsigned long recent=0);
    
    /** Save the status to the file.*/
    bool saveStatus();
    
    /** Remove the status entries from Settings.*/
    bool removeStatus();
  
    //DEBUG
    void showStatus() const;
  
    // UIDs file
    IMAP4_UIDJar* uidJar;

  private:
    // Methods
    /** Set the files name.*/
    void setupFiles();
    /** Read the config file.*/
    void config();
    /** Load the index from the file.*/
    void loadIndex();
    /** Should expunge.*/
    bool shouldExpunge();
    /** Load the status from the file.*/
    bool loadStatus();
    
    // Attributes
    /**
      * The headers file name.
      * It keeps only the headers of the messages.
      */
    QString headersFileName;
    /** The config file name.*/
    QString configFileName;
    /** IMAP account name.*/
    QString accountName;
    /** Mailbox name.*/
    QString mailboxName;
    /** IMAP status.*/
    IMAP_STATUS status;
};

#endif





