/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/*
 * This file is part of Magellan <http://www.kAlliance.org/Magellan>
 *
 * Copyright (c) 1998-2000 Teodor Mihai <teddy@ireland.com>
 * Copyright (c) 1998-2000 Laur Ivan <laur.ivan@ul.ie>
 * Copyright (c) 1999-2000 Virgil Palanciuc <vv@ulise.cs.pub.ro>
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 * Also requires the KDE libraries, available at no cost at
 * http://www.kde.org/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <stdio.h>
#include <assert.h>

#include <serverrmistub.h>
#include <qstring.h>
#include <qstringlist.h>
#include <accountmanager.h>
#include <pop3handler.h>
#include <smtphandler.h>
#include <imap4handler.h>
#include <localmailfolder.h>
#include <qdatastream.h>

//Protocols includes
#include <iosynchandler.h>

QStringList ServerRMIStub::getAccountNames(int accountType)
{
  return AccountManager::ref()->getAccountNames(accountType);
}

QString ServerRMIStub::getDefaultAccount()
{
  Account * defaultAccount = AccountManager::ref()->getDefaultAccount();
  
  if (0 == defaultAccount)
    return QString::null;
  else
    return defaultAccount->accname;
}

bool ServerRMIStub::isAccountActive(QString account)
{
  Account *acc=AccountManager::ref()->getAccount(account);
  return acc?acc->enabled:false;
}

QString ServerRMIStub::getAccountFromAddress(QString account)
{
  Account *acc=AccountManager::ref()->getAccount(account);
  return acc?QString("\"")+acc->name+"\" "+ acc->from:QString("");
}

QString ServerRMIStub::getAccountReplyToAddress(QString account)
{
  Account *acc=AccountManager::ref()->getAccount(account);
  return acc?QString("\"")+acc->name+"\" "+ acc->reply:QString("");
}

void ServerRMIStub::startSendReceive(QString account)
{
  // debug
  qDebug("server: RMI call received: startSendReceive(%s)", (const char *)account);
  startReceive(account);
  startSend(account);
}

void ServerRMIStub::startSend(QString account)
{
  // debug
  qDebug("server: RMI call received: startSend(%s)", (const char *)account);
  Account *acc=AccountManager::ref()->getAccount(account);
  if(acc)
    SMTPHandler::ref()->startSend(acc);
  else
    qDebug("server: cannot start SMTPHandler, invalid account");
}

void ServerRMIStub::startReceive(QString account)
{
  // debug
  qDebug("server: RMI call received: startReceive(%s)", (const char *)account);
  
  Account *acc=AccountManager::ref()->getAccount(account);
  if(!acc)
    qDebug("server: cannot start receiving, invalid account (null)");
  else
  {
    switch(acc->type)
    {
      case Account::POP3 :
        POP3Handler::ref()->getMail(acc);
        break;
      case Account::IMAP4 :
//        IMAP4Handler::ref()->getMail(acc);
        break;
      case Account::UNDEFINE :
      case Account::MAILBOX :
      default :
        qDebug("server: cannot start receiving, invalid account type");
        break;
    }
  }
}

void ServerRMIStub::stopPOP3Delivery()
{
  POP3Handler::ref()->stopAllDeliveries();
}

void ServerRMIStub::setAutoExpunge(bool)
{
// TODO
//  MailFolder::setAutoExpunge(mode);
}

bool ServerRMIStub::mount(QString url)
{
  return IMAP4Handler::ref()->mount(url);
}

bool ServerRMIStub::unmount(QString url)
{
  return IMAP4Handler::ref()->unmount(url);
}

bool ServerRMIStub::subfolders(QString account, QString mailbox, QString url)
{
  return IMAP4Handler::ref()->subfolders(account, mailbox, url);
}

bool ServerRMIStub::runKolabJob(const QString & account, const QString & mailbox, const QString & state)
{
  return IMAP4Handler::ref()->runKolabJob(account, mailbox, state);
}

bool ServerRMIStub::runKolabSharedJob(const QString & account, const QString & mailbox, const QString & state)
{
  return IMAP4Handler::ref()->runKolabSharedJob(account, mailbox, state);
}

bool ServerRMIStub::rename(QString url, QString newName)
{
  return IMAP4Handler::ref()->rename(url, newName);
}

bool ServerRMIStub::expunge(QString url)
{
  return IMAP4Handler::ref()->expunge(url);
}

void ServerRMIStub::loadAccounts()
{
  AccountManager::ref()->loadAccounts();
}

bool ServerRMIStub::addAccount(QByteArray a)
{
  QDataStream str(a, IO_ReadOnly);

  Account * account = new Account;

  str >> *account;

  return AccountManager::ref()->addAccount(account);
}

bool ServerRMIStub::deleteAccount(QString accountName)
{
  return AccountManager::ref()->deleteAccount(accountName);
}

void ServerRMIStub::setDefaultAccount(QString accountName)
{
  AccountManager::ref()->setDefaultAccount(accountName);
}

QByteArray ServerRMIStub::getAccount(QString accountName)
{
  // Error-prone, eh ? That's the nature of streams, unfortunately.

  QByteArray a;

  QDataStream str(a, IO_WriteOnly);

  Account * account = AccountManager::ref()->getAccount(accountName);

  // We really need this to work, or demarshalling will fuck up completely
  // and crash us, so we'd prefer an abort, wouldn't we ?
  assert(account);

  str << *account;

  return a;
}

void ServerRMIStub::toggleAccountActive(QString accountName)
{
  Account * account = AccountManager::ref()->getAccount(accountName);

  if (0 == account)
  {
    qDebug("ServerRMIStub::toggleAccountActive: Can't find account");
    return;
  }

  account->enabled = !account->enabled;
  account->saveSettings();
}

void ServerRMIStub::saveAccount(QString accountName, QByteArray a)
{
  QDataStream str(a, IO_ReadOnly);
  Account * account = AccountManager::ref()->getAccount(accountName);

  if (0 == account)
  {
    qDebug("ServerRMIStub::saveAccount: Can't find account");
    return;
  }
  
  str >> *account;

  account->saveSettings();
}

bool ServerRMIStub::importMessages( const QString& toFolder )
{
  return IOSyncHandler::ref()->importMessages( toFolder );
}

void ServerRMIStub::saveAccounts()
{
  AccountManager::ref()->saveAccounts();
}

