/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include <qlineedit.h>
#include <qspinbox.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qtoolbutton.h>
#include <qfiledialog.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qfont.h>
#include <qsize.h>
#include <qpoint.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qsettings.h>
#include <qsettingsextra.h>
#include <qcheckbox.h>
#include <qpushbutton.h>

#include <accounts.h>
#include <mimecodec.h>
#include <setupwizard.h>

#include <pop3configwidget.h>
#include <imap4configwidget.h>
#include <smtpconfigwidget.h>
#include <kolabfoldersbase.h>
#include <qiconloader.h>

#include <defines.h>
#include <settings.h>

SetupWizard::SetupWizard(QWidget * parent, const char * name)
  : QWizard                     (parent, name, true /* modal */),
    nameLineEdit_               (0),
    orgLineEdit_                (0),
    addressLineEdit_            (0),
    replyToLineEdit_            (0),
    localMailboxLineEdit_       (0),
    localMailboxToolButton_     (0),
    pluginDirLineEdit_          (0),
    pluginDirToolButton_        (0),
    serverTypeButtonGroup_      (0),
    reviewLabel_                (0),
    personalSetupPage_          (0),
    aetheraDirSetupPage_        (0),
    serverTypeSetupPage_        (0),
    popSetupPage_               (0),
    imapSetupPage_              (0),
    localSetupPage_             (0),
    smtpSetupPage_              (0),
    reviewPage_                 (0),
    finishedPage_               (0)
{
  setCaption(tr("%1 - Setup").arg(FullAppName));

  // Page widgets.

  personalSetupPage_          = new QWidget(this);
  aetheraDirSetupPage_        = new QWidget(this);
  serverTypeSetupPage_        = new QWidget(this);
  popSetupPage_               = new POP3ConfigWidget(this);
  imapSetupPage_              = new IMAP4ConfigWidget(this);
  kolabSetupPage_             = new KolabFoldersBase(this);
  localSetupPage_             = new QWidget(this);
  smtpSetupPage_              = new SMTPConfigWidget(this);
//  reviewPage_                 = new QWidget(this);
  finishedPage_               = new QWidget(this);

  // Personal page.

  QLabel * nameLabel    = new QLabel(tr("Full &name:"),     personalSetupPage_);
  QLabel * orgLabel     = new QLabel(tr("&Organisation:"),  personalSetupPage_);
  QLabel * addressLabel = new QLabel(tr("&Email Address:"), personalSetupPage_);
  QLabel * replyToLabel = new QLabel(tr("&Reply to:"),      personalSetupPage_);

  nameLineEdit_     = new QLineEdit(personalSetupPage_);
  orgLineEdit_      = new QLineEdit(personalSetupPage_);
  addressLineEdit_  = new QLineEdit(personalSetupPage_);
  replyToLineEdit_  = new QLineEdit(personalSetupPage_);

  nameLabel     ->setBuddy(nameLineEdit_);
  orgLabel      ->setBuddy(orgLineEdit_);
  addressLabel  ->setBuddy(addressLineEdit_);
  replyToLabel  ->setBuddy(replyToLineEdit_);

  // Setting the storage and plugins directories.

  QLabel * aetheraDirLabel =
    new QLabel(tr("Directory for storing the data:"), aetheraDirSetupPage_);

#ifdef _QTWIN_
  QString aetheraDir = QDir::homeDirPath() + "\\Aethera";
  aetheraDir = QDir::cleanDirPath( aetheraDir );
  aetheraDir = QDir::convertSeparators( aetheraDir );
#else
  QString aetheraDir = QDir::homeDirPath() + "/Aethera";
  aetheraDir = QDir::cleanDirPath( aetheraDir );
#endif

  aetheraDirLineEdit_= new QLineEdit(aetheraDirSetupPage_);
  aetheraDirLineEdit_->setText(aetheraDir);
  aetheraDirToolButton_  = new QToolButton(aetheraDirSetupPage_);
  aetheraDirToolButton_->setIconSet(BarIcon("hi16-filesys-folder.png"));

  QLabel * pluginDirLabel =
    new QLabel(tr("Directory for downloaded Aethera plugins:"), aetheraDirSetupPage_);

  pluginDirLineEdit_    = new QLineEdit(aetheraDirSetupPage_);
  pluginDirToolButton_  = new QToolButton(aetheraDirSetupPage_);
  pluginDirToolButton_->setIconSet(BarIcon("hi16-filesys-folder.png"));

  // Server type page.

  serverTypeButtonGroup_  =
    new QButtonGroup
    (
      3,
      Vertical,
      tr("Incoming mailbox location"),
      serverTypeSetupPage_
    );

  QRadioButton * popRadioButton =
    new QRadioButton(tr("&POP3 server"), serverTypeButtonGroup_);

  QRadioButton * imapRadioButton =
    new QRadioButton(tr("&IMAP4 server"), serverTypeButtonGroup_);

  QRadioButton * citadelRadioButton =
    new QRadioButton(tr("&Citadel server"), serverTypeButtonGroup_);

  QRadioButton * kolabRadioButton =
    new QRadioButton(tr("&Kolab server"), serverTypeButtonGroup_);

  serverTypeButtonGroup_->insert(popRadioButton,    Account::POP3);
  popRadioButton->setChecked(true);
  serverTypeButtonGroup_->insert(imapRadioButton,   Account::IMAP4);
  serverTypeButtonGroup_->insert(citadelRadioButton,  Account::GROUPWARE);
  serverTypeButtonGroup_->insert(kolabRadioButton,  Account::GROUPWARE);

  // Local page.
  QLabel * localMailboxLabel =
    new QLabel(tr("Local mailbox &location"), localSetupPage_);

  localMailboxLineEdit_   = new QLineEdit(localSetupPage_);
  localMailboxToolButton_ = new QToolButton(localSetupPage_);
  localMailboxToolButton_->setIconSet(BarIcon("hi16-filesys-folder.png"));

  localMailboxLabel->setBuddy(localMailboxLineEdit_);

  // Send page. - Form !

  // Review page.

//  reviewLabel_ = new QLabel(reviewPage_);

  // Finished page.

  finishedLabel_ = new QLabel
    (
      tr
      (
        "<qt>"
        "<p>Thankyou for providing the information needed to get started.</p>"
        "<p>Have fun using %1</p>"
        "</qt>"
      ).arg(FullAppName),
      finishedPage_
    );

  addPage(personalSetupPage_,   tr("Personal information"));
  addPage(aetheraDirSetupPage_, tr("Aethera directories"));
  addPage(serverTypeSetupPage_, tr("Incoming mail"));
  addPage(popSetupPage_,        tr("POP server configuration"));
  addPage(imapSetupPage_,       tr("IMAP server configuration"));
  addPage(kolabSetupPage_,      tr("Kolab folders configuration"));
  addPage(localSetupPage_,      tr("Local mailbox configuration"));
  addPage(smtpSetupPage_,       tr("Outgoing mail"));
// Leave it out for now. I don't think it's necessary.
//  addPage(reviewPage_,          tr("Review settings"));
  addPage(finishedPage_,        tr("Finished"));

  // Layout for personal page.

  QVBoxLayout * personalPageLayout = new QVBoxLayout(personalSetupPage_);

  QGridLayout * personalPageGrid = new QGridLayout(personalPageLayout);

  personalPageGrid->addWidget(nameLabel,        0, 0);
  personalPageGrid->addWidget(nameLineEdit_,    0, 1);
  personalPageGrid->addWidget(orgLabel,         1, 0);
  personalPageGrid->addWidget(orgLineEdit_,     1, 1);
  personalPageGrid->addWidget(addressLabel,     2, 0);
  personalPageGrid->addWidget(addressLineEdit_, 2, 1);
  personalPageGrid->addWidget(replyToLabel,     3, 0);
  personalPageGrid->addWidget(replyToLineEdit_, 3, 1);

  personalPageLayout->addStretch(1);

  // Layout for plugin dir page.

  QVBoxLayout * aetheraDirPageLayout = new QVBoxLayout(aetheraDirSetupPage_);

  aetheraDirPageLayout->addWidget(aetheraDirLabel);
  
  QHBoxLayout * aetheraDirLayout = new QHBoxLayout(aetheraDirPageLayout);

  aetheraDirLayout->addWidget(aetheraDirLineEdit_);
  aetheraDirLayout->addWidget(aetheraDirToolButton_);
  
  aetheraDirPageLayout->addWidget(pluginDirLabel);

  QHBoxLayout * dirLayout = new QHBoxLayout(aetheraDirPageLayout);

  dirLayout->addWidget(pluginDirLineEdit_);
  dirLayout->addWidget(pluginDirToolButton_);

  aetheraDirPageLayout->addStretch(1);

  // Layout for server type page.

  QVBoxLayout * serverPageLayout = new QVBoxLayout(serverTypeSetupPage_);

  serverPageLayout->addWidget(serverTypeButtonGroup_);

  serverPageLayout->addStretch(1);

  // Layout for POP page. - Form.
  // Layout for IMAP page. - Form.
  // Layout for Kolab page.

  QVBoxLayout * localPageLayout = new QVBoxLayout(localSetupPage_);

  QHBoxLayout * localPageLocationLayout = new QHBoxLayout(localPageLayout);

  localPageLocationLayout->addWidget(localMailboxLabel);
  localPageLocationLayout->addWidget(localMailboxLineEdit_);
  localPageLocationLayout->addWidget(localMailboxToolButton_);

  localPageLayout->addStretch(1);

  // Layout for review page.

//  QVBoxLayout * reviewPageLayout = new QVBoxLayout(reviewPage_);

//  reviewPageLayout->addWidget(reviewLabel_);

  // Layout for finished page.

  QVBoxLayout * finishedPageLayout = new QVBoxLayout(finishedPage_);

  finishedPageLayout->addWidget(finishedLabel_);

  // Connections.

  connect
    (
      nameLineEdit_,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotPersonalPageMaybeReady())
    );

  connect
    (
      addressLineEdit_,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotPersonalPageMaybeReady())
    );

  connect
    (
      pluginDirLineEdit_,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotDirPageMaybeReady())
    );

  connect
    (
      aetheraDirToolButton_,
      SIGNAL(clicked()),
      SLOT(slotChooseAetheraDir())
    );

  connect
    (
      pluginDirToolButton_,
      SIGNAL(clicked()),
      SLOT(slotChoosePluginDir())
    );

  connect
    (
      serverTypeButtonGroup_,
      SIGNAL(clicked(int)),
      SLOT(slotServerTypePageReady())
    );

  connect
    (
      popSetupPage_->serverLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotPOPPageMaybeReady())
    );

  connect
    (
      popSetupPage_->usernameLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotPOPPageMaybeReady())
    );

  connect
    (
      popSetupPage_->passwordLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotPOPPageMaybeReady())
    );

  connect
    (
      imapSetupPage_->serverLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotIMAPPageMaybeReady())
    );

  connect
    (
      imapSetupPage_->usernameLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotIMAPPageMaybeReady())
    );

  connect
    (
      imapSetupPage_->passwordLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotIMAPPageMaybeReady())
    );

  connect
    (
      smtpSetupPage_->addressLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotSMTPPageMaybeReady())
    );

  connect
    (
      localMailboxLineEdit_,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotLocalPageMaybeReady())
    );

  connect
    (
      localMailboxToolButton_,
      SIGNAL(clicked()),
      SLOT(slotChooseLocalMailboxLocation())
    );

  connect
    (
      kolabSetupPage_->notesLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotKolabPageMaybeReady())
    );

  connect
    (
      kolabSetupPage_->todoLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotKolabPageMaybeReady())
    );

  connect
    (
      kolabSetupPage_->calendarLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotKolabPageMaybeReady())
    );

  connect
    (
      kolabSetupPage_->contactsLineEdit,
      SIGNAL(textChanged(const QString &)),
      SLOT(slotKolabPageMaybeReady())
    );

  setNextEnabled(personalSetupPage_,    false);
  setNextEnabled(aetheraDirSetupPage_,  false);
  setNextEnabled(serverTypeSetupPage_,  false);
  setNextEnabled(popSetupPage_,         false);
  setNextEnabled(imapSetupPage_,        false);
  setNextEnabled(kolabSetupPage_,       true); // has default folders
  setNextEnabled(localSetupPage_,       false);
  setNextEnabled(smtpSetupPage_,        true); // Has 'localhost' as default.

  setAppropriate(popSetupPage_,     false);
  setAppropriate(imapSetupPage_,    false);
  setAppropriate(localSetupPage_,   false);
  
  setFinishEnabled(finishedPage_, true);
}

SetupWizard::~SetupWizard()
{
}

  void
SetupWizard::slotPersonalPageMaybeReady()
{
  setNextEnabled
    (
      personalSetupPage_,
      !nameLineEdit_->text().isEmpty() && !addressLineEdit_->text().isEmpty()
    );
}

  void
SetupWizard::slotDirPageMaybeReady()
{
  setNextEnabled
    (
      aetheraDirSetupPage_,
      !aetheraDirLineEdit_->text().isEmpty() && 
      !pluginDirLineEdit_->text().isEmpty()
    );
}

  void
SetupWizard::slotChooseAetheraDir()
{
  QString location =
    QFileDialog::getExistingDirectory(QString::null, this);

  if (!location.isEmpty())
  {
    aetheraDirLineEdit_->setText(location);
  }
}

  void
SetupWizard::slotChoosePluginDir()
{
  QString location =
    QFileDialog::getExistingDirectory(QString::null, this);

  if (!location.isEmpty())
  {
    pluginDirLineEdit_->setText(location);
    setNextEnabled(aetheraDirSetupPage_, true);
  }
}

  void
SetupWizard::slotServerTypePageReady()
{
  if (0 == serverTypeButtonGroup_->selected())
  {
    setNextEnabled(serverTypeSetupPage_, false);
    return;
  }

  setAppropriate(popSetupPage_,     false);
  setAppropriate(imapSetupPage_,    false);
  setAppropriate(kolabSetupPage_,   false);

  switch (serverTypeButtonGroup_->id(serverTypeButtonGroup_->selected()))
  {
    case Account::POP3:
      setAppropriate(popSetupPage_, true);
      break;

    case Account::IMAP4:
      setAppropriate(imapSetupPage_, true);
      break;

    case Account::GROUPWARE:
      setAppropriate(imapSetupPage_, true);
      setAppropriate(kolabSetupPage_, true);
      break;

    case Account::MAILBOX:
      setAppropriate(localSetupPage_, true);
      break;

    default:
      qDebug("BUG");
      return;
  }

  setNextEnabled(serverTypeSetupPage_, true);
}

  void
SetupWizard::slotPOPPageMaybeReady()
{
  setNextEnabled
    (
      popSetupPage_,
      !popSetupPage_->serverLineEdit->text().isEmpty()
      && 
      !popSetupPage_->usernameLineEdit->text().isEmpty()
      &&
      !popSetupPage_->passwordLineEdit->text().isEmpty()
    );
}

  void
SetupWizard::slotKolabPageMaybeReady()
{
  setNextEnabled
    (
      kolabSetupPage_,
      !kolabSetupPage_->notesLineEdit->text().isEmpty()
      &&
      !kolabSetupPage_->contactsLineEdit->text().isEmpty()
      &&
      !kolabSetupPage_->todoLineEdit->text().isEmpty()
      &&
      !kolabSetupPage_->calendarLineEdit->text().isEmpty()
    );
}

  void
SetupWizard::slotIMAPPageMaybeReady()
{
  setNextEnabled
    (
      imapSetupPage_,
      !imapSetupPage_->serverLineEdit->text().isEmpty()
      &&
      !imapSetupPage_->usernameLineEdit->text().isEmpty()
      &&
      !imapSetupPage_->passwordLineEdit->text().isEmpty()
    );
}

  void
SetupWizard::slotLocalPageMaybeReady()
{
  setNextEnabled
    (
      localSetupPage_,
      !localMailboxLineEdit_->text().isEmpty()
    );
}

  void
SetupWizard::slotChooseLocalMailboxLocation()
{
  QString location =
    QFileDialog::getOpenFileName(QString::null, QString::null, this);

  if (!location.isEmpty())
  {
    localMailboxLineEdit_->setText(location);
    setNextEnabled(localSetupPage_, true);
  }
}

  void
SetupWizard::slotSMTPPageMaybeReady()
{
  bool canProceed = !smtpSetupPage_->addressLineEdit->text().isEmpty();

  setNextEnabled(smtpSetupPage_, canProceed);

  if (canProceed)
  {
//    reviewLabel_->setText("Review this !");
  }
}

  void
SetupWizard::accept()
{
  QString aetheraDir = QDir::homeDirPath() + "/Aethera";

  // These files are used only for IMAP
  QString serverPath;
  QString imapVFSFolder;

  if( !aetheraDirLineEdit_->text().isEmpty() )
    aetheraDir = aetheraDirLineEdit_->text();
  aetheraDir = QDir::cleanDirPath( aetheraDir );
  aetheraDir = QDir::convertSeparators( aetheraDir );
  
  Settings::instance()->writeEntry
    (
      "/Aethera/Directory/AetheraDir",
      aetheraDir
    );

  Settings::instance()->writeEntry
    (
      "/Aethera/PlugInManager/PlugInDirectory",
      pluginDirLineEdit_->text()
    );

  Settings::instance()->writeEntry
    (
      "/Aethera/PlugInManager/PlugIns", 
      "homeplugin,tkcWhiteboard,tkcJabber,korganizer,mailimport,komnotesplugin,komcontactsplugin,kommailplugin"
    );

  // Appearance.

  Settings::instance()->writeEntry("/Aethera/Appearance/Browse", "200");
  Settings::instance()->writeEntry("/Aethera/Appearance/Contents", "700");
  Settings::instance()->writeEntry("/Aethera/Appearance/Geometry", "161,133,900,500");
  Settings::instance()->writeEntry("/Aethera/Appearance/Style", "kde");

  // Contacts.

  Settings::instance()->writeEntry("/Aethera/Contacts/View mode", "tile");
  Settings::instance()->writeEntry("/Aethera/Contacts/View columns", "Name:fn;E-mail:email;Work address:work address;Mobile phone:mobile");
  Settings::instance()->writeEntry("/Aethera/Contacts/View fields", "name;email;work address");

  // Composer options.

  Settings::instance()->writeEntry("/Aethera/Composer/Bcc", true);
  Settings::instance()->writeEntry("/Aethera/Composer/QuoteChar", ">");
  Settings::instance()->writeEntry("/Aethera/Composer/Wrap", true);
  QSettingsExtra::writeEntry
    (Settings::instance(), "/Aethera/Composer/Font", QFont("courier", 12));

  // Misc. mail options.

  Settings::instance()->writeEntry("/Aethera/Misc Mail/Attach VCard", false);
  Settings::instance()->writeEntry("/Aethera/Misc Mail/Goto", "Unread");
  Settings::instance()->writeEntry("/Aethera/Misc Mail/Permanent delete", false);

  // Threading.
  Settings::instance()->writeEntry("/Aethera/Threading/Enable", false);
  Settings::instance()->writeEntry("/Aethera/Threading/Complete references", false);
  Settings::instance()->writeEntry("/Aethera/Threading/Thread by", "References");
  Settings::instance()->writeEntry("/Aethera/Threading/Thread anyway", true);

  // notes options
  QColor noteBack( Qt::yellow );
  QSettingsExtra::writeEntry
    (Settings::instance(), "/Aethera/Notes/Default color", noteBack);
  QSettingsExtra::writeEntry
    (Settings::instance(), "/Aethera/Notes/Default size", QSize(200, 200));
  QSettingsExtra::writeEntry
    (Settings::instance(), "/Aethera/Notes/Default font", QFont("helvetica", 12));

  // groups pixmaps association
  Settings::instance()->writeEntry("/Aethera/Group Icons/Default pixmap", "xpm/groups/contacts-group-large.xpm");

  // quote options
  Settings::instance()->writeEntry("/Aethera/Quote Options/Compression Treshold", 3);
  Settings::instance()->writeEntry("/Aethera/Quote Options/HTML Beautify", true);
  Settings::instance()->writeEntry("/Aethera/Quote Options/HTML Compression", true);
  Settings::instance()->writeEntry("/Aethera/Quote Options/HTML Max Second Line Length", 55);
  Settings::instance()->writeEntry("/Aethera/Quote Options/HTML Quotation", true);
  Settings::instance()->writeEntry("/Aethera/Quote Options/HTML Uniformize", true);
  Settings::instance()->writeEntry("/Aethera/Quote Options/Header Mid", "bgcolor=#f3f0f0 width=100%><font face=\"Helvetica,Arial,Times\" color=#000000><b>");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Header Prefix", "<table width=100% border=0 cellpadding=0><tr><td");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Header Suffix", "</b></font></td></tr></table>");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Indenting String", "&nbsp;&nbsp;&nbsp;&nbsp;");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Normal Prefix", "<font face=\"Helvetica,Arial,Times\", size=3 color=#000000><tt>");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Normal Suffix", "</tt></font>");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Quote Prefix", "<font face=\"Helvetica,Arial,Times\" color=\"#a0a0a0\">");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Quote Separators", "\"&nbsp;\",\" \"");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Quote Strings", "\"&gt;\",\"**\",\" &gt;\"");
  Settings::instance()->writeEntry("/Aethera/Quote Options/Quote Suffix", "</font><br>");

  // read options
  Settings::instance()->writeEntry("/Aethera/Read Options/Body Color", "000000");
  Settings::instance()->writeEntry("/Aethera/Read Options/BodyBgColor", "FFFFFF");
  Settings::instance()->writeEntry("/Aethera/Read Options/Default Font", "Helvetica, Gothic, Arial, Times");
  Settings::instance()->writeEntry("/Aethera/Read Options/Default Font Color", "000000");
  Settings::instance()->writeEntry("/Aethera/Read Options/Default Font Size", "3");
  Settings::instance()->writeEntry("/Aethera/Read Options/Email default action", "new");
  Settings::instance()->writeEntry("/Aethera/Read Options/Has Font", true);
  Settings::instance()->writeEntry("/Aethera/Read Options/Preffer HTML", true);

  Settings::instance()->writeEntry("/Aethera/Folder Management/Waste", 102400);

  // -----------------------------------------------------------------------------
  // vfs roots sholuld be initialize by each plug in itself
  // anyway the info will not be showed if you don't have the right plug in loaded
  // TODO move this part into the right plug in
  Settings::instance()->writeEntry("/Aethera/VFS/TopLevel", "Contacts,Groups,Mail,Notes,Jabber,Whiteboard,Datebook");

  // Contacts root
  Settings::instance()->writeEntry("/Aethera/VFS/Contacts/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Contacts/Server Path", "syscontacts");
  Settings::instance()->writeEntry("/Aethera/VFS/Contacts/Type", "contacts");
  Settings::instance()->writeEntry("/Aethera/VFS/Contacts/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Contacts/View type", "contacts");
  Settings::instance()->writeEntry("/Aethera/VFS/Contacts/MIME type", "text/directory");

  // Group root
  Settings::instance()->writeEntry("/Aethera/VFS/Groups/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Groups/Server Path", "sysgroups");
  Settings::instance()->writeEntry("/Aethera/VFS/Groups/Type", "groups");
  Settings::instance()->writeEntry("/Aethera/VFS/Groups/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Groups/View type", "groups");
  Settings::instance()->writeEntry("/Aethera/VFS/Groups/MIME type", "application/x-groups");

  // Notes root
  Settings::instance()->writeEntry("/Aethera/VFS/Notes/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Notes/Server Path", "sysnotes");
  Settings::instance()->writeEntry("/Aethera/VFS/Notes/Type", "notes");
  Settings::instance()->writeEntry("/Aethera/VFS/Notes/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Notes/View type", "notes");
  Settings::instance()->writeEntry("/Aethera/VFS/Notes/MIME type", "application/x-notes");

  // Jabber root
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Server Path", "sysjabber");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Type", "jabber");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/View type", "jabber");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/MIME type", "jabber/x-chat");

  // Whiteboard root
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Server Path", "syswhiteboard");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Type", "whiteboard");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/View type", "whiteboard");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/MIME type", "jabber/x-whiteboard");

  // DateBook root
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Server Path", "sysdatebook");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Type", "datebook");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/View type", "datebook");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/MIME type", "application/x-datebook");

  // Mail root and main directories
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Type", "genmail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Subfolders", "Inbox,Outbox,Drafts,Saved Messages,Sent Mail,Deleted Messages");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/View type", "genmail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/MIME type", "");

  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Inbox/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Inbox/Server Path", "sysmail_inbox");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Inbox/Type", "mail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Inbox/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Inbox/View type", "inbox");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Inbox/MIME type", "text/rfc822");

  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Outbox/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Outbox/Server Path", "sysmail_outbox");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Outbox/Type", "mail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Outbox/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Outbox/View type", "outbox");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Outbox/MIME type", "text/rfc822");

  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Drafts/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Drafts/Server Path", "sysmail_drafts");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Drafts/Type", "mail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Drafts/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Drafts/View type", "drafts");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Drafts/MIME type", "text/rfc822");

  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Saved Messages/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Saved Messages/Server Path", "sysmail_saved");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Saved Messages/Type", "mail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Saved Messages/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Saved Messages/View type", "saved");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Saved Messages/MIME type", "text/rfc822");

  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Sent Mail/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Sent Mail/Server Path", "sysmail_sent");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Sent Mail/Type", "mail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Sent Mail/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Sent Mail/View type", "sent");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Sent Mail/MIME type", "text/rfc822");

  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Deleted Messages/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Deleted Messages/Server Path", "sysmail_deleted");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Deleted Messages/Type", "mail");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Deleted Messages/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Deleted Messages/View type", "deleted");
  Settings::instance()->writeEntry("/Aethera/VFS/Mail/Deleted Messages/MIME type", "text/rfc822");
  // ------------------------------------------------------------------------------

  // new html processing options
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Reply pattern","&gt; ,&gt;");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Reply begin tag",
    "<table cellpadding=\"0\" cellspacing=\"0\" width=\"100%\"><tr><td valign=\"top\">");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Reply end tag","</font></td></tr></table>");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Reply middle tag","</td><td><font color=\"#a0a0a0\">");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Reply tabulator","&nbsp;&nbsp;&nbsp;&nbsp;");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Reply duplicate tabulator",false);
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Wrote pattern","wrote:,wrote :");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Wrote begin tag",
    "<table cellpadding=\"0\" cellspacing=\"0\" width=\"100%\"><tr><td bgcolor=\"#e0e0e0\"><b>&nbsp;");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Wrote end tag","</b></td></tr></table>");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Text only",false);
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Max parsing size","8192");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Builtin rules","Important,Arrows");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Custom rules","");
  Settings::instance()->writeEntry("/Aethera/HTML Parser/Active rules","Important,Arrows");

  Settings::instance()->writeEntry("/Aethera/HTML Rule Important/Begin pattern","\\*");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Important/Body pattern",".+");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Important/End pattern","\\*");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Important/Begin HTML","<font color=\"#800000\"><b>");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Important/End HTML","</b></font>");

  Settings::instance()->writeEntry("/Aethera/HTML Rule Arrows/Begin pattern","&gt\\;&gt\\;");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Arrows/Body pattern",".*");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Arrows/End pattern","&lt\\;&lt\\;$");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Arrows/Begin HTML","<font color=\"#202050\"><b>&gt;&gt;");
  Settings::instance()->writeEntry("/Aethera/HTML Rule Arrows/End HTML","&lt;&lt;</b></font>");

  Account::ServerType serverType = (Account::ServerType)
    serverTypeButtonGroup_->id(serverTypeButtonGroup_->selected());
    
  QString accountName;

  switch (serverType)
  {
    case Account::POP3:
      accountName = popSetupPage_->serverLineEdit->text();
      break;

    case Account::GROUPWARE:
    case Account::IMAP4:
      accountName = imapSetupPage_->serverLineEdit->text();
      break;

    case Account::MAILBOX:
      accountName = localMailboxLineEdit_->text();
      break;

    default:
      qWarning("Server Type BUG");
      break;
  }

  QString aetheraAccount = QString("/Aethera/Accounts/") + accountName;

  // receive options
  Settings::instance()->writeEntry("/Aethera/Receive Options/Interval", 300);
  Settings::instance()->writeEntry("/Aethera/Receive Options/UidCount", 10000);
  Settings::instance()->writeEntry("/Aethera/Receive Options/Accounts", accountName);
  Settings::instance()->writeEntry("/Aethera/Receive Options/Default", accountName);

  // send options
  Settings::instance()->writeEntry("/Aethera/Send Options/Work offline", false);
  Settings::instance()->writeEntry("/Aethera/Send Options/Save sent", true);

  // write the account entry
  Settings::instance()->writeEntry(aetheraAccount+"/Enabled", true);

  Settings::instance()->writeEntry(aetheraAccount+"/From", addressLineEdit_->text());
  Settings::instance()->writeEntry(aetheraAccount+"/Name", nameLineEdit_->text());

  if (replyToLineEdit_->text() != addressLineEdit_->text())
    Settings::instance()->writeEntry(aetheraAccount+"/Reply to", replyToLineEdit_->text());

  Settings::instance()->writeEntry(aetheraAccount+"/Organization", orgLineEdit_->text());

  // Send info
  Settings::instance()->writeEntry
    (aetheraAccount+"/Smtp", smtpSetupPage_->addressLineEdit->text());
  Settings::instance()->writeEntry
    (aetheraAccount+"/SPort", smtpSetupPage_->portSpinBox->value());
  Settings::instance()->writeEntry
    (aetheraAccount+"/SendType", "Sendmail");
  Settings::instance()->writeEntry
    (aetheraAccount+"/SendSSL", smtpSetupPage_->secureCheckBox->isChecked());
  Settings::instance()->writeEntry
    (aetheraAccount+"/SendTLS", smtpSetupPage_->enableTLS->isChecked());
  Settings::instance()->writeEntry
    (aetheraAccount+"/SendCertificates", smtpSetupPage_->checkBoxCertificate->isChecked());
  if( smtpSetupPage_->authCheckBox->isChecked() )
  {
    Settings::instance()->writeEntry
      (aetheraAccount+"/SendUser", smtpSetupPage_->usernameLineEdit->text());
    if( smtpSetupPage_->passwordCheckBox->isChecked() )
    {
      Settings::instance()->writeEntry(
        aetheraAccount+"/SendPassword",
        QString( MIMECodec::encodeBase64(smtpSetupPage_->passwordLineEdit->text().utf8()) ) );
      Settings::instance()->writeEntry(
        aetheraAccount+"/SendRememberPassword",
        smtpSetupPage_->passwordCheckBox->isChecked() );
    }
    
    int i = smtpSetupPage_->authTypeButtonGroup->id( smtpSetupPage_->authTypeButtonGroup->selected() );
    switch(i)
    {

      case 0:
        Settings::instance()->writeEntry
          (aetheraAccount+"/SendAuthType", "LOGIN");
        break;
      case 1:
        Settings::instance()->writeEntry
          (aetheraAccount+"/SendAuthType", "PLAIN");
        break;
      case 2:
        Settings::instance()->writeEntry
          (aetheraAccount+"/SendAuthType", "DIGEST_MD5");
        break;
      case 3:
        Settings::instance()->writeEntry
          (aetheraAccount+"/SendAuthType", "CRAM_MD5");
        break;
      default:
        Settings::instance()->writeEntry
          (aetheraAccount+"/SendAuthType", "NONE");
        break;
    }
  }
  else
  {
    Settings::instance()->writeEntry
      (aetheraAccount+"/SendAuthType", "NONE");
  }
    
  // Receive info
  switch (serverType)
  {
    case Account::POP3:
      Settings::instance()->writeEntry(aetheraAccount+"/Type", "POP3");
      Settings::instance()->writeEntry
        (aetheraAccount+"/User", popSetupPage_->usernameLineEdit->text());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Server", popSetupPage_->serverLineEdit->text());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Port", popSetupPage_->portSpinBox->value());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Leave on server",
          popSetupPage_->leaveOnServerCheckBox->isChecked());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Sync",
          popSetupPage_->syncOnDeleteCheckBox->isChecked());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Persistence",
          popSetupPage_->leaveOnServerCheckBox->isChecked());
      if( popSetupPage_->rememberPasswordCheckBox->isChecked() )
      {
        Settings::instance()->writeEntry(
          aetheraAccount+"/Password",
          QString(MIMECodec::encodeBase64(popSetupPage_->passwordLineEdit->text().utf8())) );
        Settings::instance()->writeEntry
          (aetheraAccount+"/Remember password",
            popSetupPage_->rememberPasswordCheckBox->isChecked());
      }
      Settings::instance()->writeEntry
        (aetheraAccount+"/SSL", popSetupPage_->secureCheckBox->isChecked());
      Settings::instance()->writeEntry
        (aetheraAccount+"/TLS", popSetupPage_->enableTLS->isChecked());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Certificates", popSetupPage_->checkBoxCertificate->isChecked());
      break;

    case Account::GROUPWARE:
    {
      // update Kolab accounts list
      accountName = imapSetupPage_->serverLineEdit->text();
      QStringList kolabList = 
        Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
      
      if( kolabList.end() == kolabList.find( accountName ) )
      {
        kolabList.append( accountName );
        Settings::instance()->writeEntry("/Aethera/Kolab/Accounts", kolabList, ',');
      }
      // update Kolab folders settings
      Settings::instance()->writeEntry(aetheraAccount + "/Kolab", true);
      Settings::instance()->writeEntry(aetheraAccount + "/Kolab/Calendar",
        kolabSetupPage_->calendarLineEdit->text());
      Settings::instance()->writeEntry(aetheraAccount + "/Kolab/Contacts",
        kolabSetupPage_->contactsLineEdit->text());
      Settings::instance()->writeEntry(aetheraAccount + "/Kolab/Notes",
        kolabSetupPage_->notesLineEdit->text());
      Settings::instance()->writeEntry(aetheraAccount + "/Kolab/ToDo",
        kolabSetupPage_->todoLineEdit->text());
    }
    // TODO: Create the KOLAB folders

    case Account::IMAP4:
      Settings::instance()->writeEntry(aetheraAccount+"/Type", "IMAP4");
      Settings::instance()->writeEntry
        (aetheraAccount+"/User", imapSetupPage_->usernameLineEdit->text());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Server", imapSetupPage_->serverLineEdit->text());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Port", imapSetupPage_->portSpinBox->value());
      if( imapSetupPage_->rememberPasswordCheckBox->isChecked() )
      {
        Settings::instance()->writeEntry(
          aetheraAccount+"/Password",
          QString(MIMECodec::encodeBase64(imapSetupPage_->passwordLineEdit->text().utf8())) );
        Settings::instance()->writeEntry(
          aetheraAccount+"/Remember password",
          imapSetupPage_->rememberPasswordCheckBox->isChecked() );
      }
      Settings::instance()->writeEntry
        (aetheraAccount+"/SSL", imapSetupPage_->secureCheckBox->isChecked());
      Settings::instance()->writeEntry
        (aetheraAccount+"/TLS", imapSetupPage_->enableTLS->isChecked());
      Settings::instance()->writeEntry
        (aetheraAccount+"/Certificates", imapSetupPage_->checkBoxCertificate->isChecked());

      {
        // This part has to be made by the mail plug in itself.
        // Anyway the info will not be showed if you don't have the mail plug in
        // loaded.

        // TODO: Move this part into the mail plug in.

        QString IMAPfolder = accountName;

        // Create an extra VFS folder.

        Settings::instance()->writeEntry
          (
            "/Aethera/VFS/Mail/Subfolders",
            "Inbox,Outbox,Drafts,Saved Messages,Sent Mail,Deleted Messages,"
            + IMAPfolder
          );

        Settings::instance()->writeEntry("/Aethera/VFS/Mail/Flags", "c");

        imapVFSFolder="/Aethera/VFS/Mail/";
        imapVFSFolder+=IMAPfolder+"/";

        Settings::instance()->writeEntry(imapVFSFolder+"Binding", "thisInstance");
        Settings::instance()->writeEntry(imapVFSFolder+"Type", "imap");
        Settings::instance()->writeEntry(imapVFSFolder+"Flags", "c");
        Settings::instance()->writeEntry(imapVFSFolder+"MIME type", "");
        Settings::instance()->writeEntry(imapVFSFolder+"View type", "genmail");
        Settings::instance()->writeEntry(imapVFSFolder+"Account", IMAPfolder);
        // add the default INBOX folder
        serverPath=QString("imap_")+IMAPfolder;
        Settings::instance()->writeEntry(imapVFSFolder+"Subfolders", "Inbox");

        // create the default IMAP mailbox - INBOX
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/Binding", "thisInstance");
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/Server Path", serverPath);
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/Type", "imap");
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/Flags", "c");
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/View type", "mail");
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/MIME type", "text/rfc822");
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/Account", IMAPfolder);
        Settings::instance()->writeEntry(imapVFSFolder+"Inbox/Mailbox", "INBOX");
        // ---------------------------------------------------------------------------
      }

      break;

    case Account::MAILBOX:
      Settings::instance()->writeEntry(aetheraAccount+"/Type", "MAILBOX");
      Settings::instance()->writeEntry
        (aetheraAccount+"/Mailbox", localMailboxLineEdit_->text());
      break;

    default:
      qWarning("BUG");
      break;
  }

  QDir d;

  qDebug("Creating directory structure...");

  d.mkdir(aetheraDir);
  d.mkdir(aetheraDir + "/sysnotes");
  d.mkdir(aetheraDir + "/sysmail_inbox");
  d.mkdir(aetheraDir + "/sysmail_outbox");
  d.mkdir(aetheraDir + "/sysmail_drafts");
  d.mkdir(aetheraDir + "/sysmail_sent");
  d.mkdir(aetheraDir + "/sysmail_deleted");
  d.mkdir(aetheraDir + "/sysmail_saved");
  d.mkdir(aetheraDir + "/internal");

  if (!serverPath.isEmpty() && !imapVFSFolder.isEmpty())
  {
    // Create an extra IMAP folder.
    d.mkdir(aetheraDir + "/" + serverPath);

    // Create the IMAP config file.

    QFile f(aetheraDir + "/" + serverPath + "/" + "config");

    if (!f.open(IO_WriteOnly))
    {
      qWarning("Can't open imap config file for writing");
    }
    else
    {
      QDataStream s( &f );
      s << Settings::instance()->readEntry(imapVFSFolder+"Inbox/Account");
      s << Settings::instance()->readEntry(imapVFSFolder+"Inbox/Mailbox");
    }
  }

  if (!QFileInfo(aetheraDir + "/syscontacts").exists())
  {
    QFile contacts(aetheraDir + "/syscontacts");
    contacts.open(IO_WriteOnly);
    contacts.close();
  }

  if (!QFileInfo(aetheraDir + "/sysgroups").exists())
  {
    QFile groups(aetheraDir + "/sysgroups");
    groups.open(IO_WriteOnly);
    groups.close();
  }

  // Create version file.

  QFile ver(aetheraDir + "/.version");

  if (!ver.open(IO_WriteOnly))
  {
    qWarning("Can't open version file");
  }
  else
  {
    QTextStream str(&ver);
    str << AppVersion;
  }

  QWizard::accept();
}

  void
SetupWizard::showPage(QWidget * page)
{
  QWizard::showPage(page);

  // Work around broken focus handling of QWizard.

  if (personalSetupPage_ == page)
    nameLineEdit_->setFocus();

  else if (aetheraDirSetupPage_ == page)
    pluginDirLineEdit_->setFocus();

  else if (serverTypeSetupPage_ == page)
  {
    serverTypeButtonGroup_->setFocus();
    // enable next page by default
    slotServerTypePageReady();
  }

  else if (popSetupPage_ == page)
    popSetupPage_->serverLineEdit->setFocus();

  else if (imapSetupPage_ == page)
    imapSetupPage_->serverLineEdit->setFocus();

  else if (localSetupPage_ == page)
    localMailboxLineEdit_->setFocus();

  else if (smtpSetupPage_ == page)
    smtpSetupPage_->addressLineEdit->setFocus();

#if 0
  else if (reviewPage_ == page)
    nameLineEdit_->setFocus();
#endif

  else if (finishedPage_ == page)
    finishButton()->setFocus();
}

