/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include "uipart.h"
#include "uipartprivate.h"

/*
 * UIPart - a KParts-like thing, extensible to an OLE-like thing in the future
 * Inherit it and implement at least the abstract methods
 */
 
static const char* uipart_metainfo[][2]=
{
    { "name", "UI Part" },
    { "type", "Plugin" },
    { "class", "UIPart" },
    { "class/fqn", "UIPart" },
//    { "factory", "uipart_factory" },
    { "description", PART_SERVICE_DESC },
    { "version", "0.2.0" },
    { "version/major", "0" },
    { "version/minor", "2" },
    { "version/release", "0" },
    { "api/version", "0.2.0" },
    { "api/version/major", "0" },
    { "api/version/minor", "2" },
    { "api/version/release", "0" },
    { "service", PART_SERVICE },
    { 0, 0 }
};

UIPart::UIPart(void)
  : kore::ComponentRef( new kore::ComponentProxy( new kore::MetaInfo( uipart_metainfo ) ) )
{
  qDebug("UIPart::constructor");
  partActive = false;
  qthandler = NULL;
  
  // set the instance
  proxy()->setInstance( this );
}

UIPart::~UIPart()
{
//  qDebug("---------- UIPart destructor !!! -----------");
}

UIPartPrivate *UIPart::createQTHandler(void)
{
  return new UIPartPrivate(this);
}

UIPartPrivate *UIPart::getQTHandler(void)
{
  if( qthandler==NULL )
    qthandler=createQTHandler();
  return qthandler;
}

bool UIPart::hasCapability(const QString&)
{
  return false;
}

void UIPart::emitActivated(void)
{
  qthandler->emitActivated();
}

void UIPart::emitDeactivated(void)
{
  qthandler->emitDeactivated();
}

void UIPart::emitWantUIReload(void)
{
  qthandler->emitWantUIReload();
}

void UIPart::statusMessage(QString& message)
{
  qthandler->emitStatusMessage(message);
}

void UIPart::statusMessage(QString& message, int ms)
{
  qthandler->emitStatusMessage(message, ms);
}

void UIPart::linkWidget(const QString& widgetName)
{
  qthandler->linkWidget(widgetName);
}

void UIPart::activated(void)
{
  qDebug("UIPart::activated");
  partActive = true;
}

void UIPart::deactivated(void)
{
  qDebug("UIPart::deactivated");
  partActive = false;
}

bool UIPart::isActive(void)
{
  if(partActive) qDebug("UIPart::isActive returns true");
  else qDebug("UIPart::isActive returns false");
  return partActive;
}

