/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include <stdio.h>

#include <qobject.h>
#include <qevent.h>
#include <qstatusbar.h>
#include "uipartmanager.h"
#include "uipartmanagerprivate.h"

static const char* uipartmanager_metainfo[][2]=
{
    { "name", "UI Part Manager" },
    { "type", "Plugin" },
    { "class", "UIPartManager" },
    { "class/fqn", "UIPartManager" },
    { "factory", "uipartmanager_factory" },
    { "description", PARTMANAGER_SERVICE_DESC },
    { "version", "0.2.0" },
    { "version/major", "0" },
    { "version/minor", "2" },
    { "version/release", "0" },
    { "api/version", "0.2.0" },
    { "api/version/major", "0" },
    { "api/version/minor", "2" },
    { "api/version/release", "0" },
    { "service", PARTMANAGER_SERVICE },
    { 0, 0 }
};

/*
 * UIPartManager - manages parts ^)
 */

kore::Interface* uipartmanager_factory()
{
  return new UIPartManager();
}

UIPartManager::UIPartManager(void)
  : kore::ComponentRef( new kore::ComponentProxy( new kore::MetaInfo( uipartmanager_metainfo ) ) )
{
  parts.setAutoDelete(false);
  qthandler = NULL;
  mainWnd = NULL;
  mainMenu = NULL;
  mainStatus = NULL;
  activePart = NULL;
  
  // set the instance
  proxy()->setInstance( this );
}

UIPartManager::~UIPartManager(void)
{
  // Unlink the remaining parts, if any
  if(parts.first()) do {
    unlinkPart(parts.current());
  } while (parts.next());
  
  parts.setAutoDelete(true);
  parts.clear();
  
//  qDebug("UIPartManager destructor !!!");
}

UIPartManagerPrivate *UIPartManager::createQTHandler(void)
{
  return new UIPartManagerPrivate(this);
}

UIPartManagerPrivate *UIPartManager::getQTHandler(void)
{
  if( qthandler==NULL )
    qthandler = createQTHandler();
  return qthandler;
}

void UIPartManager::addPart(UIPart *part)
{
  if(part->hasCapability( WIDGET )) {
    part->createWidget( WIDGET );
    part->linkWidget( WIDGET );
  }
  parts.append(part);
  linkPart(part);
  getQTHandler()->emitPartAdded(part);
}

void UIPartManager::removePart(UIPart *part)
{
  if(part->isActive()) deactivatePart(part);
  if(!parts.removeRef(part)) return;;
  unlinkPart(part);
  getQTHandler()->emitPartRemoved(part);
}

void UIPartManager::activatePart(UIPart *part)
{
  qDebug("activatePart");
  if(part == activePart) return;
  if(activePart) deactivatePart(activePart);
  if((mainWnd) && (part->hasCapability( TOOLBARS ))) part->showToolbars(mainWnd);
  if((mainMenu) && (part->hasCapability( MENUBARS ))) part->showMenus(mainMenu);
  if(part->hasCapability( WIDGET )) {
    QWidget *w = part->getWidget( WIDGET );
    // Case 1: called from the part, in this case the widget already has focus
    // Case 2: called from user code, in this case the widget may not have focus
    if((w->isFocusEnabled()) && (!w->hasFocus())) w->setFocus();
  }
  activePart = part;
  part->activated();
  getQTHandler()->emitPartActivated(part);
}

void UIPartManager::deactivatePart(UIPart *part)
{
  qDebug("deactivatePart");
  if(part == activePart) activePart = NULL;

  if((mainWnd) && (part->hasCapability( TOOLBARS ))) part->hideToolbars(mainWnd);
  if((mainMenu) && (part->hasCapability( MENUBARS ))) part->hideMenus(mainMenu);
  part->deactivated();
  getQTHandler()->emitPartDeactivated(part);
}

void UIPartManager::reloadUIPart(UIPart*)
{
  qDebug("reloadUIPart");
}

void UIPartManager::setMainWindow(QMainWindow *wnd)
{
  mainWnd = wnd;
}

void UIPartManager::setMainMenu(QMenuBar *menu)
{
  mainMenu = menu;
}

void UIPartManager::setStatusBar(QStatusBar *status)
{
  mainStatus = status;
}

void UIPartManager::statusMessage(QString& message)
{
  if(mainStatus) mainStatus->message(message);
}

void UIPartManager::statusMessage(QString& message, int ms)
{
  if(mainStatus) mainStatus->message(message, ms);
}

void UIPartManager::linkPart(UIPart *part)
{
  QObject *partqthandler = (QObject*) part->getQTHandler();
  QObject::connect(partqthandler, SIGNAL(activated(UIPart *)), qthandler, SLOT(slotPartActivated(UIPart*)));
  QObject::connect(partqthandler, SIGNAL(deactivated(UIPart *)), qthandler, SLOT(slotPartDeactivated(UIPart*)));
  QObject::connect(partqthandler, SIGNAL(wantUIReload(UIPart *)), qthandler, SLOT(slotWantUIReload(UIPart *)));
  QObject::connect(partqthandler, SIGNAL(statusMessage(QString&)), qthandler, SLOT(slotStatusMessage(QString&)));
  QObject::connect(partqthandler, SIGNAL(statusMessage(QString&, int)), qthandler, SLOT(slotStatusMessage(QString&, int)));
}

void UIPartManager::unlinkPart(UIPart *part)
{
  QObject *partqthandler = (QObject*) part->getQTHandler();
  QObject::disconnect(partqthandler, SIGNAL(activated(UIPart *)), qthandler, SLOT(slotPartActivated(UIPart*)));
  QObject::disconnect(partqthandler, SIGNAL(deactivated(UIPart *)), qthandler, SLOT(slotPartDeactivated(UIPart*)));
  QObject::disconnect(partqthandler, SIGNAL(wantUIReload(UIPart *)), qthandler, SLOT(slotWantUIReload(UIPart *)));
  QObject::disconnect(partqthandler, SIGNAL(statusMessage(QString&)), qthandler, SLOT(slotStatusMessage(QString&)));
  QObject::disconnect(partqthandler, SIGNAL(statusMessage(QString&, int)), qthandler, SLOT(slotStatusMessage(QString&, int)));
}
