/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include "tkcoloractions.h"
#include "tkcoloractions_p.h"

#include <qcolordialog.h>
#include <qpainter.h>
#include <qiconset.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qpopupmenu.h>

static const char* const image0_data[] = {
"16 16 23 1",
". c None",
"e c #000000",
"a c #4d2500",
"u c #606060",
"d c #61360f",
"m c #613c17",
"# c #7a4b1d",
"h c #8c5b2b",
"c c #915f2f",
"b c #9e6b39",
"p c #9f6c3a",
"j c #a5733f",
"g c #aa7645",
"i c #bc8454",
"r c #c08857",
"l c #c18a5a",
"f c #c38a5a",
"q c #d09b70",
"k c #d5a57d",
"s c #e5b995",
"n c #e9be9c",
"t c #efd7c6",
"o c #ffffff",
"..........#a....",
".........#bcd...",
".......e#fgha...",
".....ee#iffjhd..",
"....ee#klefeeeee",
"...eemnneoefjpd.",
"...eeffnneqfrpd.",
"...e.dffnnnspt#d",
"...e..ddffnpt#d.",
"........dfpt#d..",
"...u.....dddd...",
"..uuu......d....",
".uuuuu..........",
"uuuuuuuuuuuuuu..",
"uuuuuuuuuuuuuuu.",
"uuuuuuuuuuuuuuuu"};

static const char* const image1_data[] = {
"16 18 35 1",
". c None",
"k c #000000",
"t c #121209",
"D c #1a1812",
"l c #1c0c01",
"m c #210f01",
"q c #231001",
"C c #534f4c",
"F c #606060",
"x c #704415",
"G c #736a5a",
"u c #7a4b1d",
"y c #825234",
"B c #897862",
"h c #92612d",
"i c #9f6c3a",
"E c #a09380",
"A c #a36f52",
"n c #b7804e",
"d c #c37b40",
"e c #c38a5a",
"a c #c47d41",
"f c #ce9669",
"p c #d49f71",
"w c #f0b797",
"r c #f1b492",
"s c #f6dccb",
"z c #f7c9ac",
"c c #fac78c",
"# c #facb8d",
"b c #fe8f0c",
"g c #fec5a7",
"v c #ffd1b1",
"o c #ffe2cd",
"j c #ffe2ce",
"....#a..........",
"....bcd.........",
"....efgh........",
"...ieefjh.......",
"...kieefjh......",
"....lieefgh.....",
".....mieenop....",
"......qinnrsp...",
".......tuunvw...",
"........nxynzp..",
"..........nxAB..",
"............CDE.",
"...F.kkkkkkk.kG.",
"..FFF...........",
".FFFFF..........",
"FFFFFFFFFFFFFF..",
"FFFFFFFFFFFFFFF.",
"FFFFFFFFFFFFFFFF"};

static const char* const image2_data[] = {
"16 16 4 1",
". c None",
"# c #000080",
"a c #808080",
"b c #c0c0c0",
"......#.........",
".....a#a........",
".....###........",
"....a###a.......",
"....##.##.......",
"...a#..##a......",
"...#######......",
"..a#....##a.....",
"..##....a##.....",
".a#.....a##a....",
"####...######...",
"................",
"bbbbbbbbbbbbbbbb",
"bbbbbbbbbbbbbbbb",
"bbbbbbbbbbbbbbbb",
"bbbbbbbbbbbbbbbb"};

/****************************************************************************************/
TKSelectColorAction::TKSelectColorAction(Type type, QObject *parent, const char *name)
: QAction(parent, name), type(type)
{
  currentColor = new QColor();
  init();
}

void TKSelectColorAction::init()
{
  popupMenu = new QPopupMenu();
  standardPanel = new ColorPanel(0);//popupMenu);
  recentPanel = new ColorPanel(0);//popupMenu);

  connect(this, SIGNAL(activated()), SLOT(slotActivated()));
  connect(standardPanel, SIGNAL(selected(const QColor &)), SLOT(panelColorSelected(const QColor &)));
  connect(standardPanel, SIGNAL(reject()), SLOT(panelReject()));
  connect(recentPanel, SIGNAL(selected(const QColor &)), SLOT(panelColorSelected(const QColor &)));
  connect(recentPanel, SIGNAL(reject()), SLOT(panelReject()));

  recentPanel->clear();

  switch (type) {
    case TextColor:
      popupMenu->insertItem("More Text Colors...",this,SLOT(selectColorDialog()));
      setIconSet(QPixmap((const char**)image2_data));
      setCurrentColor(Qt::black);
      setToolTip("Font Color");
      setText("Font Color");
      setMenuText("Font Color");
      break;
    case LineColor:
      popupMenu->insertItem("More Line Colors...",this,SLOT(selectColorDialog()));
      setIconSet(QPixmap((const char**)image1_data));
      setCurrentColor(Qt::black);
      setToolTip("Line Color");
      setText("Line Color");
      setMenuText("Line Color");
      break;
    case FillColor:
      popupMenu->insertItem("More Fill Colors...",this,SLOT(selectColorDialog()));
      setIconSet(QPixmap((const char**)image0_data));
      setCurrentColor(Qt::white);
      setToolTip("Fill Color");
      setText("Fill Color");
      setMenuText("Fill Color");
      break;
    case Color:
      break;
  }

  popupMenu->insertSeparator();
  popupMenu->insertItem(standardPanel);
//  popupMenu->insertSeparator();
//  popupMenu->insertItem(recentPanel);
}

TKSelectColorAction::~TKSelectColorAction()
{
  delete popupMenu;
  delete currentColor;

  delete recentPanel; // until i cannot put it on menu
}

void TKSelectColorAction::addedTo(QWidget *actionWidget, QWidget *)
{
  if (actionWidget->inherits("QToolButton")) {
    QToolButton *button = (QToolButton *)actionWidget;
    button->setFixedHeight(22);
    initToolButton(button);
  }
}

void TKSelectColorAction::initToolButton(QToolButton *b)
{
  b->setPopup(popupMenu);
  b->setPopupDelay(0);
}

void TKSelectColorAction::updatePixmap()
{
  QPixmap pix = iconSet().pixmap();
  QPainter p(&pix);
  switch (type) {
    case TextColor:
      p.fillRect(QRect(0,12,16,5), *currentColor);
      break;
    case LineColor:
      p.fillRect(QRect(0,13,16,5), *currentColor);
      p.fillRect(QRect(3,12,1,1), *currentColor);
      break;
    case FillColor:
      p.fillRect(QRect(0,13,16,5), *currentColor);
      p.fillRect(QRect(1,10,5,3), *currentColor);
      break;
    case Color:
      break;
  }
  p.end();
  setIconSet(pix);
}

void TKSelectColorAction::setCurrentColor(const QColor &color)
{
  delete currentColor;
  currentColor = new QColor(color);
  updatePixmap();
}

void TKSelectColorAction::setActiveColor(const QColor &color)
{
  standardPanel->setActiveColor(color);
}

void TKSelectColorAction::selectColorDialog()
{
  QColor color = QColorDialog::getColor(*currentColor);
  if (color.isValid()) {
    setCurrentColor(color);
    recentPanel->insertColor(*currentColor);
    activate();
  }
}

void TKSelectColorAction::slotActivated()
{
  activate();
}

void TKSelectColorAction::activate()
{
  emit activated(*currentColor);
}

void TKSelectColorAction::panelColorSelected(const QColor &color)
{
  popupMenu->hide();
  setCurrentColor(color);
  activate();
}

void TKSelectColorAction::panelReject()
{
  popupMenu->hide();
}

/****************************************************************************************/
ColorPanel::ColorPanel(QWidget *parent, const char *name)
: QWidget(parent,name)
{
  activeColor = black;
  layout = 0;
  setNumCols(15);
  fillPanel();
}

void ColorPanel::setNumCols(int col)
{
  panelWidth = col;
  resetGrid();

  QDictIterator<ColorPanelButton> it(colors);
  while (it.current()) {
    addToGrid(it.current());
    ++it;
  }
}

ColorPanel::~ColorPanel()
{
}

void ColorPanel::resetGrid()
{
  posX = 0;
  posY = 0;

  delete layout;
  layout = new QGridLayout(this, 0, panelWidth + 1, 0, 0);

  emit sizeChanged();
}

void ColorPanel::clear()
{
  colors.setAutoDelete(true);
  colors.clear();
  colors.setAutoDelete(false);
  resetGrid();
}

void ColorPanel::insertColor(const QColor &color, const QString &text)
{
  if (colors[color.name()])
    return;

  insertColor(color);
  QToolTip::add(colors[color.name()], text);
}

void ColorPanel::insertColor(const QColor &color)
{
  if (colors[color.name()])
    return;

  layout->setMargin(3);
  ColorPanelButton* f = new ColorPanelButton(color, this);
  colors.insert(color.name(), f);
  if (activeColor == color)
    f->setActive(true);

  connect(f, SIGNAL(selected(const QColor &)), SLOT(colorSelected(const QColor &)));

  addToGrid(f);
}

void ColorPanel::addToGrid(ColorPanelButton *f)
{
  layout->addWidget(f, posY, posX);
  f->show();
  posX++;
  if (posX == panelWidth) {
    posX = 0;
    posY++;
  }
  emit sizeChanged();
}

void ColorPanel::setActiveColor(const QColor &color)
{
  ColorPanelButton* b = colors[activeColor.name()];
  if (b)
    b->setActive(false);

  activeColor = color;

  b = colors[activeColor.name()];
  if (b)
    b->setActive(true);
}

void ColorPanel::mouseReleaseEvent( QMouseEvent* )
{
  reject();
}

void ColorPanel::colorSelected(const QColor &color)
{
  emit selected(color);
}

void ColorPanel::fillPanel()
{
  blockSignals(true);

  insertColor(QColor(),  "");
  insertColor(qRgb( 255 ,   0   ,   0   ),  "Red");
  insertColor(qRgb( 255 ,   165 ,   0   ),  "Orange");
  insertColor(qRgb( 255 ,   0   ,   255 ),  "Magenta");
  insertColor(qRgb( 0   ,   0   ,   255 ),  "Blue");
  insertColor(qRgb( 0   ,   255 ,   255 ),  "Cyan");
  insertColor(qRgb( 0   ,   255 ,   0   ),  "Green");
  insertColor(qRgb( 255 ,   255 ,   0   ),  "Yellow");
  insertColor(qRgb( 165 ,   42  ,   42  ),  "Brown");
  insertColor(qRgb( 139 ,   0   ,   0   ),  "Darkred");
  insertColor(qRgb( 255 ,   140 ,   0   ),  "Dark Orange");
  insertColor(qRgb( 139 ,   0   ,   139 ),  "Dark Magenta");
  insertColor(qRgb( 0   ,   0   ,   139 ),  "Dark Blue");
  insertColor(qRgb( 0   ,   139 ,   139 ),  "Dark Cyan");
  insertColor(qRgb( 0   ,   100 ,   0   ),  "Dark Green");
  insertColor(qRgb( 130 ,   127 ,   0   ),  "Dark Yellow");

  insertColor(qRgb(255,255,255), "White");
  insertColor(qRgb(229,229,229), "Grey 90%");
  insertColor(qRgb(204,204,204), "Grey 80%");
  insertColor(qRgb(178,178,178), "Grey 70%");
  insertColor(qRgb(153,153,153), "Grey 60%");
  insertColor(qRgb(127,127,127), "Grey 50%");
  insertColor(qRgb(102,102,102), "Grey 40%");
  insertColor(qRgb(76,76,76),    "Grey 30%");
  insertColor(qRgb(51,51,51),    "Grey 20%");
  insertColor(qRgb(25,25,25),    "Grey 10%");
  insertColor(qRgb(0,0,0),       "Black");

  insertColor(qRgb( 255 ,   255 ,   240 ),  "Ivory");
  insertColor(qRgb( 255 ,   250 ,   250 ),  "Snow");
  insertColor(qRgb( 245 ,   255 ,   250 ),  "Mint Cream");
  insertColor(qRgb( 255 ,   250 ,   240 ),  "Floral White");
  insertColor(qRgb( 255 ,   255 ,   224 ),  "Light Yellow");
  insertColor(qRgb( 240 ,   255 ,   255 ),  "Azure");
  insertColor(qRgb( 248 ,   248 ,   255 ),  "Ghost White");
  insertColor(qRgb( 240 ,   255 ,   240 ),  "Honeydew");
  insertColor(qRgb( 255 ,   245 ,   238 ),  "Seashell");
  insertColor(qRgb( 240 ,   248 ,   255 ),  "Alice Blue");
  insertColor(qRgb( 255 ,   248 ,   220 ),  "Cornsilk");
  insertColor(qRgb( 255 ,   240 ,   245 ),  "Lavender Blush");
  insertColor(qRgb( 253 ,   245 ,   230 ),  "Old Lace");
  insertColor(qRgb( 245 ,   245 ,   245 ),  "White Smoke");
  insertColor(qRgb( 255 ,   250 ,   205 ),  "Lemon Chiffon");
  insertColor(qRgb( 224 ,   255 ,   255 ),  "Light Cyan");
  insertColor(qRgb( 250 ,   250 ,   210 ),  "Light Goldenrod Yellow");
  insertColor(qRgb( 250 ,   240 ,   230 ),  "Linen");
  insertColor(qRgb( 245 ,   245 ,   220 ),  "Beige");
  insertColor(qRgb( 255 ,   239 ,   213 ),  "Papaya Whip");
  insertColor(qRgb( 255 ,   235 ,   205 ),  "Blanched Almond");
  insertColor(qRgb( 250 ,   235 ,   215 ),  "Antique White");
  insertColor(qRgb( 255 ,   228 ,   225 ),  "Misty Rose");
  insertColor(qRgb( 230 ,   230 ,   250 ),  "Lavender");
  insertColor(qRgb( 255 ,   228 ,   196 ),  "Bisque");
  insertColor(qRgb( 255 ,   228 ,   181 ),  "Moccasin");
  insertColor(qRgb( 255 ,   222 ,   173 ),  "Navajo White");
  insertColor(qRgb( 255 ,   218 ,   185 ),  "Peach Puff");
  insertColor(qRgb( 238 ,   232 ,   170 ),  "Pale Goldenrod");
  insertColor(qRgb( 245 ,   222 ,   179 ),  "Wheat");
  insertColor(qRgb( 220 ,   220 ,   220 ),  "Gainsboro");
  insertColor(qRgb( 240 ,   230 ,   140 ),  "Khaki");
  insertColor(qRgb( 175 ,   238 ,   238 ),  "Pale Turquoise");
  insertColor(qRgb( 255 ,   192 ,   203 ),  "Pink");
  insertColor(qRgb( 238 ,   221 ,   130 ),  "Light Goldenrod");
  insertColor(qRgb( 211 ,   211 ,   211 ),  "Light Grey");
  insertColor(qRgb( 255 ,   182 ,   193 ),  "Light Pink");
  insertColor(qRgb( 176 ,   224 ,   230 ),  "Powder Blue");
  insertColor(qRgb( 127 ,   255 ,   212 ),  "Aquamarine");
  insertColor(qRgb( 216 ,   191 ,   216 ),  "Thistle");
  insertColor(qRgb( 173 ,   216 ,   230 ),  "Light Blue");
  insertColor(qRgb( 152 ,   251 ,   152 ),  "Pale Green");
  insertColor(qRgb( 255 ,   215 ,   0   ),  "Gold");
  insertColor(qRgb( 173 ,   255 ,   47  ),  "Green Yellow");
  insertColor(qRgb( 176 ,   196 ,   222 ),  "Light Steel Blue");
  insertColor(qRgb( 144 ,   238 ,   144 ),  "Light Green");
  insertColor(qRgb( 221 ,   160 ,   221 ),  "Plum");
  insertColor(qRgb( 190 ,   190 ,   190 ),  "Gray");
  insertColor(qRgb( 222 ,   184 ,   135 ),  "Burly Wood");
  insertColor(qRgb( 135 ,   206 ,   250 ),  "Light Skyblue");
  insertColor(qRgb( 255 ,   160 ,   122 ),  "Light Salmon");
  insertColor(qRgb( 135 ,   206 ,   235 ),  "Sky Blue");
  insertColor(qRgb( 210 ,   180 ,   140 ),  "Tan");
  insertColor(qRgb( 238 ,   130 ,   238 ),  "Violet");
  insertColor(qRgb( 244 ,   164 ,   96  ),  "Sandy Brown");
  insertColor(qRgb( 233 ,   150 ,   122 ),  "Dark Salmon");
  insertColor(qRgb( 189 ,   183 ,   107 ),  "Dark khaki");
  insertColor(qRgb( 127 ,   255 ,   0   ),  "Chartreuse");
  insertColor(qRgb( 169 ,   169 ,   169 ),  "Dark Gray");
  insertColor(qRgb( 124 ,   252 ,   0   ),  "Lawn Green");
  insertColor(qRgb( 255 ,   105 ,   180 ),  "Hot Pink");
  insertColor(qRgb( 250 ,   128 ,   114 ),  "Salmon");
  insertColor(qRgb( 240 ,   128 ,   128 ),  "Light Coral");
  insertColor(qRgb( 64  ,   224 ,   208 ),  "Turquoise");
  insertColor(qRgb( 143 ,   188 ,   143 ),  "Dark Seagreen");
  insertColor(qRgb( 218 ,   112 ,   214 ),  "Orchid");
  insertColor(qRgb( 102 ,   205 ,   170 ),  "Medium Aquamarine");
  insertColor(qRgb( 255 ,   127 ,   80  ),  "Coral");
  insertColor(qRgb( 154 ,   205 ,   50  ),  "Yellow Green");
  insertColor(qRgb( 218 ,   165 ,   32  ),  "Goldenrod");
  insertColor(qRgb( 72  ,   209 ,   204 ),  "Medium Turquoise");
  insertColor(qRgb( 188 ,   143 ,   143 ),  "Rosy Brown");
  insertColor(qRgb( 219 ,   112 ,   147 ),  "Pale VioletRed");
  insertColor(qRgb( 0   ,   250 ,   154 ),  "Medium Spring Green");
  insertColor(qRgb( 255 ,   99  ,   71  ),  "Tomato");
  insertColor(qRgb( 0   ,   255 ,   127 ),  "Spring Green");
  insertColor(qRgb( 205 ,   133 ,   63  ),  "Peru");
  insertColor(qRgb( 100 ,   149 ,   237 ),  "Cornflower Blue");
  insertColor(qRgb( 132 ,   112 ,   255 ),  "Light Slate Blue");
  insertColor(qRgb( 147 ,   112 ,   219 ),  "Medium Purple");
  insertColor(qRgb( 186 ,   85  ,   211 ),  "Medium Orchid");
  insertColor(qRgb( 95  ,   158 ,   160 ),  "Cadet Blue");
  insertColor(qRgb( 0   ,   206 ,   209 ),  "Dark Turquoise");
  insertColor(qRgb( 0   ,   191 ,   255 ),  "Deep Skyblue");
  insertColor(qRgb( 119 ,   136 ,   153 ),  "Light Slate Grey");
  insertColor(qRgb( 184 ,   134 ,   11  ),  "Dark Goldenrod");
  insertColor(qRgb( 123 ,   104 ,   238 ),  "MediumSlate Blue");
  insertColor(qRgb( 205 ,   92  ,   92  ),  "IndianRed");
  insertColor(qRgb( 210 ,   105 ,   30  ),  "Chocolate");
  insertColor(qRgb( 60  ,   179 ,   113 ),  "Medium Sea Green");
  insertColor(qRgb( 50  ,   205 ,   50  ),  "Lime Ggreen");
  insertColor(qRgb( 32  ,   178 ,   170 ),  "Light Sea Green");
  insertColor(qRgb( 112 ,   128 ,   144 ),  "Slate Gray");
  insertColor(qRgb( 30  ,   144 ,   255 ),  "Dodger Blue");
  insertColor(qRgb( 255 ,   69  ,   0   ),  "Orange Red");
  insertColor(qRgb( 255 ,   20  ,   147 ),  "Deep Pink");
  insertColor(qRgb( 70  ,   130 ,   180 ),  "Steel Blue");
  insertColor(qRgb( 106 ,   90  ,   205 ),  "Slate Blue");
  insertColor(qRgb( 107 ,   142 ,   35  ),  "Olive Drab");
  insertColor(qRgb( 65  ,   105 ,   225 ),  "Royal Blue");
  insertColor(qRgb( 208 ,   32  ,   144 ),  "Violet Red");
  insertColor(qRgb( 153 ,   50  ,   204 ),  "Dark Orchid");
  insertColor(qRgb( 160 ,   32  ,   240 ),  "Purple");
  insertColor(qRgb( 105 ,   105 ,   105 ),  "Dim Gray");
  insertColor(qRgb( 138 ,   43  ,   226 ),  "Blue Violet");
  insertColor(qRgb( 160 ,   82  ,   45  ),  "Sienna");
  insertColor(qRgb( 199 ,   21  ,   133 ),  "Medium Violet Red");
  insertColor(qRgb( 176 ,   48  ,   96  ),  "Maroon");
  insertColor(qRgb( 46  ,   139 ,   87  ),  "Sea Green");
  insertColor(qRgb( 85  ,   107 ,   47  ),  "Dark Olive Green");
  insertColor(qRgb( 34  ,   139 ,   34  ),  "Forest Green");
  insertColor(qRgb( 139 ,   69  ,   19  ),  "Saddle Brown");
  insertColor(qRgb( 148 ,   0   ,   211 ),  "Darkviolet");
  insertColor(qRgb( 178 ,   34  ,   34  ),  "Fire Brick");
  insertColor(qRgb( 72  ,   61  ,   139 ),  "Dark Slate Blue");
  insertColor(qRgb( 47  ,   79  ,   79  ),  "Dark Slate Gray");
  insertColor(qRgb( 25  ,   25  ,   112 ),  "Midnight Blue");
  insertColor(qRgb( 0   ,   0   ,   205 ),  "Medium Blue");
  insertColor(qRgb( 0   ,   0   ,   128 ),  "Navy");

  blockSignals(false);
  emit sizeChanged();
}

/****************************************************************************************/
ColorPanelButton::ColorPanelButton(const QColor &color, QWidget *parent, const char *name)
: QFrame(parent,name), active(false)
{
  pcolor = new QColor(color);
  setFixedSize(16, 16);
  setFrameStyle(NoFrame);
}

ColorPanelButton::~ColorPanelButton()
{
  delete pcolor;
}

void ColorPanelButton::enterEvent(QEvent *)
{
  if (!active)
    setFrameStyle(Panel | Sunken);
}

void ColorPanelButton::leaveEvent(QEvent *)
{
  if (!active)
    setFrameStyle(NoFrame);
}

void ColorPanelButton::paintEvent(QPaintEvent *ev)
{
  QFrame::paintEvent(ev);

  QPainter p(this);
  if (pcolor->isValid()) {
    p.fillRect(2, 2, 12, 12, *pcolor);
    p.setPen(Qt::gray);
    p.drawRect(2 , 2, 12, 12);
  } else {
    p.fillRect(2, 2, 12, 12, Qt::white);
    p.setPen(Qt::gray);
    p.drawLine(2, 2, 13, 13);
    p.drawLine(13, 2, 2, 13);
    p.drawRect(2 , 2, 12, 12);
  }
}

void ColorPanelButton::setActive(bool f)
{
  active = f;
  if (active)
    setFrameStyle( Panel | Sunken );
  else
    setFrameStyle( NoFrame );
}

void ColorPanelButton::mouseReleaseEvent(QMouseEvent *)
{
  emit selected(*pcolor);
}
