/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include "tkcustomwindow.h"

#include <qlayout.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qcursor.h>
#include <qapplication.h>

TKCustomWindowCaption::TKCustomWindowCaption(TKCustomWindow *parent, const char *name)
: QWidget(parent, name, WResizeNoErase | WRepaintNoErase), window(parent)
{
  setBackgroundMode(Qt::NoBackground);

  QFont f(font());
  f.setBold(true);
  setFont(f);
  setFixedHeight(QMAX(20, QFontMetrics(f).lineSpacing() + 4));

  pressClose = false;
  underClose = false;
  pressFold = false;
  underFold = false;

  press = false;
  setMouseTracking(true);
  connect(window, SIGNAL(updateResizes()), SLOT(repaint()));

  window->installEventFilter(this);
  updateCaption();
}

TKCustomWindowCaption::~TKCustomWindowCaption()
{
}

void TKCustomWindowCaption::resizeEvent(QResizeEvent *ev)
{
  updateCaption();
  QWidget::resizeEvent(ev);
}

void TKCustomWindowCaption::updateCaption()
{
  cutCaption = window->caption();

  QFontMetrics fm(font());
  int w = width() - 20 - 4 - 2 * (height() - 2);
  if (fm.width(cutCaption) > w) {
    int i = cutCaption.length();
    while ((fm.width(cutCaption.left(i) + "...")  > w) && i > 0)
      i--;
    cutCaption = cutCaption.left(i) + "...";
  }
}

bool TKCustomWindowCaption::eventFilter(QObject *obj, QEvent *ev)
{
  if (obj == window) {
    if (ev->type() == QEvent::CaptionChange) {
      updateCaption();
      repaint();
    }
    if (ev->type() == QEvent::IconChange)
      repaint();
  }
  return QWidget::eventFilter(obj, ev);
}

void TKCustomWindowCaption::mouseMoveEvent(QMouseEvent *ev)
{
  if (!press) {
    QRect button(0, 0, height() - 2, height() - 2);
    button.moveBy(width() - 1 - button.width(), 1);
    bool f = button.contains(ev->pos()) && !pressFold;
    if (f && !underClose) {
      underClose = true;
      repaint();
    }
    if (!f && underClose) {
      underClose = false;
      repaint();
    }

    if (window->canFold) {
      button.moveBy(-button.width(), 0);
      f = button.contains(ev->pos()) && !pressClose;
      if (f && !underFold) {
        underFold = true;
        repaint();
      }
      if (!f && underFold) {
        underFold = false;
        repaint();
      }
    }
    return;
  }


  if (window->testWFlags(WStyle_Dialog) != 0 && window->canMove) {
    QPoint p = ev->globalPos();
    p -= pos;

    int x = geometry.topLeft().x();
    int y = geometry.topLeft().y();

    x += p.x();
    y += p.y();

    window->move(x, y);
  } else {
    QMouseEvent *e = new QMouseEvent(QEvent::MouseMove, window->mapFromGlobal(ev->globalPos()), ev->globalPos(), ev->button(), ev->state());
    QApplication::sendEvent(window, e);
  }
}

void TKCustomWindowCaption::mousePressEvent(QMouseEvent *ev)
{
  QRect button(0, 0, height() - 2, height() - 2);
  button.moveBy(width() - 1 - button.width(), 1);
  if (button.contains(ev->pos())) {
    pressClose = true;
    repaint();
    return;
  }

  if (window->canFold) {
    button.moveBy(-button.width(), 0);
    if (button.contains(ev->pos())) {
      pressFold = true;
      repaint();
      return;
    }
  }

  press = true;

  if (window->testWFlags(WStyle_Dialog) != 0 && window->canMove) {
    window->resizePressed = true;
    emit window->updateResizes();
    pos = ev->globalPos();
    geometry = window->geometry();
    setCursor(sizeAllCursor);
  } else {
    QPoint pos = ev->globalPos();
    QMouseEvent *e = new QMouseEvent(QEvent::MouseButtonPress, window->mapFromGlobal(pos), pos, ev->button(), ev->state());
    QApplication::sendEvent(window, e);
  }
}

void TKCustomWindowCaption::mouseReleaseEvent(QMouseEvent *ev)
{
  if (pressClose) {
    pressClose = false;
    repaint();
    if (underClose)
      window->close();
    return;
  }

  if (pressFold) {
    pressFold = false;
    repaint();
    if (underFold) {
      if (window->widget()->isVisible()) {
        storedHeight = window->height();
        window->widget()->hide();
        window->layout()->activate();
        window->resize(window->width(), window->minimumSizeHint().height());
      } else {
        window->widget()->show();
        window->resize(window->width(), storedHeight);
      }
      repaint();
    }
    return;
  }

  press = false;

  if (window->testWFlags(WStyle_Dialog) != 0 && window->canMove) {
    window->resizePressed = false;
    emit window->updateResizes();
    setCursor(arrowCursor);
  } else {
    QMouseEvent *e = new QMouseEvent(QEvent::MouseButtonRelease, window->mapFromGlobal(ev->globalPos()), ev->globalPos(), ev->button(), ev->state());
    QApplication::sendEvent(window, e);
  }
}

QSize TKCustomWindowCaption::minimumSizeHint() const
{
  int b = height() - 2;
  int w = 2 + 20 + 2 * b + 4;
  return QSize(w, height());
}

void TKCustomWindowCaption::paintEvent(QPaintEvent *)
{
  QPixmap buffer(size());

  QColor bg = colorGroup().background();
  buffer.fill(bg.dark(110));

  QPainter p(&buffer, this);

  QRect r0(0, 0, width() , height());
  p.setPen(colorGroup().midlight());
  p.drawRect(r0);

  QRect button(0, 0, height() - 2, height() - 2);
  
  // close button
  button.moveBy(width() - 1 - button.width(), 1);

  if (pressClose || underClose) {
    if (!pressClose || !underClose)
      p.fillRect(button, bg.dark(120));
    p.setPen(bg.dark(130));
    p.drawRect(button);
  }

  p.setPen(black);
  int x = button.x() + (button.width() - 8) / 2;
  int y = button.y() + (button.width() - 8) / 2;
  p.drawLine(x, y, x + 6, y + 6);
  p.drawLine(x + 1, y, x + 7, y + 6);

  p.drawLine(x + 7, y, x + 1, y + 6);
  p.drawLine(x + 6, y, x, y + 6);

  // folding button
  if (window->canFold) {
    button.moveBy(-button.width(), 0);

    if (pressFold || underFold) {
      if (!pressFold || !underFold)
        p.fillRect(button, bg.dark(120));
      p.setPen(bg.dark(130));
      p.drawRect(button);
    }

    p.setPen(black);
    x = button.x() + (button.width() - 8) / 2;
    y = button.y() + (button.width() - 5) / 2;

    if (window->widget()->isVisible()) {
      p.moveTo(x, y);
      p.lineTo(x + 8, y);
      p.lineTo(x + 4, y + 4);
      p.lineTo(x, y);

      p.moveTo(x + 2, y + 1);
      p.lineTo(x + 6, y + 1);
      p.lineTo(x + 4, y + 3);
      p.lineTo(x + 2, y + 1);

      p.drawPoint(x + 4, y + 2);
    } else {
      p.moveTo(x, y + 4);
      p.lineTo(x + 8, y + 4);
      p.lineTo(x + 4, y);
      p.lineTo(x, y + 4);

      p.moveTo(x + 2, y + 3);
      p.lineTo(x + 6, y + 3);
      p.lineTo(x + 4, y + 1);
      p.lineTo(x + 2, y + 3);

      p.drawPoint(x + 4, y + 2);
    }
  }

  const QPixmap *icon = window->icon();
  if (icon && !icon->isNull())
    p.drawPixmap(2, (height() - icon->height()) / 2, *icon);

  x = icon && !icon->isNull() ? 20 : 2;
  p.drawText(x, 2, button.x() - x - 4, height() - 4, AlignLeft | AlignBottom, cutCaption);

  bitBlt(this, 0, 0, &buffer);
}

void TKCustomWindowCaption::leaveEvent(QEvent *)
{
  if (underClose) {
    underClose = false;
    repaint();
  }
  if (underFold) {
    underFold = false;
    repaint();
  }
}

TKCustomWindowResizer::TKCustomWindowResizer(int flag, TKCustomWindow *parent, const char *name)
: QWidget(parent, name), window(parent), flag(flag)
{
  if (flag == AlignTop || flag == AlignBottom)
    setCursor(SizeVerCursor);
  else
    if (flag == AlignLeft || flag == AlignRight)
      setCursor(SizeHorCursor);
    else
      if (flag == (AlignLeft | AlignTop) || flag == (AlignRight | AlignBottom))
        setCursor(SizeFDiagCursor);
      else
        if (flag == (AlignLeft | AlignBottom) || flag == (AlignRight | AlignTop))
          setCursor(SizeBDiagCursor);

  if (flag == AlignTop || flag == AlignBottom)
    setFixedHeight(3);
  if (flag == AlignLeft || flag == AlignRight)
    setFixedWidth(3);

  connect(window, SIGNAL(updateResizes()), SLOT(repaint()));
  connect(window, SIGNAL(enableResizers(bool)), SLOT(enableResizer(bool)));
}

TKCustomWindowResizer::~TKCustomWindowResizer()
{
}

void TKCustomWindowResizer::paintEvent(QPaintEvent *)
{
  QPainter p(this, this);

  QColor b = colorGroup().mid();
  if (!window->resizePressed)
    b = b.light(120);

  QRect r = rect();
  if (flag == AlignTop)
    r.setRect(0, 0, width(), 2);
  else
    if (flag == AlignBottom)
      r.setRect(0, 1, width(), 2);
    else
      if (flag == AlignLeft)
        r.setRect(0, 0, 2, height());
      else
        if (flag == AlignRight)
          r.setRect(1, 0, 2, height());

  p.fillRect(r, b);
}

void TKCustomWindowResizer::mouseMoveEvent(QMouseEvent *ev)
{
  if (!window->resizePressed)
    return;

  QPoint p = ev->globalPos();
  p -= pos;

  int x = geometry.topLeft().x();
  int y = geometry.topLeft().y();
  int w = geometry.size().width();
  int h = geometry.size().height();

  int maxh = window->layout()->maximumSize().height();
  int maxw = window->layout()->maximumSize().width();
  int minh = window->layout()->minimumSize().height();
  int minw = window->layout()->minimumSize().width();

  if (flag & AlignRight)
    w = QMIN(maxw, QMAX(minw, w + p.x()));
  else
    if (flag & AlignLeft) {
      w = QMIN(maxw, QMAX(minw, w - p.x()));
      x -= w -  geometry.size().width();
    }

  if (flag & AlignBottom)
    h = QMIN(maxh, QMAX(minh, h + p.y()));
  else
    if (flag & AlignTop) {
      h = QMIN(maxh, QMAX(minh, h - p.y()));
      y -= h -  geometry.size().height();
    }

  window->move(x, y);
  window->resize(w, h);
}

void TKCustomWindowResizer::mousePressEvent(QMouseEvent *ev)
{
  if (!window->widget()->isVisible() && !(flag == AlignLeft || flag == AlignRight))
    return;

  window->resizePressed = true;
  emit window->updateResizes();
  pos = ev->globalPos();
  geometry = window->geometry();
}

void TKCustomWindowResizer::mouseReleaseEvent(QMouseEvent *)
{
  if (!window->resizePressed)
    return;

  window->resizePressed = false;
  emit window->updateResizes();
}

void TKCustomWindowResizer::enableResizer(bool enable)
{
  if (enable)
    show();
  else
    hide();
}

Qt::WFlags TKCustomWindow::wflags(bool manage)
{
  return WType_Dialog | (manage ? WStyle_Customize | WStyle_NoBorder : Qt::WStyle_Tool);
}

TKCustomWindow::TKCustomWindow(QWidget *widget, bool manage, QWidget *parent, const char *name)
: QWidget(parent, name, TKCustomWindow::wflags(manage)),
  w(widget), captionBar(0), resizePressed(false), canFold(true), canMove(true)
{
  QGridLayout *layout = new QGridLayout(this, 0, 0, 0, 0);
  w->reparent(this, QPoint(0, 0));
  if (manage) {
    captionBar = new TKCustomWindowCaption(this);

    layout->addWidget(new TKCustomWindowResizer(AlignLeft | AlignTop, this), 0, 0);
    layout->addWidget(new TKCustomWindowResizer(AlignRight | AlignTop, this), 0, 2);
    layout->addWidget(new TKCustomWindowResizer(AlignLeft | AlignBottom, this), 3, 0);
    layout->addWidget(new TKCustomWindowResizer(AlignRight | AlignBottom, this), 3, 2);
    layout->addWidget(new TKCustomWindowResizer(AlignTop, this), 0, 1);
    layout->addWidget(new TKCustomWindowResizer(AlignBottom, this), 3, 1);
  
    layout->addWidget(captionBar, 1, 1);
    layout->addMultiCellWidget(new TKCustomWindowResizer(AlignLeft, this), 1, 2, 0, 0);
    layout->addMultiCellWidget(new TKCustomWindowResizer(AlignRight, this), 1, 2, 2, 2);

    layout->addWidget(w, 2, 1);
    layout->setRowStretch(2, 1);
    layout->setColStretch(1, 1);
  } else
    layout->addWidget(w, 0, 0);

  layout->activate();

  int mw = layout->maximumSize().width();
  int mh = layout->maximumSize().height();
  int w = sizeHint().width();
  int h = sizeHint().height();
  resize(QMIN(mw, w), QMIN(mh, h));
}

TKCustomWindow::~TKCustomWindow()
{
}

void TKCustomWindow::showCaption(bool isShow)
{
  if (!captionBar)
    return;

  if (isShow)
    captionBar->show();
  else
    captionBar->hide();
}

bool TKCustomWindow::event(QEvent *ev)
{
  if (ev->type() == QEvent::Show || ev->type() == QEvent::Reparent) {
    canFold = testWFlags(WStyle_Dialog) != 0;
    emit enableResizers(canFold);
    layout()->activate();
  }    
  return QWidget::event(ev);
}

void TKCustomWindow::setOpaqueMoving(bool opaque)
{
  canMove = !opaque;  
}
