/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef TKCUSTOMWINDOW_H
#define TKCUSTOMWINDOW_H

#include <qwidget.h>

class TKCustomWindow;

class TKCustomWindowCaption : public QWidget
{ Q_OBJECT
public:
  TKCustomWindowCaption(TKCustomWindow *parent=0, const char *name=0);
  ~TKCustomWindowCaption();

  virtual QSize minimumSizeHint() const;
  virtual bool eventFilter(QObject *, QEvent *);

protected:
  void updateCaption();

  virtual void resizeEvent(QResizeEvent *);
  virtual void paintEvent(QPaintEvent *);
  virtual void mouseMoveEvent(QMouseEvent *);
  virtual void mousePressEvent(QMouseEvent *);
  virtual void mouseReleaseEvent(QMouseEvent *);
  virtual void leaveEvent(QEvent *);

private:
  TKCustomWindow *window;
  int storedHeight;
  QPoint pos;
  QRect geometry;

  bool press;
  bool pressClose;
  bool underClose;
  bool pressFold;
  bool underFold;

  QString cutCaption;
};

class TKCustomWindowResizer : public QWidget
{ Q_OBJECT
public:
  TKCustomWindowResizer(int flag, TKCustomWindow *parent=0, const char *name=0);
  ~TKCustomWindowResizer();

protected:
  virtual void paintEvent(QPaintEvent *);
  virtual void mouseMoveEvent(QMouseEvent *);
  virtual void mousePressEvent(QMouseEvent *);
  virtual void mouseReleaseEvent(QMouseEvent *);

protected slots:
  void enableResizer(bool);

private:
  TKCustomWindow *window;
  int flag;
  QPoint pos;
  QRect geometry;
};

class TKCustomWindow : public QWidget
{ Q_OBJECT
  friend class TKCustomWindowResizer;
  friend class TKCustomWindowCaption;
public:
  TKCustomWindow(QWidget *widget, bool manage, QWidget *parent=0, const char *name=0);
  ~TKCustomWindow();
  
  QWidget *widget() { return w; }

  void showCaption(bool);

  static Qt::WFlags wflags(bool manage);

  void setOpaqueMoving(bool);
  bool opaqueMoving() { return !canMove; }

  virtual bool event(QEvent *);

signals:
  void updateResizes();
  void enableResizers(bool);

private:
  QWidget *w;
  TKCustomWindowCaption *captionBar;
  bool resizePressed;
  bool canFold;
  bool canMove;
};

#endif
