/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include "tkdockmanager.h"
#include "tktabwidget.h"

#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qobjectlist.h>
#include <qlayout.h>

#include "tksplitter.h"

#include <settings.h>

TKDockManager::TKDockManager(const QString &settingsKey, QWidget *mainWidget, const char *name, bool createMainDockArea, QWidget *centerWidget)
: QObject(0, name), mainWidget(mainWidget), md(0), skey(settingsKey)
{
  notDeleteTabDock = false;
  deletedTabDock = 0;

  reading = false;
  tabDragDock = 0;
  
  if (skey.right(1) != "/")
    skey += "/";

  centerDock = 0;
  if (createMainDockArea && mainWidget->inherits("QMainWindow")) {
    QMainWindow *mainWindow = (QMainWindow *)mainWidget;
    centerDock = createDockArea(mainWindow, "centerArea");
    mainWindow->setCentralWidget(centerDock);
    md = create(centerWidget ? centerWidget : new QWidget(), "mainDock", -1);
    md->maindock = true;
    md->setCaptionEnabled(false);
    md->setCaption("Workspace");
    md->dock(centerDock, TKDockWidget::Left);
  }
}

TKDockManager::~TKDockManager()
{
  QPtrList<TKDockWidget> list = dockWidgetList();
  for (TKDockWidget *dock = list.first(); dock; dock = list.next()) {
    delete dock;
    qApp->processEvents();
  }
  for (TKDockWidget *area = areas.first(); area; area = areas.next())
    delete area;
}

void TKDockManager::addActionGroup(QActionGroup *group, int groupId)
{
  group->setExclusive(false);
  groups.insert(groupId, group); 
}

QActionGroup *TKDockManager::actions(bool usesDropDown, int groupId)
{
  QActionGroup *group = groups[groupId];
  if (group)
    group->setUsesDropDown(usesDropDown);
  return group;
}

TKDockWidget *TKDockManager::create(QWidget *widget, const char *name, int groupId)
{
  widget->installEventFilter(this);
  TKDockWidget *dock = new TKDockWidget(this, widget, TKDockWidget::Base, mainWidget, name, groupId);
  dock->installEventFilter(this);
  connect(dock, SIGNAL(docked()), SLOT(updateTabsPosition()));
  connect(dock, SIGNAL(undocked()), SLOT(updateTabsPosition()));

  docks.insert(name, dock);
  w2d.insert(widget, dock);
  d2w.insert(dock, widget);

  connect(dock, SIGNAL(destroyed()), SLOT(dockDestroyed()));

  QActionGroup *group = groups[groupId];
  if (group)
    group->add(dock->action());

  return dock;
}

TKDockWidget *TKDockManager::createGroup(QWidget *widget, TKDockWidget::DockStatus status, const char *name)
{
  static int staticId = 0;
  widget->installEventFilter(this);
  QString rname = name;
  if (rname.isEmpty()) {
    do {
      rname = QString("%1").arg(staticId++);
    } while (docks[rname]);
  }

  TKDockWidget *dock = new TKDockWidget(this, widget, status, mainWidget, rname);
  dock->installEventFilter(this);
  connect(dock, SIGNAL(docked()), SLOT(updateTabsPosition()));
  connect(dock, SIGNAL(undocked()), SLOT(updateTabsPosition()));

  docks.insert(rname, dock);
  w2d.insert(widget, dock);
  d2w.insert(dock, widget);

  connect(dock, SIGNAL(destroyed()), SLOT(dockDestroyed()));
  return dock;
}

bool TKDockManager::eventFilter(QObject *obj, QEvent *ev)
{
  if (obj->isWidgetType()) {
    QWidget *widget = (QWidget *)obj;
    if (w2d[widget]) {
      TKDockWidget *dock = w2d[widget];
      if (ev->type() == QEvent::CaptionChange)
        dock->setCaption(widget->caption());
      if (ev->type() == QEvent::IconChange)
        if (widget->icon())
          dock->setIcon(*widget->icon());
        else
          dock->setIcon(QPixmap());
    }
    if (widget->isA("TKDockWidget") && d2w[(TKDockWidget *)widget]) {
      TKDockWidget *dock = (TKDockWidget *)widget;
      if (ev->type() == QEvent::IconChange || ev->type() == QEvent::CaptionChange) {
        if (dock->parentDock)
          updateCaption(dock->parentDock);
      } else
        if (ev->type() == QEvent::ShowToParent ||
            ev->type() == QEvent::HideToParent ||
            ev->type() == QEvent::Show ||
            ev->type() == QEvent::Hide ||
            ev->type() == QEvent::Close)
        {
          updateActions();
        }
    }
  }
  return QObject::eventFilter(obj, ev);
}

void TKDockManager::dockDestroyed()
{
  debug("Dock destroyed: %s", sender()->name());

  docks.remove(sender()->name());
  QWidget *widget = d2w[(TKDockWidget *)sender()];
  w2d.remove(widget);
  d2w.remove((TKDockWidget *)sender());
}

TKDockWidget *TKDockManager::findTarget(TKDockWidget *source, const QPoint &point, QRect &rect, TKDockWidget::DockPos &pos, TKDockWidget::DockStatus status)
{
  QWidget *widget = QApplication::widgetAt(point, true);
  bool isMainDock = false;

  for (; widget; widget = widget->parentWidget()) {
    if (widget == source->widget())
      continue;

    if (mainDock() && widget == mainDock()->widget()) {
      isMainDock = true;
      break;
    }

    if (!w2d[widget] || w2d[widget]->status != status)
      continue;

    break;
  }
//  while (widget && (widget == source->widget() || w2d.count(widget) == 0 || w2d[widget]->status != status))
//    widget = widget->parentWidget();

  if (!widget) { // check for dockArea
    QWidget *widget = QApplication::widgetAt(point, true);
    QPtrList<QWidget> parents;
    for (TKDockWidget *dock = areas.first(); dock; dock = areas.next())
      if (dock->parent() && parents.findRef(dock->parentWidget()) == -1)
        parents.append(dock->parentWidget());

    while (widget) {
      if (parents.findRef(widget) != -1)
        for (TKDockWidget *dock = areas.first(); dock; dock = areas.next())
          if (dock->parentWidget() == widget) {
            if (((TKDockSplitter *)(dock->widget()))->widgetCount() != 0)
              return 0;

            int b = 20;
            QRect r = QRect(dock->x() - b, dock->y() - b, dock->width() + 2 * b, dock->height() + 2 * b);
            r = r.intersect(widget->rect());
            if (r.contains(widget->mapFromGlobal(point))) {
              rect = r;
              rect.moveTopLeft(widget->mapToGlobal(rect.topLeft()));
              pos = TKDockWidget::Left;
              return dock;
            }
          }

      widget = widget->parentWidget();
    }
    return 0;
  }

  if(!isMainDock && !w2d[widget]->acceptDrops())
    return 0;

  QPoint p = widget->mapFromGlobal(point);
  int dw = widget->width();
  int dh = widget->height();
  int w = dw / 3;
  int h = dh / 3;

  int xp = QMIN(p.x() / w, 2);
  int yp = QMIN(p.y() / h, 2);

  if (xp == 0 && yp == 0)
    pos = p.x() > p.y() ? TKDockWidget::Top : TKDockWidget::Left;

  if (xp == 1 && yp == 0)
    pos = TKDockWidget::Top;

  if (xp == 2 && yp == 0)
    pos = (dw - p.x()) > p.y() ? TKDockWidget::Top : TKDockWidget::Right;

  if (xp == 0 && yp == 1)
    pos = TKDockWidget::Left;

  if (xp == 1 && yp == 1)
    pos = TKDockWidget::Center;

  if (xp == 2 && yp == 1)
    pos = TKDockWidget::Right;

  if (xp == 0 && yp == 2)
    pos = p.x() > (dh - p.y()) ? TKDockWidget::Bottom : TKDockWidget::Left;

  if (xp == 1 && yp == 2)
    pos = TKDockWidget::Bottom;

  if (xp == 2 && yp == 2)
    pos = (dw - p.x()) > (dh - p.y()) ? TKDockWidget::Bottom : TKDockWidget::Right;

  switch (pos) {
    case TKDockWidget::Left:
      rect = QRect(0, 0, w, dh);
      break;
    case TKDockWidget::Right:
      rect = QRect(dw - w, 0, w, dh);
      break;
    case TKDockWidget::Top:
      rect = QRect(0, 0, dw, h);
      break;
    case TKDockWidget::Bottom:
      rect = QRect(0, dh - h, dw, h);
      break;
    case TKDockWidget::Center:
      rect = QRect(w, h, w, h);
      break;
  }
  rect.moveTopLeft(widget->mapToGlobal(rect.topLeft()));

  if (isMainDock)
    return pos != TKDockWidget::Center ? mainDock() : 0;

  return w2d[widget];
}

QPtrList<TKDockWidget> TKDockManager::dockWidgetList()
{
  QPtrList<TKDockWidget> list;
  QDictIterator<TKDockWidget> it(docks);
  while (it.current()) {
    if (it.current()->status == TKDockWidget::Base)
      list.append(it.current());
    ++it;
  }
  return list;
}

void TKDockManager::removeSettingsEntry(QSettings *s, const QString &key)
{
  QStringList el = s->entryList(key);
  for (QStringList::Iterator eit = el.begin(); eit != el.end(); ++eit)
    s->removeEntry(*eit);

  QStringList sl = s->subkeyList(key);
  for (QStringList::Iterator kit = sl.begin(); kit != sl.end(); ++kit)
    removeSettingsEntry(s, *kit);
}

QString TKDockManager::settingsKey(bool full)
{
  return full ? skey + name() + "/" : skey;
}

void TKDockManager::save()
{
  save(QString::null);
}

void TKDockManager::save(const QString &name)
{
  Settings * s = Settings::instance();

  QString keyb = settingsKey(true);
  QString sname = name;

  s->writeEntry(keyb + "LastConfig", "DockManager");

  keyb += sname + "/";

  QStringList areaList;
  for (TKDockWidget *dock = areas.first(); dock; dock = areas.next()) {
    QString key = keyb + "Area/" + dock->name() + "/";
    TKDockSplitter *p = (TKDockSplitter *)dock->widget();
    QStringList names;
    for (QWidget *w = p->firstWidget(); w; w = p->nextWidget(w))
      names << w->name();
    s->writeEntry(key + "dokers", names);
    QValueList<int> sizes = p->sizes();
    QStringList sizess;
    for (QValueList<int>::Iterator it = sizes.begin(); it != sizes.end(); it++)
      sizess << QString::number(*it);
    s->writeEntry(key + "sizes", sizess);
    areaList << dock->name();
  }
  s->writeEntry(keyb + "Area/names", areaList);

  QPtrList<TKDockWidget> readyDock;

  QStringList baseList;
  QStringList groupList;
  QStringList topList;
  QDictIterator<TKDockWidget> it(docks);

  while (it.current()) {
    TKDockWidget *dock = it.current();
    if (dock->parentWidget() == mainWidget) {
      topList << dock->name();
      QString key = keyb + "TopLevel/" + dock->name() + "/";
      s->writeEntry(key + "visible", dock->isVisible());
      s->writeEntry(key + "geometry", QString("%1,%2,%3,%4").arg(dock->x()).arg(dock->y()).arg(dock->width()).arg(dock->height()));
      s->writeEntry(key + "fold", dock->isVisible() && !dock->widget()->isVisible());
    }
    if (dock->status == TKDockWidget::Base) {
      readyDock.append(dock);
      baseList << dock->name();
    }
    ++it;
  }
  s->writeEntry(keyb + "Base/names", baseList);
  s->writeEntry(keyb + "TopLevel/names", topList);

  bool found = true;
  while (found) {
    found = false;
    it.toFirst();
    while (it.current()) {
      TKDockWidget *dock = (TKDockWidget *)it.current();
      ++it;
      if (readyDock.findRef(dock) != -1)
        continue;
      if (dock->status != TKDockWidget::Base) {
        bool ready = true;
        if (dock->status == TKDockWidget::SplitGroup) {
          TKDockSplitter *p = (TKDockSplitter *)dock->widget();
          for (QWidget *w = p->firstWidget(); w; w = p->nextWidget(w))
            if (readyDock.findRef((TKDockWidget *)w) == -1) {
              ready = false;
              break;
            }
        } else {
          TKTabWidget *t = (TKTabWidget *)dock->widget();
          for (QWidget *w = t->firstPage(); w; w = t->nextPage(w))
            if (readyDock.findRef((TKDockWidget *)w) == -1) {
              ready = false;
              break;
            }
        }
        if (ready) {
          found = true;
          readyDock.append(dock);
          groupList << dock->name();
          QString key = keyb + "Group/" + dock->name() + "/";
          QStringList names;
          if (dock->status == TKDockWidget::SplitGroup) {
            s->writeEntry(key + "type", "split");
            TKDockSplitter *p = (TKDockSplitter *)dock->widget();
            for (QWidget *w = p->firstWidget(); w; w = p->nextWidget(w))
              names << w->name();
            s->writeEntry(key + "dokers", names);
            s->writeEntry(key + "orientation", p->orientation() == Qt::Vertical ? "vertical":"horizontal");
            QValueList<int> sizes = p->sizes();
            QStringList sizess;
            for (QValueList<int>::Iterator it = sizes.begin(); it != sizes.end(); it++)
              sizess << QString::number(*it);
            s->writeEntry(key + "sizes", sizess);
          } else {
            s->writeEntry(key + "type", "tab");
            TKTabWidget *t = (TKTabWidget *)dock->widget();
            for (QWidget *w = t->firstPage(); w; w = t->nextPage(w))
              names << w->name();
            s->writeEntry(key + "dokers", names);
            if (t->currentPage())
              s->writeEntry(key + "current", t->currentPage()->name());
          }
        }
      }
    }
  }
  s->writeEntry(keyb + "Group/names", groupList);

  if (mainWidget && mainWidget->isTopLevel())
    s->writeEntry(keyb + "MainWidgetGeometry", QString("%1,%2,%3,%4").arg(mainWidget->x()).arg(mainWidget->y()).arg(mainWidget->width()).arg(mainWidget->height()));
}

void TKDockManager::read()
{
  read(QString::null);
}

void TKDockManager::read(const QString &name)
{
  Settings * s = Settings::instance();
  QStringList::Iterator it;
  QPtrList<TKDockWidget> list = dockWidgetList();
  TKDockWidget *dock = 0;
  for (dock = list.first(); dock; dock = list.next())
    dock->undock();

  if (md)
    md->dock(centerDock, TKDockWidget::Left);

  QString keyb = settingsKey(true);

  QString sname = name;

  reading = true;

  keyb += sname + "/";

  // check if all base dock are exists, and create notexists dock in autoDeleteList
  QPtrList<TKDockWidget> *autoDeleteList = new QPtrList<TKDockWidget>();
  autoDeleteList->setAutoDelete(true);

  QStringList baseList = s->readListEntry(keyb + "Base/names");
  for (it = baseList.begin(); it != baseList.end(); ++it)
    if (!docks[*it])
      autoDeleteList->append(create(new QWidget(), *it));

  QStringList groupList = s->readListEntry(keyb + "Group/names");
  for (it = groupList.begin(); it != groupList.end(); ++it) {
    QString key = keyb + "Group/" + (*it) + "/";

    QStringList names = s->readListEntry(key + "dokers");
    if (names.isEmpty())
      continue;

    QStringList::Iterator itn = names.begin();
    TKDockWidget *f = docks[*itn];
    if (!f)
      continue;

    ++itn;

    if (s->readEntry(key + "type") == "split") {
      TKDockWidget::DockPos pos =  s->readEntry(key + "orientation") == "vertical" ? TKDockWidget::Bottom : TKDockWidget::Right;
      TKDockWidget *splitDock = 0;
      for (; itn != names.end(); ++itn) {
        TKDockWidget *t = docks[*itn];
        splitDock = t ? t->dock(splitDock ? splitDock : f, pos, *it) : 0;
        if (!splitDock)
          break;
      }
      if (splitDock) {
        QValueList<int> sizes;
        QStringList sizess = s->readListEntry(key + "sizes");
        for (QStringList::Iterator sit = sizess.begin(); sit != sizess.end(); sit++) {
          sizes << (*sit).toInt();
        }
        ((TKDockSplitter *)splitDock->widget())->setSizes(sizes);
      }
    } else {
      TKDockWidget *tabDock = 0;
      for (; itn != names.end(); ++itn) {
        TKDockWidget *t = docks[*itn];
        tabDock = t ? t->dock(f, TKDockWidget::Center, *it) : 0;
        if (!tabDock)
          break;
      }
      QString cn = s->readEntry(key + "current");
      TKDockWidget *c = cn.isEmpty() ? 0:docks[cn];
      if (c && tabDock)
        ((TKTabWidget *)tabDock->widget())->setCurrentPage(c);
    }
  }
/******************************************************************************/
  QStringList areaList = s->readListEntry(keyb + "Area/names");
  for (it = areaList.begin(); it != areaList.end(); ++it) {
    TKDockWidget *area = 0;
    for (area = areas.first(); area; area = areas.next())
      if (*it == area->name())
        break;

    if (!area)
      continue;

    QString key = keyb + "Area/" + (*it) + "/";
    QStringList names = s->readListEntry(key + "dokers");
    QStringList::Iterator itn = names.begin();
    for (; itn != names.end(); ++itn) {
      TKDockWidget *t = docks[*itn];
      if (!t || !t->dock(area, TKDockWidget::Right))
        break;
    }
    QValueList<int> sizes;
    QStringList sizess = s->readListEntry(key + "sizes");
    for (QStringList::Iterator sit = sizess.begin(); sit != sizess.end(); sit++)
      sizes << (*sit).toInt();
    ((TKDockSplitter *)area->widget())->setSizes(sizes);
  }
/******************************************************************************/
  if (mainWidget && mainWidget->isTopLevel()) {
    QStringList g = QStringList::split(",", s->readEntry(keyb + "MainWidgetGeometry"));
    mainWidget->move((*g.at(0)).toInt(), (*g.at(1)).toInt());
    mainWidget->resize((*g.at(2)).toInt(), (*g.at(3)).toInt());
  }

  reading = false;

  if (md) {
    QWidget *mdp = md;
    while (mdp->parentWidget() != mainWidget) mdp = mdp->parentWidget();
    if (centerDock->children()->count() == 2)
      ((TKDockWidget *)mdp)->dock(centerDock, TKDockWidget::Left);
  }
  QStringList topList = s->readListEntry(keyb + "TopLevel/names");
  for (it = topList.begin(); it != topList.end(); ++it) {
    TKDockWidget *dock = docks[*it];
    if (!dock)
      continue;
    QString key = keyb + "TopLevel/" + (*it) + "/";
    QStringList g = QStringList::split(",", s->readEntry(key + "geometry"));
    dock->move((*g.at(0)).toInt(), (*g.at(1)).toInt());
    dock->resize((*g.at(2)).toInt(), (*g.at(3)).toInt());

    if (s->readBoolEntry(key + "fold", false)) {
      dock->widget()->hide();
      dock->layout()->activate();
      dock->resize(dock->width(), dock->minimumSizeHint().height());
    }

    if (s->readBoolEntry(key + "visible"))
      dock->QWidget::show();
  }

  delete autoDeleteList;
}

void TKDockManager::updateTabsPosition()
{
  QDictIterator<TKDockWidget> it(docks);
  while (it.current()) {
    TKDockWidget *dock = it.current();
    if (dock->status == TKDockWidget::TabGroup) {
      TKTabWidget *tab = (TKTabWidget *)dock->widget();
      if (!dock->parentDock) {
        if (tab->tabPosition() != TKTabWidget::Top)
          tab->setTabPosition(TKTabWidget::Top);
      } else
        if (dock->parentSplit) {
          TKDockSplitter *s = dock->parentSplit;
          if (s->widgetCount() > 1 && s->firstWidget() == (QWidget *)dock) {
            if (s->orientation() == Qt::Horizontal && tab->tabPosition() != TKTabWidget::Left)
              tab->setTabPosition(TKTabWidget::Left);
          } else
            if (s->widgetCount() > 1 && s->lastWidget() == (QWidget *)dock) {
              if (s->orientation() == Qt::Horizontal && tab->tabPosition() != TKTabWidget::Right)
                tab->setTabPosition(TKTabWidget::Right);
            } else
              if (tab->tabPosition() != TKTabWidget::Top)
                tab->setTabPosition(TKTabWidget::Top);
      }
    }
    ++it;
  }
}

TKDockWidget *TKDockManager::createDockArea(QWidget *parent, const char *name)
{
  TKDockSplitter *s = new TKDockSplitter(0, "dockArea_splitter");
  TKDockWidget *dockArea = new TKDockWidget(this, s, TKDockWidget::SplitGroup, 0, name);
  dockArea->reparent(parent, 0, QPoint(0, 0));
  areas.append(dockArea);
  return dockArea;
}

bool TKDockManager::isDockArea(TKDockWidget *dock)
{
  return areas.findRef(dock) != -1;
}

bool TKDockManager::isDockArea(QWidget *widget)
{
  return widget->isA("TKDockWidget") ? areas.findRef((TKDockWidget *)widget) != -1 : false;
}

void TKDockManager::updateCaption(TKDockWidget *dock)
{
  if (dock->status == TKDockWidget::Base)
    return;

  QStringList names;
  if (dock->status == TKDockWidget::SplitGroup) {
    TKDockSplitter *s = (TKDockSplitter *)dock->widget();
    for (QWidget *w = s->firstWidget(); w; w = s->nextWidget(w))
      names << w->caption();
  } else {
    TKTabWidget *t = (TKTabWidget *)dock->widget();
    for (QWidget *w = t->firstPage(); w; w = t->nextPage(w))
      names << w->caption();
  }

  dock->setCaption(names.join(" | "));
}

void TKDockManager::updateActions()
{
  QPtrList<TKDockWidget> list = dockWidgetList();
  for (TKDockWidget *dock = list.first(); dock; dock = list.next()) {
    bool v = dock->isVisible();
    QAction *a = dock->action();
    if (a->isOn() != v ) {
      a->blockSignals(true);
      a->setOn(v);
      a->blockSignals(false);
    }
  }
}

void TKDockManager::popup(const QPoint &pos, TKDockWidget *dock)
{
  QActionGroup * actGroup = actions(false, dock->group());

  if( actGroup )
  {
    QPopupMenu menu;
  
    actGroup->addTo(&menu);
    emit menuBeforePopup(menu, dock);
    notDeleteTabDock = true;
    menu.exec(pos);
    notDeleteTabDock = false;
    if (deletedTabDock)
      deletedTabDock->deleteLater();
    deletedTabDock = 0;
  }
}

void TKDockManager::saveAs()
{
  save("Maximus");
}

void TKDockManager::remove()
{
}

void TKDockManager::setTabViewStyle(int style)
{
  tabStyle = style;
  QMap<QWidget *, TKDockWidget *>::Iterator it = w2d.begin();
  for (; it != w2d.end(); ++it)
    if (it.key()->inherits("TKTabWidget")) {
      TKTabWidget *tab = (TKTabWidget *)it.key();
      tab->setTabViewStyle(style);
    }
}

TKTabWidget *TKDockManager::createTabWidget()
{
  TKTabWidget *tab = new TKTabWidget(topLevel());
  tab->setTabViewStyle(tabStyle);
  connect(tab, SIGNAL(startPageDrag(QWidget *, QMouseEvent *)), SLOT(pageDragStart(QWidget *, QMouseEvent *)));
  connect(tab, SIGNAL(continuePageDrag(QMouseEvent *)), SLOT(pageDragContinue(QMouseEvent *)));
  connect(tab, SIGNAL(finishPageDrag(QMouseEvent *)), SLOT(pageDragFinish(QMouseEvent *)));
  connect(tab, SIGNAL(cancelPageDrag(QMouseEvent *)), SLOT(pageDragCancel(QMouseEvent *)));
  connect(tab, SIGNAL(closePage(QWidget *)), SLOT(pageClose(QWidget *)));
  connect(tab, SIGNAL(rightButtonPressed(QWidget *, const QPoint &)), SLOT(pageMenuPopup(QWidget *, const QPoint &)));
  return tab;
}

void TKDockManager::pageDragStart(QWidget *widget, QMouseEvent *ev)
{
  if (!d2w[(TKDockWidget *)widget])
    return;
    
  tabDragDock = (TKDockWidget *)widget;
  
  QPoint pos = ev->globalPos();
  pos = tabDragDock->mapToGlobal(QPoint(-1, -1));
  QMouseEvent *e = new QMouseEvent(QEvent::MouseButtonPress, tabDragDock->mapFromGlobal(pos), pos, Qt::LeftButton, Qt::LeftButton);
  QApplication::sendEvent(tabDragDock, e);
}

void TKDockManager::pageDragContinue(QMouseEvent *ev)
{
  if (!tabDragDock)
    return;
    
  QPoint pos = ev->globalPos();
  QMouseEvent *e = new QMouseEvent(QEvent::MouseMove, tabDragDock->mapFromGlobal(pos), pos, ev->button(), ev->state());
  QApplication::sendEvent(tabDragDock, e);
}

void TKDockManager::pageDragFinish(QMouseEvent *ev)
{
  if (!tabDragDock)
    return;
    
  QPoint pos = ev->globalPos();
  QMouseEvent *e = new QMouseEvent(QEvent::MouseButtonRelease, tabDragDock->mapFromGlobal(pos), pos, ev->button(), ev->state());
  QApplication::sendEvent(tabDragDock, e);
  tabDragDock = 0;
}

void TKDockManager::pageDragCancel(QMouseEvent *)
{
  if (!tabDragDock)
    return;
  
  tabDragDock->cancelMove();  
  tabDragDock = 0;
}

void TKDockManager::pageClose(QWidget *widget)
{
  if (!d2w[(TKDockWidget *)widget])
    return;
    
  TKDockWidget *dock = (TKDockWidget *)widget;
  dock->undock();
  updateActions();
}

void TKDockManager::pageMenuPopup(QWidget *widget, const QPoint &pos)
{
  if (!d2w[(TKDockWidget *)widget])
    return;
    
  TKDockWidget *dock = (TKDockWidget *)widget;
  popup(pos, dock);
}
