/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef TKDOCKMANAGER_H
#define TKDOCKMANAGER_H

#include <qwidget.h>
#include <qptrlist.h>
#include <qintdict.h>
#include <qaction.h>
#include <qdict.h>
#include <qmap.h>

class QSettings;

class TKTabWidget;

#include "tkdockwidget.h"

class TKDockManager : public QObject
{ Q_OBJECT
friend class TKDockWidget;
public:
  TKDockManager(const QString &settingsKey, QWidget *mainWidget, const char *name, bool createMainDockArea=false, QWidget *centerWidget=0);
  ~TKDockManager();

  TKDockWidget *mainDock() { return md; }

  QString settingsKey(bool full=false);

  TKDockWidget *create(QWidget *, const char *name, int groupId=0);
  TKDockWidget *createGroup(QWidget *, TKDockWidget::DockStatus, const char *name=0);

  TKDockWidget *findTarget(TKDockWidget *source, const QPoint &, QRect &, TKDockWidget::DockPos &, TKDockWidget::DockStatus status);

  QWidget *topLevel() { return mainWidget; }

  TKDockWidget *createDockArea(QWidget *parent, const char *name);

  QPtrList<TKDockWidget> dockWidgetList();
  bool isDockArea(TKDockWidget *); 
  bool isDockArea(QWidget *);

  void addActionGroup(QActionGroup *, int groupId=0);
  QActionGroup *actions(bool usesDropDown=false, int groupId=0);

  void save(const QString &);
  void read();

  void setTabViewStyle(int);
  int tabViewStyle() { return tabStyle; }

signals:
  void menuBeforePopup(QPopupMenu &, TKDockWidget *);

public slots:
  void save();
  void read(const QString &);

protected:
  virtual bool eventFilter(QObject *, QEvent *);
  void removeSettingsEntry(QSettings *, const QString &key);
  void updateCaption(TKDockWidget *);
  void updateActions();
  void popup(const QPoint &, TKDockWidget *);
  TKTabWidget *createTabWidget();
  
protected slots:
  void dockDestroyed();
  void updateTabsPosition();
  void saveAs();
  void remove();
  
  void pageDragStart(QWidget *, QMouseEvent *);
  void pageDragContinue(QMouseEvent *);
  void pageDragFinish(QMouseEvent *);
  void pageDragCancel(QMouseEvent *);
  void pageClose(QWidget *);
  void pageMenuPopup(QWidget *, const QPoint &);

private:
  QWidget *mainWidget;
  TKDockWidget *md;
  QString skey;
  QDict<TKDockWidget> docks;
  QMap<QWidget *, TKDockWidget *> w2d;
  QMap<TKDockWidget *, QWidget *> d2w;
  QPtrList<TKDockWidget> areas;
  QIntDict<QActionGroup> groups;
  TKDockWidget *tabDragDock;
  bool reading;

  int tabStyle;
  TKDockWidget *centerDock;

  bool notDeleteTabDock;
  TKDockWidget *deletedTabDock;
};

#endif

