/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include <qapplication.h>
#include <qaction.h>
#include <qobjectlist.h>
#include <qpainter.h>
#include <qlayout.h>

#include "tkdockwidget.h"
#include "tkdockmanager.h"
#include "tktabwidget.h"
#include "tksplitter.h"

TKDockWidget::TKDockWidget(TKDockManager* manager, QWidget *widget, DockStatus status, QWidget *parent, const char *name, int groupId)
: TKCustomWindow(widget, status == TKDockWidget::Base, parent, name),
  manager(manager), status(status), parentDock(0), maindock(false), groupId(groupId)
{
  setOpaqueMoving(true);
  captionEnabled = true;
  unclippedPainter = 0;
  moving = false;
  parentTab = 0;
  parentSplit = 0;
  setAcceptDrops(true);

  if (status == Base) {
    act = new QAction(this, 0, true);
    connect(act, SIGNAL(toggled(bool)), SLOT(makeVisible(bool)));
    setCaption(widget->caption());
    if (widget->icon())
      setIcon(*widget->icon());
  } else
    act = 0;
}

TKDockWidget::~TKDockWidget()
{
  if (!maindock) {
    debug("TKDockWidget::~TKDockWidget()");
    undock();
  } else
    debug("TKDockWidget::~TKDockWidget() - MainDock");
}

bool TKDockWidget::close(bool)
{
  undock();
  act->blockSignals(true);
  act->setOn(false);
  act->blockSignals(false);
  return true;
}

void TKDockWidget::setCaptionEnabled(bool enabled)
{
  captionEnabled = enabled;
  if (!captionEnabled)
    showCaption(false);
}

bool TKDockWidget::event(QEvent *ev)
{
  if (ev->type() == QEvent::Show && captionEnabled)
    showCaption(parentTab == 0);
 
  if (act && (ev->type() == QEvent::IconChange || ev->type() == QEvent::CaptionChange)) {
    if (icon())
      act->setIconSet(*icon());
    act->setMenuText(caption());
  }

  return TKCustomWindow::event(ev);
}

void TKDockWidget::show()
{
  if (parentTab && parentTab->currentPage() != this)
    return;

  if ((manager->reading && parentWidget() == manager->topLevel()) ||
      (parentWidget() == manager->topLevel() && !manager->topLevel()->isVisible()))
    return;

  if (QApplication::desktop()->geometry().intersect(topLevelWidget()->geometry()).isEmpty())
    topLevelWidget()->move(0, 0);

  TKCustomWindow::show();

  if (status == SplitGroup)
    ((TKDockSplitter *)widget())->refresh();
}

void TKDockWidget::insertTo(TKDockWidget *parent, QWidget *after)
{
  parentDock = parent;
  if (parent->status == SplitGroup) {
    TKDockSplitter *split = (TKDockSplitter *)parent->widget();
    parentSplit = split;
    parentTab = 0;
    split->addWidget(this);
    split->moveTo(this, after);
  } else
    if (parent->status == TabGroup) {
      TKTabWidget *tab = (TKTabWidget *)parent->widget();
      parentTab = tab;
      parentSplit = 0;
      tab->insertPage(this, after);
    } else
      parentDock = 0;

  manager->updateCaption(parent);
}

void TKDockWidget::insertTo(TKDockWidget *parent)
{
  parentDock = parent;
  if (parent->status == SplitGroup) {
    TKDockSplitter *split = (TKDockSplitter *)parent->widget();
    parentSplit = split;
    parentTab = 0;
    split->addWidget(this);
  } else
    if (parent->status == TabGroup) {
      TKTabWidget *tab = (TKTabWidget *)parent->widget();
      parentTab = tab;
      parentSplit = 0;
      tab->addPage(this);
    } else
      parentDock = 0;

  manager->updateCaption(parent);
}

TKDockWidget *TKDockWidget::dockCenter(TKDockWidget *target, const char *name)
{
  TKTabWidget *tab = 0;
  TKDockWidget *tabDock = 0;

  if (target->status == TabGroup) {
    tabDock = target;
    tab = (TKTabWidget *)target->widget();
  } else {
    if (target->parentDock && target->parentDock->status == TabGroup) {
      tabDock = target->parentDock;
      tab = (TKTabWidget *)tabDock->widget();
    } else {
      tab = manager->createTabWidget();
      tabDock = manager->createGroup(tab, TabGroup, name);
      replace(target, tabDock);
      target->insertTo(tabDock);
    }
  }

  if (status == TabGroup) {
    TKTabWidget *t = (TKTabWidget *)widget();
    do {
      TKDockWidget *page = (TKDockWidget *)t->firstPage();
      t->removePage(page);
      page->insertTo(tabDock);
    } while (t->firstPage());
    delete this;
  } else
    insertTo(tabDock);
      
  tabDock->show();

  emit docked();
  return tabDock;
}

TKDockWidget *TKDockWidget::dock(TKDockWidget *target, DockPos pos, const char *name)
{
  if (manager->isDockArea(this) || (manager->isDockArea(target) && pos == Center))
    return 0;
  
  undock();
  widget()->show();

  if (!target)
    return this;

  if (pos == Center)
    return dockCenter(target, name);

  if (manager->isDockArea(target)) {
    insertTo(target);
    show();
    emit docked();
    manager->updateCaption(target);
    return target;
  }

  if (target->status != SplitGroup && status != SplitGroup) {
    TKDockSplitter *split = new TKDockSplitter();
    TKDockWidget *splitDock = manager->createGroup(split, SplitGroup, name);
    split->setOrientation(pos == Right || pos == Left ? Horizontal : Vertical);
    replace(target, splitDock);
    insertTo(splitDock);
    target->insertTo(splitDock);
    QValueList<int> sizes;
    sizes << 0;
    sizes << 1;
    split->setSizes(sizes);
    split->moveToFirst(pos == Left || pos == Top ? this : target);
    splitDock->show();
    emit docked();
    manager->updateCaption(splitDock);
    return splitDock;
  }
  if (target->status != SplitGroup && status == SplitGroup) {
    TKDockSplitter *split = new TKDockSplitter();
    TKDockWidget *splitDock = manager->createGroup(split, SplitGroup, name);
    split->setOrientation(pos == Right || pos == Left ? Horizontal : Vertical);
    replace(target, splitDock);
    target->insertTo(splitDock);
    TKDockSplitter *thisSplit = (TKDockSplitter *)widget();
    if (split->orientation() == thisSplit->orientation()) {
      while (thisSplit->firstWidget()) {
        TKDockWidget *dock = (TKDockWidget *)thisSplit->firstWidget();
        dock->insertTo(splitDock);
      }
      delete this;
    } else {
      insertTo(splitDock);
      split->moveToFirst(pos == Left || pos == Top ? this : target);
    }
    splitDock->show();
    emit docked();
    manager->updateCaption(splitDock);
    return splitDock;
  }
  if (target->status == SplitGroup && status != SplitGroup) {
    TKDockSplitter *split = (TKDockSplitter *)target->widget();
    if ((split->orientation() == Horizontal && (pos == Right || pos == Left)) ||
        (split->orientation() == Vertical && (pos == Top || pos == Bottom))) {
      insertTo(target);
      if (pos == Left || pos == Top)
        split->moveToFirst(this);
      show();
      emit docked();
      manager->updateCaption(target);
      return target;
    } else {
      split = new TKDockSplitter();
      TKDockWidget *splitDock = manager->createGroup(split, SplitGroup, name);
      split->setOrientation(pos == Right || pos == Left ? Horizontal : Vertical);
      replace(target, splitDock);
      insertTo(splitDock);
      target->insertTo(splitDock);
      split->moveToFirst(pos == Left || pos == Top ? this : target);
      splitDock->show();
      emit docked();
      manager->updateCaption(splitDock);
      return splitDock;
    }
  }
  if (target->status == SplitGroup && status == SplitGroup) {
    TKDockSplitter *thisSplit = (TKDockSplitter *)widget();
    TKDockSplitter *targetSplit = (TKDockSplitter *)target->widget();
    Qt::Orientation orientation = pos == Right || pos == Left ? Horizontal : Vertical;

    if (thisSplit->orientation() == targetSplit->orientation() && thisSplit->orientation() == orientation) {
      while (thisSplit->firstWidget()) {
        TKDockWidget *dock = (TKDockWidget *)thisSplit->firstWidget();
        dock->insertTo(target);
        dock->show();
      }
      delete this;
      emit docked();
      manager->updateCaption(target);
      return target;
    } else {
      TKDockSplitter *split = new TKDockSplitter();
      TKDockWidget *splitDock = manager->createGroup(split, SplitGroup, name);
      split->setOrientation(orientation);
      replace(target, splitDock);
      insertTo(splitDock);
      target->insertTo(splitDock);
      split->moveToFirst(pos == Left || pos == Top ? this : target);
      splitDock->show();
      emit docked();
      manager->updateCaption(splitDock);
      return splitDock;
    }
  }
  return 0;
}

void TKDockWidget::handleMove(const QPoint &pos, QMouseEvent *ev)
{
  if (!unclippedPainter)
    return;

  QRect rect(currRect);
  DockStatus findStatus = (ev->state() & Qt::ControlButton) ? SplitGroup : (ev->state() & Qt::AltButton) ? TabGroup : Base;

  target = manager->findTarget(this, ev->globalPos(), currRect, targetPos, findStatus);

  if (!target) {
    currRect = QRect(mapToGlobal(QPoint(0, 0)), size());
    QPoint offset(mapFromGlobal(pos));
    currRect.moveBy(offset.x(), offset.y());
  }

  if (rect != currRect) {
    unclippedPainter->drawRect(rect);
    unclippedPainter->drawRect(currRect);
  }
}

void TKDockWidget::startRectDraw()
{
  qApp->processEvents();
  if (unclippedPainter)
    endRectDraw();

  unclippedPainter = new QPainter(QApplication::desktop(), true);
  unclippedPainter->setPen(QPen(gray, 2));
  unclippedPainter->setRasterOp(XorROP);
  unclippedPainter->drawRect(currRect);
}

void TKDockWidget::endRectDraw()
{
  if (!unclippedPainter)
    return;

  unclippedPainter->drawRect(currRect);
  delete unclippedPainter;
  unclippedPainter = 0;
}

void TKDockWidget::cancelMove()
{
  qApp->removeEventFilter(this);
  endRectDraw();
  moving = false;
  target = 0;
  targetPos = Center;
}

void TKDockWidget::replace(TKDockWidget *target, TKDockWidget *source)
{
  TKDockWidget *parent = target->parentDock;
  target->parentTab = 0;
  target->parentSplit = 0;
  target->parentDock = 0;

  if (!parent) {
    QRect rect(target->geometry());
    source->hide();
    if (source->parentWidget() != manager->topLevel())
      source->reparent(manager->topLevel(), TKCustomWindow::wflags(source->status == TKDockWidget::Base), QPoint(0, 0));
    source->parentTab = 0;
    source->parentSplit = 0;
    source->parentDock = 0;
    source->move(rect.topLeft());
    source->resize(rect.size());
    manager->updateCaption(source);
  } else
    if (parent->status == SplitGroup) {
      TKDockSplitter *split = (TKDockSplitter *)parent->widget();
      QValueList<int> sizes = split->sizes();
      QWidget *prev = split->prevWidget(target);
      target->undock();
      source->insertTo(parent, prev);
      split->setSizes(sizes);
    } else
      if (parent->status == TabGroup) {
        TKTabWidget *tab = (TKTabWidget *)parent->widget();
        QWidget *prev = tab->prevPage(target);
        QWidget *active = tab->currentPage() == target ? source : tab->currentPage();
        target->undock();
        source->insertTo(parent, prev);
        tab->setCurrentPage(active);
      }
}

void TKDockWidget::undock()
{
  if (manager->isDockArea(this))
    return;

  if (parentWidget() != manager->topLevel())
    reparent(manager->topLevel(), TKCustomWindow::wflags(status == TKDockWidget::Base), QPoint(0, 0));
  else
    hide();

  manager->updateCaption(this);
  if (parentDock)
    manager->updateCaption(parentDock);
 
  if (parentSplit && parentSplit->widgetCount() == 1 && !manager->isDockArea(parentDock)) {
    TKDockWidget *last = (TKDockWidget *)parentSplit->firstWidget();
    replace(parentDock, last);
    last->show();
  }

  if (parentTab && parentTab->pageCount() == 1) {
    TKDockWidget *last = (TKDockWidget *)parentTab->firstPage();
    replace(parentDock, last);
    last->show();
  }

  if (!manager->isDockArea(parentDock)) {
    if (parentSplit && parentSplit->widgetCount() == 0)
      delete parentDock;

    if (parentTab && parentTab->pageCount() == 0)
      if (manager->notDeleteTabDock)
        manager->deletedTabDock = parentDock;
      else
        delete parentDock;
  }

  parentTab = 0;
  parentSplit = 0;
  parentDock = 0;
  
  emit undocked();
}

void TKDockWidget::makeVisible(bool visible)
{
  if (!visible) {
    hide();
    undock();
  } else {
    TKDockWidget *dock = this;
    while (dock) {
      if (dock->parentTab)
        dock->parentTab->setCurrentPage(dock);

      dock = dock->parentDock;
    }
    show();
    if (topLevelWidget() != manager->topLevel())
      topLevelWidget()->show();
  }
}

bool TKDockWidget::eventFilter(QObject *obj, QEvent *ev)
{
  if (moving && ev->type() == QEvent::KeyPress && ((QKeyEvent *)ev)->key() == Qt::Key_Escape)
    cancelMove();
  return TKCustomWindow::eventFilter(obj, ev);
}

/***********************************/
void TKDockWidget::mousePressEvent(QMouseEvent *ev)
{
  moving = false;

  if (widget()->geometry().contains(ev->pos()))
    return;

  if (ev->button() == Qt::RightButton) {
    manager->popup(ev->globalPos(), this);
    return;
  }

  moving = true;
  qApp->installEventFilter(this);

  target = 0;
  targetPos = Center;

  offset = ev->pos();
  currRect = QRect(mapToGlobal(QPoint(0, 0)), size());
  startRectDraw();
}

void TKDockWidget::mouseMoveEvent(QMouseEvent *ev)
{
  if (moving && ev->pos() != offset)
    handleMove(ev->globalPos() - offset, ev);
}

void TKDockWidget::mouseReleaseEvent(QMouseEvent *ev)
{
  if (!moving)
    return;

  qApp->removeEventFilter(this);
  endRectDraw();
  moving = false;

  if (offset == ev->pos()) // we don't have move here, user only clicked
    return;

  if (target)
    dock(target, targetPos);
  else {
    if (parentWidget() != manager->topLevel())
      undock();
    move(ev->globalPos() - offset);
    show();
  }

  target = 0;
  targetPos = Center;
}
