/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef TKDOCKWIDGET_H
#define TKDOCKWIDGET_H

#include "tkcustomwindow.h"

class TKDockManager;
class TKTabWidget;
class TKDockSplitter;
class QAction;

class TKDockWidget : public TKCustomWindow
{ Q_OBJECT
friend class TKDockManager;
public:
  enum DockStatus {Base, TabGroup, SplitGroup};
  enum DockPos {Left, Right, Top, Bottom, Center};

  TKDockWidget(TKDockManager*, QWidget *widget, DockStatus, QWidget *parent, const char *name, int groupId=-1);
  ~TKDockWidget();

  TKDockWidget *dock(TKDockWidget *, DockPos, const char *name=0);
  void undock();
  void replace(TKDockWidget *, TKDockWidget *);

  void setCaptionEnabled(bool);
  
  QAction *action() { return act; }
  int group() const { return groupId; }

  virtual bool eventFilter(QObject *, QEvent *);
  virtual bool event(QEvent *);
  virtual bool close(bool);

signals:
  void docked();
  void undocked();

public slots:
  void makeVisible(bool);

  virtual void show();

protected:
  TKDockWidget *dockCenter(TKDockWidget *, const char *name);

  virtual void mouseMoveEvent(QMouseEvent *);
  virtual void mousePressEvent(QMouseEvent *);
  virtual void mouseReleaseEvent(QMouseEvent *);

  void handleMove(const QPoint &, QMouseEvent *);
  void startRectDraw();
  void endRectDraw();

  void insertTo(TKDockWidget *);
  void insertTo(TKDockWidget *, QWidget *after);

  void cancelMove();

private:
  TKDockManager* manager;
  DockStatus status;
  TKDockWidget *parentDock;
  bool maindock;
  int groupId;
  TKDockSplitter *parentSplit;
  TKTabWidget *parentTab;
  TKDockWidget *target;
  DockPos targetPos;
  QPainter *unclippedPainter;
  QRect currRect;
  QPoint offset;
  bool moving;
  bool captionEnabled;

  QAction *act;
};

#endif
