/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef TKTABBAR_H
#define TKTABBAR_H

#include <qwidget.h>
#include <qiconset.h>
#include <qtimer.h>

class QPainter;
class QAccel;
struct TKTab;
struct TKTabBarPrivate;

class TKTabBar : public QWidget
{ Q_OBJECT
public:
  TKTabBar(QWidget *parent=0, const char *name=0);
  ~TKTabBar();

  enum TabPosition { Top, Bottom, Left, Right };
  enum TabViewStyle { ShowIcon = 1, ShowCloseButton = 2, CurrentBold = 4, EasyStyle = 8, ShowDragLines = 16 };
  
  virtual QSize sizeHint() const;
  virtual QSizePolicy sizePolicy() const;
  virtual void polish();

  int addTab(const QString &, int id=-1);
  int addTab(const QString &, const QIconSet &, int id=-1);

  int insertTab(const QString &, int idAfter, int id=-1);
  int insertTab(const QString &, const QIconSet &, int idAfter, int id=-1);

  void clear();
  void removeTab(int id);

  int count() const;

  void setTabTitle(int id, const QString &);
  void setTabIcon(int id, const QIconSet &);
  void setTabEnabled(int id, bool);

  QString tabTitle(int id) const;
  QIconSet *tabIcon(int id) const;
  bool isTabEnabled(int id) const;

  int firstTab(bool enabled=false);
  int lastTab(bool enabled=false);
  int prevTab(int id, bool enabled=false);
  int nextTab(int id, bool enabled=false);

  int currentTab() const { return current; }

  // properties
  void setTabPosition(TabPosition);
  TabPosition tabPosition() const { return a; }

  void setViewStyle(int);
  int viewStyle();

public slots:
  void setCurrentTab(int id);

signals:
  void currentChanged(int);
  void startTabDrag(int, QMouseEvent *);
  void continueTabDrag(QMouseEvent *);
  void finishTabDrag(QMouseEvent *);
  void cancelTabDrag(QMouseEvent *);
  void closeTab(int);
  void tabRightButtonPressed(int, const QPoint &);
  void changeTabsOrder(int, int);

protected slots:
  void breakTimeout();

protected:
  virtual void paintEvent(QPaintEvent *);
  virtual void mousePressEvent(QMouseEvent *);
  virtual void mouseReleaseEvent(QMouseEvent *);
  virtual void mouseMoveEvent(QMouseEvent *);
  virtual void resizeEvent(QResizeEvent *);
  virtual void keyPressEvent(QKeyEvent *);

  virtual void fontChange(const QFont &);
  virtual void styleChange(QStyle &);

  int insertTab(const QString &, QIconSet *, int idAfter, int id);

  virtual void drawTab(QPainter &, const QColorGroup &, TKTab *);
  int tabWidth(TKTab *);
  QRect tabRect(int);
  TKTab *tab(int) const;
  int startPos();

  bool setCurrentTabInternal(int id);
  void checkButtons(const QPoint &);
  void scrollLeft();
  void scrollRight();
  void updateBreaks();
  void makeTabVisible(int);
  int tabByPos(const QPoint &);

  int rw();
  int rh();

  void updatePixmap();
  void recalTabsWidth();

  static QString clearString(const QString &);

private:
  TKTabBarPrivate *d;
  QTimer breakTimer;
  QAccel *accel;
  QPixmap buffer;
  TabPosition a;
  int current;
  int first;
  bool breakLeft;
  bool breakRight;
  bool leftButtonPressed;
  bool rightButtonPressed;
  bool closeButtonPressed;
  bool mousePressed;
  int tabPressed;
  
  bool enableClose;
  bool showDragLine;
  bool easyStyle;
  bool selectedBold;
  bool showIcon;
  
  int vstyle;
  
  int controlsWidth;
  
  bool outside;
};

#endif
