/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#include "tktabwidget.h"

#include <qwidgetstack.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qabstractlayout.h>

TKTabWidget::TKTabWidget(QWidget *parent, const char *name, WFlags f)
: QWidget(parent, name, f | WResizeNoErase | WRepaintNoErase)
{
  stack = new QWidgetStack(this);
  tabs = new TKTabBar(this);
  connect(tabs, SIGNAL(startTabDrag(int, QMouseEvent *)), SLOT(tabDragStart(int, QMouseEvent *)));
  connect(tabs, SIGNAL(continueTabDrag(QMouseEvent *)), SLOT(tabDragContinue(QMouseEvent *)));
  connect(tabs, SIGNAL(finishTabDrag(QMouseEvent *)), SLOT(tabDragFinish(QMouseEvent *)));
  connect(tabs, SIGNAL(cancelTabDrag(QMouseEvent *)), SLOT(tabDragCancel(QMouseEvent *)));
  connect(tabs, SIGNAL(closeTab(int)), SLOT(tabClose(int)));
  connect(tabs, SIGNAL(currentChanged(int)), SLOT(currentTabChanged(int)));
  connect(tabs, SIGNAL(tabRightButtonPressed(int, const QPoint &)), SLOT(tabRightButtonPressed(int, const QPoint &)));

  tabs->hide();
  layout = 0;

  stack->installEventFilter( this );
  setFocusPolicy(TabFocus);
  setFocusProxy(tabs);
  setBackgroundMode(NoBackground);

  setMargin(2);
  setTabPosition(Top);
}

TKTabWidget::~TKTabWidget()
{
}

void TKTabWidget::tabDragStart(int id, QMouseEvent *ev)
{
  if (id2w[id]) {
    id2w[id]->setGeometry(stack->contentsRect());
    emit startPageDrag(id2w[id], ev);
  }
}

void TKTabWidget::tabDragContinue(QMouseEvent *ev)
{
  emit continuePageDrag(ev);
}

void TKTabWidget::tabDragFinish(QMouseEvent *ev)
{
  emit finishPageDrag(ev);
}

void TKTabWidget::tabDragCancel(QMouseEvent *ev)
{
  emit cancelPageDrag(ev);
}

void TKTabWidget::tabClose(int id)
{
  if (id2w.find(id) != id2w.end())
    emit closePage(id2w[id]);
}

void TKTabWidget::removePage(QWidget * page)
{
  if (w2id.find(page) != w2id.end()) {
    tabs->removeTab(w2id[page]);
    id2w.remove(w2id[page]);
    w2id.remove(page);
    stack->removeWidget(page);
  }
}

QWidget *TKTabWidget::currentPage() const
{
  return id2w[tabs->currentTab()];
}

void TKTabWidget::setCurrentPage(QWidget *page)
{
  if (w2id.find(page) != w2id.end())
    tabs->setCurrentTab(w2id[page]);
}

void TKTabWidget::setTabPosition(TabPosition pos)
{
  a = pos;
  doLayout();
}

void TKTabWidget::doLayout()
{
  delete layout;

  if (!tabs->isVisibleTo(this)) {
    layout = new QGridLayout(this, 0, 0, 1, 0);
    layout->addWidget(stack, 0, 0);
    return;
  }

  layout = new QGridLayout(this, 0, 0, 0, 0);
  TKTabBar::TabPosition p = TKTabBar::Top;
  switch (a) {
    case Top:
      p = TKTabBar::Top;
      layout->addMultiCellWidget(tabs, 0, 0, 0, 2);
      layout->addWidget(stack, 1, 1);
      layout->addItem(new QSpacerItem(1, 0, QSizePolicy::Fixed, QSizePolicy::Expanding), 1, 0);
      layout->addItem(new QSpacerItem(1, 0, QSizePolicy::Fixed, QSizePolicy::Expanding), 1, 2);
      layout->addItem(new QSpacerItem(0, 1, QSizePolicy::Expanding, QSizePolicy::Fixed), 2, 1);
      break;
    case Left:
      p = TKTabBar::Left;
      layout->addMultiCellWidget(tabs, 0, 2, 0, 0);
      layout->addWidget(stack, 1, 1);
      layout->addItem(new QSpacerItem(1, 0, QSizePolicy::Fixed, QSizePolicy::Expanding), 1, 2);
      layout->addItem(new QSpacerItem(0, 1, QSizePolicy::Expanding, QSizePolicy::Fixed), 0, 1);
      layout->addItem(new QSpacerItem(0, 1, QSizePolicy::Expanding, QSizePolicy::Fixed), 2, 1);
      break;
    case Right:
      p = TKTabBar::Right;
      layout->addMultiCellWidget(tabs, 0, 2, 2, 2);
      layout->addWidget(stack, 1, 1);
      layout->addItem(new QSpacerItem(1, 0, QSizePolicy::Fixed, QSizePolicy::Expanding), 1, 0);
      layout->addItem(new QSpacerItem(0, 1, QSizePolicy::Expanding, QSizePolicy::Fixed), 0, 1);
      layout->addItem(new QSpacerItem(0, 1, QSizePolicy::Expanding, QSizePolicy::Fixed), 2, 1);
      break;
    case Bottom:
      p = TKTabBar::Bottom;
      layout->addMultiCellWidget(tabs, 2, 2, 0, 2);
      layout->addWidget(stack, 1, 1);
      layout->addItem(new QSpacerItem(1, 0, QSizePolicy::Fixed, QSizePolicy::Expanding), 1, 0);
      layout->addItem(new QSpacerItem(1, 0, QSizePolicy::Fixed, QSizePolicy::Expanding), 1, 2);
      layout->addItem(new QSpacerItem(0, 1, QSizePolicy::Expanding, QSizePolicy::Fixed), 0, 1);
      break;
  }
  tabs->setTabPosition(p);
}

int TKTabWidget::margin() const
{
  return stack->margin();
}

void TKTabWidget::setMargin(int m)
{
  stack->setMargin(m);
}

bool TKTabWidget::eventFilter( QObject *obj, QEvent * ev)
{
  if (obj == stack && ev->type() == QEvent::ChildRemoved && ((QChildEvent *)ev)->child()->isWidgetType()) {
    removePage((QWidget *)((QChildEvent *)ev)->child());
    return true;
  }

  if (obj->isWidgetType() && autoPages.findRef((QWidget *)obj) != -1) {
    QWidget *page = (QWidget *)obj;
    if (ev->type() == QEvent::CaptionChange)
      setPageTitle(page, page->caption());

    if (ev->type() == QEvent::IconChange)
      if (page->icon())
        setPageIcon(page, *page->icon());
      else
        setPageIcon(page, QPixmap());
  }
  return false;
}

void TKTabWidget::paintEvent(QPaintEvent *)
{
  QPainter paint(this, this);
  QRect r = rect();
  const QColorGroup &g = colorGroup();
  if (tabs->viewStyle() & TKTabBar::EasyStyle)
    paint.fillRect(r, g.background());
  else
    qDrawShadePanel(&paint, r, g, false, 1);
  paint.end();
}

void TKTabWidget::currentTabChanged(int id)
{
  if (id == -1) {
    tabs->hide();
    doLayout();
    return;
  }

  stack->raiseWidget(id);
  emit currentChanged(stack->widget(id));
}

void TKTabWidget::setPageTitle(QWidget *page, const QString &title)
{
  if (w2id.find(page) != w2id.end())
    tabs->setTabTitle(w2id[page], title);
}

QString TKTabWidget::pageTitle(QWidget *page) const
{
  if (w2id.find(page) != w2id.end())
    return tabs->tabTitle(w2id[page]);
  return QString::null;
}

QIconSet *TKTabWidget::pageIcon(QWidget *page) const
{
  if (w2id.find(page) != w2id.end())
    return tabs->tabIcon(w2id[page]);
  return 0;
}

void TKTabWidget::setPageIcon(QWidget *page, const QIconSet& icon)
{
  if (w2id.find(page) != w2id.end())
    tabs->setTabIcon(w2id[page], icon);
}

bool TKTabWidget::isPageEnabled(QWidget *page) const
{
  if (w2id.find(page) != w2id.end())
    return tabs->isTabEnabled(w2id[page]);
  return false;
}

void TKTabWidget::setPageEnabled(QWidget *page, bool enabled)
{
  if (w2id.find(page) != w2id.end())
    tabs->setTabEnabled(w2id[page], enabled);
}

void TKTabWidget::addPage(QWidget *page, int id)
{
  stack->addWidget(page, id);
  w2id[page] = id;
  id2w[id] = page;

  if (!tabs->isVisible()) {
    tabs->show();
    doLayout();
  }
  page->installEventFilter(this);
  if (pageCount() == 1)
    setCurrentPage(page);
}

void TKTabWidget::addPage(QWidget *page, const QString &title)
{
  addPage(page, tabs->addTab(title));
}

void TKTabWidget::addPage(QWidget *page, const QString &title, const QIconSet &icon)
{
  addPage(page, tabs->addTab(title, icon));
}

void TKTabWidget::insertPage(QWidget *page, const QString &title, QWidget *after)
{
  if (w2id.find(after) != w2id.end())
    addPage(page, tabs->insertTab(title, w2id[after]));
  else
    addPage(page, tabs->insertTab(title, -1));
}

void TKTabWidget::insertPage(QWidget *page, const QString &title, const QIconSet& icon, QWidget *after)
{
  if (w2id.find(after) != w2id.end())
    addPage(page, tabs->insertTab(title, icon, w2id[after]));
  else
    addPage(page, tabs->insertTab(title, icon, -1));
}

QWidget *TKTabWidget::firstPage()
{
  return id2w[tabs->firstTab()];
}

QWidget *TKTabWidget::lastPage()
{
  return id2w[tabs->lastTab()];
}

QWidget *TKTabWidget::prevPage(QWidget *page)
{
  if (w2id.find(page) != w2id.end())
    return id2w[tabs->prevTab(w2id[page])];
  return 0;
}

QWidget *TKTabWidget::nextPage(QWidget *page)
{
  if (w2id.find(page) != w2id.end())
    return id2w[tabs->nextTab(w2id[page])];
  return 0;
}

void TKTabWidget::addPage(QWidget *page)
{
  autoPages.append(page);
  if (page->icon())
    addPage(page, page->caption(), *page->icon());
  else
    addPage(page, page->caption());
}

void TKTabWidget::insertPage(QWidget *page, QWidget *after)
{
  autoPages.append(page);
  if (page->icon())
    insertPage(page, page->caption(), *page->icon(), after);
  else
    insertPage(page, page->caption(), after);
}

uint TKTabWidget::pageCount() const
{
  return w2id.count();
}

int TKTabWidget::tabViewStyle()
{
  return tabs->viewStyle();
}

void TKTabWidget::setTabViewStyle(int style)
{
  tabs->setViewStyle(style);
  repaint(false);
}

void TKTabWidget::tabRightButtonPressed(int id, const QPoint &pos)
{
  emit rightButtonPressed(id2w[id], pos);
}
