/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
#ifndef TKTABWIDGET_H
#define TKTABWIDGET_H

#include <qwidget.h>
#include <qiconset.h>
#include <qptrlist.h>
#include <qmap.h>

#include "tktabbar.h"

class QWidgetStack;
class QGridLayout;
struct Page;

class TKTabWidget : public QWidget
{ Q_OBJECT
public:
  TKTabWidget(QWidget *parent=0, const char *name=0, WFlags=0);
  ~TKTabWidget();

  TKTabBar *tabBar() { return tabs; }

  enum TabPosition { Top, Bottom, Left, Right };

  void addPage(QWidget *);
  void addPage(QWidget *, const QString &);
  void addPage(QWidget *, const QString &, const QIconSet &);

  void insertPage(QWidget *, QWidget *after);
  void insertPage(QWidget *, const QString &, QWidget *after);
  void insertPage(QWidget *, const QString &, const QIconSet& , QWidget *after);

  void removePage(QWidget *);

  void setPageTitle(QWidget *, const QString &);
  QString pageTitle(QWidget *) const;

  void setPageIcon(QWidget *, const QIconSet&);
  QIconSet *pageIcon(QWidget *) const;

  void setPageEnabled(QWidget *, bool);
  bool isPageEnabled(QWidget *) const;

  QWidget *currentPage() const;
  void setCurrentPage(QWidget *);

  TabPosition tabPosition() const { return a; }
  void setTabPosition(TabPosition);

  int margin() const;
  void setMargin(int);

  QWidget *firstPage();
  QWidget *lastPage();
  QWidget *prevPage(QWidget *);
  QWidget *nextPage(QWidget *);

  uint pageCount() const;

  int tabViewStyle();
  void setTabViewStyle(int);

  virtual bool eventFilter(QObject *, QEvent *);

signals:
  void currentChanged(QWidget *);
  void startPageDrag(QWidget *, QMouseEvent *);
  void continuePageDrag(QMouseEvent *);
  void finishPageDrag(QMouseEvent *);
  void cancelPageDrag(QMouseEvent *);
  void closePage(QWidget *);
  void rightButtonPressed(QWidget *, const QPoint &);

protected slots:
  void currentTabChanged(int);
  void tabDragStart(int, QMouseEvent *);
  void tabDragContinue(QMouseEvent *);
  void tabDragFinish(QMouseEvent *);
  void tabDragCancel(QMouseEvent *);
  void tabClose(int);
  void tabRightButtonPressed(int, const QPoint &);

protected:
  virtual void paintEvent(QPaintEvent *);
  void doLayout();
  void addPage(QWidget *, int id);

private:
  QPtrList<QWidget> autoPages;
  QMap<QWidget *, int> w2id;
  QMap<int, QWidget *> id2w;
  QWidgetStack *stack;
  QGridLayout *layout;
  TKTabBar *tabs;
  TabPosition a;
};

#endif
