/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                aetheraupdate.cpp  -  perform the updates
                             -------------------
    begin                : Sat Mar 23 2002
    copyright            : (C) 2002 by theKompany.com <www.thekompany.com>
    author               : Eugen C.
    email                : eug@thekompany.com
 ***************************************************************************/

#include "aetheraupdate.h"

#include <settings.h>
#include <defines.h>
#include <vfsmimetype.h>

#include <mimecodec.h>

#include <qfile.h>
#include <qstring.h>
#include <qstringlist.h>

AetheraUpdate::AetheraUpdate()
{
}

AetheraUpdate::~AetheraUpdate()
{
}

bool AetheraUpdate::build(const QString & version)
{
  unsigned uMajor=0, uMinor=0, uBuild=0;
  
  // Beta 4 -> 0.9.6
  if( -1 == version.find('.') )
  {
    uMajor = 0;
    uMinor = 9;
    uBuild = 6;
  }
  else
  {
    QStringList numbers( QStringList::split(".", version) );
    
    if( numbers.count()<3 )
      return false;
    
    uMajor = numbers[0].toUInt();
    uMinor = numbers[1].toUInt();
    uBuild = numbers[2].toUInt();
  }

//  qDebug("Version = %u-%u-%u", uMajor, uMinor, uBuild);
  
  unsigned updateId = uMajor*10000+uMinor*100+uBuild;
  if( updateId <= 906 )
  {
    if( !update0_9_6() )
      return false;
    
    // 0.9.7
    updateId = 907;
  }
  if( updateId == 907 )
  {
    if( !update0_9_7() )
      return false;
      
    // 1.0.0
    updateId = 10000;
  }
  if( updateId == 10000 )
  {
    if( !update1_0_0() )
      return false;
      
    // 1.0.1
    updateId = 10001;
  }
  if( updateId == 10001 )
  {
    if( !update1_0_1() )
      return false;

    // 1.0.2
    updateId = 10002;
  }
  return true;
}

bool AetheraUpdate::changeVersion(const QString & newVersion)
{
  QString aetheraDir = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  
  // Check the config file for the top-level directory.
  if (aetheraDir.isEmpty())
    return false;
    
  // Check the version info file.
  QFile versionFile(aetheraDir + "/.version");
  if (!versionFile.open(IO_WriteOnly))
    return false;
  
  QTextStream txt(&versionFile);
  txt << newVersion;
  
  return true;
}

bool AetheraUpdate::update0_9_6()
{
  qDebug("AetheraUpdate::update0_9_6()");

  Settings::instance()->writeEntry
    (
      "/Aethera/PlugInManager/PlugIns", 
      "homeplugin,tkcJabber,korganizer,mailimport,komnotesplugin,komcontactsplugin,kommailplugin"
    );

  return changeVersion("0.9.7");
}

bool AetheraUpdate::update0_9_7()
{
  qDebug("AetheraUpdate::update0_9_7()");

  Settings::instance()->writeEntry
    (
      "/Aethera/PlugInManager/PlugIns", 
      "homeplugin,tkcWhiteboard,tkcJabber,korganizer,mailimport,komnotesplugin,komcontactsplugin,kommailplugin"
    );

  QString rootVFS("/Aethera/VFS");
  QStringList topLevel =
    Settings::instance()->readListEntry(rootVFS+"/TopLevel", ',');
    
  topLevel.push_back( VFS_WHITEBOARD_ROOT );
  topLevel.push_back( VFS_JABBER_ROOT );
  topLevel.push_back( VFS_CALENDAR_ROOT );

  Settings::instance()->writeEntry(rootVFS+"/TopLevel", topLevel, ',');
  
  // Jabber root
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Server Path", "sysjabber");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Type", "jabber");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/View type", "jabber");
  Settings::instance()->writeEntry("/Aethera/VFS/Jabber/MIME type", "jabber/x-chat");

  // Whiteboard root
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Server Path", "syswhiteboard");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Type", "whiteboard");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/View type", "whiteboard");
  Settings::instance()->writeEntry("/Aethera/VFS/Whiteboard/MIME type", "jabber/x-whiteboard");

  // DateBook root
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Binding", "thisInstance");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Server Path", "sysdatebook");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Type", "datebook");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/Flags", "c");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/View type", "datebook");
  Settings::instance()->writeEntry("/Aethera/VFS/Datebook/MIME type", "application/x-datebook");

  Settings::forceWrite();
  
  return changeVersion("1.0.0");
}

bool AetheraUpdate::update1_0_0()
{
  Settings::instance()->writeEntry("/Aethera/Misc Mail/Permanent delete", false);
  Settings::forceWrite();
  
  return changeVersion("1.0.1");
}

bool AetheraUpdate::update1_0_1()
{
  QStringList groups = Settings::instance()->readListEntry("/Aethera/Receive Options/Accounts/",',');
  QString group = QString("/Aethera/Accounts/");
  for (QStringList::ConstIterator it(groups.begin()); it != groups.end(); ++it)
  {
    QString acc=*it;
    QString password=Settings::instance()->readEntry(group+ acc + "/Password");
    Settings::instance()->writeEntry
                        (group + acc + "/Password",
                        QString ( MIMECodec::encodeBase64( password.utf8() ) )
                        );
  }

  QStringList plugins = Settings::instance()->readListEntry( "/Aethera/PlugInManager/PlugIns",',' );
  plugins.append("tkphone");
  Settings::instance()->writeEntry("/Aethera/PlugInManager/PlugIns",plugins,',');

  QStringList rss = Settings::instance()->readListEntry( "/Aethera/RSS/URL",',' );
  Settings::instance()->writeEntry("/Aethera/RSS/URL",rss,'^');

  QStringList weather = Settings::instance()->readListEntry( "/Aethera/Weather/URL",',' );
  Settings::instance()->writeEntry("/Aethera/Weather/URL",weather,'^');


  Settings::forceWrite();

  return changeVersion("1.0.2");
}


