/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
/***************************************************************************
                          generalconfigservice.cpp  -  description
                             -------------------
    begin                : Thu Oct 10 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/

#include "generalconfigservice.h"
#include "generalconfigprivate.h"
#include "generalconfigwidget.h"

#include <settings.h>
#include <qsettingsextra.h>
#include <qiconloader.h>
#include <externalbrowser.h>
#include <playsound.h>
#include <notificator.h>

#include <qfont.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qlineedit.h>

static const char * const configGroup = "/Aethera/General/";

kore::Interface* generalconfigservice_factory()
{
  return new GeneralConfigService();
}

class GeneralConfigService::Private
{
  public:
    Private(GeneralConfigService*);
    ~Private();
    
    QFont font;
    
    GeneralConfigWidget * widget;
    GeneralConfigPrivate * qtHandler;
    GeneralConfigService * parent_;
};

GeneralConfigService::Private::Private(GeneralConfigService * parent)
  : widget(0),
    qtHandler(0),
    parent_(parent)
{
}

GeneralConfigService::Private::~Private()
{
  delete widget;
  widget = 0;

  delete qtHandler;
  qtHandler = 0;
}

GeneralConfigService::GeneralConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "General Config Service");
  info->replaceAll("class", "GeneralConfigService");
  info->replaceAll("class/fqn", "GeneralConfigService");
  info->replaceAll("factory", "generalconfigservice_factory");
  
  d = new Private(this);

  // init sound support
  if( Settings::instance()->readBoolEntry(
        configGroup + QString("Sound/Enabled"), true) )
  {
    PlaySound::setOn();
  }
  else
  {
    PlaySound::setOff();
  }
  if( Settings::instance()->readBoolEntry(
        configGroup + QString("Sound/AutoDetect"), true) )
  {
#ifdef _QTX11_
    PlaySound::setPlayer("aethera_sound");
#endif
  }
  else
  {
#ifdef _QTX11_
    QString playerPath = Settings::instance()->readEntry(
                        configGroup + QString("Sound/Player"), QString("aethera_sound"));
#else
    QString playerPath = Settings::instance()->readEntry(
                        configGroup + QString("Sound/Player"), QString::null);
#endif
    PlaySound::setPlayer(playerPath);
  }
  
  // init the notificator
  Notificator::instance()->setSettingsGroup("/Aethera/Notification");
  // todo, play a start sound
  // set the global sounds
}

GeneralConfigService::~GeneralConfigService()
{
  delete d;
  d = 0;
}

QPixmap GeneralConfigService::icon()
{
  return QIconLoader::instance()->loadIcon( "lo32-aethera.png" );
}

QString GeneralConfigService::caption()
{
  return QString("General");
}
  
void GeneralConfigService::load()
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }
  
  QString pattern("%1 - %2");
  QFont widgetFont(d->widget->font());
  
  bool useDefaultFont = Settings::instance()->readBoolEntry(configGroup + QString("UseDefaultFont"), true);
  d->widget->defaultFontCheckBox->setChecked( useDefaultFont );
  
  bool useDefaultLanguage = Settings::instance()->readBoolEntry(configGroup + QString("UseDefaultLanguage"), true);
  d->widget->defaultLanguageCheckBox->setChecked( useDefaultLanguage );
  
  QFont savedFont = QSettingsExtra::readFontEntry
  (Settings::instance(), configGroup + QString("Default font"), &widgetFont);
    
  d->widget->fontChooserButton->setText
    (pattern.arg(savedFont.family()).arg(savedFont.pointSize()));
    
  d->font = savedFont;
  
  int iStyle = 0;
  QIconLoader::Style style = QIconLoader::instance()->iconStyle();
  
  switch(style)
  {
    case QIconLoader::KDE_Default :
      iStyle = 1;
      break;
    case QIconLoader::QT_Default :
      iStyle = 2;
      break;
    case QIconLoader::Default :
    default:
      iStyle = 0;
      break;
  }
  
  d->widget->toolbarButtonGroup->setButton(iStyle);

  bool autodetect = Settings::instance()->readBoolEntry(
                      configGroup + QString("AutoDetectFileManager"), true);
  d->widget->browserCheckBox->setChecked( autodetect );
  QString browserPath = Settings::instance()->readEntry(
                      configGroup + QString("FileManager"), QString::null);
  if( !browserPath.isEmpty() )
  {
    d->widget->pathLineEdit->setText(browserPath);
  }
  
  // Sound
  bool enableSound = Settings::instance()->readBoolEntry(
    configGroup + QString("Sound/Enabled"), true);
  d->widget->soundOnOffCheckBox->setChecked( enableSound );
  
  bool autodetectSound = Settings::instance()->readBoolEntry(
    configGroup + QString("Sound/AutoDetect"), true);
  d->widget->soundCheckBox->setChecked( autodetectSound );
#ifdef _QTX11_
  QString playerPath = Settings::instance()->readEntry(
                      configGroup + QString("Sound/Player"), QString("aethera_sound"));
#else
  QString playerPath = Settings::instance()->readEntry(
                      configGroup + QString("Sound/Player"), QString::null);
#endif
  if( !playerPath.isEmpty() )
  {
    d->widget->soundPathLineEdit->setText(playerPath);
  }
  
  QString pluginFolder = Settings::instance()->readEntry
    ( "/Aethera/PlugInManager/PlugInDirectory",
      "" );
  d->widget->pluginFolderLineEdit->setText( pluginFolder );

  d->widget->slotScanPluginsList();
}

void GeneralConfigService::save()
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }

  Settings::instance()->writeEntry(configGroup + QString("UseDefaultFont"),
    d->widget->defaultFontCheckBox->isChecked());
  Settings::instance()->writeEntry(configGroup + QString("UseDefaultLanguage"),
    d->widget->defaultLanguageCheckBox->isChecked());
  
  QString pattern("%1 - %2");
  QSettingsExtra::writeEntry( Settings::instance(), configGroup + QString("Default font"), d->font);
  
  int i = d->widget->toolbarButtonGroup->id( d->widget->toolbarButtonGroup->selected() );
  QIconLoader::Style style;
  
  switch( i )
  {
    case 1:
      style = QIconLoader::KDE_Default;
      break;
    case 2:
      style = QIconLoader::QT_Default;
      break;
    case 0:
    default:
      style = QIconLoader::Default;
      break;
  }
  
  QIconLoader::instance()->changeStyle(style);
  
  // Set Aethera app font
  if( !d->widget->defaultFontCheckBox->isChecked() )
    qApp->setFont(d->font, true);

  // Browser
  Settings::instance()->writeEntry(configGroup + QString("AutoDetectFileManager"),
    d->widget->browserCheckBox->isChecked());
  Settings::instance()->writeEntry(configGroup + QString("FileManager"),
    d->widget->pathLineEdit->text());
  if( d->widget->browserCheckBox->isChecked() )
  {
    ExternalBrowser::setBrowser(QString::null);
  }
  else
  {
    ExternalBrowser::setBrowser(d->widget->pathLineEdit->text());
  }
  
  // Sound
  Settings::instance()->writeEntry(configGroup + QString("Sound/Enabled"),
    d->widget->soundOnOffCheckBox->isChecked());
  Settings::instance()->writeEntry(configGroup + QString("Sound/AutoDetect"),
    d->widget->soundCheckBox->isChecked());
  Settings::instance()->writeEntry(configGroup + QString("Sound/Player"),
    d->widget->soundPathLineEdit->text());
  if( d->widget->soundOnOffCheckBox->isChecked() )
  {
    PlaySound::setOn();
  }
  else
  {
    PlaySound::setOff();
  }
  if( d->widget->soundCheckBox->isChecked() )
  {
#ifdef _QTX11_
    PlaySound::setPlayer("aethera_sound");
#else
    PlaySound::setPlayer(QString::null);
#endif
  }
  else
  {
    PlaySound::setPlayer( d->widget->soundPathLineEdit->text() );
  }
  
  QStringList list = d->widget->loadedPlugins();
  if( !list.isEmpty() )
    Settings::instance()->writeEntry( "/Aethera/PlugInManager/LoadedPlugins", list, ',' );

  list = d->widget->allPlugins();
  if( !list.isEmpty() )
    Settings::instance()->writeEntry( "/Aethera/PlugInManager/PlugIns", list, ',' );

  QString folderPath = d->widget->pluginsFolder();
  if( !folderPath.isEmpty() )
    Settings::instance()->writeEntry( "/Aethera/PlugInManager/PlugInDirectory",
                                      d->widget->pluginFolderLineEdit->text() );
}

void GeneralConfigService::defaults()
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }

  d->widget->toolbarButtonGroup->setButton(0);

  d->widget->defaultFontCheckBox->setChecked( true );
  d->widget->defaultLanguageCheckBox->setChecked( true );

  d->widget->browserCheckBox->setChecked( true );
  
  d->widget->soundCheckBox->setChecked( true );
  
  //user selected plugin folder - none
  d->widget->pluginFolderLineEdit->setText( "" );
  
  //default plugins to load - all (the standard plugins because user plugins are not searched)
  d->widget->slotScanPluginsList();

  while( d->widget->availabilePluginsList->firstChild() )
  {
    QListViewItemIterator it( d->widget->availabilePluginsList );
    for( ; it.current(); ++it )
    {
      (*it)->setSelected( true );
      d->widget->slotAddPlugins();
    }
  }
}
  
bool GeneralConfigService::modified()
{
  return false;
}
  
QWidget * GeneralConfigService::init(QWidget * parent)
{
  if (0 == d->widget)
  {
    d->widget = new GeneralConfigWidget(parent, "GeneralConfigService widget");
  }
  
  QObject::connect( d->widget->fontChooserButton, SIGNAL(clicked()),
                    getQTHandler(), SLOT(slotChooseFont()) );
  
  return d->widget;
}

  void
GeneralConfigService::fontChosen(QFont f)
{
  if (0 == d->widget)
  {
    qDebug("No widget !");
    return;
  }

  d->font = f;
  
  QString pattern("%1 - %2");
  d->widget->fontChooserButton->setText
    (pattern.arg(d->font.family()).arg(d->font.pointSize()));
}

  QFont
GeneralConfigService::currentFont()
{
  return d->font;
}

  QObject *
GeneralConfigService::createQTHandler()
{
  if (0 != d->qtHandler)
    return d->qtHandler;
  
  d->qtHandler = new GeneralConfigPrivate(this);
  
  return d->qtHandler;
}

  QObject *
GeneralConfigService::getQTHandler()
{
  createQTHandler();

  return d->qtHandler;
}

