/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/

#include <defines.h>
#include <dirs.h>
#include <qiconloader.h>
#include <qsettingsextra.h>
#include <settings.h>
#include <externalbrowser.h>

#include <qapplication.h>
#include <qmainwindow.h>
#include <qsettings.h>
#include <qfileinfo.h>
#include <qdatetime.h>
#include <qmessagebox.h>
#include <qtextcodec.h>

#include <countriesclass.h>
#include <clipboard.h>
#include <texthtml.h>
#include <indexclass.h>
#include <messagedevice.h>
#include <orb.h>
#include <conversions.h>
#include <vmailattribute.h>
#include <groupobj.h>
#include <attributefactories.h>
#include <mailfilter_plugin.h>
#include <komaetheraplugin.h>
#include <komaetheraman.h>
#include <generalconfigservice.h>
#include <kompluginmanager.h>
#include <vfscomponent.h>
#include <clientrmi.h>
#include <setupwizard.h>
#include <aetheraupdate.h>
#include <filter.h>
#include <kolabsharedfolder.h>
#include <accountmanager.h>

bool setupDone = false;

MailFilter * mailFilter;

// Return filename of splash graphic for current time of day.
static QString currentSplashGraphic();

// Setup-related functions.
static bool setupWanted(int, char **);
static bool shouldSetup(void);
static void runSetupWizard(void);
static bool verify(QString);
static void removeTemporaryFiles();

  int
main(int argc, char ** argv)
{
  QApplication app(argc, argv, "aethera");

  AppName = "Aethera";
  AppVersion = "1.2.1";
  FullAppName = "Aethera 1.2.1";
  AppMIME = "application/mail";

  // languages support for Aethera basic
  QString languageEnv = QTextCodec::locale();
  
  if( languageEnv.isEmpty() )
    languageEnv = "en";
    
  QString languagePath = Dirs::locateDir("i18n");
  if( languagePath.isEmpty() )
    languagePath = ".";
  
  // QT translation
  QTranslator * translator = new QTranslator(0);
  if( translator->load(QString("qt_") + languageEnv, languagePath) )
  {
    app.installTranslator( translator );
    app.setDefaultCodec( QTextCodec::codecForName( languageEnv ) );
  }
  else
  {
    qDebug("Couldn't find translator qt_%s", languageEnv.latin1());
    delete translator;
  }
  
  // Aethera translation
  translator = new QTranslator(0);
  if( translator->load(QString("aethera_") + languageEnv, languagePath) )
  {
    app.installTranslator( translator );
    app.setDefaultCodec( QTextCodec::codecForName( languageEnv ) );
//    QTextCodec::setCodecForTr( QTextCodec::codecForName(languageEnv) );
  }
  else
  {
    qDebug("Couldn't find translator aethera_%s", languageEnv.latin1());
    delete translator;
  }
  
  bool useDefaultFont = Settings::instance()->readBoolEntry("/Aethera/General/UseDefaultFont", true);
//  bool useDefaultLanguage = Settings::instance()->readBoolEntry("/Aethera/General/UseDefaultLanguage", true);
  
  if( !useDefaultFont )
  {
    QFont font = QSettingsExtra::readFontEntry( Settings::instance(), "/Aethera/General/Default font");
    qApp->setFont( font );
  }

  if (setupWanted(argc, argv) || shouldSetup())
    runSetupWizard();

  QWidget * splashScreen =
    new QWidget
    (
      0,
      "Aethera splash screen",
      Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_StaysOnTop
    );

  splashScreen->setFocusPolicy(QWidget::NoFocus);
  splashScreen->setIcon( BarIcon( "lo22-aethera" ));

  QPixmap splashPixmap(BarIcon(currentSplashGraphic()));

  if (splashPixmap.isNull())
  {
    qDebug("Can't find splash pixmap");
  }
  else
  {
    splashScreen->setFixedSize(splashPixmap.size());
    splashScreen->setErasePixmap(splashPixmap);
    splashScreen->move
      (
        app.desktop()->width() / 2 - splashPixmap.width() / 2,
        app.desktop()->height() / 2 - splashPixmap.height() / 2
      );
    splashScreen->show();
    app.processEvents(); // Let the splash screen appear.
  }

  QMainWindow * mainWindow = new QMainWindow;

  // Set the main window.

  mainWindow->setCaption(FullAppName);

  mainWindow->setIcon( BarIcon( "lo22-aethera" ));

  mainWindow->resize(640, 480);

  app.setMainWidget(mainWindow);

// Set the default font
// Ex: FontName, 12p
  QString fontName = getenv("AETHERA_FONT");
  
  if( !fontName.isEmpty() )
    mainWindow->setFont( QFont( fontName.left(fontName.find(",")), QApplication::font().pointSize(), true) );

  // Check if korelib main components are available.
  ComponentTrader * trader =
    dynamic_cast<ComponentTrader *>(ComponentTrader::traderRef().instance());

  LibraryLocator * locator = 0;

  if (0 != trader)
  {
    locator =
      dynamic_cast<LibraryLocator *>
      (trader->registeredComponent("/kore/service/locator").instance());
  }
  
  if (0 == trader || 0 == locator)
  {
    qDebug("KoreLib error: main components unavailable!");
    return 1;
  }

  // set the external file manager or browser
  QString configGroup = "/Aethera/General/";
  bool autodetect = Settings::instance()->readBoolEntry(
                      configGroup + QString("AutoDetectFileManager"), true);
  QString browserPath = Settings::instance()->readEntry(
                      configGroup + QString("FileManager"), QString::null);
  
  if( autodetect )
  {
    ExternalBrowser::setBrowser(QString::null);
  }
  else
  {
    ExternalBrowser::setBrowser(browserPath);
  }
  
  // Register the VFSComponent.
  VFSComponent * vfs = new VFSComponent();
  trader->registerComponent(*vfs);

  // Create the backend
  ObjectRequestBroker::thisInstance();

  // Create the Aethera UI manager.
  KomAetheraMan * aetheraMan = new KomAetheraMan(mainWindow);
  trader->registerComponent(* aetheraMan);
  
  // Load the general options config component.
  GeneralConfigService * generalOptions = new GeneralConfigService();
  trader->registerComponent(* generalOptions);

  // Load all the known plug ins.
  KomPlugInManager * pluginManager = new KomPlugInManager;
  pluginManager->loadPlugins();

  // Initialise mail filters.
  MailFilter * mailFilter = new MailFilter;
  trader->registerComponent(*mailFilter);

//  CountriesClass::thisInstance(Dirs::locate("data/countries.list"));

  TextHtmlConvertor::ref();

  ClipboardWrapper::thisInstance();

  (void) new ClientRMIComponent;

  qDebug("Initialising the VFS");

  vfs->initHierarchy();

  qDebug("Registering the factories");

  vfs->registerFactory
    ( VFS_ATTRIBUTE_MIMETYPE ,              new VAttributeFactory());

  vfs->registerFactory
    ( VFS_MAILATTRIBUTE_MIMETYPE ,          new VMailAttributeFactory());

  vfs->registerFactory
    ( VFS_PARTATTRIBUTE_MIMETYPE ,          new VPartAttributeFactory());

  vfs->registerFactory
    ( VFS_FOLDERATTRIBUTE_MIMETYPE ,        new VFolderAttributeFactory());

  vfs->registerFactory
    ( VFS_BLOCKOBJECTATTRIBUTE_MIMETYPE ,   new VBlockObjectAttributeFactory());

  // Load the components and build the UI.

  aetheraMan->init();

  vfs->useFactories(true);

  qDebug("Create contacts...");
  vfs->createVInstances(QString("contacts"));
  qDebug("Create notes...");
  vfs->createVInstances(QString("notes"));

  // Create the Account manager
  AccountManager::ref();
  
  // add here things to be made first time 
  // when you need VFS and plugins loaded etc
  if( setupDone )
  {
    KolabSharedFolder::ref()->checkKolabFolders();
  }
  
  delete splashScreen;
  splashScreen = 0;

  int iRet = app.exec();
  
  removeTemporaryFiles();
  
  delete ClipboardWrapper::thisInstance();
  delete ClientRMIComponent::ref();
  delete TextHtmlConvertor::ref();
//  delete CountriesClass::thisInstance();
  
  delete generalOptions;
  delete mailFilter;
  delete pluginManager;
  delete aetheraMan;
  delete vfs;
  
  delete ObjectRequestBroker::thisInstance();

  delete Settings::instance();

  return iRet;
}

  QString
currentSplashGraphic()
{
  int hour = QTime::currentTime().hour();

  if (hour < 6)
    return "aethera-splash-night.png";

  else if (hour < 12)
    return "aethera-splash-morning.png";

  else if (hour < 18)
    return "aethera-splash-day.png";

  else if (hour < 21)
    return "aethera-splash-evening.png";

  else
    return "aethera-splash-night.png";
}


  bool
setupWanted(int argc, char ** argv)
{
  // See if any command-line option looks like "-setup".

  for (int i = 0; i < argc; i++)
    if (QString(argv[i]).contains("-setup"))
      return true;

  return false;
}

  bool
shouldSetup()
{
  QString aetheraDir = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  
  // Check the config file for the top-level directory.
  if (aetheraDir.isEmpty())
    return true;
    
  // Check the version info file.
  QFile versionFile(aetheraDir + "/.version");
  if (!versionFile.open(IO_ReadOnly))
    return true;
  
  QTextStream txt(&versionFile);
  
  // The version string should look like : 01.23.45
  QString oldVersion = txt.readLine();
  versionFile.close();
  
  if( -1 == oldVersion.find('.') && oldVersion != "Beta 4" )
  {
    qDebug("Configuration found, older version (%s)", oldVersion.ascii());

    qDebug("Please make a fresh install!");
       
    return true;
  }
  else
  {
    if( oldVersion!=AppVersion && !AetheraUpdate::build(oldVersion) )
      return true;
  }
  
  // Check the top level directory and the system directories.
  if
    (
          verify(aetheraDir)
      &&  verify(aetheraDir + "/internal")
      &&  verify(aetheraDir + "/sysmail_inbox")
      &&  verify(aetheraDir + "/sysmail_outbox")
      &&  verify(aetheraDir + "/sysmail_saved")
      &&  verify(aetheraDir + "/sysmail_sent")
      &&  verify(aetheraDir + "/sysmail_drafts")
      &&  verify(aetheraDir + "/sysmail_deleted")
      &&  verify(aetheraDir + "/syscontacts")
      &&  verify(aetheraDir + "/sysgroups")
      &&  verify(aetheraDir + "/sysnotes")
    )
    return false;

  return true;
}

  bool
verify(QString name)
{
  QFileInfo f(name);

  if (f.exists() && f.isReadable() && f.isWritable())
    return true;
  
  qDebug("%s does not exist, or has incorrect permissions", name.ascii());

  return false;
}

  void
runSetupWizard()
{
  SetupWizard setupWizard;

  if (QDialog::Accepted == setupWizard.exec())
    setupDone = true;
  else
    exit(0);
}

  void
removeTemporaryFiles()
{
  QString aetheraDir = Settings::instance()->readEntry("/Aethera/Directory/AetheraDir");
  if( !aetheraDir.isEmpty() )
  {
    QDir dir(aetheraDir);
    if( dir.exists() )
    {
      QStringList tempFiles = dir.entryList("tmp_*", QDir::Files);
      for(QStringList::ConstIterator it=tempFiles.begin(); it!=tempFiles.end(); ++it)
      {
        qDebug("Remove file %s", (*it).latin1());
        dir.remove(*it);
      }
    }
  }
}
