/***************************************************************************
                          alarmconfigservice.cpp  -  description
                             -------------------
    begin                : Wed Sep 3 2003
    copyright            : (C) 2003 by theKompany
    author               : Eugen Constantinescu <eug@thekompany.com>
    author                : Vlad Bodi <bodivlad@hotmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


//own includes
#include "alarmconfigservice.h"
#include "koprefsdialog.h"
#include <alarmconfigwidgetbase.h>

//tkcBase includes
#include <qiconloader.h>

//QT includes
#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>

const char* configPath = "/Aethera/DateBook/Alarm/";

/*
----------------------------------------------------------------------------------------
The service factory
----------------------------------------------------------------------------------------
*/
kore::Interface* alarmconfigservice_factory()
{
  return new AlarmConfigService();
}

/*
----------------------------------------------------------------------------------------
KomKOConfigService::Private class
----------------------------------------------------------------------------------------
*/
          
class AlarmConfigService::Private
{
  public:

    Private(AlarmConfigService * p);
    virtual ~Private();

    AlarmConfigService* parent;
    AlarmConfigWidgetBase * widget;
};

AlarmConfigService::Private::Private(AlarmConfigService * p)
  : parent(p),
    widget(0L)
{
}

AlarmConfigService::Private::~Private()
{
  if( widget )
  {
    delete widget;
    widget = 0L;
  }
}

/*
----------------------------------------------------------------------------------------
 AlarmConfigService class
----------------------------------------------------------------------------------------
*/

AlarmConfigService::AlarmConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();

  info->replaceAll("name", "Alarm Config Service");
  info->replaceAll("class", "AlarmConfigService");
  info->replaceAll("class/fqn", "AlarmConfigService");
  info->replaceAll("factory", "alarmconfigservice_factory");

  d = new Private(this);
}

AlarmConfigService::~AlarmConfigService()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

QWidget* AlarmConfigService::init(QWidget * parent)
{
  d->widget = new AlarmConfigWidgetBase(parent,"Alarm Config Widget");
  return d->widget;
}

void AlarmConfigService::load()
{
  if( 0==d->widget )
    return;

  QString l_soundFile =Settings::instance()->readEntry( QString(configPath)+"soundPath", "" );

  d->widget->soundFileName->setText(l_soundFile);

  d->widget->CheckEnable->setChecked( Settings::instance()->readBoolEntry(QString(configPath)+"Enable",false) );

  if ( Settings::instance()->readBoolEntry(QString(configPath)+"Enable",false) )
  {
    d->widget->Settings->setEnabled( true );
  }
  else
  {
    d->widget->Settings->setDisabled( true );
  }

  if( d->widget->Settings->isEnabled() )
  {
    d->widget->SoundAlarm->setChecked( Settings::instance()->readBoolEntry(QString(configPath)+"Sound",false) );
    d->widget->StatusBarAlarm->setChecked( Settings::instance()->readBoolEntry(QString(configPath)+"Status",false) );
    d->widget->PopUpAlarm->setChecked( Settings::instance()->readBoolEntry(QString(configPath)+"PopUp",false) );
  }
}

void AlarmConfigService::save()
{
  if (0 == d->widget)
    return;

  //writing file name to be played
  Settings::instance()->writeEntry( QString(configPath)+"soundPath", d->widget->soundFileName->text() );
  Settings::instance()->writeEntry( QString(configPath)+"Enable", d->widget->CheckEnable->isChecked() );
  Settings::instance()->writeEntry( QString(configPath)+"Sound", d->widget->SoundAlarm->isChecked() );
  Settings::instance()->writeEntry( QString(configPath)+"PopUp", d->widget->PopUpAlarm->isChecked() );
  Settings::instance()->writeEntry( QString(configPath)+"Status", d->widget->StatusBarAlarm->isChecked() );
}

void AlarmConfigService::defaults()
{
 if (0 == d->widget)
    return;
  //sound file
  d->widget->soundFileName->setText( "" );
  // Notice the "Permanent delete" value is saved backwards.
  d->widget->CheckEnable->setChecked( false );
  d->widget->Settings->setDisabled( true );
  d->widget->SoundAlarm ->setChecked( false );
  d->widget->PopUpAlarm ->setChecked( false );
  d->widget->StatusBarAlarm->setChecked( false );
}

QPixmap AlarmConfigService::icon()
{
  return QIconLoader::instance()->loadIcon( "DateBook_Alarm.png" );
}

QString AlarmConfigService::caption()
{
  return QObject::tr("DateBook/Alarm");
}


bool AlarmConfigService::modified()
{
  return false;
}


