
#include "calendarlistviewitem.h"
#include "calendarobject.h"

#include <vfsmimetype.h>

class CalendarListViewItem::Private
{
  public:
    Private(CalendarListViewItem*);
    virtual ~Private();
  
    VFSModule * vfs;
    QString url;
    
  private:
    CalendarListViewItem * parent_;
};

CalendarListViewItem::Private::Private(CalendarListViewItem * parent)
  : vfs(0L),
    parent_(parent)
{
}

CalendarListViewItem::Private::~Private()
{
}

CalendarListViewItem::CalendarListViewItem(VFSModule * vfs, const QString & url, QListView * parent)
  : QListViewItem(parent)
{
//  qDebug("Create UI for %s", url.latin1());
  
  d = new Private(this);
  
  d->vfs = vfs;
  d->url = url;
  
  setDragEnabled(true);
  update();
}

CalendarListViewItem::~CalendarListViewItem()
{
  if(d)
  {
    delete d;
    d = 0L;
  }
}

void CalendarListViewItem::update()
{
  CalendarObject * object = vObject();
  
  if(object)
  {
    setText(0, object->calendarName());
  }
}

QString CalendarListViewItem::path()
{
  return d->url;
}

int CalendarListViewItem::rtti() const
{
  return CALENDAR_ITEM_ID;
}

CalendarObject * CalendarListViewItem::vObject() const
{
  return static_cast<CalendarObject*> (d->vfs->_fact_ref(d->url, VFS_CALENDAR_MIMETYPE));
}

QString CalendarListViewItem::key(int column, bool) const
{
  QString keyString;
  
  switch(column)
  {
    case 0:
    default:
      keyString = text(column);
      break;
  }
  
  return keyString;
}

