/**This takes care of calendar list view and calendar view.
  *@author Eugen Constantinescu <eug@thekompany.com>
  */

/** modified by Fabien Armand 2004/06/04 <far@atolcd.com>
*/


#include "calendarmainview.h"
#include "calendarobjectview.h"
#include "calendarlistview.h"
#include "calendarlistviewitem.h"
#include "calendarobject.h"
#include "calendarobjectview.h"
#include "kolabinterface.h"

#include <icalformat.h>
#include <calendarlocal.h>
#include <koprefs.h>
#include <koviewmanager.h>
#include <kodialogmanager.h>

#include <qapplication.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qstringlist.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qinputdialog.h>

#include <aethera/clientrmi.h>
#include <aethera/vfsmodule.h>
#include <aethera/vfolder.h>
#include <aethera/vfolderattribute.h>
#include <aethera/folderchooserdialog.h>
#include <aethera/kolabsharedfolder.h>

class CalendarMainView::Private
{
  public:
    Private(CalendarMainView*);
    ~Private();
    
    VFSModule *vfs;
    ClientRMIModule *clientRMI;
    QString parentFolder;
    
    CalendarListView * listView;
    CalendarObjectView * calendarView;
    QSplitter * splitter;
    QTimer * autosave;
    
  private:
    CalendarMainView * parent_;
};

CalendarMainView::Private::Private(CalendarMainView * parent)
  : vfs(0),
    clientRMI(0),
    parentFolder("/Aethera/VFS/Datebook/"),
    listView(0),
    calendarView(0),
    autosave(0),
    parent_(parent)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  
  splitter = new QSplitter(Vertical, parent);

  QVBoxLayout * layout = new QVBoxLayout(parent);

  layout->addWidget(splitter);

  listView = new CalendarListView(vfs, splitter, "CalendarListView");
  calendarView = new CalendarObjectView(vfs, splitter, "CalendarObjectView");
  
  autosave = new QTimer(parent, "KOAutosaveTimer");
}

CalendarMainView::Private::~Private()
{
}

CalendarMainView::CalendarMainView(QWidget *parent, const char *name )
  : QWidget(parent, name)
{
  d = new Private(this);
  
  // connections
  connect
    (
      d->listView,
      SIGNAL(currentChanged(QListViewItem *)),
      this,
      SLOT(slotCurrentChanged(QListViewItem *))
    );
 
  connect
    (
      d->listView,
      SIGNAL(selectionChanged()),
      this,
      SLOT(slotSelectionChanged())
    );

  connect
    (
      d->listView,
      SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)),
      this,
      SLOT(slotContextMenuRequested(QListViewItem *, const QPoint &, int))
    );
    
  connect
    (
      d->autosave,
      SIGNAL(timeout()),
      this,
      SLOT(slotSave())
    );
  if( KOPrefs::instance()->mAutoSave )
  {
    d->autosave->start(KOPrefs::instance()->mAutoSaveInterval * 60000 /* for min */);
  }

  // Make the preview larger than the list.
  QValueList<int> sizeList;
  sizeList << height() / 4 << height() / 4 * 3;
  d->splitter->setSizes(sizeList);
  
  slotSetFolder( d->vfs->root() + "/Datebook" );
}

CalendarMainView::~CalendarMainView()
{
  delete d;
  d = 0;
}

void CalendarMainView::slotCreateVInstance(const QString & url)
{
  QString folderPath;
      
  folderPath = url.left( url.findRev('/') );
  
  if( d->parentFolder == folderPath && 
      d->listView )
  {
    CalendarObject *item = static_cast<CalendarObject*> (d->vfs->_fact_ref(url, VFS_CALENDAR_MIMETYPE));

    if(item)
    {
      new CalendarListViewItem(d->vfs, url, d->listView);
      
      delete item;
    }
    
    updateView();
  }
}

void CalendarMainView::slotCreateVInstances(const QStringList & urls)
{
  if( urls.size() )
  {
//    qDebug("CalendarMainView::slotCreateVInstances: %s", urls[0].latin1());
    
    // if it isn't a folder, huh
    if( !d->vfs->folderRef(urls[0]) )
    {
      QStringList paths = urls;
      QString folderPath;
      
      folderPath = urls[0].left( urls[0].findRev('/') );
      
//      qDebug("Parent= %s | Folder= %s", d->parentFolder.latin1(), folderPath.latin1());

      if( d->parentFolder == folderPath && 
          d->listView )
      {
        d->listView->slotCreateVInstances(urls);
        
        updateView();
      }
    }
  }
}

void CalendarMainView::slotUpdateVInstances(const QStringList & urls)
{
  if( urls.size() )
  {
//    qDebug("CalendarMainView::slotUpdateVInstances: %s", urls[0].latin1());
    
    // if it isn't a folder, huh
    if( !d->vfs->folderRef(urls[0]) )
    {
      QString folderPath;
      
      folderPath = urls[0].left( urls[0].findRev('/') );
//      qDebug("Parent= %s | Folder= %s", d->parentFolder.latin1(), folderPath.latin1());


      if( d->parentFolder == folderPath && 
          d->listView &&
          d->calendarView )
      {
        d->listView->slotUpdateVInstances(urls);
        
        CalendarListViewItem * item = dynamic_cast<CalendarListViewItem*>(d->listView->currentItem());
        if( item )
        {
          for(QStringList::ConstIterator it=urls.begin(); it!=urls.end(); ++it)
          {
            if( item->path() == (*it) )
              d->calendarView->update();
          }
        }
      }
    }
  }
}

void CalendarMainView::slotUpdateVInstance(const QString & url)
{
  QString folderPath;
      
  folderPath = url.left( url.findRev('/') );
  
  if( d->parentFolder == folderPath && 
      d->listView )
  {
    QStringList urls;
    
    urls.push_back(url);
    
    d->listView->slotUpdateVInstances(urls);
  }
}

void CalendarMainView::slotDeleteVInstances(const QStringList & urls)
{
  if( urls.size() )
  {
    
//    qDebug("CalendarMainView::slotDeleteVInstances: %s", urls[0].latin1());
    
    // if it isn't a folder, huh
    if( !d->vfs->folderRef(urls[0]) )
    {
      QStringList paths = urls;
      QString folderPath;
      
//      qDebug("Parent= %s | Folder= %s", d->parentFolder.latin1(), folderPath.latin1());

      folderPath = urls[0].left( urls[0].findRev('/') );

      if( d->parentFolder == folderPath && 
          d->listView )
      {
        d->listView->slotDeleteVInstances(urls);
        
        updateView();
      }
    }
  }
}

QString CalendarMainView::getCurrentObject() const
{
  if( d->calendarView )
    return d->calendarView->getCurrentObject();
  else
    return QString::null;
}

QString CalendarMainView::getCurrentFolder() const
{
  return d->parentFolder;
}

void CalendarMainView::slotSetFolder(const QString & folderPath)
{
  if( d->parentFolder != folderPath )
  {
    d->parentFolder = folderPath;
    
    // check for kolab
    QString kolab = KolabInterface::instance()->kolabPath();
    if( folderPath + "/" == kolab )
    {
      // add the kolab related objects
      // one object for each kolab account
      VFolder * folder = d->vfs->folderRef(folderPath);
      if( folder )
      {
        QStringList kolabList = 
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
               
        folder->loadObjects();
        
        CalendarObject * calendarObject;
        QPtrList<VObject> objects = folder->objectList();
        for( QPtrListIterator<VObject> it(objects); it.current(); ++it)
        {
          calendarObject = static_cast<CalendarObject*>( it.current() );
          if( calendarObject )
          {
            if( !kolabList.remove( calendarObject->calendarName() ) )
            {
              // remove the objects which aren't related to kolab
              d->vfs->remove( calendarObject->path() );
            }
          }
        }
        
        // create the new calendar objects
        for(QStringList::Iterator it1=kolabList.begin(); it1!=kolabList.end(); ++it1)
        {
          QString name = d->vfs->createObject( folder->path() );
          if( !name.isEmpty() )
          {
            name = folder->path() + "/" + name;
          }
          else
          {
            qDebug("kolab: couldn't create a new object.");
          }
            
          CalendarObject * object = static_cast<CalendarObject*>(d->vfs->_fact_ref(name, VFS_CALENDAR_MIMETYPE));
          if( object )
          {
            object->setCalendarName(*it1);
              
            ICalFormat ical;
            CalendarLocal * emptyCalendar = new CalendarLocal(KOPrefs::instance()->mTimeZoneId.local8Bit());
            emptyCalendar->setOwner(KOPrefs::instance()->fullName());
            emptyCalendar->setEmail(KOPrefs::instance()->email());
            
            object->setCalendarInfo( ical.toString( emptyCalendar ) );
            object->save();
            
            delete emptyCalendar;
          }
          else
          {
            qDebug("kolab: couldn't create a new object.");
          }
        }
        
        folder->freeObjects();
      }
    }

    // check for shared kolab folders
    QString emailFolder = KolabInterface::instance()->isKolabSharedFolder(folderPath);
    if( !emailFolder.isEmpty() )
    {
      // add the kolab related objects
      // one object 
      VFolder * folder = d->vfs->folderRef(folderPath);
      if( folder )
      {
        folder->loadObjects();
      
        // remove the bad objects
        CalendarObject * calendarObject;
        QPtrList<VObject> objects = folder->objectList();
        for( QPtrListIterator<VObject> it(objects); it.current(); ++it)
        {
          calendarObject = static_cast<CalendarObject*>( it.current() );
          if( calendarObject )
          {
            if( calendarObject->calendarName() != folder->name() )
            {
              d->vfs->remove( calendarObject->path() );
            }
          }
        }
        
        objects = folder->objectList();
        if( objects.isEmpty() )
        {
          QString name = d->vfs->createObject( folder->path() );
          if( !name.isEmpty() )
          {
            name = folder->path() + "/" + name;
          }
          else
          {
            qDebug("kolab: couldn't create a new object.");
          }

          CalendarObject * object = static_cast<CalendarObject*>(d->vfs->_fact_ref(name, VFS_CALENDAR_MIMETYPE));
          if( object )
          {
            object->setCalendarName(folder->name());
            ICalFormat ical;
            CalendarLocal * emptyCalendar = new CalendarLocal(KOPrefs::instance()->mTimeZoneId.local8Bit());
            emptyCalendar->setOwner(KOPrefs::instance()->fullName());
            emptyCalendar->setEmail(KOPrefs::instance()->email());
            object->setCalendarInfo( ical.toString( emptyCalendar ) );
            object->save();
            delete emptyCalendar;
          }
          else
          {
            qDebug("kolab: couldn't create a new object.");
          }
        }
        
        folder->freeObjects();
      }
    }
    
    d->listView->slotSetFolder(folderPath);
    
    showItem(CalendarMainView::TOP);
  }
}

void CalendarMainView::slotShowCalendar(CalendarListViewItem * item)
{
  if(item)
    slotShowCalendar(item->path());
}

void CalendarMainView::slotShowCalendar(const QString & url)
{
  d->calendarView->slotSetObject(url);
}

void CalendarMainView::slotCut()
{
  if( d->listView->isSelectionEmpty() )
    return;
    
  d->listView->slotCut();
    
//  emit pastePossible(true);
}

void CalendarMainView::slotEditCut()
{
  d->calendarView->edit_cut();
}

void CalendarMainView::slotCopy()
{
  if( d->listView->isSelectionEmpty() )
    return;
    
  d->listView->slotCopy();
    
//  emit pastePossible(true);
}

void CalendarMainView::slotEditCopy()
{
  d->calendarView->edit_copy();
}

void CalendarMainView::slotPaste()
{
  d->listView->slotPaste();
}

void CalendarMainView::slotEditPaste()
{
  d->calendarView->edit_paste();
}

void CalendarMainView::slotDelete()
{
  d->listView->slotDelete();
}

void CalendarMainView::slotEditDelete()
{
  d->calendarView->appointment_delete();
}

void CalendarMainView::slotEditFind()
{
  d->calendarView->dialogManager()->showSearchDialog();
}

void CalendarMainView::slotMoveTo()
{
  QStringList pathList = d->listView->getSelection();

  QString targetPath = FolderChooserDialog::getFolder("/Datebook", this);

  if (!targetPath || targetPath == d->parentFolder)
    return;

  for
    (
      QStringList::ConstIterator it(pathList.begin());
      it != pathList.end();
      ++it
    )
    {
      d->vfs->move(*it, targetPath);
    }
}

void CalendarMainView::slotCopyTo()
{
  QStringList pathList = d->listView->getSelection();

  QString targetPath = FolderChooserDialog::getFolder("/Datebook", this);

  if (!targetPath || targetPath == d->parentFolder)
    return;

  for
    (
      QStringList::ConstIterator it(pathList.begin());
      it != pathList.end();
      ++it
    )
    {
      d->vfs->copy(*it, targetPath);
    }
}

void CalendarMainView::slotSelectAll()
{
  d->listView->selectAll();
}

void CalendarMainView::slotUnSelectAll()
{
  d->listView->unselectAll();
}

void CalendarMainView::updateView()
{
  showItem(CalendarMainView::CURRENT, true);
}

void CalendarMainView::showItem(ItemType type, bool direction)
{
//  qDebug("CalendarMainView::showItem");
  
  if( !d->listView )
    return;
  
  CalendarListViewItem * item = 0;

  switch(type)
  {
    case CalendarMainView::NEXT :
      item = dynamic_cast<CalendarListViewItem*>(d->listView->currentItem());
      if(item)
      {
        item = direction ? dynamic_cast<CalendarListViewItem*>(item->itemBelow()) : 
                           dynamic_cast<CalendarListViewItem*>(item->itemAbove());
      }
      break;
      
    case CalendarMainView::TOP :
      item = dynamic_cast<CalendarListViewItem*>(d->listView->firstChild());
      break;
      
    case CalendarMainView::BOTTOM :
      item = dynamic_cast<CalendarListViewItem*>(d->listView->firstChild());
      if(item)
      {
        while(item->nextSibling())
        {
          item = dynamic_cast<CalendarListViewItem*>(item->itemBelow());
        }
      }
      break;
    case CalendarMainView::CURRENT :
    default:
      item = dynamic_cast<CalendarListViewItem*>(d->listView->currentItem());
      break;
  }
  

  // we should display something even if the previous selection methods didn't find anything useful
  if(!item)
    item = dynamic_cast<CalendarListViewItem*>(d->listView->currentItem());
  if(!item)
    item = dynamic_cast<CalendarListViewItem*>(d->listView->firstChild());

  if(item)
  {
    d->listView->clearSelection();
    d->listView->setCurrentItem(item);
    d->listView->setSelected(item, true);
    d->listView->ensureItemVisible(item);

    slotShowCalendar(item);
  }
  else
  {
    d->calendarView->slotClear();
  }
}

void CalendarMainView::slotCurrentChanged(QListViewItem * item)
{
  qDebug("CalendarMainView::slotCurrentChanged");
  
  if(item)
  {
    slotShowCalendar(dynamic_cast<CalendarListViewItem*>(item));
  }
}

void CalendarMainView::slotSelectionChanged()
{
  if( d->listView->isSelectionEmpty() )
  {
    emit copyPossible(false);
  }
  else
  {
    emit copyPossible(true);
  }
}

void CalendarMainView::slotContextMenuRequested(QListViewItem * item, const QPoint &, int)
{
  enum { Remove, Cut, Copy, Paste, MoveTo, CopyTo, SelectAll, UnselectAll, KolabSynch };

  int action = -1;
  bool bKolab = false;
  bool bKolabShared = false;
  QString emailFolder;
       
  {
    QPopupMenu pop(d->listView);

    if( !item )
    {
      pop.insertItem(tr("&Paste"),        Paste);
      pop.insertSeparator();
      pop.insertItem(tr("&Select All"),   SelectAll);
      pop.insertItem(tr("&Unselect All"), UnselectAll);

//      if (!canPaste())
//        pop.setItemEnabled(Paste, false);
    }
    else
    {
      if( CALENDAR_ITEM_ID != item->rtti() )
      {
        return;
      }
       
      CalendarListViewItem * calendarItem = static_cast<CalendarListViewItem*>(item);
      CalendarObject * object = static_cast<CalendarObject*>(calendarItem->vObject());
      if( !object )
      {
        qDebug("The calendar item is null.");
        return;
      }
       
      // check if it's a kolab related calendar
      emailFolder = KolabInterface::instance()->isKolabCalendarObject( object->path() );
      // check if the folder is a kolab shared folder
      if( emailFolder.isEmpty() )
      {
        emailFolder = KolabInterface::instance()->isKolabSharedFolder
          ( object->parentFolder()->path() );
        if( !emailFolder.isEmpty() )
        {
          bKolabShared = true;
        }
      }
      else
      {
        bKolab = true;
      }
      
      if( !bKolab && !bKolabShared )
      {
        pop.insertItem(tr("&Cut"),  Cut);
        pop.insertItem(tr("&Copy"), Copy);
        pop.insertItem(tr("&Paste"),Paste);
        pop.insertSeparator();
        pop.insertItem(tr("&Remove"), Remove);
        pop.insertSeparator();
        pop.insertItem(tr("Copy to..."), CopyTo);
        pop.insertItem(tr("&Move to..."), MoveTo);
        pop.insertSeparator();
        pop.insertItem(tr("&Select All"),   SelectAll);
        pop.insertItem(tr("&Unselect All"), UnselectAll);
      }
      else
      {
        pop.insertItem(tr("&Kolab Synchronization"), KolabSynch);
        pop.insertSeparator();
        pop.insertItem(tr("&Copy"), Copy);
        pop.insertItem(tr("Copy to..."), CopyTo);
        pop.insertSeparator();
        pop.insertItem(tr("&Select All"),   SelectAll);
        pop.insertItem(tr("&Unselect All"), UnselectAll);
      }
    }

    action = pop.exec(QCursor::pos());
  }

  switch (action)
  {
    case Cut:
      slotCut();
      break;
    case Copy:
      slotCopy();
      break;
    case Paste:
      slotPaste();
      break;
    
    case Remove:
      slotDelete();
      break;

    case MoveTo:
      slotMoveTo();
      break;
    case CopyTo:
      slotCopyTo();
      break;
    
    case SelectAll:
      slotSelectAll();
      break;
    case UnselectAll:
      slotUnSelectAll();
    
    case KolabSynch:
      slotKolabSynch(emailFolder);
      break;

    default:
      break;
  }
}

void CalendarMainView::slotKolabSynch(const QString & kolabPath)
{
  VFolder * folder = d->vfs->folderRef(kolabPath);
  if(!folder)
  {
    KolabSharedFolder::ref()->checkKolabFolders();
  }
      
  d->clientRMI->mount(kolabPath);
  d->clientRMI->unmount(kolabPath);
}

void CalendarMainView::slotNew()
{
  QString calendarName;
  QString objectURL;
  
  QString name = d->vfs->createObject(d->parentFolder);
        
  if( !name.isEmpty() )
  {
    objectURL = d->parentFolder + "/" + name;
        
    calendarName = QInputDialog::getText("New Calendar", "Choose a name for the new calendar.");
  }
  else
  {
    qDebug("CalendarObjectView::slotNewObject: couldn't create a new object.");
    return;
  }
    
  CalendarObject * object = static_cast<CalendarObject*>(d->vfs->_fact_ref(objectURL, VFS_CALENDAR_MIMETYPE));
  if( object )
  {
    if( object->calendarName().isEmpty() || 
        "unknown" == object->calendarName() )
    {
      if( calendarName.isEmpty() )
        object->setCalendarName("unknown");
      else
        object->setCalendarName( calendarName );
    }
      
    ICalFormat ical;
    CalendarLocal * emptyCalendar = new CalendarLocal(KOPrefs::instance()->mTimeZoneId.local8Bit());
    emptyCalendar->setOwner(KOPrefs::instance()->fullName());
    emptyCalendar->setEmail(KOPrefs::instance()->email());
    
    object->setCalendarInfo( ical.toString( emptyCalendar ) );
    object->save();
    
    delete emptyCalendar;
  }
  else
  {
    qDebug("CalendarObjectView::slotNewObject: couldn't save the object.");
    return;
  }
}

void CalendarMainView::slotSave()
{
  d->calendarView->slotSaveObject();
}

void CalendarMainView::slotFileImport()
{
  QString fileName = QFileDialog::getOpenFileName();
  if ( fileName.isEmpty() ) return;
  
  QMessageBox mergeMessageBox(  tr("Import Calendar Data"),
                                tr("Do you want to merge or to overwrite the calendar data ?"),
                                QMessageBox::Information,
                                QMessageBox::Yes | QMessageBox::Default,
                                QMessageBox::No,
                                QMessageBox::Cancel | QMessageBox::Escape);
  
  mergeMessageBox.setButtonText( QMessageBox::Yes, tr("Merge"));
  mergeMessageBox.setButtonText( QMessageBox::No, tr("Overwrite"));
  
  switch( mergeMessageBox.exec() )
  {
    case QMessageBox::Yes :
    case QMessageBox::Default :
      d->calendarView->openCalendar(fileName, true);
      break;
    case QMessageBox::No :
      d->calendarView->openCalendar(fileName, false);
      break;
    case QMessageBox::Cancel :
    case QMessageBox::Escape :
    default :
      break;
  }
}

void CalendarMainView::slotFileExport()
{
  QString fileName = QFileDialog::getSaveFileName();
  if ( fileName.isEmpty() ) return;
  
  d->calendarView->saveCalendar(fileName);
}

void CalendarMainView::slotViewList()
{
  d->calendarView->viewManager()->showListView();
}

void CalendarMainView::slotViewDay()
{
  d->calendarView->viewManager()->showDayView();
}

void CalendarMainView::slotViewMonth()
{
  d->calendarView->viewManager()->showMonthView();
}

void CalendarMainView::slotViewTodo()
{
  d->calendarView->viewManager()->showTodoView();
}

void CalendarMainView::slotViewWeek()
{
  d->calendarView->viewManager()->showWeekView();
}

void CalendarMainView::slotViewWhatsNext()
{
  d->calendarView->viewManager()->showWhatsNextView();
}

void CalendarMainView::slotViewWorkWeek()
{
  d->calendarView->viewManager()->showWorkWeekView();
}

void CalendarMainView::slotPreferences()
{
  d->calendarView->edit_options();
}

void CalendarMainView::slotGoPrevious()
{
  d->calendarView->goPrevious();
}

void CalendarMainView::slotGoToday()
{
  d->calendarView->goToday();
}

void CalendarMainView::slotGoNext()
{
  d->calendarView->goNext();
}

void CalendarMainView::slotNewEvent()
{
  d->calendarView->newEvent();
}

void CalendarMainView::slotNewTodo()
{
  d->calendarView->newTodo();
}

void CalendarMainView::slotFilePrint()
{
  d->calendarView->print();
}

void CalendarMainView::showOutgoingDialog( )
{
  KODialogManager* dlgManager = new KODialogManager( d->calendarView );  
  dlgManager->showOutgoingDialog();
}

void CalendarMainView::showIncomingDialog( )
{  
  KODialogManager* dlgManager = new KODialogManager( d->calendarView );  
  dlgManager->showIncomingDialog();
}

void CalendarMainView::schedule_publish()
{
  d->calendarView->schedule_publish();
}

void CalendarMainView::schedule_request( )
{
  d->calendarView->schedule_request();
}

void CalendarMainView::schedule_refresh( )
{
  d->calendarView->schedule_refresh();
}

void CalendarMainView::schedule_reply( )
{
  d->calendarView->schedule_reply();
}

void CalendarMainView::schedule_cancel( )
{
  d->calendarView->schedule_cancel();
}

void CalendarMainView::schedule_counter( )
{
  d->calendarView->schedule_counter();
}

void CalendarMainView::schedule_publish_freebusy( )
{
  d->calendarView->schedule_publish_freebusy();
}

CalendarObjectView* CalendarMainView::getCalendarObjectView()
{
  return d->calendarView;
}
