
#ifndef CALENDARMAINVIEW_H
#define CALENDARMAINVIEW_H

#include <qwidget.h>

class QListViewItem;
class QStringList;
class QPoint;

class CalendarListViewItem;
class CalendarObjectView;

/**This takes care of calendar list view and calendar view.
  *@author Eugen Constantinescu <eug@thekompany.com>
  */

class CalendarMainView : public QWidget
{
  Q_OBJECT
   
  public:
  
    enum ItemType { CURRENT=0, NEXT, TOP, BOTTOM };
  
    CalendarMainView(QWidget *parent=0, const char *name=0);
    virtual ~CalendarMainView();
    
    virtual void updateView();
    
    virtual QString getCurrentFolder() const;
    virtual QString getCurrentObject() const;
    
  public slots:
  
    void slotCreateVInstance(const QString&);
    void slotCreateVInstances(const QStringList &);
    void slotUpdateVInstances(const QStringList &);
    void slotUpdateVInstance(const QString &);
    void slotDeleteVInstances(const QStringList &);
    void slotSetFolder(const QString&);
    
    void slotShowCalendar(CalendarListViewItem *);
    void slotShowCalendar(const QString&);
    void slotCurrentChanged(QListViewItem *);
    void slotSelectionChanged();
    void slotContextMenuRequested(QListViewItem *, const QPoint &, int);
    
    void slotNew();
    void slotSave();
    void slotCut();
    void slotCopy();
    void slotPaste();
    void slotDelete();
    void slotMoveTo();
    void slotCopyTo();
    void slotSelectAll();
    void slotUnSelectAll();
    void slotKolabSynch(const QString&);
    
    void slotFileImport();
    void slotFileExport();
    
    void slotEditCut();
    void slotEditCopy();
    void slotEditPaste();
    void slotEditDelete();
    void slotEditFind();
    
    void slotViewList();
    void slotViewDay();
    void slotViewMonth();
    void slotViewTodo();
    void slotViewWeek();
    void slotViewWhatsNext();
    void slotViewWorkWeek();
    void slotPreferences();
    void slotGoPrevious();
    void slotGoToday();
    void slotGoNext();
    void slotNewEvent();
    void slotNewTodo();
    void slotFilePrint();
    
    void showOutgoingDialog( );
    void showIncomingDialog( );
    void schedule_publish( );
    void schedule_request( );
    void schedule_refresh( );
    void schedule_reply( );
    void schedule_cancel( );
    void schedule_counter( );
    void schedule_publish_freebusy( );
    
    CalendarObjectView* getCalendarObjectView();
  
  signals:
    void copyPossible(bool);
    void pastePossible(bool);
    void pasteEditPossible(bool);
    
  protected:
    void showItem(ItemType, bool direction=true);
  
  private:
  
    class Private;
    Private * d;
};

#endif
