
#include "calendarobject.h"

#include <any.h>
#include <vfsmodule.h>

#include <qstring.h>

// Private
class CalendarObject::Private
{
  public:
  
    Private(CalendarObject*);
    virtual ~Private();
    
    // attributes
    QString calendarName;
    QString calendarInfo;
    
    unsigned int size;
    
  private:
    CalendarObject * parent_;
};

CalendarObject::Private::Private(CalendarObject * parent)
  : size(0),
    parent_(parent)
{
}

CalendarObject::Private::~Private()
{
}

CalendarObject::CalendarObject(const QString & _path)
  : VObject(_path)
{
  d = new Private(this);
  
  // restore the object properties from the server
  sync();
}

CalendarObject::CalendarObject(const QString & _path, const QByteArray &_data)
  : VObject(_path)
{
  d = new Private(this);
  
  // restore the object properties from the server
  if( !_data || !_data.size() )
    sync();
  else
    _narrow(_data);
}

CalendarObject::~CalendarObject()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

QByteArray CalendarObject::_stream()
{
  QByteArray tdata;
  QDataStream tstream(tdata, IO_WriteOnly);

//  qDebug("Saving...");
//  showInfo();
  
  // stream the object properties
  tstream << d->calendarName
          << d->calendarInfo;
          
  d->size = tdata.size();
  
  // return the byte array
  return tdata;
}

void CalendarObject::_narrow(const QByteArray &tdata)
{
  // check if the stream actually contains any data
  if(tdata.size())
  {
    QDataStream tstream(tdata, IO_ReadOnly);
    
    // unstream the object properties
    tstream >> d->calendarName
            >> d->calendarInfo;
    
    d->size = tdata.size();
  }
  else
  {
    d->calendarName = "unknown";
  }
  
//  qDebug("Loading...");
//  showInfo();
}

bool CalendarObject::getProperty(const QString&, Any&)
{
  return false;
}

bool CalendarObject::setProperty(const QString&, Any&)
{
  return false;
}

QueryProcessor::QueryType CalendarObject::queryType(const QString&)
{
  return QueryProcessor::InvalidQuery;
}

QString CalendarObject::calendarName() const
{
  return d->calendarName;
}

void CalendarObject::setCalendarName(const QString & text)
{
  d->calendarName = text;
}
     
QString CalendarObject::calendarInfo() const
{
  return d->calendarInfo;
}

void CalendarObject::setCalendarInfo(const QString & text)
{
  d->calendarInfo = text;
}
     
unsigned int CalendarObject::size() const
{
  return d->size;
}
    
void CalendarObject::showInfo()
{
  // for debugging
  qDebug("Calendar Object: %s", path().latin1());
  qDebug("Name     : %s", calendarName().latin1());
}
   
#ifdef _QTWIN_
VObject * CalendarObjectFactory::_new(const QString & _path)
#else
CalendarObject * CalendarObjectFactory::_new(const QString & _path)
#endif
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      qDebug("NO VFS Provider!");
      return 0;
    }
  }

  // check if it is possible to create a new board
  if(vfs->canDefine(_path) && !vfs->reference(_path))
    return new CalendarObject(_path);
  else
    return 0;
}

#ifdef _QTWIN_
VObject *CalendarObjectFactory::_new(const QString & _path, const QByteArray &_data)
#else
CalendarObject *CalendarObjectFactory::_new(const QString & _path, const QByteArray &_data)
#endif
{
  return new CalendarObject(_path, _data);
}

