
#ifndef CALENDAR_OBJECT_H
#define CALENDAR_OBJECT_H

#include <vobject.h>
#include <vobjectfactory.h>
#include <queryprocessor.h>

class QString;

class DATEBOOK_PLUGIN_API CalendarObject : public VObject
{
  friend class CalendarObjectFactory;
  
  public:
  
    // the destructor
    virtual ~CalendarObject();
  
    // object marshalling methods (overloaded from VObject, so that save() and sync() work)
    QByteArray _stream();
    void _narrow(const QByteArray &);
    
    // query handling
    virtual bool getProperty(const QString &query, Any &property);
    virtual bool setProperty(const QString &query, Any &property);
    virtual QueryProcessor::QueryType queryType(const QString &query);
    
    // set/get attributes
    virtual QString calendarName() const;
    virtual void setCalendarName(const QString&);
    
    virtual QString calendarInfo() const;
    virtual void setCalendarInfo(const QString&);
    
    virtual unsigned int size() const;
    
    void showInfo();
    
  protected:
  
    // the constructor is always protected or private
    CalendarObject(const QString & _path);
    CalendarObject(const QString & _path, const QByteArray &_data);
    
  private:
  
    class Private;
    Private * d;
};

class DATEBOOK_PLUGIN_API CalendarObjectFactory : public VObjectFactory
{
  public:
  
#ifdef _QTWIN_
    VObject *_new(const QString & _path, const QByteArray &_data);
    VObject *_new(const QString & _path);
#else
    CalendarObject *_new(const QString & _path, const QByteArray &_data);
    CalendarObject *_new(const QString & _path);
#endif
};

#endif


