/***************************************************************************
         calendarobjectview.cpp  - 
                             -------------------
    begin                : Fri Jan 18 21:44:25 EET 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/

#include "calendarobjectview.h"
#include "calendarobject.h"

#include <icalformat.h>
#include <calendarlocal.h>
#include <koprefs.h>

// Aethera
#include <vfsmodule.h>
#include <vfsmimetype.h>

// QT
#include <qinputdialog.h>

class CalendarObjectView::Private
{
  public:
    Private(CalendarObjectView*);
    virtual ~Private();
    
    VFSModule * vfs;
    QString objectURL;
    
  private:
    CalendarObjectView * parent_;
};

CalendarObjectView::Private::Private(CalendarObjectView * parent)
  : vfs(0L),
    parent_(parent)
{
}

CalendarObjectView::Private::~Private()
{
}

CalendarObjectView::CalendarObjectView(VFSModule * vfs, QWidget * parent, const char * name)
  : CalendarView(parent, name)
{
  d = new CalendarObjectView::Private(this);
  
  d->vfs = vfs;
}

CalendarObjectView::~CalendarObjectView()
{
  if(d)
  {
    delete d;
    d = 0L;
  }
}

QString CalendarObjectView::getCurrentObject() const
{
  return d->objectURL;
}

void CalendarObjectView::slotSetObject(const QString & url)
{
  if( !d->objectURL.isEmpty() )
  {
    slotSaveObject();
  }
  
  d->objectURL = url;
  
  update();
}

void CalendarObjectView::slotClear()
{
  if( !d->objectURL.isEmpty() )
    slotSaveObject();
  
  d->objectURL = QString::null;
  
  update();
}

void CalendarObjectView::update()
{
  closeCalendar();
  
  if( d->objectURL.isEmpty() )
  {
    // just nothing to show
    return;
  }
  
  CalendarObject * object = static_cast<CalendarObject*>(d->vfs->_fact_ref(d->objectURL, VFS_CALENDAR_MIMETYPE));
  if(object)
  {
    ICalFormat ical;
    if( object->calendarInfo().isEmpty() )
    {
      qDebug("The calendar object is new or something is bad.");
    }
    else
      ical.fromString( calendar(), object->calendarInfo() );
    
    updateView();
  }
  else
  {
    qDebug("CalendarObjectView::update() : No object");
    return;
  }
}

void CalendarObjectView::slotSaveObject()
{
  QString calendarName;
  
  // Create a new object if there isn't one already
  if( d->objectURL.isEmpty() )
  {
    QString name = d->vfs->createObject( d->vfs->root() + "/Datebook" );
        
    if( !name.isEmpty() )
    {
      d->objectURL = d->vfs->root() + "/Datebook/";
      d->objectURL += name;
        
      calendarName = QInputDialog::getText("Saving Calendar", "Choose a name for the calendar.");
    }
    else
    {
      qDebug("CalendarObjectView::slotSaveObject: couldn't create a new object.");
      return;
    }
  }
    
  qDebug("CalendarObjectView::slotSave(%s)", d->objectURL.latin1());
    
  CalendarObject * object = static_cast<CalendarObject*>(d->vfs->_fact_ref(d->objectURL, VFS_CALENDAR_MIMETYPE));
  if( object )
  {
    if( object->calendarName().isEmpty() || 
        "unknown" == object->calendarName() )
    {
      if( calendarName.isEmpty() )
        object->setCalendarName("unknown");
      else
        object->setCalendarName( calendarName );
    }
      
    ICalFormat ical;
    object->setCalendarInfo( ical.toString( calendar() ) );
      
    object->save();
  }
  else
  {
    qDebug("CalendarObjectView::slotSaveObject: couldn't save the object.");
    return;
  }
}
