/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

---------------------------------------------------------------------------------
    copyright             : (C) 2003 by theKompany.com
                            (C) 2004 by Atol C&D
    author                : Eugen Constantinescu <eug@thekompany.com>
                            Fabien Armand <far@atolcd.com>
    date                  : 15 Dec 2003
---------------------------------------------------------------------------------
*/

// own includes
#include "kolabinterface.h"
#include "komkouiservice.h"
#include "calendarmainview.h"
#include "calendarobjectview.h"
#include <koprefs.h>

// Aethera
#include <mailobj.h>
#include <partobj.h>
#include <vfsmodule.h>
#include <clientrmi.h>
#include <vfolder.h>
#include <appstatus.h>
#include <mailagentservice.h>

// support for kolab shared folder
#include <aethera/kolabsharedfolder.h>

// qt
#include <qtextcodec.h>
#include <qmessagebox.h>

// attribute separator
#define ATTR_SEP "^"

using namespace KOrg;

KolabInterface* KolabInterface::instance_ = 0L;

class KolabInterface::Private
{
  public:
  
    Private(KolabInterface*);
    ~Private();
  
    VFSModule * vfs;
    ClientRMIModule * clientRMI;
    MailAgentService * mailAgent;
    QString accountName;
    bool lock;
    
  private:
  
    KolabInterface * parent_;
};

KolabInterface::Private::Private(KolabInterface * parent)
  : vfs(0L),
    clientRMI(0L),
    mailAgent(0L),
    lock(false),
    parent_(parent)
{
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_RMI_CLIENT );
    if( ref.isValid() )
      clientRMI = static_cast<ClientRMIModule*>(ref.instance());
  }
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILAGENT_SERVICE );
    if( ref.isValid() )
       mailAgent = static_cast<MailAgentService*>( ref.instance() );
  }
  
  assert(0 != vfs);
  assert(0 != clientRMI);
}

KolabInterface::Private::~Private()
{
}

// --------------

KolabInterface * KolabInterface::instance()
{
  if( !instance_ )
  {
    instance_ = new KolabInterface();
  }
  
  return instance_;
}

KolabInterface::KolabInterface()
{
  d = new Private(this);
}

KolabInterface::~KolabInterface()
{
  if(d)
  {
    delete d;
    d = 0L;
  }
}

QString KolabInterface::kolabPath() const
{
  return d->vfs->root() + "/Datebook/Kolab/";
}

void KolabInterface::deleteKolabSharedFolder(const QString & path) const
{
  VFolder * calendarFolder = d->vfs->folderRef(path);
  if( calendarFolder )
  {
    // try with the current folder
    QString emailFolder = isKolabSharedFolder(path);
    if( !emailFolder.isEmpty() )
    {
      KolabSharedFolder::ref()->removeSharedFolder(d->accountName, emailFolder);
    }
    
    // check the subfolders
    VFolderList folderRefList;
    calendarFolder->tree(folderRefList, -1);
    
    for(VFolderListIterator it(folderRefList); it.current(); ++it)
    {
      emailFolder = isKolabSharedFolder(it.current()->path());
      if( !emailFolder.isEmpty() )
      {
        KolabSharedFolder::ref()->removeSharedFolder(d->accountName, emailFolder);
      }
    }
  }
}

QString KolabInterface::isKolabSharedFolder(const QString & path) const
{
  QString emailFolder;
  QString calendarKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( !path.find(calendarKolab + *itList) )
    {
      QString account = (*itList);
      QString mailbox = path.mid( QString(calendarKolab + account + "/").length() );
      
      if( KolabSharedFolder::ref()->isKolabSharedFolder(account, mailbox) )
      {
        emailFolder = d->vfs->root() + "/Mail/" + account + "/" + mailbox;
        d->accountName = account;
      }
      
      break;
    }
  }

  return emailFolder;
}

QString KolabInterface::isParentKolabSharedFolder(const QString & path) const
{
  QString emailFolder;
  QString calendarKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  
  for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
  {
    if( !path.find(calendarKolab + *itList) )
    {
      QString account = (*itList);
      QString mailbox = path.mid( QString(calendarKolab + account + "/").length() );
      emailFolder = d->vfs->root() + "/Mail/" + account + "/" + mailbox;
      
      QStringList sharedFolders;
      KolabSharedFolder::ref()->sharedFolders(sharedFolders, account, "calendar");
      KolabSharedFolder::ref()->sharedFolders(sharedFolders, account, "todo");
      
      for(QStringList::Iterator it=sharedFolders.begin(); it!=sharedFolders.end(); ++it)
      {
        if( !(*it).find(emailFolder) )
        {
          return emailFolder;
        }
      }
    }
  }

  return QString::null;
}

QString KolabInterface::isKolabCalendarObject(const QString & path, bool bToDo) const
{
  QString emailFolder;
  QString calendarKolab = kolabPath();
  QStringList kolabList =
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  CalendarObject * object = static_cast<CalendarObject*>
    (d->vfs->_fact_ref(path, VFS_CALENDAR_MIMETYPE));
  
  if( object && 
      calendarKolab == object->parentFolder()->path() + "/" )
  {
    QString calendarName = object->calendarName();
    for(QStringList::Iterator itList=kolabList.begin(); itList!=kolabList.end(); itList++)
    {
      if( calendarName == (*itList) )
      {
        QStringList kolabFolders;
        
        if( bToDo )
        {
          KolabSharedFolder::ref()->kolabFolders(kolabFolders, (*itList), "todo");
        }
        else
        {
          KolabSharedFolder::ref()->kolabFolders(kolabFolders, (*itList), "calendar");
        }
        
        if( kolabFolders.count() )
        {
          emailFolder = kolabFolders.first();
          d->accountName = (*itList);
        }
        
        break;
      }
    }
  }

  return emailFolder;
}

void KolabInterface::slotSynch
  ( const QString & accName,
    const QString & mailbox,
    bool bToDo )
{
  if( KolabSharedFolder::ref()->isKolabSharedFolder(accName, mailbox) )
  {
    slotSynchSharedFolder(kolabPath() + accName + "/" + mailbox);
    return;
  }

  if( d->lock )
  {
    return;
  }
  d->lock = true;
  
  CalendarObject * calendarObject = 0;

  // check for kolab object
  QString kolab = kolabPath();
  kolab = kolab.remove(kolab.length() - 1, 1);
  
  VFolder * folder = d->vfs->folderRef(kolab);
  if( folder )
  {
    folder->loadObjects();
           
    QPtrList<VObject> objects = folder->objectList();
    for( QPtrListIterator<VObject> it(objects); it.current(); ++it)
    {
      calendarObject = static_cast<CalendarObject*>( it.current() );
      if( calendarObject )
      {
        if( accName == calendarObject->calendarName() )
        {
          break;
        }
        else
        {
          // it's not found yet
          calendarObject = 0;
        }
      }
    }
  }
  else
  {
    qDebug("No kolab folder.");
    d->lock = false;
    return;
  }
  
  if( calendarObject )
  {
    QString folderPath = d->vfs->root() + "/Mail/" + accName + "/" + mailbox;
    VFolder * folder = d->vfs->folderRef( folderPath );
    QString objectPath = calendarObject->path();
        
    ICalFormat ical;
    CalendarLocal * kolabCalendar = new CalendarLocal(KOPrefs::instance()->mTimeZoneId.local8Bit());
    kolabCalendar->setOwner(KOPrefs::instance()->fullName());
    kolabCalendar->setEmail(KOPrefs::instance()->email());
    
    if( bToDo )
    {
      ical.fromString(kolabCalendar, calendarObject->calendarInfo());
    }
    
    AppStatus * status = AppStatus::instance();
    if( !status )
    {
      qDebug("Kolab calendar : no status bar");
    }
    
    QString current;
    QStringList urls = d->vfs->children( folderPath );
    QStringList partsUrl;
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      current = (*it) + ATTR_SEP + QString::number(0);
      qDebug(current);
      partsUrl.append( current );
    }
    
    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, "Kolab data loading...", 0, partsUrl.count());
    qApp->processEvents(1);
    
    d->vfs->createVInstances(partsUrl, false);
    
    int i = 1;
    for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
    {
      current = (*it1) + ATTR_SEP + QString::number(0);
  
      PartObjectInterface *part = static_cast<PartObjectInterface*>
        (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));
        
      if( part )
      {
        if( "attachment" == part->partType() &&
            part->mimeType().contains("calendar") )
        {
          QByteArray data_ = part->data();
          if( !data_.size() )
          {
            qDebug("calendar data is null");
          }
          
          QCString partData( data_ );
          QString ics;
          
          QTextCodec * codec = QTextCodec::codecForName( part->charset().latin1() );
          if(codec)
             ics = codec->toUnicode( partData );
          else
          {
            if( "8bit" == part->encoding().lower() )
              ics = QString::fromLocal8Bit( partData );
            else if( "utf-8" == part->encoding().lower() )
              ics = QString::fromUtf8( partData );
            else // 7 bit
              ics = QString::fromLatin1( partData );
          }
          
          // add the new event to the calendar
          ical.fromString(kolabCalendar, ics);
        }
      }
      else
      {
        qDebug("Warning: the calendar for %s isn't available.", current.latin1());
      }
      
      if( !(i%100) )
      {
        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, "Kolab data loading...", i);
        qApp->processEvents(1);
      }
      i++;
    }
    
    QString result = ical.toString( kolabCalendar );
    // we don't need the calendar
    delete kolabCalendar;
    kolabCalendar = 0;
    
    calendarObject = static_cast<CalendarObject*>
      (d->vfs->_fact_ref(objectPath, VFS_CALENDAR_MIMETYPE));
    
    if( calendarObject )
    {
      calendarObject->setCalendarInfo(  result );
      calendarObject->save();
    }
    else
    {
      qDebug("no kolab object");
    }
    
    if( !bToDo )
    {
      status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, "kolab events are loaded");
    }
    else
    {
      // free all the parts objects
      if( folder )
      {
        folder->freeObjects();
      }
    
      status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, "kolab todo are loaded");
    }
    
    // synch the ToDo when Events are ready
    if( !bToDo )
    {
      QString emailFolder;
      QStringList kolabFolders;
      KolabSharedFolder::ref()->kolabFolders(kolabFolders, accName, "todo");
      
      if( kolabFolders.count() )
      {
        emailFolder = kolabFolders.first();
      }
      if( !emailFolder.isEmpty() )
      {
        d->clientRMI->mount(emailFolder);
        d->clientRMI->unmount(emailFolder);
      }
      else
      {
        qDebug("No todo email folder for %s account.", accName.latin1());
      }
    }
  }
  else
  {
    qDebug("No such kolab object.");
  }
  
  d->lock = false;
}

void KolabInterface::slotSynchSharedFolder(const QString & datebookPath)
{
  QString folderPath = isKolabSharedFolder(datebookPath);
  if( d->lock || folderPath.isEmpty() )
  {
    return;
  }
  
  d->lock = true;

  CalendarObject * calendarObject = 0;

  // check for kolab object
  VFolder * folder = d->vfs->folderRef(datebookPath);
  if( folder )
  {
    folder->loadObjects();

    QPtrList<VObject> objects = folder->objectList();
    for( QPtrListIterator<VObject> it(objects); it.current(); ++it)
    {
      calendarObject = static_cast<CalendarObject*>( it.current() );      
    }
  }
  else
  {
    qDebug("No kolab folder.");
    d->lock = false;
    return;
  }

  if( calendarObject )
  {
    QString objectPath = calendarObject->path();
    
    ICalFormat ical;
    CalendarLocal * kolabCalendar = new CalendarLocal(KOPrefs::instance()->mTimeZoneId.local8Bit());
    kolabCalendar->setOwner(KOPrefs::instance()->fullName());
    kolabCalendar->setEmail(KOPrefs::instance()->email());

    AppStatus * status = AppStatus::instance();
    if( !status )
    {
      qDebug("Kolab calendar : no status bar");
    }

    QString current;
    QStringList urls = d->vfs->children( folderPath );
    QStringList partsUrl;
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      current = (*it) + ATTR_SEP + QString::number(0);
      partsUrl.append( current );
    }

    status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, "Kolab data loading...", 0, partsUrl.count());
    qApp->processEvents(1);

    d->vfs->createVInstances(partsUrl, false);

    int i = 1;
    for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
    {
      current = (*it1) + ATTR_SEP + QString::number(0);

      PartObjectInterface *part = static_cast<PartObjectInterface*>
        (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));

      if( part )
      {
        if( "attachment" == part->partType() &&
            part->mimeType().contains("calendar") )
        {
          QByteArray data_ = part->data();
          if( !data_.size() )
          {
            qDebug("calendar data is null");
          }

          QCString partData( data_ );
          QString ics;

          QTextCodec * codec = QTextCodec::codecForName( part->charset().latin1() );
          if(codec)
             ics = codec->toUnicode( partData );
          else
          {
            if( "8bit" == part->encoding().lower() )
              ics = QString::fromLocal8Bit( partData );
            else if( "utf-8" == part->encoding().lower() )
              ics = QString::fromUtf8( partData );
            else // 7 bit
              ics = QString::fromLatin1( partData );
          }

          // add the new event to the calendar
          ical.fromString(kolabCalendar, ics);
        }
      }
      else
      {
        qDebug("Warning: the calendar for %s isn't available.", current.latin1());
      }

      if( !(i%100) )
      {
        status->message(AppStatus::Client, AppStatus::Progress, AppStatus::Waiting, "Kolab data loading...", i);
        qApp->processEvents(1);
      }
      i++;
    }

    QString result = ical.toString( kolabCalendar );
    // we don't need the calendar
    delete kolabCalendar;
    kolabCalendar = 0;

    calendarObject = static_cast<CalendarObject*>
      (d->vfs->_fact_ref(objectPath, VFS_CALENDAR_MIMETYPE));

    if( calendarObject )
    {
      calendarObject->setCalendarInfo(  result );
      calendarObject->save();
    }
    else
    {
      qDebug("no kolab object");
    }

    status->message(AppStatus::Client, AppStatus::Text, AppStatus::Good, "kolab events are loaded");
  }
  else
  {
    qDebug("No such kolab object.");
  }
  
  d->lock = false;
}

void KolabInterface::slotDeleteEvent(Event * event)
{
  if( d->lock )
  {
    return;
  }
  
  if(!event)
  {
    return;
  }
  
  QString mailFolder = findMailFolder(false);
  if( !mailFolder.isEmpty() )
  {
    QString mailObject = findMailByUID(mailFolder, event->uid(), false);
    if( mailObject.isEmpty() )
    {
      qDebug("slotDeleteEvent : The mail object couldn't be found.");
    }
    else
    {
      d->vfs->remove(mailObject);
    }
  }
}

void KolabInterface::slotDeleteToDo(Todo * todo)
{
  if( d->lock )
  {
    return;
  }
  
  if(!todo)
  {
    return;
  }
  
  QString mailFolder = findMailFolder(true);
  if( !mailFolder.isEmpty() )
  {
    QString mailObject = findMailByUID(mailFolder, todo->uid(), true);
    if( mailObject.isEmpty() )
    {
      qDebug("slotDeleteEvent : The mail object couldn't be found.");
    }
    else
    {
      d->vfs->remove(mailObject);
    }
  }
}

void KolabInterface::slotUpdateEvent(Event * event)
{
  if( d->lock )
  {
    return;
  }
  
  if(!event)
  {
    return;
  }
  
  QString mailFolder = findMailFolder(false);
  if( !mailFolder.isEmpty() )
  {
    Account kolabAccount = d->clientRMI->getAccount(d->accountName);
    QString mailObject = findMailByUID(mailFolder, event->uid(), false);
    if( mailObject.isEmpty() )
    {
      qDebug("slotUpdateEvent : The mail object couldn't be found.");
    }
    else
    {
      d->vfs->remove(mailObject);
      if( d->mailAgent )
      {
        QString email;
        if( kolabAccount.name.isEmpty() )
          email = kolabAccount.from;
        else
          email = QString("\"") + kolabAccount.name + "\"" +
                          " <"  + kolabAccount.from + ">";
        
        ICalFormat ical;
        QString body = ical.createScheduleMessage(event, Scheduler::Request);
        
        d->mailAgent->appendToFolder(
          mailFolder,
          kolabAccount.accname,
          email,
          email,
          QString::null,
          QString::null,
          event->uid(),
          body,
          "text/calendar");
      }
    }
  }
}

void KolabInterface::slotUpdateToDo(Todo * todo)
{
  if( d->lock )
  {
    return;
  }
  
  if(!todo)
  {
    return;
  }
  
  QString mailFolder = findMailFolder(true);
  if( !mailFolder.isEmpty() )
  {
    Account kolabAccount = d->clientRMI->getAccount(d->accountName);
    QString mailObject = findMailByUID(mailFolder, todo->uid(), true);
    if( mailObject.isEmpty() )
    {
      qDebug("slotUpdateEvent : The mail object couldn't be found.");
    }
    else
    {
      d->vfs->remove(mailObject);
      if( d->mailAgent )
      {
        QString email;
        if( kolabAccount.name.isEmpty() )
          email = kolabAccount.from;
        else
          email = QString("\"") + kolabAccount.name + "\"" +
                          " <"  + kolabAccount.from + ">";
        
        ICalFormat ical;
        QString body = ical.createScheduleMessage(todo, Scheduler::Request);
        
        d->mailAgent->appendToFolder(
          mailFolder,
          kolabAccount.accname,
          email,
          email,
          QString::null,
          QString::null,
          todo->uid(),
          body,
          "text/calendar");
      }
    }
  }
}

void KolabInterface::slotAddEvent(Event * event)
{
  if( d->lock )
  {
    return;
  }
  
  if(!event)
  {
    return;
  }
  
  QString mailFolder = findMailFolder(false);
  
  if( !mailFolder.isEmpty() && d->mailAgent )
  {
    Account kolabAccount = d->clientRMI->getAccount(d->accountName);
    QString email;
    if( kolabAccount.name.isEmpty() )
      email = kolabAccount.from;
    else
      email = QString("\"") + kolabAccount.name + "\"" +
                      " <"  + kolabAccount.from + ">";
    
    ICalFormat ical;
    QString body = ical.createScheduleMessage(event, Scheduler::Request);
    
    d->mailAgent->appendToFolder(
      mailFolder,
      kolabAccount.accname,
      email,
      email,
      QString::null,
      QString::null,
      event->uid(),
      body,
      "text/calendar");
  }
}

void KolabInterface::slotAddToDo(Todo * todo)
{
  if( d->lock )
  {
    return;
  }
  
  if(!todo)
  {
    return;
  }
  
  QString mailFolder = findMailFolder(true);
  
  if( !mailFolder.isEmpty() && d->mailAgent )
  {
    Account kolabAccount = d->clientRMI->getAccount(d->accountName);
    QString email;
    if( kolabAccount.name.isEmpty() )
      email = kolabAccount.from;
    else
      email = QString("\"") + kolabAccount.name + "\"" +
                      " <"  + kolabAccount.from + ">";
    
    ICalFormat ical;
    QString body = ical.createScheduleMessage(todo, Scheduler::Request);
    
    d->mailAgent->appendToFolder(
      mailFolder,
      kolabAccount.accname,
      email,
      email,
      QString::null,
      QString::null,
      todo->uid(),
      body,
      "text/calendar");
  }
}

QString KolabInterface::findMailByUID
  ( const QString & folderPath,
    const QString & uid,
    bool bTodo ) const
{
  VFolder * folder = d->vfs->folderRef(folderPath);
  if(folder)
  {
    bool bFind = false;
    QStringList urls = d->vfs->children( folderPath );
    d->vfs->createVInstances(urls, false);
    
    MailObject * object = 0L;
    // try to find the UID in the subject line
    for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
    {
      object = static_cast<MailObject*> (d->vfs->_fact_ref((*it), VFS_MAILS_MIMETYPE));
      if(object)
      {
        if( -1 != object->subject().find(uid) )
        {
          bFind = true;
          break;
        }
      }
    }
    // hmm, try to find the UID in the mail body
    // this can take a while :(
    if(!bFind)
    {
      QStringList partsUrl;
      QString current;
      for(QStringList::Iterator it1=urls.begin(); it1!=urls.end(); ++it1)
      {
        current = (*it1) + ATTR_SEP + QString::number(0);
        partsUrl.append( current );
      }
      
      d->vfs->createVInstances(partsUrl, false);
      
      for(QStringList::Iterator it2=urls.begin(); it2!=urls.end() && !bFind; ++it2)
      {
        current = (*it2) + ATTR_SEP + QString::number(0);
    
        PartObjectInterface *part = static_cast<PartObjectInterface*>
          (d->vfs->_fact_ref( current, VFS_PART_MIMETYPE ));
          
        if( part )
        {
          if( "attachment" == part->partType() &&
              part->mimeType().contains("calendar") )
          {
            QByteArray data_ = part->data();
            if( !data_.size() )
            {
              qDebug("calendar data is null");
            }
            
            QCString partData( data_ );
            ICalFormat ical;
            CalendarLocal * kolabCalendar = 
              new CalendarLocal(KOPrefs::instance()->mTimeZoneId.local8Bit());
            QString ics;
            
            QTextCodec * codec = QTextCodec::codecForName( part->charset().latin1() );
            if(codec)
               ics = codec->toUnicode( partData );
            else
            {
              if( "8bit" == part->encoding().lower() )
                ics = QString::fromLocal8Bit( partData );
              else if( "utf-8" == part->encoding().lower() )
                ics = QString::fromUtf8( partData );
              else // 7 bit
                ics = QString::fromLatin1( partData );
            }
            
            ical.fromString(kolabCalendar, ics);
            
            if(bTodo)
            {
              if( kolabCalendar->todo(uid) )
              {
                object = static_cast<MailObject*>
                  (d->vfs->_fact_ref((*it2), VFS_MAILS_MIMETYPE));
                if(object)
                {
                  bFind = true;
                }
              }
            }
            else
            {
              if( kolabCalendar->event(uid) )
              {
                object = static_cast<MailObject*>
                  (d->vfs->_fact_ref((*it2), VFS_MAILS_MIMETYPE));
                if(object)
                {
                  bFind = true;
                }
              }
            }
            
            delete kolabCalendar;
            kolabCalendar = 0;
          }
        }
      }
    }
    
    if(bFind)
    {
      return object->path();
    }
    
    folder->freeObjects();
  }
  
  return QString::null;

}

QString KolabInterface::findMailFolder(VFolder * folder) const
{
  QString emailFolder;
  
  if( folder )
  {
    emailFolder = isKolabSharedFolder( folder->path() );
  }
  
  return emailFolder;
}

QString KolabInterface::findMailFolder(const QString & path, bool bToDo) const
{
  QString emailFolder;
  
  if( !path.isEmpty() )
  {
    CalendarObject * object =
      static_cast<CalendarObject*> (d->vfs->_fact_ref(path, VFS_CALENDAR_MIMETYPE));
    
    if(object)
    {
      emailFolder = isKolabCalendarObject(object->path(), bToDo);
      if( emailFolder.isEmpty() )
      {
        emailFolder = isKolabSharedFolder(object->parentFolder()->path());
      }
    }
  }
  
  return emailFolder;
}

QString KolabInterface::findMailFolder(bool bToDo) const
{
  QString emailFolder;
  KomKOUIService * uiService = KomKOUIService::instance();
  if(uiService)
  {
    QString objectPath = uiService->getCurrentObject();
    emailFolder = findMailFolder(objectPath, bToDo);
  }
  
  return emailFolder;
}
