/*
    This file is part of KOrganizer.
    Copyright (c) 1998 Barry D Benowitz
    Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    As a special exception, permission is given to link this program
    with any edition of Qt, and distribute the resulting executable,
    without including the source code for Qt in the source distribution.
*/

// #include <unistd.h>
#include <stdio.h>

#include <libkcal/event.h>
#include <libkcal/todo.h>

#include "version.h"
#include "koprefs.h"

#include "komailclient.h"

//KomMailPlugin includes
#include <mailagentservice.h>

KOMailClient::KOMailClient()
{
  qDebug( "KOMailClient::KOMailClient()" );
}

KOMailClient::~KOMailClient()
{
  qDebug( "KOMailClient::~KOMailClient()" );
}

bool KOMailClient::mailAttendees(IncidenceBase *incidence,const QString &attachment)
{
  qDebug( "KOMailClient::mailAttendees(IncidenceBase *incidence,const QString &attachment)" );  
  QPtrList<Attendee> attendees = incidence->attendees();
  if (attendees.count() == 0)
    return false;

  QString to;
  for(uint i=0; i<attendees.count();++i)
  {
    to += attendees.at(i)->email();
    if (i != attendees.count()-1) to += ", ";
  }

  QString from = KOPrefs::instance()->email();

  QString subject;
  if(incidence->type()!="FreeBusy")
  {
    Incidence *inc = static_cast<Incidence *>(incidence);
    subject = inc->summary();
  }
  else 
  {
    subject = "Free Busy Object";
  }

  QString body = createBody(incidence);

  bool bcc = KOPrefs::instance()->mBcc;

  return send(from,to,subject,body,bcc,attachment);
}

bool KOMailClient::mailOrganizer(IncidenceBase *incidence,const QString &attachment)
{
//  qDebug( "KOMailClient::mailAttendees()" );
  
  QString to = incidence->organizer();

  QString from = KOPrefs::instance()->email();

  QString subject;
  if(incidence->type()!="FreeBusy")
  {
    Incidence *inc = static_cast<Incidence *>(incidence);
    subject = inc->summary();
  } 
  else
    subject = "Free Busy Message";

  QString body = createBody(incidence);

  bool bcc = KOPrefs::instance()->mBcc;

  return send(from,to,subject,body,bcc,attachment);
  
}

bool KOMailClient::mailTo(IncidenceBase *incidence,const QString &recipients,
                          const QString &attachment)
{
//  qDebug( "KOMailClient::mailTo()" );
  QString from = KOPrefs::instance()->email();
  QString subject;
  
  if(incidence->type()!="FreeBusy")
  {
    Incidence *inc = static_cast<Incidence *>(incidence);
    subject = inc->summary();
  }
  else 
    subject = "Free Busy Message";
  
  QString body = createBody(incidence);
  qDebug( "KOMailClient::mailTo()--> body:\n%s", body.latin1() );  
  bool bcc = KOPrefs::instance()->mBcc;
  return send(from,recipients,subject,body,bcc,attachment);  
}

bool KOMailClient::send(const QString &from,const QString &to,
                        const QString &subject,const QString &body,bool bcc,
                        const QString &attachment)
{
//  qDebug( "KOMailClient::send()" );

  // Something is strange: body and attachment shouldn't be sent together
  // Hmm...
  
  MailAgentService* mailAgent = 0L;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( MAILAGENT_SERVICE );
    if( ref.isValid() )
       mailAgent = static_cast<MailAgentService*>( ref.instance() );
  }
  
  if(mailAgent)
  {
    mailAgent->send( QString::null, // use default account
                     from,
                     to,
                     QString::null, // CC
                     QString::null, // BCC
                     subject,
                     body + "\n" + attachment, // BODY
                     "text/calendar"); // BODY MIME
    return true;
  }
  
  return false;
}

int KOMailClient::kMailOpenComposer
  ( const QString& /*arg0*/, const QString& /*arg1*/,
    const QString& /*arg2*/, const QString& /*arg3*/, const QString& /*arg4*/,int /*arg5*/,
   const KURL& /*arg6*/
  )
{
  qDebug( "KOMailClient::kMailOpenComposer()" );
  
  return 0;
}

int KOMailClient::kMailOpenComposer
  ( const QString& /*arg0*/, const QString& /*arg1*/,
    const QString& /*arg2*/, const QString& /*arg3*/,
    const QString& /*arg4*/, int /*arg5*/, const QString& /*arg6*/,
    const QCString& /*arg7*/, const QCString& /*arg8*/,
    const QCString& /*arg9*/, const QCString& /*arg10*/,
    const QCString& /*arg11*/, const QString& /*arg12*/,
    const QCString& /*arg13*/
  )
{
  qDebug( "KOMailClient::kMailOpenComposer()" );
  
  return 0;
}


QString KOMailClient::createBody(IncidenceBase *incidence)
{
  qDebug( "KOMailClient::createBody()" );
  QString CR = ("\n");

  QString body;

  // mailbody for Event
  if (incidence->type()=="Event")
  {
    Event *selectedEvent = static_cast<Event *>(incidence);
    QString recurrence[]= {QObject::tr("no recurrence", "None"),QObject::tr("Daily"),QObject::tr("Weekly"),QObject::tr("Monthly Same Day"),
                           QObject::tr("Monthly Same Position"),QObject::tr("Yearly"),QObject::tr("Yearly")};
  
    if (selectedEvent->organizer() != "")
    {
      body += QObject::tr("Organizer: %1").arg(selectedEvent->organizer());
      body += CR;
    }
    body += QObject::tr("Summary: %1").arg(selectedEvent->summary());
    if (!selectedEvent->location().isEmpty())
    {
      body += CR;
      body += QObject::tr("Location: %1").arg(selectedEvent->location());
    }
    if (!selectedEvent->doesFloat())
    {
      body += CR;
      body += QObject::tr("Start Date: %1").arg(selectedEvent->dtStartDateStr());
      body += CR;
      body += QObject::tr("Start Time: %1").arg(selectedEvent->dtStartTimeStr());
      body += CR;
      if (selectedEvent->recurrence()->doesRecur())
      {
        body += QObject::tr("Recurs: %1")
                 .arg(recurrence[selectedEvent->recurrence()->frequency()]);
        body += CR;
        if (selectedEvent->recurrence()->duration() > 0 )
        {
          body += QObject::tr("Repeats %1 times")
                   .arg(QString::number(selectedEvent->recurrence()->duration()));
          body += CR;
        } 
        else 
        {
          if (selectedEvent->recurrence()->duration() != -1)
          {
            body += QObject::tr("End Date: %1")
                     .arg(selectedEvent->recurrence()->endDateStr());
            body += CR;
          }
          else
          {
            body += QObject::tr("Repeats forever");
            body += CR;
          }
        }
      }
      body += QObject::tr("End Time: %1").arg(selectedEvent->dtEndTimeStr());
      body += CR;
      QString details = selectedEvent->description();
      if (!details.isEmpty())
      {
        body += QObject::tr("Details:");
        body += CR;
        body += details;
      }
    }
  } 

  // mailbody for Todo
  if (incidence->type()=="Todo")
  {
    Todo *selectedEvent = static_cast<Todo *>(incidence);
    if (selectedEvent->organizer() != "")
    {
      body += QObject::tr("Organizer: %1").arg( selectedEvent->organizer() );
      body += CR;
    }
    body += QObject::tr("Summary: %1").arg(selectedEvent->summary());
    if (!selectedEvent->location().isEmpty())
    {
      body += CR;
      body += QObject::tr("Location: %1").arg(selectedEvent->location());
    }
    if (!selectedEvent->hasStartDate())
    {
      body += CR;
      body += QObject::tr("Start Date: %1").arg(selectedEvent->dtStartDateStr());
      body += CR;
      if (!selectedEvent->doesFloat())
      {
        body += QObject::tr("Start Time: %1").arg(selectedEvent->dtStartTimeStr());
        body += CR;
      }
    }
    if (!selectedEvent->hasDueDate())
    {
      body += CR;
      body += QObject::tr("Due Date: %1").arg(selectedEvent->dtDueDateStr());
      body += CR;
      if (!selectedEvent->doesFloat())
      {
        body += QObject::tr("Due Time: %1").arg(selectedEvent->dtDueTimeStr());
        body += CR;
      }
    }
    body += CR;
    QString details = selectedEvent->description();
    if (!details.isEmpty())
    {
      body += QObject::tr("Details:");
      body += CR;
      body += details;
    }
  } 

  // mailbody for FreeBusy
  if(incidence->type()=="FreeBusy")
    body = QObject::tr("This is a Free Busy Object");

  // mailbody for Journal
  if(incidence->type()=="Journal")
  {
    Incidence *inc = static_cast<Incidence *>(incidence);
    body = inc->summary();
    body += CR;
    body += inc->description();
  }

  return body;
}
