/***************************************************************************
    begin                : Fri Jan 18 21:42:04 EET 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen Constantinescu <eug@thekompany.com>
    author               : Iulian Ghiciuc
    author               : Bodi Vlad
 ***************************************************************************/


//own includes
#include "komeventalarmservice.h"
#include "popupwidget.h"

//tkcBase include
#include <qsettingsextra.h>
#include <settings.h>
#include <playsound.h>

//QT includes
#include <qptrlist.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdatetime.h>
#include <qsound.h>
#include <qmessagebox.h>
#include <qlistbox.h>
#include <qtextbrowser.h>

//Aethera includes
#include <vfolder.h>

//tkcBase includes
#include <dirs.h>

//KO includes
#include <alarm.h>
#include <koprefs.h>

static const char* configPath = "/Aethera/DateBook/Alarm/" ;

/*
---------------------------------------------------------------------------------------------------------------------
The KomEventAlarmService factory
---------------------------------------------------------------------------------------------------------------------
*/
kore::Interface* komeventalarmservice_factory()
{
  static KomEventAlarmService* instance = 0L;
  if( !instance )
  {
    instance = new KomEventAlarmService();
  }
  return instance;
}


/*
---------------------------------------------------------------------------------------------------------------------
The KomEventAlarmService class
---------------------------------------------------------------------------------------------------------------------
*/
KomEventAlarmService::KomEventAlarmService()
  : qtHandler( 0L ),
    vfs( 0L ),
    status( 0L ),
    timer( 0L ),
    interval( 60000 )
{
  MetaInfo *info = metaInfo();
  info->replaceAll("name", "Alarm Demon");
  info->replaceAll("class", "KomEventAlarmService");
  info->replaceAll("class/fqn", "KomEventAlarmService");
  info->replaceAll("factory", "komeventalarmservice_factory");

  //obtain status bar for event falshing
  status = AppStatus::instance();

  timer = new QTimer(0L, "timer");

  //obtain vfs
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>( ref.instance() );
  }
  assert(0 != vfs);

  qtHandler = new KomEventAlarmHandler( this );
  assert( 0 != qtHandler );

  QObject::connect( timer, SIGNAL( timeout() ),
                    qtHandler, SLOT( slotCheckEvents() )
                   );

}


KomEventAlarmService::~KomEventAlarmService()
{
  if(qtHandler)
  {
    delete qtHandler;
    qtHandler=0L;
  }
}


void KomEventAlarmService::start()
{
  timer->start( interval );
}


void KomEventAlarmService::stop()
{
  timer->stop();
}


void KomEventAlarmService::slotUseAlarm(bool use)
{
  if( !use )
  {
    Settings::instance()->writeEntry( QString(configPath)+"Enable", false );
    stop();
  }
  else
  {
    Settings::instance()->writeEntry( QString(configPath)+"Enable", true );
    start();
  }
}


void KomEventAlarmService::slotCheckEvents()
{
  eventList.clear();
  checkCalendars( vfs->root() + "/Datebook" );
  if( !eventList.isEmpty() )
  {
    if( Settings::instance()->readBoolEntry( QString(configPath)+"Enable", false ) )
    {
      if( Settings::instance()->readBoolEntry( QString(configPath)+"Sound", false) )
        playFile();
      if( Settings::instance()->readBoolEntry( QString(configPath)+"Status", false) )
        statusBar();
      if( Settings::instance()->readBoolEntry( QString(configPath)+"PopUp", false) )
        popupWindow();
    }
  }
}


void KomEventAlarmService::checkCalendars( const QString& path )
{
  bool is=true;
  if( is ) //only if we don't have events continue to check
  {
    VFolder * folder = vfs->folderRef( path );
    if(folder)
    {
      //check the calendars in the current folder
      QStringList urls = vfs->children( folder->path() );
      QStringList temp;
      for(QStringList::Iterator it=urls.begin(); it!=urls.end(); ++it)
      {
        temp.push_back(*it);
      }
  
      vfs->createVInstances(temp, false);
  
      // UTC time for checking alarms in UTC time
      QDateTime checkTime = QDateTime::currentDateTime(Qt::LocalTime);
/*      qDebug("TimeZone = %s", KOPrefs::instance()->mTimeZoneId.latin1());*/
      if( "UTC" == KOPrefs::instance()->mTimeZoneId )
      {
        checkTime = QDateTime::currentDateTime(Qt::UTC);
      }
      
      for(QStringList::Iterator it1=temp.begin(); it1!=temp.end(); ++it1)
      {
        CalendarObject * calObject = static_cast<CalendarObject*>
          ( vfs->_fact_ref( path + '/' + (*it1), VFS_CALENDAR_MIMETYPE) );
        if( !calObject )
        {
          qDebug("Bad CalendarObject reference");
        }
        else
        {
          KCal::CalendarLocal* calendar = 
            new KCal::CalendarLocal( KOPrefs::instance()->mTimeZoneId.local8Bit() );
          KCal::ICalFormat ical;
          ical.fromString( calendar, calObject->calendarInfo() );
    
          // all events
          QPtrList<KCal::Event> events = calendar->events();
          
          for( QPtrListIterator<KCal::Event> it2( events ); it2.current(); ++it2 )
          {
            KCal::Event* event = it2.current();
            QPtrList<KCal::Alarm> alarms = event->alarms();
            for( QPtrListIterator<KCal::Alarm> it3( alarms ); it3.current(); ++it3 )
            {
              KCal::Alarm *alarm  = it3.current();
              if( alarm )
              {
                if( alarm->enabled() )
                {
/*                  qDebug("EventAlarm %s = %s", event->description().latin1(),
                                               alarm->time().toString().latin1());
                  qDebug("CheckTime = %s", checkTime.toString().latin1());*/
                  
                  if( alarm->time().secsTo( checkTime ) <= interval/1000 + 4 )
                    eventList.append(*event);
                }
              }
            }
          }
          
          delete calendar;
          calendar = 0;
        }
      }
  
      temp.clear();
      folder->freeObjects();
  
      //in recursive manner check the calendars in all the subfolders
      VFolderList& subFolders = folder->folderList();
      for(VFolderListIterator it4(subFolders); it4.current(); ++it4)
      {
        VFolder* subFolder = it4.current();
        checkCalendars( subFolder->path() );
      }
    }
  }
}

bool KomEventAlarmService::isOn()
{
  if( !eventList.isEmpty() )
    return true;
  return false;
}


void KomEventAlarmService::playFile()
{
  QString soundPath;
  soundPath = Settings::instance()->readEntry( QString(configPath)+"soundPath", soundPath );
  
  if( soundPath.isEmpty() )
  {
    PlaySound::play("alarm.wav");
  }
  else
  {
    PlaySound::play(soundPath);
  }
}

void KomEventAlarmService::statusBar()
{
  status->message(AppStatus::Client, AppStatus::Text, AppStatus::Warning,QString("Events in calendar"));
}

void KomEventAlarmService::popupWindow()
{
  PopUpWidget * box = new PopUpWidget( eventList );
  box->show();
}

