/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
    copyright            : (C) 2003 by theKompany.com
    author               : Eugen Constantinescu <eug@thekompany.com>
  Date     : 27 August 2003
  Author   : Ghiciuc Ilie (ilie@eco-spaces.com.ro)
---------------------------------------------------------------------------------
*/

//own includes
#include "komkoconfigservice.h"
#include "koprefsdialog.h"

//tkcBase includes
#include <qiconloader.h>

//QT includes
#include <qtabwidget.h>
          
/*
----------------------------------------------------------------------------------------
The service factory
----------------------------------------------------------------------------------------
*/

kore::Interface* komkoconfigservice_factory()
{
  return new KomKOConfigService();
}

/*
----------------------------------------------------------------------------------------
KomKOConfigService::Private class
----------------------------------------------------------------------------------------
*/

class KomKOConfigService::Private
{
  public:
  
    Private(KomKOConfigService * p);
    virtual ~Private();
    
    QWidget * widget;
    KomKOConfigService * parent;
    KOPrefsDialog* mOptionsDialog;
};

KomKOConfigService::Private::Private(KomKOConfigService * p)
  : widget(0L),
    parent( p ),
    mOptionsDialog(0L)
{
  mOptionsDialog = new KOPrefsDialog();
  if( mOptionsDialog )
  {
    mOptionsDialog->readConfig();
    mOptionsDialog->hide();
  }
  
  widget = mOptionsDialog->tabWidget();
}

KomKOConfigService::Private::~Private()
{
  if( mOptionsDialog )
  {
    delete mOptionsDialog;
    mOptionsDialog = 0L;
  }
}

/*
----------------------------------------------------------------------------------------
 KomKOConfigService class
----------------------------------------------------------------------------------------
*/
    
KomKOConfigService::KomKOConfigService()
  : KomConfigService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "DateBook Config Service");
  info->replaceAll("class", "KomKOConfigService");
  info->replaceAll("class/fqn", "KomKOConfigService");
  info->replaceAll("factory", "komkoconfigservice_factory");
  
  d = new Private(this);
}

KomKOConfigService::~KomKOConfigService()
{
  if( d )
  {
    delete d;
    d = 0L;
  }
}

QWidget* KomKOConfigService::init(QWidget * /*parent*/)
{
  return d->widget;
}

void KomKOConfigService::load()
{
  d->mOptionsDialog->readConfig();
}

void KomKOConfigService::save()
{
  d->mOptionsDialog->writeConfig();
}

void KomKOConfigService::defaults()
{
  d->mOptionsDialog->setDefaults();
}

QPixmap KomKOConfigService::icon()
{
  return QIconLoader::instance()->loadIcon( "DateBook_General.png" );
}

QString KomKOConfigService::caption()
{
  return QObject::tr("DateBook/General");
}

bool KomKOConfigService::modified()
{
  return false;
}
