/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
    copyright            : (C) 2003 by theKompany.com
    author               : Eugen Constantinescu <eug@thekompany.com>
  Date     : 15 August 2003
  Author   : Ghiciuc Ilie (ilie@eco-spaces.com.ro)
---------------------------------------------------------------------------------
*/

#include <vfsmodule.h>
#include <clientrmi.h>

//own includes
#include "kolabinterface.h"
#include "komkormiservice.h"
#include "kodialogmanager.h"
#include "komkouiservice.h"
#include "calendarmainview.h"
#include "incomingdialog.h"
#include "calendarobjectview.h"
#include <koprefs.h>

kore::Interface* komkormiservice_factory()
{
  return new KomKORMIService();
}

KomKORMIService::KomKORMIService()
  : KomRMIService()
{
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", "Calendar RMI Service");
  info->replaceAll("class", "KomKORMIService");
  info->replaceAll("class/fqn", "KomKORMIService");
  info->replaceAll("description", KO_RMI_SERVICE_DESC );
  info->replaceAll("factory", "komkormiservice_factory");
  info->insert("service",  AETHERA_RMI_SERVICE );
  info->insert("service",  KO_RMI_SERVICE );

}

KomKORMIService::~KomKORMIService()
{
}


QDomDocument KomKORMIService::callMethod( const QDomDocument& callDoc ) 
{
  QDomDocument returnDoc;
  QDomElement returnElem = returnDoc.createElement( "return" );

  //return if the document is null  
  if( callDoc.documentElement().firstChild().toElement().isNull() )
    return returnDoc;
    
  QDomElement callElem =  callDoc.namedItem("call").toElement();
  
  //get the name of the method
  QDomElement nameElem = callElem.namedItem("name").toElement();
  QString callNameStr = nameElem.text();
  
  if( callNameStr == "retrieve" )
  {
    //get the number of parameters and the parameters
    QDomElement parametersElem = callElem.namedItem("parameters").toElement();
    int paramCount = parametersElem.attribute("count", "0").toInt();
    if( !paramCount )
    {
      //get the return type of the method call
      QDomElement retElem = callElem.namedItem("return").toElement();
      QString retTypeStr = retElem.text();
      if( retTypeStr == "void" )
      {
        method_retrieve();
      }
    }
  }
  else if( callNameStr == "kolab" )
  {
    //get the number of parameters and the parameters
    QDomElement parametersElem = callElem.namedItem("parameters").toElement();
    int paramCount = parametersElem.attribute("count", "0").toInt();
    QString signature = parametersElem.attribute("signature");
    if( paramCount == 3 && 
        signature == "QString,QString,int" )
    {
      QString accName = parametersElem.attribute("param1");
      QString mailbox = parametersElem.attribute("param2");
      int iToDo = parametersElem.attribute("param3", "0").toInt();
      
      QDomElement retElem = callElem.namedItem("return").toElement();
      QString retTypeStr = retElem.text();
      if( retTypeStr == "void" )
      {
        method_kolab(accName, mailbox, iToDo);
      }
    }
  }
  
  return returnDoc;
}

void KomKORMIService::method_retrieve()
{
  KomKOUIService * uiService = KomKOUIService::instance();
  
  if( uiService )
  {
    CalendarMainView * mainView = uiService->getCalendarMainView();
    if( mainView )
    {
      CalendarObjectView * objectView = mainView->getCalendarObjectView();
      if( objectView )
      {
        KODialogManager * dlgManager = objectView->dialogManager();
        if( dlgManager )
        {
          IncomingDialog * icd = dlgManager->incomingDialog();
          
          if(icd)
          {
            icd->retrieve();
          }
          else
          {
            qDebug("KomKORMIService: no incoming dialog.");
          }
        }
        else
        {
          qDebug("KomKORMIService: no dialog manager.");
        }
      }
      else
      {
        qDebug("KomKORMIService: no object view.");
      }
    }
    else
    {
      qDebug("KomKORMIService: no main view.");
    }
  }
  else
  {
    qDebug("KomKORMIService: no UI service.");
  }
}

void KomKORMIService::method_kolab( const QString & accName,
                                    const QString & mailbox,
                                    bool bToDo )
{
  KolabInterface * kolab = KolabInterface::instance();
  if( kolab )
  {
    kolab->slotSynch(accName, mailbox, bToDo);
  }
}
