
#include "komkotreeinterface.h"
#include "komkotreeslots.h"
#include "komkotreeitem.h"
#include <kolabinterface.h>

#include <qpopupmenu.h>

#include <aethera/vfolder.h>
#include <aethera/vfsmimetype.h>

/*
  Modifies by Ghiciuc Ilie (ilie@eco-spaces.com.ro) 10 July 2003 to add Move/Rename/Expunge options to the
  folder tree view
*/

/* Modifies by Fabien Armand 03 jun 2004 to add the search
   for shared folders
*/

KomKOTreeInterface::KomKOTreeInterface(QListView * parent)
  : KomMainTreeInterface  (parent),
    m_popupMenu           (0L),
    m_slots               (0L),
    m_currentItem         (0L),
    m_root                (0L)
{
  KomItemType *itemType =
    new KomItemType
    (
      ItemTypeRoot,
      new QString( VFS_CALENDAR_ROOT ),
      new QPixmap(BarIcon("DateBookContainer")),
      true,
      true
    );
  itemTypeList().insert("root", itemType);

  itemType = 
    new KomItemType
    (
      ItemTypeFolder,
      new QString("Folder"),
      new QPixmap(BarIcon("DateBookFolder")),
      true,
      true
    );
  itemTypeList().insert( VFS_CALENDAR_MIMETYPE , itemType);

  m_popupMenu = new QPopupMenu(parentTree(), "KomKOPopup");

  m_slots = new KomKOTreeSlots(this, "KomKOTreeSlots");
}

KomKOTreeInterface::~KomKOTreeInterface()
{
  delete m_slots;
}

  void
KomKOTreeInterface::show()
{
//  qDebug("KomKOTreeInterface::show");
  
  m_root = 
    new KomKOTreeItem(parentTree(), *const_cast<KomItemType *>(itemTypeList().getItemOfType("root")));
  m_root->setOpen(true);
  createChildren(m_root);
  parentTree()->setCurrentItem(m_root);
  parentTree()->setSelected(m_root, true);
}

  void
KomKOTreeInterface::show(const QString & focus)
{
  m_root =
    new KomKOTreeItem(parentTree(), *const_cast<KomItemType *>(itemTypeList().getItemOfType("root")));
  KomMainTreeInterface::show(focus);
}

  void
KomKOTreeInterface::hide()
{
//  qDebug("KomKOTreeInterface::hide");
  
  delete m_root;
}

QString KomKOTreeInterface::path(KomTreeItem *item)
{
  QString vfsPath = static_cast<KomKOTreeItem*>(item)->path();
  
  if( vfsPath.isEmpty() )
    return KomMainTreeInterface::path(item);
  else
    return vfsPath;
}
    
  KomTreeItem *
KomKOTreeInterface::root()
{
  return m_root;
}

  void
KomKOTreeInterface::createChildren(KomTreeItem * root)
{
  if (0 == root)
    return;

  QString rootPath = path(root);

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qWarning("No VFS module ?");
    return;
  }

  VFolder * parentFolder = vfs->folderRef(rootPath);
  if( 0 == parentFolder ) // oops
    return;
  VFolderList & folderList = parentFolder->folderList();  

  for (VFolderListIterator it(folderList); it.current(); ++it)
  {
    QString childName((it.current()->path().mid(rootPath.length() + 1)));

    // See if it exists already.

    KomTreeItem * folderItem = findChild(root, childName);

    if (0 == folderItem)
    {
      // No ? Then make it.
      folderItem = createFolderItem
                    (
                      root,
                      it.current()->path().mid(rootPath.length() + 1),
                      it.current()->viewType()
                    );
    }

    // Recurse.
    createChildren(folderItem);
  }
}

  QPopupMenu *
KomKOTreeInterface::contextMenu(KomTreeItem * item)
{
  assert(0 != m_popupMenu);

  m_popupMenu->clear();

  bool kolab = false;
  bool sharedKolab = false;
  QString emailPath;
  QStringList kolabList = 
    Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  if( !kolabList.isEmpty() )
  {
    QString folderPath = path(item);
    QString kolabMainFolder = KolabInterface::instance()->kolabPath();
    
    if( (folderPath + "/") == kolabMainFolder )
    {
      kolab = true;
    }
    else
    {
      emailPath = KolabInterface::instance()->isParentKolabSharedFolder(folderPath);
      if( !emailPath.isEmpty() )
      {
        sharedKolab = true;
      }
    }
  }
  
  switch (item->getItemType())
  {
    case ItemTypeRoot:
      m_popupMenu->insertItem
        ( "New calendar", m_slots, SLOT(slotNewObject()) );
      m_popupMenu->insertItem
        ( "New folder", m_slots, SLOT(slotNewFolder()) );
      m_popupMenu->insertItem
        ( "Expunge folder", m_slots, SLOT(slotExpungeFolder()) );

      break;

    case ItemTypeFolder:
    default:
      if( kolab )
      {
        m_popupMenu->insertItem
          ( QObject::tr("Search for shared folders"),m_slots,SLOT(slotSearchForSharedFolders()) );     
      }
      else if( sharedKolab )
      {
        m_popupMenu->insertItem
          ( "Delete folder", m_slots, SLOT(slotDeleteFolder()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Search for shared folders"),m_slots,SLOT(slotSearchForSharedFolders()) );     
      }
      else
      {
        m_popupMenu->insertItem
          ( "New calendar", m_slots, SLOT(slotNewObject()) );
        
        m_popupMenu->insertItem
          ( "New folder", m_slots, SLOT(slotNewFolder()) );
  
        m_popupMenu->insertItem
          ( "Delete folder", m_slots, SLOT(slotDeleteFolder()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Move folder"), m_slots, SLOT(slotMoveFolder()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Rename folder"), m_slots, SLOT(slotRenameFolder()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Expunge folder"), m_slots, SLOT(slotExpungeFolder()) );
  
        m_popupMenu->insertItem
          ( QObject::tr("Search for shared folders"),m_slots,SLOT(slotSearchForSharedFolders()) );     
      }
      break;
  }

  return m_popupMenu;
}

  bool
KomKOTreeInterface::hasChild(KomTreeItem * item)
{
  for (QListViewItemIterator it(parentTree()); it.current(); ++it)
  {
    if (it.current() == item)
      return true;
  }

  return false;
}

  void 
KomKOTreeInterface::updateVInstances(const QStringList & _l)
{

//  qDebug("KomKOTreeInterface::updateVInstances");

  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qWarning("No VFS module ?");
    return;
  }

  QStringList l(_l);
  l.sort();
  QString parentPath;

  for (QStringList::ConstIterator it(l.begin()); it != l.end(); ++it)
  {
    // update only for folders
    VFolder * folder = vfs->folderRef(*it);
    // try the parent folder
    if (0 == folder)
    {
      folder = vfs->folderRef((*it).left( (*it).findRev("/") ));
    }
    // wrong url, skip it
    if (0 == folder)
      continue;

    QString folderPath = folder->path();
      
    // avoid to repeat the update
    if( parentPath != folderPath )
    {
      KomKOTreeItem * parentItem = static_cast<KomKOTreeItem *>
      ( folderItem(folderPath, false /* Do not create */) );
        
      if( parentItem )
      {
        parentItem->setText(0, folder->name());
        parentItem->forceUpdate();
        parentItem->repaint();
      }
      parentPath = folderPath;
    }
  }
}

  void
KomKOTreeInterface::createVInstances(const QStringList &_l)
{
  // only objects are created with this signal
  // update the folders
  updateVInstances(_l);
}

  void 
KomKOTreeInterface::deleteVInstances(const QStringList &_l)
{
  // update the folders
  updateVInstances(_l);
}

  void 
KomKOTreeInterface::updateHierarchy(const QString & s)
{
  // XXX What are we supposed to do here ?
  // You should rebuild the tree with the root 's'
  qDebug("KomKOTreeInterface::updateHierarchy(%s): STUB", s.ascii());
}

  QObject *
KomKOTreeInterface::qObject()
{
  return m_slots;
}

  KomTreeItem *
KomKOTreeInterface::folderItem(const QString & url, bool create)
{
  QString vfsRoot = QString("/Aethera/VFS/") + VFS_CALENDAR_ROOT;
  if ( vfsRoot != url.left( vfsRoot.length() ))
  {
//    qDebug("Hmm, wrong path `%s'", url.ascii());
    return 0;
  }

//  qDebug("folderItem %s", url.latin1());
  
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      qWarning("\n NO VFS Provider !\n");
      return 0;
    }
  }
  VFolder *folder = vfs->folderRef(url);
  
  QStringList pathList(QStringList::split('/', url.mid( vfsRoot.length() )));

  KomTreeItem * root = m_root;
  KomTreeItem * child = 0;

  QStringList::ConstIterator it(pathList.begin());

  for (it = pathList.begin(); it != pathList.end(); ++it)
  {
    child = findChild(root, *it);

    if (0 == child)
    {
      if (create && folder)
      {
        return createFolderItem(root, *it, folder->viewType());
      }
      else
      {
        return 0;
      }
    }
    else
      root = child;
  }

  return child;
}

  KomTreeItem *
KomKOTreeInterface::findChild(KomTreeItem * root, const QString & name)
{
  KomTreeItem * treeItem = static_cast<KomTreeItem *>(root->firstChild());

  while (0 != treeItem)
  {
    if (treeItem->text(0) == name)
      return treeItem;

    treeItem = static_cast<KomTreeItem *>(treeItem->nextSibling());
  }

  return 0;
}

  void
KomKOTreeInterface::create(const QString & url)
{
  if ("folder" == url.lower())
  {
    folderItem(url, true);
  }
  else
  {
    qDebug("Don't yet know how to create %s", url.ascii());
  }
}

  void
KomKOTreeInterface::createVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & /*type*/,
  const QString & /*subType*/,
  const QString & /*serverPath*/
)
{
//  qDebug("KomKOTreeInterface::createVFolder = %s, %s",
//    parentFolderPath.latin1(),
//    folderName.latin1() );
  
  folderItem(parentFolderPath+"/"+folderName, true);
}

  void
KomKOTreeInterface::deleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & /*type*/
)
{
//  qDebug(
//    "KomKOTreeInterface::deleteVFolder(`%s', `%s', `%s')",
//    parentFolderPath.latin1(),
//    folderName.latin1(),
//    type.latin1()
//  );
//  
  // the VFolder should be already deleted, so only the folder item
  // has to be deleted
  KomTreeItem *deletedFolder = folderItem(parentFolderPath+"/"+folderName, false);
  if( deletedFolder )
    delete deletedFolder;
}

  KomTreeItem *
KomKOTreeInterface::createFolderItem
(
  KomTreeItem * parentItem,
  const QString & folderName,
  const QString & type
)
{
  KomItemType komItemType = *const_cast<KomItemType *>(itemTypeList().getItemOfType(type));
  komItemType.setItemText(new QString(folderName)); // new ? Grr.

  return static_cast<KomTreeItem *> (new KomKOTreeItem(parentItem, komItemType));
}

