/***************************************************************************
         komkotreeslots.cpp  - 
                             -------------------
    begin                : Fri Jan 18 21:44:25 EET 2002
    copyright            : (C) 2002 by theKompany.com
                           (C) 2004 by Atol C&D
    author               : Eugen C. <eug@thekompany.com>
                           Fabien Armand <far@atolcd.com>
 ***************************************************************************/

/*
  Modifies by Ghiciuc Ilie(ilie@eco-spaces.com.ro) 10 July 2003 to add Move/Rename/Expunge options to the
  folder tree view
*/

#include "komkotreeslots.h"
#include "komkotreeinterface.h"
#include <kolabinterface.h>

#include <komkobackendservice.h>
#include <komkouiservice.h>
#include <icalformat.h>
#include <calendarlocal.h>
#include <koprefs.h>

#include <qinputdialog.h>

#include <clientvfs.h>
#include <vfolder.h>
#include <vfsmimetype.h>

#include <qmessagebox.h>

#include <folderchooserdialog.h>

#include <aethera/kolabsharedfolder.h>

using namespace KCal;

KomKOTreeSlots::KomKOTreeSlots( KomKOTreeInterface * parent,
                                          const char * name )
  : QObject(0, name),
    m_mainTreeInterface (parent)
{
}

KomKOTreeSlots::~KomKOTreeSlots()
{
}

void KomKOTreeSlots::slotNewObject()
{
  QString calendarName;
  QString objectURL;
  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if( path.isEmpty() )
  {
    qDebug("KomKOTreeSlots::slotNewObject(): no current item");
    return;
  }
  
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qDebug("No VFS module ?");
    return;
  }

  QString name = vfs->createObject(path);
  if( !name.isEmpty() )
  {
    objectURL = path + "/" + name;
        
    calendarName = QInputDialog::getText("New Calendar", "Choose a name for the new calendar.");
  }
  else
  {
    qDebug("CalendarObjectView::slotNewObject: couldn't create a new object.");
    return;
  }
    
  CalendarObject * object = static_cast<CalendarObject*>(vfs->_fact_ref(objectURL, VFS_CALENDAR_MIMETYPE));
  if( object )
  {
    if( object->calendarName().isEmpty() || 
        "unknown" == object->calendarName() )
    {
      if( calendarName.isEmpty() )
        object->setCalendarName("unknown");
      else
        object->setCalendarName( calendarName );
    }
      
    ICalFormat ical;
    CalendarLocal * emptyCalendar = new CalendarLocal(KOPrefs::instance()->mTimeZoneId.local8Bit());
    emptyCalendar->setOwner(KOPrefs::instance()->fullName());
    emptyCalendar->setEmail(KOPrefs::instance()->email());
    
    object->setCalendarInfo( ical.toString( emptyCalendar ) );
    object->save();
    
    delete emptyCalendar;
  }
  else
  {
    qDebug("CalendarObjectView::slotNewObject: couldn't save the object.");
  }
}

void KomKOTreeSlots::slotDeleteObject()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qDebug("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if( path.isEmpty() )
  {
    qDebug("KomKOTreeSlots::slotDeleteObject(): no current item");
    return;
  }

  bool ok = vfs->remove(path);

  if (!ok)
  {
    qDebug("KomKOTreeSlots::slotDeleteObject(): couldn't delete mail");
  }
}

void KomKOTreeSlots::slotNewFolder()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
  }
  if (0 == vfs)
  {
    qDebug("No VFS module ?");
    return;
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if( path.isEmpty() )
  {
    qDebug("KomKOTreeSlots::slotNewFolder(): no current item");
    return;
  }
 
  VFolder * parentRef = vfs->folderRef(path);
  if(0 == parentRef)
  {
    qDebug("Path: `%s'", path.ascii());
    qDebug("Parent does not exist");
    return;
  }

  bool ok = false;

  QString name =
    QInputDialog::getText
    (
      tr(AppName),
      tr("Please enter a name for the new folder"),
      QLineEdit::Normal,
      QString::null,
      &ok,
      0 // FIXME: Parent widget ?
    );

  if (ok && !name.isEmpty())
  {
    VFolder * folderRef = vfs->folderRef(path + "/" + name);

    if (0 != folderRef)
    {
      qDebug("Folder already exists");
    }
    else
    {
      qDebug("Folder does not exist. Making");

      VFolder * f = vfs->createFolder(parentRef, name, VFS_CALENDAR_MIMETYPE, VFS_CALENDAR_MIMETYPE);

      if (0 == f)
      {
        qDebug("Couldn't create folder");
        return;
      }

      qDebug("Folder made successfully");
    }

    m_mainTreeInterface->createChildren(m_mainTreeInterface->currentItem());
  }
}

void KomKOTreeSlots::slotDeleteFolder()
{
  VFSModule * vfs = 0;
  ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
  if( trader )
  {
    ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
    if( ref.isValid() )
      vfs = static_cast<VFSModule*>(ref.instance());
    else
    {
      qDebug("\n NO VFS Provider !\n");
      return;
    }
  }

  QString path = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());

  if( path.isEmpty() )
  {
    qDebug("slotDeleteFolder(): no current item");
    return;
  }
 
  // if it's a Kolab shared folder
  if( !path.find(KolabInterface::instance()->kolabPath()) )
  {
    KolabInterface::instance()->deleteKolabSharedFolder(path);
  }
  
  // remove the item from the list
  delete m_mainTreeInterface->currentItem();
  
  bool ok = vfs->deleteFolder(path);

  if (!ok)
  {
    qDebug("slotDeleteFolder: path == `%s'", path.ascii());
    qDebug("slotDeleteFolder: unsuccessful");
  }
}

  void
KomKOTreeSlots::slotUpdateVInstances(const QStringList & l)
{
  m_mainTreeInterface->updateVInstances(l);
}

  void 
KomKOTreeSlots::slotCreateVInstances(const QStringList & l)
{
  m_mainTreeInterface->createVInstances(l);
} 

  void 
KomKOTreeSlots::slotDeleteVInstances(const QStringList & l)
{
  m_mainTreeInterface->deleteVInstances(l);
}

  void 
KomKOTreeSlots::slotUpdateHierarchy(const QString & s)
{
  m_mainTreeInterface->updateHierarchy(s);
}

  void
KomKOTreeSlots::slotCreate(const QString & url)
{
//  qDebug("KomKOTreeSlots::slotCreate(%s)", url.ascii());
  m_mainTreeInterface->create(url);
}

  void
KomKOTreeSlots::slotCreateVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type,
  const QString & subType,
  const QString & serverPath
)
{
  m_mainTreeInterface->createVFolder
    (
      parentFolderPath,
      folderName,
      type,
      subType,
      serverPath
    );
}

  void
KomKOTreeSlots::slotDeleteVFolder
(
  const QString & parentFolderPath,
  const QString & folderName,
  const QString & type
)
{
  m_mainTreeInterface->deleteVFolder(parentFolderPath, folderName, type);
}

  void
KomKOTreeSlots::slotMoveFolder()
{
  QString currentPath = m_mainTreeInterface->path(m_mainTreeInterface->currentItem());
  QString targetPath = FolderChooserDialog::getFolder("/Datebook", 0);

  if( targetPath.isEmpty() )
    return;

  if(targetPath != currentPath)
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }

    VFolder * folder = vfs->folderRef(currentPath);
    QStringList foldersTree;
    if( folder )
    {
      // check if the target folder is a subfolder
      folder->tree(foldersTree, -1);
      for(QStringList::const_iterator it=foldersTree.begin(); it!=foldersTree.end(); ++it)
      {
        if( targetPath == (*it) )
        {
          QMessageBox::warning( 0L,
                                QObject::tr("Folder Move Warning"),
                                QObject::tr("Moving a folder in one of its subfolders isn't allowed."),
                                QMessageBox::Ok,
                                QMessageBox::NoButton,
                                QMessageBox::NoButton );
          return;
        }
      }

      vfs->moveFolder(folder->path(), targetPath + "/" + folder->name() );
    }
  }
  else
  {
    QMessageBox::warning( 0L,
                          QObject::tr("Folder Move Warning"),
                          QObject::tr("The new folder path is the same with the old one."),
                          QMessageBox::Ok,
                          QMessageBox::NoButton,
                          QMessageBox::NoButton );
  }
}

  void
KomKOTreeSlots::slotRenameFolder()
{
  QString currentPath = m_mainTreeInterface->path( m_mainTreeInterface->currentItem() );
  bool ok = false;
  QString targetName = QInputDialog::getText( QObject::tr("Folder Rename"),
                                              QObject::tr("Please insert the new folder name:"),
                                              QLineEdit::Normal,
                                              QString::null,
                                              &ok, 0L, "NotesFolderRename");
  if( ok )
  {
    VFSModule * vfs = 0;
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
      ComponentRef ref = trader->registeredComponent( AETHERA_VFS_SERVICE );
      if( ref.isValid() )
        vfs = static_cast<VFSModule*>(ref.instance());
      else
      {
        DEBUG_MSG("\n NO VFS Provider !\n");
        return;
      }
    }

    VFolder * folder = vfs->folderRef(currentPath);
    QStringList foldersTree;
    if( folder && !targetName.isEmpty() )
    {
      vfs->moveFolder(folder->path(), folder->parentFolder()->path() + "/" + targetName);
    }
    else
    {
      QMessageBox::warning( 0L,
                            QObject::tr("Folder Rename Warning"),
                            QObject::tr("The new name is empty or the folder is not available."),
                            QMessageBox::Ok,
                            QMessageBox::NoButton,
                            QMessageBox::NoButton );
    }
  }
}

  void
KomKOTreeSlots::slotExpungeFolder()
{
}

void
KomKOTreeSlots::slotSearchForSharedFolders()
{
  QStringList kolabList =
          Settings::instance()->readListEntry("/Aethera/Kolab/Accounts", ',');
  for (QStringList::Iterator itList = kolabList.begin();itList != kolabList.end();itList++)
  {
    KolabSharedFolder::ref()->scannSharedFolders(*itList, "calendar");
  }
}


