/***************************************************************************
         komkouiservice.h  -  The UI service for the KO plug in
                             -------------------
    begin                : Fri Jan 18 21:42:04 EET 2002
    copyright            : (C) 2002 by theKompany.com
    author               : Eugen C. <eug@thekompany.com>
    updated              : Bodi Vlad <bodivlad@hotmail.com>
 ***************************************************************************/

#ifndef KOMKOUISERVICE_H
#define KOMKOUISERVICE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <aethera/komaetheraplugin.h>
#include <aethera/komaetheratools.h>

#include <komplugin/kpimexchangeservice.h>

#include <qlabel.h>
#include <qlayout.h>

#include "komeventalarmhandler.h"
#include "komeventalarmservice.h"

// the main icon
#define KO_ACTIVE             "DateBook2_active.png"
#define KO_NORMAL             "DateBook2_inactive.png"
#define KO_TOP_BANNER_LEFT    "DateBook_Type.png"
#define KO_TOP_BANNER_RIGHT   "DateBook_Banner2.png"
#define KO_LEFT_BANNER        "aethera-home.png"

// Actions string id.
#define KO_FILE_NEW           "KOFileNew"
#define KO_FILE_SAVE          "KOFileSave"
//
#define KO_FILE_CUT           "KOFileCut"
#define KO_FILE_COPY          "KOFileCopy"
#define KO_FILE_PASTE         "KOFilePaste"
#define KO_FILE_DELETE        "KOFileDelete"
#define KO_FILE_SELECTALL     "KOFileSelectAll"
#define KO_FILE_UNSELECTALL   "KOFileUnSelectAll"
//
#define KO_FILE_IMPORT        "KOFileImport"
#define KO_FILE_EXPORT        "KOFileExport"
//
#define KO_FILE_PRINT         "KOFilePrint"
//
#define KO_EDIT_CUT           "KOEditCut"
#define KO_EDIT_COPY          "KOEditCopy"
#define KO_EDIT_PASTE         "KOEditPaste"
#define KO_EDIT_DELETE        "KOEditDelete"
#define KO_EDIT_FIND          "KOEditFind"
//#define KO_EDIT_UNDO          "KOEditUndo"
//#define KO_EDIT_REDO          "KOEditRedo"
//#define KO_HELP_CONTENTS      "KOHelpContents"
//#define KO_HELP_INDEX         "KOHelpIndex"
//#define KO_HELP_ABOUT         "KOHelpAbout"
#define KO_WHATS_NEXT         "KOWhatsNext"
#define KO_VIEW_LIST          "KOViewList"
#define KO_VIEW_DAY           "KOViewDay"
#define KO_VIEW_WORKWEEK      "KOViewWorkWeek"
#define KO_VIEW_WEEK          "KOViewWeek"
#define KO_VIEW_MONTH         "KOViewMonth"
#define KO_VIEW_TODO          "KOViewToDo"
#define KO_GO_PREVIOUS        "KOGoPrevious"
#define KO_GO_FORWARD         "KOGoForward"
#define KO_GO_TODAY           "KOGoToday"
#define KO_NEW_EVENT          "KONewEvent"
#define KO_NEW_TODO           "KONewToDo"
//
#define KO_ALARM_DEMON        "KOAlarmDemon"

#define VIEWLIST_NORMAL       "cr22-action-list.png"
#define VIEWDAY_NORMAL        "cr22-action-1day.png"
#define VIEWWORKWEEK_NORMAL   "cr22-action-5days.png"
#define VIEWWEEK_NORMAL       "cr22-action-7days.png"
#define VIEWMONTH_NORMAL      "cr22-action-month.png"
#define VIEWTODO_NORMAL       "cr22-action-todo.png"
#define VIEWTODAY_NORMAL      "cr22-action-today.png"
#define NEWEVENT_NORMAL       "cr22-action-appointment.png"
#define NEWTODO_NORMAL        "cr22-action-newtodo.png"
#define GOBACKWARD_NORMAL     "cr22-action-3leftarrow.png"
#define GOFORWARD_NORMAL      "cr22-action-3rightarrow.png"
//
#define ALARMD_ICON           "cr22-action-3rightarrow.png"

#define KO_OUTGOING_DIALOG    "KO_OUTGOING_DIALOG"
#define KO_INCOMING_DIALOG    "KO_INCOMING_DIALOG"
#define KO_PUBLISH_EVENT      "KO_PUBLISH_EVENT"
#define KO_REQUEST_EVENT      "KO_REQUEST_EVENT"
#define KO_REFRESH_EVENT      "KO_REFRESH_EVENT"
#define KO_CANCEL_EVENT       "KO_CANCEL_EVENT"
#define KO_REPLY_EVENT        "KO_REPLAY_EVENT"
#define KO_COUNTER_EVENT      "KO_COUNTER_EVENT"
#define KO_PUBLISH_FREEBUSY   "KO_PUBLISH_FREEBUSY"

#define EXCHANGE_DOWNLOAD     "EXCHANGE_DOWNLOAD"
#define EXCHANGE_UPLOAD       "EXCHANGE_UPLOAD"
#define EXCHANGE_DELETE       "EXCHANGE_DELETE"
#define EXCHANGE_CONFIGURE    "EXCHANGE_CONFIGURE"

extern "C"
{
    DATEBOOK_PLUGIN_API kore::Interface* komkouiservice_factory();
};

class KOUIPartPrivate;
class KomKOTreeInterface;
class CalendarMainView;
class CalendarListView;

class DATEBOOK_PLUGIN_API KomKOUIService : public KomUserInterfaceService
{

  private:
  
    KomKOUIService();
    
  public:
  
    static KomKOUIService* instance();
    virtual ~KomKOUIService();

    // VFS Browser
    virtual KomMainTreeInterface* getMainTreeInterface( QListView *parentTree );
    
    // From UIPart
    /** Query for capabilities.*/
    virtual bool hasCapability(const QString&);
    /** Create a new widget corresponding to the string.*/
    virtual void createWidget(const QString&);
    /** Show the part toolbars.*/
    virtual void showToolbars(QMainWindow *);
    /** Hide the part toolbars.*/
    virtual void hideToolbars(QMainWindow *);
    /** Show the part manus.*/
    virtual void showMenus(QMenuBar *);
    /** Hide the part menu.*/
    virtual void hideMenus(QMenuBar *);

    virtual QString prettyName() const;
    
    virtual void buildTree();
    virtual void removeTree();
    
    virtual void execAboutDialog(QWidget*) const;
    
//  public slots:
    virtual void setCurrentURL(QString);
    virtual QString getCurrentFolder() const;
    virtual QString getCurrentObject() const;
    
    CalendarMainView* getCalendarMainView(){ return calendarView; }
    
  protected:
    /** Create the QT signal/slot handler.*/
    virtual UIPartPrivate *createQTHandler(void);
    /** Create the QActions here.*/
    virtual void createActions();
    /** Make all the needed connections.*/
    virtual void connectActions();
    /** Disconnect the acctions.*/
    virtual void disconnectActions();

    CalendarMainView *calendarView;
    
    QString fileName;
    /** Left widget.*/
    KomContainerWidget *leftWidget;
    /** Top widget.*/
    KomContainerWidget *topWidget;
    /** Top banner widget.*/
    KomContainerWidget *topBannerWidget;
    
    // VFS Tree
    KomKOTreeInterface * treeInterface;
    
  private:
  
    static KomKOUIService * instance_;

    QPopupMenu *fileMenu;
    QPopupMenu *editMenu;
    QPopupMenu *viewMenu;
    QPopupMenu *goMenu;
    QPopupMenu *actionsMenu;
    QPopupMenu *configMenu;
    QPopupMenu *exchangeMenu;
    QPopupMenu *scheduleMenu;
    
    int fileMenuId;
    int editMenuId;
    int viewMenuId;
    int goMenuId;
    int actionsMenuId;
    int configMenuId;
    int exchangeMenuId;
    int scheduleMenuId;
    
    VFSModule * vfs;
    bool bActions;

    KomEventAlarmService* alarmDemon;
    KomEventAlarmHandler* eventAlarmHandler;
    bool useAlarmDemon;
    
    bool hasExchangePlugin;
    KPimExchangeService* exchangeService;

};


#endif
