/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
  Date     : 05 August 2003
  Author   : Ghiciuc Ilie (ilie@eco-spaces.com.ro)
---------------------------------------------------------------------------------
*/

//own includes
#include "kpimexchangeservice.h"
#include "exchangedialog.h"
#include "datewidget.h"

//korganizer includes
#include <libkcal/calendar.h>
#include <korganizer/komkouiservice.h>
#include <korganizer/calendarmainview.h>
#include <korganizer/calendarobjectview.h>

//QT includes
#include <qmessagebox.h>

kore::Interface* kpimexchangeservice_factory()
{
  return new KPimExchangeService();
}
    
KPimExchangeService::KPimExchangeService()
  : uiService( 0L ),
    handler( 0L ),
    exchangeClient( 0L ),
    exchangeAccount( 0L )    
{ 
  MetaInfo *info = metaInfo();
  
  info->replaceAll("name", EXCHANGESERVICE_NAME );
  info->replaceAll("class", "KPimExchangeService");
  info->replaceAll("class/fqn", "KPimExchangeService");
  info->replaceAll("description", EXCHANGESERVICE_DESCRIPTION );
  info->replaceAll("factory", "kpimexchangeservice_factory");
  info->insert("service", EXCHANGESERVICE_SERVICE );
  
  handler = new KPimExchangeServiceHandler( this );
  Q_CHECK_PTR( handler );
  
  exchangeAccount = new KPIM::ExchangeAccount( );
  Q_CHECK_PTR( exchangeAccount );
  
  exchangeClient = new KPIM::ExchangeClient( exchangeAccount );
  Q_CHECK_PTR( exchangeClient );
}


void KPimExchangeService::setUIService( KomKOUIService* ui )
{
  uiService = ui;
}


KomKOUIService* KPimExchangeService::getUIService()
{
  return uiService;
}


KPimExchangeServiceHandler* KPimExchangeService::qtHandler()
{
  return handler;
}


void KPimExchangeService::slotDownload()
{
  KCal::Calendar* calendar = uiService->getCalendarMainView()->getCalendarObjectView()->calendar();
  
  ExchangeDialog dialog;
//    ( uiService->getCalendarMainView()->getCalendarObjectView()->startDate(), 
//      uiService->getCalendarMainView()->getCalendarObjectView()->endDate() 
//    );
    
  if( dialog.exec() != QDialog::Accepted )
    return;
  
  QDate start = dialog.m_start->date();
  QDate end = dialog.m_end->date();
  
  int result = exchangeClient->downloadSynchronous( calendar, start, end, true);
    
  if( result == KPIM::ExchangeClient::ResultOK  )
  {
//    emit calendarChanged();
    return;
  }
  else
    showError( result, exchangeClient->detailedErrorString() );
   
}


void KPimExchangeService::slotUpload()
{
  KCal::Event* event = static_cast<KCal::Event*>( uiService->getCalendarMainView()->getCalendarObjectView()->currentSelection() );
  if( !event )
  {
    QMessageBox::information( 0L, QObject::tr("Exchange upload"), QObject::tr("Please select an appointment") );
    return;
  }

  int response 
    = QMessageBox::information( 0L, QObject::tr( "Exchange upload" ), 
                                QObject::tr( "Upload is still experimental. Would you like to continue?" ),
                                QObject::tr( "Continue" ), QObject::tr( "Cancel" ) 
                              ) ;
  if( response != 0 )
    return;
  
  int result = exchangeClient->uploadSynchronous( event );
  if( result == KPIM::ExchangeClient::ResultOK  )
  {
//    emit calendarChanged();
    return;
  }
  else
    showError( result, exchangeClient->detailedErrorString() );
}


void KPimExchangeService::slotDelete()
{
  KCal::Event* event = static_cast<KCal::Event*>( uiService->getCalendarMainView()->getCalendarObjectView()->currentSelection() );
  if( !event )
  {
    QMessageBox::information( 0L, QObject::tr("Exchange upload"), QObject::tr("Please select an appointment") );
    return;
  }

  int response 
    = QMessageBox::information( 0L, QObject::tr( "Exchange upload" ), 
                                QObject::tr( "Delete is still experimental. Would you like to continue?" ),
                                QObject::tr( "Continue" ), QObject::tr( "Cancel" ) 
                              ) ;
  if( response != 0 )
    return;
  
  int result = exchangeClient->removeSynchronous( event );
  if( result == KPIM::ExchangeClient::ResultOK  )
  {
//    emit calendarChanged();
    uiService->getCalendarMainView()->getCalendarObjectView()->calendar()->deleteEvent( event );
    return;
  }
  else
    showError( result, exchangeClient->detailedErrorString() );
  
}


void KPimExchangeService::slotConfigure()
{
  qDebug( "KPimExchangeService::slotConfigure()" );
}


void KPimExchangeService::showError( int error, const QString& moreInfo /* = QString::null */ )
{
  QString errorText;
  switch( error ) {
  case KPIM::ExchangeClient::ResultOK:
    errorText = i18n( "No Error" );
    break;
  case KPIM::ExchangeClient::CommunicationError:
    errorText = i18n( "The Exchange server could not be reached or returned an error." );
    break;
  case KPIM::ExchangeClient::ServerResponseError:
    errorText = i18n( "Server response could not be interpreted." );
    break;
  case KPIM::ExchangeClient::IllegalAppointmentError:
    errorText = i18n( "Appointment data could not be interpreted." );
    break;
  case KPIM::ExchangeClient::NonEventError:
    errorText = i18n( "This should not happen: trying to upload wrong type of event." );
    break;
  case KPIM::ExchangeClient::EventWriteError:
    errorText = i18n( "An error occured trying to write an appointment to the server." );
    break;
  case KPIM::ExchangeClient::DeleteUnknownEventError:
    errorText = i18n( "Trying to delete an event that is not present on the server." );
    break;
  case KPIM::ExchangeClient::UnknownError:
  default:
    errorText = i18n( "Unknown Error" );
  }

  if ( error !=  KPIM::ExchangeClient::ResultOK ) {
    if ( moreInfo.isNull() )
      KMessageBox::error( 0L, errorText, i18n( "Exchange Plugin" ) );
    else
      KMessageBox::error( 0L, errorText + moreInfo, i18n( "Exchange Plugin" ) );
  }
}

