#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qframe.h>

#include "klocale.h"
#include "kdebug.h"

#include "kdialogbase.h"

KDialogBase::KDialogBase()
{
}

KDialogBase::KDialogBase( QWidget *parent, const char *name, bool modal,
             const QString &caption,
             int buttonMask, ButtonCode defaultButton,
             bool separator, 
             const QString &user1,
             const QString &user2,
             const QString &user3) :
    KDialog( parent, name, modal ),
    mMainWidget(0),
    mTabWidget(0),
    mPlainPage(0),
    mTopLayout(0),
    mOkButton(0),
    mApplyButton(0),
    mCancelButton(0),
    mCloseButton(0),
    mUser1Button(0),
    mUser2Button(0),
    mUser3Button(0)
{
  init(caption, buttonMask, defaultButton, user1, user2, user3);
}

KDialogBase::KDialogBase( int dialogFace, const QString &caption, 
             int buttonMask, ButtonCode defaultButton, 
             QWidget *parent, const char *name, bool modal, 
             bool separator,
             const QString &user1,
             const QString &user2,
             const QString &user3) :
    KDialog( parent, name, modal ),
    mMainWidget(0),
    mTabWidget(0),
    mPlainPage(0),
    mTopLayout(0),
    mOkButton(0),
    mApplyButton(0),
    mCancelButton(0),
    mCloseButton(0),
    mUser1Button(0),
    mUser2Button(0),
    mUser3Button(0)
{
  init(caption, buttonMask , defaultButton, user1, user2, user3);
}


void KDialogBase::init(
                  const QString &caption,
                  int buttonMask ,
                  ButtonCode defaultButton,
                  const QString &user1,
                  const QString &user2,
                  const QString &user3 )
{
  mMainWidget = 0;
  mTabWidget = 0;
  mPlainPage = 0;
  mTopLayout = 0;

  if ( !caption.isEmpty() )
  {
    setCaption( caption );
  }

  if ( buttonMask & Ok )
  {
    mOkButton = new QPushButton( i18n("Ok"), this );
    if( mOkButton )
    {
      connect( mOkButton, SIGNAL( clicked() ), SLOT( slotOk() ) );
      
      if( defaultButton == Ok )
      {
        mOkButton->setDefault(true);
      }
    }
  }

  if ( buttonMask & Apply )
  {
    mApplyButton = new QPushButton( i18n("Apply"), this );
    
    if( mApplyButton )
    {
      connect( mApplyButton, SIGNAL( clicked() ), SLOT( slotApply() ) );
      
      if( defaultButton == Apply )
      {
        mApplyButton->setDefault(true);
      }
    }
  }

  if ( buttonMask & Cancel )
  {
    mCancelButton = new QPushButton( i18n("Cancel"), this );
    
    if( mCancelButton )
    {
      connect( mCancelButton, SIGNAL( clicked() ), SLOT( slotCancel() ) );
      
      if( defaultButton == Cancel )
      {
        mCancelButton->setDefault(true);
      }
    }
  }

  if ( buttonMask & Close )
  {
    mCloseButton = new QPushButton( i18n("Close"), this );
    if( mCloseButton )
    {
      connect( mCloseButton, SIGNAL( clicked() ), SLOT( slotClose() ) );
      
      if( defaultButton == Close )
      {
        mCloseButton->setDefault(true);
      }
    }
  }

  if ( buttonMask & User1 )
  {
    mUser1Button = new QPushButton( user1, this );
    
    if( mUser1Button )
    {
      connect( mUser1Button, SIGNAL( clicked() ), SLOT( slotUser1() ) );
      
      if( defaultButton == User1 )
      {
        mUser1Button->setDefault(true);
      }
    }
  }

  if ( buttonMask & User2 )
  {
    mUser2Button = new QPushButton( user2, this );
    
    if( mUser2Button )
    {
      connect( mUser2Button, SIGNAL( clicked() ), SLOT( slotUser2() ) );
      
      if( defaultButton == User2 )
      {
        mUser2Button->setDefault(true);
      }
    }
  }

  if ( buttonMask & User3 ) {
    mUser3Button = new QPushButton( user3, this );
    
    if( mUser3Button )
    {
      connect( mUser3Button, SIGNAL( clicked() ), SLOT( slotUser3() ) );
      
      if( defaultButton == User3 )
      {
        mUser3Button->setDefault(true);
      }
    }
  }
}

QTabWidget *KDialogBase::tabWidget()
{
  if ( !mTabWidget ) {
    mTabWidget = new QTabWidget( this );
    setMainWidget( mTabWidget );
  }
  return mTabWidget;
}

void KDialogBase::initLayout()
{
  delete mTopLayout;
  mTopLayout = new QVBoxLayout( this );
  mTopLayout->setMargin( marginHint() );
  mTopLayout->setSpacing( spacingHint() );
  
  mTopLayout->addWidget( mMainWidget );
  
  QBoxLayout *buttonLayout = new QHBoxLayout;
  if( buttonLayout )
  {
    mTopLayout->addLayout( buttonLayout );
    
    buttonLayout->addStretch();
    if ( mOkButton ) buttonLayout->addWidget( mOkButton );
    if ( mApplyButton ) buttonLayout->addWidget( mApplyButton );
    if ( mCancelButton ) buttonLayout->addWidget( mCancelButton );
    if ( mCloseButton ) buttonLayout->addWidget( mCloseButton );
    if ( mUser1Button ) buttonLayout->addWidget( mUser1Button );
    if ( mUser2Button ) buttonLayout->addWidget( mUser2Button );
    if ( mUser3Button ) buttonLayout->addWidget( mUser3Button );
    buttonLayout->addStretch();
  }
}

QFrame *KDialogBase::addPage( const QString &name )
{
//  kdDebug() << "KDialogBase::addPage(): " << name << endl;

  QFrame *frame = new QFrame( tabWidget() );
  tabWidget()->addTab( frame, name );
  return frame;
}

QFrame *KDialogBase::addPage( const QString &name, int, const QPixmap & )
{
  return addPage( name );
}


void KDialogBase::setMainWidget( QWidget *widget )
{
  kdDebug() << "KDialogBase::setMainWidget()" << endl;

  mMainWidget = widget;
  initLayout();
}


void KDialogBase::enableButton( ButtonCode id, bool state )
{
  QPushButton *button = 0;
  switch ( id ) {
    case Ok:
      button = mOkButton;
      break;
    case Apply:
      button = mApplyButton;
      break;
    default:
      break;      
  }
  if ( button ) {
    button->setEnabled( state );
  }
}

void KDialogBase::enableButtonOK( bool state )
{
  enableButton( Ok, state );
}

void KDialogBase::enableButtonApply( bool state )
{
  enableButton( Apply, state );
}


int KDialogBase::pageIndex( QWidget *widget ) const
{
  return 0;
}


bool KDialogBase::showPage( int index )
{
  return false;
}

QFrame *KDialogBase::plainPage()
{
  if ( !mPlainPage ) {
    mPlainPage = new QFrame( this );
    setMainWidget( mPlainPage );
  }
  return mPlainPage;
}

void KDialogBase::slotOk()
{
  accept();
}

void KDialogBase::slotApply()
{
}

void KDialogBase::slotCancel()
{
  reject();
}

void KDialogBase::slotClose()
{
  reject();
}

void KDialogBase::slotUser1()
{
  emit user1Clicked();
}

void KDialogBase::slotUser2()
{
  emit user2Clicked();
}

void KDialogBase::slotUser3()
{
  emit user3Clicked();
}
