
//own includes
#include "davjob.h"

//WebDav includes
#include <webdavrequestheader.h>
#include <webdav.h>

using namespace KIO;

/*
----------------------------------------------------------------------------------------
DavJob class
----------------------------------------------------------------------------------------
*/

DavJob::DavJob()
{
  init();
}

void DavJob::init()
{
  webDav = new WebDav;
  
  connect( webDav, SIGNAL(requestFinished(int,bool)),
           this, SLOT( slotRequestFinished(int, bool))
         );
}


void DavJob::slotRequestFinished( int, bool )
{
  responseDoc.setContent( webDav->readAll(), true );
  emit result(this);
}


void DavJob::propFind
  ( const KURL& url, 
    const QDomDocument& proprieties, 
    QString depth, 
    bool 
  )
{
  qDebug( "DavJob::propFind()" );

  WebDavRequestHeader request( "PROPFIND", url.path() );
  request.setValue( "Host" , url.host() );
  request.setValue( "Depth", depth );
  request.setContentType( "text/xml" );
  request.setValue( "Content-Encoding", "utf8" );
  
  webDav->setHost( url.host() );  

  staticData = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n" + proprieties.toString().utf8();
  staticData.truncate( staticData.size() - 1 );

  webDav->request( request, staticData );
}

void DavJob::propPatch
  ( const KURL& url, 
    const QDomDocument& nsURI, 
    bool
  )
{
  qDebug( "DavJob::propPatch()" );

  WebDavRequestHeader request( "PROPFIND", url.path() );
  request.setValue( "Host" , url.host() );
  request.setContentType( "text/xml" );
  request.setValue( "Content-Encoding", "utf8" );
  
  webDav->setHost( url.host() );  

  staticData = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n" + nsURI.toString().utf8();
  staticData.truncate( staticData.size() - 1 );

  webDav->request( request, staticData );
}


void DavJob::propSearch
      ( const KURL& url, 
        const QString& nsURI, 
        const QString& qName, 
        const QString& query, 
        bool
      )
{
  qDebug( "DavJob::propSearch()" );

  WebDavRequestHeader request( "POST", url.path() );
  request.setValue( "Host" , url.host() );
  request.setContentType( "text/xml" );
  request.setValue( "Content-Encoding", "utf8" );
  
  webDav->setHost( url.host() );  

  QDomDocument doc;
  QDomElement searchrequest = doc.createElementNS( "DAV:", "searchrequest" );
  QDomElement searchelement = doc.createElementNS( nsURI, qName );
  QDomText text = doc.createTextNode( query );
  searchelement.appendChild( text );
  searchrequest.appendChild( searchelement );
  doc.appendChild( searchrequest );

  staticData = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n" + doc.toString().utf8();
  staticData.truncate( staticData.size() - 1 );

  webDav->request( request, staticData );
}       


void DavJob::addMetaData( const QString&, const QString& )
{
}


QDomDocument& DavJob::response()
{
  return responseDoc;
}

/*
----------------------------------------------------------------------------------------
global functions
----------------------------------------------------------------------------------------
*/
DavJob* KIO::davPropFind
  ( const KURL& url, 
    const QDomDocument& proprieties, 
    QString depth, 
    bool 
  )
{
  qDebug( "global function --> davPropFind()" );
  DavJob* job = new DavJob();
  job->propFind( url, proprieties, depth );
  return job;
}


DavJob* KIO::davSearch
  ( const KURL& url, 
    const QString& nsURI, 
    const QString& qName, 
    const QString& query, 
    bool
  )
{
  qDebug( "global function --> davPropSearch()" );
  DavJob* job = new DavJob();
  job->propSearch( url, nsURI, qName, query );
  return job; 
}


DavJob* KIO::davPropPatch
  ( const KURL& url, 
    const QDomDocument& nsURI,  
    bool
  )
{
  qDebug( "global function --> davPropPatch()" );
  DavJob* job = new DavJob();
  job->propPatch( url, nsURI );
  return job; 
}
