/***************************************************************************
                              dirs.cpp
                             -------------------
    begin                : Thu Oct 11 16:42:19 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/
 
#include "dirs.h"

#ifdef _QTE_
#include <qpe/qpeapplication.h>
#include <qpe/resource.h>
#endif

#ifdef _QTMAC_
#include <CoreServices/CoreServices.h>
#endif

#include <qfile.h>
#include <qdir.h>

#include <settings.h>

Dirs * Dirs::instance_ = 0L;

Dirs::Dirs()
{
#ifdef _QTE_
  qDebug("add path = %s", QPEApplication::qpeDir().latin1() );
  pathList_.append(QFile::encodeName( QPEApplication::qpeDir() ));
#endif
  
  qDebug("add path = %s", defaultAppDir().latin1());
  pathList_.append(QFile::encodeName( defaultAppDir() ));
  
  qDebug("add path = %s", defaultTKCDir().latin1());
  pathList_.append(QFile::encodeName( defaultTKCDir() ));
  
  qDebug("add path = %s", homeDir().latin1());
  pathList_.append(QFile::encodeName( homeDir() ));
}

Dirs::~Dirs()
{
  // Empty.
}

  QStringList
Dirs::pathList() const
{
  return Dirs::thisInstance()->pathList_;
}

  QString
Dirs::locate(const QString & name)
{
  Dirs * ad = Dirs::thisInstance();

  for
    (
      QStringList::ConstIterator it(ad->pathList_.begin());
      it != ad->pathList_.end();
      ++it
    )
  {
    QDir d(*it);

    if (d.exists(name))
      return ((*it) + "/" + name);
  }

  return QString::null;
}

  QString
Dirs::locateDir(const QString & dir)
{
  Dirs * ad  = Dirs::thisInstance();

  for
    (
      QStringList::ConstIterator it(ad->pathList_.begin());
      it != ad->pathList_.end();
      ++it
    )
  {
    QDir d((*it) + "/" + dir);

    if( d.exists() )
      return ((*it) + "/" + dir);
  }

  return QString::null;
}

  void
Dirs::prependPath(const QString & fullPath)
{
  Dirs::thisInstance()->pathList_.prepend(fullPath);
}

  void
Dirs::appendPath(const QString & fullPath)
{
  Dirs::thisInstance()->pathList_.append(fullPath);
}

  bool
Dirs::removePath(const QString & fullPath)
{
#if QT_VERSION < 300
  Dirs::thisInstance()->pathList_.remove(fullPath);
  return true;
#else
  return (0 != Dirs::thisInstance()->pathList_.remove(fullPath));
#endif
}

QString Dirs::homeDir()
{
#ifdef _QTWIN_
  QString base;
  // Windows 9x
  if( QDir::homeDirPath() == QDir::rootDirPath())
    base = ".";
  // Windows NT/2K/XP variant
  else
    base = QDir::currentDirPath();
  // no trailing slash
  if(base.at(base.length()-1) == '/')
    base.truncate(base.length()-1);
  QDir proghome(base);
  if(!proghome.exists())
    QDir home(base);
  return proghome.path();
#else
  QDir proghome(QDir::currentDirPath());
  return proghome.path();
#endif
}

QString Dirs::defaultTKCDir()
{
#ifdef _QTE_
  return QString( QPEApplication::qpeDir() + "/lib/theKompany" );
#endif
#ifdef _QTWIN_
  // TODO: x:\ProgramFiles\theKompany
  return QString( "C:/Program Files/theKompany" );
#endif
#ifdef _QTX11_
  return QString( "/usr/local/lib/theKompany" );
#endif
#ifdef _QTMAC_
  return QString( "/usr/local/lib/theKompany" );
#endif
}

QString Dirs::defaultAppDir()
{
#ifdef _QTE_
  return QString( QPEApplication::qpeDir() + "/lib/" + AppName );
#endif
#ifdef _QTWIN_
  return QString( defaultTKCDir() + "/" + AppName );
#endif
#ifdef _QTX11_
  return QString( defaultTKCDir() + "/" + AppName );
#endif
#ifdef _QTMAC_
  CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
  CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle);
  const char *pathPtr = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());

  return QString( pathPtr );
#endif
}
