/***************************************************************************
                              dirs.h
                             -------------------
    begin                : Thu Oct 11 16:42:19 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/
 
#ifndef _DIRS_H
#define _DIRS_H

#include <defines.h>

#include <qstringlist.h>

/**
 * Handles the list of directories that may be searched for installed data.
 */
class TKCBASE_API Dirs
{
  public:

    static QString homeDir();
    
    static QString defaultTKCDir();
    
    static QString defaultAppDir();
    
    /**
     * Find a file in the standard paths. Returns the first filename which
     * matches while searching the list in order.
     *
     * To find using a relative path, just prepend the path.
     *
     * Example:
     *
     * QString path = Dirs::locate("icons/bigpic.png");
     *
     * @return QString::null if file is not found, otherwise full path
     * to file.
     */
    static QString locate(const QString & filename);

    /**
     * Find a directory in the standard paths. Returns the first directory which
     * matches while searching the list in order.
     *
     * To find using a relative path, just prepend the path.
     *
     * @return QString::null if directory is not found, otherwise full path
     * to the directory.
     */
    static QString locateDir(const QString & dir);

    /**
     * Add a path to the search list, before all the others. Files in
     * this directory will be located before others.
     */
    static void prependPath(const QString & fullPath);

    /**
     * Add a path to the search list, before all the others. Files in
     * this directory will be located before others.
     */
    static void appendPath(const QString & fullPath);

    /**
     * Remove a path from the search list. Use this when a plugin is
     * unloaded, for example.
     */
    static bool removePath(const QString & fullPath);

    /**
     * Have a peek at the path list. Probably useless, but no harm
     * providing it.
     */
    QStringList pathList() const;

  private:

    Dirs();
    Dirs(const Dirs &);
    Dirs & operator = (const Dirs &);
    ~Dirs();

    static Dirs * thisInstance()
    {
      if (0 == instance_)
        instance_ = new Dirs;

      return instance_;
    }

    static Dirs * instance_;
    QStringList pathList_;
};

#endif
