#include "imageeffect.h"

#include <math.h>

#include <qimage.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qrect.h>
#include <qstringlist.h>
#include <qbitmap.h>

ImageEffect::ImageEffect()
{
}

ImageEffect::~ImageEffect()
{
}

  void
ImageEffect::deSaturate(QPixmap &pm, double value)
{
	QImage	img	= pm.convertToImage() ;

	int 	 pixels	= (img.depth() > 8) ?
				img.width() * img.height() :
				img.numColors() ;

	unsigned *data	= (img.depth() > 8) ?
				(unsigned *)img.bits() :
				(unsigned *)img.colorTable();

	QColor	color	;
	int	h, s, v	;

	for (int i = 0 ; i < pixels ; i += 1)
	{
		color.setRgb (data[i]) ;
		color.hsv    (&h, &s, &v);
		color.setHsv (h, (int)(s * (1.0 - value) + 0.5), v) ;

		data[i] = qRgba
			  (	color.red  (),
				color.green(),
				color.blue (),
				qAlpha(data[i])
			  )	;
	}

	pm.convertFromImage (img) ;
}

  void
ImageEffect::toGray(QPixmap &pm, double value)
{
	QImage	img	= pm.convertToImage() ;

	int 	 pixels	= (img.depth() > 8) ?
				img.width() * img.height() :
				img.numColors() ;

	unsigned *data	= (img.depth() > 8) ?
				(unsigned *)img.bits() :
				(unsigned *)img.colorTable();

	int	rval	;
	int	gval	;
	int	bval	;
	int	val	;

	for (int i = 0 ; i < pixels ; i += 1)
	{
		val	= qGray  (data[i]) ;

		if (value < 1.0)
		{
			rval = (int)(value * val + (1.0-value) * qRed   (data[i])) ;
			gval = (int)(value * val + (1.0-value) * qGreen (data[i])) ;
			bval = (int)(value * val + (1.0-value) * qBlue  (data[i])) ;

			data[i] = qRgb (rval, gval, bval) ;
		}
		else	data[i] = qRgb (val,  val,  val ) ;
	}

	pm.convertFromImage (img) ;
}

  void
ImageEffect::semiTransparent(QImage &img)
{
  img.setAlphaBuffer(true);

  int x, y;
  if (img.depth() == 32)
  {
    int width  = img.width();
    int height = img.height();
      
    for (y=0; y<height; y++)
    {
      QRgb *line = (QRgb *) img.scanLine(y);
        
      for (x=(y%2); x<width; x+=2)
        line[x] &= 0x00ffffff;
    }
  }
  else
  {
    // Insert transparent pixel into the clut.
    int transColor = -1;
      
    // search for a color that is already transparent
    for (x=0; x<img.numColors(); x++)
    {
      // try to find already transparent pixel
      if (qAlpha(img.color(x)) < 127)
      {
        transColor = x;
        break;
      }
    }

    if(transColor < 0 || transColor >= img.numColors())
      return;

    img.setColor(transColor, 0);
    for (y=0; y<img.height(); y++)
      for (x=(y%2); x<img.width(); x+=2)
        img.setPixel(x, y, transColor);
  }
}

  void
ImageEffect::semiTransparent(QPixmap &pm)
{
	QImage	img ;

	if (pm.mask() == 0)
	{
	        img.create (pm.size(), 1, 2, QImage::BigEndian) ;
        	img.fill   (1);
	}
	else	img = pm.mask()->convertToImage();
 
	for (int y = 0 ; y < img.height() ; y += 1)
	{
		uchar	*line	= (uchar *) img.scanLine (y) ;
		uchar	pattern = (y % 2) ? 0x55 : 0xaa ;

		for (int x = (img.width() + 7) / 8 ; x > 0; x -= 1)
			*line++ &= pattern ;
	}

	QBitmap	mask	  ;
	mask.convertFromImage (img) ;
	pm.setMask (mask) ;
}

 void
ImageEffect::toGamma(QImage &img, double value)
{
    int pixels = (img.depth() > 8) ? img.width()*img.height()
                                   : img.numColors();
    unsigned int *data = (img.depth() > 8) ? (unsigned int *) img.bits() 
                                           : (unsigned int *) img.colorTable();
    QColor color;
    int i, rval, gval, bval;
    double gamma;
    gamma = 1/(2*value+0.5);

    for (i=0; i<pixels; i++)
    {
        color.setRgb(data[i]);
        color.rgb(&rval, &gval, &bval);
        
        rval = static_cast<int>(pow(static_cast<double>(rval)/255 , gamma)*255);
        gval = static_cast<int>(pow(static_cast<double>(gval)/255 , gamma)*255);
        bval = static_cast<int>(pow(static_cast<double>(bval)/255 , gamma)*255);
        
        data[i] = qRgba(rval, gval, bval, qAlpha(data[i]));
    }
}

