/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
//
// C++ Implementation: notificator
//
// Description: 
//
//
// Author: Eugen Constantinescu <eug@thekompany.com>
//
// Copyright: (C) 2005 theKompany.com
//
//
#include "notificator.h"

#include <playsound.h>
#include <settings.h>

Notificator * Notificator::instance_ = 0;

class NotificatorPrivate
{
  public:
  
    NotificatorPrivate(Notificator*);
    
    ~NotificatorPrivate();
    
    QString group;
    bool soundNotification;
    bool messageNotification;
  
  private:
  
    Notificator * parent_;
};

NotificatorPrivate::NotificatorPrivate(Notificator * parent)
  : soundNotification(true),
    messageNotification(true),
    parent_(parent)
{
}

NotificatorPrivate::~NotificatorPrivate()
{
}

Notificator * Notificator::instance()
{
  if( !instance_ )
  {
    instance_ = new Notificator(0, "Notificator");
  }
  
  return instance_;
}

Notificator::Notificator(QObject* parent, const char* name)
  : QObject(parent, name)
{
  d = new NotificatorPrivate(this);
}

Notificator::~Notificator()
{
  delete d;
  d = 0;
}

void Notificator::setSettingsGroup(const QString & group)
{
  d->group = group;
}

QString Notificator::getSettingsGroup() const
{
  return d->group;
}
    
void Notificator::setSoundId(const QString & soundId, const QString & soundPath)
{
  if( soundPath.isEmpty() )
  {
    Settings::instance()->removeGroup(d->group + "/Sound/" + soundId);
  }
  else
  {
    Settings::instance()->writeEntry(d->group + "/Sound/" + soundId, soundPath);
  }
}

void Notificator::setSoundEnabled(const QString & soundId, bool enable)
{
  Settings::instance()->writeEntry(d->group + "/Sound/Enabled/" + soundId, enable);
}
    
QString Notificator::getSound(const QString & soundId) const
{
  return Settings::instance()->readEntry(d->group + "/Sound/" + soundId);
}

bool Notificator::soundEnabled(const QString & soundId) const
{
  return Settings::instance()->readBoolEntry(d->group + "/Sound/Enabled/" + soundId, true);
}
    
QStringList Notificator::getAvailableSounds(const QString & filter) const
{
  if( filter.isEmpty() )
  {
    return Settings::instance()->entryList(d->group + "/Sound");
  }
  else
  {
    return Settings::instance()->entryList(d->group + "/Sound/" + filter);
  }
}

void Notificator::setSoundNotification(bool enable)
{
  d->soundNotification = enable;
}

bool Notificator::soundNotification() const
{
  return d->soundNotification;
}

void Notificator::setMessageEnabled(const QString & messageId, bool enable)
{
  Settings::instance()->writeEntry(d->group + "/Message/Enabled/" + messageId, enable);
}
    
bool Notificator::messageEnabled(const QString & messageId) const
{
  return Settings::instance()->readBoolEntry(d->group + "/Message/Enabled/" + messageId, true);
}
    
void Notificator::setMessageNotification(bool enable)
{
  d->messageNotification = enable;
}
  
bool Notificator::messageNotification() const
{
  return d->messageNotification;
}

void Notificator::slotPlaySound(const QString & soundId)
{
  if( soundNotification() )
  {
    QString soundPath = getSound(soundId);
    
    if( soundPath.isEmpty() )
    {
      emit notify( tr("Sound not available: ") + soundId );
    }
    else
    {
      PlaySound::play( soundPath );
      emit notify( tr("Play sound: ") + soundPath );
    }
  }
}

void Notificator::slotShowMessage(const QString & text)
{
  if( messageNotification() )
  {
    // todo
    emit notify( tr("Show message: ") + text );
  }
}

void Notificator::slotNotify(const QString & soundId, const QString & text)
{
  QString notifyText;
  if( soundNotification() )
  {
    QString soundPath = getSound(soundId);
    if( soundPath.isEmpty() )
    {
      notifyText = tr("Sound not available: ") + soundId;
    }
    else
    {
      PlaySound::play( soundPath );
      notifyText = tr("Play sound: ") + soundPath;
    }
    
    notifyText += "\n";
  }
  if( messageNotification() )
  {
    // todo
    notifyText += tr("Show message: ") + text;
  }
  
  if( !notifyText.isEmpty() )
  {
    emit notify(notifyText);
  }
}

