/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
//
// C++ Interface: notificator
//
// Description: 
//
//
// Author: Eugen Constantinescu <eug@thekompany.com>
//
// Copyright: (C) 2005 theKompany.com
//
//
#ifndef NOTIFICATOR_H
#define NOTIFICATOR_H

#include <defines.h>

#include <qobject.h>

class NotificatorPrivate;

/**
It takes care about the popup messages and sounds available for your app.

@author Eugen Constantinescu
*/
class TKCBASE_API Notificator : public QObject
{
  Q_OBJECT

  public:
  
    static Notificator * instance();
    
    virtual ~Notificator();
    
    /** Set the group where to look for notification settings.
      * Ex: If the settings group is '/Aethera', then it will search for sound 
      * settings in '/Aethera/Notification/Sound/'...
      */
    virtual void setSettingsGroup(const QString&);
    
    /** Returns the current settings group, for debugging resons.*/
    virtual QString getSettingsGroup() const;
    
    // SOUND
    
    /** Associate the sound ID with the sound path.*/
    virtual void setSoundId(const QString &, const QString &);
    
    /** Enable/disable a sound.*/
    virtual void setSoundEnabled(const QString &, bool);
    
    /** Check is there is a sound associated with the ID.*/
    virtual QString getSound(const QString &) const;
    
    /** Check if the sound is enabled.*/
    virtual bool soundEnabled(const QString &) const;
    
    /** Get a list with available sound IDs.
      * Filter is a _subkey_ for the settings group!
      */
    virtual QStringList getAvailableSounds(const QString & filter=QString::null) const;
    
    /** Enable/disable the sound notifications.*/
    virtual void setSoundNotification(bool enable=true);
    
    /** Returns true if you can get sound notifications.*/
    virtual bool soundNotification() const;

    // MESSAGE
    
    /** Enable/disable a message.*/
    virtual void setMessageEnabled(const QString &, bool);
    
    /** Check if the message is enabled.*/
    virtual bool messageEnabled(const QString &) const;
    
    /** Enable/disable the message notifications.*/
    virtual void setMessageNotification(bool enable=true);
  
    /** Returns true if you can get message notifications.*/
    virtual bool messageNotification() const;
    
  public slots:
  
    /** Play the sound related to the notification id.*/
    void slotPlaySound(const QString&);
    
    /** Show a popup off-line message with the following text.*/
    void slotShowMessage(const QString&);
    
    /** Create a notification with both sound and popup message.*/
    void slotNotify(const QString&, const QString&);
  
  signals:
  
    /** If you want to log the notifications, just connect this signal.*/
    void notify(const QString &);
  
  private:

    // disable the default constructor
    Notificator();
    
    // it should be used only in the instance() call
    Notificator(QObject* parent, const char* name);
    
    static Notificator * instance_;
    
    NotificatorPrivate * d;
    
};

#endif
