/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    By PLUGINS we mean the commercial plug-ins developed by theKompany.com
    for Aethera.

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with PLUGINS (or with
    modified versions of PLUGINS that use the same license as PLUGINS),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than PLUGINS.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use PLUGINS with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
*/
//
// C++ Implementation: playsound
//
// Description: 
//
//
// Author: Eugen Constantinescu <eug@thekompany.com>
//
// Copyright: (C) 2005 theKompany.com
//
//
#include "playsound.h"

#include "dirs.h"

#include <qprocess.h>

QString PlaySound::player_ = QString::null;
bool PlaySound::enable_ = true;

class PlaySound::Private
{
  public:
  
    Private(PlaySound*);
    ~Private();
    
    QProcess * process_;
    
  private:
  
    PlaySound * parent_;
};

PlaySound::Private::Private(PlaySound * parent)
  : process_(0),
    parent_(parent)
{
}

PlaySound::Private::~Private()
{
}

PlaySound::PlaySound(const QString& filename, QObject* parent, const char* name)
  : QSound(filename, parent, name)
{
  d = new Private(this);
}

PlaySound::~PlaySound()
{
  delete d;
  d = 0;
}

bool PlaySound::isAvailable()
{
  if( !enable_ )
    return false;
  
  if( player().isEmpty() )
  {
    return QSound::isAvailable();
  }
  else
  {
    return true;
  }
}

void PlaySound::play(const QString& filename)
{
  if( enable_ )
  {
    QString path = locateSound(filename);
    if( !path.isEmpty() )
    {
      if( player().isEmpty() )
      {
        QSound::play(path);
      }
      else
      {
        QProcess * playProcess = new QProcess(0, "PlaySoundProcess");
        if( playProcess )
        {
          // it should be deleted once the playing is finished
          connect(playProcess, SIGNAL(launchFinished()),
                  playProcess, SLOT(deleteLater()));
          
          playProcess->clearArguments();
          playProcess->addArgument(player());
          playProcess->addArgument(path);
          playProcess->launch(QByteArray());
        }
      }
    }
  }
}

bool PlaySound::isPlayerAvailable(const QString& filename)
{
  if( !filename.isEmpty() )
  {
    // quick check
    QFile playFile(filename);
    if( playFile.exists() )
    {
      return true;
    }
    else
    {
      // try to locate it
      QString path = Dirs::locate(filename);
      if( path.isEmpty() )
      {
        path = Dirs::locate(QString("bin/") + filename);
      }
      
      if( !path.isEmpty() )
      {
        return true;
      }
      
      // todo: locate it as executable
    }
  }
  
  return false;
}

void PlaySound::setPlayer(const QString& filename)
{
  QString path;
  if( !filename.isEmpty() )
  {
    // quick check
    QFile playFile(filename);
    if( playFile.exists() )
    {
      path = filename;
    }
    else
    {
      // try to locate it
      path = Dirs::locate(filename);
      if( path.isEmpty() )
      {
        path = Dirs::locate(QString("bin/") + filename);
      }
      
      // todo: locate it as executable
    }
    
    if( !path.isEmpty() )
    {
      player_ = path;
    }
  }
  else
  {
    player_ = QString::null;
  }
}

QString PlaySound::player()
{
  return player_;
}

QString PlaySound::locateSound(const QString & name)
{
  QString path;
  
  if( !name.isEmpty() )
  {
    // quick check
    QFile soundFile(name);
    if( soundFile.exists() && soundFile.isReadable() )
    {
      path = name;
    }
    else
    {
      // try to locate it
      path = Dirs::locate(name);
      if( path.isEmpty() )
      {
        path = Dirs::locate(QString("sound/") + name);
      }
      if( path.isEmpty() )
      {
        path = Dirs::locate(QString("sound/") + name + ".wav");
      }
    }
  }
    
  return path;
}
    
void PlaySound::setOn(bool enable)
{
  enable_ = enable;
}

void PlaySound::setOff(bool disable)
{
  enable_ = !disable;
}

bool PlaySound::isFinished() const
{
  if( player().isEmpty() )
  {
    return QSound::isFinished();
  }
  else
  {
    if( d->process_ && d->process_->isRunning() )
    {
      return false;
    }
    
    return true;
  }
}

int PlaySound::loops() const
{
  if( player().isEmpty() )
  {
    return QSound::loops();
  }
  else
  {
    return 0;
  }
}

int PlaySound::loopsRemaining() const
{
  if( player().isEmpty() )
  {
    return QSound::loopsRemaining();
  }
  else
  {
    return 0;
  }
}

void PlaySound::setLoops(int l)
{
  if( player().isEmpty() )
  {
    QSound::setLoops(l);
  }
  else
  {
    // just nothing right now
  }
}
      
void PlaySound::play()
{
  if( enable_ )
  {
    if( player().isEmpty() )
    {
      QSound::play();
    }
    else
    {
      d->process_ = new QProcess(parent(), "play a sound");
      if( d->process_ )
      {
        // it should be deleted once the playing is finished
        connect(d->process_, SIGNAL(launchFinished()),
                this, SLOT(stop()));
        
        d->process_->clearArguments();
        d->process_->addArgument(player());
        d->process_->addArgument( locateSound(fileName()) );
        d->process_->launch(QByteArray());
      }
    }
  }
}

void PlaySound::stop()
{
  if( player().isEmpty() )
  {
    QSound::stop();
  }
  else
  {
    if( d->process_ )
    {
      if( d->process_->isRunning() )
      {
        d->process_->tryTerminate();
        // leave 0.5 sec before killing it
//        QTimer::singleShot(500, d->process_, SLOT(kill()));
        // it should be deleted once the playing is finished
        connect(d->process_, SIGNAL(processExited()),
                d->process_, SLOT(deleteLater()));
      }
      else
      {
        delete d->process_;
      }
      
      d->process_ = 0;
    }
  }
}
