/***************************************************************************
                              qiconloader.cpp
                             -------------------
    begin                : Thu Oct 11 16:42:19 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/

#include "qiconloader.h"

#ifdef _QTE_
#include <qpe/resource.h>
#endif

#include <defines.h>
#include <settings.h>
#include <dirs.h>
#include <imageeffect.h>

#ifdef _QTWIN_
#include <qapplication.h>
#endif

QIconLoader * QIconLoader::instance_ = 0L;

QIconLoader::QIconLoader()
#ifdef _QTE_
  : style_( QIconLoader::Default ),
    iconsPath_( QString("pics/") + AppName + "/" )
#else
  : images_(349),
    style_( QIconLoader::Default ),
    iconsPath_( "pics/" )
#endif
{
  readSettings();
}

QIconLoader::~QIconLoader()
{
}

void QIconLoader::setPath(const QString & relativePath)
{
  iconsPath_ = relativePath;
}

QIconLoader::Style QIconLoader::iconStyle()
{
  return style_;
}

void QIconLoader::changeStyle(QIconLoader::Style style)
{
  style_ = style;
  
  writeSettings();
}

void QIconLoader::readSettings()
{
// TODO: for QTE
#ifndef _QTE_
  style_ = (QIconLoader::Style)
    Settings::instance()->readNumEntry(Settings::instance()->name() + "/General/IconStyle", QIconLoader::Default);
#endif
}

void QIconLoader::writeSettings()
{
// TODO: for QTE
#ifndef _QTE_
  Settings::instance()->writeEntry(Settings::instance()->name() + "/General/IconStyle", (int)style_);
  Settings::forceWrite();
#endif
}

  QPixmap
QIconLoader::barIcon(const QString & name)
{
  // check the cache first
  QPixmap * ptrImage = 0L;
  ptrImage = images_[name];
  if( ptrImage )
  {
    return *ptrImage;
  }

  QString path = Dirs::locate(iconsPath_ + name);

  if (path.isEmpty())
  {
    path = Dirs::locate(iconsPath_ + name + ".png");

    if (path.isEmpty())
    {
      path = Dirs::locate(iconsPath_ + name + ".xpm");
      
      if (path.isEmpty())
      {
        path = Dirs::locate(iconsPath_ + name + ".jpg");
        
        if (path.isEmpty())
        {
          path = Dirs::locate(iconsPath_ + name + ".gif");
        }
      }
    }
  }

#ifdef _QTE_
  if( path.isEmpty() )
  {
    path = Resource::findPixmap(name);
  }
  
  return QPixmap(path, 0, QPixmap::DiffuseAlphaDither);
#elif defined( _QTWIN_ )
  QPixmap::setDefaultOptimization( QPixmap::MemoryOptim );
  ptrImage = new QPixmap(path);
  QPixmap::setDefaultOptimization( QPixmap::NormalOptim );
#else
  ptrImage = new QPixmap(path);
#endif
  
  if(!ptrImage->isNull())
  {
    setImage(name, ptrImage);
    return *ptrImage;
  }
  else
  {
    delete ptrImage;
    ptrImage = 0L;
    return QPixmap();
  }
}

QIconSet QIconLoader::iconSet(const QPixmap & pixmap)
{
  QIconSet icons;
  
  if( pixmap.isNull() )
    return icons;

#ifdef _QTWIN_
  QPixmap::setDefaultOptimization( QPixmap::MemoryOptim );
#endif
  
  QPixmap activePixmap;
  QPixmap disabledPixmap;
  
  QImage activeImage = pixmap.convertToImage();
  QImage disabledImage(activeImage);
  disabledImage.detach();
  
  switch(style_)
  {
    case KDE_Default:
      // add semi-transparent effect
      ImageEffect::semiTransparent(disabledImage);
      
      disabledPixmap.convertFromImage(disabledImage);
      icons.setPixmap(disabledPixmap, QIconSet::Automatic, QIconSet::Disabled);
      break;
      
    case QT_Default:
      break;
      
    default:
      // add intensity effect
      ImageEffect::toGamma(activeImage);
      // add semi-transparent effect
      ImageEffect::semiTransparent(disabledImage);
      
      activePixmap.convertFromImage(activeImage);
      disabledPixmap.convertFromImage(disabledImage);
      icons.setPixmap(activePixmap, QIconSet::Automatic, QIconSet::Active);
      icons.setPixmap(disabledPixmap, QIconSet::Automatic, QIconSet::Disabled);
      break;
  }
  
  
  icons.setPixmap(pixmap, QIconSet::Automatic, QIconSet::Normal);

#ifdef _QTWIN_
  QPixmap::setDefaultOptimization( QPixmap::NormalOptim );
#endif
  
  return icons;
}

QPixmap QIconLoader::loadIcon( const QString &name, int )
{
  return barIcon(name);
}

QString QIconLoader::iconPath( const QString &name, int )
{
  QString path = Dirs::locate(iconsPath_ + name);

  if (path.isEmpty())
  {
    path = Dirs::locate(iconsPath_ + name + ".png");

    if (path.isEmpty())
    {
      path = Dirs::locate(iconsPath_ + name + ".xpm");
      
      if (path.isEmpty())
      {
        path = Dirs::locate(iconsPath_ + name + ".jpg");
        
        if (path.isEmpty())
        {
          path = Dirs::locate(iconsPath_ + name + ".gif");
        }
      }
    }
  }

#ifdef _QTE_
  if( path.isEmpty() )
  {
    path = Resource::findPixmap(name);
  }
#endif

  return path;
}

QIconSet QIconLoader::iconSet(const QString & name)
{
#ifdef _QTE_
  // this isn't working well yet
  // return Resource::loadIconSet(name);
  // so, just use my icon set
  QPixmap pixmap = barIcon(name);
  
  return iconSet( pixmap );
#else
  QPixmap pixmap = barIcon(name);
  
  return iconSet( pixmap );
#endif
}

void QIconLoader::setImage(const QString &name, QPixmap *pixmap)
{
  if( !name.isEmpty() && !pixmap->isNull() )
    images_.insert(name, pixmap);
}

QPixmap QIconLoader::image(const QString &name)
{
  QPixmap * image = 0L;

  image = images_[name];
  if( !image )
  {
      image = new QPixmap( barIcon(name) );

      if( !image->isNull() )
        images_.insert(name, image);
      else
      {
        delete image;
        image = 0L;
      }
  }

  if( image )
    return *image;
  else
    return QPixmap();
}

// ---------- STATIC ----------------

QPixmap BarIcon(const QString & name)
{
  return QIconLoader::instance()->barIcon(name);
}

QPixmap DesktopIcon( const QString &name, int )
{
  return BarIcon(name);
}

QPixmap SmallIcon( const QString &name )
{
  return BarIcon(name);
}

QPixmap SmallIconSet( const QString &name )
{
  return BarIcon(name);
}


