/***************************************************************************
                              qiconloader.h
                             -------------------
    begin                : Thu Oct 11 16:42:19 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
    author               : Eugen C. <eug@thekompany.com>
 ***************************************************************************/
 
#ifndef QICON_LOADER_H
#define QICON_LOADER_H

#include <defines.h>

#include <qpixmap.h>
#include <qimage.h>
#include <qiconset.h>
#include <qstring.h>
#include <qdict.h>

#ifdef Q_WS_WIN
#if defined(Q_TEMPLATEDLL)
// MOC_SKIP_BEGIN
template class TKCBASE_API QDict<QPixmap>;
// MOC_SKIP_END
#endif
#endif

/**
 * QIconLoader provides icons based on name only. It knows the paths
 * to use.
 */
class TKCBASE_API QIconLoader
{
  public:

    enum Style { Default=0, KDE_Default, QT_Default };
    
    static QIconLoader * instance()
    {
      if (0 == instance_)
        instance_ = new QIconLoader;

      return instance_;
    }

    void setPath(const QString&);
    
    /** Search and return a pixmap using the name.*/
    QPixmap barIcon(const QString & name);

    /** Search and return a pixmap using the name.*/
    QPixmap loadIcon( const QString &name, int i=0 );
    
    /** Search and return a pixmap using the name.*/
    QString iconPath( const QString &name, int i=0 );
    
    /** Return an icon set using the name.*/
    QIconSet iconSet(const QString & name);
    
    /** Return an icon set using the pixmap.*/
    QIconSet iconSet(const QPixmap & pixmap);
    
    /** Return a pixmap from cache.*/
    QPixmap image(const QString &name);
    /** Insert a pixmap in cache.*/
    void setImage(const QString &name, QPixmap*);
    
    QIconLoader::Style iconStyle();
    void changeStyle(QIconLoader::Style);
    
  private:

    QIconLoader();
    ~QIconLoader();
    
    void readSettings();
    void writeSettings();

    static QIconLoader * instance_;
    
    QDict<QPixmap> images_;
    QIconLoader::Style style_;
    QString iconsPath_;
};

/**
 * This is the usual namespace-polluting interface to QIconLoader.
 * Returns an icon set suitable for a toolbar.
 */
TKCBASE_API QPixmap BarIcon(const QString & name);

/**
 * This is the usual namespace-polluting interface to QIconLoader.
 * Returns an icon set suitable for a toolbar.
 */
TKCBASE_API QPixmap DesktopIcon(const QString& name, int);

/**
 * This is the usual namespace-polluting interface to QIconLoader.
 * Returns an icon set suitable for a toolbar.
 */
TKCBASE_API QPixmap SmallIcon(const QString& name);

/**
 * This is the usual namespace-polluting interface to QIconLoader.
 * Returns an icon set suitable for a toolbar.
 */
TKCBASE_API QPixmap SmallIconSet( const QString &name );

#endif
