/***************************************************************************
  qsystemtrayicon.cpp
  -  System tray icon with popup menu.
                             -------------------
    begin                : Tue Oct  9 21:01:08 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#include <qapplication.h>
#include <qpopupmenu.h>
#include <qsystemtrayicon.h>

#ifdef Q_WS_X11
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#define SYSTEM_TRAY_REQUEST_DOCK    0
#endif // Q_WS_X11

class QSystemTrayIcon::Private
{
  public:

    QWidget     * managed;
    QPopupMenu  * menu;
    int           minimizeRestoreID;
#ifdef Q_WS_X11
//    Atom          kde_net_wm_system_tray_window_for;
//    Atom          net_active_window;

#endif // Q_WS_X11
};

QSystemTrayIcon::QSystemTrayIcon(QWidget * managed)
  : QLabel(0, "QSystemTrayIcon")
{

  d = new Private;
  d->managed            = managed;
  d->menu               = 0L;       // Delay creation until needed.
  d->minimizeRestoreID  = -1;
  #ifdef Q_WS_X11
    setBackgroundMode(X11ParentRelative);
    Display *dpy = qt_xdisplay();

    WId myWin  = winId();

    // System Tray Protocol Specification. This will work with both KDE and GNOME properly
    // see http://standards.freedesktop.org/systemtray-spec/systemtray-spec-0.2.html

    Screen *screen = XDefaultScreenOfDisplay(dpy);
    int screen_num = XScreenNumberOfScreen(screen);
    char atom[32];
    snprintf(atom, sizeof(atom), "_NET_SYSTEM_TRAY_S%d", screen_num);
    Atom selectionAtom = XInternAtom(dpy, atom, false);
    XGrabServer(dpy);
    Window win = XGetSelectionOwner(dpy, selectionAtom);
    if (win != None)
    {
        XSelectInput(dpy, win, StructureNotifyMask);
    }
    XUngrabServer(dpy);
    XFlush(dpy);
    if (win != None)
    {
        XEvent ev;
        memset(&ev, 0, sizeof(ev));
        ev.xclient.type = ClientMessage;
        ev.xclient.window = win;
        ev.xclient.message_type = XInternAtom(dpy, "_NET_SYSTEM_TRAY_OPCODE", false);
        ev.xclient.format = 32;
        ev.xclient.data.l[0] = CurrentTime;
        ev.xclient.data.l[1] = SYSTEM_TRAY_REQUEST_DOCK;
        ev.xclient.data.l[2] = myWin;
        ev.xclient.data.l[3] = 0;
        ev.xclient.data.l[4] = 0;
        XSendEvent(dpy, win, false, NoEventMask, &ev);
        XSync(dpy, false);
    }
    #endif // Q_WS_X11
  setFixedSize(20, 20);
  resize(20, 20);

  show();
}

QSystemTrayIcon::~QSystemTrayIcon()
{
  delete d;
}

  QPopupMenu *
QSystemTrayIcon::popupMenu()
{
  if (0 == d->menu)
  {
    d->menu = new QPopupMenu(this);

    d->menu->insertSeparator();

    // Save this one, because we might change it later.

    d->minimizeRestoreID =
      d->menu->insertItem
      (
        tr("Minimize"),
        this,
        SLOT(slotToggleMinimizeRestore())
      );

    d->menu->insertItem(tr("&Quit"), d->managed, SLOT(close()));
  }

  return d->menu;
}

  void
QSystemTrayIcon::mousePressEvent(QMouseEvent * e)
{
  switch (e->button())
  {
    case LeftButton:
      slotToggleMinimizeRestore();
      break;

    case MidButton:
    case RightButton:

      if (d->managed->isVisible())
      {
        popupMenu()->changeItem(d->minimizeRestoreID, tr("Minimize"));
      }
      else
      {
        popupMenu()->changeItem(d->minimizeRestoreID, tr("Restore"));
      }

      // Inform derived classes.
      menuAboutToShow(popupMenu());

      popupMenu()->popup(e->globalPos());

    default:
      break;
  }
}

  void
QSystemTrayIcon::enterEvent(QEvent *)
{
  // TODO ? Is this necessary ? Seems to be some code in KSystemTray
  // to handle a focus problem.
}

  void
QSystemTrayIcon::slotToggleMinimizeRestore()
{
  if (!d->managed->isVisible())
  {
    d->managed->show();
  }
  else
  {
    d->managed->hide();
  }
}

  void
QSystemTrayIcon::menuAboutToShow(QPopupMenu *)
{
  // Empty. Used by derived classes.
}

