/***************************************************************************
  qsystemtrayicon.h
  -  System tray icon with popup menu.
                             -------------------
    begin                : Tue Oct  9 21:01:08 BST 2001
    copyright            : (C) 2001 by theKompany.com
    author               : Rik Hemsley <rik@kde.org>
 ***************************************************************************/

#ifndef QSYSTEM_TRAY_ICON_H
#define QSYSTEM_TRAY_ICON_H

#include <qlabel.h>

class QPopupMenu;

/**
 * QSystemTrayIcon provides a label which sits in the 'system tray'.
 * It manages a QWidget and updates itself to reflect the state of
 * that widget. Users can minimize/restore and close the managed
 * widget by using the menu items provided on the popup menu which
 * appears on a RMB press.
 *
 * Example usage:
 *
 * QSystemTrayIcon * sti = new QSystemTrayIcon(myWidget);
 * sti->setPixmap(some20x20Pixmap);
 */ 
class QSystemTrayIcon : public QLabel
{
  Q_OBJECT

  public:

    /**
     * Creates a system tray icon, managing @param managedWidget.
     * The 'icon' will be blank, so use setPixmap() to provide on.
     * KDE stipulates that icons should be a maximum of 20x20 pixels.
     * If your icon is smaller, it will be displayed at the top left
     * corner of the visible area.
     */
    QSystemTrayIcon(QWidget * managedWidget = 0);

    virtual ~QSystemTrayIcon();

    /**
     * Provided for easy access to the QPopupMenu shown on RMB press.
     * Use this in conjunction with menuAboutToShow() in order to
     * manage menu items. 'Minimize' (/'Restore') and 'Close' are
     * provided already, below a separator.
     */
    virtual QPopupMenu * popupMenu();

  protected:

    /**
     * @reimp
     */
    virtual void mousePressEvent      (QMouseEvent *);

    /**
     * @reimp
     */
    virtual void enterEvent           (QEvent *);

    /**
     * Implement this if you wish to alter the items on the popup
     * menu just before the menu is shown.
     */
    virtual void menuAboutToShow      (QPopupMenu *);

  private slots:

    void slotToggleMinimizeRestore();

  private:

    class Private;
    Private * d;
};

#endif // QSYSTEM_TRAY_ICON_H
