/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, theKompany.com gives permission
    to link the code of this program with the tkwidgets library (or with
    modified versions of tkwidgets that use the same license as tkwidgets),
    and distribute linked combinations including the two.  You must obey
    the GNU General Public License in all respects for all of the code used
    other than tkwidgets.  If you modify this file, you may extend this
    exception to your version of the file, but you are not obligated to do so.
    If you do not wish to do so, delete this exception statement from your
    version.

    This license grants you the ability to use tkwidgets with Aethera only
    and may not be used outside of Aethera.
    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
  Date     : July 2003
  Author   : Constantinescu Eugen <eug@thekompany.com> 
  Author   : Bodi Vlad <vlad@eco-spaces.com.ro> 
  Author   : Ghiciuc Ilie <ilie@eco-spaces.com.ro>
---------------------------------------------------------------------------------
*/

#ifndef FILE_BROWSER
#define FILE_BROWSER

//own includes
#include "filebrowserrequest.h"
#include "davfsentitylist.h"
#include "filebrowserrequestlist.h"

//WebDav includes
#include <webdav.h>

//Qt includes
#include <qstring.h>
#include <qobject.h>
#include <qiodevice.h>
#include <qhttp.h>
#include <qdom.h>

//internal defines
#define DAV_CAT          "DAV_CAT"
#define DAV_DEL          "DAV_DEL"
#define DAV_CREATE_FILE  "DAV_CREATE_FILE"
#define DAV_MKDIR        "DAV_MKDIR"
#define DAV_LS           "DAV_LS"
#define DAV_FLAT_LS      "DAV_FLAT_LS"
#define DAV_FIND         "DAV_FIND"
#define DAV_UPLOAD       "DAV_UPLOAD" 

class FileBrowserPrivate;

class FileBrowser : public QObject
{
  Q_OBJECT

  public:
    FileBrowser( QObject* parent=0L, const char* name=0L );
    FileBrowser( const QString& host, QObject* parent=0L, const char* name=0L );
    
    ~FileBrowser();

    void setHost( const QString& host );
    QString getHost();
    
    void setDavTransport( WebDav* transport );
    WebDav* getDavTransport( );

    //read the file from path to data
    int cat( const QString& path, QIODevice* data );
    //delete the file path
    int del( const QString& path );
    //create the file path with the content data
    int createFile( const QString& path, QIODevice* data=0L );
    //create a folder named path
    int mkdir( const QString& path );
    //list the current folder
    int ls( const QString& path );
    //a list of only the files from the path
    int flat_ls (const QString& path, bool recurse=false );
    //see if a file exists in a folder
    int find( const QString& path, const QString& file, bool recurse=false );
    //create a unique file on the server and the upload the content there
    int upload( QString& path, QIODevice* data=0L );

  private slots:
    void slotReadyRead( const QHttpResponseHeader& );
    void slotRequestFinished( int id, bool err);
 
  signals:
    void catFinished( int );
    void delFinished( int );
    void createFileFinished( int );
    void mkdirFinished( int );
    void lsFinished( int, DavFSEntityList );
    void flatLsFinished( int , DavFSEntityList );
    void findFinished( int, bool );
    void uploadFinished( int );
        
    void dataReceived( const QHttpResponseHeader& resp );

  private:
    void init();
    FileBrowserPrivate* d;
};

#endif
