/*
    Copyright (C) 2001-2002 by theKompany.com <www.thekompany.com>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    See also http://www.thekompany.com/products/license.txt for details.
---------------------------------------------------------------------------------
  Date     : July 2003
  Author   : Constantinescu Eugen <eug@thekompany.com> 
  Author   : Bodi Vlad <vlad@eco-spaces.com.ro> 
  Author   : Ghiciuc Ilie <ilie@eco-spaces.com.ro>  
---------------------------------------------------------------------------------
*/

//own includes
#include "webdav.h"
#include "authorization.h"
/*

----------------------------------------------------------------------------------------
WebDavPrivate class
----------------------------------------------------------------------------------------
*/
class WebDavPrivate
{
  public:
    bool useAuthorization;
    AuthModule* auth;
};


/*
----------------------------------------------------------------------------------------
WebDav class
----------------------------------------------------------------------------------------
*/
WebDav::WebDav( QObject* parent, const char* name)
  : QHttp( parent, name ),
    d( new WebDavPrivate )
{
}

WebDav::~WebDav()
{
  if( !d )
  {
    delete d;
    d = 0L;
  }
}


int WebDav:: request( QHttpRequestHeader& header, QIODevice* data, QIODevice* to )
{
  if( getUseAuthorization() )
    header.setValue( "Authorization", getAuthModule()->createAuthorization() );
 
  return QHttp::request( header, data, to );
}


int WebDav::request( QHttpRequestHeader& header, const QByteArray& data, QIODevice* to )
{
  if( getUseAuthorization() )
    header.setValue( "Authorization", getAuthModule()->createAuthorization() );
    
  return QHttp::request( header, data, to );
}


int WebDav::request( WebDavRequestHeader& header, QIODevice* data, QIODevice* to )
{
  if( getUseAuthorization() )
    header.setValue( "Authorization", getAuthModule()->createAuthorization() );
  
  return QHttp::request( header, data, to );
}


int WebDav::request( WebDavRequestHeader& header, const QByteArray& data, QIODevice* to )
{
  if( getUseAuthorization() )
    header.setValue( "Authorization", getAuthModule()->createAuthorization() );

  return QHttp::request( header, data, to );
}


void WebDav::setUseAuthorization( bool use )
{
  d->useAuthorization = use;
}


bool WebDav::getUseAuthorization()
{
  return d->useAuthorization;
}


void WebDav::setAuthModule( AuthModule* module )
{
  d->auth = module;
}


AuthModule* WebDav::getAuthModule()
{
  return d->auth;
}

